// FILE: ./assets/templates/controllers/SelectInlineController.js
var SelectInlineController = function(templateManager, $scope, $rootScope, metaDataFactory,  ScreenService, ZhUtil, SelectService, ZHPromise, maskEngine, EventChildPropagation) {

    var self = this;
    
    this.setScopeField = function(field) {
        if(!$scope.field) $scope.field = field;  
    };
    
    this.initFieldSelectInline = function(field) {
        this.setScopeField(field);
    };

    this.showCloseXIconSelect = function(row, fieldName) {
    	if(row[fieldName]) return true;
    	else return false;
    };
    
    this.showSearchIconSelect = function(row, fieldName) {
    	if(row[fieldName]) return false;
    	else return true;
    };
    
    this.clearCurrentRowField = function(fields, widget, $event, row, element, field) {
    	$event.stopPropagation();
    	if(!field.conditions) {
    		widget.currentRow = row;
	    	fields.forEach((field) => {
	    		field = field.name;
		    	if(!widget.currentRow['changed_'.concat(field)]) widget.currentRow['changed_'.concat(field)] = widget.currentRow[field]; 
		        widget.currentRow[field.concat('_inline')] = ''; 
		        widget.currentRow[field] = ''; 
		        if(widget.mode !== 'inline_edit') {
	                widget.inlineEdit();
	                templateManager.updateTemplate();
		        }
	        });
	        widget.lastInlineEditingElement = element.element;
    	}
        else {
        	let valid = true;
        	field.conditions.forEach((condition) => {
        		if(condition.operator == '=' || condition.operator == '==') {
        			valid (valid && row[condition.columnName] == condition.columnValue);
        		}
        		else if(condition.operator == '!=' || condition.operator == '<>') {
        			valid = (valid && row[condition.columnName] != condition.columnValue); 
        		}
        	});
        	if(valid) {
        		widget.currentRow = row;
		    	fields.forEach((field) => {
		    		field = field.name;
			    	if(!widget.currentRow['changed_'.concat(field)]) widget.currentRow['changed_'.concat(field)] = widget.currentRow[field]; 
			        widget.currentRow[field.concat('_inline')] = ''; 
			        widget.currentRow[field] = ''; 
			        if(widget.mode !== 'inline_edit') {
		                widget.inlineEdit();
		                templateManager.updateTemplate();
			        }
		        });
		        widget.lastInlineEditingElement = element.element;
        	}
        	else ScreenService.showMessage(field.errorMessage);
        }
    	
    };
    
    this.openField = function(field, $event ,row, isSpan) {
    	if(!field.conditions) {
    		this.openSelect(field, $event, isSpan);
    	}
        else {
        	let valid = true;
        	field.conditions.forEach((condition) => {
        		if(condition.operator == '=' || condition.operator == '==') {
        			valid (valid && row[condition.columnName] == condition.columnValue);
        		}
        		else if(condition.operator == '!=' || condition.operator == '<>') {
        			valid = (valid && row[condition.columnName] != condition.columnValue); 
        		}
        	});
        	if(valid) this.openSelect(field, $event, isSpan);
        	else ScreenService.showMessage(field.errorMessage);
        }
	};
	 
	this.openSelect = function(field, $event, isSpan) {
		if(isSpan) field.widget.lastInlineEditingElement = $event.toElement.parentElement.parentElement;
        else field.widget.lastInlineEditingElement = $event.toElement;
		var defer = ZHPromise.defer();
		var actions = [{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"name": "ActionEvent",
				"code": "ScreenService.closePopup(); args.owner.widget.afterClosePopup(args.owner.widget.selectField);",
				"id": 9998
			}]
		}];

		if (field.showAddButtonAs == 'action') {
			actions.push({
				"icon": "plus",
				"label": field.addButtonText,
				"showAsAction": "always|edit|checked_rows|view",
				"isVisible": true,
				"showOnForm": true,
				"showOnList": true,
				"events": [{
					"name": "ActionEvent",
					"code": "args.owner.widget.onAdd()",
					"id": 9999
				}]
			});
		}

		innerSearch("", field);

		var template;
		if (!field.showSelectAs) {
			template = "widget/list.html";
		} else if (field.showSelectAs === "tree") {
			template = "widget/tree/grid.html";
		} else {
			template = "widget/" + field.showSelectAs + ".html";
		}

		var fields;
		if (field.fields && field.fields.length) {
			fields = field.fields;
		} else {
			fields = [{
				"label": field.sourceLabel,
				"name": field.descriptionField,
				"showOnList": true,
				"mask": field.mask
			}];
		}

		var isToReload = !!(field.selectWidget && Util.objectHasAnyValue(field.selectWidget.searchCriteria));
		if (!field.selectWidget) {
			var widgetEvents = [{
				"name": "WidgetAfterMoveRow",
				"code": "args.owner.setSelected(args);",
				"id": 9999
			}];
			field.selectWidget = metaDataFactory.widgetFactory({
				"id": 1,
				"name": "Widget",
				"label": field.label,
				"template": template,
				"container": field.widget.container,
				"isVisible": true,
				"popupNoBlock": false,
				"showDescriptionOrder": false,
				"floatingControl": {
					"openSearch": !Util.isMobile(),
					"defaultSearchField": field.defaultSearchField || field.descriptionField,
					"pdfAction": false,
					"xlsAction": false,
					"searchAction": true,
					"customizationAction": {
						"filter": false,
						"layout": false,
						"view": false
					}
				},

				"parentProperty": field.parentProperty,
				"primaryProperty": field.primaryProperty,
				"expandingProperty": field.expandingProperty,
				"breadcrumbProperty": field.breadcrumbProperty,

				"order": field.order,
				"reverse": field.reverse,
				"bringGroupsCollapsed": field.bringGroupsCollapsed,
				"orderFromGroups": field.orderFromGroups,
				"selectField": field,
				"showCheckbox": false,
				"onAdd": field.onAdd,
				"showAddButtonAs": field.showAddButtonAs,
				"addButtonText": field.addButtonText,
				"noMatchText": field.noMatchText,
				"searchNoMatchText": field.searchNoMatchText,
				"fields": fields,
				"dataSource": field.dataSource,
				"dataSourceFilter": field.dataSourceFilter,
				"itemsPerPage": field.itemsPerPage,
				"searchWithPagination": Util.isUndefinedOrNull(field.searchWithPagination) ? true : field.searchWithPagination,
				"hasPagination": Util.isUndefinedOrNull(field.hasPagination) ? true : field.hasPagination,
				"onDemand": field.onDemand,
				"allowSelectOnlyLastChild": field.allowSelectOnlyLastChild,
				"searchCriteria": field.searchCriteria,

				"actions": actions,
				"events": EventChildPropagation.propagate(widgetEvents, field),

				"styles": field.selectWidgetStyles
			});
			field.selectWidget.setSelected = setSelected;
			field.selectWidget.afterClosePopup = afterClosePopup;
			isToReload = !!(field.dataSource && field.dataSource.lazyLoad);
		}

		if (isToReload) {
			field.selectWidget.searchCriteria = {};
			field.selectWidget.searchList = "";
			field.reload().then(defer.resolve, defer.resolve);
		} else {
			defer.resolve();
		}

		if(isSpan) ScreenService.openPopup(field.selectWidget);
		
		return defer.promise;	
	};
    
    
    function innerSearch(selectListSearch, field) {
		if (Util.isArray(field.filterDefinitions) && field.filterDefinitions.length) {
			var dataSource = field.dataSource;
			var filter = ZhUtil.buildFiltersInField(field, selectListSearch);
			dataSource.filter(filter, function(data) {
				dataSource.data = data;
				templateManager.updateTemplate();
			});
		}
	}
	
	var isLastChild = function(row){
		return row._childCount === 0;
	};

	
	function outDataInlineSelect(selectWidget, widgetOutData, fields, changed) {
        fields.forEach((field) => {
           field = field.name;
           if(!selectWidget.currentRow['changed_'.concat(field)]) widgetOutData.currentRow['changed_'.concat(field)] = widgetOutData.currentRow[field]; 
           widgetOutData.currentRow[field.concat('_inline')] = selectWidget.currentRow[field]; 
           if(!widgetOutData.currentRow.changed) widgetOutData.currentRow.changed = true;
           widgetOutData.currentRow[field] = selectWidget.currentRow[field]; 
        });
        if(changed == false) selectWidget.changed = true;
        if(widgetOutData.mode !== 'inline_edit') {
            widgetOutData.inlineEdit();
            templateManager.updateTemplate();
        }
	}
	
	function setSelected(args) {
	    var row = args.row;
		var field = args.owner.selectField;
		outDataInlineSelect(field.selectWidget, field.widget, field.fields, field.widget.changed);
		
		ScreenService.closePopup();

		if (field.element) {
			field.element.focus();
		}

		(field.outData ? changeByOut : changeByName)(field, row);
		field.dataSource.clearCheckedRows();
		field.dataSource.addCheckedRows(row);
		setChanged(field);

		if (field._afterSelected) {
			field._afterSelected(row);
		}
		if (field.change) {
			field.change();
		}
		
	}
	
	function afterClosePopup(field) {
		if (field.element) {
			field.element.blur();
		}
	}
	
	function changeByName(field, row) {
		var valueField = field.valueField ? field.valueField : field.name;
		var descriptionField = field.descriptionField ? field.descriptionField : field.name;
		field.widget.currentRow[valueField] = row[valueField];
		field.widget.currentRow[field.name] = row[descriptionField];
	}

	function changeByOut(field, row) {
		if (field.widget.currentRow) {
			var outData = field.outData || {};

			Object.keys(outData).forEach(function(key) {
				if (Array.isArray(outData[key])) {
					outData[key].forEach(function(valueKey) {
						field.widget.currentRow[valueKey] = row[key];
					});
				} else {
					field.widget.currentRow[outData[key]] = row[key];
				}
			});
		}
	}
	
	var setChanged = function(field) {
		field.widget.currentRow.changed = true;
	};
	
	$scope.$watchCollection('field.dataSourceFilter', propagateFieldDataSourceFilterToSelectWidget);
	function propagateFieldDataSourceFilterToSelectWidget(fieldDataSourceFilter) {
		if ($scope.field.selectWidget) {
			$scope.field.selectWidget.dataSourceFilter = fieldDataSourceFilter;
		}
	}
};


// FILE: ./assets/templates/controllers/zRegisterDfComponents.js
var zRegisterEstComponents = function(decodeTemplateFieldService, eventAggregator) {
    eventAggregator.subscribe('beforeProjectStartup', function register() {
        decodeTemplateFieldService.registerViewTemplate('pebbian-component#widget/pebbian-grid.html', 'pebbian#widget/pebbian-grid.html');
    });
};

Configuration(function (ContextRegister, eventEngine) {
	ContextRegister.register('zRegisterEstComponents', zRegisterEstComponents);
});


(function registerTemplates() {
    window.templateUrls = window.templateUrls || [];
    window.templateUrls.push(
        {
            'package': 'pebbian-component',
            'baseUrl': './assets/templates/'
        }
    );
})();

// FILE: ./assets/js/utilities/Itlanctoest_Repository.js

var Itlanctoest_Repository = function (restEngine) {

    this.saveItlanctoest = function(row) {
        return restEngine.post('est#/crud/itlanctoest/save', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.saveItlanctoestValidation = function(row) {
        return restEngine.post('est_ajuste#/itlanctoest/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteItlanctoest = function(row) {
        return restEngine.post('est#/crud/itlanctoest/remove', {
            "requestType": "Row",
            "row": row
        });
    };
}; 

Configuration(function(ContextRegister){
    ContextRegister.register('Itlanctoest_Repository', Itlanctoest_Repository);
});


// FILE: ./assets/js/utilities/Lanctoestoq_Repository.js
var Lanctoestoq_Repository = function (restEngine) {

    this.saveLanctoestoq = function(row,rowItem) {
        row.ITENS = [];
        if(typeof rowItem==='object')
            row.ITENS.push(rowItem);
        return restEngine.post('est#/lanctoestoq/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteLanctoestoq = function(row) {
        return restEngine.post('est#/crud/lanctoestoq/remove', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('Lanctoestoq_Repository', Lanctoestoq_Repository);
});


// FILE: ./assets/js/utilities/Lib00206_InfComplProduto_Repository.js
var Lib00206_InfComplProduto_Repository = function(restEngine) {
    this.getProximoLote = function(filter){
        return restEngine.post('est#/prox_lote', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getProdFili= function(filter){
        return restEngine.post('est#/crud/prodfili', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.saveLote= function(row){
        return restEngine.post('est#/save_lote', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Repository', Lib00206_InfComplProduto_Repository);
});

// FILE: ./assets/js/utilities/ValidacoesEstoque_Repository.js
var ValidacoesEstoque_Repository = function (localStorageEngine, restEngine) {
    var row = {};

    this.validaDataLancamento = function (filial, dtLancamento, operacao, sistema, dtEmissao) {

        row.CDFILIAL = filial;
        row.DTLANCTOEST = dtLancamento;
        row.OPERACAO = operacao;
        row.IDSISTEMA = sistema;
        row.DTEMISSAO = dtEmissao;

        return restEngine.post('est#/validacoes_estoque/data_lancamento', {
            "requestType": "Row",
            "row": row
        });
    };

    /* **************************************************
     * Função que requisita a execução da atualização de
     * estoque. Para maiores informações, favor verificar
     * a função de mesmo nome no Service de validações de
     * estoque.
     *
     * @param string cdfilial
     * @param string nrlancestq
     * @param boolean verifyIrregularities
     * @param boolean abortAtIrregularities
     * @param Array seqItemCodes
     *
     * @return Promise
     */
    this.atualizaEstoque = function (cdfilial, nrlancestq, verifyIrregularities, abortAtIrregularities, seqItemCodes, boolConsolidateToStock) {
        //sets row values
        row.CDFILIAL    = cdfilial;
        row.NRLANCESTQ  = nrlancestq;
        row.IDVERIRREG  = verifyIrregularities;
        row.IDABRIRREG  = abortAtIrregularities;
        row.SEQITEMS    = seqItemCodes;
        row.CONSOLIDATE = boolConsolidateToStock;

        //sends a request to the backend
        return restEngine.post('est#/validacoes_estoque/atualiza_estoque', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.atualizaEstoqueRetirada = function (cdfilial, nrlancestq, verifyIrregularities, abortAtIrregularities, seqItemCodes, boolConsolidateToStock, rowAjust) {
        //sets row values
        row.CDFILIAL = cdfilial;
        row.NRLANCESTQ = nrlancestq;
        row.IDVERIRREG = verifyIrregularities;
        row.IDABRIRREG = abortAtIrregularities;
        row.SEQITEMS = seqItemCodes;
        row.CONSOLIDATE = boolConsolidateToStock;
        row.ROW = rowAjust;

        //sends a request to the backend
        return restEngine.post('est#/validacoes_estoque/atualiza_estoque_retirada', {
            "requestType": "Row",
            "row": row
        });
    };

    this.zeraInventario = function (filial, nrLancamento, idordemclc, boolConsolidateToStock) {

        row.CDFILIAL = filial;
        row.NRLANCAMENTO = nrLancamento;
        row.IDORDEMCLC = idordemclc;
        row.CONSOLIDATE = boolConsolidateToStock;
        
        //sends a request to the backend
        return restEngine.post('est#/validacoes_estoque/zera_inventario', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.zeraInventarioRetirada = function (filial, nrLancamento, idordemclc, boolConsolidateToStock, rowAjust) {

        row.CDFILIAL = filial;
        row.NRLANCAMENTO = nrLancamento;
        row.IDORDEMCLC = idordemclc;
        row.CONSOLIDATE = boolConsolidateToStock;
        row.ROW = rowAjust;
        
        //sends a request to the backend
        return restEngine.post('est#/validacoes_estoque/zera_inventario_retirada', {
            "requestType": "Row",
            "row": row
        });
    };

    this.consultaCustoQuantidadeProduto = function (
        filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote
    )
    {
        row.CDFILIAL = filial;
        row.CDTIPOCUST = tipoCusto;
        row.CDPRODUTO = produto;
        row.CDALMOXARIFE = almoxarifado;
        row.CDLOCALESTOQ = localEstoque;
        row.NRLOTE = lote;
        row.NRSUBLOTE = subLote;
        row.DTLANCAMENTO = dtLancamento;

        return restEngine.post('est#/validacoes_estoque/consulta_custo', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.getPosicaoDeEstoquePorProduto = function(filter) {
        return restEngine.post('est_lancamentos#/retirada_posestdia', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ValidacoesEstoque_Repository', ValidacoesEstoque_Repository);
});

// FILE: ./assets/js/utilities/Lib00206_InfComplProduto_Service.js
var Lib00206_InfComplProduto_Service = function(Lib00206_InfComplProduto_Repository, restEngine) {
    this.getDtprocessa = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return restEngine.post("est#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getParamFilial = function(cdFiliMovi){
        var filial = cdFiliMovi || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getInfoLote = function(cdFiliMovi,cdProdMovi,nrLoteEstoq,nrSubLote){
        return this.getDtprocessa();
    };

    this.saveLote = function(row){
        row.NRLOTEESTQ = row.NRLOTEESTQ || ' ';
        row.NRSUBLOTE  = row.NRSUBLOTE  || ' ';
        row.DTVALILOTE = row.DTVALILOTE || null;
        row.DTFABRPROD = row.DTFABRPROD || null;

        var saveRow = {
            CDFILIMOVI:row.CDFILIMOVI,
            NRLOTEESTQ:row.NRLOTEESTQ,
            NRSUBLOTE:row.NRSUBLOTE,
            CDPRODESTO:row.CDPRODESTO,
            DTVALILOTE:row.DTVALILOTE,
            DTFABRPROD:row.DTFABRPROD,
            VRFATOCONV: row.VRFATOCONV
        };

        return Lib00206_InfComplProduto_Repository.saveLote(saveRow);
    };

    this.getProdFili = function(cdproduto,cdfilial){
        var filter = [{
            'name':"CDPRODUTO",
            "operator":"=",
            "value":cdproduto
        },
        {
            'name':"CDFILIAL",
            "operator":"=",
            "value":cdfilial
        }];
        return Lib00206_InfComplProduto_Repository.getProdFili(filter);
    };

    this.getProximoLote = function(cdFilial,cdProdesto){
        var filter = [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value":cdFilial
            },
            {
                "name":"DTVALILTE",
                "operator":"=",
                "value":moment().format('DD/MM/YYYY')
            },
            {
                "name":"CDPRODUTO",
                "operator":"=",
                "value":cdProdesto
            }
        ];
        return Lib00206_InfComplProduto_Repository.getProximoLote(filter);
    };
    
    this.validateInventory = function(widget, params = {}){
        let filial = widget.currentRow.CDFILIMOVI;
        let almoxarifado = widget.getField('CDALMOXARIFE');
        let local = widget.getField('CDLOCALESTOQ');
        let lote = widget.getField('NRLOTEESTQ');
        let dataSourceName = 'inverota';
        if(params != {})
            widget.params = params;
        
        let filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
            },
            {
                "name": "CDALMOXARIFE",
                "operator": "=",
                "value": almoxarifado? (almoxarifado.value() || ' ') : ' '
            },
            {
                "name": "CDLOCALESTOQ",
                "operator": "=",
                "value": local? (local.value() || ' ') : ' '
            },
        ];
        
        if(widget.params.IDUTILLOTE != 'S'){
            dataSourceName = 'inverota';
            filter.push({
                "name": "IDINVENTOK",
                "operator": "=",
                "value": "N"
            });
        }else{
            dataSourceName = 'inverotalote';
            filter.push({
                "name": "IDINVLOTEOK",
                "operator": "=",
                "value": "N"
            });
        }
        
        return restEngine.post('est#/crud/'+dataSourceName, {
            "requestType": "FilterData",
            "filter": filter
        }).then(resolve => {
            return resolve.dataset[dataSourceName];
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Service', Lib00206_InfComplProduto_Service);
});

// FILE: ./assets/js/utilities/Lib00207_PosicaoEstoque_Service.js
var Lib00207_PosicaoEstoque_Service = function(General) {
    
    this.getDtprocessa = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("df#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getParamFilial = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("df#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getPosicaoDeEstoquePorProduto = function(row) {
	    var filter = [
	    	{
	    		"name": "CDPRODUTO",
				"operator": "=",
				"value": row.CDPRODUTO
			},
			{
	    		"name": "CDFILIAL",
				"operator": "=",
				"value": row.CDFILIAL
			},
			{
	    		"name": "CDALMOXARIFE",
				"operator": "=",
				"value": row.CDALMOXARIFE
			},
			{
	    		"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": row.CDLOCALESTOQ
			},
			{
	    		"name": "NRLOTEESTQ",
				"operator": "=",
				"value": row.NRLOTEESTQ
			},
			{
	    		"name": "NRSUBLOTE",
				"operator": "=",
				"value": row.NRSUBLOTE
			}
		];

        return General.callRestEngine('est_lancamentos#/retirada_posestdia', 'FilterData', filter).then(function(success){
            return success.dataset.posestdia[0];
        });
	
	};
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00207_PosicaoEstoque_Service', Lib00207_PosicaoEstoque_Service);
});

// FILE: ./assets/js/utilities/ValidacoesEstoque_Service.js
var ValidacoesEstoque_Service = function(ValidacoesEstoque_Repository, ScreenService, metaDataFactory) {

    var TIPO_OPERACAO = {
        'ENTRADA':    'E',
        'SAIDA':      'S',
        'AJUSTE_IMP': 'A',
        'RETIRADA_IMP': 'R'
    };

    var TIPO_SISTEMA = [
        'DFE', 'DFS', 'EST', 'FAI', 'FAZ', 'FAC', 'REC', 'PAG'
    ];

    var TIPO_CUSTO_PADRAO = '03',
        VAZIO_EST         = ' ';

    this.validaDataLancamento = function(filial, dataLancamento, operacao, sistema, dataEmissao) {
        var validaOperacaoSistema = (!TIPO_OPERACAO.hasOwnProperty(operacao)) || TIPO_SISTEMA.indexOf(sistema) === -1;

        if(!dataLancamento || !operacao || validaOperacaoSistema) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        operacao = TIPO_OPERACAO[operacao];
        dataLancamento = moment(dataLancamento, "DD/MM/YYYY").format('DD/MM/YYYY');
        dataEmissao = !dataEmissao ? dataLancamento : moment(dataEmissao, "DD/MM/YYYY").format('DD/MM/YYYY');

        return ValidacoesEstoque_Repository.validaDataLancamento(filial, dataLancamento, operacao, sistema, dataEmissao);
    };

    /* **************************************************
     * Função que requisita a execução da atualização de
     * estoque. Possui como comportamento padrão a
     * atualização de todos os itens de um lançamento de
     * uma unidade, com verificação de irregularidades e
     * abortamento das operações no caso de invalidez
     *
     * Permite porém que a verificação de irregularidades
     * não seja efetuada ou que as operações não sejam
     * abortadas após a verificação através da passagem
     * dos parâmetros "verifyIrregularities" e
     * "abortAtIrregularities". Note que o último não
     * pode ser verdadeiro se o primeiro for falso.
     *
     * Permite também que sejam selecionados um ou mais
     * itens para a execução da atualização de estoque
     * através da passagem de seus códigos na variável
     * "seqItemCodes". Ela é nula no comportamento
     * padrão, o que garante que todos os itens do
     * lançamento sejam atualizados.
     *
     * @param string  cdfilial
     * @param string  nrlancestq
     * @param boolean verifyIrregularities
     * @param boolean abortAtIrregularities
     * @param Array   seqItemCodes
     *
     * @return Promise
     */
    this.atualizaEstoque = function(cdfilial, nrlancestq, boolConsolidateToStock, verifyIrregularities, abortAtIrregularities, seqItemCodes) {
        //ensures that "seqItemCodes" will never be "undefined", as such value doesn't exist in PHP
        seqItemCodes = seqItemCodes || null;

        //sets default flag values if and only if they haven't been passed
        if(typeof verifyIrregularities === 'undefined')
            verifyIrregularities = true;
        if(typeof abortAtIrregularities === 'undefined')
            abortAtIrregularities = verifyIrregularities;
        if(typeof boolConsolidateToStock === 'undefined')
            boolConsolidateToStock = false;

        //checks for invalid parameter values and returns rejected promise
        if(!cdfilial || !nrlancestq ||
           (typeof verifyIrregularities !== 'boolean') ||
           (typeof abortAtIrregularities !== 'boolean') ||
           (typeof boolConsolidateToStock !== 'boolean') ||
           (!verifyIrregularities && abortAtIrregularities)) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';

            return Promise.reject(response);
        }

        //calls repository's request function, returning its promise
        return ValidacoesEstoque_Repository.atualizaEstoque(
            cdfilial, nrlancestq, verifyIrregularities,
            abortAtIrregularities, seqItemCodes, boolConsolidateToStock
        );
    };
    
    this.atualizaEstoqueRetirada = function(cdfilial, nrlancestq, boolConsolidateToStock, row, verifyIrregularities, abortAtIrregularities, seqItemCodes) {
        //ensures that "seqItemCodes" will never be "undefined", as such value doesn't exist in PHP
        seqItemCodes = seqItemCodes || null;

        //sets default flag values if and only if they haven't been passed
        if(typeof verifyIrregularities === 'undefined')
            verifyIrregularities = true;
        if(typeof abortAtIrregularities === 'undefined')
            abortAtIrregularities = verifyIrregularities;
        if(typeof boolConsolidateToStock === 'undefined')
            boolConsolidateToStock = false;

        //checks for invalid parameter values and returns rejected promise
        if(!cdfilial || !nrlancestq ||
           (typeof verifyIrregularities !== 'boolean') ||
           (typeof abortAtIrregularities !== 'boolean') ||
           (typeof boolConsolidateToStock !== 'boolean') ||
           (!verifyIrregularities && abortAtIrregularities)) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';

            return Promise.reject(response);
        }

        //calls repository's request function, returning its promise
        return ValidacoesEstoque_Repository.atualizaEstoqueRetirada(
            cdfilial, nrlancestq, verifyIrregularities,
            abortAtIrregularities, seqItemCodes, boolConsolidateToStock, row
        );
    };

    this.zeraInventario = function(filial, nrLancamento, idordemclc, boolConsolidateToStock = false) {
        idordemclc = (idordemclc !== 'U')? 0 :'U';

        if(!filial || !nrLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        return ValidacoesEstoque_Repository.zeraInventario(filial, nrLancamento, idordemclc, boolConsolidateToStock);
    };
    
    this.zeraInventarioRetirada = function(filial, nrLancamento, idordemclc, boolConsolidateToStock = false, row = undefined) {
        idordemclc = (idordemclc !== 'U')? 0 :'U';

        if(!filial || !nrLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        return ValidacoesEstoque_Repository.zeraInventarioRetirada(filial, nrLancamento, idordemclc, boolConsolidateToStock, row);
    };

    this.consultaCustoQuantidadeProduto = function(filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote) {
        if(!filial || !produto || !dtLancamento) {
            var response = {};
            response.data = {};
            response.data.error = 'Parâmetros incorretos ou insuficientes.';
            return Promise.reject(response);
        }

        almoxarifado = almoxarifado || VAZIO_EST;
        localEstoque = localEstoque || VAZIO_EST;

        lote    =    lote || VAZIO_EST;
        subLote = subLote || VAZIO_EST;

        tipoCusto = tipoCusto || null;

        return ValidacoesEstoque_Repository.consultaCustoQuantidadeProduto(
            filial, produto, dtLancamento, tipoCusto, almoxarifado, localEstoque, lote, subLote
        );
    };
    
    this.openPosicaoDeEstoque = function(widget) {
    	var popup = this.getPosicaoDeEstoquePopup(widget);

		ScreenService.openPopup(popup);

		popup.currentRow = widget.currentRow;
		popup.parentWidget = widget;

		this.getDataForPosicaoDeEstoquePopup(popup);
    };
    
    this.getDataForPosicaoDeEstoquePopup = function(popup) {
    	this.getPosicaoDeEstoquePorProduto(popup.currentRow).then(function(posestdia) {
    		if(posestdia) {
	    		popup.currentRow = posestdia;
	    		popup.parentWidget.currentRow.QTESTOQDIA = posestdia.QTESTOQDIA;
    		}
    		else{
    			popup.currentRow = 0;
    		}
    	}).catch(function(error) {
    		ScreenService.showMessage(error.data.error);
    	});
    };
    
    this.getPosicaoDeEstoquePopup = function(parentWidget) {
        var widget = parentWidget.posestPopup;

        if(!widget) {
            widget = this.getPosicaoDeEstoquePopUpObject();
            widget.container = parentWidget.container;
            widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

            widget = metaDataFactory.widgetFactory(widget, parentWidget);

            parentWidget.posestPopup = widget;
            widget.parent = parentWidget;
        } else
            widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

        return widget;
    };

    this.getPosicaoDeEstoquePopUpObject = function() {
    	return {
            "name": "posestPopup",
            "defaultLabel": "Posição de Estoque do Produto ",
            "template": "widget/form.html",
            "isVisible": true,
            "showAsTab": false,
            "actions": [
                {
                    "name": "back",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup()"
                        }
                    ]
                },
                {
                    "name": "refresh",
                    "label": "Atualizar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ValidacoesEstoque_Service.getDataForPosicaoDeEstoquePopup(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "QTESTOQDIA",
                    "label": "Quantidade",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true,
                    "sequenceInForm": 80,
                    "readOnly": true,
                    "defaultValue": null,
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "VRCUSTOPROD",
                    "label": "Custo",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "showSymbol": true,
                                    "precision": 3,
                                    "thousands": ",",
                                    "decimal": "."
                            }
                    },
                    "events": []
                    
                },
                
            ]
        };
    };
    
    this.getPosicaoDeEstoquePorProduto = function(row) {
	    var filter = [
	    	{
	    		"name": "CDPRODUTO",
				"operator": "=",
				"value": row.CDPRODUTO
			},
			{
	    		"name": "CDFILIAL",
				"operator": "=",
				"value": row.CDFILIAL
			},
			{
	    		"name": "CDALMOXARIFE",
				"operator": "=",
				"value": row.CDALMOXARIFE
			},
			{
	    		"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": row.CDLOCALESTOQ
			},
			{
	    		"name": "NRLOTEESTQ",
				"operator": "=",
				"value": row.NRLOTEESTQ
			},
			{
	    		"name": "NRSUBLOTE",
				"operator": "=",
				"value": row.NRSUBLOTE
			}
		];

	    return ValidacoesEstoque_Repository.getPosicaoDeEstoquePorProduto(filter).then(function(success){
	    	return success.dataset.posestdia[0];
	    });
	};
};

Configuration (function(ContextRegister) {
    ContextRegister.register('ValidacoesEstoque_Service', ValidacoesEstoque_Service);
});

// FILE: ./assets/js/utilities/Endereco_Controller.js
function Endereco_Controller(General, ScreenService, templateManager) {
    var self = this;

    //////////////////////////////////////////////////////////
    //                                                      //
    // ESSE CONTROLLER ESPERA AS SEGUINTES CARACTERÍSTICAS: //
    //                                                      //
    // na widget principal:                                 //
    //     addressWidget, que indica o nome da widget       //
    //     do grid de endereço;                             //
    //                                                      //
    //     addressWidgetToForm, que indica o nome da        //
    //     widget do form de endereço.                      //
    //                                                      //
    // na widget de formulário de endereço:                 //
    //     addressTypeColumn, que indica o nome do field    //
    //     do tipo de endereço.                             //
    //                                                      //
    //     event WidgetOnEnter com o método                 //
    //                     addressWidgetOnEnter em seu code //
    //                                                      //
    //////////////////////////////////////////////////////////

    /* **************************************************
     * Procedimento qeu verifica a flag de visitação da
     * widget de formulário de endereço.
     *
     * @param Widget widget
     */
    self.addressWidgetOnEnter = function(widget) {
        if(widget.visited == 404)
            setTimeout(function() {widget.isValid();}, 5000); //blame Wagão for setTimeout
                                                              //R.I.P. Wagão ??? - 2016

        widget.visited = true;
    };

    /* **************************************************
     * Procedimento que cancela o processo de inserção,
     * fechando o swipe e retornando ao funcionamento
     * normal.
     *
     * Recebe o nome da widget principal e permite o uso
     * de uma função de validação opcional a ser
     * executada após o fechamento do swipe, que pode ou
     * não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.cancelInsertion = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);

        self.clearRow(mainWidget);

        self.exit(mainWidgetName, validationFunction, validationParams);
    };

    /* **************************************************
     * Procedimento que remove a currentRow da widget
     * principal para o caso de cancelamento da inserção.
     *
     * @param Widget mainWidget
     */
    self.clearRow = function(mainWidget) {
        var indexOfCurrentRow = mainWidget.dataSource.data.indexOf(mainWidget.currentRow);

        if(indexOfCurrentRow !== -1) mainWidget.dataSource.data.pop();

        mainWidget.setCurrentRow({});
    };

    /* **************************************************
     * Procedimento que encerra o processo de inserção,
     * fechando o swipe e retornando ao funcionamento
     * normal.
     *
     * Recebe o nome da widget principal e permite o uso
     * de uma função de validação opcional a ser
     * executada após o fechamento do swipe, que pode ou
     * não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.exit = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var mainAddressWidget = templateManager.container.getWidget(mainWidget.addressWidget);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);

        if(validationFunction instanceof Function)
            validationFunction.apply(this, validationParams);

        self.switchAddressVisibility(mainAddressWidget, formAddressWidget);

        var indexOfFormWidget = mainWidget.widgets.indexOf(formAddressWidget);
        General.changeWidgetAcessibility(mainWidget.widgets.slice(indexOfFormWidget), true);

        self.prepareView(mainWidget);
    };

    /* **************************************************
     * Procedimento que abre o swipe de inserção da
     * widget principal junto com a sua widget de
     * formulário de endereço.
     *
     * Recebe o nome da widget principal que será aberta,
     * e permite o uso de uma função de validação
     * opcional a ser executada após a abertura do swipe,
     * que pode ou não receber parâmetros.
     *
     * @param string mainWidgetName
     * @param Function validationFunction
     * @param Array validationParams
     */
    self.newRow = function(mainWidgetName, validationFunction, validationParams) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var mainAddressWidget = templateManager.container.getWidget(mainWidget.addressWidget);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);

        formAddressWidget.visited = false;

        var indexOfFormWidget = mainWidget.widgets.indexOf(formAddressWidget);
        General.changeWidgetAcessibility(mainWidget.widgets.slice(indexOfFormWidget), false);

        self.switchAddressVisibility(mainAddressWidget, formAddressWidget);

        self.prepareForm(mainWidget, formAddressWidget);

        if(validationFunction instanceof Function)
            validationFunction.apply(this, validationParams);
    };

    /* **************************************************
     * Procedimento que configura as widgets da inserção
     * de um registro com endereço para dito processo.
     *
     * @param Widget mainWidget
     * @param Widget formAddressWidget
     */
    self.prepareForm = function(mainWidget, formAddressWidget) {
        mainWidget.newRow();

        formAddressWidget.setCurrentRow({});
        formAddressWidget.edit();

        formAddressWidget.currentRow.__is_new = true;
        formAddressWidget.currentRow[formAddressWidget.addressTypeColumn] = 'P';

        formAddressWidget.getField(formAddressWidget.addressTypeColumn).readOnly = true;
    };

    /* **************************************************
     * Procedimento que configura a widget principal para
     * sair do modo de inserção.
     *
     * @param Widget mainWidget
     */
    self.prepareView = function(mainWidget) {
        mainWidget.list();

        ScreenService.closeSwipe();

        mainWidget.activate();
    };

    /* **************************************************
     * Procedimento que checa a validez das duas widgets
     * de inserção e efetua o salvamento quando possível.
     *
     * Também avisa o usuário sobre qual(is) widget(s)
     * estão inválidas, e direciona-o caso necessário.
     *
     * Recebe o nome da widget principal que será salva,
     * além da função que será executada para efetuar o
     * salvamento. Permite o uso de funções de sucesso e
     * erro opcionais a serem executadas após o
     * salvamento.
     *
     * IMPORTANTE: A função "saveParams" DEVE retornar
     * promise!
     *
     * @param string mainWidgetName
     * @param Function saveFunction
     * @param Array saveParams
     * @param Function successFunction
     * @param Function errorFunction
     */
    self.save = function(mainWidgetName, saveFunction, saveParams, successFunction, errorFunction) {
        var mainWidget = templateManager.container.getWidget(mainWidgetName);
        var formAddressWidget = templateManager.container.getWidget(mainWidget.addressWidgetToForm);

        var mainIsValid = mainWidget.isValid();

        //checks address widget validity
            /* note: Widget's "isValid" function returns true if the widget wasn't visited,
             * even if it's not actually valid. So, its necessary to check the real address
             * validity, in this case, using a flag. */
        var addressIsValid = (formAddressWidget.isValid() && (formAddressWidget.visited === true));

        switch(true) {
            case mainIsValid && addressIsValid:
                if(saveFunction instanceof Function)
                    saveFunction.apply(this, saveParams).then(function(callback) {
                        self.exit(mainWidgetName);

                        if(successFunction instanceof Function)
                            successFunction.apply(this, callback);
                    }, errorFunction);
                break;

            case !mainIsValid && !addressIsValid:
                if(formAddressWidget.visited === true)
                    ScreenService.showMessage("Preencha todos os campos obrigatórios das duas telas!");
                else
                    formAddressWidget.visited = 404;
                break;

            case !mainIsValid && !mainWidget.isActive():
                ScreenService.showMessage("Preencha os campos obrigatórios!");

                mainWidget.currentWidget = mainWidget;

                mainWidget.activate();
                break;

            case !addressIsValid && !formAddressWidget.isActive():
                formAddressWidget.visited = 404;

                ScreenService.showMessage("Preencha os campos obrigatórios!");

                formAddressWidget.activate();
                break;
        }
    };

    /* **************************************************
     * Procedimento que troca entre a widget de grid de
     * endereço e widget de formulário de endereço.
     *
     * Para que o procedimento funcione corretamente, a
     * widget de grid deve ter sua visibilidade marcada
     * inicialmente como true.
     *
     * @param Widget mainAddressWidget
     * @param Widget formAddressWidget
     */
    self.switchAddressVisibility = function(mainAddressWidget, formAddressWidget) {
        var previousState = mainAddressWidget.isVisible;

        mainAddressWidget.enabled = mainAddressWidget.showAsTab = mainAddressWidget.isVisible = !previousState;

        formAddressWidget.enabled = formAddressWidget.isVisible = formAddressWidget.showAsTab = previousState;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Endereco_Controller', Endereco_Controller);
});

// FILE: ./assets/js/utilities/Lib00206_InfComplProduto_Controller.js
var Lib00206_InfComplProduto_Controller = function(ScreenService, ZHPromise, templateManager, metaDataFactory, Lib00206_InfComplProduto_Service, eventEngine, MessageService) {

    var infoComplEvents = {
        "InfoComplBeforeConfirm": "beforeConfirmInfoCompl",
        "InfoComplAfterConfirm": "afterConfirmInfoCompl",
        "InfoComplBeforeCancel": "beforeCancelInfoCompl",
        "InfoComplAfterCancel": "afterCancelInfoCompl",
        "InfoComplInvalidStock": "infoComplInvalidStock",
        "InfoComplOnOpenPopup": "infoComplOpenPopup"
    };

    Object.keys(infoComplEvents).forEach(function(eventName) {
        if (!eventEngine.hasEvent(eventName)) {
            eventEngine.registerEvent(eventName, infoComplEvents[eventName]);
        }
    });
    var rowParent;
    var lotAutomatico;

    this.infComplementares = function(widget, rowItem) {
        var defaultParams = {
            ShowLote: true,
            ShowEntr: false,
            Ajuste: false,
            AtuEstoq: true,
            VerQtEst: true,
            DocuDevo: false,
            AfPiloto: false,
            IsEntrada: true,
            DontEditLocals: false
        };

        rowParent = rowItem;
        defaultParams.ShowLote = rowItem.__is_new;

        Util.extend(defaultParams, widget.paramInfoCompl);

        if (!rowParent.IDCNTRESTOQ) // IDCNTRESTOQ = null ou inexistente produto controlara estoque
            rowParent.IDCNTRESTOQ = 'S';

        if (!(rowParent.CDFILIAL && rowParent.CDFILIMOVI && rowParent.CDPRODESTO && rowParent.CDPRODUTO && rowParent.CDARVPROD && rowParent.NMPRODUTO)) {
            console.error('Row do Item incompleta');
        }
        else if (rowParent.IDCNTRESTOQ != 'N') { // IDCNTRESTOQ = null produto controlara estoque
            Lib00206_InfComplProduto_Service.getParamFilial(rowParent.CDFILIMOVI).then(function(paramfilial) {
                paramfilial = paramfilial.dataset.paramfilial[0];
                // IDCTRLESTQ = null filial controlara estoque
                if (paramfilial.IDCTRLESTQ != 'N' && (paramfilial.IDUTILLOTE == 'S' || paramfilial.IDUTILALMOX == 'S')) {
                    if (paramfilial.IDUTILLOTE == 'S' && widget.container.name == 'dfe11000_lancamento_entrada')
                        defaultParams.ShowEntr = true;
                    this.openInfCompl(widget, defaultParams, paramfilial);
                }
                else {
                    notOpenPopup(widget, paramfilial);
                }
            }.bind(this));
        }
        else {
            notOpenPopup(widget, {});
        }

    };

    this.limpaCamposInfCompl = function(widget) {
        widget.fields.forEach(function(field) {
            field.value(null);
        });
    };

    this.openInfCompl = function(parentWidget, params, paramfilial) {
        var popupWidget = createInfComplProdutoPopup(parentWidget);
        delete(rowParent.$error);
        Util.extend(popupWidget.currentRow, rowParent);
        //this.limpaCamposInfCompl(popupWidget);

        Lib00206_InfComplProduto_Service.getDtprocessa().then(function(success) {
            var dtprocessa = success.dataset.paramestoque_processamento[0].DTPROCESSA;

            if (!rowParent.DTLANCMOVI)
                rowParent.DTLANCMOVI = dtprocessa;

            popupWidget.getField('DTVALILOTE').minDate = dtprocessa;

            var filialFilter = {
                "name": "CDFILIAL",
                "operator": "=",
                "value": rowParent.CDFILIMOVI
            };

            ['DSALMOXARIFE', 'DSLOCALESTOQ'].forEach(function(fieldName) {
                var fieldFilter = popupWidget.getField(fieldName).dataSourceFilter;
                var filterIndex = fieldFilter.findIndex(function(element) { return element.name == "CDFILIAL"; });

                if (filterIndex == -1)
                    fieldFilter.push(filialFilter);
                else
                    fieldFilter[filterIndex] = filialFilter;

                if (['DSLOCALESTOQ'].includes(fieldName)) {
                    filterIndex = fieldFilter.findIndex(function(element) { return element.name == "DSALMOXARIFE"; });

                    if (filterIndex == -1) {
                        var almoxariFilter = {
                            "name": "CDALMOXARIFE",
                            "operator": "=",
                            "value": "@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('" + parentWidget.name + "','CDALMOXARIFE')@"
                        };

                        fieldFilter.push(almoxariFilter);
                    }
                }
            });

            this.setFieldLote(popupWidget, parentWidget, params);

            if (!params.AtuEstoq) {
                popupWidget.getField('DSALMOXARIFE').setValue(' ');
                popupWidget.getField('DSLOCALESTOQ').setValue(' ');
                popupWidget.getField('NRSUBLOTE').setValue(' ');
                popupWidget.getField('NRLOTEESTQ').setValue(' ');
            }

            popupWidget.params = params;
            popupWidget.getField('IDNOVOLOTE').value(params.ShowEntr);
            popupWidget.getField('IDNOVOLOTE').defaultValue = params.ShowEntr;
            popupWidget.getField('IDNOVOLOTE').readOnly = !params.ShowEntr;
            this.setReadOnlyField(popupWidget, params).then(function(paramfilial) {
                Lib00206_InfComplProduto_Service.getProdFili(rowParent.CDPRODUTO, rowParent.CDFILIMOVI).then(function(prodfili) {
                    prodfili = prodfili.dataset.prodfili[0];

                    var almoxField = popupWidget.getField('DSALMOXARIFE'),
                        localField = popupWidget.getField('DSLOCALESTOQ'),
                        selectLote = popupWidget.getField('IDNOVOLOTE'),
                        loteField = popupWidget.getField('NRLOTEESTQ'),
                        subloteField = popupWidget.getField('NRSUBLOTE');

                    loteField.validations.required = true;
                    subloteField.setValue(' ');

                    if (prodfili.IDUTCTRLLOTE == 'N' || paramfilial.IDUTILLOTE == 'N') {
                        delete loteField.validations.required;
                        loteField.readOnly = true;
                        loteField.setValue(' ');
                        selectLote.readOnly = true;
                        popupWidget.getField('IDNOVOLOTE').isVisible = false;
                        popupWidget.getField('NRLOTEESTQ').isVisible = false;
                        popupWidget.getField('NRSUBLOTE').isVisible = false;
                    }
                    else {
                        popupWidget.getField('IDNOVOLOTE').isVisible = true;
                        popupWidget.getField('NRLOTEESTQ').isVisible = true;
                        popupWidget.getField('NRSUBLOTE').isVisible = true;

                        if (!parentWidget.currentRow.NRLOTEESTQ) {
                            loteField.value(null);

                            if (prodfili && params.ShowLote) {
                                Lib00206_InfComplProduto_Service.getProximoLote(rowParent.CDFILIMOVI, rowParent.CDPRODESTO).then(function(success) {
                                    loteField.value(success.dataset.prox_lote.NRLOTEESTQ);
                                });
                            }
                        }
                    }

                    if (prodfili.IDCNTRESTOQ == 'N') {
                        loteField.readOnly = true;
                        loteField.setValue(' ');
                        subloteField.readOnly = true;
                        subloteField.setValue(' ');
                        almoxField.readOnly = true;
                        almoxField.setValue(' ');
                        delete almoxField.validations.required;
                        localField.readOnly = true;
                        localField.setValue(' ');
                        delete localField.validations.required;
                    }

                    //popupWidget.currentRow.CDALMOXARIFE = !almoxField.readOnly ? parentWidget.currentRow.CDALMOXARIFE||prodfili.CDALMOXARIFE||paramfilial.CDALMOXARIFE||null:' ';
                    //popupWidget.currentRow.CDLOCALESTOQ = !localField.readOnly ? parentWidget.currentRow.CDLOCALESTOQ||prodfili.CDLOCALESTOQ||paramfilial.CDLOCALESTOQ||null:' ';
                    if (!almoxField.readOnly || params.DontEditLocals) {
                        if (parentWidget.currentRow.CDALMOXARIFE) {
                            popupWidget.currentRow.CDALMOXARIFE = parentWidget.currentRow.CDALMOXARIFE;
                        }
                        else if (prodfili.CDALMOXARIFE && paramfilial.IDUTILALMOX == 'S') {
                            popupWidget.currentRow.CDALMOXARIFE = prodfili.CDALMOXARIFE;
                        }
                        else if (paramfilial.CDALMOXARIFE && paramfilial.IDUTILALMOX == 'S') {
                            popupWidget.currentRow.CDALMOXARIFE = paramfilial.CDALMOXARIFE;
                        }
                        else {
                            popupWidget.currentRow.CDALMOXARIFE = ' ';
                        }
                    }
                    else {
                        popupWidget.currentRow.CDALMOXARIFE = ' ';
                    }

                    if (!localField.readOnly || params.DontEditLocals) {
                        if (parentWidget.currentRow.CDLOCALESTOQ) {
                            popupWidget.currentRow.CDLOCALESTOQ = parentWidget.currentRow.CDLOCALESTOQ;
                        }
                        else if (prodfili.CDLOCALESTOQ && paramfilial.IDUTILLCESTQ == 'S') {
                            popupWidget.currentRow.CDLOCALESTOQ = prodfili.CDLOCALESTOQ;
                        }
                        else if (paramfilial.CDLOCALESTOQ && paramfilial.IDUTILLCESTQ == 'S') {
                            popupWidget.currentRow.CDLOCALESTOQ = paramfilial.CDLOCALESTOQ;
                        }
                        else {
                            popupWidget.currentRow.CDLOCALESTOQ = ' ';
                        }
                    }
                    else {
                        popupWidget.currentRow.CDLOCALESTOQ = ' ';
                    }

                    almoxField.reload().then((data) => {
                        if (popupWidget.currentRow.CDALMOXARIFE) {
                            let almoxData = data.dataset.almoxarifado.filter(field => {
                                return field.CDALMOXARIFE == popupWidget.currentRow.CDALMOXARIFE;
                            });

                            if (almoxData.length != 0) {
                                popupWidget.currentRow.DSALMOXARIFE = almoxData[0].DSALMOXARIFE;
                                almoxField.setValue(almoxData[0].DSALMOXARIFE);
                            }
                            else {
                                almoxField.setValue('');
                                popupWidget.currentRow.DSALMOXARIFE = '';
                            }

                            if (popupWidget.currentRow.CDLOCALESTOQ) {
                                localField.reload().then((data) => {
                                    localField.setValue(popupWidget.currentRow.CDLOCALESTOQ);
                                    let localData = data.dataset.localestoque.filter(field => {
                                        return field.CDLOCALESTOQ == popupWidget.currentRow.CDLOCALESTOQ;
                                    });
                                    if (localData.length != 0) {
                                        popupWidget.currentRow.DSLOCALESTOQ = localData[0].DSLOCALESTOQ;
                                        localField.setValue(localData[0].DSLOCALESTOQ);
                                    }
                                    else {
                                        localField.setValue(' ');
                                        popupWidget.currentRow.DSLOCALESTOQ = '';
                                    }
                                });
                            }
                        }
                    });
                });
            });
            ScreenService.openPopup(popupWidget).then(function() {
                if (parentWidget.infoComplOpenPopup)
                    parentWidget.infoComplOpenPopup();
            }.bind(this));
        }.bind(this));
    };

    this.setFieldLote = function(popupWidget, parentWidget, params) {
        var fieldLote = popupWidget.getField('NRLOTEESTQ');
        fieldLote.dataSourceFilter = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": rowParent.CDFILIMOVI
                }, {
                "name": "P_CDPRODUTO",
                "operator": "=",
                "value": rowParent.CDPRODESTO
                }, {
                "name": "P_NRLOTEESTQ",
                "operator": "=",
                "value": " "
                }, {
                "name": "P_NRSUBLOTE",
                "operator": "=",
                "value": " "
                }, {
                "name": "P_DTPOSICAO",
                "operator": "=",
                "value": rowParent.DTLANCMOVI
                }
            ];
        if (!params.ShowEntr || !params.Ajuste) {
            fieldLote.dataSource.name = 'est#/lote_infCompl_almox';
            fieldLote.dataSourceFilter = fieldLote.dataSourceFilter.concat([
                {
                    "name": "P_CDALMOXARIFE",
                    "operator": "=",
                    "value": "@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('" + parentWidget.name + "','CDALMOXARIFE')|| ' '@"
                }, {
                    "name": "P_CDLOCALESTOQ",
                    "operator": "=",
                    "value": "@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('" + parentWidget.name + "','CDLOCALESTOQ')|| ' '@"
                }, {
                    "name": "P_IDQUANTID",
                    "operator": "=",
                    "value": "S"
                }
            ]);
        }
        else {
            if (params.VerQtEst) {
                fieldLote.dataSource.name = 'est#/lote_infCompl_almox';
                fieldLote.dataSourceFilter = fieldLote.dataSourceFilter.concat([
                    {
                        "name": "P_CDALMOXARIFE",
                        "operator": "=",
                        "value": "@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('" + parentWidget.name + "','CDALMOXARIFE')|| ' '@"
                    }, {
                        "name": "P_DSLOCALESTOQ",
                        "operator": "=",
                        "value": "@Lib00206_InfComplProduto_Controller.getInfComplFieldVal('" + parentWidget.name + "','CDLOCALESTOQ')|| ' '@"
                    }, {
                        "name": "P_IDQUANTID",
                        "operator": "=",
                        "value": "N"
                    }
                ]);
            }
            else {
                fieldLote.dataSource.name = 'est#/lote_infCompl_semAlmox';
            }
        }
    };

    function notOpenPopup(widget, params = {}) {
        Lib00206_InfComplProduto_Service.validateInventory(widget, params).then(resolve => {
            let pendencies = resolve.length;
            if (pendencies > 0) {
                ScreenService.showMessage('Operação bloqueada! Inventário pendente para a filial.');
                ZHPromise.when(widget.beforeConfirmInfoCompl && widget.AlwaysCallsBeforeEvent !== false ? widget.beforeConfirmInfoCompl() : undefined).then(function() {
                    if (widget.infoComplInvalidStock)
                        widget.infoComplInvalidStock();
                });
            }
            else {
                ZHPromise.when(widget.beforeConfirmInfoCompl && widget.AlwaysCallsBeforeEvent !== false ? widget.beforeConfirmInfoCompl() : undefined).then(function() {
                    if (widget.afterConfirmInfoCompl && widget.AlwaysCallsAfterEvents !== false)
                        widget.afterConfirmInfoCompl();
                });
            }
        });
    }

    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };

    var locaisValidos = function(widget) {
        var almoxarifado = widget.getField('DSALMOXARIFE');
        var local = widget.getField('DSLOCALESTOQ');
        var nlote = widget.getField('NRLOTEESTQ');
        var utAlmoxarifado = !almoxarifado.readOnly;
        var utLocal = !local.readOnly;
        var utLote = !nlote.readOnly || lotAutomatico;
        var almoxarifadoValido = false;
        var localValido = false;
        var loteValido = false;
        var locaisValidos = false;
        var DontEditLocals = widget.getParent().InfoCompl.params.DontEditLocals;

        var arrLocais = ['DSALMOXARIFE', 'DSLOCALESTOQ', 'CDALMOXARIFE', 'CDLOCALESTOQ', 'NRLOTEESTQ', 'NRSUBLOTE'];

        arrLocais.forEach(local => {
            if (widget.currentRow[local] == "")
                widget.currentRow[local] = " ";
        });

        if ((utAlmoxarifado && almoxarifado.value() !== ' ') || (DontEditLocals && almoxarifado.value() !== ' '))
            almoxarifadoValido = true;
        else
        if (!utAlmoxarifado && almoxarifado.value() === ' ')
            almoxarifadoValido = true;

        if ((utLocal && local.value() !== ' ') || (DontEditLocals && local.value() !== ' '))
            localValido = true;
        else
        if (!utLocal && local.value() === ' ')
            localValido = true;

        if (utLote && nlote.value() !== ' ')
            loteValido = true;
        else
        if (!utLote && nlote.value() === ' ')
            loteValido = true;

        if (almoxarifadoValido && localValido && loteValido)
            locaisValidos = true;

        return locaisValidos;
    };

    this.saveLote = function(widget) {
        if (widget.isValid() && locaisValidos(widget)) {
            var row = widget.currentRow;
            if (row.DTVALILOTE && row.DTFABRPROD && moment(row.DTFABRPROD, 'DD/MM/YYYY').isAfter(moment(row.DTVALILOTE, 'DD/MM/YYYY')) && widget.params.IsEntrada) {
                ScreenService.showMessage('Data de fabricação deve ser menor ou igual a data de validade.');
            }
            else if (row.DTVALILOTE && moment(row.DTVALILOTE, 'DD/MM/YYYY').isBefore(moment(widget.getField('DTVALILOTE').minDate, 'DD/MM/YYYY')) && widget.params.IsEntrada) {
                ScreenService.showMessage('Data de validade deve ser maior que a data de processamento.');
            }
            else {
                var parent = widget.getParent();

                //atualiza currentrow com posição de estoque
                var lote = widget.getField('NRLOTEESTQ');
                var subLote = widget.getField('NRSUBLOTE');
                var loteOutData = lote.outData;

                lote.dataSourceFilter.forEach(function(params) {
                    if (params.name == "P_NRSUBLOTE")
                        params.value = subLote.value();
                    if (params.name == "P_NRLOTEESTQ")
                        params.value = lote.value();
                }.bind(this));

                Lib00206_InfComplProduto_Service.validateInventory(widget).then(resolve => {
                    let pendencies = resolve.length;
                    if (pendencies > 0) {
                        MessageService.getMessageByCode("VALIDAINVEROTAALMOX").then(almoxPhrase => {
                            let messageAlmox = ScreenService.i18n(almoxPhrase, { CDALMOX: widget.currentRow.CDALMOXARIFE });
                            if (widget.currentRow.CDALMOXARIFE != ' ' && widget.currentRow.CDLOCALESTOQ != ' ') {
                                MessageService.getMessageByCode("VALIDAINVEROTAALMOX").then(localPhrase => {
                                    let messageLocal = ScreenService.i18n(localPhrase, { CDLOCAL: widget.currentRow.CDLOCALESTOQ });
                                    ScreenService.showMessage(messageAlmox + messageLocal);
                                });
                            }
                            else if (widget.currentRow.CDALMOXARIFE != ' ' && widget.currentRow.CDLOCALESTOQ == ' ') {
                                ScreenService.showMessage(messageAlmox);
                            }
                            else {
                                ScreenService.showMessage('Operação bloqueada! Inventário pendente para a filial.');
                            }

                            if (parent.infoComplInvalidStock)
                                parent.infoComplInvalidStock();

                        });
                    }
                    else {
                        ZHPromise.when(parent.beforeConfirmInfoCompl ? parent.beforeConfirmInfoCompl() : undefined).then(function() {
                            Lib00206_InfComplProduto_Service.saveLote(row).then(
                                function() {
                                    rowParent.CDALMOXARIFE = row.CDALMOXARIFE;
                                    rowParent.CDLOCALESTOQ = row.CDLOCALESTOQ;
                                    rowParent.NRLOTEESTQ = row.NRLOTEESTQ;
                                    rowParent.NRSUBLOTE = row.NRSUBLOTE;
                                    rowParent.DTVALILOTE = row.DTVALILOTE;
                                    rowParent.QTESTOQDIA = (row.QTESTOQDIA) ? row.QTESTOQDIA : 0;
                                    ScreenService.closePopup();
                                    parent.activate();
                                    lote.reload().then(function(posEst) {
                                        try{
                                            if (posEst.dataset && posEst.dataset.lote_infCompl_almox.length > 0) {
                                                var valuesToSet = posEst.dataset.lote_infCompl_almox[0];
                                                setCurrentRowWithOutData(row, valuesToSet, loteOutData);
                                                setCurrentRowWithOutData(rowParent, valuesToSet, loteOutData);
                                            }
                                            else {
                                                rowParent.QTESTOQDIA = 0;
                                                rowParent.VRESTOQDIA = 0;
                                            }
                                        }catch(error){
                                            rowParent.QTESTOQDIA = 0;
                                            rowParent.VRESTOQDIA = 0;
                                        }
                                        if (parent.afterConfirmInfoCompl)
                                            parent.afterConfirmInfoCompl();
                                    }.bind(this));

                                },
                                function(error) {
                                    ScreenService.showMessage(error.data.error);
                                }
                            );
                        });
                    }
                });

            }
        }
    };

    var createInfComplProdutoPopup = function(parentWidget) {
        var widget = parentWidget.InfoCompl;
        if (!widget) {
            widget = getPopUpObject();
            widget.container = parentWidget.container;
            widget = metaDataFactory.widgetFactory(widget, parentWidget);
            parentWidget.InfoCompl = widget;
            widget.parent = parentWidget;
            return widget;
        }
        else {
            return widget;
        }
    };

    this.getInfComplFieldVal = function(parentName, fieldName) {
        return templateManager.container.getWidget(parentName).InfoCompl.getField(fieldName).value();
    };

    this.setReadOnlyField = function(popupWidget, params) {
        return Lib00206_InfComplProduto_Service.getParamFilial(popupWidget.currentRow.CDFILIMOVI).then(function(paramfilial) {
            paramfilial = paramfilial.dataset.paramfilial[0];
            lotAutomatico = paramfilial.IDENTRAUTLOTE === 'S';
            var idnovolote = popupWidget.getField('IDNOVOLOTE').value();
            popupWidget.getField('DSALMOXARIFE').readOnly = paramfilial.IDUTILALMOX == 'N' || !params.AtuEstoq || params.DontEditLocals;
            popupWidget.getField('DSLOCALESTOQ').readOnly = paramfilial.IDUTILLCESTQ == 'N' || !params.AtuEstoq || params.DontEditLocals;
            popupWidget.getField('DTVALILOTE').readOnly = paramfilial.IDUTILLOTE == 'N' || !params.ShowEntr || params.DocuDevo || !idnovolote;
            popupWidget.getField('NRLOTEESTQ').readOnly = paramfilial.IDUTILLOTE == 'N' || !params.AtuEstoq || params.DocuDevo || (paramfilial.IDENTRAUTLOTE === 'S' && params.ShowEntr);

            popupWidget.getField('NRSUBLOTE').readOnly = popupWidget.getField('NRLOTEESTQ').readOnly || !idnovolote || (paramfilial.IDENTRAUTLOTE === 'S' && params.ShowEntr);
            popupWidget.getField('DTFABRPROD').readOnly = (paramfilial.IDUTILLOTE !== 'S') || !params.ShowEntr || params.DocuDevo || !idnovolote;

            if (paramfilial.IDUTILLOTE === 'S' && idnovolote) {
                if (!popupWidget.getField('DTVALILOTE').validations.hasOwnProperty('required'))
                    popupWidget.getField('DTVALILOTE').validations = { "required": { "message": "Campo Obrigatorio" } };
            }
            else {
                delete popupWidget.getField('DTVALILOTE').validations.required;
                popupWidget.getField('DTVALILOTE').setValue(null);
                popupWidget.currentRow.DTVALILOTE = null;
            }

            if (!popupWidget.getField('DSALMOXARIFE').readOnly) {
                if (!popupWidget.getField('DSALMOXARIFE').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = { "required": { "message": "Campo Obrigatorio" } };
            }
            else {
                delete popupWidget.getField('DSALMOXARIFE').validations.required;
            }

            if (!popupWidget.getField('DSLOCALESTOQ').readOnly) {
                if (!popupWidget.getField('DSLOCALESTOQ').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = { "required": { "message": "Campo Obrigatorio" } };
            }
            else {
                delete popupWidget.getField('DSLOCALESTOQ').validations.required;
            }

            if (!popupWidget.getField('NRLOTEESTQ').readOnly) {
                if (!popupWidget.getField('NRLOTEESTQ').validations.hasOwnProperty('required'))
                    popupWidget.getField('NRLOTEESTQ').validations = { "required": { "message": "Campo Obrigatorio" } };
            }
            else {
                delete popupWidget.getField('NRLOTEESTQ').validations.required;
            }

            if (idnovolote) {
                popupWidget.getField('NRLOTEESTQ').template = 'field/text-edit.html';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[2].value = ' ';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[3].value = ' ';
            }
            else {
                popupWidget.getField('NRLOTEESTQ').template = 'zh-select-autocomplete#field/select-autocomplete.html';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[2].value = 'TD';
                popupWidget.getField('NRLOTEESTQ').dataSourceFilter[3].value = 'TD';
            }

            let useUniLote = paramfilial.IDENTRUNILOTE == 'S' ? true : false;
            this.liberaEdit(popupWidget, !useUniLote);

            popupWidget.reload();

            return paramfilial;
        }.bind(this));
    };

    this.idNovoLoteChange = function(widget) {
        this.setReadOnlyField(widget, widget.params);

        widget.getField('NRLOTEESTQ').value(null);
        widget.getField('NRSUBLOTE').value(null);

        if (widget.getField('IDNOVOLOTE').value())
            widget.getField('NRSUBLOTE').setValue(' ');

        widget.getField('DTVALILOTE').value(null);
        widget.getField('DTFABRPROD').value(null);
    };

    this.loteOnChange = function(widget) {
        var novoLote = widget.getField('IDNOVOLOTE').value();

        if (novoLote) {
            var nrLote = widget.getField('NRLOTEESTQ');
            nrLote.dataSourceFilter[2].value = nrLote.value();
        }

    };

    this.cancelaLancamento = function(widget) {
        var parent = widget.getParent();

        //Lib00206_InfComplProduto_Service.getParamFilial(rowParent.CDFILIMOVI).then(function(paramfilial){
        // paramfilial = paramfilial.dataset.paramfilial[0];
        ZHPromise.when(parent.beforeCancelInfoCompl ? parent.beforeCancelInfoCompl() : undefined).then(function() {
            ScreenService.closePopup();
            if (parent.afterCancelInfoCompl)
                parent.afterCancelInfoCompl();
        });

        //});
    };

    this.almoxarifeChange = function(args) {
        var fieldLocalEstoq = args.owner.field.widget.getField('DSLOCALESTOQ');
        var fieldCdLocalEstoq = args.owner.field.widget.getField('CDLOCALESTOQ');
        var fieldLote = args.owner.field.widget.getField('NRLOTEESTQ');
        var fieldSubLote = args.owner.field.widget.getField('NRSUBLOTE');

        fieldCdLocalEstoq.setValue(' ');
        fieldLocalEstoq.setValue(' ');
        // if (fieldLocalEstoq.readOnly || fieldLocalEstoq.readOnlyOnEdit){
        //     fieldCdLocalEstoq.setValue(' ');
        //     fieldLocalEstoq.setValue(' ');
        // }else{
        //     fieldCdLocalEstoq.setValue(' ');
        //     fieldLocalEstoq.setValue(' ');
        // }
        fieldLocalEstoq.reload();

        if (fieldLote.readOnly || fieldLote.readOnlyOnEdit)
            fieldLote.setValue(' ');
        else
            fieldLote.setValue('');

        fieldLote.reload();

        if (fieldSubLote.readOnly || fieldSubLote.readOnlyOnEdit)
            fieldSubLote.setValue(' ');
        else
            fieldSubLote.setValue('');

    };

    this.localChange = function(args) {

        var fieldLote = args.owner.field.widget.getField('NRLOTEESTQ');
        var fieldSubLote = args.owner.field.widget.getField('NRSUBLOTE');

        if (!lotAutomatico) {
            if (fieldLote.readOnly || fieldLote.readOnlyOnEdit)
                fieldLote.setValue(' ');
            else
                fieldLote.setValue('');
        }

        fieldLote.reload();

        if (fieldSubLote.readOnly || fieldSubLote.readOnlyOnEdit)
            fieldSubLote.setValue(' ');
        else
            fieldSubLote.setValue('');
    };

    this.liberaEdit = function(widget, status = false) {
        let actionEdit = widget.getAction('liberar');
        actionEdit.readOnly = status;
        actionEdit.isVisible = !status;
    };

    this.liberaJustificativa = function(widget) {
        let fieldJustific = widget.getField('DSJUSTIFICATIVA');
        fieldJustific.validations = { "required": { "message": "Campo Obrigatorio" } };
        fieldJustific.isVisible = true;
        widget.getField('NRLOTEESTQ').readOnly = false;
        widget.getField('NRSUBLOTE').readOnly = false;
    };

    var getPopUpObject = function() {
        return {
            "id": "21216069734072063069284",
            "isVisible": true,
            "name": "infComplProduto",
            "label": "Informações Complementares do Produto",
            "template": "widget/form.html",
            "currentRow": {

            },
            "showAsTab": false,
            "actions": [
                {
                    "id": "7645275172362581639320",
                    "name": "cancelAction",
                    "label": "Cancelar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "id": "7645275172117518733321",
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.cancelaLancamento(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "7645275174220205724322",
                    "name": "confirmAction",
                    "label": "Confirmar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "id": "7645275171976449286323",
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.saveLote(args.owner.widget)"
                        }
                    ]
                },
                {
                    "id": "7645275173212107606324",
                    "name": "liberar",
                    "label": "Liberar Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "showAsAction": "view|edit|always",
                    "icon": "key",
                    "events": [
                        {
                            "id": "7645275172373060524325",
                            "name": "ActionEvent",
                            "code": "Lib00206_InfComplProduto_Controller.liberaJustificativa(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "id": "7645275172050376200285",
                    "name": "NMPRODUTO",
                    "label": "Produto",
                    "isVisible": true,
                    "readOnly": true,
                    "readOnlyOnEdit": true,
                    "class": "6",
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "defaultValue": null,
                    "descriptionOrder": 1,
                    "template": "field/text-edit.html"
                },
                {
                    "id": "764527517786673367286",
                    "name": "CDARVPROD",
                    "label": "Cod. Produto",
                    "isVisible": true,
                    "readOnly": true,
                    "readOnlyOnEdit": true,
                    "class": "6",
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "defaultValue": null,
                    "descriptionOrder": 1,
                    "template": "field/text-edit.html"
                },
                {
                    "id": "7645275171667128356287",
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "template": "field/text-edit.html",
                    "isVisible": false
                },
                {
                    "id": "764527517256954914288",
                    "name": "DSALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "showOnView": true,
                    "class": 6,
                    "order": "CDALMOXARIFE",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "CDALMOXARIFE",
                        "DSALMOXARIFE"
                    ],
                    "displayedFields": [
                        "CDALMOXARIFE",
                        "DSALMOXARIFE"
                    ],
                    "showSelectAs": "grid",
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "est#/crud/almoxarifado"
                    },
                    "dataSourceFilter": [
                        {
                            "name": "CDALMOXARIFE",
                            "operator": "<>",
                            "value": " "
                        }
                    ],
                    "descriptionField": "DSALMOXARIFE",
                    "valueField": "CDALMOXARIFE",
                    "outData": {
                        "CDALMOXARIFE": "CDALMOXARIFE",
                        "DSALMOXARIFE": "DSALMOXARIFE"
                    },
                    "fields": [
                        {
                            "id": "7645275172423013164289",
                            "label": "Código",
                            "name": "CDALMOXARIFE",
                            "showOnList": true,
                            "template": "field/text-edit.html",
                            "isVisible": true
                        },
                        {
                            "id": "7645275172010074121290",
                            "label": "Almoxarifado",
                            "name": "DSALMOXARIFE",
                            "showOnList": true,
                            "isVisible": true
                        }
                    ],
                    "events": [
                        {
                            "id": "7645275174009701249291",
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
                        {
                            "id": "7645275171594863820292",
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.initializeField();"
                        },
                        {
                            "id": "7645275172458436873293",
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.almoxarifeChange(args);"
                        }
                    ],
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": "7645275172096538440294",
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "template": "field/text-edit.html",
                    "isVisible": false
                },
                {
                    "id": "7645275173944624007295",
                    "name": "DSLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 6,
                    "order": "CDLOCALESTOQ",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "CDLOCALESTOQ",
                        "DSLOCALESTOQ"
                    ],
                    "displayedFields": [
                        "CDLOCALESTOQ",
                        "DSLOCALESTOQ"
                    ],
                    "showSelectAs": "grid",
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "est#/crud/localestoque"
                    },
                    "dataSourceFilter": [
                        {
                            "name": "CDLOCALESTOQ",
                            "operator": "<>",
                            "value": " "
                        }
                    ],
                    "descriptionField": "DSLOCALESTOQ",
                    "valueField": "CDLOCALESTOQ",
                    "outData": {
                        "CDLOCALESTOQ": "CDLOCALESTOQ",
                        "DSLOCALESTOQ": "DSLOCALESTOQ"
                    },
                    "fields": [
                        {
                            "id": "764527517131182512296",
                            "label": "Código",
                            "name": "CDLOCALESTOQ",
                            "showOnList": true
                        },
                        {
                            "id": "7645275173062385830297",
                            "label": "Descrição",
                            "name": "DSLOCALESTOQ",
                            "showOnList": true
                        }
                    ],
                    "events": [
                        {
                            "id": "7645275172456446980298",
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
                        {
                            "id": "7645275171701435431299",
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.localChange(args);"
                        }
                    ],
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": "7645275172186633453300",
                    "name": "IDNOVOLOTE",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": false,
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "dataSource": {
                        "data": [
                            {
                                "IDNOVOLOTE": false,
                                "DSNOVOLOTE": "Lote Existente"
                            },
                            {
                                "IDNOVOLOTE": true,
                                "DSNOVOLOTE": "Novo Lote"
                            }
                        ]
                    },
                    "valueField": "IDNOVOLOTE",
                    "descriptionField": "DSNOVOLOTE",
                    "hideEmptyValue": true,
                    "template": "field/select-native.html",
                    "class": 2,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "placeholder": null,
                    "events": [
                        {
                            "id": "7645275172057310023301",
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.idNovoLoteChange(args.owner.field.widget)"
                        },
                        {
                            "id": "7645275172888296845302",
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.reload()"
                        },
                        {
                            "id": "7645275173605769469303",
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.reload()"
                        }
                    ]
                },
                {
                    "id": "7645275171631791325304",
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 5,
                    "order": "NRLOTEESTQ",
                    "template": "zh-select-autocomplete#field/select-autocomplete.html",
                    "searchableFields": [
                        "NRLOTEESTQ",
                        "DTVALILOTE"
                    ],
                    "displayedFields": [
                        "NRLOTEESTQ",
                        "DTVALILOTE"
                    ],
                    "showSelectAs": "grid",
                    "maxlength": 20,
                    "dataSource": {
                        "rest": true,
                        "lazyLoad": true,
                        "name": "est#/lote_infCompl_semAlmox"
                    },
                    "dataSourceFilter": [

                    ],
                    "descriptionField": "NRLOTEESTQ",
                    "valueField": "NRLOTEESTQ",
                    "outData": {
                        "NRLOTEESTQ": "NRLOTEESTQ",
                        "NRSUBLOTE": "NRSUBLOTE",
                        "DTVALILOTE": "DTVALILOTE",
                        "QTESTOQDIA": "QTESTOQDIA",
                        "VRESTOQDIA": "VRESTOQDIA",
                        "CDFABRIC": "CDFABRIC"
                    },
                    "fields": [
                        {
                            "id": "7645275173452057162305",
                            "label": "Lote",
                            "name": "NRLOTEESTQ",
                            "showOnList": true
                        },
                        {
                            "id": "7645275171246163737306",
                            "label": "Sublote",
                            "name": "NRSUBLOTE",
                            "showOnList": true
                        },
                        {
                            "id": "764527517734593004307",
                            "label": "Validade",
                            "name": "DTVALILOTE",
                            "showOnList": true,
                            "displayFormat": "DD/MM/YYYY"
                        },
                        {
                            "id": "7645275172010963284308",
                            "label": "Almoxarifado",
                            "name": "CDALMOXARIFE",
                            "showOnList": true
                        },
                        {
                            "id": "7645275172611142362309",
                            "label": "Localização",
                            "name": "CDLOCALESTOQ",
                            "showOnList": true
                        },
                        {
                            "id": "7645275173894634281310",
                            "label": "Quantidade",
                            "name": "QTESTOQDIA",
                            "showOnList": true,
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "precision": 2,
                                    "showSymbol": false
                                }
                            }
                        },
                        {
                            "id": "764527517211814200311",
                            "label": "Valor Total",
                            "name": "VRESTOQDIA",
                            "showOnList": true,
                            "mask": {
                                "type": "viewFloat",
                                "params": {
                                    "precision": "2"
                                }
                            }
                        }
                    ],
                    "events": [
                        {
                            "id": "7645275172782039121312",
                            "name": "FieldOnClick",
                            "code": "args.owner.field.initializeField();"
                        },
                        {
                            "id": "7645275172665846479313",
                            "name": "FieldBeforeSelectOpen",
                            "code": "args.owner.field.reload()"
                        },
                        {
                            "id": "764527517516568362314",
                            "name": "FieldOnChange",
                            "code": "Lib00206_InfComplProduto_Controller.loteOnChange(args.owner.field.widget)"
                        }
                    ],
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    }
                },
                {
                    "id": "7645275171199021922315",
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "readOnly": false,
                    "defaultValue": null,
                    "maxlength": 4,
                    "order": "NRSUBLOTE",
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 5,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {

                    },
                    "placeholder": null,
                    "events": [

                    ],
                    "validations": {

                    }
                },
                {
                    "id": "76452751786488369316",
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true,
                    "readOnly": true,
                    "defaultValue": null,
                    "filterProperties": {
                        "template": "field/interval.html",
                        "readOnly": false
                    },
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {

                    },
                    "placeholder": null,
                    "validations": {
                        "required": {
                            "message": "Campo obrigatório"
                        }
                    },
                    "events": [
                        {
                            "id": "7645275171546533583317",
                            "name": "FieldOnChange",
                            "code": "args.owner.field.widget.activate();"
                        }
                    ]
                },
                {
                    "id": "7645275171441536159318",
                    "name": "DTFABRPROD",
                    "label": "Data de Fabricação",
                    "isVisible": true,
                    "readOnly": false,
                    "defaultValue": null,
                    "filterProperties": {
                        "template": "field/interval.html"
                    },
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "descriptionOrder": 4,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {

                    },
                    "placeholder": null,
                    "events": [

                    ]
                },
                {
                    "id": "7645275171723222255319",
                    "name": "DSJUSTIFICATIVA",
                    "label": "Justificativa",
                    "isVisible": false,
                    "readOnly": false,
                    "defaultValue": null,
                    "maxlength": 50,
                    "filterProperties": {
                        "isVisible": false,
                        "class": 0,
                        "newLine": false
                    },
                    "template": "field/text-edit.html",
                    "class": 12,
                    "showOnForm": true,
                    "showOnList": true,
                    "newLine": true,
                    "dataSource": {

                    },
                    "placeholder": null,
                    "events": [

                    ],
                    "validations": {

                    }
                }
            ],
            "events": [
                {
                    "id": "764527517322794014326",
                    "name": "WidgetOnEnter",
                    "code": "args.owner.getField('DSALMOXARIFE').reload();"
                }
            ]
        };
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00206_InfComplProduto_Controller', Lib00206_InfComplProduto_Controller);
});


// FILE: ./assets/js/utilities/Lib00207_PosicaoEstoque_Controller.js
var Lib00207_PosicaoEstoque_Controller = function(ScreenService,ZHPromise,metaDataFactory,Lib00207_PosicaoEstoque_Service,eventEngine) {

    
    this.posicaoEstoque = function(widget, rowItem) {
        var rowParent = rowItem;
        var popUp = this.getPosicaoDeEstoquePopUpObject();
        popUp.label = popUp.defaultLabel + rowItem.NMPRODUTO;
        
        if(!(rowParent.CDFILIAL && rowParent.CDPRODUTO && rowParent.CDARVPROD && rowParent.NMPRODUTO)){
            console.error('Row do Item incompleta');
        }
        else {
          this.openPosicaoDeEstoque(widget, rowItem);
        }
    };
    
    this.openPosicaoDeEstoque = function(parentWidget, rowItem) {
        var widget = this.getPosicaoDeEstoquePopUpObject();
        widget.container = parentWidget.container;
        widget = metaDataFactory.widgetFactory(widget,parentWidget);
        parentWidget.PosEstoq = widget;
        widget.parent = parentWidget;
        widget.currentRow = rowItem;
        widget.label = widget.defaultLabel + rowItem.NMPRODUTO;
		ScreenService.openPopup(widget);
        this.getDataForPosicaoDeEstoquePopup(widget);
    };
    
    this.getPosicaoDeEstoquePopUpObject = function() {
    	return {
            "name": "posestPopup",
            "defaultLabel": "Posição de Estoque do Produto ",
            "template": "widget/form.html",
            "isVisible": true,
            "showAsTab": false,
            "actions": [
                {
                    "name": "back",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup()"
                        }
                    ]
                },
                {
                    "name": "refresh",
                    "label": "Atualizar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "Lib00207_PosicaoEstoque_Controller.getDataForPosicaoDeEstoquePopup(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "QTESTOQDIA",
                    "label": "Quantidade",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true,
                    "sequenceInForm": 80,
                    "readOnly": true,
                    "defaultValue": null,
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                }
            ]
        };
    };
    
    this.getDataForPosicaoDeEstoquePopup = function(popup) {
    	Lib00207_PosicaoEstoque_Service.getPosicaoDeEstoquePorProduto(popup.currentRow).then(function(posestdia) {
    		if(posestdia) {
	    		popup.currentRow = posestdia;
	    		popup.parent.currentRow.QTESTOQDIA = posestdia.QTESTOQDIA;
    		}
    	}).catch(function(error) {
    		ScreenService.showMessage(error.data.error);
    	});
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Lib00207_PosicaoEstoque_Controller', Lib00207_PosicaoEstoque_Controller);
});

// FILE: ./assets/js/utilities/EstUtil.js
var EstUtil = function(templateManager, General, ZhFilterPreferences, ZHPromise, ScreenService, restEngine, LibUtilities, MessageService, localStorageEngine) {

	this.getFieldValue = function(filedName, widgetName, fromFilter = false) {
		try {
			var container = templateManager.container;
			if (container) {
				let valuesWidget = container.getWidget(widgetName);
				if (fromFilter) {
					valuesWidget = templateManager.container.factoredWidgets.filter((widget) => { return widget._filteredWidgetId == valuesWidget.id; })[0];
				}

				if (valuesWidget) {
					var valueField = valuesWidget.currentRow[filedName];
					return valueField || null;
				}
				else return null;
			}
			else return null;
		}
		catch (error) {
			return null;
		}
	};

	// CRUD
	/**
	 * closeSwipe parametro que fecha ou nao o swipeApós o save
     * 
     * modeToReturn força o mode para o do parametro após o save, caso contrario volta para list para newRow e view para edit
     * 
     * O segundo parâmetro é opcional, sendo ele um objeto contendo os atributos
     * callback (function) e params (array). Após a validação da widget,
     * o método callback é executado.
     * Ex: var afterValid = {
     *      callback: function(a) {console.log(a);},
            params: ['valorDeA']
     * };
     */
	this.saveCrud = function(widget, closeSwipe = true, modeToReturn = undefined, afterValid = undefined, autoUpdateColumnsAfter = true) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {
			if (afterValid && angular.isObject(afterValid)) {
				if (angular.isFunction(afterValid.callback))
					afterValid.callback.apply(null, afterValid.params);
			}

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_interval = paramsNewCode.__new_code_interval || 1;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__new_code_column_size = paramsNewCode.__new_code_column_size || 10;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
			}

			if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
				widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length) {
				setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function(success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.currentRow.__is_new = false;
					widget.currentRow.__createdLocal = false;

					if (modeToReturn)
						widget[modeToReturn]();
					else
						widget.list(closeSwipe);

					if (closeSwipe)
						ScreenService.goBack();

					widget.reload().then(function() {
						deferred.resolve(success);
					});
				}
				else {
					widget.computeDescriptionOrder();

					if (modeToReturn)
						widget[modeToReturn]();
					else
						widget.view();

					deferred.resolve(success);
				}

				if (autoUpdateColumnsAfter && widget.currentRow.__columns_to_return) {
					widget.currentRow.__columns_to_return.forEach(column => {
						widget.currentRow[column] = widget.currentRow.__columns_to_return[0][column];
					});
				}
			}, function(error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		}
		else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};

	/* setDuplicationValidateParamsToCurrentRow
	 *
	 * Caso o widget contenha o array 'fieldsToValidateDuplication', será verifi-
	 * cado, antes de salvar, se os valores dos campos a serem validados já exis-
	 * tem no banco.
	 *
	 * O array deve conter objeto(s) com o nome da coluna na tabela e a descricao
	 * do campo que sera exibida na resposta em caso de duplicacao.
	 *
	 * Exemplo:
	 *      "fieldsToValidateDuplication": [
	 *        {
	 *	        "name": "NMCAMPO",
	 *	        "description": "campo"
	 *        }
	 *      ]
	 */
	var setDuplicationValidateParamsToCurrentRow = function(widget) {
		try {
			var fieldsToValidate = widget.fieldsToValidateDuplication;
			var validationArray = [];

			checkValidateFieldsArray(fieldsToValidate);

			for (var index in fieldsToValidate) {
				var field = fieldsToValidate[index];

				if (!field.description) {
					field.description = field.name;
					var warningMsg = "Missing description for " + field.name +
						" field in 'fieldsToValidateDuplication' of " + widget.name;
					console.warn(warningMsg);
				}
				field.description = field.description.toLowerCase();

				validationArray.push(field);
			}

			if (validationArray.length) {
				widget.currentRow.__fieldsToValidateDuplication = validationArray;
			}
		}
		catch (e) {
			ScreenService.showMessage(e);
		}
	};

	var checkValidateFieldsArray = function(arrayToCheck) {
		for (var index in arrayToCheck) {
			var objectToCheck = arrayToCheck[index];

			if (typeof(objectToCheck) == "object") {
				if (!objectToCheck.name) {
					throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has elements without name.";
				}
			}
			else {
				throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has non-object elements.";
			}
		}
	};

	this.parseStrToFloat = function(value) {
		if (!!value) {
			value = String(value);
			var numberOfDots = (value.match(/\./g) || []).length,
				hasVirgula = !!(value.match(/\,/g) || []).length;

			if (hasVirgula) {
				if (!!numberOfDots) {
					value = value.replace(/\./g, "");
				}

				value = value.replace(",", ".");
			}
		}
		else {
			value = 0.00;
		}

		return parseFloat(value);
	};

	this.rowEditedInline = function(row) {
		let boolEdited = false;

		$.each(row, function(index, value) {
			if (row.hasOwnProperty('changed_' + index)) {
				boolEdited = true;
			}
		});

		return boolEdited;
	};

	this.fixInlineEdit = function(row) {
		$.each(row, function(index, value) {
			if (row.hasOwnProperty('changed_' + index)) {
				row[index] = row[index + '_inline'] || row[index];
				row['changed_' + index] = row[index + '_inline'] || row[index];
			}
		});
		return row;
	};

	this.fixInlineEditDataSet = function(dataSet) {
		dataSet.forEach(row => {
			row = this.fixInlineEdit(row);
		});
		return dataSet;
	};

	this.saveDataSetCrud = function(dataSet, name) {
		return restEngine.post("est#/crud/" + name + "/save", {
			"requestType": "DataSet",
			"dataset": dataSet
		});
	};

	this.cancelInlineEdit = function(widget) {
		/* jshint ignore:start */

		// TODO criar função para desmarcar linhas editadas
		// GerUtil.desmarcaLinhasEditadas(lancamentoEncargoWidget, CANCELAR);

		for (var index in widget.dataSource.data) {
			let row = widget.dataSource.data[index];
			Object.keys(row).forEach(function(key) {
				if (key.indexOf('changed_') > -1) {
					row[key.replace('changed_', '')] = row[key];
					delete row[key];
				}
			});
			delete row.changed;
		}

		widget.list();

		/* jshint ignore:end */
	};

	this.getFilterWidget = function(valuesWidget) {
		return templateManager.container.factoredWidgets.filter((widget) => { return widget._filteredWidgetId == valuesWidget.id; })[0];
	};

	this.openBlankWindow = function(args) {
		args.e.preventDefault();
		ScreenService.openWindow("est#ger03302_blank_window");
	};

	this.saveCollectedDataGeneric = function(widget, description = undefined, fromFilter = false, ids = undefined) {
		let collectionIds;
		let valuesWidget = widget;
		if (fromFilter) {
			valuesWidget = templateManager.container.factoredWidgets.filter((widget) => { return widget._filteredWidgetId == valuesWidget.id; })[0];
		}

		if (valuesWidget) {
			if (valuesWidget.currentRow) {
				if (valuesWidget.currentRow.CDFILIMOVI)
					collectionIds = valuesWidget.currentRow.CDFILIMOVI;
				else if (valuesWidget.currentRow.CDFILIAL)
					collectionIds = valuesWidget.currentRow.CDFILIAL;
				else
					return null;
			}
			else {
				return null;
			}
		}
		else {
			return null;
		}

		//widget instanceof Container
		let operationId = '';
		MessageService.getMessageByCode("UTLTELA").then(fraseComParametro => {
			operationId = ScreenService.i18n(fraseComParametro, { NMTELA: description });
			/*
			 * Lista de descriptions utilizadas para tentar padronizar chamadas:
			 * ContainerAfterinit
			 */
			/*
			 * @param $collectionIds  => ID ou ARRAY da(s) coleta(s) (Cod Filial, Cod Operador, Cod da Loja, ...)
			 * @param $operationId    => ID da operação (Retirada, Cardápio, Login, ...)
			 * @param $productId      => Código do Produto (opcional).
			 * @param $operatorCode   => Código do Operador (opcional).
			 * @param $operatorName   => Nome do Operador (opcional).
			 * @param $organizationId => Código do organização (opcional).
			 */
			LibUtilities.saveCollectedData(collectionIds, operationId);
		});
	};

	var getWidgetClientsByOperator = function() {
		return templateManager.container.getWidget('clientsByOperator');
	};

	// this.getLocalVar = function(key) {
	//     return localStorageEngine.getLocalVar(key);
	// };

	this.nextContainerAfterInit = function(container, route) {
		setTimeout(() => {

			var token = localStorage.getItem('LOGIN_TOKEN');
			var hash = localStorage.getItem('LOGIN_HASH') || '0';
			var languange = localStorage.getItem('LOGIN_LANGUAGE');
			var user = getLocalVar('NEXTSESSION_ENCRYPTED');
			var productId = window.projectId;

			var url = window.nextUrl + 'modules/est-next/' + route + "?header=0&menu=0" +
				"&token=" + token +
				"&hash=" + hash +
				"&user=" + user +
				"&languange=" + languange +
				"&productId=" + productId +
				"&isZhAngular=1";

			// Clear OM session timeout to use only OM Next session timeout.
			//clearTimeout(LibUtilities.expiredSessionMethod);

			container.widgets[0].sourceUrl = url;
			container.widgets[0].reload();
		}, 1000);
	};

	// this.nextContainerAfterInit = function (container, route) {
	// 	try{
	// 		if(typeof memoryStorage !== 'undefined') {
	//         localStorageEngine.getLocalVar = memoryStorage.getLocalVar;
	//         localStorageEngine.setLocalVar = memoryStorage.setLocalVar;
	//         setLocalVar = memoryStorage.setLocalVar;
	//         getLocalVar = memoryStorage.getLocalVar;
	//     }

	// 		getLocalVar('loginPromise').then(()=>{
	//      var token = localStorage.getItem('LOGIN_TOKEN');
	//      var hash = localStorage.getItem('LOGIN_HASH') || '0';
	//      var user      = localStorage.getItem('NEXTSESSION_ENCRYPTED');
	//      var languange = localStorage.getItem('LOGIN_LANGUAGE');
	//      var productId = window.projectId;

	//      var url = window.nextUrl + 'modules/est-next/' + route + "?header=0&menu=0" +
	//                                                   "&token=" + token +
	//                                                   "&hash=" + hash +
	//                                                   "&user=" + user +
	//                                                   "&languange=" + languange +
	//                                                   "&productId=" + productId +
	//                                                   "&isZhAngular=1";

	//      // Clear OM session timeout to use only OM Next session timeout.
	//      //clearTimeout(LibUtilities.expiredSessionMethod);

	//      container.widgets[0].sourceUrl = url;
	// 		});
	// 	}catch(error){
	// 		return null;
	// 	}
	// };

	/*
	 * This function validate all filter required fields and checks if all of 
	 * them have values before opening a new row.
	 *
	 * It Only Checks Fields which has validations on filterProperties
	 *
	 * @param widget => The widget which contains add action,
	 * @param openNewRow => boolean value, it checks if the function will open a new row or not,
	 */

	this.onValidateFilterRequirement = function(widget, openNewRow = false) {
		let checkedFields = [];
		const filterWidget = this.getFilterWidget(widget);
		const requiredFields = widget.fields.filter((field) => {
			if (field.hasOwnProperty('filterProperties')) {
				if (field.filterProperties.hasOwnProperty('validations')) {
					if (field.filterProperties.validations.hasOwnProperty('required')) {
						if (field.filterProperties.validations.required)
							return field.name;
					}
				}
			}
		});

		if (typeof widget.appliedFilter == 'undefined') {
			ZhFilterPreferences.openFilterWidget(widget, filterWidget.currentRow);
			MessageService.showMessageByCode("BLOCK_FILTER_ERROR");
			return true;
		}
		const { FILTER_ROW: filteredFields } = widget.appliedFilter;

		requiredFields.map((reqField) => {
			for (const filField in filteredFields) {
				if (reqField.name === filField && filteredFields[filField]) {
					checkedFields.push(filField);
					break;
				}
			}
		});

		if (requiredFields.length !== checkedFields.length) {
			ZhFilterPreferences.openFilterWidget(widget, filterWidget.currentRow);
			MessageService.showMessageByCode("BLOCK_FILTER_ERROR");
			return true;
		}
		else {
			if (openNewRow) widget.newRow();
			return false;
		}
		
		// if (!filterWidget.isValid()) {
		// 	ZhFilterPreferences.openFilterWidget(widget, filterWidget.currentRow);
		// 	MessageService.showMessageByCode("BLOCK_FILTER_ERROR");
		// 	return true;
		// }
		// else {
		// 	if (openNewRow) widget.newRow();
		// 	return false;
		// }
	};
};

Configuration(function(ContextRegister) {
	ContextRegister.register('EstUtil', EstUtil);
});


// FILE: ./assets/js/utilities/ProductField.js
function ProductField(templateManager) {
    const isUserFiltering = ({selectWidget: widget}) => !!widget.filterApplied.length;

    const updateInnerWidgetExhibitionMode = ({selectWidget: widget}, mode) => {
        const treeModifier = mode === 'tree'? 'tree/' : '';

        widget.template = `widget/${treeModifier}grid.html`;

        return templateManager.updateTemplate();
    };

    //Tree behavior blocked, waiting Zeedhi Framework to become faster
    this.updateExhibitionMode = field => {
        const mode = 'grid'; //isUserFiltering(field)? 'grid' : 'tree';

        field.showSelectAs = mode;

        return updateInnerWidgetExhibitionMode(field, mode).then(() => mode);
    };

    this.updateProductFilters = widget => {
        const fields = widget.fields.filter(field => field.label.startsWith('Produto'));
        const {CDFILIAL} = widget.currentRow;


        //switch to updateSelectFieldFilialFilter when Zeedhi Framework manage to become faster
        fields.forEach(removeSelectFieldFilialFilter);
    };

    const isFilterable = field => field.template.includes("select") &&
                                 !field.template.includes("native") &&
                                !!field.dataSourceFilter;

    const removeSelectFieldFilialFilter = field => {
        if(isFilterable(field)) {
            field.dataSourceFilter = field.dataSourceFilter.filter(filter => filter.name != 'CDFILIAL');
            field.reload();
        }
    };

    const updateSelectFieldFilialFilter = field => {
        if(isFilterable(field)) {
            field.dataSourceFilter.find(filter => filter.name == 'CDFILIAL').value = CDFILIAL;
            field.reload();
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ProductField', ProductField);
});


// FILE: ./assets/js/repositories/Est33000_RequisicaoEstoque_Repository.js
var Est33000_RequisicaoEstoque_Repository = function (restEngine) {

    this.saveRequisicaoProdutos = function(rows) {
        return restEngine.post('est#/saveNovaRequisicaoProdutos', {
            "requestType": "Row",
            "row": rows
        });
    };

    this.saveNovoProduto = function(row) {
        return restEngine.post('est#/saveNovoProduto', {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteProduto = function(row) {
        return restEngine.post('est#/deleteProdutoRequisicao', {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteRequisicao = function(row) {
        return restEngine.post('est#/deleteRequisicao', {
            "requestType": "Row",
            "row": row
        });
    };


};

Configuration(function(ContextRegister){
    ContextRegister.register('Est33000_RequisicaoEstoque_Repository', Est33000_RequisicaoEstoque_Repository);
});

// FILE: ./assets/js/repositories/Est57000_TransferenciaEstoque_Repository.js
var Est57000_TransferenciaEstoque_Repository = function (restEngine) {

    this.buscaDataProcessamento = function(row) {
        return restEngine.post('est#/dataProcessamentoUnidade', {
            "requestType": "Row",
            "row": row
        });
    };

    this.saveNovaTransferencia = function(rows) {
        return restEngine.post('est#/saveNewTransferencia', {
            "requestType": "Row",
            "row": rows
        });
    };

    this.saveNovoProdutoTransferencia = function(row) {
        return restEngine.post('est#/saveNovoProdutoTransferencia', {
            "requestType": "Row",
            "row": row
        });
    };

    this.saveNovaTransferenciaComRequisicao = function(row) {
        return restEngine.post('est#/saveNewTransferenciaRequisicao', {
            "requestType": "Row",
            "row": row
        });
    };

    this.editProduto = function(row) {
        return restEngine.post('est#/editProdutoTransferencia', {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteProduto = function(row) {
        return restEngine.post('est#/deleteProdutoTransferencia', {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteTransferencia = function(row) {
        return restEngine.post('est#/deleteTransferencia', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.getAlmoxRequisicao = function(row) {
        return restEngine.post('est#/getAlmoxRequisicao', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.verificaEstoqueProduto = function(row) {
         var filtro = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": row.CDFILIMOVI
            },
            {
                "name": "P_CDPRODUTO",
                "operator": "=",
                "value": row.CDPRODUTO
            },
            {
                "name": "P_NRLOTEESTQ",
                "operator": "=",
                "value": row.NRLOTEESTQ
            },
            {
                "name": "P_NRSUBLOTE",
                "operator": "=",
                "value": row.NRSUBLOTE
            },
            {
                "name": "P_DTPOSICAO",
                "operator": "=",
                "value": row.DTLANCMOVI
            },
            {
                "name": "P_CDALMOXARIFE",
                "operator": "=",
                "value": row.CDALMOXARIFE
            },
            {
                "name": "P_CDLOCALESTOQ",
                "operator": "=",
                "value": row.CDLOCALESTOQ
            },
            {
                "name": "P_IDQUANTID",
                "operator": "=",
                "value": row.IDQUANTID
            }
        ];
        return restEngine.post('est#/lote_infCompl_almox', {
            "requestType": "FilterData",
            "filter": filtro
        });
    };
    
    this.buscaProdutosRequisicao = function(row) {
        var filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": row.CDFILIAL
            },
            {
                "name": "NRREQUESTO",
                "operator": "=",
                "value": row.NRREQUESTO
            },
            {
                "name": "IDSTITREQU",
                "operator": "=",
                "value": 'A'
            }
        ];
        return restEngine.post('est#/itrequesto_itens_requisicao', {
            "requestType": "FilterData",
            "filter": filtro
        });
    };
    
    this.getCdtipocusto = function(cdfilial) {
        return restEngine.post('est#/crud/paramfilial', {
            "requestType": "FilterData",
            "filter": [{
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": cdfilial
                        }]
        });
    };
    
    
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est57000_TransferenciaEstoque_Repository', Est57000_TransferenciaEstoque_Repository);
});

// FILE: ./assets/js/repositories/Ger03300_FecharDia_Repository.js
var Ger03300_FecharDia_Repository = function(restEngine, localStorageEngine) {

    this.consulta = function(rota, filtro){
        if(!filtro){
            filtro = [];
        }
        return restEngine.post(rota, {
            "requestType": "FilterData",
            "filter": filtro
        });
    };

    this.getLocalVar = function(campo){
        return localStorageEngine.getLocalVar(campo);
    };

    this.getDataSource = function(name){
        return localStorageEngine.getDataSource(name);
    };

    this.setDataSource = function(name, data){
        localStorageEngine.setDataSource(name, data);
        return;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger03300_FecharDia_Repository', Ger03300_FecharDia_Repository);
});

// FILE: ./assets/js/repositories/Ger03301_RelatorioInventarioRotativo_Repository.js
var Ger03301_RelatorioInventarioRotativo_Repository = function(localStorageEngine) {
    this.paramFilial = function(filtro) {
        return localStorageEngine.download("est#/crud/paramfilial", filtro);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger03301_RelatorioInventarioRotativo_Repository', Ger03301_RelatorioInventarioRotativo_Repository);
});

// FILE: ./assets/js/repositories/Ger27000_LocalEstoqueProduto_Repository.js
var Ger27000_LocalEstoqueProduto_Repository = function(restEngine) {
    
    this.validateFilial = function(filter) {
        return restEngine.post('est#/crud/paramfilial', {
            "requestType": "FilterData",
            "filter": filter
        });
	};
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger27000_LocalEstoqueProduto_Repository', Ger27000_LocalEstoqueProduto_Repository);
});

// FILE: ./assets/js/repositories/InventarioRotativo_Repository.js
var InventarioRotativo_Repository = function (restEngine) {

    this.saveInverota = function(rows) {
        return restEngine.post('est#/crud/inverota/save', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };

    this.saveInverotalote = function(rows) {
        return restEngine.post('est#/crud/inverotalote/save', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };

    this.atualizaInverota = function(filter){
        return restEngine.post('est#/atualiza_inverota', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    /*
     *  Envia uma requisição vazia para somente verificar se cairá na
     *  exception do preDispatch de inventário rotativo no backend.
     */
    this.checkInventarioPendente = function() {
        return restEngine.post('est#/verifica_inventario_pendente', {
            "requestType": "Row",
            "row": {}
        });
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('InventarioRotativo_Repository', InventarioRotativo_Repository);
});


// FILE: ./assets/js/repositories/ParamfilialRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {

	var ParamfilialRepository = RepositoryFactory.factory('est#/crud/paramfilial', 'LOCAL', 1, 30000);

	ParamfilialRepository.getParamfilial = function(nrorg, cdfilial) {
		return this.download(Query.build()
			.where('nrorg').equals(nrorg)
			.where('cdfilial').equals(cdfilial)
		);
	};

	ContextRegister.register('ParamfilialRepository', ParamfilialRepository);

});

// FILE: ./assets/js/repositories/ParamgeralRepository.js
Configuration(function(RepositoryFactory, ContextRegister, Query) {

	var ParamgeralRepository = RepositoryFactory.factory('est#/crud/paramgeral', 'LOCAL', 1, 30000);

	ParamgeralRepository.getParamgeral = function(nrorg) {
		return this.download(Query.build()
			.where('nrorg').equals(nrorg)
		);
	};

	ContextRegister.register('ParamgeralRepository', ParamgeralRepository);

});

// FILE: ./assets/js/repositories/Unidade_Repository.js
var Unidade_Repository = function(localStorageEngine, restEngine) {
    this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return localStorageEngine.download("est#/crud/filial", filter);
    };

    this.retornaParamUnidade = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getDtprocessa = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("est#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Unidade_Repository', Unidade_Repository);
});

// FILE: ./assets/js/repositories/est53000_parametros_de_estoque_repository.js
var est53000_parametros_de_estoque_repository = function(restEngine) {
    var self = this;

    self.deleteAllRealizationDays = function(CDFILIAL) {
        return restEngine.post("est#/paramestoque_invrotativo_dias/remove", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: CDFILIAL
                }
            ]
        });
    };

    self.getDataForGeralWidget = function() {
        return restEngine.post("est#/paramestoque_geral", {
            "requestType": "FilterData",
            "filter": []
        });
    };

    self.getDataForProdutoField = function() {
        return restEngine.post("est#/paramestoque_produto_field", {
            "requestType": "FilterData",
            "filter": []
        });
    };

    self.getDataForProdutoWidget = function(CDALMOXARIFE, CDLOCALESTOQ, CDFILIAL) {
        return restEngine.post("est#/paramestoque_produto_widget", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "CDALMOXARIFE",
                    operator: "=",
                    value: CDALMOXARIFE
                },
                {
                    name: "CDLOCALESTOQ",
                    operator: "=",
                    value: CDLOCALESTOQ
                },
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: CDFILIAL
                }
            ]
        });
    };

    self.getDataForUnidadeWidget = function(CDFILIAL) {
        return restEngine.post("est#/paramestoque_unidade", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: CDFILIAL
                }
            ]
        });
    };

    self.saveProduct = function(row) {
        return restEngine.post("est#/paramestoque_produto/save", {
            "requestType": "Row",
            "row": row
        });
    };

    self.validateProduct = function(CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, CDARVPROD, IDCURVAABC) {
        return restEngine.post("est#/paramestoque_produto_validation", {
            "requestType": "FilterData",
            "filter": [
                {
                    name: "CDFILIAL",
                    operator: "=",
                    value: CDFILIAL
                },
                {
                    name: "CDALMOXARIFE",
                    operator: "=",
                    value: CDALMOXARIFE
                },
                {
                    name: "CDLOCALESTOQ",
                    operator: "=",
                    value: CDLOCALESTOQ
                },
                {
                    name: "CDPRODUTO",
                    operator: "=",
                    value: CDPRODUTO
                },
                {
                    name: "CDARVPROD",
                    operator: "=",
                    value: CDARVPROD
                },
                {
                    name: "IDCURVAABC",
                    operator: "=",
                    value: IDCURVAABC
                }
            ]
        });
    };

    self.saveGeral = function(row) {
        return restEngine.post('est#/paramestoque_geral/save', {
            "requestType": "Row",
            "row": row
        });
    };

    self.saveUnidade = function(row) {
        return restEngine.post('est#/paramestoque_unidade/save', {
            "requestType": "Row",
            "row": row
        });
    };

    self.saveParametrosCurvaAbc = function(row) {
        return restEngine.post('est#/curvaAbc_parametros/save', {
            "requestType": "Row",
            "row": row
        });
    };

    self.deleteProdutoCurvaAbc = function(row) {
        return restEngine.post('est#/produtos_curvaAbc/remove', {
            "requestType": "Row",
            "row": row
        });
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('est53000_parametros_de_estoque_repository', est53000_parametros_de_estoque_repository);
});

// FILE: ./assets/modules/ajuste/assets/js/repositories/Ajuste_Repository.js
var Ajuste_Repository = function(restEngine) {

    this.validaAjuste = function(filter){
        return restEngine.post('est_ajuste#/valida_ajuste', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.importProducts = function(filter){
        return restEngine.post('est_ajuste#/ajuste_import_products',
            {
                "requestType": "FilterData",
                "filter": filter
            }
        );
    };

    this.deleteLancto = function(row){
        return restEngine.post('est_ajuste#/lanctoestoq_ajuste/delete', {
            "requestType": "Row",
            "row": row
        });
    };

    this.deleteItem = function(row){
        return restEngine.post('est_ajuste#/itlanctoest_ajuste/delete', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.importarProdutosTxt = function(row){
        return restEngine.post('est_ajuste#/ajuste_import_products_txt', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.saveDataset = function(dataSourceName, dataToSave, otherParams = {}) {
        return restEngine.post(dataSourceName, Util.extend({
            "requestType": "DataSet",
            "dataset": dataToSave
        }, otherParams || {}), undefined, undefined, undefined, undefined, 8001000);
    }; 

    this.saveItlanctoest = function(dataSet) {
        return restEngine.post('est_ajuste#/crud/itlanctoest/save', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
    
    
    this.importProductsFilial = function(filter){
        return restEngine.post('est_ajuste#/ajuste_import_products_filial',
            {
                "requestType": "FilterData",
                "filter": filter
            }
        );
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ajuste_Repository', Ajuste_Repository);
});

// FILE: ./assets/modules/ajuste/assets/js/repositories/Est44000RecalculoEstoque_Repository.js
var Est44000RecalculoEstoque_Repository = function(restEngine,localStorageEngine) {
	this.recalcularEstoque = function(row){
	    return restEngine.post('est_ajuste#/recalculo_estoque', {
            "requestType": "Row",
            "row": row
        });	
	};
	
	this.saveProdutosIrregulares = function(data){
	    return localStorageEngine.setDataSource('est_ajuste#/produtos_irregulares_recalculo',data);	
	};
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est44000RecalculoEstoque_Repository', Est44000RecalculoEstoque_Repository);
});

// FILE: ./assets/modules/estoqueCego/assets/js/repositories/Est90000_PerfilOperadoresInventarioCego_Repository.js
var Est90000_PerfilOperadoresInventarioCego_Repository = function(restEngine) {

    this.onSaveOperation = function(row) {
        return restEngine.post('est_estoqueCego#/saveOperation', {
            "requestType": "Row",
            "row": row
        });
    };

    this.onDeleteOperation = function(row) {
        return restEngine.post('est_estoqueCego#/deleteOperation', {
            "requestType": "Row",
            "row": row
        });
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est90000_PerfilOperadoresInventarioCego_Repository', Est90000_PerfilOperadoresInventarioCego_Repository);
});


// FILE: ./assets/modules/estoqueCego/assets/js/repositories/Est91000_ProgramacaoDoInventarioCego_Repository.js
var Est91000_ProgramacaoDoInventarioCego_Repository = function(restEngine) {

    this.saveProducts = function(row) {
        return restEngine.post('est_estoqueCego#/saveProducts', {
            "requestType": "Row",
            "row": row
        });
    };

    this.checkInventoryHandler = function(row) {
        return restEngine.post('est_estoqueCego#/checkInventoryHandler', {
            "requestType": "Row",
            "row": row
        });
    };
    this.onGenerateInventory = function(row) {
        return restEngine.post('est_estoqueCego#/generateInventory', {
            "requestType": "Row",
            "row": row
        });
    };

    this.onDeleteInventory = function(row) {
        return restEngine.post('est_estoqueCego#/deleteInventory', {
            "requestType": "Row",
            "row": row
        });
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est91000_ProgramacaoDoInventarioCego_Repository', Est91000_ProgramacaoDoInventarioCego_Repository);
});


// FILE: ./assets/modules/estoqueCego/assets/js/repositories/Est92000_ConsultaInventarioCego_Repository.js
var Est92000_ConsultaInventarioCego_Repository = function(restEngine) {

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est92000_ConsultaInventarioCego_Repository', Est92000_ConsultaInventarioCego_Repository);
});


// FILE: ./assets/modules/estoqueCego/assets/js/repositories/Est93000_DigitacaoInventarioCego_Repository.js
var Est93000_DigitacaoInventarioCego_Repository = function(restEngine) {

    this.save = function(row) {
        return restEngine.post('est_estoqueCego#/save', {
            "requestType": "Row",
            "row": row
        });
    };
    this.onConcludeDigitation = function(row) {
        return restEngine.post('est_estoqueCego#/concludeDigitation', {
            "requestType": "Row",
            "row": row
        });
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est93000_DigitacaoInventarioCego_Repository', Est93000_DigitacaoInventarioCego_Repository);
});


// FILE: ./assets/modules/estoqueCego/assets/js/repositories/Est94000_AprovacaoInventarioCego_Repository.js
var Est94000_AprovacaoInventarioCego_Repository = function(restEngine) {

    this.save = function(row) {
        return restEngine.post('est_estoqueCego#/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.onChangeSituation = function(dataSet,otherParams = {}) {
        return restEngine.post('est_estoqueCego#/inventorySituationHandler', Util.extend({
            "requestType": "DataSet",
            "dataset": dataSet
        }, otherParams || {}), undefined, undefined, undefined, undefined, undefined);
    }; 
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est94000_AprovacaoInventarioCego_Repository', Est94000_AprovacaoInventarioCego_Repository);
});


// FILE: ./assets/modules/estoqueCego/assets/js/repositories/Est95100_AnaliseInventarioCego_Repository.js
var Est95100_AnaliseInventarioCego_Repository = function(restEngine) {

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est95100_AnaliseInventarioCego_Repository', Est95100_AnaliseInventarioCego_Repository);
});


// FILE: ./assets/modules/estoqueCego/assets/js/repositories/Est96000_DivergenciasInventarioCego_Repository.js
var Est96000_DivergenciasInventarioCego_Repository = function(restEngine) {

    this.actionsManagementHandler = function(InventoryRow, row, hasItems, action) {
        return restEngine.post('est_estoqueCego#/actionsManagement', Util.extend({
            "requestType": "DataSet",
            "dataset": row
        }, { hasItems, action, InventoryRow } || {}), undefined, undefined, undefined, undefined, undefined);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est96000_DivergenciasInventarioCego_Repository', Est96000_DivergenciasInventarioCego_Repository);
});


// FILE: ./assets/modules/lancamentos/assets/js/repositories/Est11000_EntradaEstoque_Repository.js
function Est11000_EntradaEstoque_Repository(restEngine) {
    this.getRequestoItens = function(filter){
        return restEngine.post('est_lancamentos#/itrequesto_retirada', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.saveRetirada = function(row) {
        return restEngine.post('est_lancamentos#/retirada/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getParamFilial = function(filter) {
        return restEngine.post('est#/paramfilial_paramvalidacoes', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getPosicaoDeEstoquePorProduto = function(filter) {
        return restEngine.post('est_lancamentos#/retirada_posestdia', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getCdtipocusto = function(cdfilial) {
        return restEngine.post('est#/crud/paramfilial', {
            "requestType": "FilterData",
            "filter": [{
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": cdfilial
                        }]
        });
    };

    this.findItensOfRequisicao = function(filter) {
        return restEngine.post('est_lancamentos#/itrequesto_itens_requisicao', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est11000_EntradaEstoque_Repository', Est11000_EntradaEstoque_Repository);
});

// FILE: ./assets/modules/lancamentos/assets/js/repositories/Est58000_RetiradaEstoque_Repository.js
function Est58000_RetiradaEstoque_Repository(restEngine) {
    
    this.getRequestoItens = function(filter){
        return restEngine.post('est_lancamentos#/itrequesto_retirada', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.saveRetirada = function(row) {
        return restEngine.post('est_lancamentos#/retirada/save', {
            "requestType": "Row",
            "row": row
        });
    };

    this.getParamFilial = function(filter) {
        return restEngine.post('est#/paramfilial_paramvalidacoes', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getPosicaoDeEstoquePorProduto = function(filter) {
        return restEngine.post('est_lancamentos#/retirada_posestdia', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.getCdtipocusto = function(cdfilial) {
        return restEngine.post('est#/crud/paramfilial', {
            "requestType": "FilterData",
            "filter": [{
                        "name": "CDFILIAL",
                        "operator": "=",
                        "value": cdfilial
                        }]
        });
    };

    this.findItensOfRequisicao = function(filter) {
        return restEngine.post('est_lancamentos#/crud/itrequesto_itens_requisicao', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.importarProdutosTxt = function(row){
        return restEngine.post('est_lancamentos#/retirada_import_products_txt', {
            "requestType": "Row",
            "row": row
        });
    };
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est58000_RetiradaEstoque_Repository', Est58000_RetiradaEstoque_Repository);
});

// FILE: ./assets/modules/relatorios/assets/js/repositories/Est31300_MovimentacaodeEstoque_Repository.js
var Est31300_MovimentacaodeEstoque_Repository = function(restEngine) {
    this.getReport = function(row) {
        return restEngine.post('est_relatorios#/relatorioMovimentacao', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est31300_MovimentacaodeEstoque_Repository', Est31300_MovimentacaodeEstoque_Repository);
});
 

// FILE: ./assets/modules/relatorios/assets/js/repositories/Est32100_EvolucaoDeCusto_Repository.js
var Est32100_EvolucaoDeCusto_Repository = function(restEngine) {
    this.getReport = function(row) {
        return restEngine.post('est_relatorios#/relatorioEvolcusto', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est32100_EvolucaoDeCusto_Repository', Est32100_EvolucaoDeCusto_Repository);
});


// FILE: ./assets/modules/relatorios/assets/js/repositories/Est32200_ProdutosSemMovimentacao_Repository.js
function Est32200_ProdutosSemMovimentacao_Repository(restEngine) {

    this.getParamFilial = function(cdfilial){
        return restEngine.post('est#/paramfilial_paramvalidacoes', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "P_CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });
    };

    this.gerarRelatorio = function(row){
        return restEngine.post('est_relatorios#/gerarRelatorioEst32200', {
            "requestType": "Row",
            "row": row
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est32200_ProdutosSemMovimentacao_Repository', Est32200_ProdutosSemMovimentacao_Repository);
});

// FILE: ./assets/modules/relatorios/assets/js/repositories/Est45000_EmissaoInventario_Repository.js
var est45000_EmissaoInventario_Repository = function (restEngine) {

    this.getParamFilial = function(cdfilial){
        return restEngine.post('est#/crud/paramfilial', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Est45000_EmissaoInventario_Repository', est45000_EmissaoInventario_Repository);
});


// FILE: ./assets/modules/relatorios/assets/js/repositories/Est46000_EmissaoAjusteInventario_Repository.js
var Est46000_EmissaoAjusteInventario_Repository = function(restEngine) {
    
    this.getReport = function(row) {
        return restEngine.post('est_relatorios#/relatorioEmissaoAjuste', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est46000_EmissaoAjusteInventario_Repository', Est46000_EmissaoAjusteInventario_Repository);
});


// FILE: ./assets/modules/relatorios/assets/js/repositories/Est70000_ResumoDeOperacoes_Repository.js
var Est70000_ResumoDeOperacoes_Repository = function(restEngine) {
    
    this.getReport = function(row) {
        return restEngine.post('est_relatorios#/relatorioEmissaoAjuste', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est70000_ResumoDeOperacoes_Repository', Est70000_ResumoDeOperacoes_Repository);
});


// FILE: ./assets/js/services/Est33000_RequisicaoEstoque_Service.js
var Est33000_RequisicaoEstoque_Service = function (General, Est33000_RequisicaoEstoque_Repository) {
    
    var dtprocessa;
    var idctrlestq;
    var idutilalmox;

    this.salvarRequisicaoProdutos = function(rows){
        return Est33000_RequisicaoEstoque_Repository.saveRequisicaoProdutos(rows);
    };

    this.salvarnovoProduto = function(row){
        return Est33000_RequisicaoEstoque_Repository.saveNovoProduto(row);
    };

	this.deletarProduto = function(row){
        return Est33000_RequisicaoEstoque_Repository.deleteProduto(row);
    };

    this.deletarRequisicao = function(row){
        return Est33000_RequisicaoEstoque_Repository.deleteRequisicao(row);
    };
    
    this.getParamFilial = function(fililal){
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": fililal
                }
		    ];
        return General.callRestEngine('est#/paramfilial_paramvalidacoes', 'FilterData', filter);
    };
    
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est33000_RequisicaoEstoque_Service', Est33000_RequisicaoEstoque_Service);
});

// FILE: ./assets/js/services/Est51000_ParametrosImportacaoEstoque_Service.js
var Est51000_ParametrosImportacaoEstoque_Service = function(General) {

    this.checkDataBeforeSave = function(widget) {
        return new Promise((resolve, reject) => {
            General.callRestEngine('est#/crud/paramimpest', 'FilterData', []).then(paramsData => {
                if (widget.currentRow.IDFILIPADIMPEST == 'S') {
                    let hasDefaultParams = paramsData.dataset.paramimpest.filter(row => {
                        return row.IDFILIPADIMPEST == 'S';
                    });
                return hasDefaultParams.length != 0 ? reject(hasDefaultParams) : resolve(true);
                }else{
                    return resolve(true);
                }
            });
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est51000_ParametrosImportacaoEstoque_Service', Est51000_ParametrosImportacaoEstoque_Service);
});


// FILE: ./assets/js/services/Est57000_TransferenciaEstoque_Service.js
var Est57000_TransferenciaEstoque_Service = function (General,Est57000_TransferenciaEstoque_Repository,ZHPromise,ScreenService) {

    this.buscaDataProcessamento = function(row){
        return Est57000_TransferenciaEstoque_Repository.buscaDataProcessamento(row);
    };

    this.salvarNovaTransferencia = function(rows){
        return Est57000_TransferenciaEstoque_Repository.saveNovaTransferencia(rows);
    };

    this.salvarNovoProdutoTransferencia = function(row){
        return Est57000_TransferenciaEstoque_Repository.saveNovoProdutoTransferencia(row);
    };

    this.salvarNovaTransferenciaComRequisicao = function(row){
        return Est57000_TransferenciaEstoque_Repository.saveNovaTransferenciaComRequisicao(row);
    };

    this.editarProduto = function(row){
        return Est57000_TransferenciaEstoque_Repository.editProduto(row);
    };

    this.deletarProduto = function(row){
        return Est57000_TransferenciaEstoque_Repository.deleteProduto(row);
    };

    this.deletarTransferencia = function(row){
        return Est57000_TransferenciaEstoque_Repository.deleteTransferencia(row);
    };
    
    this.getAlmoxRequisicao = function(row) {
        return Est57000_TransferenciaEstoque_Repository.getAlmoxRequisicao(row);  
    };

    this.verificaEstoqueProduto = function(row) {
        return Est57000_TransferenciaEstoque_Repository.verificaEstoqueProduto(row);  
    };
    
    this.buscaProdutosRequisicao = function(row) {
        return Est57000_TransferenciaEstoque_Repository.buscaProdutosRequisicao(row);  
    };
    
    this.setVrTotLancto = function(rowItem) {
    	rowItem.VRTOTLANCTO = rowItem.QTTOTLANCTO * rowItem.VRUNILANCTO;
    };
    
    this.checaItensDaRetirada = function(data, cdarvprod, cdalmoxarife, cdlocalestoq, nrloteestq, nrsublote) {
		var contem = false;
		data.splice(data.length -1, 1);
	
    	data.forEach(function(item){
			 if(item.CDARVPROD === cdarvprod) {
			 	contem = true;
			 }
		});
		
		return contem;
	};

    this.getParamFilial = function(){
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est_relatorios#/crud/filial_paramfilial_tipocusto', 'FilterData', filter);
    };

    this.importarProdutosTxt = function(row){
    	return Est57000_TransferenciaEstoque_Repository.importarProdutosTxt(row);
    };
    
    this.geraNrsequitem= function(itens) {
		var nrsequitemArray=[0];
		itens.forEach(function(item) {
		    if(item.NRSEQUITEM)
			    nrsequitemArray.push(item.NRSEQUITEM);
		});
		return ('000'+(Math.max.apply(this,nrsequitemArray)+1)).slice(-4);
	};
	
	this.getCdtipocusto = function(cdfilial) {
		return Est57000_TransferenciaEstoque_Repository.getCdtipocusto(cdfilial).then(function(callback){
			return callback.dataset.paramfilial[0].CDTIPCUSTEST;
		});
	};
	
	this.setBlankLocalEstFields = function(widget){
		if(!widget.currentRow.hasOwnProperty('CDALMOXARIFE')){
			widget.currentRow.CDALMOXARIFE = ' ';
		}
		if(!widget.currentRow.hasOwnProperty('CDLOCALESTOQ')){
			widget.currentRow.CDLOCALESTOQ = ' ';
		}
		
		if(!widget.currentRow.hasOwnProperty('NRLOTEESTQ')){
			widget.currentRow.NRLOTEESTQ = ' ';
		}
		
		if(!widget.currentRow.hasOwnProperty('NRSUBLOTE')){
			widget.currentRow.NRSUBLOTE = ' ';
		} 

	};
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est57000_TransferenciaEstoque_Service', Est57000_TransferenciaEstoque_Service);
});

// FILE: ./assets/js/services/Ger27000_LocalEstoqueProduto_Service.js
var Ger27000_LocalEstoqueProduto_Service = function(Ger27000_LocalEstoqueProduto_Repository) {
    

    this.validateFilialService = function(filter) {
        return Ger27000_LocalEstoqueProduto_Repository.validateFilial(filter);
    };
    
    this.limpaFieldsCADASTROAUTOMATICO = function(widget) {
        if(widget.name === 'CADASTROAUTOMATICO'){
            //Almoxarifado Matéria-Prima / Insumos para Produção
    		widget.currentRow.DSLOCALINSPR = '';
    		widget.currentRow.DSLOCALINSPR = '';
    	
    	    //Almoxarifado Produto Semiacabado
    		widget.currentRow.DSLOCALPRSEMACA = '';
    		widget.currentRow.DSLOCALPRSEMACA = '';
    	
    	    //DSALMOXINSPRPIL -- Almoxarifado Matéria-prima Piloto / Insumos para Produção
    		widget.currentRow.DSLOCALINSPRPIL = '';
    		widget.currentRow.DSLOCALINSPRPIL = '';
    	
    	    //DSALPROD -- Almoxarifado Matéria-prima Piloto / Insumos para Produção
    		widget.currentRow.DSLOCPROD = '';
    		widget.currentRow.DSLOCPROD = '';
    	
    	    //DSALPRODPIL -- Almoxarifado Matéria-prima Piloto / Insumos para Produção
    		widget.currentRow.DSLOCPRODPRIL = '';
    		widget.currentRow.DSLOCPRODPRIL = '';
    	
    	    //Almoxarifado - Produtos Acabados
    		widget.currentRow.DSLOCALPRACA = '';
    		widget.currentRow.DSLOCALPRACA = '';
    	
    	    //Almoxarifado Expedição / Reserva de Pedidos
    		widget.currentRow.DSLOCALPRCLI = '';
    		widget.currentRow.DSLOCALPRCLI = '';
    	
    	    //Almoxarifado Quarentena - Matéria Prima / Insumos
    		widget.currentRow.DSLOCALESTOQ = '';
    		widget.currentRow.DSLOCALESTOQ = '';
    	
    	    //Almoxarifado Quarentena - Matéria-Prima Produto Piloto
    		widget.currentRow.DSLOCALQUAMPPIL = '';
    		widget.currentRow.DSLOCALQUAMPPIL = '';
    	
    	    //Almoxarifado Quarentena - Produto Acabado
    		widget.currentRow.DSLOCALQUAR = '';
    		widget.currentRow.DSLOCALQUAR = '';
    	
    	    //Almoxarifado Produtos Reprovados
    		widget.currentRow.DSLOCALREPROV = '';
    		widget.currentRow.DSLOCALREPROV = '';
        }
    };
    
};


	
Configuration(function(ContextRegister) {
    ContextRegister.register('Ger27000_LocalEstoqueProduto_Service', Ger27000_LocalEstoqueProduto_Service);
});

// FILE: ./assets/js/services/InventarioRotativo_Service.js
var InventarioRotativo_Service = function(localStorageEngine, restEngine, InventarioRotativo_Repository, ZHPromise) {

    this.verificaInvePendente = function(cdfilial) {
        var filter = [{
            "name": "P_CDFILIAL",
            "operator": "=",
            "value": cdfilial
        }];
        return localStorageEngine.download('/verificaInvePend', filter);
    };

    this.save = function(rowsToSave, utLote) {
        if (utLote) {
            return InventarioRotativo_Repository.saveInverotalote(rowsToSave);
        }
        else {
            return InventarioRotativo_Repository.saveInverota(rowsToSave);
        }
    };

    this.getOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];

        return localStorageEngine.download("est#/crud/operador", filter);
    };

    this.atualizaInventarioRotativo = function(filter) {
        return InventarioRotativo_Repository.atualizaInverota(filter);
    };

    /*
     *  Função para verificar se há inventário pendente.
     *  Como há um preDispatch no backend, o retorno cairá num callback
     *  de erro se houver inventário rotativo pendente, devido a uma
     *  exception que é lançada. Se não houver inventário rotativo, o retorno
     *  será no callback de sucesso.
     */
    this.checkInventarioPendente = function() {
        var deferred = ZHPromise.defer();
        InventarioRotativo_Repository.checkInventarioPendente().then(function(response) {
            // Verifica se realmente entrou na função de verificação e
            // retornou o dataset "inventario_valido".
            if (response.dataset && response.dataset.inventario_valido) {
                deferred.resolve({ "result": true });
            }
            else {
                deferred.reject(response);
            }
        }, function(error) {
            // Verifica se o erro tratado é a exceção de inventário rotativo ou inventário cego pendente
            if (error && Util.isObject(error.data)) {
                switch (error.data.error.replace("Exception: ", "")) {
                    case "Inventário Rotativo Pendente":
                        deferred.resolve({ "result": false, "type": 'InventRota' });
                        break;
                    case "Inventário Cego Pendente":
                        deferred.resolve({ "result": false, "type": 'InventCego' });
                        break;
                    case "Contagem física Pendente":
                        deferred.resolve({ "result": false, "type": 'ContPend' });
                        break;
                    case "Contagem Física Pendente Digitada":
                        deferred.resolve({ "result": false, "type": 'ContPendDig' });
                        break;
                    case "Realizada a Contagem do Estoque Físico":
                        deferred.resolve({ "result": false, "type": 'RealContEstFis' });
                        break;
                    default:
                        deferred.reject(error);
                }
            }
        });
        return deferred;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('InventarioRotativo_Service', InventarioRotativo_Service);
});


// FILE: ./assets/js/services/ParamfilialService.js
var ParamfilialService = function(ParamfilialRepository) {

	this.getParamfilial = function(nrorg, cdfilial) {
		return ParamfilialRepository.getParamfilial(nrorg, cdfilial);
	};

	this.getDtprocessa = function(nrorg, cdfilial) {
		return this.getParamfilial(nrorg, cdfilial).then(function(paramfilial) {
			return paramfilial[0].DTPROCESSA;
		});
	};

};

Configuration(function(ContextRegister) {
    ContextRegister.register('ParamfilialService', ParamfilialService);
});

// FILE: ./assets/js/services/ParamgeralService.js
var ParamgeralService = function(ParamgeralRepository) {

	this.getParamgeral = function(nrorg) {
		return ParamgeralRepository.getParamgeral(nrorg);
	};

};

Configuration(function(ContextRegister) {
    ContextRegister.register('ParamgeralService', ParamgeralService);
});

// FILE: ./assets/js/services/ProcProdWorkflowService.js
var ProcProdWorkflowService = function(restEngine) {

	this.setProcWorkflowProfile = function(){

		var filter = [
			{
				"name": "NRPROCESSOAPROVA",
				"operator": "=",
				"value": "2"
			}
		];

		return restEngine.post("est#/crud/processoaprova", {
			"requestType":"FilterData",
			"filter": filter
		}).then(function(callback){
			setLocalVar("INFLIGHTOR_WORKFLOWPROD", callback.dataset.processoaprova[0].IDATIVO);
		});

	};

	this.procProdWorkflow = function(row){

		return restEngine.post("est#/ProcProdWorkflow/procProdWf", {
			"requestType":"Row",
			"row": row
		});

	};

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('ProcProdWorkflowService', ProcProdWorkflowService);
});

// FILE: ./assets/js/services/ProdutoService.js
var ProdutoService = function(restEngine) {

	this.saveProd = function(row){

		return restEngine.post("est#/produtocrud/save", {
			"requestType":"Row",
			"row": row
		});

	};

	this.saveProdArvProd = function(row){

		return restEngine.post("est#/arvoreprodtree/save", {
			"requestType":"Row",
			"row": row
		});

	};
	

};

Configuration(function(ContextRegister, eventEngine) {
    ContextRegister.register('ProdutoService', ProdutoService);
});

// FILE: ./assets/js/services/Est41000_ajuste_inventario.js
function Est41000_ajuste_inventario(DecodeValueFieldService) {
    function addThousandSeparators(value) {
        value += '';
        var x = value.split(',');
        var x1 = x[0];
        var x2 = x.length > 1 ? ',' + x[1] : '';
        var rgx = /(\d+)(\d{3})/;
        while (rgx.test(x1)) {
            x1 = x1.replace(rgx, '$1.$2');
        }

        return x1 + x2;
    }

    function decodeTextViewFloat(field, fieldRow) {
    	if(typeof fieldRow === 'string'){
            fieldRow = fieldRow.replace(/\./g, '');
            fieldRow = fieldRow.replace(',', '.');
    	}
        var viewRow = parseFloat(fieldRow);
    	viewRow = isNaN(viewRow) ? "" : viewRow.toFixed(3).replace('.', ',');
        return addThousandSeparators(viewRow);
    }

    DecodeValueFieldService.register('text-edit-view-float.html', decodeTextViewFloat);
}

Configuration(function(ContextRegister){
    ContextRegister.register('Est41000_ajuste_inventario', Est41000_ajuste_inventario);
});

// FILE: ./assets/js/services/est53000_parametros_de_estoque_service.js
var est53000_parametros_de_estoque_service = function(est53000_parametros_de_estoque_repository, ZHPromise) {
    var self = this;

    self.deleteAllRealizationDays = function(row) {
        var CDFILIAL = row.CDFILIAL;

        if(CDFILIAL)
            return est53000_parametros_de_estoque_repository.deleteAllRealizationDays(CDFILIAL);
        else {
            var promise = ZHPromise.defer();
            promise.reject('Erro Catastrófico! Erro na requisição da exclusão dos dias de realização do Inventário Rotativo!');

            return promise;
        }
    };

    self.getDataForGeralWidget = function() {
        return est53000_parametros_de_estoque_repository.getDataForGeralWidget().then(function(response) {
            var data = response.dataset.paramestoque_geral;

            return data;
        });
    };

    self.getDataForProdutoField = function() {
        return est53000_parametros_de_estoque_repository.getDataForProdutoField().then(function(response) {
            var data = response.dataset.paramestoque_produto_field;

            return data;
        });
    };

    self.validateProduct = function(row) {
        var CDFILIAL     = row.CDFILIAL;
        var CDALMOXARIFE = (row.CDALMOXARIFE)? row.CDALMOXARIFE : ' ';
        var CDLOCALESTOQ = (row.CDLOCALESTOQ)? row.CDLOCALESTOQ : ' ';
        var CDPRODUTO    = row.CDPRODUTO;
        var CDARVPROD    = row.CDARVPROD;
        var IDCURVAABC   = row.IDCURVAABC;

        return est53000_parametros_de_estoque_repository.validateProduct(CDFILIAL, CDALMOXARIFE, CDLOCALESTOQ, CDPRODUTO, CDARVPROD, IDCURVAABC).then(function(response) {
            var isValid = response.dataset.paramestoque_produto_validation[0];

            return isValid;
        });
    };

    self.saveProduct = function(row) {
        row.CDALMOXARIFE = (row.CDALMOXARIFE)? row.CDALMOXARIFE : ' ';
        row.CDLOCALESTOQ = (row.CDLOCALESTOQ)? row.CDLOCALESTOQ : ' ';

        return est53000_parametros_de_estoque_repository.saveProduct(row);
    };

    self.getDataForProdutoWidget = function(row) {
        var CDALMOXARIFE = (row.CDALMOXARIFE)? row.CDALMOXARIFE : ' ';
        var CDLOCALESTOQ = (row.CDLOCALESTOQ)? row.CDLOCALESTOQ : ' ';
        var CDFILIAL     = row.CDFILIAL;

        return est53000_parametros_de_estoque_repository.getDataForProdutoWidget(CDALMOXARIFE, CDLOCALESTOQ, CDFILIAL).then(function(response) {
            var data = response.dataset.paramestoque_produto_widget;

            return data;
        });
    };

    self.getDataForUnidadeWidget = function(row) {
        var promise = ZHPromise.defer();
        var CDFILIAL = row.CDFILIAL;

        if(CDFILIAL)
            est53000_parametros_de_estoque_repository.getDataForUnidadeWidget(CDFILIAL).then(function(response) {
                var data = response.dataset.paramestoque_unidade;

                data.CDFILIAL     = CDFILIAL;
                data.NMRAZSOCFILI = row.NMRAZSOCFILI;
                data.NMFILIAL = row.NMFILIAL;

                promise.resolve(data);
            }, function(error) {promise.reject(error);});
        else
            promise.resolve({});

        return promise;
    };

    self.saveGeral = function(row){
        return est53000_parametros_de_estoque_repository.saveGeral(row);
    };

    self.saveUnidade = function(row){
        return est53000_parametros_de_estoque_repository.saveUnidade(row);
    };

    self.saveParametrosCurvaAbc = function(row){
        return est53000_parametros_de_estoque_repository.saveParametrosCurvaAbc(row);
    };

    self.deleteProdutoCurvaAbc = function(row){
        return est53000_parametros_de_estoque_repository.deleteProdutoCurvaAbc(row);
    };


    self.concatenaCampoParamfilial = function(row){
        var campo = row.IDBLOQESTNEG;
        var campoCompleto = row.DSPARFILFIS;
        if(campo){
            if (!campoCompleto) {
                campoCompleto = "   " + campo;
            }else {
                switch(campoCompleto.length) {
                    case 1:
                        campoCompleto= campoCompleto+"  "+campo;
                        break;
                    case 2:
                        campoCompleto = campoCompleto+" "+campo;
                        break;
                    case 3:
                        campoCompleto = campoCompleto+campo;
                        break;
                    default:
                        var parteinicial = campoCompleto.substr(0, 3);
                        var partefinal = campoCompleto.substr(4);
                        campoCompleto = parteinicial + campo + partefinal;
                        break;
                }
            }
        }
        return campoCompleto;
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('est53000_parametros_de_estoque_service', est53000_parametros_de_estoque_service);
});

// FILE: ./assets/modules/ajuste/assets/js/services/Ajuste_Service.js
var Ajuste_Service = function(Lanctoestoq_Repository,Itlanctoest_Repository,restEngine,localStorageEngine,Ajuste_Repository,ZHPromise, ScreenService, templateManager, ValidacoesEstoque_Service, General) {
	this.idOrdAjuInv=null;
	this.idTipoCustoBrut=null;
	this.idTipoCustoLiq=null;
	
    this.retornaParamUnidade = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    }; 
    
	this.setIdOrdAjuInv = function(args){
		/* ADICIONADO PARA TRATAR CONSIDERAR FILIAL DO FILTRO E NÃO A LOGADA */
		var cdfilial = args.owner.appliedFilter.FILTER_ROW.CDFILIAL;
		this.retornaParamUnidade(cdfilial).then(function(paramfilial){
			this.idOrdAjuInv=paramfilial.dataset.paramfilial[0].IDORDAJUINVE;
		}.bind(this));
	};

	this.getOperadorLogado = function(){
		var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];

        return localStorageEngine.download("est#/crud/operador", filter);
	};

	this.setParamImpl = function(args){
		var cdfilial = args.owner.appliedFilter.FILTER_ROW.CDFILIAL;
		this.retornaParamUnidade(cdfilial).then(function(paramfilial){
			paramfilial = paramfilial.dataset.paramfilial[0];
			this.idOrdAjuInv=paramfilial.IDORDIMPLANT; 
			var idtcusnfroma=paramfilial.IDTCUSNFROMA||'3';
			switch(idtcusnfroma){
				case '3':
				case '4':
					this.idTipoCustoLiq = '3';
					this.idTipoCustoBrut = '4';
				break;
				case '8':
				case '9':
					this.idTipoCustoLiq = '8';
					this.idTipoCustoBrut = '9';
				break;
				case '1':
				case '2':
					this.idTipoCustoLiq = '1';
					this.idTipoCustoBrut = '2';
				break;
				default:
					this.idTipoCustoLiq = paramfilial.IDTCUSNFROMA;
					this.idTipoCustoBrut = this.idTipoCustoLiq;
		  } 
		}.bind(this));
	};

	this.getOrdemAjuste = function(){
		return this.idOrdAjuInv === 'P' ? 'primeiro do dia' : 'último do dia';
	};
	
	this.getIdordemclc = function(){
		return this.idOrdAjuInv === 'P' ? 0 : 'U';
	}; 

	this.editAjuste = function(row){
		return Lanctoestoq_Repository.saveLanctoestoq(row);
	};

	this.saveImportRows = function(itemRows, ajusteRow){
		itemRows.forEach(function(itemRow) {
			if(!itemRow.QTTOTLANCTO)
				itemRow.QTTOTLANCTO = 0;
			if(itemRow.QTTOTLANCTO.toString().search(',') !== -1){
				itemRow.QTTOTLANCTO = $.currency.unmaskString(itemRow.QTTOTLANCTO,{decimal:',',thousands:'.'});
			}
			if(!itemRow.VRUNILANCTO)
				itemRow.VRUNILANCTO = 0;
			if(itemRow.VRUNILANCTO.toString().search(',') !== -1){
				itemRow.VRUNILANCTO = $.currency.unmaskString(itemRow.VRUNILANCTO,{decimal:',',thousands:'.'});
			}
			if(!itemRow.VRUNITBRUT)
				itemRow.VRUNITBRUT = 0;
			if(itemRow.VRUNITBRUT.toString().search(',') !== -1){
				itemRow.VRUNITBRUT = $.currency.unmaskString(itemRow.VRUNITBRUT,{decimal:',',thousands:'.'});
			}
			if(!itemRow.VRTOTLANCTO)
				itemRow.VRTOTLANCTO = 0;

			itemRow.VRTOTLANCTO = itemRow.VRUNILANCTO * itemRow.QTTOTLANCTO;
			itemRow.VRLANCTOEST = itemRow.VRTOTLANCTO;
			itemRow.VRLANCTOBRUT = itemRow.VRUNITBRUT * itemRow.QTTOTLANCTO;

			itemRow.QTLANCTOEST = itemRow.QTTOTLANCTO*(itemRow.VRFATOCONV||1);

			itemRow.IDEVOLESTOQ = 'N';
			itemRow.CDFILIAL=ajusteRow.CDFILIAL;
			itemRow.CDFILIMOVI=ajusteRow.CDFILIMOVI;
			itemRow.DTLANCMOVI = ajusteRow.DTLANCESTQ = moment(ajusteRow.DTLANCESTQ,'DD/MM/YYYY hh:mm:ss').format('DD/MM/YYYY');
			itemRow.CDOPERADOR=ajusteRow.CDOPERADOR;
			itemRow.IDORDEMCLC = this.idOrdAjuInv == 'P' ? 0 : 'U';

			itemRow.NRLANCESTQ=ajusteRow.NRLANCESTQ;

            delete itemRow.__rightActions;
            delete itemRow.__leftActions;
            
		}.bind(this));
		
		let newRows = Util.clone(itemRows, ["__rightActions", "__leftActions"]);
		
		if(itemRows.length <= 600){
			return Ajuste_Repository.saveItlanctoest(newRows);
		}
		else{
			ScreenService.alertNotification('Devido ao grande volume de dados este processo pode levar alguns minutos.', 20000);
			return saveDatasetPagination('est_ajuste#/crud/itlanctoest/save', newRows);
		}
	};


	this.save = function(itemRow,ajusteRow){
		if(!itemRow.QTTOTLANCTO)
				itemRow.QTTOTLANCTO = 0; 
		if(itemRow.QTTOTLANCTO.toString().search(',') !== -1){
			itemRow.QTTOTLANCTO = $.currency.unmaskString(itemRow.QTTOTLANCTO,{decimal:',',thousands:'.'});
		}
		
		if(!itemRow.VRUNILANCTO)
				itemRow.VRUNILANCTO = 0;
		if(itemRow.VRUNILANCTO.toString().search(',') !== -1){
			itemRow.VRUNILANCTO = $.currency.unmaskString(itemRow.VRUNILANCTO,{decimal:',',thousands:'.'});
		}
		
		if(!itemRow.VRUNITBRUT)
				itemRow.VRUNITBRUT = 0;
		if(itemRow.VRUNITBRUT.toString().search(',') !== -1){
			itemRow.VRUNITBRUT = $.currency.unmaskString(itemRow.VRUNITBRUT,{decimal:',',thousands:'.'});
		}
		
		if(!itemRow.VRTOTLANCTO)
				itemRow.VRTOTLANCTO = 0;

		itemRow.VRTOTLANCTO = itemRow.VRUNILANCTO * itemRow.QTTOTLANCTO;
		itemRow.VRLANCTOEST = itemRow.VRTOTLANCTO;
		itemRow.VRLANCTOBRUT = itemRow.VRUNITBRUT * itemRow.QTTOTLANCTO;

		itemRow.QTLANCTOEST = itemRow.QTTOTLANCTO*(itemRow.VRFATOCONV||1);

		itemRow.IDEVOLESTOQ = 'N';
		itemRow.CDFILIAL=ajusteRow.CDFILIAL;
		itemRow.CDFILIMOVI=ajusteRow.CDFILIMOVI;
		itemRow.DTLANCMOVI = ajusteRow.DTLANCESTQ = moment(ajusteRow.DTLANCESTQ,'DD/MM/YYYY hh:mm:ss').format('DD/MM/YYYY');
		itemRow.CDOPERADOR=ajusteRow.CDOPERADOR;
		itemRow.IDORDEMCLC = this.idOrdAjuInv == 'P' ? 0 : 'U';

		if(ajusteRow.__is_new){
			return Lanctoestoq_Repository.saveLanctoestoq(ajusteRow,itemRow);
		}else{
			itemRow.NRLANCESTQ=ajusteRow.NRLANCESTQ;
			return Itlanctoest_Repository.saveItlanctoestValidation(itemRow);
		}
	};
	
	function saveDatasetPagination(route, dataToSave, datasetName = 'columnsToReturn', page = 0, registers = {}){
		const rowPerPage = 600;
		var maxPages = Math.ceil((dataToSave.length / rowPerPage));
		
		var paginatedData = dataToSave.filter((row, index) =>{
            	if(index >= rowPerPage*page &&  index < rowPerPage*(page+1))
            		return true;
            	else
            		return false;
            });
		return Ajuste_Repository.saveDataset(route, paginatedData).then(callback => {
			var registersResult = callback.dataset[datasetName];
			
			if(!registers.hasOwnProperty('dataset'))
				registers.dataset = callback.dataset;
			else
            	registers.dataset[datasetName] = registers.dataset[datasetName].concat(registersResult);
            	
            if (page >= maxPages) {
                return registers;
            } else {
                page++;
                return saveDatasetPagination(route, dataToSave, datasetName, page, registers);
            }
		});
		
	}

	this.filialControlEstq = function(){
		var defer = ZHPromise.defer();
		this.retornaParamUnidade().then(
			function(success){
				if(success.dataset.paramfilial[0].IDCTRLESTQ === 'S'){
					defer.resolve();
				}else{
					defer.reject('Unidade não controla estoque');
				}
			},
			function(error){
				defer.reject('Unidade não controla estoque');
			}
		);
		return defer;
	};
	

	this.validaItem = function(currentRow,data){
		var validacao = true;

		let arrayFieldsToSetDefaultValue = ['CDALMOXARIFE', 'CDLOCALESTOQ', 'NRLOTEESTQ', 'NRSUBLOTE'];
		data.forEach(row => {
			arrayFieldsToSetDefaultValue.forEach( field => {
				if(!row[field])
					row[field] = ' ';
				else
					row[field] =  row[field] == '' ? ' ' : row[field];
			});
		});
		
		data.forEach(function(row) {
			if(currentRow.CDPRODUTO === row.CDPRODUTO && currentRow !== row && currentRow.CDALMOXARIFE === row.CDALMOXARIFE && currentRow.CDLOCALESTOQ === row.CDLOCALESTOQ && currentRow.NRLOTEESTQ === row.NRLOTEESTQ)
				validacao = false;
		});

		return validacao;
	};

	this.geraNrsequitem = function(itens){
		var nrsequitemArray=[0];
		itens.forEach(function(item){
			if(item.NRSEQUITEM)
				nrsequitemArray.push(item.NRSEQUITEM);
		});
		return ('000'+(Math.max.apply(this,nrsequitemArray)+1)).slice(-4);
	};

	this.deleteLancamento = function(row){
		return Ajuste_Repository.deleteLancto(row);
	};

	this.importProducts = function(CDGRUPPROD, CDORDENA, CDFILIAL, DTLANCMOVI, IDGRUPPROD) {
		var IDORDEMCLC = this.idOrdAjuInv =='P' ? 0 : 'U';

		var filter = [
			{
				"name": "CDGRUPPROD",
				"operator": "=",
				"value": CDGRUPPROD
			},
			{
				"name": "CDORDENA",
				"operator": "=",
				"value": CDORDENA
			},
			{
				"name": "CDFILIAL",
				"operator": "=",
				"value": CDFILIAL
			},
			{
				"name": "CDALMOXARIFE",
				"operator": "=",
				"value": ' '
			},
			{
				"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": ' '
			},
			{
				"name": "NRLOTEESTQ",
				"operator": "=",
				"value": ' '
			},
			{
				"name": "NRSUBLOTE",
				"operator": "=",
				"value": ' '
			},
			{
				"name": "IDORDEMCLC",
				"operator": "=",
				"value": IDORDEMCLC
			},
			{
				"name": "DTLANCMOVI",
				"operator": "=",
				"value": DTLANCMOVI
			},
			{
				"name": "IDGRUPPROD",
				"operator": "=",
				"value": IDGRUPPROD
			},
		];
		return Ajuste_Repository.importProducts(filter);
	};

	this.deleteItem = function(row){
		return Ajuste_Repository.deleteItem(row);
	};

	this.validaAjuste = function(rowProduto, dtLanc) {
		if(rowProduto.IDTIPOMOVI == 1) {
			var idOrdemClc = this.idOrdAjuInv == 'P' ? 0 : 'U';
			var filter = [
				{
					"name": "CDFILIMOVI",
					"operator": "=",
					"value": rowProduto.CDFILIMOVI
				},
				{
					"name": "CDPRODMOVI",
					"operator": "=",
					"value": rowProduto.CDPRODMOVI
				},
				{
					"name": "CDALMOXARIFE",
					"operator": "=",
					"value": rowProduto.CDALMOXARIFE
				},
				{
					"name": "CDLOCALESTOQ",
					"operator": "=",
					"value": rowProduto.CDLOCALESTOQ
				},
				{
					"name": "NRLOTEESTQ",
					"operator": "=",
					"value": rowProduto.NRLOTEESTQ
				},
				{
					"name": "NRSUBLOTE",
					"operator": "=",
					"value": rowProduto.NRSUBLOTE
				},
				{
					"name": "DTLANCMOVI",
					"operator": "=",
					"value": dtLanc
				},
				{
					"name": "IDORDEMCLC",
					"operator": "=",
					"value": idOrdemClc
				},
				{
					"name": "IDAJUSTEIMPORT",
					"operator": "=",
					"value": rowProduto.IDAJUSTEIMPORT || 'N'
				}
			];

			return Ajuste_Repository.validaAjuste(filter);
		}else
			return ZHPromise.when();
	};
	
	this.openPopupCusto = function(arrayCustos, popup){
		popup.dataSourceFilter = [];
		var popupPromisse = ZHPromise.defer();
		let data = popup.dataSource.data;
		let produtosInvalidos = [];
		arrayCustos.forEach(rowCusto =>{
			
			let rowItem = rowCusto.item;
			let valrAnte = rowCusto.valrAnte*rowItem.VRFATOCONV;
			let qtdeAnte = rowCusto.qtdeAnte;
			let valrMedi = rowCusto.valrMedi*rowItem.VRFATOCONV;
			let qtdeAtu = $.currency.unmaskString(rowItem.QTTOTLANCTO,{decimal:',',thousands:'.'});
			let valrAtu = valrMedi*qtdeAtu;
			let qntValida = this.validaQuantidade(valrAnte, qtdeAnte, valrMedi, qtdeAtu, valrAtu , rowItem.VRFATOCONV , rowItem.VRPTOLAJUSINV);
			data.push({'SITUACAO' : rowItem.CDARVPROD+' - '+rowItem.NMPRODUTO, 'VALORTOT' : NaN, 'QUANTIDADE' : NaN, 'CUSTMEDIO' : NaN});
			data.push({'SITUACAO' : 'Atual', 'VALORTOT' : valrAnte, 'QUANTIDADE' : qtdeAnte, 'CUSTMEDIO' : valrMedi});
			data.push({'SITUACAO' : 'Após Ajuste', 'VALORTOT' : valrAtu, 'QUANTIDADE' : qtdeAtu, 'CUSTMEDIO' : valrMedi , 'qntValida' : qntValida});
			
			if(!qntValida){
				data[0].ajusteValido = false; 
				produtosInvalidos.push(rowItem.CDARVPROD);
			}else{
				data[0].ajusteValido = true; 
			}
		}); 
		
		//montar datasource para mostrar no grid
		popup.promisse = popupPromisse;
		ScreenService.openPopup(popup);
		
		// troca Style das rows invalidas na widget Produto
		this.quantidadeInvalida(produtosInvalidos);
		return popupPromisse;
	};
	
	this.validaQuantidade = function(valrAnte, qtdeAnte, valrMedi, qtdeAtu, valrAtu , VRFATOCONV, VRPTOLAJUSINV  ){
            if(VRPTOLAJUSINV > 0 && VRPTOLAJUSINV !=null){
                if (valrAnte * VRFATOCONV !=  0){
                    if (Math.abs(((valrMedi*qtdeAtu*VRFATOCONV*100)/valrAnte*VRFATOCONV)-100) > VRPTOLAJUSINV){
                        return  false ;
                    }
                }
            }
         return true ; 
    };
    
	this.quantidadeInvalida =  function(produtos){
		let rowsProduto = templateManager.container.getWidget("Produtos").dataSource.data;
        if(produtos.length > 0){
            rowsProduto.forEach(row => {
            	produtos.forEach (produto => {
                if(produto === row.CDARVPROD){
                    row.qntValida =  false;
                }else {
                	 row.qntValida =  true;
                }     
            	});
			 });    
        }else {
        	 rowsProduto.forEach(row => {
        	 	 row.qntValida =  true;
        	 });
        }		
    };
    
    this.importarProdutosTxt = function(row){
    	return Ajuste_Repository.importarProdutosTxt(row);
    };
    
	this.importProductsFilial = function(CDORDENA, CDFILIAL, DTLANCESTQ, DATAPROC) {
		var filter = [
			{
				"name": "CDORDENA",
				"operator": "=",
				"value": CDORDENA
			},
			{
				"name": "P_CDFILIAL",
				"operator": "=",
				"value": CDFILIAL
			},
			{
				"name": "DTLANCESTQ",
				"operator": "=",
				"value": DTLANCESTQ
			},
			{
				"name": "P_DATAPROC",
				"operator": "=",
				"value": DATAPROC
			}
		];
		return Ajuste_Repository.importProductsFilial(filter);
	};

    this.gerarImplantacao = function(rowAjuste, itens){
    	return ValidacoesEstoque_Service.validaDataLancamento(rowAjuste.CDFILIAL,rowAjuste.DTLANCESTQ,'AJUSTE_IMP','EST',null).then(
    		() => {
				return this.save(itens.pop(), rowAjuste).then(
					success => {
						if(itens.length > 0){
							rowAjuste.NRLANCESTQ = success.dataset["/lanctoestoq/save"].NRLANCESTQ;
							return this.saveImportRows(itens, rowAjuste);	
						}
					}
				).catch(
					error => {
						if(error.data){
		                    ScreenService.showMessage(error.data.error);
		                }else{
		                    ScreenService.showMessage(error);
		                }	
					}
				);
    		}
    	).catch(
    		error => {
    			if(error.data){
                    ScreenService.showMessage(error.data.error);
                }else{
                    ScreenService.showMessage(error);
                }
    		}
    	);
    };
    
    this.questionConsolidaEmProdEstoque = function(datasetProdutos){
    	var defer = ZHPromise.defer();
    	var boolQuestion =  datasetProdutos.some(row => {
	    	return datasetProdutos.some(row2 => {
	                            if( (row2.NRSEQUITEM != row.NRSEQUITEM) && 
	                                (row2.CDPRODMOVI == row.CDPRODMOVI) &&
	                                (row2.NRLOTEESTQ == row.NRLOTEESTQ) &&
	                                (row2.CDALMOXARIFE == row.CDALMOXARIFE) &&
	                                (row2.CDLOCALESTOQ == row.CDLOCALESTOQ) &&
	                                (row2.NRSUBLOTE == row.NRSUBLOTE) )
	                                {
	                                    return true;
	                                }
	                        });
    	});
    	if(boolQuestion){
	    	ScreenService.confirmMessage('Existem dois ou mais produtos relacionados ao mesmo produto de estoque. Deseja consolidar a quantidade lançada para o produto de estoque?',
		        'question',
		        function() { //sim
		            defer.resolve(true);
		        }.bind(this),
		        function() { //não
		            defer.resolve(false);
	        }.bind(this));
    	}else{
    		defer.resolve(false);
    	}
    	return defer;
    };
    
    this.getImportMessage = function(importGroupCode){
    	
    };
    
    this.alreadyHasRetirada = function(lancamentoRow){
    	let filter = [
			{
				"name": "NRLANCESTQ",
				"operator": "=",
				"value": lancamentoRow.NRLANCESTQ
			},
			{
				"name": "CDFILIAL",
				"operator": "=",
				"value": lancamentoRow.CDFILIMOVI
			}
		];
    	return General.callRestEngine('est#/crud/confereRetiradaAjuste', 'FilterData', filter);
    };
    
    this.validaProdutoPRetirada = function(tipoRetirada){
    	let filter = [
			{
				"name": "CDTIPORETI",
				"operator": "=",
				"value": tipoRetirada
			}
		];
    	return General.callRestEngine('est#/crud/prodreti', 'FilterData', filter);
    };
    
    this.validaAllProdutosPRetirada = function(arrArvprod ,tipoRetirada){
    	let filter = [
    		{
				"name": "CDARVPROD",
				"operator": "IN",
				"value": arrArvprod
			},
			{
				"name": "CDTIPORETI",
				"operator": "=",
				"value": tipoRetirada
			}
		];
    	return General.callRestEngine('est#/crud/produto_prodreti', 'FilterData', filter);
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ajuste_Service', Ajuste_Service);
});

// FILE: ./assets/modules/ajuste/assets/js/services/Est44000RecalculoEstoque_Service.js
var Est44000RecalculoEstoque_Service = function(restEngine,Est44000RecalculoEstoque_Repository, maskEngine, ZhUtil) {
	this.getParamFilial = function(){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        }); 
	};

	this.recalcularEstoque = function(row){
      var mask = {
          type: "fix",
          params: {
              mask: getLocalVar('GER_CDPICTPROD')
          }
      };
	  var pfinal = Util.applyMask(row.CDPRODFIN, mask, maskEngine, ZhUtil).split('.');
	  var nvfinal = pfinal.length;
      var filterRow = Util.clone(row, ['__rightActions', '__leftActions']);
      filterRow.CDPRODFIN = row.CDPRODFIN + "zzzzzzzzzzzzzzzzzzzz";
      filterRow.CDPRODFIN = filterRow.CDPRODFIN.substring(0, 20);
	  return Est44000RecalculoEstoque_Repository.recalcularEstoque(filterRow);
	};
	
	this.saveProdutosIrregulares = function(data){
		return Est44000RecalculoEstoque_Repository.saveProdutosIrregulares(data);
	};
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est44000RecalculoEstoque_Service', Est44000RecalculoEstoque_Service);
});

// FILE: ./assets/modules/estoqueCego/assets/js/services/Est90000_PerfilOperadoresInventarioCego_Services.js
var Est90000_PerfilOperadoresInventarioCego_Services = function(General, Lib00206_InfComplProduto_Controller, ZHPromise, EstUtil, Est90000_PerfilOperadoresInventarioCego_Repository) {

    this.onSaveOperation = function(row) {
        return Est90000_PerfilOperadoresInventarioCego_Repository.onSaveOperation(row);
    };

    this.onDeleteOperation = function(row) {
        return Est90000_PerfilOperadoresInventarioCego_Repository.onDeleteOperation(row);
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est90000_PerfilOperadoresInventarioCego_Services', Est90000_PerfilOperadoresInventarioCego_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est91000_ProgramacaoDoInventarioCego_Services.js
var Est91000_ProgramacaoDoInventarioCego_Services = function(General, Lib00206_InfComplProduto_Controller, ZHPromise, EstUtil, Est91000_ProgramacaoDoInventarioCego_Repository) {

    this.getFilialLogada = function() {
        const filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.getComplementaryProductInformation = function(widget) {
        const promise = ZHPromise.defer();
        if (widget.currentRow.CDARVPROD) {
            widget.promise = promise;
            Lib00206_InfComplProduto_Controller.infComplementares(widget, widget.currentRow);
        }
        else {
            promise.reject();
        }
        return promise;
    };

    this.setGenerationDateandOperator = function(widget) {
        widget.dataSource.data.forEach(inventory => {
            inventory.DTGERACAOINVEST = inventory.DTGERACAOINVEST.substr(0, 10) + " - " + inventory.CDOPERADOR;
        });
    };

    this.saveProducts = function(row) {

        row.NRLOTEESTQ = row.NRLOTEESTQ && row.NRLOTEESTQ !== '' ? row.NRLOTEESTQ : ' ';
        row.NRSUBLOTE = row.NRSUBLOTE && row.NRSUBLOTE !== '' ? row.NRSUBLOTE : ' ';
        row.CDFILIAL = row.CDFILIAL == row.CDFILIMOVI ? row.CDFILIAL : row.CDFILIMOVI;

        return Est91000_ProgramacaoDoInventarioCego_Repository.saveProducts(row);
    };

    // Gets data about unit params and set it in filter currentRow
    this.getParamFilialData = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDFILIAL', 'inventoryGrid', false)
            }
	    ];

        return General.callRestEngine('est_ajuste#/crud/paramfilial', 'FilterData', filter);
    };
    
    this.getParamEstoqueData = function() {
        return General.callRestEngine('est#/crud/paramestoque', 'FilterData');
    };

    this.checkInventoryHandler = function(cdFilial, dtContagem, firstTest = true) {
        const row = {
            "CDFILIAL": cdFilial,
            "DTCONTAGEM": dtContagem,
            "FIRSTTEST": firstTest
        };

        return Est91000_ProgramacaoDoInventarioCego_Repository.checkInventoryHandler(row);
    };

    this.onGenerateInventory = function(row, filiais) {
        row.CDFILIAL = filiais;
        return Est91000_ProgramacaoDoInventarioCego_Repository.onGenerateInventory(row);
    };

    this.onDeleteInventory = function(dataSet) {
        return Est91000_ProgramacaoDoInventarioCego_Repository.onDeleteInventory(dataSet);
    };

    this.getDate = function(dateString) {
        let string = dateString.replaceAll('/', '').replaceAll(' ', '').replaceAll(':', '');
        return new Date(moment(string, "DDMMYYYYHHmmss").format('MM/DD/YYYY HH:mm:ss'));
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est91000_ProgramacaoDoInventarioCego_Services', Est91000_ProgramacaoDoInventarioCego_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est92000_ConsultaInventarioCego_Services.js
var Est92000_ConsultaInventarioCego_Services = function(General, Lib00206_InfComplProduto_Controller, ZHPromise, EstUtil, Est92000_ConsultaInventarioCego_Repository) {

    this.getFilialLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
    
    this.getDate = function(dateString) {
        let string = dateString.replaceAll('/','').replaceAll(' ','').replaceAll(':','');
        return new Date(moment(string, "DDMMYYYYHHmmss").format('MM/DD/YYYY HH:mm:ss'));
    };
    
};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est92000_ConsultaInventarioCego_Services', Est92000_ConsultaInventarioCego_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est93000_DigitacaoInventarioCego_Services.js
var Est93000_DigitacaoInventarioCego_Services = function(General, restEngine, Lib00206_InfComplProduto_Controller, ZHPromise, EstUtil, Est93000_DigitacaoInventarioCego_Repository) {

    this.getComplementaryProductInformation = function(widget) {
        const promise = ZHPromise.defer();
        if (widget.currentRow.CDARVPROD) {
            widget.promise = promise;
            Lib00206_InfComplProduto_Controller.infComplementares(widget, widget.currentRow);
        }
        else {
            promise.reject();
        }
        return promise;
    };

    this.getDtProcessa = function() {
        let filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        }).then((response) => {
            return response.dataset.paramfilial[0].DTPROCESSA;
        });
    };

    this.getDate = function(dateString) {
        let string = dateString.replaceAll('/', '').replaceAll(' ', '').replaceAll(':', '');
        return new Date(moment(string, "DDMMYYYYHHmmss").format('MM/DD/YYYY HH:mm:ss'));
    };

    this.save = function(row) {
        return Est93000_DigitacaoInventarioCego_Repository.save(row);
    };
    this.onConcludeDigitation = function(row) {
        return Est93000_DigitacaoInventarioCego_Repository.onConcludeDigitation(row);
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est93000_DigitacaoInventarioCego_Services', Est93000_DigitacaoInventarioCego_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est94000_AprovacaoInventarioCego_Services.js
var Est94000_AprovacaoInventarioCego_Services = function(General, restEngine, Lib00206_InfComplProduto_Controller, ZHPromise, EstUtil, Est94000_AprovacaoInventarioCego_Repository) {
    // Gets logged unit
    this.getFilialLogada = function() {
        const filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    // Gets data about unit params and set it in filter currentRow
    this.getParamFilialData = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": ""
            }
	    ];
        filter[0].value = EstUtil.getFieldValue('CDFILIAL', 'inventoryGrid', true);

        return General.callRestEngine('est_ajuste#/crud/paramfilial', 'FilterData', filter);
    };

    // Calls factory function
    this.onChangeSituation = function(rows, action) {
        let situationAction = { "situation": action };
        return Est94000_AprovacaoInventarioCego_Repository.onChangeSituation(rows, situationAction );
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est94000_AprovacaoInventarioCego_Services', Est94000_AprovacaoInventarioCego_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est95100_AnaliseInventarioCego_Services.js
var Est95100_AnaliseInventarioCego_Services = function(General) {
    // Gets logged unit
    this.getFilialLogada = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
			}
	    ];
        return General.callRestEngine('est_relatorios#/crud/filial_paramfilial_tipocusto', 'FilterData', filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est95100_AnaliseInventarioCego_Services', Est95100_AnaliseInventarioCego_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est95200_AceiteRecusaDivergencias_Services.js
var Est95200_AceiteRecusaDivergencias_Services = function(General) {
    // Gets logged unit
    this.getFilialLogada = function() {
        const filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est95200_AceiteRecusaDivergencias_Services', Est95200_AceiteRecusaDivergencias_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est95300_ControleFiliaisInventarioCego_Services.js
var Est95300_ControleFiliaisInventarioCego_Services = function(General) {
    // Gets logged unit
    this.getFilialLogada = function() {
        const filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est95300_ControleFiliaisInventarioCego_Services', Est95300_ControleFiliaisInventarioCego_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est96000_DivergenciasInventarioCego_Services.js
var Est96000_DivergenciasInventarioCego_Services = function(General, EstUtil, Est96000_DivergenciasInventarioCego_Repository) {
    // Gets logged unit
    this.getFilialLogada = function() {
        const filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    // Gets data about unit params and set it in filter currentRow
    this.getParamFilialData = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDFILIAL', 'inventoryGrid', true) || "FILIAL_LOGADA"
            }
	    ];

        return General.callRestEngine('est_ajuste#/crud/paramfilial', 'FilterData', filter);
    };
    
    // Manages actions functionalities data to sending to factory
    this.actionsManagementHandler = function(InventoryRow, rows, hasItems, action) {
        if(action == 'refuse'){
            if(InventoryRow.DSOBSRECAJUST == '') return;
        }
        return Est96000_DivergenciasInventarioCego_Repository.actionsManagementHandler(InventoryRow,rows, hasItems, action);
    };
    
    this.getGroupOperatorInfo = function() {
        const filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
	    ];
        return General.callRestEngine('est_estoqueCego#/crud/autgrupop_grupoper_opergrupop', 'FilterData', filter);
    };
    
    // get date from a string 
    this.getDate = function(dateString) {
        let string = dateString.replaceAll('/', '').replaceAll(' ', '').replaceAll(':', '');
        return new Date(moment(string, "DDMMYYYYHHmmss").format('MM/DD/YYYY HH:mm:ss'));
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est96000_DivergenciasInventarioCego_Services', Est96000_DivergenciasInventarioCego_Services);
});


// FILE: ./assets/modules/estoqueCego/assets/js/services/Est97000_PercentualDivergenciasInventarioCego_Services.js
var Est97000_PercentualDivergenciasInventarioCego_Services = function(General) {
    // Gets logged unit
    this.getFilialLogada = function() {
        const filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    // checks and Builds dataSet
    this.onCheckDataset = (row) => new Promise((resolve) => {
        let dataSet = [];
        let count = 0;
        const { CDFILIAL: Unities, CDPRODUTO, VRPERDIVESTCEG } = row;
        const getProductInfo = (unity) => {
            const filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": unity
                },
                {
                    "name": "CDPRODUTO",
                    "operator": "=",
                    "value": CDPRODUTO
                }
	        ];
            return General.callRestEngine('est#/crud/parperdivestceg', 'FilterData', filter).then((success) => {
                return success.dataset.parperdivestceg.length != 0 ? false : true;
            });
        };

        Unities.forEach((unity) => {
            getProductInfo(unity).then((newProduct) => {
                let newIndex = {
                    "__is_new": newProduct,
                    "changed": !newProduct,
                    "CDFILIAL": unity,
                    "CDPRODUTO": CDPRODUTO,
                    "VRPERDIVESTCEG": VRPERDIVESTCEG
                };
                dataSet.push(newIndex);
                count++;
                if (count === Unities.length) return resolve(dataSet);
            });
        });
        return Promise;
    });
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est97000_PercentualDivergenciasInventarioCego_Services', Est97000_PercentualDivergenciasInventarioCego_Services);
});


// FILE: ./assets/modules/lancamentos/assets/js/services/Est11000_EntradaEstoque_Service.js
function Est11000_EntradaEstoque_Service(Est11000_EntradaEstoque_Repository) {
    var permiteEstNegat;

    var dtprocessa;
    var idctrlestq;
    var idutilalmox;
    var idutillcestq;
    var idpermestneg;
    var idutillote;

	/**
	 * Essa função tem como finalidade verificar se um dataSource existe um produto igual ao
	 * produto que está sendo inserido no momento. É retirada a ultiam posição do array pois
	 * ela é o registro que o usuário está inserindo no momento.
	 *
	 * @param data | array de itens ja inclusos na retirada
	 * @param cdarvprod | código do produto que está sendo inserido
	 * @param cdalmoxarife | código do almoxarifado do produto que está sendo inserido
	 * @param cdlocalestoq | código do localização do produto que está sendo inserido
	 * @param nrloteestq | código do lote do produto que está sendo inserido
	 * @param nrsublote | código do sublote do produto que está sendo inserido
	 */
	this.checaItensDaRetirada = function(data, cdarvprod, cdalmoxarife, cdlocalestoq, nrloteestq, nrsublote) {
		var contem = false;
		data.splice(data.length -1, 1);
		data.forEach(function(item){
			 if(item.CDARVPROD === cdarvprod) {
			 	contem = true;
			 	if(item.CDALMOXARIFE !== cdalmoxarife) {
			 		contem = false;
			 	} else if(item.CDLOCALESTOQ !== cdlocalestoq) {
			 		contem = false;
			 	} else if(item.NRLOTEESTQ !== nrloteestq) {
			 		contem = false;
			 	} else if(item.NRSUBLOTE !== nrsublote) {
			 		contem = false;
			 	}
			 }
		});
		return contem;
	};

	/**
	 * Essa função tem a finalidade de consultar o cdtipocusto.
	 *
	 * @param cdfilial | filial movimentada
	 *
	 * @return string | valor do CDTIPOCUSTO
	 */
	this.getCdtipocusto = function(cdfilial) {
		return Est11000_EntradaEstoque_Repository.getCdtipocusto(cdfilial).then(function(callback){
			return callback.dataset.paramfilial[0].CDTIPCUSTEST;
		});
	};

	/**
	 * Essa função serve para consultar os itens da requisição que foi passada.
	 *
	 * @param cdfilial | código da filial movimentada
	 * @param nrrequesto | código da requisição passada
	 *
	 */
	this.getItensOfRequisicao = function(cdfilial, nrrequesto) {
		var filter = [{
						"name": "CDFILIAL",
						"operator": "=",
						"value": cdfilial
					  },
					  {
					  	"name": "NRREQUESTO",
						"operator": "=",
						"value": nrrequesto
					  },
					  {
					  	"name": "IDSTITREQU",
						"operator": "=",
						"value": 'A'
					  }];
		return Est11000_EntradaEstoque_Repository.findItensOfRequisicao(filter).then(function(succes){
			return succes.dataset.itrequesto_itens_requisicao;
		});
	};

    /**
     *
     *
     */
    this.geraNrsequitem= function(itens) {
		var nrsequitemArray=[0];
		itens.forEach(function(item) {
		    if(item.NRSEQUITEM)
			    nrsequitemArray.push(item.NRSEQUITEM);
		});
		return ('000'+(Math.max.apply(this,nrsequitemArray)+1)).slice(-4);
	};

	/**
	 * Essa função tem como finalidade montar chamar a função do repository que retorne
	 * as parametrizações da filial que foi passada para essa função. Depois disso é
	 * chegado se essa função foi chamada no inicio da tela. Se sim seta variavéis
	 * globais com as parametrizações necessárias na tela. Se não, retorna a row pesqui-
	 * sada.
	 *
	 * @param cdfilial | codigo da filial que foi selecionada no filtro
	 * @return array | retorna o retorno da função (Nome e código da filial)
	 * @return objetc | objeto com os campos que retornaram do select das parametrizações
	 */
	this.getParamFilial = function(cdfilial, onInit) {
	    var filter = [{ "name": "CDFILIAL",
	    				"operator": "=",
	    				"value": cdfilial
	    			},{ "name": "P_CDFILIAL",
	    				"operator": "=",
	    				"value": null}];
	    return Est11000_EntradaEstoque_Repository.getParamFilial(filter).then(function(succes){
	        var parans = succes.dataset.paramfilial_paramvalidacoes[0];
			if(parans) {
	        	if(onInit) {
	        		return this.setParametrizacoes(parans);
	        	} else  {
	        		return parans;
	        	}
	    	} else {
	    		return 'Não retornaram dados de parametrização para essa filial.';
	    	}
	    }.bind(this));
	};
	
	this.setBlankLocalEstFields = function(widget){
		if(!widget.currentRow.hasOwnProperty('CDALMOXARIFE')){
			widget.currentRow.CDALMOXARIFE = ' ';
		}
		if(!widget.currentRow.hasOwnProperty('CDLOCALESTOQ')){
			widget.currentRow.CDLOCALESTOQ = ' ';
		}
		
		if(!widget.currentRow.hasOwnProperty('NRLOTEESTQ')){
			widget.currentRow.NRLOTEESTQ = ' ';
		}
		
		if(!widget.currentRow.hasOwnProperty('NRSUBLOTE')){
			widget.currentRow.NRSUBLOTE = ' ';
		} 

	};

	this.getPosicaoDeEstoquePorProduto = function(row) {
	    var filter = [
	    	{
	    		"name": "CDPRODUTO",
				"operator": "=",
				"value": row.CDPRODUTO
			},
			{
	    		"name": "CDFILIAL",
				"operator": "=",
				"value": row.CDFILIAL
			},
			{
	    		"name": "CDALMOXARIFE",
				"operator": "=",
				"value": row.CDALMOXARIFE
			},
			{
	    		"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": row.CDLOCALESTOQ
			},
			{
	    		"name": "NRLOTEESTQ",
				"operator": "=",
				"value": row.NRLOTEESTQ
			},
			{
	    		"name": "NRSUBLOTE",
				"operator": "=",
				"value": row.NRSUBLOTE
			}
		];

	    return Est11000_EntradaEstoque_Repository.getPosicaoDeEstoquePorProduto(filter).then(function(success){
	    	return success.dataset.posestdia[0];
	    });
	};

	/**
	 *
	 *
	 */
	this.setQtLanctoEst = function(rowItem) {
		let arrValues = ['QTTOTLANCTO', 'QTDEVOLESTOQ', 'VRFATOCONV'];
		arrValues.forEach(value =>{
			if(rowItem[value] == '' || rowItem[value] == undefined)
				rowItem[value] = value == 'VRFATOCONV' ? '1' : '0.0';
		});
		
    	rowItem.QTLANCTOEST = (parseFloat(rowItem.QTTOTLANCTO.toString().replace(',', '.')) - parseFloat(rowItem.QTDEVOLESTOQ.toString().replace(',', '.'))) * rowItem.VRFATOCONV;
    };

	/**
	 *
	 *
	 */
	this.save = function(lancamento,itens) {
	    // itens.forEach(function(item){
			// item.CDTIPORETI = lancamento.CDTIPORETI;
			// item.CDCENTCUST = lancamento.CDCENTCUST;
			// item.DTLANCMOVI = lancamento.DTLANCESTQ;

			//if(item.QTLANCTOEST > item.QTESTOQDIA){
 // código delphi para validar estoque negativo, conferir com fabricio se precisa, ao que parece o parametro que IDPERMESTNEG hoje nunca será 'S'
 // if (cdsParamFilialIDPERMESTNEG.Value = 'S') then begin
 //   cdsItLanctoEstCDARVPROD.EditMask := DBPict(ptProduto);
 //   if cdsParamFilialIDPERMVISPOSEST.Value <> 'N' then begin
 //     if MessageDlg(Format(msgPadrao.msg('Quant_Est'),[cdsItLanctoEstCDARVPROD.AsString, FormatFloat('#,##0.000 ',cdsVdPosEstDiaQTESTOQDIA.Value),cdsVdPosEstDiaSGUNIDADE.Value ])+ #13 + msgPadrao.msg('Confirma'),mtConfirmation,[mbYes,mbNo],0) = mrNo then begin
 //       dbgItemReti.SelectedIndex := IndiceCol(dbgItemReti,'QTLANCTOEST');
 //       Abort;
 //     end;
 //   end
 //   else begin
 //     if MessageDlg(msgPadrao.msg('Quant-Est')+msgPadrao.msg('Confirma'),mtConfirmation,[mbYes,mbNo],0) = mrNo then begin
 //       dbgItemReti.SelectedIndex := IndiceCol(dbgItemReti,'QTLANCTOEST');
 //       Abort;
 //     end;
 //   end;
 // end
			//}
			// Est11000_EntradaEstoque_Repository.save(lancamento,itens);
		// });

		itens.CDTIPORETI = lancamento.CDTIPORETI;
		itens.CDCENTCUST = lancamento.CDCENTCUST;
		itens.DTLANCMOVI = lancamento.DTLANCESTQ;

	};

	/**
	 *
	 *
	 */
	this.saveRetirada = function(row) {
		return Est11000_EntradaEstoque_Repository.saveRetirada(row);
	};

	/**
	 *
	 *
	 */
    this.setVrTotLancto = function(rowItem) {
    	rowItem.VRTOTLANCTO = parseFloat(rowItem.QTTOTLANCTO.toString().replace(',', '.'))  * parseFloat(rowItem.VRUNILANCTO.toString().replace(',', '.'));
    	rowItem.VRCUSTENTR  = rowItem.VRTOTLANCTO;
    };
    
    this.setVrlanctoest = function(rowItem) {
    	rowItem.VRLANCTOEST    = parseFloat(rowItem.VRUNILANCTO.toString().replace(',', '.')) / parseFloat(rowItem.VRFATOCONV.toString().replace(',', '.'));
    	rowItem.VRLANCTOBRUT   = rowItem.VRLANCTOEST;
    };

	/**
	 * Essa função de tem como finalidade de setar as variavies desse controle com as
	 * clonuas que retornaram do selec das parametrizações da filial. Retorna a filial
	 * que foi passada para o select
	 *
	 * @param parans | um objeto com a row de parametrizações da filial
	 * @return array | array com o nome código da filial do select
	 */
	this.setParametrizacoes = function(parans) {
	    dtprocessa = parans.DTPROCESSA;
    	idctrlestq = parans.IDCTRLESTQ;
    	idutilalmox = parans.IDUTILALMOX;
    	idutillcestq = parans.IDUTILLCESTQ;
    	idpermestneg = parans.IDPERMESTNEG;
    	idutillote = parans.IDUTILLOTE;
    	
    	let paransReturn = {};
    	paransReturn.CDFILIAL = parans.CDFILIAL;
    	paransReturn.NMFILIAL = parans.NMFILIAL;
    	paransReturn.IDCTRLESTQ = parans.IDCTRLESTQ;
    	paransReturn.IDUTILLOTE = parans.IDUTILLOTE;
    	return paransReturn;
	};

	/**
	 *
	 *
	 */
	this.validaItemRepetido=function(currentRow,data) {
		var validacao = true;
		data.forEach(function(row){
			if(currentRow.CDPRODUTO ===  row.CDPRODUTO && currentRow!== row)
				validacao = false;
		});
		return validacao;
	};

	/************************** Retorna parametrizações da filial **************************/

    /**
	 * Essa função tem como finalidae retornar se a filial selecionada no fritro utiliza
	 * controle estoque (true) ou não (false).
	 *
	 * @return bool | retorna true se utiliza e false se não utiliza.
	 */
	this.controlaEstq = function() {
	    if(idctrlestq === 'S') {
	        return true;
	    } else {
	        return false;
	    }
	};

    /**
     * Essa função tem como finalidade retornar a data de processamento da filial selecionada
     * no filtro. Se a filial não usar data de processamento é retornado a data atual, mas
     * essa validação é feita no backend pela query.
     *
     * @return string | data de processamento da filial
     */
    this.getDtprocessa = function() {
        return dtprocessa;
    };

	/**
	 * Essa função tem como finalidae retornar se a filial selecionada no fritro utiliza
	 * almoxarifado (true) ou não (false).
	 *
	 * @return bool | retorna true se utiliza e false se não utiliza.
	 */
	this.utilizaAlmoraifado = function() {
	    if(idutilalmox === 'S') {
	        return true;
	    } else {
	        return false;
	    }
	};

	/**
	 * Essa função tem como finalidae retornar se a filial selecionada no fritro utiliza
	 * localizacao (true) ou não (false).
	 *
	 * @return bool | retorna true se utiliza e false se não utiliza.
	 */
	this.utilizaLocalizacao = function() {
	    if(idutillcestq === 'S') {
	        return true;
	    } else {
	        return false;
	    }
	};
	
	var decodeTextViewFloat = function (fieldRow) {
    	if(typeof fieldRow === 'string'){
            fieldRow = fieldRow.replace(/\./g, '');
            fieldRow = fieldRow.replace(',', '.');
    	}
        var viewRow = parseFloat(fieldRow);
    	viewRow = isNaN(viewRow) ? "" : viewRow;
        return addThousandSeparators(viewRow);
    };
    
    function addThousandSeparators(value) {
        value += '';
        var x = value.split(',');
        var x1 = x[0];
        var x2 = x.length > 1 ? ',' + x[1] : '';
        var rgx = /(\d+)(\d{3})/;
        while (rgx.test(x1)) {
            x1 = x1.replace(rgx, '$1.$2');
        }

        return x1 + x2;
    }
	

}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est11000_EntradaEstoque_Service', Est11000_EntradaEstoque_Service);
});

// FILE: ./assets/modules/lancamentos/assets/js/services/Est58000_RetiradaEstoque_Service.js
function Est58000_RetiradaEstoque_Service(Est58000_RetiradaEstoque_Repository,localStorageEngine, Lanctoestoq_Repository,Itlanctoest_Repository, ScreenService ) {
    var permiteEstNegat;

    var dtprocessa;
    var idctrlestq;
    var idutilalmox;
    var idutillcestq;
    var idpermestneg;

	/**
	 * Essa função tem como finalidade verificar se um dataSource existe um produto igual ao
	 * produto que está sendo inserido no momento. É retirada a ultiam posição do array pois
	 * ela é o registro que o usuário está inserindo no momento.
	 *
	 * @param data | array de itens ja inclusos na retirada
	 * @param cdarvprod | código do produto que está sendo inserido
	 * @param cdalmoxarife | código do almoxarifado do produto que está sendo inserido
	 * @param cdlocalestoq | código do localização do produto que está sendo inserido
	 * @param nrloteestq | código do lote do produto que está sendo inserido
	 * @param nrsublote | código do sublote do produto que está sendo inserido
	 */
	this.checaItensDaRetirada = function(data, cdarvprod, cdalmoxarife, cdlocalestoq, nrloteestq, nrsublote) {
		var contem = false;
		data.splice(data.length -1, 1);
		data.forEach(function(item){
			 if(item.CDARVPROD === cdarvprod) {
			 	contem = true;
			 	if(item.CDALMOXARIFE !== cdalmoxarife) {
			 		contem = false;
			 	} else if(item.CDLOCALESTOQ !== cdlocalestoq) {
			 		contem = false;
			 	} else if(item.NRLOTEESTQ !== nrloteestq) {
			 		contem = false;
			 	} else if(item.NRSUBLOTE !== nrsublote) {
			 		contem = false;
			 	}
			 }
		});
		return contem;
	};

	/**
	 * Essa função tem a finalidade de consultar o cdtipocusto.
	 *
	 * @param cdfilial | filial movimentada
	 *
	 * @return string | valor do CDTIPOCUSTO
	 */
	this.getCdtipocusto = function(cdfilial) {
		return Est58000_RetiradaEstoque_Repository.getCdtipocusto(cdfilial).then(function(callback){
			return callback.dataset.paramfilial[0].CDTIPCUSTEST;
		});
	};

	/**
	 * Essa função serve para consultar os itens da requisição que foi passada.
	 *
	 * @param cdfilial | código da filial movimentada
	 * @param nrrequesto | código da requisição passada
	 *
	 */
	this.getItensOfRequisicao = function(cdfilial, nrrequesto) {
		var filter = [{
						"name": "CDFILIAL",
						"operator": "=",
						"value": cdfilial
					  },
					  {
					  	"name": "NRREQUESTO",
						"operator": "=",
						"value": nrrequesto
					  },
					  {
					  	"name": "IDSTITREQU",
						"operator": "=",
						"value": 'A'
					  }];
		return Est58000_RetiradaEstoque_Repository.findItensOfRequisicao(filter).then(function(succes){
			return succes.dataset.itrequesto_itens_requisicao;
		});
	};

    /**
     *
     *
     */
    this.geraNrsequitem= function(itens) {
		var nrsequitemArray=[0];
		itens.forEach(function(item) {
		    if(item.NRSEQUITEM)
			    nrsequitemArray.push(item.NRSEQUITEM);
		});
		return ('000'+(Math.max.apply(this,nrsequitemArray)+1)).slice(-4);
	};

	/**
	 * Essa função tem como finalidade montar chamar a função do repository que retorne
	 * as parametrizações da filial que foi passada para essa função. Depois disso é
	 * chegado se essa função foi chamada no inicio da tela. Se sim seta variavéis
	 * globais com as parametrizações necessárias na tela. Se não, retorna a row pesqui-
	 * sada.
	 *
	 * @param cdfilial | codigo da filial que foi selecionada no filtro
	 * @return array | retorna o retorno da função (Nome e código da filial)
	 * @return objetc | objeto com os campos que retornaram do select das parametrizações
	 */
	this.getParamFilial = function(cdfilial, onInit) {
	    var filter = [{ "name": "CDFILIAL",
	    				"operator": "=",
	    				"value": cdfilial
	    			},{ "name": "P_CDFILIAL",
	    				"operator": "=",
	    				"value": null}];
	    return Est58000_RetiradaEstoque_Repository.getParamFilial(filter).then(function(succes){
	        var parans = succes.dataset.paramfilial_paramvalidacoes[0];
			if(parans) {
	        	if(onInit) {
	        		return this.setParametrizacoes(parans);
	        	} else  {
	        		return parans;
	        	}
	    	} else {
	    		return 'Não retornaram dados de parametrização para essa filial.';
	    	}
	    }.bind(this));
	};
	
	this.setBlankLocalEstFields = function(widget){
		if(!widget.currentRow.hasOwnProperty('CDALMOXARIFE')){
			widget.currentRow.CDALMOXARIFE = ' ';
		}
		if(!widget.currentRow.hasOwnProperty('CDLOCALESTOQ')){
			widget.currentRow.CDLOCALESTOQ = ' ';
		}
		
		if(!widget.currentRow.hasOwnProperty('NRLOTEESTQ')){
			widget.currentRow.NRLOTEESTQ = ' ';
		}
		
		if(!widget.currentRow.hasOwnProperty('NRSUBLOTE')){
			widget.currentRow.NRSUBLOTE = ' ';
		} 

	};

	this.getPosicaoDeEstoquePorProduto = function(row) {
	    var filter = [
	    	{
	    		"name": "CDPRODUTO",
				"operator": "=",
				"value": row.CDPRODUTO
			},
			{
	    		"name": "CDFILIAL",
				"operator": "=",
				"value": row.CDFILIAL
			},
			{
	    		"name": "CDALMOXARIFE",
				"operator": "=",
				"value": row.CDALMOXARIFE
			},
			{
	    		"name": "CDLOCALESTOQ",
				"operator": "=",
				"value": row.CDLOCALESTOQ
			},
			{
	    		"name": "NRLOTEESTQ",
				"operator": "=",
				"value": row.NRLOTEESTQ
			},
			{
	    		"name": "NRSUBLOTE",
				"operator": "=",
				"value": row.NRSUBLOTE
			}
		];

	    return Est58000_RetiradaEstoque_Repository.getPosicaoDeEstoquePorProduto(filter).then(function(success){
	    	return success.dataset.posestdia[0];
	    });
	};

	/**
	 *
	 *
	 */
	this.setQtLanctoEst = function(rowItem) {
    	rowItem.QTLANCTOEST = (rowItem.QTTOTLANCTO - rowItem.QTDEVOLESTOQ)*rowItem.VRFATOCONV;
    };

	/**
	 *
	 *
	 */
	this.save = function(lancamento,itens) {
	    // itens.forEach(function(item){
			// item.CDTIPORETI = lancamento.CDTIPORETI;
			// item.CDCENTCUST = lancamento.CDCENTCUST;
			// item.DTLANCMOVI = lancamento.DTLANCESTQ;

			//if(item.QTLANCTOEST > item.QTESTOQDIA){
 // código delphi para validar estoque negativo, conferir com fabricio se precisa, ao que parece o parametro que IDPERMESTNEG hoje nunca será 'S'
 // if (cdsParamFilialIDPERMESTNEG.Value = 'S') then begin
 //   cdsItLanctoEstCDARVPROD.EditMask := DBPict(ptProduto);
 //   if cdsParamFilialIDPERMVISPOSEST.Value <> 'N' then begin
 //     if MessageDlg(Format(msgPadrao.msg('Quant_Est'),[cdsItLanctoEstCDARVPROD.AsString, FormatFloat('#,##0.000 ',cdsVdPosEstDiaQTESTOQDIA.Value),cdsVdPosEstDiaSGUNIDADE.Value ])+ #13 + msgPadrao.msg('Confirma'),mtConfirmation,[mbYes,mbNo],0) = mrNo then begin
 //       dbgItemReti.SelectedIndex := IndiceCol(dbgItemReti,'QTLANCTOEST');
 //       Abort;
 //     end;
 //   end
 //   else begin
 //     if MessageDlg(msgPadrao.msg('Quant-Est')+msgPadrao.msg('Confirma'),mtConfirmation,[mbYes,mbNo],0) = mrNo then begin
 //       dbgItemReti.SelectedIndex := IndiceCol(dbgItemReti,'QTLANCTOEST');
 //       Abort;
 //     end;
 //   end;
 // end
			//}
			// Est58000_RetiradaEstoque_Repository.save(lancamento,itens);
		// });

		itens.CDTIPORETI = lancamento.CDTIPORETI;
		itens.CDCENTCUST = lancamento.CDCENTCUST;
		itens.DTLANCMOVI = lancamento.DTLANCESTQ;

	};

	/**
	 *
	 *
	 */
	this.saveRetirada = function(row) {
		return Est58000_RetiradaEstoque_Repository.saveRetirada(row);
	};

	/**
	 *
	 *
	 */
    this.setVrTotLancto = function(rowItem) {
    	rowItem.VRTOTLANCTO = rowItem.QTTOTLANCTO * rowItem.VRUNILANCTO;
    };

	/**
	 * Essa função de tem como finalidade de setar as variavies desse controle com as
	 * clonuas que retornaram do selec das parametrizações da filial. Retorna a filial
	 * que foi passada para o select
	 *
	 * @param parans | um objeto com a row de parametrizações da filial
	 * @return array | array com o nome código da filial do select
	 */
	this.setParametrizacoes = function(parans) {
	    dtprocessa = parans.DTPROCESSA;
    	idctrlestq = parans.IDCTRLESTQ;
    	idutilalmox = parans.IDUTILALMOX;
    	idutillcestq = parans.IDUTILLCESTQ;
    	idpermestneg = parans.IDPERMESTNEG;
    	return [parans.CDFILIAL, parans.NMFILIAL];
	};

	/**
	 *
	 *
	 */
	this.validaItemRepetido=function(currentRow,data) {
		var validacao = true;
		data.forEach(function(row){
			if(currentRow.CDPRODUTO ===  row.CDPRODUTO && currentRow!== row)
				validacao = false;
		});
		return validacao;
	};

	/************************** Retorna parametrizações da filial **************************/

    /**
	 * Essa função tem como finalidae retornar se a filial selecionada no fritro utiliza
	 * controle estoque (true) ou não (false).
	 *
	 * @return bool | retorna true se utiliza e false se não utiliza.
	 */
	this.controlaEstq = function() {
	    if(idctrlestq === 'S') {
	        return true;
	    } else {
	        return false;
	    }
	};

    /**
     * Essa função tem como finalidade retornar a data de processamento da filial selecionada
     * no filtro. Se a filial não usar data de processamento é retornado a data atual, mas
     * essa validação é feita no backend pela query.
     *
     * @return string | data de processamento da filial
     */
    this.getDtprocessa = function() {
        return dtprocessa;
    };

	/**
	 * Essa função tem como finalidae retornar se a filial selecionada no fritro utiliza
	 * almoxarifado (true) ou não (false).
	 *
	 * @return bool | retorna true se utiliza e false se não utiliza.
	 */
	this.utilizaAlmoraifado = function() {
	    if(idutilalmox === 'S') {
	        return true;
	    } else {
	        return false;
	    }
	};

	/**
	 * Essa função tem como finalidae retornar se a filial selecionada no fritro utiliza
	 * localizacao (true) ou não (false).
	 *
	 * @return bool | retorna true se utiliza e false se não utiliza.
	 */
	this.utilizaLocalizacao = function() {
	    if(idutillcestq === 'S') {
	        return true;
	    } else {
	        return false;
	    }
	};
	
	this.getOperadorLogado = function(){
		var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];

        return localStorageEngine.download("est#/crud/operador", filter);
	};
	
	this.importarProdutosTxt = function(row){
    	return Est58000_RetiradaEstoque_Repository.importarProdutosTxt(row);
    };
    
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est58000_RetiradaEstoque_Service', Est58000_RetiradaEstoque_Service);
});

// FILE: ./assets/modules/relatorios/assets/js/services/Est12000_EmissaoDosLancamentosDeEntrada_Service.js
function Est12000_EmissaoDosLancamentosDeEntrada_Service(ScreenService, General, templateManager,ZhFilterPreferences,EstUtil) {
    this.getParamFilialData = function(widget){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": ""
            }
	    ];
	   filter[0].value = EstUtil.getFieldValue('CDFILIAL','EmissaoEntradaEstoque',true);
	    
	    return General.callRestEngine('est_ajuste#/crud/paramfilial', 'FilterData', filter).then(paramFilialData => {
	        if (paramFilialData.dataset.paramfilial.length != 0){
    	        widget.currentRow.IDUTILALMOX = paramFilialData.dataset.paramfilial[0].IDUTILALMOX;
                widget.currentRow.IDUTILLCESTQ = paramFilialData.dataset.paramfilial[0].IDUTILLCESTQ;
	        }
	    });
    };
}
    Configuration(function(ContextRegister){
    ContextRegister.register('Est12000_EmissaoDosLancamentosDeEntrada_Service', Est12000_EmissaoDosLancamentosDeEntrada_Service);
});

// FILE: ./assets/modules/relatorios/assets/js/services/Est31300_MovimentacaodeEstoque_Service.js
var Est31300_MovimentacaodeEstoque_Service = function(Est31300_MovimentacaodeEstoque_Repository, restEngine, ZHPromise) {

    this.requestReport = function(row) {

        var mascara = 20 - row.CDPRODFIN.length;
        row.CDPRODFIN = row.CDPRODFIN + 'z'.repeat(mascara).substring(0, mascara);

        // row.CDPRODFIN = row.CDPRODFIN + "zzzzzzzzzz";
        // row.CDPRODFIN = row.CDPRODFIN.substring(0, 10);

        if (row.REPORT_NAME === 'Est31300' ||
            row.REPORT_NAME === 'Est31300MovEstoqueSintetico' ||
            row.REPORT_NAME === 'Est31400MovEstoqueConsolidadoAnali' ||
            row.REPORT_NAME === 'Est31400MovEstoqueConsolidadoSint' ||
            row.REPORT_NAME === 'Est31300MovEstoqueSinteticoQuantidade' ||
            row.REPORT_NAME == 'EST31300SINTCONS') {

            row.IDLANCTO = row.IDLANCTO || 3;
            row.CUSTOS = row.CUSTOS || 1;
        }
        return Est31300_MovimentacaodeEstoque_Repository.getReport(row);
    };

    this.retornaParamUnidade = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.requestValidations = function(cdfilial) {
        var deferred = ZHPromise.defer();

        this.retornaParamUnidade(cdfilial).then(function(response) {
            if (response.dataset.paramfilial) {
                var ret = Array();

                ret.IDUTILALMOX = (response.dataset.paramfilial[0].IDUTILALMOX == 'S') ? true : false;
                ret.IDUTILLCESTQ = (response.dataset.paramfilial[0].IDUTILLCESTQ == 'S') ? true : false;
                ret.IDUTILLOTE = (response.dataset.paramfilial[0].IDUTILLOTE == 'S') ? true : false;

                deferred.resolve(ret);
            }
            else {
                deferred.reject({ "error": { "message": "Request error." } });
            }
        });

        return deferred;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est31300_MovimentacaodeEstoque_Service', Est31300_MovimentacaodeEstoque_Service);
});


// FILE: ./assets/modules/relatorios/assets/js/services/Est32000_GiroDeEstoque_Service.js
var Est32000_GiroDeEstoque_Service = function (Est45000_EmissaoInventario_Repository) {

    this.getParamFilial = function(cdfilial){
        return Est45000_EmissaoInventario_Repository.getParamFilial(cdfilial);
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Est32000_GiroDeEstoque_Service', Est32000_GiroDeEstoque_Service);
});


// FILE: ./assets/modules/relatorios/assets/js/services/Est32100_EvolucaoDeCusto_Service.js
var Est32100_EvolucaoDeCusto_Service = function(Est32100_EvolucaoDeCusto_Repository, restEngine, ZHPromise) {
    this.requestReport = function(row) {
        row.CDARVPRODFIN = row.CDARVPRODFIN + "zzzzzzzzzzzzzzzzzzz";
        row.CDARVPRODFIN = row.CDARVPRODFIN.substring(0, 20);
        if(row.REPORT_NAME != 'Est32101PercentMinimo') 
            row.PERMINIMO  = '';
            
        return Est32100_EvolucaoDeCusto_Repository.getReport(row);
    };

    this.retornaParamUnidade = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.requestValidations = function(cdfilial) {
        var deferred = ZHPromise.defer();

        this.retornaParamUnidade(cdfilial).then(function(response){
            if(response.dataset.paramfilial) {
                var ret = Array();
                
                deferred.resolve(ret);
            } else {
                deferred.reject({"error": {"message": "Request error."}});
            }
        });

        return deferred;
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est32100_EvolucaoDeCusto_Service', Est32100_EvolucaoDeCusto_Service);
});


// FILE: ./assets/modules/relatorios/assets/js/services/Est32200_ProdutosSemMovimentacao_Service.js
function Est32200_ProdutosSemMovimentacao_Service(Est32200_ProdutosSemMovimentacao_Repository, General) {
	var idutilalmox;
	var idutillcestq;
	var Dtprocessa;

	this.setParamFilial = function(cdfilial) {
		return Est32200_ProdutosSemMovimentacao_Repository.getParamFilial(cdfilial).then(function(banco) {
			var paramFilial = banco.dataset.paramfilial_paramvalidacoes[0];
			idutilalmox = paramFilial.IDUTILALMOX;
			idutillcestq = paramFilial.IDUTILLCESTQ;
			Dtprocessa = paramFilial.DTPROCESSA;
		});
	};

	this.utilizaAlmox = function() {
		if(idutilalmox === 'S'){
			return true;
		} else {
			return false;
		}
	};

	this.utilizaLocal = function() {
		if(idutillcestq === 'S') {
			return true;
		} else {
			return false;
		}
	};

	this.getDtlimite = function(dtmilite){
		var data = moment(Dtprocessa, 'DD/MM/YYYY').subtract(dtmilite, 'days').format('DD/MM/YYYY');
		return data;
	};
    
    this.getFilter = function(widget, fields, prodFin) {
        let row = widget.appliedFilter.FILTER_ROW;
        let filter = [];
        fields.forEach((f) => {
            filter.push(
            {
                "name": f,
                "operator": "=",
                "value": (f !== 'CDPRODFIN')? (f == 'NRARVOREPROD')? getNrarvoreprod(row) : row[f] : getProdFinFormatado(row.CDPRODFIN)
            });
        });
        return filter;
	};
	
	function getNrarvoreprod(row) {
		if (row.NRARVOREPRODFILTER) {
			return row.NRARVOREPRODFILTER;
		} else {
			return "T";
		}
	}
    
    function getProdFinFormatado(prodfin) {
        var masksize = getLocalVar('CDPICTPROD').replace(/[\.\-]/g, '').length - prodfin.length;
        return General.fill(prodfin, masksize, true, 'z');
    }

	this.gerarRelatorio = function(row){
		var newRow = {
			"REPORT_NAME" : 'EST32200',
			"NRORG" : row.NRORG,
			"CDFILIAL" : row.CDFILIAL,
			"CDALMOXARIFE" : row.CDALMOXARIFE,
			"CDLOCALESTOQ" : row.CDLOCALESTOQ,
			"CDPRODINI" : row.CDPRODINI,
			"CDPRODFIN" : row.CDPRODFIN,
			"DTLIMITE" : row.DTLIMITE,
			"IDTIPOMOVI" : row.IDTIPOMOVI,
			"ORDENACAO" : row.ORDENACAO,
			"NRDIAS" : row.NRDIAS,
			"NRTPARVPROD" : row.NRTPARVPROD,
			"NRARVOREPROD" : row.NRARVOREPRODFILTER,
			"DTPROCESSA" : moment(Dtprocessa, 'DD/MM/YYYY').format('DD/MM/YYYY')
		};
		return newRow;
		//return Est32200_ProdutosSemMovimentacao_Repository.gerarRelatorio(newRow);
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est32200_ProdutosSemMovimentacao_Service', Est32200_ProdutosSemMovimentacao_Service);
});

// FILE: ./assets/modules/relatorios/assets/js/services/Est32700_ValidadeDeProdutos_Service.js
function Est32700_ValidadeDeProdutos_Service(ScreenService, General, templateManager,ZhFilterPreferences,EstUtil) {
    this.getParamFilialData = function(widget){
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": ""
            }
	    ];
	   filter[0].value = EstUtil.getFieldValue('CDFILIAL','ValidadeDeProdutos',true);
	    
	    return General.callRestEngine('est_ajuste#/crud/paramfilial', 'FilterData', filter).then(paramFilialData => {
	        if (paramFilialData.dataset.paramfilial.length != 0){
    	        widget.currentRow.IDUTILALMOX = paramFilialData.dataset.paramfilial[0].IDUTILALMOX;
                widget.currentRow.IDUTILLCESTQ = paramFilialData.dataset.paramfilial[0].IDUTILLCESTQ;
	        }
	    });
    };
}
    Configuration(function(ContextRegister){
    ContextRegister.register('Est32700_ValidadeDeProdutos_Service', Est32700_ValidadeDeProdutos_Service);
});

// FILE: ./assets/modules/relatorios/assets/js/services/Est45000_EmissaoInventario_Service.js
var est45000_EmissaoInventario_Service = function (Est45000_EmissaoInventario_Repository) {

    this.getParamFilial = function(cdfilial){
        return Est45000_EmissaoInventario_Repository.getParamFilial(cdfilial);
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Est45000_EmissaoInventario_Service', est45000_EmissaoInventario_Service);
});


// FILE: ./assets/modules/relatorios/assets/js/services/Est46000_EmissaoAjusteInventario_Service.js
var Est46000_EmissaoAjusteInventario_Service = function(Est46000_EmissaoAjusteInventario_Repository, restEngine, ZHPromise) {
    
    this.requestReport = function(row) {
        row.CDARVPRODFIN = row.CDARVPRODFIN + "zzzzzzzzzzzzzzzzzzzz";
        row.CDARVPRODFIN = row.CDARVPRODFIN.substring(0, 20);
        if(row.REPORT_NAME === 'EST46000' || row.REPORT_NAME === 'EST46000_retirada') {
            row.IDLANCTO  = row.IDLANCTO || 3;
            row.CUSTOS    = row.CUSTOS || 1;
            row.ORDENANOME = row.ORDENA || 0;
        }
        return Est46000_EmissaoAjusteInventario_Repository.getReport(row);
    };

    this.retornaParamUnidade = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.requestValidations = function(cdfilial) {
        var deferred = ZHPromise.defer();

        this.retornaParamUnidade(cdfilial).then(function(response){
            if(response.dataset.paramfilial) {
                var ret = Array();

                ret.IDUTILALMOX = (response.dataset.paramfilial[0].IDUTILALMOX == 'S')? true : false;
                ret.IDUTILLCESTQ = (response.dataset.paramfilial[0].IDUTILLCESTQ == 'S')? true : false;
                ret.IDUTILLOTE = (response.dataset.paramfilial[0].IDUTILLOTE == 'S')? true : false;
                
                deferred.resolve(ret);
            } else {
                deferred.reject({"error": {"message": "Request error."}});
            }
        });

        return deferred;
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('Est46000_EmissaoAjusteInventario_Service', Est46000_EmissaoAjusteInventario_Service);
    
});


// FILE: ./assets/modules/relatorios/assets/js/services/Est70000_ResumoDeOperacoes_Service.js
var Est70000_ResumoDeOperacoes_Service = function (General, Est70000_ResumoDeOperacoes_Repository) {

    this.getUnityInfo = function(cdfilial) {
        const filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial || "FILIAL_LOGADA"
            }
	    ];
        return General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter);
    };
    
    this.getfieldsLabels = function(cdgrupprod) {
        const filter = [
            {
                "name": "CDGRUPPROD",
                "operator": "=",
                "value": cdgrupprod
            }
	    ];
        return General.callRestEngine('est#/crud/subgrupprod', 'FilterData', filter);
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Est70000_ResumoDeOperacoes_Service', Est70000_ResumoDeOperacoes_Service);
});


// FILE: ./assets/js/controllers/Est25000_produtos_processados_Controller.js
var Est25000_produtos_processados_Controller = function(General, ScreenService, templateManager, ZHPromise, EstUtil, restEngine){
    
    this.openForm = function(widget){
        let widgetGridPrincipal = widget;
        let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado');
        let widgetGrid   = templateManager.container.getWidget('gridFilhaProdutoProcessado');
        widgetTabbed.currentRow = widgetGridPrincipal.currentRow;
        let fieldDsconv = widgetTabbed.getField('DSCONVERCAO');
        let fieldPropQuant = widgetGrid.getField('VRPROPQUANTIDADE');
        let dsSGUNID = widget.currentRow.SGUNIDADE? widget.currentRow.SGUNIDADE : '';
        let dsSGUNIDESTO = widget.currentRow.SGUNIDADEPESTO? widget.currentRow.SGUNIDADEPESTO : '';
        fieldDsconv.label = 'Fator Conversão ' + dsSGUNID + ' para ' + dsSGUNIDESTO;
        fieldPropQuant.label = 'Proporção ' + dsSGUNIDESTO;
        widget.currentRow.DSCONVERCAO = widget.currentRow.VRFATOCONV;
        ScreenService.openSwipe(widgetTabbed);
    };
    
    this.produtoProcessadoOnChange = function(args){
    	let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado'); 
    	let widgetGrid   = templateManager.container.getWidget('gridFilhaProdutoProcessado');
    	let fieldDsconv = widgetTabbed.getField('DSCONVERCAO');
        let fieldPropQuant = widgetGrid.getField('VRPROPQUANTIDADE');
    	let dsSGUNID = widgetTabbed.currentRow.SGUNIDADE? widgetTabbed.currentRow.SGUNIDADE : '';
        let dsSGUNIDESTO = widgetTabbed.currentRow.SGUNIDADEPESTO? widgetTabbed.currentRow.SGUNIDADEPESTO : '';
        fieldDsconv.label = 'Fator Conversão ' + dsSGUNID + ' para ' + dsSGUNIDESTO;
        fieldPropQuant.label = 'Proporção ' + dsSGUNIDESTO;
        widgetTabbed.currentRow.DSCONVERCAO = widgetTabbed.currentRow.VRFATOCONV;
    };
    
    this.openCloseForm = function(){
        let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado');
        
        // solução paliativa para exibição de Swipe sem formatação
        ScreenService.openSwipe(widgetTabbed).then(opened => {
        	setTimeout(function() {
        		ScreenService.goBack();
        	}, 1000);
        });
    };
    
    this.newProdutoProcessado = function(widget){
        let widgetGridPrincipal = widget;
        widgetGridPrincipal.dataSource.newRow();
        let arrayWidgetGridPrincipal = widgetGridPrincipal.dataSource.data;
        let nextCurrentRow = arrayWidgetGridPrincipal[arrayWidgetGridPrincipal.length-1];
        widgetGridPrincipal.currentRow = nextCurrentRow;
        this.openForm(widgetGridPrincipal);
    };
    
    this.rulezToReadOnlyTabbedFields = function(){
        let widgetGridPrincipal = templateManager.container.getWidget('gridPaiProdutoProcessado');
        let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado');
        if(widgetGridPrincipal.currentRow.__is_new){
            widgetTabbed.getField('NMPRODUTO').readOnly = false;
            widgetTabbed.getField('DSPRODPROCESSA').readOnly = false;
        }else{
            widgetTabbed.getField('NMPRODUTO').readOnly = true;
            widgetTabbed.getField('DSPRODPROCESSA').readOnly = false;
        }
    };
    
    this.popNewRowOnCancel = function(args){
        let widgetGridPrincipal = templateManager.container.getWidget('gridPaiProdutoProcessado');
        let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado');
        
        if(widgetGridPrincipal.currentRow.__is_new){
            widgetGridPrincipal.dataSource.data.pop();
        }
        
        /*let widget = args.owner.widget;
        if( widget.mode == 'view' || widget.mode == 'list'){
            ScreenService.goBack();
        }else{
            General.backOrCancel(args);
        }*/
        
    };
    
    this.cancelGrid = function(args){
		var deferred = ZHPromise.defer();
		if (args.row.__is_new) {
			args.owner.widget.list(false);
			args.owner.widget.dataSource.data.pop();
			args.owner.widget.currentRow = {};
			deferred.resolve();
		} else if (args.owner.widget.mode === 'edit') {
			args.owner.widget.view();
			for (var i in args.owner.widget.backupRow) {
				args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
			}
			deferred.resolve();
			return deferred.promise;
		}else{
			deferred.resolve();
		}

		return deferred.promise;
	};
	
	this.cancelarGrid = function(args){
		let widgetFakeToEdit = templateManager.container.getWidget('wid_toEdit');
		let widgetFilha = templateManager.container.getWidget('gridFilhaProdutoProcessado');
		let widgetGrid = args.owner.widget;
		this.cancelGrid(args).then(()=>{
			widgetFakeToEdit.view();
			widgetFilha.list();
			if(widgetGrid.getParent().currentRow.__is_new)
				widgetGrid.dataSource.data.pop();
		});
	};
	
	this.newItprodprocessa = function(args){
	    let widgetItprodprocessa = templateManager.container.getWidget('gridFilhaProdutoProcessado');
	    let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado');
	    
	    widgetItprodprocessa.newRow();
	};
	
	this.saveItprodprocessa = function(args){
	    var promFirstSave = ZHPromise.defer();
	    let widgetGridPrincipal = templateManager.container.getWidget('gridPaiProdutoProcessado');
	    let widgetItprodprocessa = templateManager.container.getWidget('gridFilhaProdutoProcessado');
	    let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado');
	    let widgetFakeToEdit = templateManager.container.getWidget('wid_toEdit');
	    widgetItprodprocessa.currentRow.CDPRODUTO = widgetTabbed.currentRow.CDPRODUTO;
	    if(widgetGridPrincipal.currentRow.__is_new || widgetGridPrincipal.currentRow.changed){
	        if(widgetGridPrincipal.isValid() && widgetItprodprocessa.isValid()){
	        	if(widgetGridPrincipal.currentRow.__is_new == undefined){
					widgetGridPrincipal.currentRow.__is_new = false;
	    		}
	            EstUtil.saveCrud(widgetGridPrincipal, false, 'list').then(saved => {
	                widgetGridPrincipal.currentRow.__is_new = false;
	                promFirstSave.resolve();
	                widgetItprodprocessa.activate();
	            }, error => {
	                promFirstSave.reject();
	            }
				);
	        }else{
	        	widgetFakeToEdit.list();
	            return;
	        }
	    }else{
	    	if(widgetTabbed.currentRow.changed){
	    		EstUtil.saveCrud(widgetTabbed, false, 'list').then(saved =>{
	        		promFirstSave.resolve();
	    		});
	    	}else{
	    		promFirstSave.resolve();
	    	}
	    }
	    if(widgetItprodprocessa.currentRow.__is_new || widgetItprodprocessa.currentRow.changed){
		    if(widgetItprodprocessa.isValid()){
		        let row = widgetItprodprocessa.currentRow;
		        row.VRPROPVALOR = row.VRPROPVALOR_inline || row.VRPROPVALOR;
		        row.VRPROPVALOR = EstUtil.parseStrToFloat(row.VRPROPVALOR);   
		        promFirstSave.then(()=>{
		            EstUtil.saveCrud(widgetItprodprocessa, false, 'list').then(saved =>{
		            	widgetItprodprocessa.currentRow = {};
		            });
		        });
		    }
	    }
	};
	
	this.unckeckOldCheckboxes = function(args){
		let widget = args.owner;
		widget.dataSource.clearCheckedRows();
	};
	
	this.saveInlineItprodprocessa = function(){
        let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado');
		let widgetItprodprocessa = templateManager.container.getWidget('gridFilhaProdutoProcessado');
		var dataSet = widgetItprodprocessa.dataSource.data;
            
        dataSet = dataSet.filter(function(row){
            return row.changed === true && (!row.__is_new || EstUtil.rowEditedInline(row));
        });
        
        dataSet = EstUtil.fixInlineEditDataSet(dataSet);
        
        //parseFloat
        dataSet.forEach(row =>{
        	row.VRPROPVALOR = EstUtil.parseStrToFloat(row.VRPROPVALOR);
        	if(row.changed_VRPROPVALOR)
        		row.changed_VRPROPVALOR = EstUtil.parseStrToFloat(row.VRPROPVALOR);
        });
        
        
        EstUtil.saveDataSetCrud(dataSet, 'itprodprocessa').then(saved =>{
        	widgetItprodprocessa.list();
        	EstUtil.cancelInlineEdit(widgetItprodprocessa);
        }, failed => {
        	ScreenService.showMessage(failed.error);
        });
        
        if(widgetTabbed.currentRow.changed)
        	widgetTabbed.saveRow();
        
        EstUtil.cancelInlineEdit(widgetItprodprocessa);
        
	};
	
	this.updateItprodGrid = function(){
		let widgetItprodprocessa = templateManager.container.getWidget('gridFilhaProdutoProcessado');
		if(widgetItprodprocessa.dataSource.data.length > 0)
			widgetItprodprocessa.reload();
	};
	
	this.enterEditionMode = function(args){
		let activeWidget = templateManager.container.getActiveWidget();
		let widgetTabbed = templateManager.container.getWidget('tabbedProdutoProcessado');
		let widgetItprodprocessa = templateManager.container.getWidget('gridFilhaProdutoProcessado');
		let widgetFakeToEdit = templateManager.container.getWidget('wid_toEdit');
		
		let onEdit = ['inline_edit', 'edit'];
		if(!onEdit.includes(activeWidget.mode) && widgetTabbed.currentRow.__is_new == false){
			widgetFakeToEdit.edit();
		}
	};
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('Est25000_produtos_processados_Controller', Est25000_produtos_processados_Controller);
});


// FILE: ./assets/js/controllers/Est33000_RequisicaoEstoque_Controller.js
var Est33000_RequisicaoEstoque_Controller = function(ScreenService, General, templateManager, EstUtil, ZhLayoutPreferences, Est33000_RequisicaoEstoque_Service, ZhFilterPreferences) {

    var OnEnterFilter = 0;
    var ParamFilialResponse = {};

    var mensagens = {
        reqEncerrada: 'Operação bloqueada. Requisição já encerrada',
        filter: 'Selecione uma filial no filtro antes de adicionar.'
    };

    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        /*this.getFilialLogada().then(function(success) {
           
           
        }.bind(this));*/
        rowsToSet.CDFILIAL = getLocalVar("CDFILIAL");
        rowsToSet.NMFILIAL = getLocalVar("NMFILIAL");

        if (getLocalVar("IDUTILDTPROC") == "S" && getLocalVar("IDUTILDTPROC")) {
            var dataProcessamento = getLocalVar("DTPROCESSA");
            console.log(dataProcessamento);
            rowsToSet.DTREQUESTO = { start: "01" + dataProcessamento.substring(2), end: dataProcessamento };
            rowsToSet.DTPREVREQU = { start: "01" + dataProcessamento.substring(2), end: dataProcessamento };

        }
        else {
            var data = moment().format('DD/MM/YYYY');
            rowsToSet.DTREQUESTO = { start: "01" + data.substring(2), end: data };
            rowsToSet.DTPREVREQU = { start: "01" + data.substring(2), end: data };
        }

        ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(function() {
            let filtro = templateManager.container.getActiveWidget();
            this.onChangeUnidade(filtro, true);
        }.bind(this));
    };

    this.getFilialLogada = function() {
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": "FILIAL_LOGADA"
        }];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.setFieldsGerais = function(widget) {
        templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
    };

    this.onChangeUnidade = function(widget, onEnter) {
        if (OnEnterFilter === 0 || widget.name == 'zhFilterPreferencesSelectConditions') {
            this.apagaValorAlmoxarifadoLocalizacao(widget, 'CDALMOXORIG', 'DSALMOXORIG');
            this.apagaValorAlmoxarifadoLocalizacao(widget, 'CDLOCALORIG', 'DSLOCALORIG');
            this.apagaValorAlmoxarifadoLocalizacao(widget, 'CDALMOXDEST', 'DSALMOXDEST');
            this.apagaValorAlmoxarifadoLocalizacao(widget, 'CDLOCALDEST', 'DSLOCALDEST');
            if (onEnter === true) {
                this.getParamFilial(widget);
            }
            OnEnterFilter++;
        }
    };

    this.apagaValorAlmoxarifadoLocalizacao = function(widget, codigo, nome) {
        widget.currentRow[codigo] = '';
        widget.currentRow[nome] = '';
    };

    this.filialControlaEstoque = function() {
        let activeWidget = templateManager.container.getActiveWidget();

        this.getFilialLogada(activeWidget.currentRow.CDFILIAL).then((success) => {
            let returnedDataset = success.dataset.paramfilial[0];
            if (returnedDataset.IDCTRLESTQ != 'S') {
                activeWidget.currentRow.CDCDFILIAL = null;
                activeWidget.currentRow.NMFILIAL = null;
                ScreenService.showMessage('Filial logada não controla estoque.');
            }
        });
    };

    this.getFilialLogada = function(filial) {

        let filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filial
                }
		    ];
        return General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter);
    };

    this.getParamFilial = function(widget) {
        var filial = widget.currentRow.CDFILIAL || 'FILIAL_LOGADA';
        Est33000_RequisicaoEstoque_Service.getParamFilial(filial).then(function(succes) {
            if (succes.dataset) {
                if (succes.dataset.paramfilial_paramvalidacoes[0]) {
                    this.setCampos(widget, succes.dataset.paramfilial_paramvalidacoes[0], filial);
                }
                else {
                    ScreenService.showMessage("Não retornaram dados de parametrização para essa filial.");
                }
            }
        }.bind(this), function(error) {
            ScreenService.showMessage(error.data.error);
        });


    };

    this.setCampos = function(widget, array, filial) {
        //Setando datas do campo Data
        var dtprocessa = array.DTPROCESSA;
        var dtinicial = moment(dtprocessa, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY');
        if (!widget.currentRow.DTREQUESTO) {
            widget.currentRow.DTREQUESTO = {
                end: dtprocessa,
                period: undefined,
                start: dtinicial
            };

        }
        if (!widget.currentRow.DTPREVREQU) {
            widget.currentRow.DTPREVREQU = {
                end: dtprocessa,
                period: undefined,
                start: dtinicial
            };
        }
        if (!widget.currentRow.DTREQUESTO.start) widget.currentRow.DTREQUESTO.start = dtinicial;
        if (!widget.currentRow.DTREQUESTO.period) widget.currentRow.DTREQUESTO.period = undefined;
        if (!widget.currentRow.DTREQUESTO.end) widget.currentRow.DTREQUESTO.end = dtprocessa;
        if (!widget.currentRow.DTPREVREQU.start) widget.currentRow.DTPREVREQU.start = dtinicial;
        if (!widget.currentRow.DTPREVREQU.period) widget.currentRow.DTPREVREQU.period = undefined;
        if (!widget.currentRow.DTPREVREQU.end) widget.currentRow.DTPREVREQU.end = dtprocessa;
        widget.currentRow.IDUTILALMOX = array.IDUTILALMOX;
        widget.currentRow.IDUTILLOTE = array.IDUTILLOTE;
        widget.currentRow.IDUTILLCESTQ = array.IDUTILLCESTQ;

        ParamFilialResponse.IDUTILALMOX = array.IDUTILALMOX;
        ParamFilialResponse.IDUTILLOTE = array.IDUTILLOTE;
        ParamFilialResponse.IDUTILLCESTQ = array.IDUTILLCESTQ;

        if (filial === 'FILIAL_LOGADA') {
            widget.currentRow.CDFILIAL = array.CDFILIAL;
            widget.currentRow.NMFILIAL = array.NMFILIAL;
        }
        this.controlaUtilizacaoAlmoxarifadoLocalizacao(templateManager.container.getWidget('gridRequisicaoEstoque'), widget.currentRow.IDUTILALMOX, widget.currentRow.IDUTILLCESTQ);
        this.fixDataSourceFilter(widget);
    };

    this.controlaUtilizacaoAlmoxarifadoLocalizacao = function(widget, controlaAlmoxarifado, controlaLocal) {
        var utilizaAlmoxarifado = controlaAlmoxarifado || 'N';
        var utilizaLocalizacao = controlaLocal || 'N';
        if (utilizaAlmoxarifado == 'S') {
            if (utilizaLocalizacao == 'S') {
                widget.getField('DSALMOXORIG').readOnly = false;
                widget.getField('DSALMOXDEST').readOnly = false;

                widget.getField('DSLOCALORIG').readOnly = false;
                widget.getField('DSLOCALDEST').readOnly = false;

                if (widget.name == "zhFilterPreferencesSelectConditions") {

                    widget.getField('DSALMOXORIG').readOnly = false;
                    widget.getField('DSALMOXDEST').readOnly = false;

                    widget.getField('DSLOCALORIG').readOnly = false;
                    widget.getField('DSLOCALDEST').readOnly = false;
                }

            }
            else {
                widget.getField('DSALMOXORIG').readOnly = false;
                widget.getField('DSALMOXDEST').readOnly = false;

                widget.getField('DSLOCALORIG').readOnly = true;
                widget.getField('DSLOCALDEST').readOnly = true;

                if (widget.name == "zhFilterPreferencesSelectConditions") {

                    widget.getField('DSALMOXORIG').readOnly = false;
                    widget.getField('DSALMOXDEST').readOnly = false;

                    widget.getField('DSLOCALORIG').readOnly = true;
                    widget.getField('DSLOCALDEST').readOnly = true;

                }
            }
        }
        else {
            widget.getField('DSALMOXORIG').readOnly = true;
            widget.getField('DSALMOXDEST').readOnly = true;

            widget.getField('DSLOCALORIG').readOnly = true;
            widget.getField('DSLOCALDEST').readOnly = true;

            if (widget.name == "zhFilterPreferencesSelectConditions") {
                widget.getField('DSALMOXORIG').readOnly = true;
                widget.getField('DSALMOXDEST').readOnly = true;

                widget.getField('DSLOCALORIG').readOnly = true;
                widget.getField('DSLOCALDEST').readOnly = true;
            }
        }
    };

    //realiza controle para não permirir almoxarifado e localização  de origem igual ao de destino
    this.controlaValoresDestinoOrigem = function(field, tipo, campoAtual, campoDestino) {
        const widget = field.widget;
        let filtro = widget.currentRow;

        if (tipo == 'Almoxarifado') {
            if (field.value()) {
                // se a unidade controla localização, o almoxarifado de origem pode ser igual ao de destino
                if (filtro.IDUTILLCESTQ == 'N') {
                    if (widget.currentRow[campoAtual] == widget.currentRow[campoDestino]) {
                        ScreenService.showMessage('Almoxarifado de origem igual ao Almoxarifado de destino.');
                        field.value('');
                        widget.currentRow[campoAtual] = '';
                    }
                }
            }
        }
        else if ('Localização') {
            if (field.value()) {
                if (widget.currentRow[campoAtual] == widget.currentRow[campoDestino]) {
                    ScreenService.showMessage('Localização de origem igual a Localização de destino.');
                    field.value('');
                    widget.currentRow[campoAtual] = '';
                }
            }
        }
    };

    this.recarregarDataField = function(field) {
        field.reload().then(
            function(Success) {
                if (field.dataSource.data.length == 0) {
                    self.mensagemDataVazio(field.name);
                }
            },
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    self.mensagemDataVazio = function(nomeField) {
        switch (nomeField) {
            case 'DSALMOXORIG':
                /* falls through */
            case 'DSALMOXDEST':
                ScreenService.showMessage('Não há almoxarifado cadastrado(a) para a unidade informada.');
                break;
            case 'DSLOCALORIG':
                /* falls through */
            case 'DSLOCALDEST':
                ScreenService.showMessage('Não há localização cadastrado(a) para o almoxarifado informado.');
                break;
        }
    };

    this.preparaParametosQueryRequisicaoEstoque = function(widget) {
        var widgetPrincipal = widget,
            dataSourceFilter = widgetPrincipal.dataSourceFilter,
            currentRowPopup = widgetPrincipal.appliedFilter.FILTER_ROW;
        var filtroSetor = [];
        if (currentRowPopup.CDSETOR) {
            filtroSetor = [{
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": currentRowPopup.CDSETOR
                },
                {
                    "name": "P_TCDSETOR",
                    "operator": "=",
                    "value": ""
                }
            ];
        }
        else {
            filtroSetor = [{
                    "name": "CDSETOR",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_TCDSETOR",
                    "operator": "=",
                    "value": "T"
                }
            ];
        }

        var filtro = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRowPopup.CDFILIAL
            },
            {
                "name": "P_DTREQUINI",
                "operator": "=",
                "value": currentRowPopup.DTREQUESTO != null ? currentRowPopup.DTREQUESTO.start : null
            },
            {
                "name": "P_DTREQUFIN",
                "operator": "=",
                "value": currentRowPopup.DTREQUESTO != null ? currentRowPopup.DTREQUESTO.end : null
            },
            {
                "name": "P_DTPREVINI",
                "operator": "=",
                "value": currentRowPopup.DTPREVREQU != null ? currentRowPopup.DTREQUESTO.start : null
            },
            {
                "name": "P_DTPREVFIN",
                "operator": "=",
                "value": currentRowPopup.DTPREVREQU != null ? currentRowPopup.DTREQUESTO.end : null
            },
            {
                "name": "CDALMOXDEST",
                "operator": "=",
                "value": currentRowPopup.CDALMOXDEST || ''
            },
            {
                "name": "CDALMOXORIG",
                "operator": "=",
                "value": currentRowPopup.CDALMOXORIG || ''
            },
            {
                "name": "CDLOCALDEST",
                "operator": "=",
                "value": currentRowPopup.CDLOCALDEST || ''
            },
            {
                "name": "CDLOCALORIG",
                "operator": "=",
                "value": currentRowPopup.CDLOCALORIG || ''
            },
            {
                "name": "IDSTATUSREQ",
                "operator": "=",
                "value": currentRowPopup.IDSTATUSREQ || 'T'
            }
        ];

        filtro.push(filtroSetor[0]);
        filtro.push(filtroSetor[1]);
        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    this.addRequisicaoEstoque = function(widget) {
        //var gridProduto = templateManager.container.getWidget('gridProdutosRequisicao');
        //gridProduto.newRow();
        if (EstUtil.onValidateFilterRequirement(widget, true)) return;
        var Data = new Date();
        var dataDia;
        var dataMonth;
        if (Data.getDate() < 10) dataDia = '0' + Data.getDate();
        else dataDia = Data.getDate();
        if (Data.getMonth() <= 8) dataMonth = '0' + (Data.getMonth() + 1);
        else dataMonth = Data.getMonth() + 1;
        var dataAtual = dataDia + '/' + dataMonth + '/' + Data.getFullYear();
        widget.currentRow.DTREQUESTO = dataAtual;
        widget.currentRow.DTPREVREQU = dataAtual;
        widget.widgets[0].getField('NRLOTEESTQ').readOnly = true;
        this.controlaUtilizacaoAlmoxarifadoLocalizacao(widget, widget.appliedFilter.FILTER_ROW.IDUTILALMOX, widget.appliedFilter.FILTER_ROW.IDUTILLCESTQ);
        this.controlaStatusWidgets(widget, widget.name, false);
        this.validaDatas('DTREQUESTO', 'DTPREVREQU', 'LIMPAR');
        this.valoresMinMaxData(widget, 'DTREQUESTO', 'DTPREVREQU', 'LIMPAR');
        this.fixDataSourceFilter(widget);
    };

    this.cancelarRequisicaoEstoque = function(args) {
        var gridProdutos = templateManager.container.getWidget('gridProdutosRequisicao');
        this.controlaStatusWidgets(args.owner.widget, 'gridRequisicaoEstoque', true);
        General.backOrCancel(args);
        if (gridProdutos.mode == 'edit') {
            gridProdutos.list();
            gridProdutos.getField('NRLOTEESTQ').readOnly = false;
            var gridMensagemProduto = templateManager.container.getWidget('gridMensagemPorduto');
            gridMensagemProduto.enabled = true;
        }
    };

    this.cancelarProdutosRequisicao = function(args) {
        var widget = args.owner.widget;
        var gridRequisicao = templateManager.container.getWidget('gridRequisicaoEstoque');
        General.backOrCancel(args);
        this.controlaStatusWidgets(widget, 'gridProdutosRequisicao', true);
        if (gridRequisicao.mode == 'edit') {
            gridRequisicao.list(true);
            gridRequisicao.reload();
            gridRequisicao.currentRow = {};

            widget.list(true);
            widget.reload();
            widget.currentRow = {};
            ScreenService.closeSwipe(0);

        }
    };

    this.recarrecarWidgetsFilhas = function(widget) {
        widget.widgets.forEach(function(widg) {
            widg.reload();
        });
    };

    this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status) {
        mainWidget.widgets.forEach(function(widg) {
            if (nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if (mainWidget.name !== nomeWidgetAtual)
            mainWidget.enabled = status;
    };

    this.salvarRequisicaoEstoque = function(widget) {
        //caso esteja cadastrando a requisição , faz o procedimento para adicionar um produto , pois não se pode cadastrar a requsição sem produtos.
        var gridProdutos = templateManager.container.getWidget('gridProdutosRequisicao');
        if (widget.currentRow.__is_new) {
            if (widget.isValid()) {
                widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
                gridProdutos.enabled = true;
                gridProdutos.activate().then(function() {
                    //gridProdutos.newRow();
                    var gridMensagemProduto = templateManager.container.getWidget('gridMensagemPorduto');
                    gridMensagemProduto.enabled = false;
                });
                ScreenService.showMessage('É necessário cadastrar ao menos um produto para salvar a requisição.');
            }
        }
        else {
            //caso esteja apenas editando algum dado da requisição
            General.saveCrud(widget).then(
                function(Sucess) {
                    this.controlaStatusWidgets(widget, widget.name, true);
                    gridProdutos.getField('NRLOTEESTQ').readOnly = false;
                }.bind(this),
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }
    };

    this.salvarProdutosRequisicao = function(widget) {
        var gridRequisicao = templateManager.container.getWidget('gridRequisicaoEstoque');
        var gridMensagemProduto = templateManager.container.getWidget('gridMensagemPorduto');
        if (widget.currentRow.__is_new) {
            widget.currentRow.CDFILIAL = gridRequisicao.currentRow.CDFILIAL;
            widget.currentRow.NRREQUESTO = gridRequisicao.currentRow.NRREQUESTO;
        }

        if (gridRequisicao.mode == 'edit') {
            //caso esteja adicionado a requisição e o produto
            if (widget.isValid()) {
                widget.currentRow.NRSEQITEM = '';
                var rows = {
                    "requisicao": gridRequisicao.currentRow,
                    "produtos": widget.currentRow
                };
                ScreenService.showLoader();
                Est33000_RequisicaoEstoque_Service.salvarRequisicaoProdutos(rows).then(
                    function(Success) {
                        gridRequisicao.view();
                        gridRequisicao.currentRow.IDSTATUSREQ = Success.dataset.requesto.idstatusreq;
                        gridRequisicao.currentRow.IDREQUESTO = Success.dataset.requesto.idrequesto;
                        gridRequisicao.currentRow.NRREQUESTO = Success.dataset.requesto.nrrequesto;
                        gridRequisicao.currentRow.NRORG = Success.dataset.requesto.nrorg;
                        gridRequisicao.currentRow.__is_new = false;
                        gridMensagemProduto.enabled = true;
                        ScreenService.goBack();
                        widget.activate();
                        widget.list();
                        widget.reload();
                        widget.getField('NRLOTEESTQ').readOnly = false;
                        ScreenService.hideLoader();
                    }.bind(this),
                    function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                        ScreenService.hideLoader();
                    }
                );
            }
        }
        else {
            if (widget.currentRow.__is_new == true) {
                //caso esteja adicionado apenas o produto
                if (widget.isValid()) {
                    widget.currentRow.NRSEQITEM = '';
                    ScreenService.showLoader();
                    Est33000_RequisicaoEstoque_Service.salvarnovoProduto(widget.currentRow).then(
                        function(Success) {
                            gridRequisicao.currentRow.IDSTATUSREQ = Success.dataset.requesto.idstatusreq;
                            gridRequisicao.currentRow.IDREQUESTO = Success.dataset.requesto.idrequesto;
                            gridMensagemProduto.enabled = true;
                            ScreenService.goBack();
                            widget.activate();
                            widget.list();
                            widget.reload();
                            ScreenService.hideLoader();
                        }.bind(this),
                        function(callback) {
                            if (callback.data) {
                                ScreenService.showMessage(callback.data.error);
                            }
                            ScreenService.hideLoader();
                        }
                    );
                }
            }
            else {
                //caso esteja atualizando o produto
                ScreenService.showLoader();
                General.saveCrud(widget).then(
                    function(Sucess) {
                        this.controlaStatusWidgets(widget, widget.name, true);
                        ScreenService.hideLoader();
                    }.bind(this),
                    function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                        ScreenService.hideLoader();
                    }
                );
            }
        }
    };

    this.deletarProdutoRequisicao = function(widget) {
        if (widget.getParent().currentRow.IDSTATUSREQ === 'E') return ScreenService.showMessage(mensagens.reqEncerrada);
        var gridRequisicao = templateManager.container.getWidget('gridRequisicaoEstoque');
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
            Est33000_RequisicaoEstoque_Service.deletarProduto(widget.currentRow).then(
                function(Success) {
                    gridRequisicao.currentRow.IDSTATUSREQ = Success.dataset.requesto.idstatusreq;
                    gridRequisicao.currentRow.IDREQUESTO = Success.dataset.requesto.idrequesto;
                    ScreenService.goBack();
                    widget.activate();
                    widget.reload();
                }.bind(this),
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        });
    };

    this.addProductRequisition = function(widget) {
        if (widget.getParent().currentRow.IDSTATUSREQ === 'E') return ScreenService.showMessage(mensagens.reqEncerrada);
        this.controlaStatusWidgets(widget, 'gridProdutosRequisicao', false);
        widget.newRow();
    };

    this.editProductRequisition = function(widget) {
        if (widget.getParent().currentRow.IDSTATUSREQ === 'E') return ScreenService.showMessage(mensagens.reqEncerrada);
        this.controlaStatusWidgets(widget, 'gridProdutosRequisicao', false);
        General.edit(widget);

    };

    this.editarRequisicao = function(widget) {
        if (widget.currentRow.IDSTATUSREQ === 'E') ScreenService.showMessage(mensagens.reqEncerrada);
        else {
            General.edit(widget);
            Est33000_RequisicaoEstoque_Service.getParamFilial(widget.currentRow.CDFILIAL).then((succes) => {
                var paramfilial = succes.dataset.paramfilial_paramvalidacoes[0];
                if (paramfilial.IDUTILALMOX == 'S') {

                    widget.getField('DSALMOXORIG').readOnly = false;
                    widget.getField('DSALMOXDEST').readOnly = false;

                    if (paramfilial.IDUTILLCESTQ == 'S') {
                        widget.getField('DSLOCALORIG').readOnly = false;
                        widget.getField('DSLOCALDEST').readOnly = false;
                    }
                }
                this.controlaStatusWidgets(widget, 'gridRequisicaoEstoque', false);
                this.valoresMinMaxData(widget, 'DTREQUESTO', 'DTPREVREQU', 'SETAR');
            });
        }
    };

    this.deletarRequisicao = function(widget) {
        var gridRequisicao = templateManager.container.getWidget('gridRequisicaoEstoque');
        if (gridRequisicao.currentRow.IDSTATUSREQ === 'E') ScreenService.showMessage(mensagens.reqEncerrada);
        else {
            ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
                Est33000_RequisicaoEstoque_Service.deletarRequisicao(widget.currentRow).then(
                    function(Success) {
                        ScreenService.goBack();
                        widget.activate();
                        widget.reload();
                    }.bind(this),
                    function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                    }
                );
            });
        }
    };

    this.salvarMensagemProduto = function(widget) {
        widget.saveRow().then(
            function(Sucess) {
                this.controlaStatusWidgets(templateManager.container.getWidget('gridProdutosRequisicao'), widget.name, true);
                var currentRow = widget.currentRow;
                currentRow.__is_new = false;
                widget.template = widget.templateToView;
                widget.view();
                widget.reload();
            }.bind(this),
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.editMessage = function(widget) {
        if (widget.getParent().getParent().currentRow.IDSTATUSREQ === 'E') return ScreenService.showMessage(mensagens.reqEncerrada);
        General.widgetEdit(widget);
    };

    this.widgetOnEnter = function(args) {
        var widget = args.owner;
        if (widget.dataSource.data[0]) {
            widget.currentRow = widget.dataSource.data[0];
        }
        else {
            widget.currentRow = {
                "__is_new": true,
                "CDFILIAL": widget.getParent().currentRow.CDFILIAL,
                "NRREQUESTO": widget.getParent().currentRow.NRREQUESTO,
                "CDPRODUTO": widget.getParent().currentRow.CDPRODUTO,
                "NRSEQITEM": widget.getParent().currentRow.NRSEQITEM
            };
        }
    };

    this.validaDatas = function(field, nomeFieldOutraData, verificadorMinMax) {
        var widget = field.widget;
        if (verificadorMinMax == 'MIN') {
            widget.getField(nomeFieldOutraData).minDate = widget.currentRow[field.name];
        }
        else if (verificadorMinMax == 'MAX') {
            widget.getField(nomeFieldOutraData).maxDate = widget.currentRow[field.name];
        }
    };

    this.valoresMinMaxData = function(widget, nomeDataMIN, nomeDataMAX, verificadorLimpaSetta) {
        if (verificadorLimpaSetta == 'LIMPAR') {
            widget.getField(nomeDataMIN).maxDate = null;
            widget.getField(nomeDataMAX).minDate = null;
        }
        else if (verificadorLimpaSetta == 'SETAR') {
            widget.getField(nomeDataMIN).maxDate = widget.currentRow[nomeDataMAX];
            widget.getField(nomeDataMAX).minDate = widget.currentRow[nomeDataMIN];
        }
    };

    this.verficaProdutoControlaEstoque = function(widget) {
        if (widget.currentRow.CDARVPROD) {
            if (widget.currentRow.IDPRODATIV == 'A') {
                if (widget.currentRow.IDUTCTRLLOTE && templateManager.container.getWidget('gridRequisicaoEstoque').appliedFilter.FILTER_ROW.IDUTILLOTE) {
                    widget.currentRow.NRLOTEESTQ = '';
                    widget.getField('NRLOTEESTQ').readOnly = false;
                }
                if (widget.currentRow.IDCNTRESTOQ == 'N') {
                    widget.currentRow.CDARVPROD = null;
                    widget.currentRow.NMPRODUTO = null;
                    widget.currentRow.SGUNIDADE = null;
                    ScreenService.showMessage('Produto não controla estoque.');
                }
            }
            else {
                widget.currentRow.CDARVPROD = null;
                widget.currentRow.NMPRODUTO = null;
                widget.currentRow.SGUNIDADE = null;
                ScreenService.showMessage('O produto encontra-se inativo para a filial.');
            }
        }
    };

    this.controlaLoteProduto = function(field) {
        var widget = field.widget;
        field.reload().then(
            function(Success) {
                if (ParamFilialResponse.IDUTILLOTE == 'S') {
                    if (widget.currentRow.CDARVPROD) {
                        if (widget.currentRow.IDUTCTRLLOTE == 'S') {
                            if (field.dataSource.data.length == 0) {
                                ScreenService.showMessage('Lote não cadastrado para o produto informado.');
                                field.dataSource.data = null;
                            }
                        }
                        else {
                            ScreenService.showMessage('Produto não controla lote.');
                            field.dataSource.data = null;
                        }
                    }
                    else {
                        ScreenService.showMessage('Lote não cadastrado para o produto informado.');
                        field.dataSource.data = null;
                    }
                }
            },
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.apagaValoresLote = function(widget) {
        widget.currentRow.NRLOTEESTQ = null;
        widget.currentRow.NRSUBLOTE = null;
    };

    this.fixDataSourceFilter = function(widget) {
        if (widget.currentRow.CDFILIAL || widget.appliedFilter.FILTER_ROW.CDFILIAL) {
            var filial = widget.currentRow.CDFILIAL || widget.appliedFilter.FILTER_ROW.CDFILIAL;

            widget.getField('DSALMOXORIG').dataSourceFilter.forEach(function(fil) {
                if (fil.name == 'CDFILIAL') {
                    fil.value = filial;
                }
            });

            widget.getField('DSALMOXDEST').dataSourceFilter.forEach(function(fil) {
                if (fil.name == 'CDFILIAL') {
                    fil.value = filial;
                }
            });

            widget.getField('DSLOCALORIG').dataSourceFilter.forEach(function(fil) {
                if (fil.name == 'CDFILIAL') {
                    fil.value = filial;
                }
            });

            widget.getField('DSLOCALDEST').dataSourceFilter.forEach(function(fil) {
                if (fil.name == 'CDFILIAL') {
                    fil.value = filial;
                }
            });

            templateManager.container.getWidget('gridProdutosRequisicao').getField('NMPRODUTO').dataSourceFilter.forEach(function(fil) {
                if (fil.name == 'CDFILIAL') {
                    fil.value = filial;
                }
            });

            widget.getField('DSALMOXORIG').reload();
            widget.getField('DSALMOXDEST').reload();
            widget.getField('DSLOCALORIG').reload();
            widget.getField('DSLOCALDEST').reload();
            templateManager.container.getWidget('gridProdutosRequisicao').getField('NMPRODUTO').reload();

        }

    };

    this.reportProcess = function(args, afterOpenReport = false) {
        const widget = args.owner;
        const { CDFILIAL, NRREQUESTO, DSREQUESTO, DTREQUESTO } = widget.getParent().currentRow;

        if (afterOpenReport) {
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
        else {

            let filter = [
                {
                    "label": "Filial",
                    "value": CDFILIAL
                },
                {
                    "label": "Requisição",
                    "value": NRREQUESTO
                },
                {
                    "label": "Data Requisição",
                    "value": DTREQUESTO.substring(0, 10)
                },
                {
                    "label": "Descrição",
                    "value": DSREQUESTO
                }
            ];

            args.e.metadata.filter = filter;
            args.e.metadata.title = "Emissão Requisição de Estoque";

            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est33000_RequisicaoEstoque_Controller', Est33000_RequisicaoEstoque_Controller);
});


// FILE: ./assets/js/controllers/Est51000_ParametrosImportacaoEstoque_Controller.js
var Est51000_ParametrosImportacaoEstoque_Controller = function(General, MessageService, ScreenService, templateManager, Est51000_ParametrosImportacaoEstoque_Service) {
    var mensagens = {
        unidadeNaoControlaEstoque: 'A unidade selcionada não controla estoque.',
        editErro: 'Selecione uma filial antes de editar.'
    };

    this.validaFilial = function(widget) {
        if (widget.currentRow.NMFILIAL) {
            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                }
            ];
            General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter).then(function(success) {
                var paramfilial = success.dataset.paramfilial[0];
                if (paramfilial.IDCTRLESTQ === 'N') {
                    widget.currentRow.CDFILIAL = '';
                    widget.currentRow.NMFILIAL = '';
                    ScreenService.showMessage(mensagens.unidadeNaoControlaEstoque);
                }
                else {
                    this.changeReadOnly(templateManager.container.getWidget('widgetParams'), paramfilial);
                }
            }.bind(this));

        }
        else templateManager.container.getWidget('widgetParams').reload();
    };

    this.changeReadOnly = function(widget, paramfilial) {
        widget.getField('QTTAMALMOX').readOnly = this.getReadOnlyByParam(paramfilial.IDUTILALMOX);
        widget.getField('QTTAMLOCAL').readOnly = this.getReadOnlyByParam(paramfilial.IDUTILLCESTQ);
        widget.getField('QTTAMLOTE').readOnly = this.getReadOnlyByParam(paramfilial.IDUTILLOTE);
        widget.getField('QTTAMSUBLOTE').readOnly = this.getReadOnlyByParam(paramfilial.IDUTILLOTE);

        if (widget.getField('QTTAMLOTE').readOnly)
            widget.getField('QTTAMLOTE').validations = {};
        else
            widget.getField('QTTAMLOTE').validations = { "max": { "value": "20" } };

        if (widget.getField('QTTAMSUBLOTE').readOnly)
            widget.getField('QTTAMSUBLOTE').validations = {};
        else
            widget.getField('QTTAMSUBLOTE').validations = { "max": { "value": "4" } };

        widget.reload();
    };

    this.getReadOnlyByParam = function(param) {
        if (param === 'S') return false;
        else return true;
    };

    this.setReadOnlyCodBarra = function(field) {
        if (field.value() === 'B') {
            templateManager.container.getWidget('widgetParams').getField('QTTAMCDBARRA').readOnly = false;
        }
        else {
            templateManager.container.getWidget('widgetParams').getField('QTTAMCDBARRA').readOnly = true;
            templateManager.container.getWidget('widgetParams').currentRow.QTTAMCDBARRA = '';
        }
    };

    this.setFormCurrentRow = function(widget) {
        if (widget.dataSource.data[0] === undefined && widget.getParent().currentRow.CDFILIAL) {
            widget.currentRow = {
                CDFILPARIMPEST: widget.getParent().currentRow.CDFILIAL,
                IDTIPOPRODPAR: 'C',
                IDCONSQTDEST: 'N',
                IDFILIPADIMPEST: 'N',
                IDTPSEPARADOR: '2',
                __is_new: true
            };
            General.saveCrud(widget).then(function(success) {
                widget.reload();
            });
        }
        else if (!widget.getParent().currentRow.CDFILIAL) {
            widget.currentRow = { OBSERVACAO: ScreenService.i18n('Observação: ao gerar um arquivo para ser importado, os campos deverão obedecer ao layout definido, o que inclui o tamanho do campo aqui parametrizado. Caso o registro possua tamanho inferior ao definido, este deverá ser ajustado com zeros à esquerda no caso de quantidade ou com espaços à esquerda para os demais campos, até atingir o tamanho estipulado.') };
        }
        else {
            widget.currentRow = widget.dataSource.data[0];
            this.setObservacao(widget);
            widget.currentRow.__is_new = false;
        }
    };

    this.setObservacao = function(widget) {
        widget.currentRow.OBSERVACAO = ScreenService.i18n('Observação: ao gerar um arquivo para ser importado, os campos deverão obedecer ao layout definido, o que inclui o tamanho do campo aqui parametrizado. Caso o registro possua tamanho inferior ao definido, este deverá ser ajustado com zeros à esquerda no caso de quantidade ou com espaços à esquerda para os demais campos, até atingir o tamanho estipulado.');
    };

    this.validaQtMaxima = function(widget) {
        if (widget.currentRow.QTNUMEROINT > 10 || widget.currentRow.QTNUMERODEC > 10) {
            ScreenService.showMessage('A Faixa permitida é de 1 a 10.');
            widget.currentRow.QTNUMEROINT = widget.backupRow.QTNUMEROINT;
            widget.currentRow.QTNUMERODEC = widget.backupRow.QTNUMERODEC;
        }
    };

    this.edit = function(widget) {
        if (!widget.currentRow.CDFILPARIMPEST) ScreenService.showMessage(mensagens.editErro);
        else General.widgetEdit(widget);
    };

    this.save = function(widget) {
        Est51000_ParametrosImportacaoEstoque_Service.checkDataBeforeSave(widget).then(success => {
            General.saveCrud(widget).then(success => {
                widget.template = 'widget/form-view.html';
                widget.list();
                widget.reload();
            });
        }).catch(error => {
            let param = { CDFILIAL: error[0].CDFILPARIMPEST };
            MessageService.showMessageByCode("ERROR_UNIDADE_PADRAO", param);
        });
    };

    this.cancel = function(args) {
        General.backOrCancel(args).then(function(success) {
            var widget = args.owner.widget;
            widget.template = 'widget/form-view.html';
            widget.list();
            widget.reload();
        });

    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est51000_ParametrosImportacaoEstoque_Controller', Est51000_ParametrosImportacaoEstoque_Controller);
});


// FILE: ./assets/js/controllers/Est53000_Parametros_de_Estoque_Controller.js
var Est53000_Parametros_de_Estoque_Controller = function(est53000_parametros_de_estoque_service, ZhFilterPreferences, General, ScreenService, templateManager, ZHPromise, EstUtil) {
    var self = this;
    var once = 0;

    /* **************************************************
     * Função que cria um backup dos dados da aba de Dias
     * do Inventário Rotativo, marcando-os como "changed"
     * para que sejam re-inseridos caso necessária a
     * restauração do backup.
     *
     * @param Widget widget
     *
     * @return Promise
     */
    self.backupRealizationDaysData = function(widget) {
        //reloads widget to get all data
        return widget.reload().then(function() {
            //checks whether there should be any data in widget
            if (!widget.enabled || widget.dataSource.data.length < 1)
                //creates invalid backup in case of invalid parametrization
                widget.backupDataset = false;
            else {
                //backups data
                widget.backupDataset = widget.dataSource.data;

                //sets every backup row as changed in order to correctly apply backup later
                widget.backupDataset.forEach(function(row) {
                    row.changed = true;
                });
            }

            //resolves promise with created backup
            return widget.backupDataset;
        });
    };

    /* **************************************************
     * Função que cancela a edição de dados na aba de
     * inventário rotativo, efetuando o controle de seu
     * fluxo apropriadamente e restaurando o backup dos
     * dias previamente parametrizados.
     *
     * @param Widget widget
     *
     * @return Promise
     */
    self.cancelRotativeInventoryEditing = function(widget) {
        //gets widgets
        var tabbedWdgt = widget.getParent();
        var formWidget = tabbedWdgt.widgets[0];
        var gridWidget = tabbedWdgt.widgets[1];

        //returns to form widget to avoid validation errors
        if (!formWidget.isActive())
            formWidget.activate();

        //cancels editing and returns widget to normal operation
        General.widgetCancelForm(formWidget);
        General.changeWidgetAcessibility(tabbedWdgt.getParent().widgets.concat(tabbedWdgt.getParent().getParent().widgets), true);

        //returns backup of grid widget
        return self.restoreRealizationDaysBackup(gridWidget).then(function() {
            //performs validations
            self.checkWidget(formWidget);

            //checks grid widget actions
            self.checkRealizationDaysActionsVisibility(gridWidget);
        });
    };

    /* **************************************************
     * Procedimento que muda a acessibilidade das widgets
     * recebidas e de suas actions.
     *
     * @param Array widgets
     * @param boolean enabled
     */
    self.changeSubWidgetAcessibility = function(widgets, enabled) {
        //iterates over recieved widgets
        widgets.forEach(function(widget) {
            //changes widget acessibility
            widget.enabled = enabled;
            //changes action acessibility
            widget.actions.forEach(function(action) {
                action.readOnly = !enabled;
            });
        });

        //updates template to exhibit changes
        templateManager.updateTemplate();
    };

    /* **************************************************
     * Procedimento que checa a validade de algum valor
     * percentual. Possui mensagens padrão para campos
     * específicos.
     *
     * @param Field field
     */
    self.checkAdmittedPercentage = function(field) {
        var value = field.widget.currentRow[field.name];

        //parametrized error messages
        var msgs = {
            VRPERCINVROT: "Percentual para inventário rotativo deve estar entre 0 e 100%.",
            VRPETOLEENTR: "Percentual para tolerância admitida deve estar entre 0 e 100%.",
            VRPETOLEINVROT: "Percentual para tolerância admitida deve estar entre 0 e 100%.",
            DEFAULT: "Valor percentual deve estar entre 0 e 100%."
        };

        //in case of invalid percentage
        if (value !== null && (0 > value || value > 100)) {
            //cleans invalid data
            field.widget.currentRow[field.name] = null;

            //picks error message
            var msg = (msgs[field.name]) ? msgs[field.name] : msgs.DEFAULT;

            ScreenService.showMessage(msg);
        }
    };

    /* **************************************************
     * Procedimento que checa a parametrização de
     * utilização de controle de lote e modifica a
     * disponibilidade dos campos dos grupos "Descrição
     * de Número de Lote" e "Parâmetros para Controle de
     * Lote" de acordo.
     *
     * @param Widget widget
     */
    self.checkAllotment = function(widget) {
        var check = widget.currentRow.IDUTILLOTE != 'S';
        var subFieldNames = ["DSNUMELOTE", "DSSEQULOTE", "DSORDENALOTE", "IDENTRSEQLOTE", "IDENTRAUTLOTE", "IDENTRUNILOTE", "CDGRUPPRCTRLTEM"];

        //iterates over every dependent field
        subFieldNames.forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            //in case of negative parametrization, cleans field data
            if (check) {
                if (field.fieldGroup == 'DNL' || field.fieldGroup == 'ADP') //nullifies text fields
                    widget.currentRow[fieldName] = null;
                else { //set negative parametrization for option fields
                    if (field.name === '')
                        widget.currentRow[field.valueField] = 'N';

                }

            }
        });
        self.checkAllotmentNumberDescription(widget);
    };

    /* **************************************************
     * Procedimento que checa a parametrização de número
     * de lote e modifica a disponibilidade do campo de
     * sequencial de lote de acordo.
     *
     * @param Widget widget
     */
    self.checkAllotmentNumberDescription = function(widget) {
        var subField = null;
        //checks parameter existence
        if (widget.currentRow.DSNUMELOTE) {
            //searches for specific substring that makes DSSEQULOTE field required
            var check = widget.currentRow.DSNUMELOTE.search("\\[SEQ") != -1;
            subField = widget.getField('DSSEQULOTE');

            //verifies if current condition is diferent from what was computed and executes changes as necessary
            if (check ^ subField.validations.required)
                this.switchFieldRequirement(subField);

        }
        else {
            subField = widget.getField('DSSEQULOTE');
            delete(subField.validations.required);
        }
    };

    /* **************************************************
     * Função que troca a condição "required" de um field
     * e retorna um boolean que indica se o field passou
     * ou não a sê-lo.
     *
     * @param Field field
     *
     * @return boolean
     */
    self.switchFieldRequirement = function(field) {
        if (field.validations) {
            if (field.validations.required) {
                delete field.validations.required;
                return false;
            }
            else {
                field.validations.required = true;
                return true;
            }
        }
        return false;
    };

    /* **************************************************
     * Procedimento que checa a validade do campo de
     * ajuste/implantação automática e altera a
     * disponibilidade de seu campo de seleção de acordo.
     *
     * @param Widget widget
     */
    self.checkAutomaticInventoryDeploymentGeneration = function(widget) {
        var check = widget.currentRow.IDAJUSIMPLAU != 'S';
        var subField = widget.getField('IDOBRIGAAJUINVE');

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDOBRIGAAJUINVE = "F";
    };

    /* **************************************************
     * Procedimento que valida a parametrização de
     * inventário rotativo à partir do campo "Realiza
     * Inventário por Estoque Cego", habilitando e
     * desabilitando o acesso à metade dos campos, de
     * acordo com o valor observado.
     *
     * @param Widget widget
     */
    self.BlindInventoryParametrization = function(widget) {
        let checkParamsInvRot = (
            widget.currentRow.IDINVROTEST == 'S' || widget.currentRow.IDINVEROTREQ == 'S' ||
            widget.currentRow.IDCALCRECBRUTA == 'S' || widget.currentRow.IDINVROTDIA == 'S' ||
            widget.currentRow.IDAJUSIMPLAU == 'S' || widget.currentRow.IDPERMVISPOSEST == 'S'
        );

        var field = widget.getField('IDINVESTCEGO');
        if (checkParamsInvRot) {
            field.readOnly = true;
            widget.currentRow.IDINVESTCEGO = 'N';
        }
        else {
            field.readOnly = false;
        }

    };

    self.checkBlindInventoryRealization = function(widget) {
        var check = widget.currentRow.IDINVESTCEGO == 'S';

        //default empty values
        var subFieldNames = [
            {
                "IDINVROTEST": "N",
                "IDINVEROTREQ": "N",
                "VRPETOLEINVROT": null,
                "VRPERCINVROT": null,
                "IDINVROTFMES": "S",
                "NRDIAFECHMES": null,
                "IDINVROTDIA": "N",
                "IDAJUSIMPLAU": "N",
                "IDPERMVISPOSEST": "N"
            },
            {
                "NMSERVICO": null,
                "NMTIPORETI": null,
                "IDBLOQFECINVPE": "N",
                "IDOBREALACEDIV": "N"
            }
        ];

        //iterates over first group of dependent fields
        Object.keys(subFieldNames[0]).forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            //in case of negative parametrization, cleans field data
            if (check)
                widget.currentRow[fieldName] = subFieldNames[0][fieldName];
        });

        //inverts the value of the check variable to obtain the opposite effect in the blind inventory fields
        check = !check;

        //iterates over second group of dependent fields
        Object.keys(subFieldNames[1]).forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            //in case of negative parametrization, cleans field data
            if (check)
                widget.currentRow[fieldName] = subFieldNames[1][fieldName];
        });

        //checks rotative inventory days widget
        self.checkRealizationDaysWidget(widget);

        //unable field Tolerância Admitida (%)
        widget.getField('VRPETOLEINVROT').readOnly = true;
    };

    /* **************************************************
     * Procedimento que controla a troca de unidade da
     * aba de mesmo nome. Checa a validade do contrato da
     * unidade escolhida para a tab, mostrando uma
     * mensagem de aviso caso o mesmo esteja rescindido.
     *
     * Retorna o foco para a primeira aba filha e
     * requisita os dados da unidade selecionada.
     *
     * @param Widget widget
     */
    self.checkBranch = function(widget) {
        var promiseTipoCusto = ZHPromise.defer();
        widget.promise = promiseTipoCusto;

        //gets current active subwidget
        var activeWidget = widget.container.getActiveWidget();
        //gets first subwidget
        var firstWidget = widget.widgets[0];
        //default value
        var promise = false;

        //checks if branch code value is valid
        var check = widget.currentRow.CDFILIAL !== "" &&
            widget.currentRow.CDFILIAL !== null &&
            widget.currentRow.CDFILIAL !== undefined;

        if (!check && once == 0) {
            this.getFilialLogadaOperador().then(function(success) {
                let filial = success.dataset.filial[0];
                if (filial.IDCNTRRESC == 'S') {
                    widget.currentRow.CDFILIAL = '';
                    widget.currentRow.NMFILIAL = '';
                    ScreenService.showMessage('Filial com o contrato rescindido.');
                }
                else {
                    widget.currentRow.CDFILIAL = filial.CDFILIAL;
                    widget.currentRow.NMFILIAL = filial.NMFILIAL;

                    self.replicateDataForChildren(widget);


                    once = 1;
                    check = true;

                    //checks if "Inventário Rotativo" houses the current active widget
                    if (activeWidget.name == 'paramsrotativo' || activeWidget.name == 'diasrotativo') {
                        //checks if editing is currently in progress and cancels it, receiving a promise
                        if (activeWidget.getParent().widgets[0].mode == 'edit')
                            promise = self.cancelRotativeInventoryEditing(activeWidget);
                    }
                    else if (activeWidget.mode == 'edit') {
                        //cancels other generic editing cases
                        General.widgetCancelForm(activeWidget);

                        //restore acessibility for subwidgets
                        General.changeWidgetAcessibility(widget.getParent().widgets, true);
                    }

                    //checks if a promise was received and creates one otherwise
                    if (!promise) {
                        promise = ZHPromise.defer();

                        //resolved by default
                        promise.resolve();
                    }

                    //waits for other processes to be done
                    promise.then(function() {
                        //changes widget acessibility accordingly
                        self.changeSubWidgetAcessibility(widget.widgets, check);

                        //activates first widget if necessary
                        if (activeWidget != firstWidget)
                            firstWidget.activate();

                        //gets data for "Unidade" tab and its subwidgets
                        self.getDataForUnidadeWidget(widget);
                    });
                }

            }.bind(this));
        }
        else {
            if (widget.currentRow.IDCNTRRESC == 'S') {
                widget.currentRow.CDFILIAL = '';
                widget.currentRow.NMFILIAL = '';
                ScreenService.showMessage('Filial com o contrato rescindido.');
            }
            else {
                //checks if "Inventário Rotativo" houses the current active widget
                if (activeWidget.name == 'paramsrotativo' || activeWidget.name == 'diasrotativo') {
                    //checks if editing is currently in progress and cancels it, receiving a promise
                    if (activeWidget.getParent().widgets[0].mode == 'edit')
                        promise = self.cancelRotativeInventoryEditing(activeWidget);
                }
                else if (activeWidget.mode == 'edit') {
                    //cancels other generic editing cases
                    General.widgetCancelForm(activeWidget);

                    //restore acessibility for subwidgets
                    General.changeWidgetAcessibility(widget.getParent().widgets, true);
                }

                //checks if a promise was received and creates one otherwise
                if (!promise) {
                    promise = ZHPromise.defer();

                    //resolved by default
                    promise.resolve();
                }

                //waits for other processes to be done
                promise.then(function() {
                    //changes widget acessibility accordingly
                    self.changeSubWidgetAcessibility(widget.widgets, check);

                    //activates first widget if necessary
                    if (activeWidget != firstWidget)
                        firstWidget.activate();

                    //gets data for "Unidade" tab and its subwidgets
                    self.getDataForUnidadeWidget(widget);

                });
            }
        }

    };

    /* **************************************************
     * Procedimento que atualiza o filtro do campo
     * "Serviço para Baixa de Quantidades Divergentes"
     * para corresponder à unidade selecionada da aba
     * "Unidade".
     *
     * @param Widget widget
     */
    self.checkBranchForServiceForStockReductionField = function(widget) {
        var field = widget.getField("NMSERVICO");

        //builds filter
        var filter = {};
        filter.name = "CDFILIAL";
        filter.operator = "=";
        filter.value = widget.getParent().getParent().currentRow.CDFILIAL;

        //adds filter
        field.dataSourceFilter = [];
        field.dataSourceFilter.push(filter);

        //reloads data
        field.reload();
    };

    /* **************************************************
     * Procedimento que checa a parametrização de
     * utilização de controle por almoxarifado e modifica
     * a disponibilidade do campo de utilização de
     * controle de local de estoque de acordo.
     *
     * @param Widget widget
     */
    self.checkControlUsage = function(widget) {
        var check = widget.currentRow.IDUTILALMOX != 'S';
        var subField = widget.getField('IDUTILLCESTQ');

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDUTILLCESTQ = 'N';
    };

    self.setOldControlUsageOnRow = function(widget) {
        let arrFieldsToStore = ['IDUTILALMOX', 'IDUTILLCESTQ', 'IDUTILLOTE'];
        arrFieldsToStore.forEach(field => widget.currentRow['old_' + field] = widget.currentRow[field]);
    };

    self.editCurveABC = function(widget) {
        let wCurrentRow = widget.currentRow;
        let wParentRow = widget.getParent().currentRow;
        General.widgetEdit(widget);
        //restaura valores da curva abc removidos pelo edit.
        setTimeout(call => {
            wCurrentRow.VRPECURVAA = wParentRow.VRPECURVAA;
            wCurrentRow.VRPECURVAB = wParentRow.VRPECURVAB;
            wCurrentRow.VRPECURVAC = wParentRow.VRPECURVAC;
        }, 1000);
    };

    /* **************************************************
     * Função que checa a parametrização das curvas A, B
     * e C. Exibe mensagens de erro caso alguma das
     * curvas esteja nula ou a soma total de percentuais
     * ultrapasse 100%.
     *
     * @param Widget widget
     *
     * @return boolean
     */
    self.checkCurves = function(widget) {
        var tot = 0;
        //checks A curve validity
        if (!widget.currentRow.VRPECURVAA || widget.currentRow.VRPECURVAA <= 0) {
            ScreenService.showMessage("Valor do percentual da curva A deve ser maior que zero.");

            return false;
        }
        else //adds valid A curve value to total
            tot += parseFloat(widget.currentRow.VRPECURVAA);

        //checks B curve validity
        if (!widget.currentRow.VRPECURVAB || widget.currentRow.VRPECURVAB <= 0) {
            ScreenService.showMessage("Valor do percentual da curva B deve ser maior que zero.");

            return false;
        }
        else //adds valid B curve value to total
            tot += parseFloat(widget.currentRow.VRPECURVAB);

        //checks C curve validity
        if (!widget.currentRow.VRPECURVAC || widget.currentRow.VRPECURVAC <= 0) {
            ScreenService.showMessage("Valor do percentual da curva C deve ser maior que zero.");

            return false;
        }
        else //adds valid C curve value to total
            tot += parseFloat(widget.currentRow.VRPECURVAC);

        //checks total percentage validity
        if (tot != 100) {
            ScreenService.showMessage("O somatório dos percentuais da curva ABC deve totalizar 100%");

            return false;
        }
        else
            return true;
    };

    /* **************************************************
     * Procedimento que checa a disponibilidade do campo
     * de realização diária de inventário rotativo
     * de acordo com a parametrização de estoque.
     *
     * @param Widget widget
     */
    self.checkDailyRealization = function(widget) {
        //checks relevant stock parametrization
        var check = !((widget.currentRow.IDINVESTCEGO == 'N') &&
            ((widget.currentRow.IDINVROTEST == 'S') ||
                (widget.currentRow.IDINVEROTREQ == 'S') ||
                (widget.currentRow.VRPERCINVROT > 0)));

        var subField = widget.getField('IDINVROTDIA');

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDINVROTDIA = 'N';

        //checks rotative inventory days widget
        self.checkRealizationDaysWidget(widget);
    };

    /* **************************************************
     * Procedimento que checa a validade do campo de dia
     * para fechamento do mês e altera a disponibilidade
     * do campo de realização de cálculo da receita bruta
     * de acordo.
     *
     * @param Widget widget
     */
    self.checkMonthClosureDay = function(widget) {
        var day = widget.currentRow.NRDIAFECHMES;
        var subField = widget.getField('IDCALCRECBRUTA');

        //in case of invalid date
        if (day !== null && (1 > day || day > 31)) {
            //cleans invalid data
            widget.currentRow.NRDIAFECHMES = null;
            ScreenService.showMessage("Dia para fechamento do mês deve estar entre 1 e 31.");
        }

        //check = true if date is invalid
        var check = !(day >= 1 && day <= 31);

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) { //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDCALCRECBRUTA = 'N';
            widget.getField('IDINVROTDIA').readOnly = false;
        }
        else {
            widget.getField('IDINVROTDIA').readOnly = true;
        }

    };

    /* **************************************************
     * Procedimento que checa a validade do campo de
     * calendário de dias não-úteis e altera a
     * disponibilidade dos campos do field group de dias
     * não-úteis de acordo.
     *
     * @param Widget widget
     */
    self.checkNonBusinessDayCalendarConsideration = function(widget) {
        var check = widget.currentRow.IDCONSDIAUTFEC != 'S';

        //checks every field in the widget
        widget.fields.forEach(function(field) {
            //checks every non business day field
            if (field.fieldGroup == 'DNU') {
                field.readOnly = check;

                if (check) //in case of field unavailability, sets negative parametrization
                    widget.currentRow[field.name] = 'N';
            }
        });
    };

    /* **************************************************
     * Função que checa a validade da parametrização de
     * inventário rotativo, quanto aos dias de realização
     * de inventário.
     *
     * @param Widget widget
     */
    self.checkRealizationDays = function(widget) {
        //checks if the widget is enabled and if there's no saved data in it
        if (widget.enabled && widget.dataSource.data && widget.dataSource.data.length < 1) {
            //activates invalid widget to guide user
            if (!widget.isActive())
                widget.activate();

            ScreenService.showMessage("Dia para inventário deve ser informado.");

            return false;
        }
        else //valid widget
            return true;
    };

    /* **************************************************
     * Procedimento que controla a visibilidade das
     * actions da aba de Dias do Inventário Rotativo de
     * acordo com o estado atual (visualização, inserção)
     *
     * @param Widget widget
     */
    self.checkRealizationDaysActionsVisibility = function(widget) {
        //checks if parent widget is being edited
        var status = widget.enabled && widget.getParent().widgets[0].mode === 'edit';
        var actions = ["BACK_LIST", "BACK_EDIT", "ADD", "DELETE", "SAVE"];

        //switches action visibility accordingly
        actions.forEach(function(actionName) {
            widget.getAction(actionName).isVisible = status;
        });
    };

    /* **************************************************
     * Procedimento que checa a disponibilidade da aba de
     * dias do inventário rotativo de acordo com a
     * parametrização de estoque.
     *
     * CAUTION! Esse procedimento DELETA todos os dias de
     * realização de inventário caso a widget esteja
     * desabilitada!
     *
     * @param Widget widget
     */
    self.checkRealizationDaysWidget = function(formWidget) {
        var dailyField = formWidget.getField('IDINVROTDIA');
        var gridWidget = formWidget.container.getWidget('diasrotativo');

        //checks daily field to avoid repeating extensive validation calculations
        if (dailyField.value() == 'S' || dailyField.readOnly)
            //removes all saved days in case of widget
            est53000_parametros_de_estoque_service.deleteAllRealizationDays(formWidget.currentRow).then(function() {
                //disables tab
                //gridWidget.enabled = false;
            });
        else {
            //enables tab
            gridWidget.enabled = true;

            //constructs datasource filter
            gridWidget.dataSourceFilter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": formWidget.currentRow.CDFILIAL
            }];
        }

        //checks grid widget actions
        self.checkRealizationDaysActionsVisibility(gridWidget);
    };

    /* **************************************************
     * Procedimento que checa a parametrização de
     * bloqueio de lançamento e modifica a
     * disponibilidade do campo de dia de bloqueio de
     * acordo.
     *
     * @param Widget widget
     */
    self.checkReleaseBlockingForClosedMonths = function(widget) {
        var check = widget.currentRow.IDBLQESTFMES != 'S';
        var numberField = widget.getField('NRDIABLOQEST');

        //changes field avaliability according to parametrization
        numberField.readOnly = check;

        //verifies if current condition is diferent from what was computed and executes changes as necessary
        if (!check ^ numberField.validations.required)
            this.switchFieldRequirement(numberField);

        if (check) { //in case of negative parametrization, cleans field data
            delete numberField.validations.min;
            delete numberField.validations.max;
            widget.currentRow.NRDIABLOQEST = null;
        }
        else {
            numberField.validations.min = {};
            numberField.validations.max = {};
            numberField.validations.min.limit = 1;
            numberField.validations.max.value = 31;
        }
    };

    /* **************************************************
     * Procedimento que checa as parametrizações de
     * realização de inventário rotativo. Bloqueia uma
     * opção se a outra estiver validamente marcada.
     *
     * Exibe mensagem de erro no caso da marcação
     * inválida da realização por requisição de
     * alimentação escolar.
     *
     * @param Widget widget
     */
    self.checkRotativeInventory = function(widget) {
        //checks stock position inventory realization option
        if (widget.currentRow.IDINVROTEST == 'S') {
            //blocks the other inventory realization option
            widget.currentRow.IDINVEROTREQ = 'N';

            widget.getField('IDINVEROTREQ').readOnly = true;
        }
        else //unblocks the other inventory realization option
            widget.getField('IDINVEROTREQ').readOnly = false;

        //checks school feeding requisition inventory realization option
        if (widget.currentRow.IDINVEROTREQ == 'S') {
            //checks school feeding requisition validity
            if (!widget.getParent().currentRow.CDFILCENTPME) {
                //blocks operation if it is invalid
                ScreenService.showMessage("Operação bloqueada! Unidade não associada a uma central merenda.");
                widget.currentRow.IDINVEROTREQ = 'N';
                //unblocks the other inventory realization option
                widget.getField('IDINVROTEST').readOnly = false;
            }
            else {
                //in case of validity, blocks the other inventory realization option
                widget.currentRow.IDINVROTEST = 'N';

                widget.getField('IDINVROTEST').readOnly = true;
            }
        }
        else //unblocks the other inventory realization option
            widget.getField('IDINVROTEST').readOnly = false;

        //checks whether one of the inventory realization options is enabled
        var check = widget.currentRow.IDINVROTEST == 'S' || widget.currentRow.IDINVEROTREQ == 'S';
        var subFieldNames = ["VRPERCINVROT", "IDINVROTFMES"];

        //iterates over every dependent field
        subFieldNames.forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            if (check) //in case of negative parametrization, cleans field data
                widget.currentRow[fieldName] = null;
        });
    };

    /* **************************************************
     * Procedimento que checa a disponibilidade do campo
     * "Tipo de Inventário Rotativo no Fechamento Mês" de
     * acordo com a parametrização de estoque.
     *
     * @param Widget widget
     */
    self.checkRotativeInventoryOnMonthlyClosing = function(widget) {
        //checks relevant stock parametrization
        var check = !((widget.currentRow.IDINVROTEST == 'N') &&
            (widget.currentRow.IDINVEROTREQ == 'N') &&
            (widget.currentRow.IDINVESTCEGO == 'N') &&
            (widget.currentRow.VRPERCINVROT > 0));

        var subField = widget.getField('IDINVROTFMES');

        //changes field avaliability according to parametrization
        subField.readOnly = check;

        if (check) //in case of field unavailability, sets negative parametrization
            widget.currentRow.IDINVROTFMES = 'S';
    };

    /* **************************************************
     * Procedimento que inicia o procedimento de edição
     * de uma widget à partir de seu nome.
     *
     * @param Widget widget
     */
    self.editWidget = function(widget) {
        //searches for widget editing code block
        switch (widget.name) {
            case 'paramsrotativo':
                var tabbedWdgt = widget.getParent();
                var formWidget = tabbedWdgt.widgets[0];
                var gridWidget = tabbedWdgt.widgets[1];

                //starts editing process and revokes access to sibling and superior widgets
                General.widgetEdit(formWidget);
                let widgetsFilhas = tabbedWdgt.getParent().widgets.concat(tabbedWdgt.getParent().getParent().widgets);
                widgetsFilhas = widgetsFilhas.filter(widgets => widget.name !== 'diasrotativo');

                General.changeWidgetAcessibility(widgetsFilhas, false);

                //checks grid widget actions
                self.checkRealizationDaysActionsVisibility(gridWidget);
                gridWidget.enabled = true;

                //creates grid widget data backup
                self.backupRealizationDaysData(gridWidget);

                break;
            case 'diasrotativo':
                let widgetParams = widget.container.getWidget('paramsrotativo');
                let widgetInventario = widget.container.getWidget('inventariorotativo');
                let otherWidgets = widgetInventario.getParent().widgets.concat(widgetInventario.getParent().getParent().widgets);
                otherWidgets = otherWidgets.filter(widgets => widget.name !== 'diasrotativo');

                General.changeWidgetAcessibility(otherWidgets, false);
                let widgetDias = widget.container.getWidget('diasrotativo');
                //checks grid widget actions
                widgetDias.mode = 'edit';
                var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
                //faz backup da currentRow para permitir a recuperação dos dados
                widgetParams.backupRow = Util.clone(widgetParams.currentRow, ignoredKeys);
                self.checkRealizationDaysActionsVisibility(widgetDias);

                //creates grid widget data backup
                self.backupRealizationDaysData(widgetDias);
                break;
        }
    };

    /* **************************************************
     * Procedimento que solicita os dados de
     * parametrização de estoque geral de acordo com a
     * organização logada e replica o retorno para todas
     * as subwidgets da aba "Geral".
     *
     * @param Widget widget
     */
    self.getDataForGeralWidget = function(widget) {
        //requests data for "Geral" widget
        est53000_parametros_de_estoque_service.getDataForGeralWidget().then(function(data) {
            //sets data on "Geral" widget
            widget.currentRow = data;
            var widgetLancamento = widget.container.getWidget('lancamentos');

            //replicates data for all "Geral" tab widgets
            self.replicateDataForChildren(widget);

            if (!data.NRMESANTEMOVIEST) {
                var fieldInterval = widgetLancamento.getField('DTINTERVMOVIEST');
                fieldInterval.isVisible = true;
                widgetLancamento.getField('NRMESANTEMOVIEST').isVisible = false;
                widgetLancamento.getField('TIPOGERACAO').value('DEATE');
                fieldInterval.start = data.DTMOVIESTINI;
                fieldInterval.end = data.DTMOVIESTFIN;
                if (!widgetLancamento.currentRow.DTINTERVMOVIEST) {
                    widgetLancamento.currentRow.DTINTERVMOVIEST = {};
                    widgetLancamento.currentRow.DTINTERVMOVIEST.start = data.DTMOVIESTINI;
                    widgetLancamento.currentRow.DTINTERVMOVIEST.end = data.DTMOVIESTFIN;
                }
            }
            else {
                widgetLancamento.getField('DTINTERVMOVIEST').isVisible = false;
                widgetLancamento.getField('NRMESANTEMOVIEST').isVisible = true;
                widgetLancamento.getField('TIPOGERACAO').value('MES');
                if (!widgetLancamento.currentRow.DTINTERVMOVIEST) {
                    widgetLancamento.currentRow.DTINTERVMOVIEST = {};
                    widgetLancamento.currentRow.DTINTERVMOVIEST.start = data.DTMOVIESTINI;
                    widgetLancamento.currentRow.DTINTERVMOVIEST.end = data.DTMOVIESTFIN;
                }
            }

        }, function(error) {
            if (error) {
                var errorMessage = 'Timeout'; //default value

                //if theres an appropriate error message, gets it
                if (error.data)
                    errorMessage = error.data.error;

                //shows error message
                ScreenService.showMessage(errorMessage);
            }
        });
    };

    /* **************************************************
     * Procedimento que solicita os dados do field de
     * produtos da tab de mesmo nome dentro de CurvaABC.
     *
     * @param Field field
     */
    self.getDataForProdutoField = function(field) {
        //requests data for "Produto" field
        if (field.dataSource.data.length == 0) {
            est53000_parametros_de_estoque_service.getDataForProdutoField().then(function(data) {
                //sets data on "Produto" field
                field.dataSource.data = data;
            }, function(error) {
                if (error) {
                    var errorMessage = 'Timeout'; //default value

                    //if theres an appropriate error message, gets it
                    if (error.data)
                        errorMessage = error.data.error;

                    //shows error message
                    ScreenService.showMessage(errorMessage);
                }
            });
        }
    };


    /* **************************************************
     * Procedimento que solicita os dados da tab Produto
     * de CurvaABC.
     *
     * @param Widget widget
     */
    self.getDataForProdutoWidget = function(widget) {
        //requests data for "Produto" widget
        est53000_parametros_de_estoque_service.getDataForProdutoWidget(widget.currentRow).then(function(data) {
            //sets data on "Produto" widget
            var gridProduto = templateManager.container.getWidget('produtosgrid');
            gridProduto.dataSource.data = data;
            self.controlaActionAddProduto(widget);


        }, function(error) {
            if (error) {
                var errorMessage = 'Timeout'; //default value

                //if theres an appropriate error message, gets it
                if (error.data)
                    errorMessage = error.data.error;

                //shows error message
                ScreenService.showMessage(errorMessage);
            }
        });
    };

    self.controlaActionAddProduto = function(widget) {
        var gridProdutoFiltro = templateManager.container.getWidget('produtos');
        if (gridProdutoFiltro.currentRow.IDUTILALMOX == 'S') {
            if (gridProdutoFiltro.currentRow.IDUTILLCESTQ == 'S') {
                if (gridProdutoFiltro.currentRow.CDLOCALESTOQ) {
                    self.checkCurvaABCsProductsGrid(widget, true);
                }
                else {
                    self.checkCurvaABCsProductsGrid(widget, false);
                }
            }
            else {
                if (gridProdutoFiltro.currentRow.CDALMOXARIFE) {
                    self.checkCurvaABCsProductsGrid(widget, true);
                }
                else {
                    self.checkCurvaABCsProductsGrid(widget, false);
                }
            }
        }
        else {
            self.checkCurvaABCsProductsGrid(widget, true);
        }
    };

    /* **************************************************
     * Procedimento que solicita os dados de
     * parametrização de estoque de uma unidade e replica
     * o retorno para toda a árvore de subwidgets da aba
     * "Unidade", ignorando aquelas do tipo "grid".
     *
     * @param Widget widget
     */
    self.getDataForUnidadeWidget = function(widget) {
        //requests data for "Unidade" widget
        est53000_parametros_de_estoque_service.getDataForUnidadeWidget(widget.currentRow).then(function(data) {
            //sets data on "Unidade" widget
            widget.currentRow = data;

            widget.promise.resolve();

            //replicates data for all "Unidade" tab widgets
            self.replicateDataForChildren(widget, false);
            let widgetControleEstoque = templateManager.container.getWidget('controle');
            self.setOldControlUsageOnRow(widgetControleEstoque);
        }, function(error) {
            if (error) {
                var errorMessage = 'Timeout'; //default value

                //if theres an appropriate error message, gets it
                if (error.data)
                    errorMessage = error.data.error;

                //shows error message
                ScreenService.showMessage(errorMessage);
            }
        });
    };

    /* **************************************************
     * Procedimento que replica os dados de uma widget
     * para todas a sua árvore de subwidgets, permitindo
     * que widgets do tipo "grid" sejam ignoradas.
     *
     * @param Widget  widget
     * @param boolean dontIgnoreGrids
     */
    self.replicateDataForChildren = function(widget, dontIgnoreGrids) {
        //checks parameter passage and sets default value
        if (typeof dontIgnoreGrids === 'undefined')
            dontIgnoreGrids = true;

        //checks children widget existence
        if (widget.widgets.length)
            //iterates over children widget array
            widget.widgets.forEach(function(wdg) {
                //checks if widget is to receive data
                if (dontIgnoreGrids || wdg.template.split('/').reverse()[0] != "grid.html") {
                    //cleans child widget data
                    wdg.currentRow = {};

                    //checks whether there's data on mother widget's current row
                    if (Object.keys(widget.currentRow).length)
                        //copies every column to child widget
                        Object.keys(widget.currentRow).forEach(function(key) {
                            wdg.currentRow[key] = widget.currentRow[key];
                        });

                    //checks if child widget has children and replicates data for it, keeping the same parametrization
                    if (wdg.widgets.length)
                        self.replicateDataForChildren(wdg, dontIgnoreGrids);
                }
            });
    };

    /* **************************************************
     * Função que restaura os dados da aba de Dias do
     * Inventário Rotativo de acordo com o backup gerado
     * pela função "backupRealizationDaysData".
     *
     * CAUTION! Esse procedimento deleta todos os dias
     * atualmente existentes no banco e os substitui por
     * aqueles encontrados no backup!
     *
     * @param Widget widget
     *
     * @return Promise
     */
    self.restoreRealizationDaysBackup = function(gridWidget) {
        var formWidget = gridWidget.getParent().widgets[0];

        //removes every saved day in order to have a clean environment to apply backup
        return est53000_parametros_de_estoque_service.deleteAllRealizationDays(formWidget.currentRow).then(function() {
            //checks whether there is a valid backup
            if (gridWidget.backupDataset) {
                //restores backup
                gridWidget.dataSource.data = gridWidget.backupDataset;

                //selects first row, as the saving method requires a currentRow to exist
                gridWidget.moveToFirst();
                //saves data
                gridWidget.saveRow();
            }
            else //cleans data
                gridWidget.dataSource.data = [];
        });
    };

    //procedimento de salvar os dados da tabbed unidade
    self.saveTabbedUnidade = function(widget) {
        if (widget.isValid()) {
            widget.currentRow.DSPARFILFIS = est53000_parametros_de_estoque_service.concatenaCampoParamfilial(widget.currentRow);
            ScreenService.showLoader();
            est53000_parametros_de_estoque_service.saveUnidade(widget.currentRow).then(
                function() {
                    self.reloadUnidade();
                    General.widgetCancelForm(widget);
                    General.changeWidgetAcessibility(widget.getParent().getParent().widgets, true);
                    self.controlaStatusFiltro(false);
                    ScreenService.hideLoader();
                },
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                    ScreenService.hideLoader();
                }
            );
        }
    };

    /* **************************************************
     * Procedimento que executa o processo de salvamento
     * de uma widget à partir de seu nome.
     *
     * @param Widget widget
     */
    self.saveWidget = function(widget) {
        //searches for widget saving code block
        switch (widget.name) {

            case 'inventario':
                widget.currentRow.WIDGETNAME = widget.name;
                /* falls through */
            case 'lancamentos':
                /* falls through */
            case 'grupoDeOperadores':
                /* falls through */
            case 'restricoes':
                if (widget.isValid()) {
                    ScreenService.showLoader();
                    est53000_parametros_de_estoque_service.saveGeral(widget.currentRow).then(
                        function() {
                            self.reloadGeral();
                            General.widgetCancelForm(widget);
                            General.changeWidgetAcessibility(widget.getParent().getParent().widgets, true);
                            ScreenService.hideLoader();
                        },
                        function(callback) {
                            if (callback.data) {
                                ScreenService.showMessage(callback.data.error);
                            }
                            ScreenService.hideLoader();
                        }
                    );
                }
                break;
            case 'paramsrotativo':
                if (widget.isValid()) {
                    if (widget.currentRow.IDINVROTEST == "S" && widget.currentRow.IDINVEROTREQ == "S") {
                        ScreenService.showMessage("Operação Bloqueada! Filial não associada a uma central merenda.");
                        widget.mode = 'edit';
                    }
                    else {
                        let widgetDias = templateManager.container.getWidget('diasrotativo');
                        if (widget.currentRow.IDINVROTEST == "S" && widgetDias.dataSource.data.length == 0) {
                            ScreenService.showMessage("Dia de Inventário deve ser informado");
                            widget.mode = 'edit';
                        }
                        else {
                            widget.currentRow.DSPARFILFIS = est53000_parametros_de_estoque_service.concatenaCampoParamfilial(widget.currentRow);
                            ScreenService.showLoader();
                            self.checkRotativeInventory(widget);

                            est53000_parametros_de_estoque_service.saveUnidade(widget.currentRow).then(
                                function() {
                                    self.reloadUnidade();
                                    widget.template = widget.templateToView;
                                    widget.view();
                                    General.changeWidgetAcessibility(widget.getParent().getParent().widgets, true);
                                    General.changeWidgetAcessibility(widget.getParent().getParent().getParent().widgets, true);
                                    self.controlaStatusFiltro(false);
                                    ScreenService.hideLoader();
                                },
                                function(callback) {
                                    if (callback.data) {
                                        ScreenService.showMessage(callback.data.error);
                                    }
                                    ScreenService.hideLoader();
                                }
                            );
                        }
                    }
                }
                break;

            case 'fechamento':
                /* falls through */
            case 'processamento':
                /* falls through */
            case 'atualizacao':
                /* falls through */
            case 'controle':
                /* falls through */
            case 'restlanc':
                self.saveTabbedUnidade(widget);
                break;

                //saves widget "Parâmetros" under tab "CurvaABC"
            case 'parametroscurva':
                //checks validity of each curve before saving
                if (self.checkCurves(widget)) {
                    if (widget.isValid()) {
                        ScreenService.showLoader();
                        est53000_parametros_de_estoque_service.saveParametrosCurvaAbc(widget.currentRow).then(
                            function() {
                                self.reloadUnidade();
                                General.widgetCancelForm(widget);
                                General.changeWidgetAcessibility(widget.getParent().getParent().widgets.concat(widget.getParent().getParent().getParent().widgets), true);
                                self.controlaStatusFiltro(false);
                                ScreenService.hideLoader();
                            },
                            function(callback) {
                                if (callback.data) {
                                    ScreenService.showMessage(callback.data.error);
                                }
                                ScreenService.hideLoader();
                            }
                        );
                    }
                }
                break;
                //saves widget "Dias" under tab "Inventário Rotativo"
            case 'diasrotativo':
                //checks widget mode to assert current status
                //if it is in 'edit' mode, it is trying to insert a day to the grid
                //if it is in 'list' mode, it is trying to conclude rotative inventory's parametrization
                if (widget.mode === 'edit') {
                    //checks day validity
                    if (widget.currentRow.NRDIAINVFILI < 1 || widget.currentRow.NRDIAINVFILI > 31)
                        ScreenService.showMessage("Dia para realização de inventário deve estar entre 1 e 31.");
                    else { //saves data
                        widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
                        General.widgetSave(widget);
                    }

                    //ends saving process
                    break;
                    /* falls through */ //it'll reach this point only if the widget is in 'list' mode
                }
                else if (widget.mode === 'list') {
                    let widgetParamsRot = templateManager.container.getWidget('paramsrotativo');
                    widgetParamsRot.activate();
                    self.saveWidget(widgetParamsRot);
                    break;
                }
                break;

            case 'produtosgrid':
                widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
                widget.currentRow.CDALMOXARIFE = widget.getParent().currentRow.CDALMOXARIFE;
                widget.currentRow.CDLOCALESTOQ = widget.getParent().currentRow.CDLOCALESTOQ;
                if (widget.isValid()) {
                    if (widget.currentRow.__is_new) {
                        ScreenService.showLoader();
                        est53000_parametros_de_estoque_service.validateProduct(widget.currentRow).then(function(isValid) {
                            if (isValid) {
                                self.saveProduct(widget);
                            }
                            ScreenService.hideLoader();
                        }, function(error) {
                            if (error) {
                                var errorMessage = 'Timeout'; //default value

                                //if theres an appropriate error message, gets it
                                if (error.data)
                                    errorMessage = error.data.error;

                                //shows error message
                                ScreenService.showMessage(errorMessage);
                            }
                            ScreenService.hideLoader();
                        });
                    }
                    else {
                        widget.currentRow.__is_new = false;
                        self.saveProduct(widget);
                    }
                }
                break;


                //saves widget "Inventário Rotativo"
            case 'inventariorotativo':
                //saves data
                self.saveRotativeInventory(widget);
                break;
        }
    };

    self.saveProduct = function(widget) {
        est53000_parametros_de_estoque_service.saveProduct(widget.currentRow).then(
            function() {
                if (widget.currentRow.__is_new) {
                    ScreenService.closeSwipe().then(function() {
                        self.reloadProduto(widget);
                        widget.list();
                    });
                }
                else {
                    widget.view();
                }
            },
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    /* **************************************************
     * Procedimento que lida com um defeito dito
     * "incorrigível" do framework, que faz o modo padrão
     * de abertura de uma widget do tipo form ser "list"
     * ao invés de "view".
     *
     * @param Widget widget
     */
    self.setForms = function(widget) {
        //sets form widget to view mode if it is in list mode
        if (widget.mode === 'list')
            widget.view();
    };

    ////////////////////////////////////////////////////////
    // -------------------------------------------------- //
    // -----------[UNDOCUMENTED/WIP FUNCTIONS]----------- //
    // -------------------------------------------------- //
    ////////////////////////////////////////////////////////

    /* **************************************************
     * Procedimento que efetua troca de tabs pra evitar erro de grid 
     * no inicio da pagina
     *
     */
    self.openCloseTab = function() {
        let widgetGeral = templateManager.container.getWidget('geral');
        let widgetLancamentos = templateManager.container.getWidget('lancamentos');
        let widgetUnidade = templateManager.container.getWidget('unidade');
        let widgetTabbed = templateManager.container.getWidget('tabbed');

        ScreenService.showLoader();

        setTimeout(() => {
            widgetUnidade.enabled = true;
            widgetTabbed.openTab("unidade");
            setTimeout(() => {
                widgetGeral.enabled = true;
                widgetTabbed.openTab("geral");
                setTimeout(() => {
                    widgetLancamentos.activate();
                    setTimeout(() => {
                        if (!widgetLancamentos.isActive())
                            widgetLancamentos.activate();
                        setTimeout(() => {
                            if (!widgetLancamentos.isActive()) {
                                self.openCloseTab();
                            }
                            else {
                                ScreenService.hideLoader();
                            }
                        }, 500);
                    }, 500);
                }, 1000);
            }, 2000);
        }, 200);
    };

    /* **************************************************
     * Procedimento que efetua uma checagem padrão de uma
     * widget à partir de seu nome.
     *
     * @param Widget widget
     */

    self.checkWidget = function(widget) {
        //searches for widget validation code block
        switch (widget.name) {
            case 'moedaForteTabbed':
                this.checkAutofillFields(widget);
                break;
                //checks widget "Atualização de Estoque"
            case 'atualizacao':
                self.checkReleaseBlockingForClosedMonths(widget);
                break;
                //checks widget "Controle de Estoque"
            case 'controle':
                self.checkControlUsage(widget);
                self.checkAllotment(widget);
                self.checkAllotmentNumberDescription(widget);
                break;
                //checks widget "Fechamento do Dia"
            case 'fechamento':
                self.checkNonBusinessDayCalendarConsideration(widget);
                break;
                //checks widget "Geral"
            case 'geral':
                self.getDataForGeralWidget(widget);
                break;
                //checks widget "Parâmetros" under "Inventário Rotativo"
            case 'paramsrotativo':
                self.checkBranchForServiceForStockReductionField(widget);
                self.checkRotativeInventory(widget);
                self.checkRotativeInventoryOnMonthlyClosing(widget);
                self.checkDailyRealization(widget);
                self.checkMonthClosureDay(widget);
                self.checkBlindInventoryRealization(widget);
                self.checkAutomaticInventoryDeploymentGeneration(widget);
                break;
                //checks widget "Produtos"
            case 'produtos':
                widget.currentRow.CDALMOXARIFE = null;
                widget.currentRow.DSALMOXARIFE = null;

                self.checkCurvaABCsProductsFieldsFilters(widget);
                self.checkCurvaABCsProductsWarehouse(widget);
                break;
                //checks widget "Unidade"
            case 'unidade':

                self.checkBranch(widget);
                break;
        }
    };

    this.setCurrentRowUnidadeLogada = function(widget) {
        this.getFilialLogadaOperador().then(function(success) {
            let filial = success.dataset.filial[0];

            widget.currentRow.CDFILIAL = filial.CDFILIAL;
            widget.currentRow.NMFILIAL = filial.NMFILIAL;

            self.replicateDataForChildren(widget);

        }.bind(this));
    };

    this.getFilialLogadaOperador = function() {
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": "FILIAL_LOGADA"
        }];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    self.checkCurvaABCsProductsFieldsFilters = function(tabbedWdgt) {
        var CDFILIAL = tabbedWdgt.currentRow.CDFILIAL;

        tabbedWdgt.getField('CDALMOXARIFE').dataSourceFilter[0].value = CDFILIAL;
        tabbedWdgt.getField('CDLOCALESTOQ').dataSourceFilter[0].value = CDFILIAL;

        tabbedWdgt.getField('CDALMOXARIFE').reload();
        tabbedWdgt.getField('CDLOCALESTOQ').reload();

    };

    self.checkCurvaABCsProductsWarehouse = function(tabbedWdgt) {
        var check = tabbedWdgt.currentRow.IDUTILALMOX == 'S';
        var field = tabbedWdgt.getField('CDALMOXARIFE');
        var subField = tabbedWdgt.getField('CDLOCALESTOQ');

        if (check) {
            field.readOnly = false;
            check = !!(tabbedWdgt.currentRow.CDALMOXARIFE);

            if (check) {
                subField.readOnly = false;

                check = tabbedWdgt.currentRow.IDUTILLCESTQ == 'S';

                if (check) {
                    if (tabbedWdgt.currentRow.CDALMOXARIFE != subField.dataSourceFilter[1].value) {
                        subField.dataSourceFilter[1].value = tabbedWdgt.currentRow.CDALMOXARIFE;

                        tabbedWdgt.currentRow.CDLOCALESTOQ = null;
                        tabbedWdgt.currentRow.DSLOCALESTOQ = null;

                        subField.reload();

                        self.clearCurvaABCsProductsGrid(tabbedWdgt);
                    }
                    else
                        self.checkCurvaABCsProducts(tabbedWdgt);
                }
                else {
                    subField.readOnly = true;

                    tabbedWdgt.currentRow.CDLOCALESTOQ = ' ';
                    tabbedWdgt.currentRow.DSLOCALESTOQ = ' ';

                    self.checkCurvaABCsProducts(tabbedWdgt);
                }
            }
            else {
                subField.readOnly = true;

                tabbedWdgt.currentRow.CDLOCALESTOQ = null;
                tabbedWdgt.currentRow.DSLOCALESTOQ = null;

                self.clearCurvaABCsProductsGrid(tabbedWdgt);
            }
        }
        else {
            field.readOnly = true;

            subField.readOnly = true;
            tabbedWdgt.widgets[0].dataSource.data = null;
            tabbedWdgt.widgets[0].getAction('ADD').readOnly = false;

            tabbedWdgt.currentRow.CDALMOXARIFE = ' ';
            tabbedWdgt.currentRow.DSALMOXARIFE = ' ';

            tabbedWdgt.currentRow.CDLOCALESTOQ = ' ';
            tabbedWdgt.currentRow.DSLOCALESTOQ = ' ';

            var gridWidget = tabbedWdgt.widgets[0];

            gridWidget.currentRow.CDFILIAL = gridWidget.getParent().currentRow.CDFILIAL;
            gridWidget.currentRow.CDALMOXARIFE = gridWidget.getParent().currentRow.CDALMOXARIFE;
            gridWidget.currentRow.CDLOCALESTOQ = gridWidget.getParent().currentRow.CDLOCALESTOQ;

            self.getDataForProdutoWidget(gridWidget);

        }
    };

    self.checkCurvaABCsProducts = function(tabbedWdgt) {
        var gridWidget = tabbedWdgt.widgets[0];
        var check = (tabbedWdgt.currentRow.IDUTILALMOX == 'S' &&
                tabbedWdgt.currentRow.CDALMOXARIFE) &&
            (tabbedWdgt.currentRow.IDUTILLCESTQ == 'S' ||
                tabbedWdgt.currentRow.CDLOCALESTOQ);

        if (check) {
            gridWidget.currentRow.CDFILIAL = gridWidget.getParent().currentRow.CDFILIAL;
            gridWidget.currentRow.CDALMOXARIFE = gridWidget.getParent().currentRow.CDALMOXARIFE;
            gridWidget.currentRow.CDLOCALESTOQ = gridWidget.getParent().currentRow.CDLOCALESTOQ;

            self.getDataForProdutoWidget(gridWidget);
        }
    };

    self.clearCurvaABCsProductsGrid = function(tabbedWdgt) {
        var gridWidget = tabbedWdgt.widgets[0];

        gridWidget.dataSource.data = [];
        self.checkCurvaABCsProductsGrid(gridWidget, false);
    };

    self.checkCurvaABCsProductsGrid = function(widget, enabled) {
        widget.actions.forEach(function(action) {
            action.readOnly = !enabled;
        });

        widget.reload();
        templateManager.updateTemplate();
    };

    self.saveRotativeInventory = function(tabbedWdgt) {
        // var formWidget = tabbedWdgt.widgets[0];
        var gridWidget = tabbedWdgt.widgets[1];

        if (self.checkRealizationDays(gridWidget)) {
            ScreenService.showMessage("I SHOULD SAVE \"" + tabbedWdgt.label + "\"'S DATA");
            // General.widgetSave(widget);
            // self.paradaquemudaasactionsdagridwidget
            // General.changeWidgetAcessibility(widget.getParent().getParent().widgets.concat(widget.getParent().getParent().getParent().widgets), true);
        }
    };

    //recarregar dados da tabbed Geral
    self.reloadGeral = function() {
        var widget = templateManager.container.getWidget('geral');
        self.getDataForGeralWidget(widget);
    };

    //recarregar dados da tabbed Unidade
    self.reloadUnidade = function() {
        var widget = templateManager.container.getWidget('unidade');
        self.getDataForUnidadeWidget(widget);
    };

    self.reloadProduto = function() {
        var widget = templateManager.container.getWidget('produtosgrid');
        // so ira permirtir recarregar a tela caso o campo de almoxarifado esteja preenchido;
        if (widget.getParent().getField('CDALMOXARIFE').value()) {

            if (widget.getParent().getField('CDALMOXARIFE').value() !== ' ') {
                //caso a unidade utiliza localizao so irá permitir recarregar a tela caso o campo de loclaização esteja preenchido
                if (widget.getParent().currentRow.IDUTILLCESTQ == 'S') {

                    if (widget.getParent().getField('CDLOCALESTOQ').value()) {
                        if (widget.getParent().getField('CDLOCALESTOQ').value() !== ' ') {
                            self.getDataForProdutoWidget(widget.getParent());
                        }
                    }

                }
                else {
                    self.getDataForProdutoWidget(widget.getParent());
                }
            }
            else {
                self.getDataForProdutoWidget(widget.getParent());
            }
        }
    };

    //controla valicao na aba restrições nos lançamentos referentes a data
    self.verificaRestricoesDatas = function(widget, diasEntrada, diasSaida, diasAjuste, campoVerifica) {
        var diasLancoEntrada = widget.getField(diasEntrada);
        var diasLancoSaida = widget.getField(diasSaida);
        var diasLancoAjuste = widget.getField(diasAjuste);
        var verifica = widget.getField(campoVerifica);

        if (verifica.value() == 'S') {
            diasLancoSaida.readOnly = false;
            diasLancoEntrada.readOnly = false;
            diasLancoAjuste.readOnly = false;

        }
        else {
            diasLancoSaida.readOnly = true;
            diasLancoEntrada.readOnly = true;
            diasLancoAjuste.readOnly = true;
            diasLancoEntrada.value('');
            diasLancoSaida.value('');
            diasLancoAjuste.value('');
        }
    };

    self.checkValidationsDatageral = function(widget) {
        self.verificaRestricoesDatas(widget, 'NRDIASENTR', 'NRDIASSAID', 'NRDIASAJUS', 'IDLANCDTMEN');
        self.verificaRestricoesDatas(widget, 'NRDIASENTRPD', 'NRDIASSAIDPD', 'NRDIASAJUSPD', 'IDLANCDTMAI');
    };

    self.checkValidationsDataUnidade = function(widget) {
        self.verificaRestricoesDatas(widget, 'NRDIASENTRFIL', 'NRDIASSAIDFIL', 'NRDIASAJUSFIL', 'IDLANCDTMENFIL');
        self.verificaRestricoesDatas(widget, 'NRDIASENTRPDFIL', 'NRDIASSAIDPDFIL', 'NRDIASAJUSPDFIL', 'IDLANCDTMAIFIL');
    };

    self.setarValorAntigoCurvaAbc = function(widget) {
        widget.currentRow.IDCURVAABC_OLD = widget.currentRow.IDCURVAABC;
    };

    self.deleteProdutoCurvaAbc = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Excluir Registro?'), 'question', function() {
            est53000_parametros_de_estoque_service.deleteProdutoCurvaAbc(widget.currentRow).then(function() {
                ScreenService.closeSwipe().then(function() {
                    widget.activate();
                    self.reloadProduto();
                });
            }, function(error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    ScreenService.showMessage('Exclusão não permitida! O(s) registro(s) possui(em) registros filhos.');
                }
                else {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    self.recarregarField = function(nomeWidget, nomeField) {
        var widget = templateManager.container.getWidget(nomeWidget);
        widget.getField(nomeField).reload().then(
            function() {
                if (nomeField == 'CDALMOXARIFE' || nomeField == 'CDLOCALESTOQ') {
                    self.verificaCadastroAlmoxarifadoLocalizacao(widget.getField(nomeField));
                }
            },
            function() {}
        );

    };

    // apaga valor da localização toda vez que alterar o almoxaridado
    self.controlaAlmoxarifadoLocalizacao = function(widget) {
        widget.currentRow.CDLOCALESTOQ = '';
        widget.currentRow.DSLOCALESTOQ = '';
    };

    self.verificaCadastroAlmoxarifadoLocalizacao = function(field) {
        if (field.name == 'CDALMOXARIFE') {
            if (field.dataSource.data.length === 0) {
                ScreenService.showMessage('Não há almoxaridado cadastrado(a) para a unidade informada.');
            }
        }
        else if (field.name == 'CDLOCALESTOQ') {
            if (field.dataSource.data.length === 0) {
                ScreenService.showMessage('Não há localização cadastrado(a) para o almoxaridado informado.');
            }
        }
        else if (field.name == 'CDGRUPPRCTRLTEM' || field.name == 'CDGRUPPRMOVEST') {
            if (field.dataSource.data.length === 0) {
                ScreenService.showMessage('Não há agrupamento cadastrado.');
            }
        }


    };

    // 
    self.alteraRealizaInventario = function(widget) {
        //checks stock position inventory realization option
        if (widget.currentRow.IDINVROTEST == 'S') {
            //blocks the other inventory realization option
            widget.currentRow.IDINVEROTREQ = 'N';

            widget.getField('IDINVEROTREQ').readOnly = true;
        }
        else //unblocks the other inventory realization option
            widget.getField('IDINVEROTREQ').readOnly = false;

        //checks school feeding requisition inventory realization option
        if (widget.currentRow.IDINVEROTREQ == 'S') {
            //checks school feeding requisition validity
            if (!widget.getParent().currentRow.CDFILCENTPME) {
                //blocks operation if it is invalid
                ScreenService.showMessage("Operação bloqueada! Unidade não associada a uma central merenda.");

                widget.currentRow.IDINVEROTREQ = 'N';

                //unblocks the other inventory realization option
                widget.getField('IDINVROTEST').readOnly = false;
            }
            else {
                //in case of validity, blocks the other inventory realization option
                widget.currentRow.IDINVROTEST = 'N';

                widget.getField('IDINVROTEST').readOnly = true;
            }
        }
        else //unblocks the other inventory realization option
            widget.getField('IDINVROTEST').readOnly = false;

        //checks whether one of the inventory realization options is enabled
        var check = widget.currentRow.IDINVROTEST == 'S' || widget.currentRow.IDINVEROTREQ == 'S';
        var subFieldNames = ["VRPERCINVROT", "IDINVROTFMES"];

        //iterates over every dependent field
        subFieldNames.forEach(function(fieldName) {
            var field = widget.getField(fieldName);

            //changes field avaliability according to parametrization
            field.readOnly = check;

            if (check) //in case of negative parametrization, cleans field data
                widget.currentRow[fieldName] = null;
        });
    };

    //função para não permetir que se troque o filtro de unidade enquanto se edita um registro
    self.controlaStatusFiltro = function(status) {
        var widget = templateManager.container.getWidget('unidade');
        widget.getField('NMFILIAL').readOnly = status;
    };

    self.changeMoviestParameter = function(fieldMoviestType) {
        let widgetLancamento = fieldMoviestType.widget;
        let fieldInterval = widgetLancamento.getField('DTINTERVMOVIEST');
        let fieldMonthNumber = widgetLancamento.getField('NRMESANTEMOVIEST');
        if (fieldMoviestType.value() == 'MES') {
            fieldMonthNumber.isVisible = true;
            fieldInterval.isVisible = false;
            fieldInterval.start = null;
            fieldInterval.end = null;
            widgetLancamento.currentRow.DTINTERVMOVIEST.start = null;
            widgetLancamento.currentRow.DTINTERVMOVIEST.end = null;
            templateManager.updateTemplate();

        }
        else {
            fieldMonthNumber.isVisible = false;
            fieldInterval.isVisible = true;
            fieldMonthNumber.value(null);
            templateManager.updateTemplate();
        }

    };

    this.validaHora = ({ owner: { field } }) => {
        const HORA = field.value() || false;
        let toTime = moment(HORA, 'hh:mm');
        if (toTime._d == 'Invalid Date') {
            ScreenService.showMessage('Hora Invalida');
            field.setValue('00:00');
        }

        if (toTime._d >= moment("08:00", "hh:mm") && toTime._d <= moment("18:00", "hh:mm")) {
            ScreenService.showMessage('Hora Inválida! A movimentação de estoque não deve ser agendada no período entre 08:00 às 18:00.');
            field.setValue('00:00');
        }
    };

    this.defineIdiomaTipoCustoGrid = function(widget) {
        // 	setLocalVar('language', 'es_es');
        var idioma = getLocalVar('language');
        var nomeField = null;

        if (idioma == 'pt_br')
            nomeField = 'NMTIPOCUSTO';
        else if (idioma == 'en_us')
            nomeField = 'NMTIPOCUSING';
        else if (idioma == 'es_es' || idioma == 'es_cl')
            nomeField = 'NMTIPOCUSESP';

        var fields = widget.fields;

        fields.forEach(function(field) {
            if (field.name == nomeField || field.name == 'CDTIPOCUSTO' || field.name == 'IDUTTPCUSTO' || field.name == 'INDICE')
                field.isVisible = true;
            else
                field.isVisible = false;
        });
    };

    this.defineIdiomaTipoCusto = function(widget) {
        // 	setLocalVar('language', 'en_us');
        var idioma = getLocalVar('language');
        var nomeField = null;
        var nomeFieldLabel = null;

        if (idioma == 'pt_br') {
            nomeField = 'NMTIPOCUSTO';
            nomeFieldLabel = 'NMTIPCUSTEST';
        }
        else if (idioma == 'en_us') {
            nomeField = 'NMTIPOCUSING';
            nomeFieldLabel = 'NMTIPOCUSING';
        }
        else if (idioma == 'es_es' || idioma == 'es_cl') {
            nomeField = 'NMTIPOCUSESP';
            nomeFieldLabel = 'NMTIPOCUSESP';
        }
        var fields = widget.getField('NMTIPCUSTEST').fields;

        fields.forEach(function(field) {
            if (field.name == nomeField || field.name == 'CDTIPOCUSTO')
                field.isVisible = true;
            else
                field.isVisible = false;
        });

        widget.getField('NMTIPCUSTEST').displayedFields[1] = nomeFieldLabel;
        widget.getField('NMTIPCUSTEST').searchableFields[1] = nomeFieldLabel;
        delete widget.getField('NMTIPCUSTEST').outData.NMTIPOCUSTO;
        widget.getField('NMTIPCUSTEST').outData[nomeFieldLabel] = "NMTIPCUSTEST";

        widget.currentRow.NMTIPCUSTEST = widget.getField('NMTIPCUSTEST').value();

        widget.getField('NMTIPCUSTEST').reload();

    };

    this.setTipoCusto = function(widget) {
        var widgetUnidade = templateManager.container.getWidget('unidade');
        widgetUnidade.promise.then(() => {
            var cdtipcustest = templateManager.container.getWidget('unidade').currentRow.CDTIPCUSTEST;

            if (cdtipcustest) {
                this.getTipoCusto(cdtipcustest).then(function(success) {
                    var idioma = getLocalVar('language');
                    var labelTipoCusto = null;

                    if (idioma == 'pt_br') {
                        labelTipoCusto = success.dataset.tipocusto[0].NMTIPOCUSTO;
                    }
                    else if (idioma == 'en_us') {
                        labelTipoCusto = success.dataset.tipocusto[0].NMTIPOCUSING;
                    }
                    else if (idioma == 'es_es' || idioma == 'es_cl') {
                        labelTipoCusto = success.dataset.tipocusto[0].NMTIPOCUSESP;
                    }

                    widget.currentRow.NMTIPCUSTEST = labelTipoCusto;

                });
            }

        });

    };

    this.getTipoCusto = function(cdtipocusto) {
        var filter = [
            {
                "name": "CDTIPOCUSTO",
                "operator": "=",
                "value": cdtipocusto
            }
        ];
        return General.callRestEngine('est#/crud/tipocusto', 'FilterData', filter);
    };

    this.indexHandler = function(widgetTabbed, clickedField) {
        const widgetMoedaForte = templateManager.container.getWidget('moedaForte');
        const firstIndexCode = widgetTabbed.currentRow.CDINDICEMF;
        const secondIndexCode = widgetTabbed.currentRow.CDINDICEMF2;

        this.validateDuplicatedIndex(widgetTabbed, clickedField, firstIndexCode, secondIndexCode);

        widgetTabbed.currentRow.__is_new = false;
        widgetTabbed.currentRow.CDPARGERAL = '0001';

        EstUtil.saveCrud(widgetTabbed).then(_ => {
            this.checkIndexNameOnGrid(widgetTabbed, widgetMoedaForte, firstIndexCode, secondIndexCode);
            widgetTabbed.list();
            widgetMoedaForte.activate();
            widgetTabbed.reload();
        });
    };

    this.validateDuplicatedIndex = function(widgetTabbed, clickedField, firstIndexCode, secondIndexCode) {
        if (firstIndexCode == secondIndexCode) {
            if (firstIndexCode != "" && secondIndexCode != "") {
                clickedField.setValue("");
                return ScreenService.showMessage('Índice já cadastrado!');
            }
        }
    };

    this.checkIndexNameOnGrid = function(widgetTabbed, widgetMoedaForte, firstIndexCode, secondIndexCode) {

        widgetMoedaForte.dataSource.data.forEach(item => {
            if (item.IDMOEDCUSTO == '1') {
                item.INDICE = firstIndexCode === "" ? "Índice não definido" : widgetTabbed.currentRow.NMINDICEMF;
            }
            else if (item.IDMOEDCUSTO == '2') {
                item.INDICE = secondIndexCode === "" ? "Índice não definido" : widgetTabbed.currentRow.NMINDICEMF2;
            }
            else {
                item.INDICE = "Índice não definido";
            }
        });
    };

    this.checkAutofillFields = function(widget) {
        const currentRow = widget.currentRow;
        const data = widget.dataSource.data[0];

        widget.fields.forEach(field => {
            if (field.name === 'NMINDICEMF') {
                this.getIndexData(data.CDINDICEMF).then((success) => {
                    currentRow.CDINDICEMF = data.CDINDICEMF;
                    currentRow.NMINDICEMF = success.dataset.indice.length != 0 ? success.dataset.indice[0].NMINDICE : "";
                });
            }
            else if (field.name === 'NMINDICEMF2') {
                this.getIndexData(data.CDINDICEMF2).then((success) => {
                    currentRow.CDINDICEMF2 = data.CDINDICEMF2;
                    currentRow.NMINDICEMF2 = success.dataset.indice.length != 0 ? success.dataset.indice[0].NMINDICE : "";
                });
            }
            field.reload();
        });
    };

    this.getIndexData = function(fieldValue) {
        var filter = [
            {
                "name": "CDTPINDICE",
                "operator": "=",
                "value": "1"
            },
            {
                "name": "IDPERVALOR",
                "operator": "=",
                "value": "2"
            },
            {
                "name": "CDINDICE",
                "operator": "=",
                "value": fieldValue
            }
        ];
        return General.callRestEngine('est#/crud/indice', 'FilterData', filter);
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est53000_Parametros_de_Estoque_Controller', Est53000_Parametros_de_Estoque_Controller);
});


// FILE: ./assets/js/controllers/Est56000_EstoqueMinimoProdutos_Controller.js
var Est56000_EstoqueMinimoProdutos_Controller = function(ScreenService, General, templateManager, ZhFilterPreferences){
    var mensagens = {
        tempoMedio: 'O tempo médio de reposição deve estar entre 0 e 999.',
        unidadeNaoControlaEstoque: 'A unidade selcionada não controla estoque.',
        unidadeNaoControlaAlmox: 'Atenção, está unidade não controla almoxarifado.'
    };
    
    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        this.getFilialLogada().then(function(success) {
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
        });
    };
    
	this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
	
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };

    
    
    this.validaFilial = function(widget) {
        if(widget.currentRow.CDFILIAL) {
            var filter = [
                {
                    "name":"CDFILIAL",
                    "operator":"=",
                    "value": widget.currentRow.CDFILIAL
                }
            ];
            General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter).then(function(success){
                if(success.dataset.paramfilial[0].IDCTRLESTQ ==='N'){
                    widget.currentRow.CDFILIAL = '';
                    widget.currentRow.NMFILIAL = '';
                    ScreenService.showMessage(mensagens.unidadeNaoControlaEstoque); 
                }
                else {
                    //Valida controle de almoxarifado.
                    if(success.dataset.paramfilial[0].IDUTILALMOX === 'N') {
                        ScreenService.showMessage(mensagens.unidadeNaoControlaAlmox); 
                        templateManager.container.getWidget('widgetProdutoAlmox').enabled = false;
                        templateManager.container.getWidget('tabbed').currentRow = {DSALMOXARIFE: '', CDALMOXARIFE: ''};
                        templateManager.container.getWidget('tabbed').getField('DSALMOXARIFE').readOnly = true;
                        templateManager.container.getWidget('widgetProdutoAlmox').reload();
                    }
                    else {
                        templateManager.container.getWidget('widgetProdutoAlmox').enabled = true;
                        templateManager.container.getWidget('tabbed').getField('DSALMOXARIFE').readOnly = false;
                        templateManager.container.getWidget('tabbed').currentRow = {DSALMOXARIFE: '', CDALMOXARIFE: ''};
                    }
                }
            });
        }
        else {
            templateManager.container.getWidget('tabbed').currentRow = {DSALMOXARIFE: '', CDALMOXARIFE: ''};
            templateManager.container.getWidget('widgetProdutoAlmox').enabled = true;
            templateManager.container.getWidget('widgetProdutoAlmox').reload();
        }
    };
    
    this.verificaTempoMedio = function(widget) {
        if(widget.currentRow.NRTEMPREPEST && 
        (widget.currentRow.NRTEMPREPEST > 999 || widget.currentRow.NRTEMPREPEST < 0))  {
            ScreenService.showMessage(mensagens.tempoMedio);
            widget.currentRow.NRTEMPREPEST = '';
        }
    };
    this.saveInlineEdit = function(widget) {
        var rows = widget.dataSource.data;
        rows = rows.filter(function(row){
            return row.changed === true && !row.__is_new;
        });
        rows.forEach(function(row) {
            var rowItem = row;
            rowItem.QTESTOQMIN = rowItem.QTESTOQMIN_inline || rowItem.QTESTOQMIN;
            rowItem.QTESTMINSAID = rowItem.QTESTMINSAID_inline || rowItem.QTESTMINSAID;
            rowItem.NRTEMPREPEST = rowItem.NRTEMPREPEST_inline || rowItem.NRTEMPREPEST;
            General.callRestEngine('est#/saveEstqMinProd', 'Row', row).then(function(success){
                widget.reload();
                
            },
            function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                        widget.reload();
                    }
                }
            );
            
        });
        widget.list();
    };
    
    this.saveInlineEditAlmox = function(widget) {
        var rows = widget.dataSource.data;
        rows = rows.filter(function(row){
            return row.changed === true && !row.__is_new;
        });
        rows.forEach(function(row) {
            var rowItem = row;
            rowItem.CDALMOXARIFE = row.CDALMOXARIFE || widget.appliedFilter.FILTER_ROW.CDALMOXARIFE;
            rowItem.QTMINPRODALMOX = rowItem.QTMINPRODALMOX_inline || rowItem.QTMINPRODALMOX;
            General.callRestEngine('est#/saveEstqMinProdAlmox', 'Row', row).then(function(success){
                widget.reload();
            },
            function(callback){
                    if(callback.data){
                        ScreenService.showMessage(callback.data.error);
                        widget.reload();
                    }
                }
            );
            
        });
        widget.list();
    };
    
    this.setFilial = function() {
        if(templateManager.container) {
            var container = templateManager.container;
            var cdfilial = container.getWidget('widgetProduto').appliedFilter.FILTER_ROW.CDFILIAL;
            container.getWidget('widgetProdutoAlmox').dataSourceFilter[0].value = cdfilial;
            container.getWidget('tabbed').getField('DSALMOXARIFE').dataSourceFilter[0].value = cdfilial;
        }
    };
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('Est56000_EstoqueMinimoProdutos_Controller', Est56000_EstoqueMinimoProdutos_Controller);
});


// FILE: ./assets/js/controllers/Est57000_TransferenciaEstoque_Controller.js
var Est57000_TransferenciaEstoque_Controller = function(ScreenService, General, templateManager, Est57000_TransferenciaEstoque_Service, Lib00206_InfComplProduto_Controller, Est58000_RetiradaEstoque_Service, ValidacoesEstoque_Service, ZHPromise, ZhFilterPreferences, MessageService, EstUtil) {
    var mensagens = {
        produtoIncompleto: 'Não é possivel salvar pois faltam informações para o produto ',
        unidadeNEstoque: 'Unidade não controla estoque.',
        unidadeNAlmox: 'Unidade não controla almoxarifado.',
        naoPossuiFiltro: "É necessario realizar o filtro primeiro"
    };

    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        var defaultFilter = widget.dataSourceFilter;
        this.getFilialLogada().then(function(success) {
            var filialLogada = success.dataset.filial[0];
            defaultFilter.CDFILIAL = filialLogada.CDFILIAL;
            defaultFilter.NMFILIAL = filialLogada.NMFILIAL;
            ZhFilterPreferences.setOriginalFilter(widget.id, defaultFilter);
            ZhFilterPreferences.openFilterWidget(widget, defaultFilter).then(function() {
                let filtro = templateManager.container.getActiveWidget();
                this.getParamFilial(filtro, true);
            }.bind(this));
        }.bind(this));
    };

    this.setFieldsGerais = function(widget) {
        templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
    };

    this.onChangeUnidade = function(widget, atualizaDataLancamento) {

        this.apagaValorAlmoxarifadoLocalizacao(widget, 'P_CDALMOXORIG', 'DSALMOXSAID');
        this.apagaValorAlmoxarifadoLocalizacao(widget, 'P_CDLOCALORIG', 'DSLOCALSAID');
        this.apagaValorAlmoxarifadoLocalizacao(widget, 'P_CDALMOXDEST', 'DSALMOXENTR');
        this.apagaValorAlmoxarifadoLocalizacao(widget, 'P_CDLOCALDEST', 'DSLOCALENTR');
        this.controlaUtilizacaoAlmoxarifadoLocalizacao(widget, widget.currentRow.IDUTILALMOX, widget.currentRow.IDUTILLCESTQ, widget.currentRow.IDCTRLESTQ);

        if (widget.currentRow.IDCTRLESTQ == 'N') {
            ScreenService.showMessage(mensagens.unidadeNEstoque);
            widget.currentRow.CDFILIAL = '';
            widget.currentRow.NMFILIAL = '';
            widget.getField('DTMOVIESTQ').value({ start: '', end: '' });
        }
        else {
            if (widget.currentRow.IDUTILALMOX == 'N') {
                ScreenService.showMessage(mensagens.unidadeNAlmox);
                widget.currentRow.CDFILIAL = '';
                widget.currentRow.NMFILIAL = '';
                widget.getField('DTMOVIESTQ').value({ start: '', end: '' });
            }
            else if (atualizaDataLancamento) {
                this.buscaDataProcessamentoFilial(widget);
            }
        }

        if (widget.currentRow.CDFILIAL) {
            widget.getField('DSALMOXENTR').dataSourceFilter.forEach(function(fil) {
                if (fil.name === 'CDFILIAL') {
                    fil.value = widget.currentRow.CDFILIAL;
                }
            });

            widget.getField('DSLOCALENTR').dataSourceFilter.forEach(function(fil) {
                if (fil.name === 'CDFILIAL') {
                    fil.value = widget.currentRow.CDFILIAL;
                }
            });

            widget.getField('DSALMOXSAID').dataSourceFilter.forEach(function(fil) {
                if (fil.name === 'CDFILIAL') {
                    fil.value = widget.currentRow.CDFILIAL;
                }
            });

            widget.getField('DSLOCALSAID').dataSourceFilter.forEach(function(fil) {
                if (fil.name === 'CDFILIAL') {
                    fil.value = widget.currentRow.CDFILIAL;
                }
            });

            widget.getField('NRREQUESTO').dataSourceFilter.forEach(function(fil) {
                if (fil.name === 'CDFILIAL') {
                    fil.value = widget.currentRow.CDFILIAL;
                }
            });

            templateManager.container.getWidget('gridProdutosTransferencia').getField('NMPRODUTO').dataSourceFilter.forEach(function(fil) {
                if (fil.name === 'CDFILIAL') {
                    fil.value = widget.currentRow.CDFILIAL;
                }
            });

            widget.getField('DSALMOXENTR').reload();
            widget.getField('DSLOCALENTR').reload();
            widget.getField('DSALMOXSAID').reload();
            widget.getField('DSLOCALSAID').reload();
            widget.getField('NRREQUESTO').reload();
            templateManager.container.getWidget('gridProdutosTransferencia').getField('NMPRODUTO').reload();
        }


    };

    this.getParamFilial = function(widget, isFilter) {
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": widget.currentRow.CDFILIAL
        }];
        General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter).then(function(success) {
            var dataset = success.dataset.paramfilial[0];
            widget.currentRow.IDCTRLESTQ = dataset.IDCTRLESTQ;
            widget.currentRow.IDUTILALMOX = dataset.IDUTILALMOX;
            widget.currentRow.IDUTILLCESTQ = dataset.IDUTILLCESTQ;
            widget.currentRow.IDUTCTRLLOTE = dataset.IDUTCTRLLOTE;
            if (isFilter === true) {
                this.onChangeUnidade(widget, true);
            }
        }.bind(this));
    };

    this.verficaProdutoControlaEstoque = function(widget) {
        if (widget.currentRow.CDARVPROD) {
            if (widget.currentRow.IDPRODATIV == 'A') {
                if (widget.currentRow.IDUTCTRLLOTE && templateManager.container.getWidget('gridTransferencia').appliedFilter.FILTER_ROW.IDUTILLOTE) {
                    widget.currentRow.NRLOTEESTQ = '';
                    widget.getField('NRLOTEESTQ').readOnly = false;
                }
                if (widget.currentRow.IDCNTRESTOQ == 'N') {
                    widget.currentRow.CDARVPROD = null;
                    widget.currentRow.NMPRODUTO = null;
                    widget.currentRow.SGUNIDADE = null;
                    ScreenService.showMessage('Produto não controla estoque.');
                }
                else {
                    if (widget.currentRow.$error) {
                        delete(widget.currentRow.$error);
                    }
                    widget.getField('QTTOTLANCTO').readOnly = true;
                    widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
                    widget.currentRow.CDFILIMOVI = widget.getParent().currentRow.CDFILIMOVI;
                    Lib00206_InfComplProduto_Controller.infComplementares(widget, widget.currentRow);
                    widget.currentRow.QTTOTLANCTO = null;
                }

            }
            else {
                widget.currentRow.CDARVPROD = null;
                widget.currentRow.NMPRODUTO = null;
                widget.currentRow.SGUNIDADE = null;
                ScreenService.showMessage('O produto encontra-se inativo para a filial.');
            }
        }
    };

    this.atualizaMaxQuantidade = function(widget) {
        widget.currentRow.IDQUANTID = 'S';
        var gridTransferencia = templateManager.container.getWidget('gridTransferencia');
        var result = 0;
        if (widget.currentRow.QTESTOQDIA) {
            result = widget.currentRow.QTESTOQDIA;
        }

        if (gridTransferencia.mode == 'edit') {
            if (widget.getParent().currentRow.NRREQUESTO) {
                if (widget.currentRow.QTREQUESTO) {
                    if (widget.currentRow.QTREQUESTO > result) {
                        widget.currentRow.QTTOTLANCTO = result;
                    }
                    else {
                        widget.currentRow.QTTOTLANCTO = widget.currentRow.QTREQUESTO;
                    }
                }
            }
        }
        var mensagem = "Posição Estoque: ".concat(result);
        widget.getField('QTTOTLANCTO').validations.max = { "value": result, "message": mensagem };

    };

    this.apagaValoresLote = function(widget) {
        widget.currentRow.NRLOTEESTQ = null;
        widget.currentRow.NRSUBLOTE = null;
    };

    this.newRowProduto = function(widget) {
        var gridTransferencia = templateManager.container.getWidget('gridTransferencia');
        if (gridTransferencia.mode == 'edit') {
            if (widget.getParent().currentRow.NRREQUESTO) {
                if (!widget.currentRow.NRREQUESTO) {
                    widget.getField('NMPRODUTO').readOnly = false;
                    widget.getField('QTTOTLANCTO').readOnly = true;
                }
            }
        }
        else {
            widget.getField('NMPRODUTO').readOnly = false;
            widget.getField('QTTOTLANCTO').readOnly = true;
        }
        widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
        widget.currentRow.CDFILIMOVI = widget.getParent().currentRow.CDFILIMOVI;
        if (widget.getParent().currentRow.CDALMOXSAID)
            widget.currentRow.CDALMOXARIFE = widget.getParent().currentRow.CDALMOXSAID;
        if (widget.getParent().currentRow.CDLCESTSAID)
            widget.currentRow.CDLOCALESTOQ = widget.getParent().currentRow.CDLCESTSAID;
    };

    this.apagaValorAlmoxarifadoLocalizacao = function(widget, codigo, nome) {
        widget.currentRow[codigo] = '';
        widget.currentRow[nome] = '';
    };

    //realiza controle para não permirir almoxarifado e localização  de origem igual ao de destino
    this.controlaValoresDestinoOrigem = function(field, tipo, campoAtual, campoDestino) {
        var widget;
        var filtro;

        if (field.widget.name == 'zhFilterPreferencesSelectConditions') {
            widget = templateManager.container.getWidget('gridTransferencia');
            filtro = field.widget.currentRow;
        }
        else {
            widget = field.widget;
            filtro = field.widget.appliedFilter.FILTER_ROW;
        }

        if (tipo == 'Almoxarifado') {
            if (field.value()) {
                // se a unidade controla localização, o almoxarifado de origem pode ser igual ao de destino
                if (filtro.IDUTILLCESTQ == 'N') {
                    if (field.widget.currentRow[campoAtual] == field.widget.currentRow[campoDestino]) {
                        ScreenService.showMessage('Almoxarifado de origem igual ao Almoxarifado de destino.');
                        field.value('');
                        field.widget.currentRow[campoAtual] = '';
                    }
                }
            }
        }
        else if ('Loacalização') {
            if (field.value()) {
                if (field.widget.currentRow[campoAtual] == field.widget.currentRow[campoDestino]) {
                    ScreenService.showMessage('Loacalização de origem igual a Localização de destino.');
                    field.value('');
                    field.widget.currentRow[campoAtual] = '';
                }
            }
        }
    };

    this.fieldIsSet = function(fieldValue) {
        if (fieldValue) return true;
        else return false;
    };

    // habilita/desabilita campos de localização de almoxarifado
    this.habilitaLocalAlmoxarifado = function(widget, campo, isFilter) {
        var row = widget.currentRow;
        var almoxarifadoIsSet = false;
        if (campo == 'DSALMOXSAID' && row.IDUTILLCESTQ == 'S') {
            almoxarifadoIsSet = this.fieldIsSet(row.DSALMOXSAID);
            widget.getField('DSLOCALSAID').readOnly = !almoxarifadoIsSet;
            widget.getField('DSLOCALSAID').readOnlyOnEdit = !almoxarifadoIsSet;
            widget.getField('DSLOCALSAID').reload();

        }
        else if (campo == 'DSALMOXENTR' && row.IDUTILLCESTQ == 'S') {
            almoxarifadoIsSet = this.fieldIsSet(row.DSALMOXENTR);
            widget.getField('DSLOCALENTR').readOnly = !almoxarifadoIsSet;
            widget.getField('DSLOCALENTR').readOnlyOnEdit = !almoxarifadoIsSet;
            widget.getField('DSLOCALENTR').reload();

        }
    };

    this.getFieldValueFilter = function(filedName, widgetName) {
        var container = templateManager.container;
        if (container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if (filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if (f) {
                    filterWidget = container.getWidget(widgetName).appliedFilter;
                }

            }

            if (filterWidget) {
                var valueField;
                if (filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[filedName];
                else
                    valueField = filterWidget.FILTER_ROW[filedName];

                return valueField || null;
            }
            else return null;
        }
        else return null;
    };

    this.controlaUtilizacaoAlmoxarifadoLocalizacao = function(widget, controlaAlmoxarifado, controlaLocal, controlaEstoque) {
        var utilizaAlmoxarifado = controlaAlmoxarifado || 'N';
        var utilizaLocalizacao = controlaLocal || 'N';
        var utilizaEstoque = controlaEstoque || 'N';

        if (utilizaEstoque == 'S') {
            if (utilizaAlmoxarifado == 'S') {
                if (utilizaLocalizacao == 'S') {
                    widget.getField('DSALMOXSAID').readOnly = false;
                    widget.getField('DSALMOXENTR').readOnly = false;

                    widget.getField('DSLOCALSAID').readOnly = false;
                    widget.getField('DSLOCALENTR').readOnly = false;
                }
                else {
                    widget.getField('DSALMOXSAID').readOnly = false;
                    widget.getField('DSALMOXENTR').readOnly = false;

                    widget.getField('DSLOCALSAID').readOnly = true;
                    widget.getField('DSLOCALENTR').readOnly = true;

                }
            }
            else {
                widget.getField('DSALMOXSAID').readOnly = true;
                widget.getField('DSALMOXENTR').readOnly = true;

                widget.getField('DSLOCALSAID').readOnly = true;
                widget.getField('DSLOCALENTR').readOnly = true;
            }
            if (widget.name == 'gridTransferencia') {
                this.controlaObrigatoriedadeAlmoxarifadoLocalizacao(widget, controlaAlmoxarifado, controlaLocal, controlaEstoque);
            }
        }
        else {
            widget.getField('DSALMOXSAID').readOnly = true;
            widget.getField('DSALMOXENTR').readOnly = true;
            widget.getField('DSLOCALSAID').readOnly = true;
            widget.getField('DSLOCALENTR').readOnly = true;
            if (widget.name == 'gridTransferencia') {
                this.controlaObrigatoriedadeAlmoxarifadoLocalizacao(widget, controlaAlmoxarifado, controlaLocal, controlaEstoque);
            }
        }
    };

    this.controlaObrigatoriedadeAlmoxarifadoLocalizacao = function(widget, controlaAlmoxarifado, controlaLocal, controlaEstoque) {
        var utilizaAlmoxarifado = controlaAlmoxarifado || 'N';
        var utilizaLocalizacao = controlaLocal || 'N';
        var utilizaEstoque = controlaEstoque || 'N';

        if (utilizaEstoque == 'S') {
            if (utilizaAlmoxarifado == 'S') {
                if (utilizaLocalizacao == 'S') {
                    widget.getField('DSALMOXSAID').validations.required = true;
                    widget.getField('DSALMOXENTR').validations.required = true;

                    widget.getField('DSLOCALSAID').validations.required = true;
                    widget.getField('DSLOCALENTR').validations.required = true;
                }
                else {
                    widget.getField('DSALMOXSAID').validations.required = true;
                    widget.getField('DSALMOXENTR').validations.required = true;

                    delete(widget.getField('DSLOCALSAID').validations.required);
                    delete(widget.getField('DSLOCALENTR').validations.required);

                }
            }
            else {
                delete(widget.getField('DSALMOXSAID').validations.required);
                delete(widget.getField('DSALMOXENTR').validations.required);

                delete(widget.getField('DSLOCALSAID').validations.required);
                delete(widget.getField('DSLOCALENTR').validations.required);

            }
        }
        else {
            delete(widget.getField('DSALMOXSAID').validations.required);
            delete(widget.getField('DSALMOXENTR').validations.required);

            delete(widget.getField('DSLOCALSAID').validations.required);
            delete(widget.getField('DSLOCALENTR').validations.required);
        }
    };

    this.recarrecarWidgetsFilhas = function(widget) {
        if (!widget.currentRow.__is_new) {
            widget.widgets.forEach(function(widg) {
                widg.reload();
            });
        }
    };

    this.recarregarDataField = function(field) {
        /*field.dataSourceFilter.forEach(function(fil){
           if(fil.name == 'CDFILIAL'){
               fil.value = field.widget.currentRow.CDFILIAL;
               field.reload();
           } 
        }); 
        */
        field.reload().then(
            function(Success) {
                if (field.dataSource.data.length == 0) {
                    self.mensagemDataVazio(field.name);
                }
            },
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.mensagemDataVazio = function(nomeField) {
        switch (nomeField) {
            case 'DSALMOXENTR':
                /* falls through */
            case 'DSALMOXSAID':
                ScreenService.showMessage('Não há almoxarifado cadastrado(a) para a unidade informada.');
                break;
            case 'DSLOCALENTR':
                /* falls through */
            case 'DSLOCALSAID':
                ScreenService.showMessage('Não há localização cadastrado(a) para o almoxarifado informado.');
                break;
        }
    };

    this.cancelTransferencia = function(args) {
        var widget = args.owner.widget;
        if (widget.actions[3].label === 'Adicionar Produtos') {
            this.changeHideInconLabelVisible(widget.actions, 'Salvar', false, 'salvar', true);
        }
        General.backOrCancel(args);
        templateManager.container.getWidget('gridTransferencia').enabled = true;
        this.controlaStatusWidgets(widget, 'gridTransferencia', true);
    };

    this.addTransferencia = function(widget) {
        if (EstUtil.onValidateFilterRequirement(widget)) return;
        this.changeHideInconLabelVisible(widget.actions, 'Adicionar Produtos', false, 'salvar', true);
        this.controlaStatusWidgets(widget, widget.name, false);
        if (widget.appliedFilter !== undefined) {
            var filtroPopup = widget.appliedFilter.FILTER_ROW;
            widget.newRow();
            widget.currentRow.CDLCESTENTR = ' ';
            widget.currentRow.CDLCESTSAID = ' ';
            widget.currentRow.CDALMOXSAID = ' ';
            widget.currentRow.CDALMOXENTR = ' ';
            this.controlaUtilizacaoAlmoxarifadoLocalizacao(widget, filtroPopup.IDUTILALMOX, filtroPopup.IDUTILLCESTQ, filtroPopup.IDCTRLESTQ);
            if (filtroPopup.CDFILIAL) {
                widget.currentRow.CDFILIMOVI = filtroPopup.CDFILIAL;
                widget.currentRow.CDFILIAL = filtroPopup.CDFILIAL;
                widget.currentRow.NMFILIAL = filtroPopup.NMFILIAL;

                Object.assign(widget.currentRow, filtroPopup);

                widget.getField('DSALMOXSAID').readOnly = !!filtroPopup.DSALMOXSAID;
                widget.getField('DSLOCALSAID').readOnly = !!filtroPopup.DSLOCALSAID;
                widget.getField('DSALMOXENTR').readOnly = !!filtroPopup.DSALMOXENTR;
                widget.getField('DSLOCALENTR').readOnly = !!filtroPopup.DSLOCALENTR;

                Est57000_TransferenciaEstoque_Service.buscaDataProcessamento(filtroPopup).then(
                    function(Success) {
                        var data = Success.dataset.dataProcessamento.DTPROCESSA;
                        widget.getField('DTMOVIESTQ').value(data);
                    },
                    function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                    }
                );
            }
        }
        else {
            ScreenService.showMessage(mensagens.naoPossuiFiltro);

        }
    };

    this.editTransferencia = function(widget) {
        this.controlaStatusWidgets(widget, widget.name, false);
        General.edit(widget);
        widget.getField('DSALMOXSAID').readOnly = true;
        widget.getField('DSALMOXENTR').readOnly = true;
        widget.getField('DSLOCALSAID').readOnly = true;
        widget.getField('DSLOCALENTR').readOnly = true;
        this.changeHideInconLabelVisible(widget.actions, "Salvar", true, "salvar", true);
    };

    this.cancelaItens = function(args) {
        var widget = args.owner.widget;
        var widgetItens = templateManager.container.getWidget('gridProdutosTransferencia');
        var gridTransferencia = templateManager.container.getWidget('gridTransferencia');
        if (gridTransferencia.mode == 'edit') {
            if (widget.getParent().currentRow.NRREQUESTO) {
                if (widget.currentRow.NRREQUESTO) {
                    args.row.__is_new = false;
                    General.backOrCancel(args);
                    args.row.__is_new = true;
                    this.changeHideInconLabelVisible(widgetItens.actions, 'Salvar', true, 'saveProdutos', true);
                }
                else {
                    widget.getField('NMPRODUTO').readOnly = true;
                    widgetItens.dataSource.data.pop();
                    ScreenService.goBack();
                }
            }
            else {
                General.backOrCancel(args);
                ScreenService.goBack();
                widget.reload();
            }
        }
        else {
            templateManager.container.getWidget('gridTransferencia').enabled = true;
            General.backOrCancel(args);
            ScreenService.goBack();
        }

    };

    this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status) {
        mainWidget.widgets.forEach(function(widg) {
            if (nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if (mainWidget.name !== nomeWidgetAtual)
            mainWidget.enabled = status;
    };

    this.buscaDataProcessamentoFilial = function(widget) {
        if (widget.currentRow.CDFILIAL) {
            Est57000_TransferenciaEstoque_Service.buscaDataProcessamento(widget.currentRow).then(
                function(Success) {
                    var data = Success.dataset.dataProcessamento.DTPROCESSA;
                    var start = moment(data, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY');
                    var end = moment(data, 'DD/MM/YYYY').format('DD/MM/YYYY');
                    widget.getField('DTMOVIESTQ').value({ start: start, end: end });
                },
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }
        else {
            widget.getField('DTMOVIESTQ').value({ start: '', end: '' });
        }
    };

    this.preparaParametosQueryTranferencia = function(widget) {
        var filtroPopupCurrentRow = widget.appliedFilter.FILTER_ROW;
        var filtro = [{
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": filtroPopupCurrentRow.CDFILIAL
            },
            {
                "name": "P_DTLANCINI",
                "operator": "=",
                "value": filtroPopupCurrentRow.DTMOVIESTQ != null ? filtroPopupCurrentRow.DTMOVIESTQ.start : ""
            },
            {
                "name": "P_DTLANCFIN",
                "operator": "=",
                "value": filtroPopupCurrentRow.DTMOVIESTQ != null ? filtroPopupCurrentRow.DTMOVIESTQ.end : ""
            },
            {
                "name": "P_CDALMOXORIG",
                "operator": "=",
                "value": filtroPopupCurrentRow.P_CDALMOXORIG
            },
            {
                "name": "P_CDALMOXDEST",
                "operator": "=",
                "value": filtroPopupCurrentRow.P_CDALMOXDEST
            },
            {
                "name": "P_CDLOCALDEST",
                "operator": "=",
                "value": filtroPopupCurrentRow.P_CDLOCALDEST
            },
            {
                "name": "P_CDLOCALORIG",
                "operator": "=",
                "value": filtroPopupCurrentRow.P_CDLOCALORIG
            }
        ];
        // caso não informe a data o parâmetro P_DATA recebe 'T'
        if (!filtroPopupCurrentRow.DTMOVIESTQ.start) {
            filtro.push({
                "name": "P_DATA",
                "operator": "=",
                "value": 'T'
            });
        }
        else {
            filtro.push({
                "name": "P_DATA",
                "operator": "=",
                "value": ''
            });
        }
        widget.dataSourceFilter = filtro;
        widget.reload();
        if (!filtroPopupCurrentRow.IDUTILALMOX) {
            this.getParamFilial(filtroPopupCurrentRow);
        }
    };

    this.setaFiltroRequisicao = function(field) {
        var filtro = field.widget.appliedFilter.FILTER_ROW;
        field.dataSourceFilter[0].value = filtro.CDFILIAL;
        field.reload();
    };

    this.salvarTransferencia = function(widget) {

        if (widget.isValid()) {
            //save para nova transferência
            const widgetItens = templateManager.container.getWidget('gridProdutosTransferencia');

            if (widget.currentRow.__is_new) {
                //caso seja selecionada alguma requisicao os produtos delas devem ser adicionado
                if (widget.currentRow.NRREQUESTO) {
                    widget.enabled = false;
                    this.getItensOfRequisicao(widgetItens, widget.currentRow.CDFILIAL, widget.currentRow.NRREQUESTO);
                    this.focusAddProduto(widgetItens);
                    this.changeHideInconLabelVisible(widgetItens.actions, 'Voltar', true, 'voltarPagina', false);
                    this.changeHideInconLabelVisible(widgetItens.actions, 'Cancelar', true, 'cancelProdRequisicao', true);
                    this.controlButtonSaveAndImportVisibility(widgetItens);
                    widgetItens.getField('NMPRODUTO').readOnly = true;
                    widgetItens.getField('QTTOTLANCTO').readOnly = true;
                }
                //ativa a aba de produtos para selecionar o prdouto;
                else {
                    widgetItens.enabled = true;
                    widgetItens.dataSource.data = [];
                    widgetItens.activate();
                    widget.openTab('gridProdutosTransferencia');
                }
            }
            //save para editar tranferencias
            else {
                General.saveCrud(widget).then(
                    function(Success) {
                        this.controlaStatusWidgets(widget, widget.name, true);
                    }.bind(this),
                    function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                    }
                );
            }
        }
        this.changeHideInconLabelVisible(widget.actions, 'Salvar', false, 'salvar', false);
    };

    this.getItensOfRequisicao = function(widgetItens, cdfilial, nrrequesto, itemData, firstTime = true) {
        let widgetCabecalho = widgetItens.getParent();
        if (!firstTime) {
            const item = itemData.shift();
            if (item !== undefined) {
                widgetItens.dataSource.data.shift();
                widgetItens.currentRow = item;
                widgetItens.currentRow.QTTOTLANCTO = item.QTREQUESTO;
                widgetItens.currentRow.IDCNTRESTOQ = 'S';
                this.setCdfilimovi(widgetItens);
                if (widgetCabecalho.currentRow.IDUTILLOTE == 'S') {
                    this.callInformacoesComplementares(widgetItens).then(_ => {
                        this.setValorUnitario(widgetItens).then(_ => {
                            widgetItens.dataSource.data.push(widgetItens.currentRow);
                            this.getItensOfRequisicao(widgetItens, cdfilial, nrrequesto, itemData, false);
                        });
                    });
                }
                else {
                    this.setValorUnitario(widgetItens).then(_ => {
                        widgetItens.dataSource.data.push(widgetItens.currentRow);
                        this.getItensOfRequisicao(widgetItens, cdfilial, nrrequesto, itemData, false);
                    });
                }
            }
        }
        else {
            const row = { CDFILIAL: cdfilial, NRREQUESTO: nrrequesto };

            Est57000_TransferenciaEstoque_Service.buscaProdutosRequisicao(row).then(itens => {
                widgetItens.dataSource.data = itens.dataset.itrequesto_itens_requisicao;
                widgetItens.dataSource.data.forEach(item => {
                    item.__is_new = true;
                    item.checkRepeat = true;
                    item.itemInserido = false;
                    item.CDALMOXARIFE = widgetItens.getParent().currentRow.CDALMOXSAID;
                    item.CDLOCALESTOQ = widgetItens.getParent().currentRow.CDLCESTSAID;
                });
                const data = Util.clone(widgetItens.dataSource.data);
                this.getItensOfRequisicao(widgetItens, cdfilial, nrrequesto, data, false);
            });
        }

    };

    this.setCdfilimovi = function(widget) {
        var newRow = widget.currentRow;
        newRow.CDFILIMOVI = EstUtil.getFieldValue('CDFILIAL', 'gridTransferencia', true);
    };

    this.focusAddProduto = function(widget) {
        this.enabledWidget(true);
        widget.call = 'insertTransferencia';
        widget.activate();
        widget.floatingControl.refreshAction = false;
    };

    this.changeHideInconLabelVisible = function(data, label, hide, name, visibility) {
        data.forEach(element => {
            if (element.name === name) {
                element.label = label;
                element.hideIcon = hide;
                element.isVisible = visibility;
            }
        });
    };

    this.enabledWidget = function(enabled) {
        const widget = templateManager.container.getWidget('gridProdutosTransferencia');
        widget.enabled = enabled;
    };

    this.callInformacoesComplementares = function(widget) {
        let promise = ZHPromise.defer();
        if (widget.currentRow.CDARVPROD) {
            widget.promise = promise;
            let widgetTransf = widget.container.getWidget('gridTransferencia');
            let filterRow = isset(() => widgetTransf.appliedFilter.FILTER_ROW) ? widgetTransf.appliedFilter.FILTER_ROW : false;
            if (isset(() => filterRow.CDALMOXARIFE)) {
                if (filterRow.CDALMOXARIFE != ' ')
                    widget.currentRow.CDALMOXARIFE = filterRow.CDALMOXARIFE;
            }
            if (isset(() => filterRow.CDLOCALESTOQ)) {
                if (filterRow.CDLOCALESTOQ != ' ')
                    widget.currentRow.CDLOCALESTOQ = filterRow.CDLOCALESTOQ;
            }
            Lib00206_InfComplProduto_Controller.infComplementares(widget, widget.currentRow);
        }
        else {
            this.changeReadOnlyByNm(widget, "QTTOTLANCTO", true);
            this.clearRowItens(widget, true);
            promise.reject();
        }
        return promise;
    };

    const isset = (accessor) => {
        try {
            // Note we're seeing if the returned value of our function is not
            // undefined
            return typeof accessor() !== 'undefined' && accessor() !== null;
        }
        catch (e) {
            // And we're able to catch the Error it would normally throw for
            // referencing a property of undefined
            return false;
        }
    };

    this.setValorUnitario = function(widget) {
        const deffer = ZHPromise.defer();
        Est57000_TransferenciaEstoque_Service.getCdtipocusto(widget.currentRow.CDFILIAL).then(function(cdtipocusto) {
            Est57000_TransferenciaEstoque_Service.setBlankLocalEstFields(widget);
            ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(
                widget.currentRow.CDFILIAL,
                widget.currentRow.CDPRODUTO,
                widget.getParent().currentRow.DTMOVIESTQ,
                cdtipocusto,
                widget.currentRow.CDALMOXARIFE,
                widget.currentRow.CDLOCALESTOQ,
                widget.currentRow.NRLOTEESTQ,
                widget.currentRow.NRSUBLOTE
            ).then(function(success) {
                const consulta = success.dataset.consultaCustoQuantidadeProduto;
                widget.currentRow.QTESTOQDIA = consulta.QTESTOQDIA;
                widget.currentRow.VRUNILANCTO = consulta.VRCUSTOPROD;
                Est57000_TransferenciaEstoque_Service.setVrTotLancto(widget.currentRow);
                widget.currentRow.COMPLETO = 'S';
                deffer.resolve();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
                deffer.reject();
            });
        }, function(error) {
            ScreenService.showMessage(error.data.error);
            deffer.reject();
        });
        return deffer;
    };

    this.cancelProdRequisicao = function(args) {
        var widget = args.owner.widget;
        this.changeHideInconLabelVisible(widget.actions, 'Voltar', true, 'voltarPagina', true);
        this.changeHideInconLabelVisible(widget.actions, 'Salvar', true, 'saveControl', false);
        this.changeHideInconLabelVisible(widget.actions, 'Cancelar', true, 'cancelProdRequisicao', false);
        templateManager.container.getWidget('gridTransferencia').enabled = true;
        templateManager.container.getWidget('gridTransferencia').reload();
        General.backOrCancel(args);
    };

    this.checkProdutosCompletos = function(data) {
        var validação = true;
        data.forEach(function(produto) {
            if (produto.COMPLETO !== 'S' && validação === true) {
                validação = false;

                // ScreenService.showMessage(mensagens.produtoIncompleto + produto.NMPRODUTO + '.');

                /* MSG NOVA */
                var variaveis = {
                    NMPRODUTO: produto.NMPRODUTO
                };

                MessageService.showMessageByCode("PRODUTOIRREGULAR", variaveis);
                /* FIM MSG NOVA */

            }
        });
        return validação;
    };

    this.backProdRequisicao = function(widget) {
        let parent = widget.getParent();
        if (parent.currentRow.__is_new)
            parent.dataSource.data.pop();

        ScreenService.goBack();

    };

    this.SaveItems = function(widgetItens, widgetTransf, dataItems) {
        if (dataItems.length != 0) {
            let ignoredKeys = ["__rightActions", "__leftActions", "__createdLocal", "__index", "$error"];
            let changedRows = [];

            dataItems.forEach(row => {
                if (row.changed == true) {
                    delete row.__rightActions;
                    changedRows.push(row);
                }
            });

            let rows = {
                "transferencia": Util.clone(widgetItens.getParent().currentRow, ignoredKeys),
                "produtos": changedRows
            };

            if (changedRows.length > 0) {
                if (changedRows[changedRows.length - 1].__is_new) widgetItens.currentRow = changedRows[changedRows.length - 1];
            }
            else {
                ScreenService.showMessage('Não há novos itens para serem salvos.');
            }

            if (widgetTransf.currentRow.__is_new) {
                if (widgetItens.currentRow.__is_new) {
                    Est57000_TransferenciaEstoque_Service.salvarNovaTransferencia(rows).then(function(Success) {

                            widgetTransf.currentRow.NRORG = Success.dataset.transferencia.nrorg;
                            widgetTransf.currentRow.NRCTRLMOVEST = Success.dataset.transferencia.nrctrlmovest;
                            widgetTransf.currentRow.NRLANESTENTR = Success.dataset.transferencia.nrlanestentr;
                            widgetTransf.currentRow.NRLANESTSAID = Success.dataset.transferencia.nrlanestsaid;
                            widgetItens.dataSourceFilter[1].value = Success.dataset.transferencia.nrlanestsaid;
                            widgetTransf.currentRow.__is_new = false;

                            widgetTransf.view();
                            widgetItens.list();

                            this.changeHideInconLabelVisible(widgetItens.actions, 'Salvar', false, 'saveControl', false);
                            this.changeHideInconLabelVisible(widgetItens.actions, 'Importar por txt', false, 'importTxt', false);

                        }.bind(this),
                        function(callback) {
                            if (callback.data) {
                                ScreenService.showMessage(callback.data.error);
                            }
                        });
                }
            }
            else {
                if (widgetItens.currentRow.__is_new) {
                    Est57000_TransferenciaEstoque_Service.salvarNovaTransferencia(rows).then((Success) => {

                        this.changeHideInconLabelVisible(widgetItens.actions, 'Salvar', false, 'saveControl', false);
                        this.changeHideInconLabelVisible(widgetItens.actions, 'Importar por txt', false, 'importTxt', false);

                        widgetItens.activate();
                        widgetItens.list();
                        widgetItens.reload();

                    }, function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                    });
                }
                else {
                    Est57000_TransferenciaEstoque_Service.editarProduto(rows).then((Success) => {
                        for (let key = 0; key < changedRows.length; key++) {
                            changedRows[key].__is_new = false;
                            changedRows[key].QTLANCTOEST = Success.dataset.produto[key].qtlanctoest;
                            changedRows[key].QTTOTLANCTO = Success.dataset.produto[key].qttotlancto;
                            changedRows[key].VRTOTLANCTO = Success.dataset.produto[key].vrtotlancto;
                        }
                        this.changeHideInconLabelVisible(widgetItens.actions, 'Salvar', false, 'saveControl', false);
                        this.changeHideInconLabelVisible(widgetItens.actions, 'Importar por txt', false, 'importTxt', false);
                        widgetItens.activate();
                        widgetItens.list();
                        widgetItens.reload();
                    }, function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                    });
                }
            }
        }

    };

    this.SaveItemsWithRequisition = function(widgetItens, widgetTransf, dataItemsFromRequisition) {
        const promise = ZHPromise.defer();
        if (dataItemsFromRequisition.length != 0) {
            var row = {
                transferencia: widgetTransf.currentRow,
                itens: dataItemsFromRequisition
            };

            if (this.checkProdutosCompletos(widgetItens.dataSource.data)) {
                Est57000_TransferenciaEstoque_Service.salvarNovaTransferenciaComRequisicao(row).then(function(Success) {

                    widgetTransf.currentRow.NRORG = Success.dataset.transferencia.nrorg;
                    widgetTransf.currentRow.NRCTRLMOVEST = Success.dataset.transferencia.nrctrlmovest;
                    widgetTransf.currentRow.NRLANESTENTR = Success.dataset.transferencia.nrlanestentr;
                    widgetTransf.currentRow.NRLANESTSAID = Success.dataset.transferencia.nrlanestsaid;

                    this.changeHideInconLabelVisible(widgetItens.actions, 'Voltar', true, 'voltarPagina', true);
                    this.changeHideInconLabelVisible(widgetItens.actions, 'Salvar', true, 'saveControl', false);
                    this.changeHideInconLabelVisible(widgetItens.actions, 'Cancelar', true, 'cancelProdRequisicao', false);

                    widgetItens.enabled = true;
                    widgetTransf.enabled = true;

                    ScreenService.closeSwipe();

                    widgetItens.list();
                    widgetTransf.list();
                    widgetTransf.activate();
                    widgetTransf.reload();
                    promise.resolve();
                }.bind(this), function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                        promise.reject();
                    }
                });
            }
        }
        else {
            promise.resolve();
        }
        return promise;
    };

    this.SaveItemsController = function(widgetItens) {
        const widgetTransf = templateManager.container.getWidget('gridTransferencia');

        let dataItems = widgetItens.dataSource.data.filter(item => {
            return (!item.hasOwnProperty('NRREQUESTO')) || (item.NRREQUESTO == null);
        });

        let dataItemsFromRequisition = widgetItens.dataSource.data.filter(item => {
            return ((item.hasOwnProperty('NRREQUESTO')) && (item.NRREQUESTO != null)) && (item.hasOwnProperty('itemInserido'));
        });

        this.SaveItemsWithRequisition(widgetItens, widgetTransf, dataItemsFromRequisition).then(_ => {
            this.SaveItems(widgetItens, widgetTransf, dataItems);
        }, _ => {});

    };

    this.checkRequisicaoComplementarCreation = function(widget) {
        var promise = ZHPromise.defer();
        var item = widget.currentRow;

        Est57000_TransferenciaEstoque_Service.verificaEstoqueProduto(widget.currentRow).then(function(success) {
            var result = 0;
            if (success.dataset.lote_infCompl_almox[0]) {
                result = success.dataset.lote_infCompl_almox[0].QTESTOQDIA;
            }
            item.QTESTOQDIA = result;
            var filter = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIMOVI
            }];

            General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter).then(function(success) {
                widget.currentRow.IDREQCOMPEST = success.dataset.paramfilial[0].IDREQCOMPEST;
                if (this.checkShouldCreateRequisicaoComplementar(item)) {
                    var msg = "Quantidade requisitada não disponível em estoque para o produto " + item.NMPRODUTO + " : " + item.QTESTOQDIA + " " + item.SGUNIDADE + ". Deseja gerar uma requisição complementar?";

                    ScreenService.confirmMessage(msg,
                        'question',
                        function() { //sim
                            item.createReqComp = true;
                            item.QTLANCREQCOMP = item.QTREQUESTO - item.QTESTOQDIA;
                            ScreenService.closeSwipe();
                            widget.currentRow.COMPLETO = 'S';
                            promise.resolve();
                        }.bind(this),
                        function() { //não
                            item.createReqComp = false;
                            ScreenService.closeSwipe();
                            widget.currentRow.COMPLETO = 'S';
                            promise.resolve();
                        }.bind(this));
                }
                else {
                    ScreenService.closeSwipe();
                    widget.currentRow.COMPLETO = 'S';
                    promise.resolve();
                }
            }.bind(this));
        }.bind(this));

        return promise;
    };

    this.checkShouldCreateRequisicaoComplementar = function(item) {

        return item.NRREQUESTO && item.IDREQCOMPEST == 'S' &&
            item.QTESTOQDIA == item.QTTOTLANCTO &&
            item.QTESTOQDIA < item.QTREQUESTO;
    };

    this.checkSave = function(widget) {

        if (widget.isValid()) {
            let row = widget.currentRow;
            if (!Est57000_TransferenciaEstoque_Service.checaItensDaRetirada(this.copyDataWidgetItens(widget.dataSource.data), row.CDARVPROD, row.CDALMOXARIFE, row.CDLOCALESTOQ, row.NRLOTEESTQ, row.NRSUBLOTE, row.itemInserido)) {
                this.checkRequisicaoComplementarCreation(widget).then(function() {
                    widget.currentRow.novo = false;
                    let data = widget.dataSource.data;
                    data[data.length - 1].itemInserido = true;
                }.bind(this));
                widget.currentRow.COMPLETO = 'S';

                this.controlButtonSaveAndImportVisibility(widget);
            }
            else {
                ScreenService.showMessage('Produto informado já está sendo incluido para essa transferência.');
            }
        }
        else {
            if (widget.currentRow.$error.QTTOTLANCTO[0]) ScreenService.showMessage('Quantidade indisponivel em estoque');
        }

    };

    this.editItens = function(widget) {

        widget.backupRow = Util.clone(widget.currentRow, ['__rightActions', '__leftActions']);
        widget.edit();

    };

    this.checkAlmoxStatus = function(args) {
        var widget = args.owner;
        var widgetTransferencia = widget.getParent();
        if (widget.currentRow.COMPLETO !== 'S' && widgetTransferencia.mode == 'edit') {
            widget.currentRow.CDFILIMOVI = widget.getParent().currentRow.CDFILIMOVI;
            this.editItens(widget);
            this.callInformacoesComplmentares(widget);
        }
    };

    this.changeReadOnlyByNm = function(widget, name, read) {
        widget.getField(name).readOnly = read;
    };

    this.callInformacoesComplmentares = function(widget) {
        this.getFilialLogada().then(function(success) {
            widget.currentRow.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            if (widget.currentRow.CDARVPROD) {
                widget.paramInfoCompl.AtuEstoq = widget.currentRow.__is_new;
                widget.currentRow.CDFILIAL = widget.getParent().currentRow.CDFILIAL;
                widget.currentRow.CDFILIMOVI = widget.getParent().currentRow.CDFILIMOVI;
                Lib00206_InfComplProduto_Controller.infComplementares(widget, widget.currentRow);

            }
            else {
                this.changeReadOnlyByNm(widget, "QTTOTLANCTO", true);
                this.clearRowItens(widget, true);
            }
        }.bind(this));

    };

    this.deletarProduto = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', (success) => {
            delete widget.currentRow.__rightActions;
            var rows = {
                "transferencia": widget.getParent().currentRow,
                "produtos": widget.currentRow
            };
            var widgetTransferencia = widget.getParent();
            if (widgetTransferencia.mode == 'edit') {
                if (widget.currentRow.NRREQUESTO) {
                    var index = widget.dataSource.data.indexOf(widget.currentRow);
                    widget.dataSource.data.splice(index, 1);
                    ScreenService.goBack();
                }
                else {
                    this.removeItemDataSource(widget);
                }
                this.controlButtonSaveAndImportVisibility(widget);
            }
            else {
                Est57000_TransferenciaEstoque_Service.deletarProduto(rows).then(
                    function(Success) {
                        ScreenService.closeSwipe().then(() => {
                            widget.activate();
                        });
                        widget.reload();
                    }, (error) => {
                        if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                            ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
                        }
                        else {
                            ScreenService.showMessage(error.data.error);
                        }
                    }
                );
            }
        });
    };

    this.removeItemDataSource = function(widget) {
        var itens = widget.dataSource.data;
        for (var i = 0; i < itens.length; i++) {
            if (itens[i].CDARVPROD === widget.currentRow.CDARVPROD &&
                itens[i].CDALMOXARIFE === widget.currentRow.CDALMOXARIFE &&
                itens[i].CDLOCALESTOQ === widget.currentRow.CDLOCALESTOQ &&
                itens[i].NRLOTEESTQ === widget.currentRow.NRLOTEESTQ &&
                itens[i].NRSUBLOTE === widget.currentRow.NRSUBLOTE
            ) {
                itens.splice(i, 1);
                i = itens.length;
                ScreenService.closeSwipe();
            }
        }
    };

    this.clearRowItens = function(widget, clearAll) {
        if (widget.currentRow.COMPLETO === 'S') {
            widget.currentRow.QTREQUESTO = "";
            widget.currentRow.QTLANCTOEST = "";
            widget.currentRow.VRTOTLANCTO = "";
            if (clearAll) {
                widget.currentRow.NMPRODUTO = "";
                widget.currentRow.CDARVPROD = "";
                widget.currentRow.SGUNIDADE = "";
            }
        }
    };

    this.editItensTransferencia = function(widget) {
        var widgetTransferencia = widget.getParent();
        if (widgetTransferencia.mode == 'edit') {
            if (widget.currentRow.NRREQUESTO) {
                this.callInformacoesComplmentares(widget);
                widget.backupRow = Util.clone(widget.currentRow, ['__rightActions', '__leftActions']);
                widget.edit();
                widget.getField('QTTOTLANCTO').readOnly = true;
            }
            else {
                widget.edit();
            }
        }
        else {
            widget.edit();
            widget.getField('QTTOTLANCTO').readOnly = false;
            widget.currentRow.IDQUANTID = 'S';
            Est57000_TransferenciaEstoque_Service.verificaEstoqueProduto(widget.currentRow).then(function(success) {
                var result = 0;
                if (success.dataset.lote_infCompl_almox[0]) {
                    result = success.dataset.lote_infCompl_almox[0].QTESTOQDIA;
                }
                var mensagem = "Posição Estoque: ".concat(result);
                widget.getField('QTTOTLANCTO').validations.max = { "value": result, "message": mensagem };
            });
        }
    };

    this.deletarTransferencia = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function(success) {
            this.getFilialLogada().then(function(success) {
                Est57000_TransferenciaEstoque_Service.deletarTransferencia(widget.currentRow).then(
                    function(Success) {
                        ScreenService.closeSwipe().then(function() {
                            widget.activate();
                        });
                        widget.reload();
                    }.bind(this),
                    function(callback) {
                        if (callback.data) {
                            ScreenService.showMessage(callback.data.error);
                        }
                    }
                );
            }.bind(this));
        }.bind(this));
    };

    this.setAlmoxRequisicao = function(widget) {
        if (widget.currentRow.NRREQUESTO) {
            var row = {
                NRREQUESTO: widget.currentRow.NRREQUESTO
            };
            var controle = templateManager.container.getWidget('gridTransferencia').appliedFilter.FILTER_ROW;
            if (controle.IDUTILALMOX === 'S') {
                Est57000_TransferenciaEstoque_Service.getAlmoxRequisicao(row).then(function(success) {
                    var result = success.dataset.getAlmoxRequisicao[0];
                    if (result.CDALMOXSAIDA) {
                        widget.currentRow.DSALMOXSAID = result.DSALMOXSAIDA;
                        widget.currentRow.CDALMOXSAID = result.CDALMOXSAIDA;
                        widget.currentRow.P_CDALMOXORIG = result.CDALMOXSAIDA;
                    }
                    if (result.CDALMOXENTR) {
                        widget.currentRow.DSALMOXENTR = result.DSALMOXENTR;
                        widget.currentRow.CDALMOXENTR = result.CDALMOXENTR;
                        widget.currentRow.P_CDALMOXDEST = result.CDALMOXENTR;
                    }
                    if (controle.IDUTILLCESTQ === 'S') {
                        widget.getField('DSLOCALSAID').readOnly = false;
                        widget.getField('DSLOCALENTR').readOnly = false;
                        if (result.CDLESTQSAIDA) {
                            widget.currentRow.DSLOCALSAID = result.DSLESTQSAIDA;
                            widget.currentRow.CDLOCALSAID = result.CDLESTQSAIDA;
                            widget.currentRow.P_CDLOCALORIG = result.CDLESTQSAIDA;
                        }
                        if (result.CDLESTQENTR) {
                            widget.currentRow.DSLOCALENTR = result.DSLESTQENTR;
                            widget.currentRow.CDLOCALENTR = result.CDLESTQENTR;
                            widget.currentRow.P_CDLOCALDEST = result.CDLESTQENTR;
                        }
                    }

                });
            }
        }
    };

    this.getFilialLogada = function(widget) {
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": "FILIAL_LOGADA"
        }];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.controlaLoteProduto = function(field) {
        var widget = field.widget;
        var filtroPopupCurrentRow = widget.getParent().appliedFilter.FILTER_ROW;
        field.reload().then(
            function(Success) {
                if (filtroPopupCurrentRow.IDUTILLOTE == 'S') {
                    if (widget.currentRow.CDARVPROD) {
                        if (widget.currentRow.IDUTCTRLLOTE == 'S') {
                            if (field.dataSource.data.length == 0) {
                                ScreenService.showMessage('Lote não cadastrado para o produto informado.');
                                field.dataSource.data = null;
                            }
                        }
                        else {
                            ScreenService.showMessage('Produto não controla lote.');
                            field.dataSource.data = null;
                        }
                    }
                    else {
                        ScreenService.showMessage('Lote não cadastrado para o produto informado.');
                        field.dataSource.data = null;
                    }
                }
            },
            function(callback) {
                if (callback.data) {
                    ScreenService.showMessage(callback.data.error);
                }
            }
        );
    };

    this.copyDataWidgetItens = function(data) {
        var copy = [];
        data.forEach(function(item) {
            copy.push(Util.clone(item, ['__rightActions', '__leftActions']));
        });
        return copy;
    };

    var setCurrentRowWithOutData = function(currentRow, valuesToSetOnCurrentRow, outDataValue) {
        for (var key in outDataValue) {
            if (Array.isArray(outDataValue[key])) {
                let newArray = [];
                outDataValue[key].forEach(value => newArray[value] = value); // jshint ignore:line
                setCurrentRowWithOutData(currentRow, valuesToSetOnCurrentRow, newArray);
            }
            else
                currentRow[outDataValue[key]] = valuesToSetOnCurrentRow[key];
        }
    };

    // var decodeTextViewFloat = function (fieldRow) {
    // 	if(typeof fieldRow === 'string'){
    //         fieldRow = fieldRow.replace(/\./g, '');
    //         fieldRow = fieldRow.replace(',', '.');
    // 	}
    //     var viewRow = parseFloat(fieldRow);
    // 	viewRow = isNaN(viewRow) ? "" : viewRow;
    //     return addThousandSeparators(viewRow);
    // };

    // function addThousandSeparators(value) {
    //     value += '';
    //     var x = value.split(',');
    //     var x1 = x[0];
    //     var x2 = x.length > 1 ? ',' + x[1] : '';
    //     var rgx = /(\d+)(\d{3})/;
    //     while (rgx.test(x1)) {
    //         x1 = x1.replace(rgx, '$1.$2');
    //     }

    //     return x1 + x2;
    // }

    this.openAnexoSwipe = function() {
        let widgetImportTxt = templateManager.container.getWidget('wdgUploadTxt');
        widgetImportTxt.isVisible = true;
        ScreenService.openSwipe(widgetImportTxt).then(function() {
            widgetImportTxt.newRow();
        });
    };

    this.closeAnexoSwipe = function(widget) {
        let widgetProduto = templateManager.container.getWidget('gridProdutosTransferencia');
        widgetProduto.activate();
        widget.dataSource.data.pop();
        ScreenService.closeSwipe();
        widget.isVisible = false;
    };

    this.importarProdutosTxt = function(widget) {

        if (!widget.isValid()) {
            return;
        }

        var widgetProduto = templateManager.container.getWidget('gridProdutosTransferencia');
        var widgetTransferencia = templateManager.container.getWidget('gridTransferencia');
        var ignoredKeys = ["__rightActions", "__createdLocal", "__index", "$error"];

        widget.isVisible = false;
        ScreenService.closeSwipe().then(() => {
            widget.currentRow.CDFILIAL = widgetTransferencia.currentRow.CDFILIAL;
            widget.currentRow.DTMOVIESTQ = widgetTransferencia.currentRow.DTMOVIESTQ;
            widget.currentRow.CONTAINER = widget.container.name;
            widget.currentRow.CDFILIMOVI = widgetTransferencia.appliedFilter.FILTER_ROW.CDFILIAL;
            this.getParamsImpEst(widget.currentRow.CDFILIMOVI).then(response => {
                if (response.dataset.paramimpest_params_codebar.length > 0) {
                    widget.currentRow.PARAMS = response.dataset.paramimpest_params_codebar[0];
                    Est57000_TransferenciaEstoque_Service.importarProdutosTxt(Util.clone(widget.currentRow, ignoredKeys)).then(success => {
                        var itens = success.dataset.itensTransferencia;
                        var itensDataSource = widgetProduto.dataSource;

                        if (itens.length > 0) {
                            widgetTransferencia.currentRow.DTMOVIESTQ = itens[0].DTMOVIESTQ;
                            itensDataSource.data = itens;

                            itensDataSource.data.forEach((row) => {

                                Est57000_TransferenciaEstoque_Service.getCdtipocusto(row.CDFILIAL).then((cdtipocusto) => {
                                    row.DTLANCMOVI = widgetTransferencia.currentRow.DTMOVIESTQ;

                                    if (!row.CDALMOXARIFE) row.CDALMOXARIFE = ' ';
                                    if (!row.CDLOCALESTOQ) row.CDLOCALESTOQ = ' ';
                                    if (!row.NRLOTEESTQ) row.NRLOTEESTQ = ' ';
                                    if (!row.NRSUBLOTE) row.NRSUBLOTE = ' ';

                                    ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(
                                        row.CDFILIAL,
                                        row.CDPRODUTO,
                                        row.DTLANCMOVI.substr(0, 10),
                                        cdtipocusto,
                                        row.CDALMOXARIFE,
                                        row.CDLOCALESTOQ,
                                        row.NRLOTEESTQ,
                                        row.NRSUBLOTE
                                    ).then((success) => {
                                        var consulta = success.dataset.consultaCustoQuantidadeProduto;
                                        row.QTESTOQDIA = consulta.QTESTOQDIA;
                                        row.VRUNILANCTO = consulta.VRCUSTOPROD;
                                        Est57000_TransferenciaEstoque_Service.setVrTotLancto(row);
                                        row.__is_new = true;
                                        row.changed = true;
                                        row.COMPLETO = 'S';
                                    });
                                });
                            });
                            this.controlButtonSaveAndImportVisibility(widgetProduto);
                        }
                        else {
                            ScreenService.showMessage("Não foram encontrados nenhum produto a partir do arquivo");
                        }
                    }).catch(error => {
                        if (error.data) {
                            if (error.data.errorCode == 400) {
                                ScreenService.closeSwipe().then(() => {
                                    let impAction = widget.getParent().getAction("importTxt");
                                    this.openAnexoSwipe(impAction);
                                    ScreenService.showMessage(error.data.error);
                                });
                            }
                            else {
                                ScreenService.showMessage(error.data.error);
                            }
                        }
                        else {
                            ScreenService.showMessage(error);
                        }

                    });
                }
                else {
                    ScreenService.showMessage("Parâmetros para importação de estoque não encontrados.");
                }
            }).catch(error => {
                if (error.data) {
                    ScreenService.showMessage(error.data.error);
                }
                else {
                    ScreenService.showMessage(error);
                }
            });
        });
        widgetProduto.activate();
        widgetProduto.list();
        widgetProduto.reload();
    };

    this.getParamsImpEst = function(cdfilial) {
        var filter = [
            {
                "name": "CDFILPARIMPEST",
                "operator": "=",
                "value": cdfilial
                }
		    ];
        return General.callRestEngine('est_ajuste#/crud/paramimpest_params_codebar', 'FilterData', filter);
    };

    this.controlButtonSaveAndImportVisibility = function(widget) {
        let widgetTranferencia = templateManager.container.getWidget('gridTransferencia');
        if (!widgetTranferencia.currentRow.NRCTRLMOVEST) {
            if (widget.dataSource.data.length != 0) {
                this.changeHideInconLabelVisible(widget.actions, 'Salvar', true, 'saveControl', true);
                this.changeHideInconLabelVisible(widget.actions, 'Importar por txt', false, 'importTxt', false);
            }
            else {
                if (widgetTranferencia.currentRow.NRREQUESTO) {
                    this.changeHideInconLabelVisible(widget.actions, 'Salvar', true, 'saveControl', true);
                    this.changeHideInconLabelVisible(widget.actions, 'Importar por txt', false, 'importTxt', false);
                }
                else {
                    this.changeHideInconLabelVisible(widget.actions, 'Salvar', false, 'saveControl', false);
                    this.changeHideInconLabelVisible(widget.actions, 'Importar por txt', true, 'importTxt', true);
                }
            }
        }
        else {
            if (widget.currentRow.__is_new) {
                if (widget.dataSource.data.length != 0) {
                    this.changeHideInconLabelVisible(widget.actions, 'Salvar', true, 'saveControl', true);
                    this.changeHideInconLabelVisible(widget.actions, 'Importar por txt', false, 'importTxt', false);
                }
                else {
                    this.changeHideInconLabelVisible(widget.actions, 'Salvar', false, 'saveControl', false);
                    this.changeHideInconLabelVisible(widget.actions, 'Importar por txt', true, 'importTxt', true);
                }
            }
            else {
                this.changeHideInconLabelVisible(widget.actions, 'Salvar', false, 'saveControl', false);
                this.changeHideInconLabelVisible(widget.actions, 'Importar por txt', false, 'importTxt', false);
            }
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est57000_TransferenciaEstoque_Controller', Est57000_TransferenciaEstoque_Controller);
});


// FILE: ./assets/js/controllers/Ger03300_FecharDia_Controller.js
var Ger03300_FecharDia_Controller = function(MessageService,ScreenService, Ger03300_FecharDia_Repository, templateManager, Unidade_Repository, General) {

    this.fecharDia = function(widget) {
        var proxDia =  widget.getField('PROXDIAPROCESS').value();
        var dataProces =  widget.getField('DIAPROCESS').value();
        var cdfilial = widget.getField('CDFILIAL').value();
        
        ScreenService.showLoader();
        Ger03300_FecharDia_Repository.getDataSource("est#/data_proxdataproces").then(((dataSugerida)=>{
            if (moment(dataProces,'DD/MM/YYYY').isAfter(moment(dataSugerida[0].DTDATA,'DD/MM/YYYY'))){
                ScreenService.alertNotification('O fechamento do dia já foi realizado para a data de processamento informada. Clique em OK para atualizar a data de processamento.');
            }else if(proxDia === null || proxDia === undefined || proxDia === ""){
                ScreenService.alertNotification('Prox. data de processamento deve ser informada.');
            }else if(moment(proxDia,'DD/MM/YYYY').isBefore(moment(dataProces,'DD/MM/YYYY'))){
                ScreenService.alertNotification('Próxima data de processamento deve ser maior que a data de processamento atual.');
            }else{
                if(General.isValidFiliaisValuesCP(cdfilial)){
                    var filterValues = [];
                    filterValues.CDFILIAL = cdfilial;
                    filterValues.DTPROCESSA = dataProces;
                    filterValues.PROXDTPROCESSA = proxDia;
    
                    this.callFecharDia(filterValues, false, function(response) {
                        if(response.dataset.fechardia.userprompt)
                            ScreenService.confirmMessage(response.messages[0].message, 'question', function() {
                                this.callFecharDia(response.dataset.fechardia, true, this.callbackFecharDia);
                            }.bind(this));
                        else
                            this.callbackFecharDia(response);
                    });
                }else{
                    MessageService.showMessageByCode("UNIDADE_NAO_HABILITADA_CP");
                }
            }
            this.setDtProcessa(cdfilial);
            ScreenService.hideLoader();
        }).bind(this));
    };

    this.callbackFecharDia = function(response) {
        this.setDtProcessa(response.dataset.fechardia.CDFILIAL);

        if(response.dataset.fechardia.InveRota) {
            ScreenService.confirmMessage(response.messages[0].message + ' Deseja emitir o relatório para Inventário rotativo?').then(function(){
                ScreenService.openWindow('est#ger03301_relatorio_inventario_rotativo');
            });
        } else {
            ScreenService.showMessage(response.messages[0].message);
            templateManager.headerInfo = templateManager.headerInfo.replace(response.dataset.fechardia.DTPROCESSA, response.dataset.fechardia.PROXDTPROCESSA);
            templateManager.userData.descriptions[0] = templateManager.userData.descriptions[0].replace(response.dataset.fechardia.DTPROCESSA, response.dataset.fechardia.PROXDTPROCESSA);
        }
        
        ScreenService.hideLoader();
    };

    this.callFecharDia = function(filterValues, SKIPINVENTARIOCEGO, callback) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filterValues.CDFILIAL
            },
            {
                "name": "DTPROCESSA",
                "operator": "=",
                "value": filterValues.DTPROCESSA
            },
            {
                "name": "PROXDTPROCESSA",
                "operator": "=",
                "value": filterValues.PROXDTPROCESSA
            },
            {
                "name": "SKIPINVENTARIOCEGO",
                "operator": "=",
                "value": SKIPINVENTARIOCEGO
            }
        ];
        
        Ger03300_FecharDia_Repository.consulta('est#/fechardia', filter).then(callback.bind(this),
            function(error) {
                ScreenService.showMessage(error.data.error);
                ScreenService.hideLoader();
            }
            
        );
    };

    this.fecharDiaOnEnter = function(widget, cdfilialField, nmfilialField) {
        var cdunidade;
        var nmunidade;
        var row = {
            CDFILIAL: getLocalVar('CDFILIAL')
        };
        
        General.callRestEngine("est#/getDadosFecharDia", "Row", row).then((result) => {
            result = result.dataset.dados_fechardia;
            cdfilialField.setValue(result.CDFILIAL);
            nmfilialField.setValue(result.NMFILIAL);
            
            widget.getField('DIAPROCESS').setValue(moment(result.DTPROCESSA,'DD/MM/YYYY').format('DD/MM/YYYY'));
            widget.getField('NMDIAPROCESS').setValue(moment(result.DTPROCESSA,'DD/MM/YYYY').format('dddd'));
            
            if(result.IDALTDTFECDIA === 'S'){
                widget.getField('PROXDIAPROCESS').readOnly = false;
            }else{
                widget.getField('PROXDIAPROCESS').readOnly = true;
            }
            if(result.IDCONSDIAUTFEC != 'S'){
                widget.getField('PROXDIAPROCESS').setValue(moment(result.DTPROCESSA,'DD/MM/YYYY').add(1,'day').format('DD/MM/YYYY'));
                widget.getField('NMPROXDIAPROCESS').setValue(moment(result.DTPROCESSA,'DD/MM/YYYY').add(1,'day').format('dddd'));
                
                let nextDate = widget.getField('PROXDIAPROCESS').value();
                Ger03300_FecharDia_Repository.setDataSource("est#/data_proxdataproces", nextDate);
            }else{
                var filter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": result.CDFILIAL
                },{
                    "name": "DTPROCES",
                    "operator": "=",
                    "value": moment(result.DTPROCESSA,'DD/MM/YYYY').format('DD/MM/YYYY')
                }];
                
                Ger03300_FecharDia_Repository.consulta("est#/data_proxdataproces", filter).then(function(banco2) {
                    var proxdata = banco2.dataset.data_proxdataproces;
                    widget.getField('PROXDIAPROCESS').setValue(proxdata[0].DTDATA);
                    widget.getField('NMPROXDIAPROCESS').setValue(moment(proxdata[0].DTDATA,'DD/MM/YYYY').format('dddd'));
                    Ger03300_FecharDia_Repository.setDataSource("est#/data_proxdataproces", proxdata);
                }, function(error) {
                    ScreenService.showMessage(error.data.error);
                });
            }
        });
    };

    this.setDtProcessa = function(cdfilial){
        var container = templateManager.container;
        var widget = container.getWidget('fecharDia');
        var filial = widget.getField('CDFILIAL').value();
        var filterParamgeral = [
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": filial
            }
        ];
        Ger03300_FecharDia_Repository.consulta("est#/paramfilial_paramvalidacoes", filterParamgeral).then(function(banco) {
            var paramGeral = banco.dataset.paramfilial_paramvalidacoes;
            widget.getField('DIAPROCESS').setValue(moment(paramGeral[0].DTPROCESSA,'DD/MM/YYYY').format('DD/MM/YYYY'));
            widget.getField('NMDIAPROCESS').setValue(moment(paramGeral[0].DTPROCESSA,'DD/MM/YYYY').format('dddd'));
            if(paramGeral[0].IDALTDTFECDIA === 'S'){
                widget.getField('PROXDIAPROCESS').readOnly = false;
            }else{
                widget.getField('PROXDIAPROCESS').readOnly = true;
            }
            if(paramGeral[0].IDCONSDIAUTFEC != 'S'){
                widget.getField('PROXDIAPROCESS').setValue(moment(paramGeral[0].DTPROCESSA,'DD/MM/YYYY').add(1,'day').format('DD/MM/YYYY'));
                widget.getField('NMPROXDIAPROCESS').setValue(moment(paramGeral[0].DTPROCESSA,'DD/MM/YYYY').add(1,'day').format('dddd'));
            }else{
                var filter = [{
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },{
                    "name": "DTPROCES",
                    "operator": "=",
                    "value": moment(paramGeral[0].DTPROCESSA,'DD/MM/YYYY').format('DD/MM/YYYY')
                }];
                Ger03300_FecharDia_Repository.consulta("est#/data_proxdataproces", filter).then(function(banco2) {
                    var proxdata = banco2.dataset.data_proxdataproces;
                    widget.getField('PROXDIAPROCESS').setValue(proxdata[0].DTDATA);
                    widget.getField('NMPROXDIAPROCESS').setValue(moment(proxdata[0].DTDATA,'DD/MM/YYYY').format('dddd'));
                    Ger03300_FecharDia_Repository.setDataSource("est#/data_proxdataproces", proxdata);
                }, function(error) {
                    ScreenService.showMessage(error.data.error);
                });
            }
        });
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger03300_FecharDia_Controller', Ger03300_FecharDia_Controller);
});

// FILE: ./assets/js/controllers/Ger03301_RelatorioInventarioRotativo_Controller.js
var Ger03301_RelatorioInventarioRotativo_Controller = function(ScreenService, Ger03301_RelatorioInventarioRotativo_Repository, Unidade_Repository, ZhFilterPreferences, templateManager) {
    var readOnlyLocalizacao;
    var cdfilial;
    var dtprocessa;
    var cdalmoxarifado;
    var cdlocalizacao;

    this.setCdfilialAndReadOnly = function(widget) {
       /* Unidade_Repository.retornaUnidadeLogada().then(function(sucess){
            cdfilial = sucess[0].CDFILIAL;
            widget.getField('REPORT_NAME').setValue('CD'); // DEFAULTVALUE não estava funcionando para select-ntive, se estiver funcionando, essa linha pode ser apagada
            var data = [{
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            }];
            Ger03301_RelatorioInventarioRotativo_Repository.paramFilial(data).then(function(sucess){
                dtprocessa = moment(sucess[0].DTPROCESSA, "DD/MM/YYYY").format("DD/MM/YYYY");
                if(sucess[0].IDUTILALMOX == 'S'){
                    var field = widget.getField('DSALMOXARIFE');
                    field.readOnly = false;
                    field.dataSourceFilter[0].value = cdfilial;
                    field.filterProperties.validations.required = true;
                    if(sucess[0].IDUTILLCESTQ == 'S'){
                        field = widget.getField('DSLOCALESTOQ');
                        field.readOnly = false;
                        readOnlyLocalizacao = false;
                        field.dataSourceFilter[2].value = cdfilial;
                        field.filterProperties.validations.required = true;
                    } else {
                        readOnlyLocalizacao = true;
                    }
                }
            });
        }); */
    };

    this.setDataSourceAndReadOnly = function(widget){
        cdalmoxarifado = widget.currentRow.CDALMOXARIFE;
        if(!readOnlyLocalizacao){
            var field = widget.getField('DSLOCALESTOQ');
            field.readOnly = false;
            field.dataSourceFilter[2].value = cdalmoxarifado;
        }
    };

    this.buildDataSourceFilter = function(widget){
        var widgetPopupFilter = widget.appliedFilter.FILTER_ROW;
        var filtro = [
            {
                "name": "PARAM",
                "operator": "=",
                "value": widgetPopupFilter.REPORT_NAME
            },
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": cdfilial
            },
            {
                "name": "P_DTPROCESSA",
                "operator": "=",
                "value": dtprocessa
            },
            {
                "name": "P_CDALMOXARIFE",
                "operator": "=",
                "value": widgetPopupFilter.CDALMOXARIFE||" "
            },
            {
                "name": "P_CDLOCALESTOQ",
                "operator": "=",
                "value": widgetPopupFilter.CDLOCALESTOQ||" "
            }
        ];
        widget.dataSourceFilter = filtro;
    };

    this.setCdlocalizacao = function(fieldValue){
        cdlocalizacao = fieldValue;
    };

    this.buildDatSetForReport = function(widget){
        var dataSet = widget.dataSource.data;
        let codAlmox = ' ';//dataSet[0].CDALMOXARIFE;
        let dsAlmox = '';
        let codLocal = ' ';//dataSet[0].CDLOCALESTOQ;
        let dsLocal = '';
        
        var rowLocais = {
                            "NMPRODUTO": codAlmox+' - '+dsAlmox,
                            "NRLOTEESTQ": codLocal+' - '+dsLocal
                        }; 
        //dataSet.splice(0, 0, rowLocais);
        for (var i = 0; i < dataSet.length; i++) {
            if(( dataSet[i].CDALMOXARIFE != codAlmox) || (dataSet[i].CDLOCALESTOQ != codLocal )) {
                codAlmox = dataSet[i].CDALMOXARIFE;
                dsAlmox = dataSet[i].DSALMOXARIFE != ' '? ' - '+dataSet[i].DSALMOXARIFE : '';
                codLocal = dataSet[i].CDLOCALESTOQ;
                dsLocal = dataSet[i].DSLOCALESTOQ != ' '? ' - '+dataSet[i].DSLOCALESTOQ : '';
                
                rowLocais = {
                                "NMPRODUTO": codAlmox + dsAlmox,
                                "NRLOTEESTQ": codLocal + dsLocal
                            }; 
                dataSet.splice(i, 0, rowLocais);
            } else {
                dataSet[i].SALDO = "_____________";
                if(dataSet[i].DTVALILOTE){
                    dataSet[i].DTVALILOTE = moment(widget.dataSource.data[i].DTVALILOTE, "DD/MM/YYYY").format("DD/MM/YYYY");
                }
            }
            
        }
        widget.dataSource.data = dataSet;
    };
    
    this.changeFilterFilial = function(cdfilial){
        var data = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": cdfilial
        }];
        return Ger03301_RelatorioInventarioRotativo_Repository.paramFilial(data).then((sucess) => {
            let widgetFiltro = templateManager.container.getActiveWidget();
            dtprocessa = moment(sucess[0].DTPROCESSA, "DD/MM/YYYY").format("DD/MM/YYYY");
            if(sucess[0].IDUTILALMOX == 'S'){
                var field = widgetFiltro.getField('DSALMOXARIFE');
                field.readOnly = false;
                //field.validations.required = true;  remover required dos campos de almox pois existem nvl na query
                if(sucess[0].IDUTILLCESTQ == 'S'){
                    field = widgetFiltro.getField('DSLOCALESTOQ');
                    field.readOnly = false;
                    readOnlyLocalizacao = false;
                    //field.validations.required = true;  remover required dos campos de almox pois existem nvl na query
                } else {
                    readOnlyLocalizacao = true;
                }
            }
        });
    };
    
    this.startFilter = function(widget) {
        var filterRow = {};
        Unidade_Repository.retornaUnidadeLogada().then((sucess) => {
            filterRow = sucess[0];
            filterRow.REPORT_NAME = 'CD';
            widget.getField('REPORT_NAME').setValue('CD'); // DEFAULTVALUE não estava funcionando para select-ntive, se estiver funcionando, essa linha pode ser apagada
            ZhFilterPreferences.openFilterWidget(widget, filterRow).then(opened => {
                 this.changeFilterFilial(filterRow.CDFILIAL);
            });
        });
    };
    
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger03301_RelatorioInventarioRotativo_Controller', Ger03301_RelatorioInventarioRotativo_Controller);
});

// FILE: ./assets/js/controllers/Ger03302_InventarioRotativo_Controller.js
var Ger03302_InventarioRotativo_Controller = function(ScreenService, ParamfilialService, ParamgeralService, templateManager, InventarioRotativo_Service, General, ZhFilterPreferences, EstUtil) {

    this.ajustou = false;

     this.validateInventarioRotativo = function(args){
        InventarioRotativo_Service.getOperadorLogado().then(function(operador) {
            operador = operador[0];
            var cdfilial = operador.CDFILIALSELECIONADA;
            var nrorg = operador.NRORG;
            var route = 'est#/validateInventarioRotativo';
            General.callRestEngine(route, 'Row', {'CDFILIAL': cdfilial})
            .then(function(response){
                var inventarioIsValid = (!!response.dataset) ? response.dataset.AtualizaInveRota[0] : false;
                if (inventarioIsValid) {
                    this.ABRIR_FILTRO = false;
                    ScreenService.openWindow("est#ger03302_blank_window").then(function(){
                        ScreenService.showMessage("<br />Não há inventário pendente para a unidade.");     
                    });
                }else{
                    this.ABRIR_FILTRO = true;
                }
            }.bind(this))
            .catch(function(error){
                ScreenService.showMessage(error.data.error);
            });
        }.bind(this))
        .catch(function(error){
            ScreenService.showMessage(error.data.error);
        });
    };
    
    this.onEnter = function(){
        var filterRow = {};
        var container = templateManager.container;
        var widgetGrid = container.getWidget('Lancamento');
        InventarioRotativo_Service.getOperadorLogado().then((operador) => {
            operador = operador[0];
            var cdfilial = operador.CDFILIALSELECIONADA;
            var nrorg = operador.NRORG;
            ParamfilialService.getParamfilial(nrorg,cdfilial).then((paramFilial) => {
                paramFilial= paramFilial[0];
                ParamgeralService.getParamgeral(nrorg).then((paramGeral) => {
                    paramGeral= paramGeral[0];
                    var dtprocessa = (paramGeral.IDUTILDTPROC === 'S') ? paramFilial.DTPROCESSA : moment().format("DD/MM/YYYY");
                    var idUtAlmox = paramFilial.IDUTILALMOX==='S';
                    var idUtLocal = paramFilial.IDUTILLCESTQ==='S';
                    var idUtLote = paramFilial.IDUTILLOTE==='S';
                    var fieldLocal = widgetGrid.getField("CDLOCALESTOQ");
                    var fieldAlmox = widgetGrid.getField("CDALMOXARIFE");
                    fieldAlmox.readOnly = !idUtAlmox;
                    fieldLocal.readOnly = !(idUtAlmox&&idUtLocal);
                    fieldAlmox.dataSource.name = (idUtLote) ? "est#/almoxarifado_inverotalote" : "est#/almoxarifado_inverota";
                    fieldLocal.dataSource.name = (idUtLote) ? "est#/localestoque_inverotalote" : "est#/localestoque_inverota";
                    if (idUtAlmox) {
                        fieldAlmox.filterProperties.validations = {required: true};
                    }else{
                        delete fieldAlmox.filterProperties.validations;
                    }
                    if (idUtLocal) {
                        fieldLocal.filterProperties.validations = {required: true};
                    }else{
                        delete fieldLocal.filterProperties.validations;
                    }
                    widgetGrid.getField("NRLOTEESTQ").isVisible = idUtLote;
                    widgetGrid.getField("NRSUBLOTE").isVisible = idUtLote;
                    widgetGrid.getField("DTVALILOTE").isVisible = idUtLote;
                    
                    
                    filterRow.CDFILIAL = cdfilial;
                    filterRow.NMFILIAL = getLocalVar('NMFILIAL');
                    filterRow.DTINVEROTA = dtprocessa;
                    filterRow.CDALMOXARIFE = idUtAlmox?null:' ';
                    filterRow.CDLOCALESTOQ = (idUtAlmox&&idUtLocal)?null:' ';
                    filterRow.UTLOTE = idUtLote;
                    /*widgetGrid.dataSourceFilter = [
                        {
                            "name": "CDFILIAL",
                            "operator": "=",
                            "value": cdfilial
                        },{
                            "name": "DTINVEROTA",
                            "operator": "=",
                            "value": dtprocessa
                        },{
                            "name": "CDALMOXARIFE",
                            "operator": "=",
                            "value": idUtAlmox?null:' '
                        },{
                            "name": "CDLOCALESTOQ",
                            "operator": "=",
                            "value": (idUtAlmox&&idUtLocal)?null:' '
                        },{
                            "name":"UTLOTE",
                            "operator":"=",
                            "value":idUtLote
                        }
                    ];
                    fieldAlmox.dataSourceFilter[0].value = cdfilial;
                    fieldLocal.dataSourceFilter[0].value = cdfilial;
                    fieldAlmox.dataSourceFilter[1].value = dtprocessa;
                    fieldLocal.dataSourceFilter[1].value = dtprocessa;
                    */
                    var validaOpenFilter = true;
                    
                    //nao remover a linha do ignore do lint
                    /* jshint ignore:start */
                    while (!this.hasOwnProperty('ABRIR_FILTRO') || validaOpenFilter){
                        if (this.ABRIR_FILTRO){
                            //ZhGridFilter.startFiltering(templateManager.container.widgets[0]);
                            this.openFilter(templateManager.container.widgets[0], filterRow).then(opened => {
                                let filterWidget = templateManager.container.getActiveWidget();
                                var fieldLocalFilter = filterWidget.getField("CDLOCALESTOQ");
                                var fieldAlmoxFilter = filterWidget.getField("CDALMOXARIFE");
                                fieldAlmoxFilter.readOnly = !idUtAlmox;
                                fieldLocalFilter.readOnly = !(idUtAlmox&&idUtLocal);
                                fieldAlmoxFilter.dataSource.name = (idUtLote) ? "est#/almoxarifado_inverotalote" : "est#/almoxarifado_inverota";
                                fieldLocalFilter.dataSource.name = (idUtLote) ? "est#/localestoque_inverotalote" : "est#/localestoque_inverota";
                                if (idUtAlmox) {
                                    fieldAlmoxFilter.validations = {required: true};
                                }else{
                                    delete fieldAlmoxFilter.validations;
                                }
                                if (idUtLocal) {
                                    fieldLocalFilter.validations = {required: true};
                                }else{
                                    delete fieldLocalFilter.validations;
                                }
                            });
                        }
                        validaOpenFilter = false;
                    }
                    /* jshint ignore:end */
                });
            });
        });
    };
    
    this.onChangeFilial = function(args) {
        let filterWidget = args.owner.field.widget;
        let cdfilial = filterWidget.currentRow.CDFILIAL;
        let nrorg = getLocalVar('NRORG');
        ParamfilialService.getParamfilial(nrorg,cdfilial).then((paramFilial) => {
            paramFilial= paramFilial[0];
            ParamgeralService.getParamgeral(nrorg).then((paramGeral) => {
                paramGeral= paramGeral[0];
                var dtprocessa = (paramGeral.IDUTILDTPROC === 'S') ? paramFilial.DTPROCESSA : moment().format("DD/MM/YYYY");
                var idUtAlmox = paramFilial.IDUTILALMOX==='S';
                var idUtLocal = paramFilial.IDUTILLCESTQ==='S';
                var idUtLote = paramFilial.IDUTILLOTE==='S';
                var fieldLocal = filterWidget.getField("CDLOCALESTOQ");
                var fieldAlmox = filterWidget.getField("CDALMOXARIFE");
                fieldAlmox.readOnly = !idUtAlmox;
                fieldLocal.readOnly = !(idUtAlmox&&idUtLocal);
                fieldAlmox.dataSource.name = (idUtLote) ? "est#/almoxarifado_inverotalote" : "est#/almoxarifado_inverota";
                fieldLocal.dataSource.name = (idUtLote) ? "est#/localestoque_inverotalote" : "est#/localestoque_inverota";
                if (idUtAlmox) {
                    fieldAlmox.validations = {required: true};
                }else{
                    delete fieldAlmox.validations;
                }
                if (idUtLocal) {
                    fieldLocal.validations = {required: true};
                }else{
                    delete fieldLocal.validations;
                }
                filterWidget.getField("NRLOTEESTQ").isVisible = idUtLote;
                filterWidget.getField("NRSUBLOTE").isVisible = idUtLote;
                filterWidget.getField("DTVALILOTE").isVisible = idUtLote;
                
                
                filterWidget.currentRow.CDFILIAL = cdfilial;
                filterWidget.currentRow.DTINVEROTA = dtprocessa;
                filterWidget.currentRow.CDALMOXARIFE = idUtAlmox?null:' ';
                filterWidget.currentRow.CDLOCALESTOQ = (idUtAlmox&&idUtLocal)?null:' ';
                filterWidget.currentRow.UTLOTE = idUtLote;

            });
        }); 
    };
    
    this.openFilter = function(widget, row = {}) {
        this.setFieldsGerais(widget);
        ZhFilterPreferences.setOriginalFilter(widget.id, row);
        return ZhFilterPreferences.openFilterWidget(widget, row);
    };
    
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };
    
    this.qtinverotaOnInlineEdit = function(field) {
        if (!field.value()) {
            field.setValue("0.000");
        }
    };

    this.setFilterLocal = function(widget){
        //nao esta senda usada esta funcao
        var row = widget.currentRow;
        var fieldLocal = widget.getField("CDLOCALESTOQ");
        var cdAlmoxarife = row.CDALMOXARIFE;
        fieldLocal.dataSourceFilter[2].value = cdAlmoxarife;
        fieldLocal.readOnlyOnEdit = !cdAlmoxarife;
        fieldLocal.value(fieldLocal.readOnly?" ":null);
    };

    this.montaItemAjuste = function(row){
        var container = templateManager.container;
        var stripe = container.getWidget('stripe');
        var cdalmoxarife = stripe.getField('CDALMOXARIFE').value();
        var cdlocalestoq = stripe.getField('CDLOCALESTOQ').value();
        var cdfilial = stripe.getField('DSFILIAL').value().substr(0,4);
        var dtinverota = stripe.getField('DTPROCESSA').value();
        var newRow = {
            "CDALMOXARIFE": cdalmoxarife,
            "CDFILIAL": cdfilial,
            "CDLOCALESTOQ": cdlocalestoq,
            "CDPRODUTO": row.CDPRODUTO,
            "DTINVEROTA": dtinverota,
            "QTINVEROTA": row.QTINVEROTA,
        };
        return newRow;
    };

    this.save = function(rows,widget){
        var utlote = widget.getField("NRLOTEESTQ").isVisible;
        rows.forEach(function(row){
            row[utlote?"DTINVELOTE":"DTINVEROTA"]  = row.DATA;
            row.QTINVELOTE = row.QTINVEROTA;
        });
        InventarioRotativo_Service.save(rows,utlote).then(function(response){
            widget.commitInlineEdit();
            widget.list();
            this.ajustou = true;
        }.bind(this));
    };



    this.finaliza = function() {
        if(this.ajustou){
            ScreenService.showCustomDialog('Deseja atualizar estoque?', 'question', 'question', [{
                label: ScreenService.i18n("Sim"),
                code: function(){this.chamaProcedure('10');}.bind(this)
            },{
                label: ScreenService.i18n("Não"),
                code: function(){this.chamaProcedure('11');}.bind(this)
            },{
                label: ScreenService.i18n("Zerar"),
                code: function(){this.chamaProcedure('12');}.bind(this)
            }]);

        }else{
            ScreenService.showMessage('Ajuste um produto antes de concluir!');
        }
    };

    this.bloqueiaSaida = function(args){
        if(this.ajustou){
            args.e.preventDefault();
            ScreenService.showMessage("Conclua o lançamento");
        }
        else {
            templateManager.container.validaEstoque = true;
        }
    };

    this.chamaProcedure = function(opcao){
        var container = templateManager.container;
        var lancamento = container.getWidget('Lancamento');
        let widgetFiltro = EstUtil.getFilterWidget(lancamento);
        var filterPopup = lancamento.appliedFilter;

        var cdalmoxarife = widgetFiltro.currentRow.CDALMOXARIFE;
        var cdlocalestoq = widgetFiltro.currentRow.CDLOCALESTOQ;

        var cdfilial = widgetFiltro.currentRow.CDFILIAL;
        var dtprocessa = widgetFiltro.currentRow.DTINVEROTA;
        
        InventarioRotativo_Service.getOperadorLogado().then(function(operador){
            operador = operador[0];
            var cdfilimovi = operador.CDFILIALSELECIONADA;
            var cdoperador = operador.CDOPERADOR;
            var evolestoq;
            if(opcao =='11'){
                evolestoq = 'N';
            }else{
                evolestoq = 'S';
            }

            var filter = [{"name":"EVOLESTOQ","operator":"=","value":evolestoq},
                          {"name":"CDFILIMOVI","operator":"=","value":cdfilimovi},
                          {"name":"CDFILIAL","operator":"=","value":cdfilial},
                          {"name":"CDOPERADOR","operator":"=","value":cdoperador},
                          {"name":"DTPROCESSA","operator":"=","value":dtprocessa.substr(0,10)},
                          {"name":"INOPCAO","operator":"=","value":opcao},
                          {"name":"CDSERVFILI","operator":"=","value":null},
                          {"name":"CDTIPORET","operator":"=","value":null},
                          {"name":"CDCENTCUST","operator":"=","value":null},
                          {"name":"CDSETOR","operator":"=","value":null},
                          {"name":"RETIESTQ","operator":"=","value":"N"},
                          {"name":"CDALMOXARIFE","operator":"=","value":'_'+cdalmoxarife+'_'},
                          {"name":"CDLOCALESTOQ","operator":"=","value":'_'+cdlocalestoq+'_'}];
            InventarioRotativo_Service.atualizaInventarioRotativo(filter).then(function(response){
                var inventarioIsValid = (!!response.dataset) ? response.dataset.AtualizaInveRota[0] : false;
                var message = "";
                if (opcao == '11') {
                    message = 'Operação realizada com sucesso!';
                }
                else {
                    message = 'Estoque atualizado com sucesso!';
                }
                if (inventarioIsValid) {
                    message += "<br />Não há inventário pendente para a unidade.";
                }
                ScreenService.showMessage(ScreenService.i18n(message));
                lancamento.dataSource.data = [];
                lancamento.dataSourceFilter= [];
                filterPopup.FILTER_ROW = {};
                this.ajustou = false;
                this.onEnter();
            }.bind(this),function(error){
                ScreenService.showMessage("ATENÇÃO! A confirmação desta operação fará com que a posição de estoque de um ou mais produtos se torne irregular.",10000);
            });
        }.bind(this));
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger03302_InventarioRotativo_Controller', Ger03302_InventarioRotativo_Controller);
});

// FILE: ./assets/js/controllers/Ger03700_AlmoxarifadoLocalicazao_Controller.js
var Ger03700_AlmoxarifadoLocalicazao_Controller = function(templateManager, ScreenService, General, restEngine, ZhFilterPreferences) {

    var cdfilialGeral;
    var cdalmoxarifeGeral;

    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        var rowsToSet = {};

        rowsToSet.CDFILIAL = getLocalVar("CDFILIAL");
        rowsToSet.NMFILIAL = getLocalVar("NMFILIAL");

        ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
    }; 

    this.setFieldsGerais = function(widget) {
        templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
    };



    this.setStripeData = function(row) {
        var stripeWidget = templateManager.container.getWidget("stripe");

        stripeWidget.currentRow = {};
        stripeWidget.currentRow.NMFILIAL = row.CDFILIAL + " - " + row.NMFILIAL;
    };

    this.retornaParamUnidade = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [{
            "name": "CDFILIAL",
            "operator": "=",
            "value": cdFilialFiltrada
        }];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.retornaOperadorLogado = function() {
        var cdOperador = "OPERADOR_LOGADO";
        var filter = [{
            "name": "CDOPERADOR",
            "operator": "=",
            "value": cdOperador
        }];
        return restEngine.post("est#/crud/operador", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.retornaFilialOper = function(operador, filialselecionada) {
        var filter = [{
                "name": "CDOPERADOR",
                "operator": "=",
                "value": operador
            },
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("est#/crud/filioper", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.onEnterOnce = function(widget) {
        this.openFilter(widget);
    };

    //beforeFilterAply
    this.setVisibleWidgetLocal = function(args) {
        var widget = args.owner,
            currentRowPopup = widget.appliedFilter.FILTER_ROW;
        this.setStripeData(currentRowPopup);
        var cdfilial = currentRowPopup.CDFILIAL;
        cdfilialGeral = cdfilial;
        this.retornaParamUnidade(cdfilial).then(function(paramFilial) {
            var idutilalmox = paramFilial.dataset.paramfilial[0].IDUTILALMOX;
            if (idutilalmox === 'S') {
                var idutillcestq = paramFilial.dataset.paramfilial[0].IDUTILLCESTQ;
                if (idutillcestq === 'S') {
                    widget.widgets[0].isVisible = true;
                    widget.getField('IDCHECK').isVisible = false;
                }
                else {
                    widget.widgets[0].isVisible = false;
                    widget.getField('IDCHECK').isVisible = true;
                }
                var bloqueia = false;
                widget.filtrada = true;
            }
            else {
                setTimeout(function() {
                    ScreenService.showMessage('Unidade selecionada não utiliza controle de almoxarifado.');
                }, 3000);
            }
        }, function(error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    this.controlaAlmoxarifado = function(widget) {
        var row = widget.currentRow;
        this.retornaParamUnidade(row.CDFILIAL).then(function(paramFilial) {
            var idutilalmox = paramFilial.dataset.paramfilial[0].IDUTILALMOX;
            if (idutilalmox != 'S') {
                ScreenService.showMessage('Unidade selecionada não utiliza controle de almoxarifado.');
                widget.currentRow.CDFILIAL = '';
                widget.currentRow.NMFILIAL = '';
            }
        });
    };

    //AfterMoveRow
    this.setDataSourceFilterLocal = function(widget) {
        var widgetLocal = widget.widgets[0];
        if (widgetLocal.isVisible === true) {
            var row = widget.currentRow;
            widgetLocal.dataSourceFilter[0].value = row.CDFILIAL;
            widgetLocal.dataSourceFilter[1].value = row.CDALMOXARIFE;
            widgetLocal.reload();
            cdalmoxarifeGeral = row.CDALMOXARIFE;
        }
    };

    //Editar e cancelar da widget principal
    this.setEnabled = function(widget, enabled) {
        if (widget.widgets[0]) {
            widget.widgets[0].enabled = enabled;
        }
    };

    this.deleteAlmoxarifado = function(widget) {
        ScreenService.confirmMessage(ScreenService.i18n('Deseja excluir o almoxarifado e todas as suas localizações?'), 'question', function(success) {
            widget.removeRow(widget.currentRow).then(function(success) {
                ScreenService.closeSwipe();
                widget.reload();
            }, function(error) {
                if (error.data.error) {
                    ScreenService.showMessage("Não é possível excluir. Este almoxarifado está relacionado a outras operações!");
                }
                else {
                    ScreenService.showMessage(error.data);
                }
            });
        });
    };

    this.newAlmoxarifado = function(widget) {
        if (cdfilialGeral === undefined)
            cdfilialGeral = widget.currentRow.CDFILIAL;
        if (cdfilialGeral && widget.filtrada != false) {
            widget.newRow();
            widget.currentRow.CDFILIAL = cdfilialGeral;
            this.setEnabled(widget, false);
        }
        else {
            ScreenService.showMessage("Selecione uma unidade no filtro antes de adicionar.");
        }

    };

    this.newLocalizacao = function(widget) {
        if (cdfilialGeral && widget.filtrada != false) {
            widget.newRow();
            widget.currentRow.CDFILIAL = cdfilialGeral;
            widget.currentRow.CDALMOXARIFE = cdalmoxarifeGeral;
            this.setEnabled(widget, false);
        }
        else {
            ScreenService.showMessage("Selecione uma unidade no filtro antes de adicionar.");
        }
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Ger03700_AlmoxarifadoLocalicazao_Controller', Ger03700_AlmoxarifadoLocalicazao_Controller);
});


// FILE: ./assets/js/controllers/Ger27000_LocalEstoqueProduto_Controller.js
var Ger27000_LocalEstoqueProduto_Controller = function(ScreenService, EstUtil, templateManager, General,restEngine , Ger27000_LocalEstoqueProduto_Service, ZhFilterPreferences, ZhReports, MessageService) {
	//boolean para o bloqueio da edição de locais
	var fullLocalLock = false;
	var fullLocalLockCad = false;
	//boolean para o bloqueio da edição de almoxarifados
	var fullAlmoxLock = false;
	var fullAlmoxLockCad = false;
	//variavel que guarda o ultimo codigo de filial que passou pela função validadeFilialController
	var previousFilial;

	var filialPad;
	var nmFilial;
	var cdProd;
	var nmProd;

	this.openFilter = function(widget) {
		this.setFieldsGerais(widget);
		var rowsToSet = {};
		rowsToSet.defaultFilter = widget.dataSourceFilter;
		rowsToSet.CDFILIAL = getLocalVar("CDFILIAL");
		rowsToSet.NMFILIAL = getLocalVar("NMFILIAL");
		rowsToSet.P_CDPRODINI = null;
		rowsToSet.P_CDPRODFIN = null;
		
		//ZhFilterPreferences.setOriginalFilter(widget.id, rowsToSet);
		ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(opened => {
			var filtro = widget.container.getActiveWidget();
			this.filiProd(filtro);
		});
	};

	this.setFieldsGerais = function(widget) {
		templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
	};

	this.getFilial = function() {
		return filialPad;
	};

	//função que checa se a edição de locais e/ou almoxarifados estão bloqueados na filial
	this.validateFilialController = function(widget) {
		var filial;
		var filter;
		if (widget.name !== 'CADASTROAUTOMATICO') {
			if (widget.dataSourceFilter[1]) {
				filial = widget.dataSourceFilter[1].value;
				filter = [{
					"name": "CDFILIAL",
					"operator": "=",
					"value": filial
				}];
			}
		}
		else {
			filial = widget.currentRow.CDFILIAL;
			filter = [{
				"name": "CDFILIAL",
				"operator": "=",
				"value": filial
			}];
		}

		//chamando o backend a partir do service e do repository
		Ger27000_LocalEstoqueProduto_Service.validateFilialService(filter).then(function(success) {
			if (success.dataset.paramfilial[0]) {
				if (success.dataset.paramfilial[0].IDUTILALMOX == 'S') {
					if (widget.name === 'CADASTROAUTOMATICO') {
						fullAlmoxLockCad = false;
					}
					else {
						fullAlmoxLock = false;
					}
					this.almoxLocker(widget);
					if (success.dataset.paramfilial[0].IDUTILLCESTQ == 'S') {
						if (widget.name === 'CADASTROAUTOMATICO') {
							fullAlmoxLockCad = false;
						}
						else {
							fullLocalLock = false;
						}
					}
					else {
						//retorna uma mensagen somente se esta filial for diferente da ultima filial carregada
						if (previousFilial != success.dataset.paramfilial[0].CDFILIAL && previousFilial !== undefined) {
							Ger27000_LocalEstoqueProduto_Service.limpaFieldsCADASTROAUTOMATICO(widget);
							fullLocalLock = true;
							fullLocalLockCad = true;
							this.almoxLocker(widget);
							ScreenService.showMessage("Atenção: Esta Filial está com a edição de Locais bloqueada.");
						}
						if (widget.name === 'CADASTROAUTOMATICO') {
							fullLocalLockCad = true;
						}
						else {
							fullLocalLock = true;
						}
					}
				}
				else {
					//retorna uma mensagen somente se esta filial for diferente da ultima filial carregada
					if (previousFilial != success.dataset.paramfilial[0].CDFILIAL) {
						Ger27000_LocalEstoqueProduto_Service.limpaFieldsCADASTROAUTOMATICO(widget);
						fullLocalLock = true;
						fullLocalLockCad = true;
						this.almoxLocker(widget);
						ScreenService.showMessage("Atenção: Esta Filial está com a edição de Almoxarifados bloqueada.");
					}
					if (widget.name === 'CADASTROAUTOMATICO') {
						fullAlmoxLockCad = true;
					}
					else {
						fullAlmoxLock = true;
					}
					this.almoxLocker(widget);
				}
				previousFilial = success.dataset.paramfilial[0].CDFILIAL;
			}
		}.bind(this));
	};

	this.setFormCurrentRow = function(widget) {
		widget.currentRow = widget.dataSource.data[0];
		this.setColorLegenda(widget.getParent());
	};

	var getZ = function() {
		var numFormatado = "";
		var num = getLocalVar("CDPICTPROD");
		var zAux = "z";
		num = num.replace("*", "");
		// TODO: trocar for por replace. (replace não funcionou com ponto final (.).
		for (var i = 0; i < num.length; i++) {
			if (num[i] != ".") {
				numFormatado += num[i];
			}
		}
		return zAux.repeat(numFormatado.length);
	};

	this.ajustaFiltro = function(args) {
		var widget = args.owner;
		var filtros = args.e.preference.FILTER_ROW;
		if (filtros.P_CDPRODINI && filtros.P_CDPRODFIN) {
			widget.dataSourceFilter.forEach(function(data) {
				if (data.name == "P_CDPRODINI") {
					data.value = filtros.P_CDPRODINI;
				}
				else if (data.name == "P_CDPRODFIN") {
					data.value = filtros.P_CDPRODFIN + getZ();
				}

			});
		}
	};

	//função responsável por bloquear e desbloquear a edição de almoxarifados
	this.almoxLocker = function(widgetPai) {
		//widgetPai.widgets.forEach(widgetPai => {
		//	widFilha.getField('')
		if (widgetPai.name !== 'CADASTROAUTOMATICO' || widgetPai.appliedFilter) {
			var currentRowPopup = widgetPai.appliedFilter.FILTER_ROW;
			var widgetControle = templateManager.container.getWidget('CONTROLEPRODUCAO');
			var widgetPedidos = templateManager.container.getWidget('PEDIDOSEXPEDICAO');
			var widgetControleQualidade = templateManager.container.getWidget('CONTROLEQUALIDADE');
			var widgetGestaoVendas = templateManager.container.getWidget('GESTAODEVENDAS');
			var widgetProcessaProduto = widget.container.getWidget('PROCESSAPRODUTO');
			widgetControle.getField('DSALMOXINSPR').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControle.getField('DSALMOXPRSEMACA').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControle.getField('DSALMOXINSPRPIL').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControle.getField('DSALPROD').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControle.getField('DSALPRODPIL').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControle.getField('DSALMOXPRACA').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetPedidos.getField('DSALMOXPRCLI').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControleQualidade.getField('DSALMOXARIFE').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControleQualidade.getField('DSALMOXQUAMPPIL').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControleQualidade.getField('DSALMOXQUAR').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControleQualidade.getField('DSALMOXREPROV').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetGestaoVendas.getField('DSALMOXCONSVND').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetGestaoVendas.getField('DSLOCALCONSVND').dataSourceFilter[0].value = currentRowPopup.CDFILIAL;
			widgetControle.getField('DSALMOXINSPR').reload();
			widgetControle.getField('DSALMOXPRSEMACA').reload();
			widgetControle.getField('DSALMOXINSPRPIL').reload();
			widgetControle.getField('DSALPROD').reload();
			widgetControle.getField('DSALPRODPIL').reload();
			widgetControle.getField('DSALMOXPRACA').reload();
			widgetPedidos.getField('DSALMOXPRCLI').reload();
			widgetControleQualidade.getField('DSALMOXARIFE').reload();
			widgetControleQualidade.getField('DSALMOXQUAMPPIL').reload();
			widgetControleQualidade.getField('DSALMOXQUAR').reload();
			widgetControleQualidade.getField('DSALMOXREPROV').reload();
			widgetProcessaProduto.getField('DSALMOXCONSVND').reload();
			widgetProcessaProduto.getField('DSLOCALCONSVND').reload();
			if (fullAlmoxLock) {
				//1
				widgetControle.currentRow.DSALMOXINSPR = '';
				widgetControle.getField('DSALMOXINSPR').readOnly = true;
				widgetControle.getField('DSALMOXINSPR').readOnlyOnEdit = true;
				widgetControle.getField('DSLOCALINSPR').readOnly = true;
				widgetControle.getField('DSLOCALINSPR').readOnlyOnEdit = true;
				//2
				widgetControle.currentRow.DSALMOXPRSEMACA = '';
				widgetControle.getField('DSALMOXPRSEMACA').readOnly = true;
				widgetControle.getField('DSALMOXPRSEMACA').readOnlyOnEdit = true;
				widgetControle.getField('DSLOCALPRSEMACA').readOnly = true;
				widgetControle.getField('DSLOCALPRSEMACA').readOnlyOnEdit = true;
				//3
				widgetControle.currentRow.DSALMOXINSPRPIL = '';
				widgetControle.getField('DSALMOXINSPRPIL').readOnly = true;
				widgetControle.getField('DSALMOXINSPRPIL').readOnlyOnEdit = true;
				widgetControle.getField('DSLOCALINSPRPIL').readOnly = true;
				widgetControle.getField('DSLOCALINSPRPIL').readOnlyOnEdit = true;
				//4
				widgetControle.currentRow.DSALPROD = '';
				widgetControle.getField('DSALPROD').readOnly = true;
				widgetControle.getField('DSALPROD').readOnlyOnEdit = true;
				widgetControle.getField('DSLOCPROD').readOnly = true;
				widgetControle.getField('DSLOCPROD').readOnlyOnEdit = true;
				//5
				widgetControle.currentRow.DSALPRODPIL = '';
				widgetControle.getField('DSALPRODPIL').readOnly = true;
				widgetControle.getField('DSALPRODPIL').readOnlyOnEdit = true;
				widgetControle.getField('DSLOCPRODPRIL').readOnly = true;
				widgetControle.getField('DSLOCPRODPRIL').readOnlyOnEdit = true;
				//6
				widgetControle.currentRow.DSALMOXPRACA = '';
				widgetControle.getField('DSALMOXPRACA').readOnly = true;
				widgetControle.getField('DSALMOXPRACA').readOnlyOnEdit = true;
				widgetControle.getField('DSLOCALPRACA').readOnly = true;
				widgetControle.getField('DSLOCALPRACA').readOnlyOnEdit = true;
				//7
				widgetPedidos.currentRow.DSALMOXPRCLI = '';
				widgetPedidos.getField('DSALMOXPRCLI').readOnly = true;
				widgetPedidos.getField('DSALMOXPRCLI').readOnlyOnEdit = true;
				widgetPedidos.getField('DSLOCALPRCLI').readOnly = true;
				widgetPedidos.getField('DSLOCALPRCLI').readOnlyOnEdit = true;
				//8
				widgetControleQualidade.currentRow.DSALMOXARIFE = '';
				widgetControleQualidade.getField('DSALMOXARIFE').readOnly = true;
				widgetControleQualidade.getField('DSALMOXARIFE').readOnlyOnEdit = true;
				widgetControleQualidade.getField('DSLOCALESTOQ').readOnly = true;
				widgetControleQualidade.getField('DSLOCALESTOQ').readOnlyOnEdit = true;
				//9
				widgetControleQualidade.currentRow.DSALMOXQUAMPPIL = '';
				widgetControleQualidade.getField('DSALMOXQUAMPPIL').readOnly = true;
				widgetControleQualidade.getField('DSALMOXQUAMPPIL').readOnlyOnEdit = true;
				widgetControleQualidade.getField('DSLOCALQUAMPPIL').readOnly = true;
				widgetControleQualidade.getField('DSLOCALQUAMPPIL').readOnlyOnEdit = true;
				//10
				widgetControleQualidade.currentRow.DSALMOXQUAR = '';
				widgetControleQualidade.getField('DSALMOXQUAR').readOnly = true;
				widgetControleQualidade.getField('DSALMOXQUAR').readOnlyOnEdit = true;
				widgetControleQualidade.getField('DSLOCALQUAR').readOnly = true;
				widgetControleQualidade.getField('DSLOCALQUAR').readOnlyOnEdit = true;
				//11
				widgetControleQualidade.currentRow.DSALMOXREPROV = '';
				widgetControleQualidade.getField('DSALMOXREPROV').readOnly = true;
				widgetControleQualidade.getField('DSALMOXREPROV').readOnlyOnEdit = true;
				widgetControleQualidade.getField('DSLOCALREPROV').readOnly = true;
				widgetControleQualidade.getField('DSLOCALREPROV').readOnlyOnEdit = true;
				//12
				widgetGestaoVendas.currentRow.DSALMOXCONSVND = '';
				widgetGestaoVendas.getField('DSALMOXCONSVND').readOnly = true;
				widgetGestaoVendas.getField('DSALMOXCONSVND').readOnlyOnEdit = true;
				widgetGestaoVendas.getField('DSLOCALCONSVND').readOnly = true;
				widgetGestaoVendas.getField('DSLOCALCONSVND').readOnlyOnEdit = true;
				//13
				widgetProcessaProduto.currentRow.DSMOXPRODSERPROC = '';
				widgetProcessaProduto.currentRow.DSLOCPRODSERPROC = '';
				widgetProcessaProduto.getField('DSMOXPRODSERPROC').readOnly = true;
				widgetProcessaProduto.getField('DSMOXPRODSERPROC').readOnlyOnEdit = true;
				widgetProcessaProduto.getField('DSLOCPRODSERPROC').readOnly = true;
				widgetProcessaProduto.getField('DSLOCPRODSERPROC').readOnlyOnEdit = true;
				//14
				widgetProcessaProduto.currentRow.DSALMOXPRODPROC = '';
				widgetProcessaProduto.currentRow.DSLOCALPRODPROC = '';
				widgetProcessaProduto.getField('DSALMOXPRODPROC').readOnly = true;
				widgetProcessaProduto.getField('DSALMOXPRODPROC').readOnlyOnEdit = true;
				widgetProcessaProduto.getField('DSLOCALPRODPROC').readOnly = true;
				widgetProcessaProduto.getField('DSLOCALPRODPROC').readOnlyOnEdit = true;
			}
			else {
				//1
				widgetControle.currentRow.DSALMOXINSPR = '';
				widgetControle.getField('DSALMOXINSPR').readOnly = false;
				widgetControle.getField('DSALMOXINSPR').readOnlyOnEdit = false;
				//2
				widgetControle.currentRow.DSALMOXPRSEMACA = '';
				widgetControle.getField('DSALMOXPRSEMACA').readOnly = false;
				widgetControle.getField('DSALMOXPRSEMACA').readOnlyOnEdit = false;
				//3
				widgetControle.currentRow.LMOXINSPRPIL = '';
				widgetControle.getField('DSALMOXINSPRPIL').readOnly = false;
				widgetControle.getField('DSALMOXINSPRPIL').readOnlyOnEdit = false;
				//4
				widgetControle.currentRow.DSALPROD = '';
				widgetControle.getField('DSALPROD').readOnly = false;
				widgetControle.getField('DSALPROD').readOnlyOnEdit = false;
				//5
				widgetControle.currentRow.DSALPRODPIL = '';
				widgetControle.getField('DSALPRODPIL').readOnly = false;
				widgetControle.getField('DSALPRODPIL').readOnlyOnEdit = false;
				//6
				widgetControle.currentRow.DSALMOXPRACA = '';
				widgetControle.getField('DSALMOXPRACA').readOnly = false;
				widgetControle.getField('DSALMOXPRACA').readOnlyOnEdit = false;
				//7
				widgetPedidos.currentRow.DSALMOXPRCLI = '';
				widgetPedidos.getField('DSALMOXPRCLI').readOnly = false;
				widgetPedidos.getField('DSALMOXPRCLI').readOnlyOnEdit = false;
				//8
				widgetControleQualidade.currentRow.DSALMOXARIFE = '';
				widgetControleQualidade.getField('DSALMOXARIFE').readOnly = false;
				widgetControleQualidade.getField('DSALMOXARIFE').readOnlyOnEdit = false;
				//9
				widgetControleQualidade.currentRow.DSALMOXQUAMPPIL = '';
				widgetControleQualidade.getField('DSALMOXQUAMPPIL').readOnly = false;
				widgetControleQualidade.getField('DSALMOXQUAMPPIL').readOnlyOnEdit = false;
				//10
				widgetControleQualidade.currentRow.DSALMOXQUAR = '';
				widgetControleQualidade.getField('DSALMOXQUAR').readOnly = false;
				widgetControleQualidade.getField('DSALMOXQUAR').readOnlyOnEdit = false;
				//11
				widgetControleQualidade.currentRow.DSALMOXREPROV = '';
				widgetControleQualidade.getField('DSALMOXREPROV').readOnly = false;
				widgetControleQualidade.getField('DSALMOXREPROV').readOnlyOnEdit = false;
				//12
				widgetGestaoVendas.currentRow.DSALMOXCONSVND = '';
				widgetGestaoVendas.getField('DSALMOXCONSVND').readOnly = false;
				widgetGestaoVendas.getField('DSALMOXCONSVND').readOnlyOnEdit = false;
				//13
				widgetProcessaProduto.currentRow.DSMOXPRODSERPROC = '';
				widgetProcessaProduto.getField('DSMOXPRODSERPROC').readOnly = false;
				widgetProcessaProduto.getField('DSMOXPRODSERPROC').readOnlyOnEdit = false;
				//14
				widgetProcessaProduto.currentRow.DSALMOXPRODPROC = '';
				widgetProcessaProduto.getField('DSALMOXPRODPROC').readOnly = false;
				widgetProcessaProduto.getField('DSALMOXPRODPROC').readOnlyOnEdit = false;
			}
			//});
		}
		else {
				widgetPai.getField('DSALMOXINSPR').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXPRSEMACA').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXINSPRPIL').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALPROD').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALPRODPIL').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXPRACA').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXPRCLI').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXARIFE').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXQUAMPPIL').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXQUAR').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXREPROV').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXCONSVND').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSLOCALCONSVND').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSMOXPRODSERPROC').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSLOCPRODSERPROC').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXPRODPROC').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSLOCALPRODPROC').dataSourceFilter[0].value = widgetPai.currentRow.CDFILIAL;
				widgetPai.getField('DSALMOXINSPR').reload();
				widgetPai.getField('DSALMOXPRSEMACA').reload();
				widgetPai.getField('DSALMOXINSPRPIL').reload();
				widgetPai.getField('DSALPROD').reload();
				widgetPai.getField('DSALPRODPIL').reload();
				widgetPai.getField('DSALMOXPRACA').reload();
				widgetPai.getField('DSALMOXPRCLI').reload();
				widgetPai.getField('DSALMOXARIFE').reload();
				widgetPai.getField('DSALMOXQUAMPPIL').reload();
				widgetPai.getField('DSALMOXQUAR').reload();
				widgetPai.getField('DSALMOXREPROV').reload();
				widgetPai.getField('DSALMOXCONSVND').reload();
				widgetPai.getField('DSLOCALCONSVND').reload();
				widgetPai.getField('DSMOXPRODSERPROC').reload();
				widgetPai.getField('DSLOCPRODSERPROC').reload();
				widgetPai.getField('DSALMOXPRODPROC').reload();
				widgetPai.getField('DSLOCALPRODPROC').reload();
	
				if (fullAlmoxLockCad) {
					//1
					widgetPai.currentRow.DSALMOXINSPR = '';
					widgetPai.getField('DSALMOXINSPR').readOnly = true;
					widgetPai.getField('DSALMOXINSPR').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALINSPR').readOnly = true;
					widgetPai.getField('DSLOCALINSPR').readOnlyOnEdit = true;
					//2
					widgetPai.currentRow.DSALMOXPRSEMACA = '';
					widgetPai.getField('DSALMOXPRSEMACA').readOnly = true;
					widgetPai.getField('DSALMOXPRSEMACA').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALPRSEMACA').readOnly = true;
					widgetPai.getField('DSLOCALPRSEMACA').readOnlyOnEdit = true;
					//3
					widgetPai.currentRow.DSALMOXINSPRPIL = '';
					widgetPai.getField('DSALMOXINSPRPIL').readOnly = true;
					widgetPai.getField('DSALMOXINSPRPIL').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALINSPRPIL').readOnly = true;
					widgetPai.getField('DSLOCALINSPRPIL').readOnlyOnEdit = true;
					//4
					widgetPai.currentRow.DSALPROD = '';
					widgetPai.getField('DSALPROD').readOnly = true;
					widgetPai.getField('DSALPROD').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCPROD').readOnly = true;
					widgetPai.getField('DSLOCPROD').readOnlyOnEdit = true;
					//5
					widgetPai.currentRow.DSALPRODPIL = '';
					widgetPai.getField('DSALPRODPIL').readOnly = true;
					widgetPai.getField('DSALPRODPIL').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCPRODPRIL').readOnly = true;
					widgetPai.getField('DSLOCPRODPRIL').readOnlyOnEdit = true;
					//6
					widgetPai.currentRow.DSALMOXPRACA = '';
					widgetPai.getField('DSALMOXPRACA').readOnly = true;
					widgetPai.getField('DSALMOXPRACA').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALPRACA').readOnly = true;
					widgetPai.getField('DSLOCALPRACA').readOnlyOnEdit = true;
					//7
					widgetPai.currentRow.DSALMOXPRCLI = '';
					widgetPai.getField('DSALMOXPRCLI').readOnly = true;
					widgetPai.getField('DSALMOXPRCLI').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALPRCLI').readOnly = true;
					widgetPai.getField('DSLOCALPRCLI').readOnlyOnEdit = true;
					//8
					widgetPai.currentRow.DSALMOXARIFE = '';
					widgetPai.getField('DSALMOXARIFE').readOnly = true;
					widgetPai.getField('DSALMOXARIFE').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALESTOQ').readOnly = true;
					widgetPai.getField('DSLOCALESTOQ').readOnlyOnEdit = true;
					//9
					widgetPai.currentRow.DSALMOXQUAMPPIL = '';
					widgetPai.getField('DSALMOXQUAMPPIL').readOnly = true;
					widgetPai.getField('DSALMOXQUAMPPIL').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALQUAMPPIL').readOnly = true;
					widgetPai.getField('DSLOCALQUAMPPIL').readOnlyOnEdit = true;
					//10
					widgetPai.currentRow.DSALMOXQUAR = '';
					widgetPai.getField('DSALMOXQUAR').readOnly = true;
					widgetPai.getField('DSALMOXQUAR').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALQUAR').readOnly = true;
					widgetPai.getField('DSLOCALQUAR').readOnlyOnEdit = true;
					//11
					widgetPai.currentRow.DSALMOXREPROV = '';
					widgetPai.getField('DSALMOXREPROV').readOnly = true;
					widgetPai.getField('DSALMOXREPROV').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALREPROV').readOnly = true;
					widgetPai.getField('DSLOCALREPROV').readOnlyOnEdit = true;
					//12
					widgetPai.currentRow.DSALMOXCONSVND = '';
					widgetPai.getField('DSALMOXCONSVND').readOnly = true;
					widgetPai.getField('DSALMOXCONSVND').readOnlyOnEdit = true;
					widgetPai.getField('DSLOCALCONSVND').readOnly = true;
					widgetPai.getField('DSLOCALCONSVND').readOnlyOnEdit = true;
					//13
					widgetPai.currentRow.DSMOXPRODSERPROC = '';
					widgetPai.getField('DSMOXPRODSERPROC').readOnly = true;
					widgetPai.getField('DSMOXPRODSERPROC').readOnlyOnEdit = true;
					widgetPai.currentRow.DSLOCPRODSERPROC = '';
					widgetPai.getField('DSLOCPRODSERPROC').readOnly = true;
					widgetPai.getField('DSLOCPRODSERPROC').readOnlyOnEdit = true;
					//14
					widgetPai.currentRow.DSALMOXPRODPROC = '';
					widgetPai.getField('DSALMOXPRODPROC').readOnly = true;
					widgetPai.getField('DSALMOXPRODPROC').readOnlyOnEdit = true;
					widgetPai.currentRow.DSLOCALPRODPROC = '';
					widgetPai.getField('DSLOCALPRODPROC').readOnly = true;
					widgetPai.getField('DSLOCALPRODPROC').readOnlyOnEdit = true;
				}
				else {
					//1
					widgetPai.currentRow.DSALMOXINSPR = '';
					widgetPai.getField('DSALMOXINSPR').readOnly = false;
					widgetPai.getField('DSALMOXINSPR').readOnlyOnEdit = false;
					//2
					widgetPai.currentRow.DSALMOXPRSEMACA = '';
					widgetPai.getField('DSALMOXPRSEMACA').readOnly = false;
					widgetPai.getField('DSALMOXPRSEMACA').readOnlyOnEdit = false;
					//3
					widgetPai.currentRow.DSALMOXINSPRPIL = '';
					widgetPai.getField('DSALMOXINSPRPIL').readOnly = false;
					widgetPai.getField('DSALMOXINSPRPIL').readOnlyOnEdit = false;
					//4
					widgetPai.currentRow.DSALPROD = '';
					widgetPai.getField('DSALPROD').readOnly = false;
					widgetPai.getField('DSALPROD').readOnlyOnEdit = false;
					//5
					widgetPai.currentRow.DSALPRODPIL = '';
					widgetPai.getField('DSALPRODPIL').readOnly = false;
					widgetPai.getField('DSALPRODPIL').readOnlyOnEdit = false;
					//6
					widgetPai.currentRow.DSALMOXPRACA = '';
					widgetPai.getField('DSALMOXPRACA').readOnly = false;
					widgetPai.getField('DSALMOXPRACA').readOnlyOnEdit = false;
					//7
					widgetPai.currentRow.DSALMOXPRCLI = '';
					widgetPai.getField('DSALMOXPRCLI').readOnly = false;
					widgetPai.getField('DSALMOXPRCLI').readOnlyOnEdit = false;
					//8
					widgetPai.currentRow.DSALMOXARIFE = '';
					widgetPai.getField('DSALMOXARIFE').readOnly = false;
					widgetPai.getField('DSALMOXARIFE').readOnlyOnEdit = false;
					//9
					widgetPai.currentRow.DSALMOXQUAMPPIL = '';
					widgetPai.getField('DSALMOXQUAMPPIL').readOnly = false;
					widgetPai.getField('DSALMOXQUAMPPIL').readOnlyOnEdit = false;
					//10
					widgetPai.currentRow.DSALMOXQUAR = '';
					widgetPai.getField('DSALMOXQUAR').readOnly = false;
					widgetPai.getField('DSALMOXQUAR').readOnlyOnEdit = false;
					//11
					widgetPai.currentRow.DSALMOXREPROV = '';
					widgetPai.getField('DSALMOXREPROV').readOnly = false;
					widgetPai.getField('DSALMOXREPROV').readOnlyOnEdit = false;
					//12
					widgetPai.currentRow.DSALMOXCONSVND = '';
					widgetPai.getField('DSALMOXCONSVND').readOnly = false;
					widgetPai.getField('DSALMOXCONSVND').readOnlyOnEdit = false;
					//13
					widgetPai.currentRow.DSMOXPRODSERPROC = '';
					widgetPai.getField('DSMOXPRODSERPROC').readOnly = false;
					widgetPai.getField('DSMOXPRODSERPROC').readOnlyOnEdit = false;
					//14
					widgetPai.currentRow.DSALMOXPRODPROC = '';
					widgetPai.getField('DSALMOXPRODPROC').readOnly = false;
					widgetPai.getField('DSALMOXPRODPROC').readOnlyOnEdit = false;
				}
			
		}


		//	

	};

	//função responsável por limitar a seleção do produto inicial a partir do filial
	this.filiProd = function(widget) {
		var row = widget.currentRow;
		var filialIsSet = this.fieldIsSet(row.CDFILIAL);
		if (filialIsSet && widget.name === 'CADASTROAUTOMATICO') {
			this.validateFilialController(widget);
		}
		if (!filialIsSet) {
			Ger27000_LocalEstoqueProduto_Service.limpaFieldsCADASTROAUTOMATICO(widget);
			fullAlmoxLockCad = true;
			fullLocalLockCad = true;
			this.almoxLocker(widget);
			widget.currentRow.NMPRODINI = '';
			widget.currentRow.NMPRODFIN = '';
			widget.currentRow.P_CDPRODINI = '';
			widget.currentRow.P_CDPRODFIN = '';
			delete widget.getField('NMPRODINI').validations.required;
			delete widget.getField('NMPRODFIN').validations.required;
			widget.getField('NMPRODINI').readOnly = true;
			widget.getField('NMPRODINI').readOnlyOnEdit = true;
			widget.getField('NMPRODFIN').readOnly = true;
			widget.getField('NMPRODFIN').readOnlyOnEdit = true;
		} 
		if (row.NMFILIAL && (row.NMFILIAL != "")) {
			widget.getField('NMPRODINI').dataSourceFilter.forEach(function(fil) {
				if (fil.name === 'CDFILIAL') {
					fil.value = widget.currentRow.CDFILIAL;
				}
			});
			widget.getField('NMPRODFIN').dataSourceFilter.forEach(function(fil) {
				if (fil.name === 'CDFILIAL') {
					fil.value = widget.currentRow.CDFILIAL;
				}
			});
		}
		widget.getField('NMPRODINI').reload();
		widget.getField('NMPRODFIN').reload();
		if(widget.name === 'CADASTROAUTOMATICO') {
			if(filialIsSet) {
				widget.getField('NMPRODINI').validations.required = true;
				widget.getField('NMPRODINI').readOnly = false;
				widget.getField('NMPRODINI').readOnlyOnEdit = false;
				delete widget.getField('NMPRODFIN').validations.required;
				widget.getField('NMPRODFIN').readOnly = true;
				widget.getField('NMPRODFIN').readOnlyOnEdit = true;
			} else {
				delete widget.getField('NMPRODINI').validations.required;
				widget.getField('NMPRODINI').readOnly = true;
				widget.getField('NMPRODINI').readOnlyOnEdit = true;
				delete widget.getField('NMPRODFIN').validations.required;
				widget.getField('NMPRODFIN').readOnly = true;
				widget.getField('NMPRODFIN').readOnlyOnEdit = true;
			}
		} else {
			widget.getField('NMPRODINI').readOnly = false;
			widget.getField('NMPRODINI').readOnlyOnEdit = false;
		}
		widget.getField('NMPRODINI').reload();
		widget.getField('NMPRODFIN').reload();
	};

	this.FilterProdIni = function(args) {
		var widget = args.owner.field.widget;
		var row = widget.currentRow;
		var prodIniIsSet = this.fieldIsSet(row.CDPRODINI);
		if (!prodIniIsSet) {
			widget.currentRow.NMPRODFIN = '';
			widget.currentRow.P_CDPRODFIN = '';
			if (widget.name === 'widgetProduto' || widget.name === 'zhFilterPreferencesSelectConditions') {
				delete widget.getField('NMPRODINI').validations.required;
				delete widget.getField('NMPRODFIN').validations.required;
			}
		}
		else if (widget.name === 'widgetProduto' || widget.name === 'zhFilterPreferencesSelectConditions') {
			widget.getField('NMPRODFIN').validations.required = true;
		}
		
		widget.getField('NMPRODFIN').readOnly = !prodIniIsSet;
		widget.getField('NMPRODFIN').readOnlyOnEdit = !prodIniIsSet;

		if (!prodIniIsSet)
			delete widget.getField('NMPRODFIN').validations.required;
		else
			widget.getField('NMPRODFIN').validations.required = prodIniIsSet;
	};

	this.fieldIsSet = function(fieldValue) {
		if (fieldValue) return true;
		else return false;
	};

	this.FilterProdFin = function(args) {
		var widget = args.owner.field.widget;
		var row = widget.currentRow;
		var prodFinIsSet = this.fieldIsSet(row.P_CDPRODFIN);
		var prodIniIsSet = this.fieldIsSet(row.P_CDPRODINI);
		if (prodFinIsSet) {
			if (widget.name === 'widgetProduto' || widget.name === 'zhFilterPreferencesSelectConditions') {
				widget.getField('NMPRODINI').validations.required = true;
			}
			if (row.P_CDPRODINI > row.P_PRODFIN) {
				ScreenService.showMessage("O codigo do produto inicial não pode ser maior que do final");
				return 0;
			}
		}
		else if (widget.name === 'widgetProduto' || widget.name === 'zhFilterPreferencesSelectConditions') {
			delete widget.getField('NMPRODINI').validations.required;
			if (!prodIniIsSet) {
				delete widget.getField('NMPRODFIN').validations.required;
			}
		}
		widget.reload();
	};

	this.setColorLegenda = function(widget) {
		var widgetControle = templateManager.container.getWidget('TABBEDCONTROLEPRODUCAO');
		var widgetPedidos = templateManager.container.getWidget('TABBEDPEDIDOS');
		var widgetControleQualidade = templateManager.container.getWidget('TABBEDCONTROLEQUALIDADE');
		var widgetGestaoVendas = templateManager.container.getWidget('TABBEDGESTAODEVENDAS');
		var widgetProcessaProduto = templateManager.container.getWidget('TABBEDPROCESSAPRODUTO');
		var obj;
		// Campos Legenda Controle de produção
		if (widget.name === 'TABBEDCONTROLEPRODUCAO') {
			var rowControle = {
				LEGALMOXMATP: tinycolor('#007f80'),
				LEGALMOXPRODS: tinycolor('#007f80'),
				LEGALMOXMATPP1: tinycolor('#800000'),
				LEGALMOXMATPP2: tinycolor('#007f80'),
				LEGALMOXMATPI: tinycolor('#800000'),
				LEGALMOXMATPPI1: tinycolor('#800000'),
				LEGALMOXMATPPI2: tinycolor('#808000'),
				LEGALMOXPRODA: tinycolor('#000080')
			};
			if (widgetControle.widgets[0].currentRow) {
				obj = Object.assign(widgetControle.widgets[0].currentRow, rowControle);
				widgetControle.widgets[0].currentRow = obj;
			}
			else {
				widgetControle.widgets[0].currentRow = rowControle;
			}
		}
		else if (widget.name === 'TABBEDPEDIDOS') {
			var rowPedidos = {
				LEGALMOXEXPRE: tinycolor('#000080')
			};
			if (widgetPedidos.widgets[0].currentRow) {
				obj = Object.assign(widgetPedidos.widgets[0].currentRow, rowPedidos);
				widgetPedidos.widgets[0].currentRow = obj;
			}
			else {
				widgetPedidos.widgets[0].currentRow = rowPedidos;
			}
		}

		else if (widget.name === 'TABBEDCONTROLEQUALIDADE') {
			var rowControleQualidade = {
				LEGALMOXQMI: tinycolor('#99B4D1'),
				LEGALMOXQMP1: tinycolor('#800000'),
				LEGALMOXQMP2: tinycolor('#99B4D1'),
				LEGALMOXQPA: tinycolor('#99B4D1'),
				LEGALMOXREPROV: tinycolor('#FF0000')
			};
			if (widgetControleQualidade.widgets[0].currentRow) {
				obj = Object.assign(widgetControleQualidade.widgets[0].currentRow, rowControleQualidade);
				widgetControleQualidade.widgets[0].currentRow = obj;
			}
			else {
				widgetControleQualidade.widgets[0].currentRow = rowControleQualidade;
			}
		}
		else if (widget.name === 'TABBEDGESTAODEVENDAS'){
			var rowGestaoVendas = {};
			if (widgetGestaoVendas.widgets[0].currentRow) {
				obj = Object.assign(widgetGestaoVendas.widgets[0].currentRow, rowGestaoVendas);
				widgetGestaoVendas.widgets[0].currentRow = obj;
			}
			else {
				widgetGestaoVendas.widgets[0].currentRow = rowGestaoVendas;
			}
		}
		else if (widget.name === 'TABBEDPROCESSAPRODUTO'){
			var rowProcessaProuto = {};
			if (widgetProcessaProduto.widgets[0].currentRow) {
				obj = Object.assign(widgetProcessaProduto.widgets[0].currentRow, rowProcessaProuto);
				widgetProcessaProduto.widgets[0].currentRow = obj;
			}
			else {
				widgetProcessaProduto.widgets[0].currentRow = rowProcessaProuto;
			}
		}

		//Legenda padrão
		var rowLegenda = {
			VINHO: tinycolor('#800000'),
			VERDEAGUA: tinycolor('#008080'),
			AZULCLARO: tinycolor('#99B4D1'),
			BEGE: tinycolor('#808000'),
			AZULESCURO: tinycolor('#000080'),
			VERMELHO: tinycolor('#FF0000')
		};
		widgetControle.currentRow = rowLegenda;
		widgetPedidos.currentRow = rowLegenda;
		widgetControleQualidade.currentRow = rowLegenda;
		widgetGestaoVendas.currentRow = rowLegenda;
		widgetProcessaProduto.currentRow = rowLegenda;
	};

	//função default para retornar o valor de um campo
	this.setFieldsAddress = function(widgetName, fieldName) {
		return General.getFieldValue(fieldName, widgetName);
	};

	//função responsável por recarregar campos que sofreram mudanças
	this.reloadWidget = function(args, fieldName) {
		var field = args.owner.field.widget.getField(fieldName);
		field.reload();
	};

	/*função responsável por bloquear/desbloquear os campos de local caso o campo
	de almoxarifado correspondente esteja preenchido*/
	this.localLock = function(args, number) {
		var widget = args.owner.field.widget;
		var row;
		if (widget.name === 'CADASTROAUTOMATICO') {
			if (!fullLocalLockCad) {
				row = args.owner.field.widget.currentRow;

				switch (number) {
					case 1:
						args.owner.field.widget.currentRow.DSLOCALINSPR = '';
						args.owner.field.widget.getField('DSLOCALINSPR').readOnly = !row.DSALMOXINSPR;
						args.owner.field.widget.getField('DSLOCALINSPR').readOnlyOnEdit = !row.DSALMOXINSPR;
						break;
					case 2:
						args.owner.field.widget.currentRow.DSLOCALPRSEMACA = '';
						args.owner.field.widget.getField('DSLOCALPRSEMACA').readOnly = !row.DSALMOXPRSEMACA;
						args.owner.field.widget.getField('DSLOCALPRSEMACA').readOnlyOnEdit = !row.DSALMOXPRSEMACA;
						break;
					case 3:
						args.owner.field.widget.currentRow.DSLOCALINSPRPIL = '';
						args.owner.field.widget.getField('DSLOCALINSPRPIL').readOnly = !row.DSALMOXINSPRPIL;
						args.owner.field.widget.getField('DSLOCALINSPRPIL').readOnlyOnEdit = !row.DSALMOXINSPRPIL;
						break;
					case 4:
						args.owner.field.widget.currentRow.DSLOCPROD = '';
						args.owner.field.widget.getField('DSLOCPROD').readOnly = !row.DSALPROD;
						args.owner.field.widget.getField('DSLOCPROD').readOnlyOnEdit = !row.DSALPROD;
						break;
					case 5:
						args.owner.field.widget.currentRow.DSLOCPRODPRIL = '';
						args.owner.field.widget.getField('DSLOCPRODPRIL').readOnly = !row.DSALPRODPIL;
						args.owner.field.widget.getField('DSLOCPRODPRIL').readOnlyOnEdit = !row.DSALPRODPIL;
						break;
					case 6:
						args.owner.field.widget.currentRow.DSLOCALPRACA = '';
						args.owner.field.widget.getField('DSLOCALPRACA').readOnly = !row.DSALMOXPRACA;
						args.owner.field.widget.getField('DSLOCALPRACA').readOnlyOnEdit = !row.DSALMOXPRACA;
						break;
					case 7:
						args.owner.field.widget.currentRow.DSLOCALPRCLI = '';
						args.owner.field.widget.getField('DSLOCALPRCLI').readOnly = !row.DSALMOXPRCLI;
						args.owner.field.widget.getField('DSLOCALPRCLI').readOnlyOnEdit = !row.DSALMOXPRCLI;
						break;
					case 8:
						args.owner.field.widget.currentRow.DSLOCALESTOQ = '';
						args.owner.field.widget.getField('DSLOCALESTOQ').readOnly = !row.DSALMOXARIFE;
						args.owner.field.widget.getField('DSLOCALESTOQ').readOnlyOnEdit = !row.DSALMOXARIFE;
						break;
					case 9:
						args.owner.field.widget.currentRow.DSLOCALQUAMPPIL = '';
						args.owner.field.widget.getField('DSLOCALQUAMPPIL').readOnly = !row.DSALMOXQUAMPPIL;
						args.owner.field.widget.getField('DSLOCALQUAMPPIL').readOnlyOnEdit = !row.DSALMOXQUAMPPIL;
						break;
					case 10:
						args.owner.field.widget.currentRow.DSLOCALQUAR = '';
						args.owner.field.widget.getField('DSLOCALQUAR').readOnly = !row.DSALMOXQUAR;
						args.owner.field.widget.getField('DSLOCALQUAR').readOnlyOnEdit = !row.DSALMOXQUAR;
						break;
					case 11:
						args.owner.field.widget.currentRow.DSLOCALREPROV = '';
						args.owner.field.widget.getField('DSLOCALREPROV').readOnly = !row.DSALMOXREPROV;
						args.owner.field.widget.getField('DSLOCALREPROV').readOnlyOnEdit = !row.DSALMOXREPROV;
						break;
					case 12:
						args.owner.field.widget.currentRow.DSLOCALCONSVND = '';
						args.owner.field.widget.getField('DSLOCALCONSVND').readOnly = !row.DSALMOXCONSVND;
						args.owner.field.widget.getField('DSLOCALCONSVND').readOnlyOnEdit = !row.DSALMOXCONSVND;
						break;
					case 13:
						args.owner.field.widget.currentRow.DSLOCPRODSERPROC = '';
						args.owner.field.widget.getField('DSLOCPRODSERPROC').readOnly = !row.DSMOXPRODSERPROC;
						args.owner.field.widget.getField('DSLOCPRODSERPROC').readOnlyOnEdit = !row.DSMOXPRODSERPROC;
						break;
					case 14:
						args.owner.field.widget.currentRow.DSLOCALPRODPROC = '';
						args.owner.field.widget.getField('DSLOCALPRODPROC').readOnly = !row.DSALMOXPRODPROC;
						args.owner.field.widget.getField('DSLOCALPRODPROC').readOnlyOnEdit = !row.DSALMOXPRODPROC;
						break;
				}

			}
		}
		else {
			if (!fullLocalLock) {
				row = args.owner.field.widget.currentRow;

				switch (number) {
					case 1:
						args.owner.field.widget.currentRow.DSLOCALINSPR = '';
						args.owner.field.widget.getField('DSLOCALINSPR').readOnly = !row.DSALMOXINSPR;
						args.owner.field.widget.getField('DSLOCALINSPR').readOnlyOnEdit = !row.DSALMOXINSPR;
						break;
					case 2:
						args.owner.field.widget.currentRow.DSLOCALPRSEMACA = '';
						args.owner.field.widget.getField('DSLOCALPRSEMACA').readOnly = !row.DSALMOXPRSEMACA;
						args.owner.field.widget.getField('DSLOCALPRSEMACA').readOnlyOnEdit = !row.DSALMOXPRSEMACA;
						break;
					case 3:
						args.owner.field.widget.currentRow.DSLOCALINSPRPIL = '';
						args.owner.field.widget.getField('DSLOCALINSPRPIL').readOnly = !row.DSALMOXINSPRPIL;
						args.owner.field.widget.getField('DSLOCALINSPRPIL').readOnlyOnEdit = !row.DSALMOXINSPRPIL;
						break;
					case 4:
						args.owner.field.widget.currentRow.DSLOCPROD = '';
						args.owner.field.widget.getField('DSLOCPROD').readOnly = !row.DSALPROD;
						args.owner.field.widget.getField('DSLOCPROD').readOnlyOnEdit = !row.DSALPROD;
						break;
					case 5:
						args.owner.field.widget.currentRow.DSLOCPRODPRIL = '';
						args.owner.field.widget.getField('DSLOCPRODPRIL').readOnly = !row.DSALPRODPIL;
						args.owner.field.widget.getField('DSLOCPRODPRIL').readOnlyOnEdit = !row.DSALPRODPIL;
						break;
					case 6:
						args.owner.field.widget.currentRow.DSLOCALPRACA = '';
						args.owner.field.widget.getField('DSLOCALPRACA').readOnly = !row.DSALMOXPRACA;
						args.owner.field.widget.getField('DSLOCALPRACA').readOnlyOnEdit = !row.DSALMOXPRACA;
						break;
					case 7:
						args.owner.field.widget.currentRow.DSLOCALPRCLI = '';
						args.owner.field.widget.getField('DSLOCALPRCLI').readOnly = !row.DSALMOXPRCLI;
						args.owner.field.widget.getField('DSLOCALPRCLI').readOnlyOnEdit = !row.DSALMOXPRCLI;
						break;
					case 8:
						args.owner.field.widget.currentRow.DSLOCALESTOQ = '';
						args.owner.field.widget.getField('DSLOCALESTOQ').readOnly = !row.DSALMOXARIFE;
						args.owner.field.widget.getField('DSLOCALESTOQ').readOnlyOnEdit = !row.DSALMOXARIFE;
						break;
					case 9:
						args.owner.field.widget.currentRow.DSLOCALQUAMPPIL = '';
						args.owner.field.widget.getField('DSLOCALQUAMPPIL').readOnly = !row.DSALMOXQUAMPPIL;
						args.owner.field.widget.getField('DSLOCALQUAMPPIL').readOnlyOnEdit = !row.DSALMOXQUAMPPIL;
						break;
					case 10:
						args.owner.field.widget.currentRow.DSLOCALQUAR = '';
						args.owner.field.widget.getField('DSLOCALQUAR').readOnly = !row.DSALMOXQUAR;
						args.owner.field.widget.getField('DSLOCALQUAR').readOnlyOnEdit = !row.DSALMOXQUAR;
						break;
					case 11:
						args.owner.field.widget.currentRow.DSLOCALREPROV = '';
						args.owner.field.widget.getField('DSLOCALREPROV').readOnly = !row.DSALMOXREPROV;
						args.owner.field.widget.getField('DSLOCALREPROV').readOnlyOnEdit = !row.DSALMOXREPROV;
						break;
					case 12:
						args.owner.field.widget.currentRow.DSLOCALCONSVND = '';
						args.owner.field.widget.getField('DSLOCALCONSVND').readOnly = !row.DSALMOXCONSVND;
						args.owner.field.widget.getField('DSLOCALCONSVND').readOnlyOnEdit = !row.DSALMOXCONSVND;
						break;
					case 13:
						args.owner.field.widget.currentRow.DSLOCPRODSERPROC = '';
						args.owner.field.widget.getField('DSLOCPRODSERPROC').readOnly = !row.DSMOXPRODSERPROC;
						args.owner.field.widget.getField('DSLOCPRODSERPROC').readOnlyOnEdit = !row.DSMOXPRODSERPROC;
						break;
					case 14:
						args.owner.field.widget.currentRow.DSLOCALPRODPROC = '';
						args.owner.field.widget.getField('DSLOCALPRODPROC').readOnly = !row.DSALMOXPRODPROC;
						args.owner.field.widget.getField('DSLOCALPRODPROC').readOnlyOnEdit = !row.DSALMOXPRODPROC;
						break;
				}
			}
		}

	};

	// função responsável por remover o bloqueio dos campos de local que já estão
	// com o campo de almoxarifado preenchido ao carregar uma nova stripe
	this.lockBreaker = function(widget) {
		if (widget.currentRow) {
			var widgetControle = widget.widgets[0].widgets[0];
			var widgetPedidos = widget.widgets[1].widgets[0];
			var widgetControleQualidade = widget.widgets[2].widgets[0];
			var widgetGestaoVendas = widget.container.getWidget('GESTAODEVENDAS');
			var widgetProcessaProduto = widget.container.getWidget('PROCESSAPRODUTO');

			if (!fullLocalLock) {
				//1
				if (widgetControle.getField('DSALMOXINSPR').value()) {
					widgetControle.getField('DSLOCALINSPR').readOnly = false;
					widgetControle.getField('DSLOCALINSPR').readOnlyOnEdit = false;
				}
				else {
					widgetControle.getField('DSLOCALINSPR').readOnly = true;
					widgetControle.getField('DSLOCALINSPR').readOnlyOnEdit = true;
				}
				//2
				if (widgetControle.getField('DSALMOXPRSEMACA').value()) {
					widgetControle.getField('DSLOCALPRSEMACA').readOnly = false;
					widgetControle.getField('DSLOCALPRSEMACA').readOnlyOnEdit = false;
				}
				else {
					widgetControle.getField('DSLOCALPRSEMACA').readOnly = true;
					widgetControle.getField('DSLOCALPRSEMACA').readOnlyOnEdit = true;
				}
				//3
				if (widgetControle.getField('DSALMOXINSPRPIL').value()) {
					widgetControle.getField('DSLOCALINSPRPIL').readOnly = false;
					widgetControle.getField('DSLOCALINSPRPIL').readOnlyOnEdit = false;
				}
				else {
					widgetControle.getField('DSLOCALINSPRPIL').readOnly = true;
					widgetControle.getField('DSLOCALINSPRPIL').readOnlyOnEdit = true;
				}
				//4
				if (widgetControle.getField('DSALPROD').value()) {
					widgetControle.getField('DSLOCPROD').readOnly = false;
					widgetControle.getField('DSLOCPROD').readOnlyOnEdit = false;
				}
				else {
					widgetControle.getField('DSLOCPROD').readOnly = true;
					widgetControle.getField('DSLOCPROD').readOnlyOnEdit = true;
				}
				//5
				if (widgetControle.getField('DSALPRODPIL').value()) {
					widgetControle.getField('DSLOCPRODPRIL').readOnly = false;
					widgetControle.getField('DSLOCPRODPRIL').readOnlyOnEdit = false;
				}
				else {
					widgetControle.getField('DSLOCPRODPRIL').readOnly = true;
					widgetControle.getField('DSLOCPRODPRIL').readOnlyOnEdit = true;
				}
				//6
				if (widgetControle.getField('DSALMOXPRACA').value()) {
					widgetControle.getField('DSLOCALPRACA').readOnly = false;
					widgetControle.getField('DSLOCALPRACA').readOnlyOnEdit = false;
				}
				else {
					widgetControle.getField('DSLOCALPRACA').readOnly = true;
					widgetControle.getField('DSLOCALPRACA').readOnlyOnEdit = true;
				}
				//7
				if (widgetPedidos.getField('DSALMOXPRCLI').value()) {
					widgetPedidos.getField('DSLOCALPRCLI').readOnly = false;
					widgetPedidos.getField('DSLOCALPRCLI').readOnlyOnEdit = false;
				}
				else {
					widgetPedidos.getField('DSLOCALPRCLI').readOnly = true;
					widgetPedidos.getField('DSLOCALPRCLI').readOnlyOnEdit = true;
				}
				//8
				if (widgetControleQualidade.getField('DSALMOXARIFE').value()) {
					widgetControleQualidade.getField('DSLOCALESTOQ').readOnly = false;
					widgetControleQualidade.getField('DSLOCALESTOQ').readOnlyOnEdit = false;
				}
				else {
					widgetControleQualidade.getField('DSLOCALESTOQ').readOnly = true;
					widgetControleQualidade.getField('DSLOCALESTOQ').readOnlyOnEdit = true;
				}
				//9
				if (widgetControleQualidade.getField('DSALMOXQUAMPPIL').value()) {
					widgetControleQualidade.getField('DSLOCALQUAMPPIL').readOnly = false;
					widgetControleQualidade.getField('DSLOCALQUAMPPIL').readOnlyOnEdit = false;
				}
				else {
					widgetControleQualidade.getField('DSLOCALQUAMPPIL').readOnly = true;
					widgetControleQualidade.getField('DSLOCALQUAMPPIL').readOnlyOnEdit = true;
				}
				//10
				if (widgetControleQualidade.getField('DSALMOXQUAR').value()) {
					widgetControleQualidade.getField('DSLOCALQUAR').readOnly = false;
					widgetControleQualidade.getField('DSLOCALQUAR').readOnlyOnEdit = false;
				}
				else {
					widgetControleQualidade.getField('DSLOCALQUAR').readOnly = true;
					widgetControleQualidade.getField('DSLOCALQUAR').readOnlyOnEdit = true;
				}

				if (widgetControleQualidade.getField('DSALMOXREPROV').value()) {
					widgetControleQualidade.getField('DSLOCALREPROV').readOnly = false;
					widgetControleQualidade.getField('DSLOCALREPROV').readOnlyOnEdit = false;
				}
				else {
					widgetControleQualidade.getField('DSLOCALREPROV').readOnly = true;
					widgetControleQualidade.getField('DSLOCALREPROV').readOnlyOnEdit = true;
				}
				//12
				if(widgetGestaoVendas.getField('DSALMOXCONSVND').value()){
					widgetGestaoVendas.getField('DSLOCALCONSVND').readOnly = false;
					widgetGestaoVendas.getField('DSLOCALCONSVND').readOnlyOnEdit = false;
				}
				else {
					widgetGestaoVendas.getField('DSLOCALCONSVND').readOnly = true;
					widgetGestaoVendas.getField('DSLOCALCONSVND').readOnlyOnEdit = true;
				}
				//13
				if(widgetProcessaProduto.getField('DSMOXPRODSERPROC').value()){
					widgetProcessaProduto.getField('DSLOCPRODSERPROC').readOnly = false;
					widgetProcessaProduto.getField('DSLOCPRODSERPROC').readOnlyOnEdit = false;
				}
				else {
					widgetProcessaProduto.getField('DSLOCPRODSERPROC').readOnly = true;
					widgetProcessaProduto.getField('DSLOCPRODSERPROC').readOnlyOnEdit = true;
				}
				//14
				if(widgetProcessaProduto.getField('DSALMOXPRODPROC').value()){
					widgetProcessaProduto.getField('DSLOCALPRODPROC').readOnly = false;
					widgetProcessaProduto.getField('DSLOCALPRODPROC').readOnlyOnEdit = false;
				}
				else {
					widgetProcessaProduto.getField('DSLOCALPRODPROC').readOnly = true;
					widgetProcessaProduto.getField('DSLOCALPRODPROC').readOnlyOnEdit = true;
				}

				widgetControle.reload();
				widgetPedidos.reload();
				widgetControleQualidade.reload();
				widgetGestaoVendas.reload();
				widgetProcessaProduto.reload();
			}
		}
	};

	this.openPopupCadastro = function(widget) {
		var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICO');
		var filtroAplicadoRow = widget.appliedFilter.FILTER_ROW;
		widgetPopup.isVisible = true;
		widgetPopup.activate();
		widgetPopup.edit();
		this.blockAlmoxCadastro(widgetPopup);
		ScreenService.openPopup(widgetPopup).then(success => {
			widgetPopup.currentRow.CDFILIAL = filtroAplicadoRow.CDFILIAL;
			widgetPopup.currentRow.NMFILIAL = filtroAplicadoRow.NMFILIAL;
			this.filiProd(widgetPopup);
		});
	};

	this.blockAlmoxCadastro = function(widget) {
		widget.getField('DSALMOXINSPR').readOnly = true;
		widget.getField('DSALMOXINSPR').readOnlyOnEdit = true;
		//2
		widget.getField('DSALMOXPRSEMACA').readOnly = true;
		widget.getField('DSALMOXPRSEMACA').readOnlyOnEdit = true;
		//3
		widget.getField('DSALMOXINSPRPIL').readOnly = true;
		widget.getField('DSALMOXINSPRPIL').readOnlyOnEdit = true;
		//4
		widget.getField('DSALPROD').readOnly = true;
		widget.getField('DSALPROD').readOnlyOnEdit = true;
		//5
		widget.getField('DSALPRODPIL').readOnly = true;
		widget.getField('DSALPRODPIL').readOnlyOnEdit = true;
		//6
		widget.getField('DSALMOXPRACA').readOnly = true;
		widget.getField('DSALMOXPRACA').readOnlyOnEdit = true;
		//7
		widget.getField('DSALMOXPRCLI').readOnly = true;
		widget.getField('DSALMOXPRCLI').readOnlyOnEdit = true;
		//8
		widget.getField('DSALMOXARIFE').readOnly = true;
		widget.getField('DSALMOXARIFE').readOnlyOnEdit = true;
		//9
		widget.getField('DSALMOXQUAMPPIL').readOnly = true;
		widget.getField('DSALMOXQUAMPPIL').readOnlyOnEdit = true;
		//10
		widget.getField('DSALMOXQUAR').readOnly = true;
		widget.getField('DSALMOXQUAR').readOnlyOnEdit = true;

		//11
		widget.getField('DSALMOXREPROV').readOnly = true;
		widget.getField('DSALMOXREPROV').readOnlyOnEdit = true;
		
		//12
		widget.getField('DSALMOXCONSVND').readOnly = true;
		widget.getField('DSALMOXCONSVND').readOnlyOnEdit = true;
		
		//13
		widget.getField('DSMOXPRODSERPROC').readOnly = true;
		widget.getField('DSMOXPRODSERPROC').readOnlyOnEdit = true;
		
		//14
		widget.getField('DSALMOXPRODPROC').readOnly = true;
		widget.getField('DSALMOXPRODPROC').readOnlyOnEdit = true;
	};
	
	// concatenação de código para relatorio produto final
	this.getValueProdFin = function(){
		let filterValueProdFin = EstUtil.getFieldValue('P_CDPRODFIN','widgetProduto',true);
		
		if(filterValueProdFin){
			let concatString = filterValueProdFin.concat('zzzzzzzzzzzzzzzzzzzz');
			let newResult = concatString.substring(0,20);
			return newResult;
		}else{
			return 'zzzzzzzzzzzzzzzzzzzz';
		}
	};
	
	// verificacao de valores para produto inicial
	this.getValueProdIni = function() {
		let filterValueProdIni = EstUtil.getFieldValue('P_CDPRODINI', 'widgetProduto', true);
				
		if(filterValueProdIni) {
			return filterValueProdIni;
		} else {
			return '1';
		}
	};
	
	// Gera relatórios de produtos XML/PDF
	this.generateReportXmlPdf = function(type){
        let REPORT_JSON = "./json/containers/ger27000_local_de_estoque_por_produto.json";
        let widgetRelatorio = templateManager.container.getWidget('TABBEDRELATORIOS');
        let widgetProduto = templateManager.container.getWidget('widgetProduto');
        ScreenService.showLoader();
        ScreenService.openSwipe(widgetRelatorio).then((opened) => {
	        widgetRelatorio.reload().then(loaded => {
	            restEngine.requestMetaData(REPORT_JSON).then(function (reportMetaData) {
	                ZhReports.openReport(reportMetaData.data, widgetRelatorio, null, null, type);
	                setTimeout(() => {
        				ScreenService.goBack();
	                	ScreenService.hideLoader();
	                }, 1500);
	            });
	        });
        });
        widgetProduto.activate();
    };
	
	// atualizar os dados automaticamente ao abrir o relatório de produtos. 
	this.autoUpdateOnEnter = ()=>{
	let widgetRelatorios = templateManager.container.getWidget('TABBEDRELATORIOS');
		widgetRelatorios.reload();
	};
	
	this.saveCadastroAutomatico = function(widget) {
		var row = widget.currentRow;
		row.CDMOXPRODSERPROC = row.CDMOXPRODSERPROC ? row.CDMOXPRODSERPROC : null;
		row.CDLOCPRODSERPROC = row.CDLOCPRODSERPROC ? row.CDLOCPRODSERPROC : null;
		row.CDALMOXPRODPROC = row.CDALMOXPRODPROC ? row.CDALMOXPRODPROC : null;
		row.CDLOCALPRODPROC = row.CDLOCALPRODPROC ? row.CDLOCALPRODPROC : null;
		
		row.CDNVPRODUTO = General.getLevelByMask('CDPICTPROD', 1).length;
		if (widget.isValid()) {
			General.callRestEngine('est#/cadastroAutomaticoLocalProduto', 'Row', row).then(function(success) {
					var widgetActivate = templateManager.container.getWidget('widgetProduto');
					var widgetPopup = templateManager.container.getWidget('CADASTROAUTOMATICO');
					widgetPopup.isVisible = false;
					widgetPopup.currentRow = {};
					ScreenService.closePopup(widgetPopup);
					widgetActivate.activate().then(activated => {
						widgetActivate.reload();
					}).catch(errorActivate => {
						ScreenService.showMessage(errorActivate);
					});
				},
				function(callback) {
					if (callback.data) {
						ScreenService.showMessage(callback.data.error);
					}
				}
			);
		}
	};

	this.cancelarCadastroAutomaticoItem = function() {
		var widget = templateManager.container.getWidget('CADASTROAUTOMATICO');
		widget.currentRow = {};
		var widgetActivate = templateManager.container.getWidget('widgetProduto');
		widget.isVisible = false;
		ScreenService.closePopup(widget);
		widgetActivate.activate();
		widgetActivate.reload();
	};

	this.checkProductRange = function(field) {
		var currentRow = field.widget.currentRow;

		if (currentRow.P_CDPRODINI && currentRow.P_CDPRODFIN) {
			var compproduto = (currentRow.P_CDPRODFIN) ? currentRow.P_CDPRODFIN.replace(/[\.\-]/g, '') : null;

			if (!General.comparaPratoProduto(currentRow.P_CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
				currentRow[field.outData[field.descriptionField]] = null;
				currentRow[field.outData[field.valueField]] = null;

				ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
			}
		}
	};
	//-------------------------- funções CRUD ---------------
	//função default para editar
	this.edit = function(widget) {

		General.edit(widget);
		controlaStatusWidgets(widget.getParent().getParent(), widget.name, false, []);
		var gridProd = templateManager.container.getWidget("widgetProduto");
		cdProd = gridProd.currentRow.CDPRODUTO;
		nmProd = gridProd.currentRow.NMPRODUTO;
		widget.template = 'widget/form.html';
	};


	var controlaStatusWidgets = function(widgetPai, nomeWidgetAtual, status, ignoredWidgets) {
		var allWidgets = widgetPai.widgets;

		allWidgets.filter(widget => widget.name !== nomeWidgetAtual).forEach(widget => widget.enabled = status);

		if (widgetPai.name !== nomeWidgetAtual)
			widgetPai.enabled = status;

		if (Array.isArray(ignoredWidgets))
			allWidgets.filter(widget => ignoredWidgets.indexOf(widget.name) !== -1).forEach(widget => widget.enabled = !status);
	};

	//função default para cancelar uma edição
	this.cancelEdit = function(args) {
		args.owner.widget.template = 'widget/form-view.html';
		var widget = args.owner.widget;
		controlaStatusWidgets(widget.getParent().getParent(), widget.name, true, []);
		General.backOrCancel(args);
		args.owner.widget.list();
		args.owner.widget.reload();
	};

	//função default para salvar uma edição
	this.saveEdit = function(widget) {
		var gridProd = templateManager.container.getWidget("widgetProduto");

		General.saveCrud(widget).then(function(Succes) {
			widget.template = 'widget/form-view.html';
			widget.list();
			controlaStatusWidgets(widget.getParent().getParent(), widget.name, true, []);
			widget.reload();
		}.bind(this));
	};
	//--------------------------------------------------------
};

Configuration(function(ContextRegister) {
	ContextRegister.register('Ger27000_LocalEstoqueProduto_Controller', Ger27000_LocalEstoqueProduto_Controller);
});


// FILE: ./assets/js/controllers/Ger28000_TipoRetirada_Controller.js
var Ger28000_TipoRetirada_Controller = function(General, templateManager, ZhFilterPreferences) {

    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        var defaultFilter = widget.dataSourceFilter;
        ZhFilterPreferences.setOriginalFilter(widget.id, defaultFilter);
        ZhFilterPreferences.openFilterWidget(widget, defaultFilter);
    };
    
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };

	this.save = function(widget) {
	    if(widget.isValid()){
	        widget.currentRow.CDTIPORETI = widget.getParent().currentRow.CDTIPORETI;
            General.saveCrud(widget);
	    }
    };

    this.controlaStatusWidgets = function(mainWidget, nomeWidgetAtual, status){
        mainWidget.widgets.forEach(function(widg) {
            if(nomeWidgetAtual !== widg.name)
                widg.enabled = status;
        });

        if(mainWidget.name !== nomeWidgetAtual)
            mainWidget.enabled = status;
    };
    
    this.salvarTipoRetirada = function(widget){
        if(widget.isValid()){
            General.saveCrud(widget);
            this.controlaStatusWidgets(widget,'tipo_de_retirada', true); 
        }
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ger28000_TipoRetirada_Controller', Ger28000_TipoRetirada_Controller);
});

// FILE: ./assets/js/controllers/InventarioRotativoDecorator.js
function InventarioRotativoDecorator(ZHPromise, PreRequests) {

    /*
     * Decorator do Inventário Rotativo. Valida toda abertura de telas,
     * impedindo que aquelas que validam estoque sejam abertas caso haja
     * inventário rotativo pendente, assim como nos sistemas Delphi.
     */
    this.apply = function(containerName, container) {
        if (container.validaEstoque === true) {
            // PreRequests.setContainerBeingValidated();
            var deferred = ZHPromise.defer();
            PreRequests.validaInventarioRotativo(container).then(function(resolve) {
                var inventarioValido = resolve.inventarioValido;
                var container = resolve.container;
                var inveRotaContainer = (containerName === "est#ger03302_inventario_rotativo");
                if (inventarioValido !== inveRotaContainer) {
                    container.events = container.events || [];
                    container.events.push({
                        name: "ContainerAfterinit",
                        code: "PreRequests.validaInventarioRotativo(args.owner)"
                    });
                }
            }).finally(function() {
                deferred.resolve(container);
            });
            return deferred.promise;
        }
        else {
            return container;
        }
    };
}

// FILE: ./assets/js/controllers/PreRequests.js
/*
 * Classe que tem o objetivo de executar validações gerais em
 * telas e requisições. São utilizados três elementos: eventAggregator,
 * interceptor (restEngine) e Decorator. O eventAggregator é executado
 * ao iniciar ou finalizar requisições; o interceptor é executado antes
 * de iniciar as requisições, fazendo validações antes de permitir que
 * as mesmas prossigam; e o Decorator é executado antes de abrir qualquer
 * container, alterando-o antes de ser aberto.
 */
var PreRequests = function(ContainerDecoratorProvider, MessageService, eventAggregator, ZHPromise, InventarioRotativo_Service, ScreenService, restEngine, templateManager) {

	var containerBeingValidated = 0;
	var requestSent = false;
	var self = this;
	let containerIsValidated = [];

	/*
	 * Função para indicar que um container está sendo validado, impedindo
	 * que o interceptor sobrescreva as validações feitas pelo Decorator.
	 * É utilizado um valor numérico para que, caso haja mais de um Decorator
	 * sendo utilizado, não haja conflito de condição de corrida.
	 */
	self.setContainerBeingValidated = function() {
		containerBeingValidated++;
	};

	/*
	 * Função utilizado no final de uma validação de Decorator, para que
	 * o interceptor possa voltar a validar as requisições quando a variável
	 * chegar ao valor 0 (quando não houver validação de qualquer Decorator).
	 */
	self.freeContainerBeingValidated = function() {
		containerBeingValidated--;
	};

	/*
	 * Função para verificar se há algum Decorator validando um
	 * container. Caso haja, o número na variável 'containerBeingValidated'
	 * deve ser maior que 0, retornando true.
	 */
	self.getContainerIsBeingValidated = function() {
		return containerBeingValidated > 0;
	};

	/*
	 * 	Função que executa a validação de inventário rotativo ao entrar
	 * 	em telas que estão parametrizadas para realizar tal validação.
	 * 	Se houver inventário rotativo pendente para a unidade, será exibida
	 * 	uma mensagem e haverá o redirecionamento para uma página em branco,
	 * 	necessária para impedir o acesso à tela acessada.
	 */
	self.validaInventarioRotativo = function(container) {
		if (container.validaEstoque === true) {
			self.setContainerBeingValidated();
			var deferred = ZHPromise.defer();
			var containerName = container.name;
			InventarioRotativo_Service.checkInventarioPendente().then((inventarioValido) => {
				var inveRotaContainer = (containerName === "ger03302_inventario_rotativo");
				if (!(inventarioValido.result || inveRotaContainer)) {
					setTimeout(() => {
						switch (inventarioValido.type) {
							case 'InventCego':
								MessageService.showMessageByCode('BLOCK_DIFF_INVENT_CEGO').then(() => {
									ScreenService.openWindow("est#ger03302_blank_window");
								});
								break;
							case 'ContPend':
								MessageService.showMessageByCode('WARNING_PENDING_PHYSICAL_COUNT');
								break;
							case 'ContPendDig':
								MessageService.showMessageByCode('WARNING_PENDING_PHYSICAL_COUNT_TYPED');
								break;
							case 'RealContEstFis':
								MessageService.showMessageByCode('WARNING_PHYSICAL_COUNT_WILL_BE_REALIZED');
								break;
							default:
								MessageService.showMessageByCode('BLOCK_PENDING_INVENTORY').then(() => {
									ScreenService.openWindow("est#ger03302_blank_window");
								});
						}
					}, 400);
				}
				else if (inveRotaContainer && inventarioValido.result) {
					container.widgets = [];
					MessageService.showMessageByCode('EMPTY_INVENTORY').then(() => {
						ScreenService.openWindow("est#ger03302_blank_window");
					});
				}
				else if (inveRotaContainer) {
					container.validaEstoque = false;
				}
				deferred.resolve({
					container: container,
					inventarioValido: inventarioValido.result
				});
			}).catch(function(error) {
				container.widgets = [];
				container.events = [];
				MessageService.showMessageByCode('ERROR_CONNECTION_SERVER').then(() => {
					ScreenService.openWindow("est#ger03302_blank_window");
				});
				deferred.reject(error);
			}).finally(() => {
				self.freeContainerBeingValidated();
			});
			return deferred;
		}
	};

	/*
	 * Função que envia nos headers de qualquer requisição parâmetros que
	 * indicam se a tela atual valida estoque e qual unidade será validada.
	 * Para isso, o container da tela deve conter as seguinter propriedades:
	 *
	 *   * filialOnRequest - String  - CDFILIAL utilizada no processo.
	 *   Opcional. Buscará a unidade logada como default.
	 *
	 *   * validaEstoque   - Boolean - Indica se a tela verifica estoque.
	 *   Opcional. Valor default é false, não validando inventário rotativo
	 *   automaticamente.
	 */
	eventAggregator.subscribe('EventOnRequestStart', function(eventData) {
		var url = eventData.requestConfig.url;
		// if para impedir de adicionar os headers caso seja uma requisição
		// para gerar relatórios
		if (url.indexOf('generateReport') === -1 && url.indexOf('generateXLS') === -1) {
			var container = templateManager.container;
			var filial = false;
			var validaEstoque = false;
			if (container) {
				filial = container.filialOnRequest || false;
				validaEstoque = (container.validaEstoque !== undefined) ? container.validaEstoque : false;
			}
			eventData.requestConfig.headers.Filial = filial;
			eventData.requestConfig.headers.ValidaEstoque = validaEstoque;
		}
	});

	/*
	 * Registrador do interceptador de requisições do Inventário Rotativo.
	 * Toda requisição pro backend proveniente de telas que validam estoque
	 * será validada antes de ser concretizada. Isso impede o funcionamento
	 * de telas caso haja inventário rotativo pendente, assim como nos
	 * sistemas Delphi.
	 */
	restEngine.registerInterceptor({
		method: "ALL",
		interceptor: function(data) {
			if (!(requestSent || self.getContainerIsBeingValidated())) {
				var container = templateManager.container;
				if (container && container.validaEstoque === true && container.name !== 'ger03302_inventario_rotativo') {
					if (containerIsValidated.includes(container.name)) {
						containerIsValidated.push(container.name);
						var deferred = ZHPromise.defer();
						// Variável de controle, para que a requisição da validação
						// realizada aqui não seja verificada.
						requestSent = true;
						InventarioRotativo_Service.checkInventarioPendente().then(function(inventarioIsValid) {
							if (inventarioIsValid.result) {
								deferred.resolve();
							}
							else if (inventarioIsValid.type == 'InventRota') {
								MessageService.showMessageByCode('BLOCK_PENDING_INVENTORY').then(() => {
									ScreenService.openWindow("est#ger03302_blank_window");
								});
								deferred.reject();
							}
						}).catch(function() {
							// Caso haja algum erro, impede a requisição.
							deferred.reject();
						}).finally(function() {
							requestSent = false;
						});
						return deferred;
					}
				}
			}
		}
	});


	/*
	 * Função que trabalha a recursividade do mecanismo de preenchimento do
	 * DescriptionField. Recebe uma widget e caso a mesma tenha widgets filhas,
	 * todos os fields do campo "select" dessas será verificado.
	 */
	self.verificaSelectWidgetsAndChilds = function(widget) {
		validaDescriptionFieldSelect(widget);
		if (widget.widgets && widget.widgets.length) {
			widget.widgets.forEach(function(childWidget) {
				self.verificaSelectWidgetsAndChilds(childWidget);
			});
		}
	};

	/*
	 * Função local auxiliar a "verificaSelectWidgetsAndChilds", responsavél por
	 * validar se os campos select de uma widget tem a propriedade description
	 * field preenchida. Caso algum dos fields não tenham a propriedade, a função
	 * 'preencheDescriptionFieldSelect' é executada.
	 */
	function validaDescriptionFieldSelect(widget) {
		if (widget.fields && widget.fields.length) {
			var description;
			widget.fields.forEach(function(field) {
				description = null;
				if (field.isVisible && field.fields && (field.template == "field/select.html" || field.template == "field/select-multiple.html")) {
					if (field.descriptionField) {
						field.fields.forEach(function(childField) {
							if (childField.name == field.descriptionField)
								description = true;
						});
						if (description === null) {
							preencheDescriptionFieldSelect(field);
						}
					}
					else {
						preencheDescriptionFieldSelect(field);
					}
				}
			});
		}
	}

	/*
	 * Função local auxiliar a "validaDescriptionFieldSelect",
	 * preenche a propriedade "descriptionField" de fields select
	 * com valores default. Ordem de preenchimento: mesmo nome do
	 * field ou primeiro field filho visível.
	 */
	function preencheDescriptionFieldSelect(field) {
		var description = null;
		description = field.fields.filter(function(childField) {
			return childField.isVisible !== false && childField.name == field.name && childField.showOnList;
		});
		field.descriptionField = description && description.length ? description[0].name : null;

		if (!description.length) {
			description = field.fields.filter(function(childField) {
				return childField.isVisible !== false && childField.showOnList;
			});
			field.descriptionField = description && description.length ? description[0].name : null;
		}
	}

	/*
	 * Registrador do Decorator do Inventário Rotativo. Valida toda abertura
	 * de telas, impedindo que aquelas que validam estoque sejam abertas
	 * caso haja inventário rotativo pendente, assim como nos sistemas Delphi.
	 */
	ContainerDecoratorProvider.registerProvider(new InventarioRotativoDecorator(ZHPromise, this));

	/*
	 * Registrador do Decorator de preenchimento default de Description Field.
	 * Valida toda abertura de telas, e aquelas que conterem fields select sem
	 * a propriedade de DescriptionField serão alteradas. O Decorator só é ativado
	 * caso a propriedade "useDecoratorInDescriptionFieldSelectInSelect" tenha valor true.
	 * Normalmente a propriedade é definida no enviroment.js.
	 */
	if (window.useDecoratorInDescriptionFieldSelect)
		ContainerDecoratorProvider.registerProvider(new DescriptionFieldSelectDecorator(ZHPromise, this));
};

Configuration(function(ContextRegister) {
	ContextRegister.register('PreRequests', PreRequests);
});


// FILE: ./assets/js/controllers/ProdTreeGroup.js
var ProdTreeGroup = function(General, ZHPromise){
    /**
     * [getAllGroups Trata casos de seleção de grupos cuja grupo
     * ainda não foi carregado e possui filhos.]
     * @param  {[Array]} fieldTreeProd [Agrupamento de Produto]
     * @return {[Array]}    [Grupos e respectivas dependencias]
     */
    this.getAllGroups = function(fieldTreeProd){
        var deferred = ZHPromise.defer();
        var groups = [];
        var fullGroups = [];
        var selectedRows = fieldTreeProd.dataSource.getCheckedRows();
        selectedRows.forEach(function(pai){
            if (pai._hasChild){
                if (pai.hasOwnProperty('__processed')){
                    groups.push(pai.NRARVOREPROD);
                }else{
                    fullGroups.push(pai.NRARVOREPROD);
                }
            }else{
                groups.push(pai.NRARVOREPROD);
            }
        });
        var route = 'est#/getAllGroupsByParents';
        General.callRestEngine(route, 'Row', fullGroups)
            .then(function(response){
                var childrenoffullgroups = response.dataset.all_groups;
                 var allGroups = childrenoffullgroups.concat(groups);
                deferred.resolve(allGroups);
            })
            .catch(function(erro){
                deferred.reject(erro);
            });
        return deferred.promise;
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('ProdTreeGroup', ProdTreeGroup);
});

// FILE: ./assets/js/controllers/zhFavorites.js
var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhNavbarDefaultHeader', function(templateManager, ScreenService, eventAggregator, $compile, ApplicationContext) {
	return {
		restrict: 'C',
		link: function($scope, element) {
		    $scope.favoritesList = [];
		    $scope.lastDeletedItem = undefined;
		    
		    var favoritesText = 'Favoritos';
		    var favoritesAddText = 'Adicionar aos Favoritos';
		    var favoritesRemoveText = 'Remover dos Favoritos';
		    var favoritesUndoText = 'DESFAZER';

			// TRANSLATE - Adicionar essas variaveis a lista da aplicação (words.jon)
		    // var favoritesText = ScreenService.i18n('FAVORITES');
		    // var favoritesAddText = ScreenService.i18n('ADD_TO_FAVORITES');
		    // var favoritesRemoveText = ScreenService.i18n('REMOVE_FROM_FAVORITES');
		    // var favoritesUndoText = ScreenService.i18n('UNDO');
		    
		    var favoritesContainer;
		    
		    var startHollowCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M15,4.21l3.51,7.1l7.84,1.14l-5.67,5.53l1.34,7.81L15,22.1l-7.01,3.69l1.34-7.81l-5.67-5.53l7.84-1.14L15,4.21'+
				'			 M15,2.21c-0.76,0-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05'+
				'			l4.92,4.8l-1.16,6.77c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			c0.55-0.53,0.74-1.33,0.51-2.05c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21L15,2.21z"/>'+
				'	</g>'+
				'</svg>';
		    var startFilledCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M28.25,11.83c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21'+
				'			s-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05l4.92,4.8l-1.16,6.77'+
				'			c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			C28.29,13.35,28.48,12.56,28.25,11.83z"/>'+
				'	</g>'+
				'</svg>';
		    
		    function createVisualElements() {
    			var headerContainer = $(element).children()[0];
    			headerContainer.style.width = 'calc(100% - 50px)';
    			
    			favoritesContainer = $('<div class="zh-favorites"></div>').appendTo(element);
    			
    			var favoritesButtonElement = $compile(
    				'<div class="zh-favorites-button" ng-click="toggleList()">'+
    				'  <div class="zh-favorites-button-star" ng-if="!isFavorited">'+
    				'    '+startHollowCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-star" ng-if="isFavorited">'+
    				'    '+startFilledCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-text">'+favoritesText+'</div>'+
    				'</div>'
    			)($scope);
    			var favoritesButton = $(favoritesButtonElement).appendTo(favoritesContainer);
    			
    			var favoritesListElement = $compile(
    				'<div class="zh-favorites-list" ng-class="{\'opened\': opened}">'+
    				'  <div class="zh-favorites-list-button" ng-if="!isFavorited" ng-click="addToFavorites()">'+favoritesAddText+'</div>'+
    				'  <div class="zh-favorites-list-button" ng-if="isFavorited" ng-click="removeFromFavorites()">'+favoritesRemoveText+'</div>'+
    				'  <div class="zh-favorites-list-undo" ng-show="lastDeletedItem" ng-click="undo()">'+favoritesUndoText+'</div>'+
    				'  <div class="zh-favorites-list-items" ng-show="favoritesList.length > 0">'+
    				'	 <div class="zh-favorites-list-item" ng-repeat="item in favoritesList track by item.id" ng-click="openItem(item)">'+
    				'      {{ item.label }}'+
    				'      <span class="zh-favorites-list-item-delete" data-zh-icon="trash icon-white no-border" ng-click="deleteItem(item, $index, $event)"></span>'+
    				'    </div>'+
    				'  </div>'+
    				'</div>'
    			)($scope);
    			var favoritesList = $(favoritesListElement).appendTo(favoritesContainer);
		    }
		    
		    function checkIsFavorited(containerId) {
		    	var oldIsFavorited = $scope.isFavorited;
			    $scope.isFavorited = $scope.favoritesList.findIndex((item) => item.id === containerId) !== -1;
			    if (oldIsFavorited !== $scope.isFavorited) {
			    	templateManager.updateTemplate();
			    }
		    }
		    
		    $scope.addToFavorites = function() {
		    	var item = {
		    		id: templateManager.container.id,
		    		name: templateManager.container.fileName.split('.json')[0],
		    		label: templateManager.container.label,
		    	};
		    	$scope.favoritesList.push(item);
		    	$scope.isFavorited = true;
		    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'add' }});
		    };
		    
		    $scope.removeFromFavorites = function() {
		    	var index = $scope.favoritesList.findIndex((item) => item.id === templateManager.container.id);
		    	if (index > -1) {
			    	var item = $scope.favoritesList.splice(index, 1);
			    	$scope.isFavorited = false;
			    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'delete' }});
		    	}
		    };
		    
		    $scope.openItem = function(item) {
		    	ApplicationContext.LibUtilities.openWindow(item.name);
	    		$scope.opened = false;
		    };
		    
		    var undoTimeout;
		    
		    $scope.deleteItem = function(item, index, event) {
		    	$scope.favoritesList.splice(index, 1);
		    	$scope.lastDeletedItem = { item, index };

		    	clearTimeout(undoTimeout);
		    	undoTimeout = setTimeout(() => {
		    		$scope.lastDeletedItem = undefined;
		    		templateManager.updateTemplate();
		    	}, 10000);

			    eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'delete' }});

		    	if (item.id === templateManager.container.id) {
		    		$scope.isFavorited = false;
		    	}

		    	event.stopImmediatePropagation();
		    };
		    
		    $scope.undo = function() {
		    	if ($scope.lastDeletedItem) {
		    		$scope.favoritesList.splice($scope.lastDeletedItem.index, 0, $scope.lastDeletedItem.item);
		    	}
		    	$scope.lastDeletedItem = undefined;
		    };

		    $scope.toggleList = function() {
		    	$scope.opened = !$scope.opened;
		    };
		    
		    document.addEventListener('mousedown', (evt) => {
		    	if ($(favoritesContainer).find(evt.target).length === 0 && $scope.opened) {
		    		$scope.opened = false;
			    	templateManager.updateTemplate();
		    	}	
		    });
		    
		    eventAggregator.subscribe(eventAggregator.ON_CONTAINER_OPEN, (args) => {
		    	checkIsFavorited(args.container.id);
		    });
		    
		    eventAggregator.subscribe('ZhFavoritesOnLoad', (favorites) => {
		    	
		    	$scope.favoritesList = favorites;
			    if (templateManager.container) {
			    	checkIsFavorited(templateManager.container.id);
			    }
		    });

		    createVisualElements();
		}
	};
});

// FILE: ./assets/modules/ajuste/assets/js/controllers/Ajuste_Controller.js
var Ajuste_Controller = function(ScreenService, ZhLayoutPreferences, General, templateManager, EstUtil, Ajuste_Service, ZHPromise, ValidacoesEstoque_Service, maskEngine, ZhUtil, metaDataFactory, Lib00206_InfComplProduto_Controller, restEngine, ZhFilterPreferences, MessageService) {
    let self = this;
    let bkpDatasourceFilter = [];
    let ignoredCloneKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
    const glossary = {

        "CDBARPRODUTO": {
            'desc': 'Código de Barras do produto',
            'template': 'field/textarea.html',
            'textRows': 2

        },
        "CDARVPROD": {
            'desc': 'Código da Árvore de produto',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "CDPRODUTO": {
            'desc': 'Código do Produto',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "DTLANCESTQ": {
            'desc': 'Data de Lançamento de Estoque (DDMMYYYY)',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "QTTOTLANCTO": {
            'desc': 'Quantidade de Itens',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "VRUNILANCTO": {
            'desc': 'Valor Unitário',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "CDALMOXARIFE": {
            'desc': 'Código do Almoxarifado',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "CDLOCALESTOQ": {
            'desc': 'Código do Local de Estoque',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "NRLOTEESTQ": {
            'desc': 'Código do Lote',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "NRSUBLOTE": {
            'desc': 'Código do Sub Lote',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "DTFABRPROD": {
            'desc': 'Data de Fabricação do Produto (DDMMYYYY)',
            'template': 'field/textarea.html',
            'textRows': 2
        },
        "DTVALILOTE": {
            'desc': 'Data de Validade de Lote (DDMMYYYY)',
            'template': 'field/textarea.html',
            'textRows': 2
        }
    };

    this.validaFilial = true;

    let filutlote = 'N';

    Array.prototype.move = function(old_index, new_index) {
        if (new_index >= this.length) {
            var k = new_index - this.length;
            while ((k--) + 1) {
                this.push(undefined);
            }
        }
        this.splice(new_index, 0, this.splice(old_index, 1)[0]);
    };

    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        this.getFilialLogada().then(success => {
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(filtro => {
                this.validaFilialControlaAlmoxarifado();
            });
        });
    };

    this.validaFilialControlaAlmoxarifado = function() {
        var filtroWidget = templateManager.container.getActiveWidget();

        var cdfilial = filtroWidget.getField('CDFILIAL').value();

        Ajuste_Service.retornaParamUnidade(cdfilial).then(function(paramfilial) {

            var filial = paramfilial.dataset.paramfilial[0];
            let filtroRow = filtroWidget.currentRow;
            const container = filtroWidget.getParent();
            const defaultFields = ['CDOPERADOR', 'CDALMOXARIFE', 'CDLOCALESTOQ'];

            Object.keys(filial).forEach(key => {
                if (container.name == 'est41000_ajuste_inventario') {
                    if (key != 'CDOPERADOR')
                        filtroRow[key] = filial[key] ? filial[key] : filtroRow[key];
                }
                else if (container.name == 'est42000_implantacao_estoque') {
                    if (defaultFields.includes(key))
                        filtroRow[key] = key == 'CDOPERADOR' ? '' : ' ';
                    else
                        filtroRow[key] = filial[key] ? filial[key] : filtroRow[key];
                }
            });

            sessionStorage.setItem('IDUTILALMOX', filial.IDUTILALMOX);
            sessionStorage.setItem('IDUTILLCESTQ', filial.IDUTILLCESTQ);
            sessionStorage.setItem('IDUTILLOTE', filial.IDUTILLOTE);
            sessionStorage.setItem('FILIAL_CTRLESTQE', filial.IDCTRLESTQ);
            filutlote = filial.IDUTILLOTE == 'S' ? 'S' : 'N';

            if (filtroWidget.getField('DSALMOXARIFE')) {
                if (filial.IDUTILALMOX == 'S') {
                    filtroWidget.getField('DSALMOXARIFE').isVisible = true;
                }
                else {
                    filtroWidget.getField('DSALMOXARIFE').isVisible = false;
                }

                if (filial.IDUTILLCESTQ == 'S') {
                    filtroWidget.getField('DSLOCALESTOQ').isVisible = true;
                }
                else {
                    filtroWidget.getField('DSLOCALESTOQ').isVisible = false;
                }
                filtroWidget.getField('DSALMOXARIFE').reload();
            }
            filtroWidget.reload();

        });
    };

    this.getFilialLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.setFieldsGerais = function(widget) {
        templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
    };

    //Função que ajusta casas decimais de valor unitario de acordo com o parametro
    //Desativada por enquanto até futura definição
    this.ajustaCasasDecimais = function() {
        var produtosWidget = getProdutosWidget();
        var fieldVrLiq = produtosWidget.getField('VRUNILANCTO');
        var fieldVrBrut = produtosWidget.getField('VRUNITBRUT');
        /* General.callRestEngine('df#/crud/paramgeral', 'FilterData', []).then(function(success) {
             if(success.dataset.paramgeral[0]){
                 var paramgeral = success.dataset.paramgeral[0];
                 fieldVrLiq.mask.params.precision = paramgeral.NRDECITOTANF;
                 fieldVrBrut.mask.params.precision = paramgeral.NRDECITOTANF;
             }
         });*/
    };

    var getProdutosWidget = function() {
        return templateManager.container.getWidget("Produtos");
    };

    this.openLibPopup = function(widget) {
        let arrAllowedTypes = ['inline_edit', 'list'];
        if (!arrAllowedTypes.some(type => type == widget.mode))
            widget.getField('QTTOTLANCTO').readOnly = true;

        if (widget.currentRow.CDARVPROD) {
            Lib00206_InfComplProduto_Controller.infComplementares(widget, widget.currentRow);
        }
        else {
            widget.currentRow.QTTOTLANCTO = null;
        }
    };

    this.openImportPopup = function(widget) {
        var popup = this.createImportPopup(widget);
        ScreenService.openPopup(popup);
    };

    this.setFiliMovi = function(widget) {
        var filiMovi = widget.appliedFilter.FILTER_ROW.CDFILIAL;
        widget.currentRow.CDFILIMOVI = filiMovi;

        if (sessionStorage.getItem('FILIAL_CTRLESTQE') == 'N') {
            ScreenService.showMessage('Filial não controla Estoque.');

            widget.getAction('Novo').isVisible = false;
            widget.getAction('importTxt').isVisible = false;
            widget.reload();
        }
        else {
            widget.getAction('Novo').isVisible = true;
            widget.getAction('importTxt').isVisible = true;
            widget.reload();
        }
    };

    this.createImportPopup = function(parentWidget) {
        var popup = parentWidget.importPopup;

        if (!popup) {
            popup = this.getPopupObject();
            popup.container = parentWidget.container;
            popup = metaDataFactory.widgetFactory(popup, parentWidget);
            parentWidget.importPopup = popup;
            popup.parent = parentWidget;
            popup.currentRow.CDORDENA = "C";
            popup.currentRow.IDGRUPPROD = "P";
            popup.currentRow.AJUSTEPOR = "P";
        }

        return popup;
    };

    this.importSwipeManager = function(action, widget = null) {
        const tabbedWidget = templateManager.container.getWidget('tabImportWidget');
        const widgetErrors = templateManager.container.getWidget('errorsWidget');

        if (action === 'onOpen') {
            let isAjuste = widget.name === 'est41000_ajusteInventario' ? true : false;

            this.onManageXlsModelFields(isAjuste).then(() => {
                ScreenService.openSwipe(tabbedWidget).then(() => {
                    if (widgetErrors.dataSource.data.length != 0) widgetErrors.dataSource.data = [];
                });

            });
        }
        else {
            widgetErrors.dataSource.data.pop();
            ScreenService.closeSwipe();
        }
    };

    this.getPopupObject = function() {
        return {
            "isVisible": true,
            "name": "importPopup",
            "label": "Importação de Produtos por Grupo",
            "template": "widget/form.html",
            "actions": [
                {
                    "name": "cancel",
                    "label": "Cancelar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup()"
                        }
                    ]
                },
                {
                    "name": "confirmAction",
                    "label": "Confirmar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "Ajuste_Controller.importProducts(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "AJUSTEPOR",
                    "label": "Agrupamento por",
                    "template": "field/select-native.html",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 2,
                    "hideEmptyValue": true,
                    "defaultValue": "P",
                    "dataSource": {
                        "data": [
                            {
                                "IDAJUSTEPOR": "P",
                                "AJUSTEPOR": "Produto"
                            },
                            {
                                "IDAJUSTEPOR": "U",
                                "AJUSTEPOR": "Unidade"
                            }
                        ]
                    },
                    "descriptionField": "AJUSTEPOR",
                    "valueField": "IDAJUSTEPOR",
                    "outData": {
                        "AJUSTEPOR": "AJUSTEPOR",
                        "IDAJUSTEPOR": "IDAJUSTEPOR"
                    },
                    "validations": {
                        "required": "Campo Obrigatório"
                    },
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "Ajuste_Controller.habilitaCamposImportacao(args.owner.field.widget)"
                        }
                        ]
                },
                {
                    "name": "IDGRUPPROD",
                    "label": "Origem",
                    "template": "field/select-native.html",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 2,
                    "hideEmptyValue": true,
                    "defaultValue": "P",
                    "dataSource": {
                        "data": [
                            {
                                "IDGRUPPROD": "P",
                                "DSGRUPPROD": "Padrão"
                            },
                            {
                                "IDGRUPPROD": "F",
                                "DSGRUPPROD": "Unidade"
                            }
                        ]
                    },
                    "descriptionField": "DSGRUPPROD",
                    "valueField": "IDGRUPPROD",
                    "outData": {
                        "IDGRUPPROD": "IDGRUPPROD",
                        "DSGRUPPROD": "DSGRUPPROD"
                    },
                    "validations": {
                        "required": "Campo Obrigatório"
                    },
                    "events": []
                },
                {
                    "name": "CDORDENA",
                    "label": "Ordenar Por",
                    "template": "field/select-native.html",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 2,
                    "hideEmptyValue": true,
                    "dataSource": {
                        "data": [
                            {
                                "CDORDENA": "C",
                                "NMORDENA": "Código"
                            },
                            {
                                "CDORDENA": "N",
                                "NMORDENA": "Nome"
                            }
                        ]
                    },
                    "defaultValue": "C",
                    "descriptionField": "NMORDENA",
                    "valueField": "CDORDENA",
                    "outData": {
                        "CDORDENA": "CDORDENA",
                        "NMORDENA": "NMORDENA"
                    },
                    "validations": {
                        "required": "Campo Obrigatório"
                    },
                    "events": []
                },
                {
                    "name": "NMGRUPPROD",
                    "label": "Agrupamento de Produtos",
                    "template": "field/select.html",
                    "isVisible": true,
                    "showOnForm": true,
                    "showOnList": true,
                    "class": 6,
                    "showSelectAs": "grid",
                    "dataSource": {
                        "rest": true,
                        "name": "est_ajuste#/crud/grupprod_ajuste"
                    },
                    "descriptionField": "NMGRUPPROD",
                    "valueField": "CDGRUPPROD",
                    "outData": {
                        "CDGRUPPROD": "CDGRUPPROD",
                        "NMGRUPPROD": "NMGRUPPROD"
                    },
                    "order": "CDGRUPPROD",
                    "validations": {
                        "required": "Campo Obrigatório"
                    },
                    "fields": [
                        {
                            "label": "Código",
                            "name": "CDGRUPPROD",
                            "showOnList": true
                        },
                        {
                            "label": "Agrupamento",
                            "name": "NMGRUPPROD",
                            "showOnList": true
                        }
                    ],
                    "events": []
                }
            ],
            "widgets": []
        };
    };

    this.setAjusteImportTxt = function(widget) {
        widget.currentRow.DSLANCESTQ = "Lançamento realizado por importação";
        return Ajuste_Service.getOperadorLogado().then((operador) => {
            widget.currentRow.NMOPERADOR = operador[0].NMOPERADOR;
            widget.currentRow.CDOPERADOR = operador[0].CDOPERADOR;
            widget.currentRow.CDFILIAL = operador[0].CDFILIALSELECIONADA;
            widget.currentRow.CDFILIMOVI = widget.appliedFilter.FILTER_ROW.CDFILIAL;
            widget.currentRow.CONTAINER = widget.container.name;

            return this.getDtprocessa(operador[0].CDFILIALSELECIONADA).then((DTPROCESSA) => {
                widget.currentRow.DTLANCESTQ = (DTPROCESSA) ? DTPROCESSA : moment().format('DD/MM/YYYY');
            });
        });
    };

    this.importarProdutosTxt = function(widget) {

        if (!widget.isValid()) return;

        var mainWidget = widget.container.getWidget();
        var ignoredKeys = ["__rightActions", "__createdLocal", "__index", "$error"];
        const modelXlsWidget = templateManager.container.getWidget('modelWidget');
        const widgetAjuste = templateManager.container.getWidget('est41000_ajusteInventario');
        const catchManager = (error) => {
            if (typeof error.dataSet !== 'undefined' && !Util.isEmptyOrBlank(error.dataset.itensWithErrors))
                this.showImportationDivergencies(error.dataset.itensWithErrors);

            if (error.data) {
                if (error.data.error.includes('XLS_READER_FILE_FORMAT_ERROR')) {
                    if (widget.container.name == 'est41000_ajuste_inventario') {
                        ScreenService.goBack().then(() => {
                            ScreenService.closeSwipe();
                        });
                    }
                    MessageService.showMessageByCode('INVALID_XLS_FILE');
                }
                else
                    ScreenService.showMessage(error.data.error);
            }
            else
                ScreenService.showMessage(error);
            ScreenService.hideLoader();
        };

        ScreenService.showLoader();
        if (widget.container.name == 'est41000_ajuste_inventario')
            mainWidget.newRow();

        this.setAjusteImportTxt(mainWidget).then(() => {
            widget.currentRow.CDFILIAL = mainWidget.currentRow.CDFILIAL;
            widget.currentRow.DTLANCESTQ = mainWidget.currentRow.DTLANCESTQ;
            widget.currentRow.CONTAINER = widget.container.name;
            widget.currentRow.CDFILIMOVI = mainWidget.appliedFilter.FILTER_ROW.CDFILIAL;
            //check if the file is xls or xlsx
            if (!widget.currentRow.UPLOADFILE[0].type.includes('text/plain')) {
                let file = widget.currentRow.UPLOADFILE;
                let pos = file[0].b64File.indexOf(',') + 1;

                widget.currentRow.CAMPOS = modelXlsWidget.fields.filter(field => field.isVisible == true).map(field => field.name);
                widget.currentRow.FILE = {
                    'name': file[0].name,
                    'content': file[0].b64File.substr(pos)
                };
            }
            Ajuste_Service.importarProdutosTxt(Util.clone(widget.currentRow, ignoredKeys)).then(success => {
                var itensAjuste = success.dataset.itensAjuste;
                var itensImplantacao = success.dataset.itensImplantacao;
                var produtosDataSource = templateManager.container.getWidget('Produtos').dataSource;
                ScreenService.closeSwipe().then(() => {
                    mainWidget.activate().then(() => {
                        if (itensImplantacao.length > 0) {
                            if (widget.container.name == 'est41000_ajuste_inventario') {
                                ScreenService.hideLoader();
                                ScreenService.confirmMessage("Existem produtos sem custo em estoque. Deseja gerar implantação para estes produtos?", "CONFIRMATION",
                                    () => {
                                        itensImplantacao.forEach((row) => {
                                            row.NRSEQUITEM = Ajuste_Service.geraNrsequitem(itensImplantacao);
                                            row.__is_new = true;
                                            row.changed = true;
                                            row.IDTIPOMOVI = '0';
                                        });

                                        var rowAjuste = Util.clone(mainWidget.currentRow, ignoredKeys.concat(["ITENS"]));
                                        rowAjuste.__is_new = true;
                                        rowAjuste.IDTPLANCTO = '0';
                                        rowAjuste.NRLANCESTQ = "";
                                        rowAjuste.DSLANCESTQ = "Lançamento realizado por importação de inventário";
                                        rowAjuste.DTLANCESTQ = itensImplantacao[0].DTLANCESTQ;

                                        Ajuste_Service.gerarImplantacao(rowAjuste, itensImplantacao).then(() => {
                                            MessageService.successNotificationByCode('IMPLANTATION_SUCCESS');
                                            mainWidget.activate();
                                            mainWidget.reload().then(_ => {
                                                ScreenService.closeSwipe();
                                                if (widget.container.name == 'est42000_implantacao_estoque') {
                                                    ScreenService.closeSwipe();
                                                    widget.container.getWidget('est42000_implantacao_estoque').reload();
                                                }
                                            });
                                        }).catch(error => {
                                            if (error.data) {
                                                ScreenService.showMessage(error.data.error);
                                            }
                                            else {
                                                ScreenService.showMessage(error);
                                            }
                                        });
                                    }, () => {
                                        mainWidget.activate();
                                        mainWidget.reload().then(_ => {
                                            ScreenService.closeSwipe();
                                        });
                                    });
                            }
                            else { //fim widget.container.name == 'est41000_ajuste_inventario'
                                itensImplantacao.forEach((row) => {
                                    row.NRSEQUITEM = Ajuste_Service.geraNrsequitem(itensImplantacao);
                                    row.__is_new = true;
                                    row.changed = true;
                                    row.IDTIPOMOVI = '0';

                                });

                                var rowAjuste = Util.clone(mainWidget.currentRow, ignoredKeys.concat(["ITENS"]));
                                rowAjuste.__is_new = true;
                                rowAjuste.IDTPLANCTO = '0';
                                rowAjuste.NRLANCESTQ = "";
                                rowAjuste.DSLANCESTQ = "Lançamento realizado por importação de inventário";
                                rowAjuste.DTLANCESTQ = itensImplantacao[0].DTLANCESTQ;

                                Ajuste_Service.gerarImplantacao(rowAjuste, itensImplantacao).then(() => {
                                    MessageService.successNotificationByCode('IMPLANTATION_SUCCESS');
                                    ScreenService.hideLoader();
                                    mainWidget.reload();
                                    // ScreenService.closeSwipe().then(_ => {
                                    //     mainWidget.reload();
                                    //     if (widget.container.name == 'est42000_implantacao_estoque') {
                                    //         ScreenService.closeSwipe();
                                    //         widget.container.getWidget('est42000_implantacao_estoque').reload();
                                    //     }
                                    // });
                                }).catch(error => {
                                    if (error.data) {
                                        ScreenService.showMessage(error.data.error);
                                    }
                                    else {
                                        ScreenService.showMessage(error);
                                    }
                                });
                            }
                        }
                        else if (itensAjuste.length == 0) {
                            ScreenService.showMessage("Não foram encontrados nenhum produto a partir do arquivo");
                        }

                        if (itensAjuste.length > 0) {
                            widgetAjuste.currentRow.DTLANCESTQ = itensAjuste[0].DTLANCESTQ;
                            produtosDataSource.data = itensAjuste;
                            produtosDataSource.data.forEach((row) => {
                                row.NRSEQUITEM = Ajuste_Service.geraNrsequitem(produtosDataSource.data);
                                row.IDAJUSTEIMPORT = 'S';
                                row.__is_new = true;
                                row.d = true;
                                row.IDTIPOMchangeOVI = row.IDTIPOMOVI ? row.IDTIPOMOVI : '1';
                            });
                            this.saveImport(widget).then(() => {
                                ScreenService.closeSwipe().then(_ => {
                                    widgetAjuste.reload();
                                    widgetAjuste.list();
                                    MessageService.successNotificationByCode('ADJUST_SUCCESS');
                                });
                            });

                        }
                        else if (itensImplantacao.length == 0) {
                            ScreenService.showMessage("Não foram encontrados nenhum produto a partir do arquivo");
                        }

                    });
                });
            }).catch((error) => {
                ScreenService.closeSwipe().then(() => {
                    ScreenService.closeSwipe();
                    widget.reload();
                }).then(() => {
                    catchManager(error);
                });
            });
        });
    };

    this.showImportationDivergencies = function(errorData) {
        let adjustWidget = templateManager.container.getWidget('est41000_ajusteInventario');
        let tabWidget = templateManager.container.getWidget('tabImportWidget');
        let widgetErrors = templateManager.container.getWidget('errorsWidget');

        adjustWidget.list(true);
        adjustWidget.dataSource.data.pop();
        adjustWidget.currentRow = {};

        ScreenService.closeSwipe().then(_ => {
            widgetErrors.dataSource.data = errorData;
            tabWidget.isVisible = true;
            widgetErrors.reload().then(_ => {
                widgetErrors.activate();
            });
        });
    };

    this.importProducts = function(widget) {
        if (widget.isValid()) {
            try {
                if (widget.getField("AJUSTEPOR").value() === "U") {
                    ScreenService.showLoader();
                    Ajuste_Service.retornaParamUnidade(widget.getParent().getParent().currentRow.CDFILIMOVI).then(function(paramfilial) {
                        Ajuste_Service.importProductsFilial(widget.currentRow.CDORDENA, widget.getParent().getParent().currentRow.CDFILIMOVI, widget.getParent().getParent().currentRow.DTLANCESTQ, paramfilial.dataset.paramfilial[0].DTPROCESSA).then(function(success) {
                            var products = success.dataset.products;
                            var parentDataSource = widget.getParent().dataSource;

                            if (products.length > 0) {
                                parentDataSource.data = products;

                                parentDataSource.data.forEach(function(row) {
                                    row.NRSEQUITEM = Ajuste_Service.geraNrsequitem(parentDataSource.data);
                                    row.IDAJUSTEIMPORT = 'S';
                                    row.__is_new = true;
                                    row.changed = true;
                                    row.CDFILIAL = widget.getParent().getParent().currentRow.CDFILIAL;
                                    //row.IDTIPOMOVI = row.IDTIPOMOVI ? row.IDTIPOMOVI : '1';
                                    row.IDTIPOMOVI = (widget.getParent().getParent().currentRow.IDTPLANCTO == 0) ? '0' : '1';
                                });

                                //O CÓDIGO DESSA TELA É INACEITÁVEL E É IMPERATIVO QUE ESTE SEJA COMPLETAMENTE REESCRITO.
                                this.saveImport(widget);

                                ScreenService.closePopup();
                            }
                            else {
                                ScreenService.hideLoader();
                                ScreenService.confirmMessage("A importação de itens não encontrou produtos válidos cadastrados para a origem selecionada. Inserir manualmente?", "CONFIRMATION", function confimed() {
                                    ScreenService.closePopup();

                                    this.newItem(widget.getParent());
                                }.bind(this));
                            }

                        }.bind(this), (error) => {
                            ScreenService.showMessage(error.error);
                            ScreenService.hideLoader();
                        });
                    }.bind(this));
                }
                else {
                    ScreenService.showLoader();
                    Ajuste_Service.importProducts(widget.currentRow.CDGRUPPROD, widget.currentRow.CDORDENA, widget.getParent().getParent().currentRow.CDFILIMOVI, widget.getParent().getParent().currentRow.DTLANCESTQ, widget.currentRow.IDGRUPPROD).then(function(success) {
                        var products = success.dataset.products;
                        var parentDataSource = widget.getParent().dataSource;

                        if (products.length > 0) {
                            parentDataSource.data = products;

                            parentDataSource.data.forEach(function(row) {
                                row.NRSEQUITEM = Ajuste_Service.geraNrsequitem(parentDataSource.data);
                                row.IDAJUSTEIMPORT = 'S';
                                row.__is_new = true;
                                row.changed = true;
                                row.CDFILIAL = widget.getParent().getParent().currentRow.CDFILIAL;
                                //row.IDTIPOMOVI = row.IDTIPOMOVI ? row.IDTIPOMOVI : '1';
                                row.IDTIPOMOVI = (widget.getParent().getParent().currentRow.IDTPLANCTO == 0) ? '0' : '1';
                            });

                            //O CÓDIGO DESSA TELA É INACEITÁVEL E É IMPERATIVO QUE ESTE SEJA COMPLETAMENTE REESCRITO.
                            this.saveImport(widget);

                            ScreenService.closePopup();
                        }
                        else {
                            // ScreenService.confirmMessage("A importação de itens a partir do agrupamento \"" + widget.currentRow.NMGRUPPROD + "\" não encontrou produtos válidos cadastrados para a origem selecionada. Inserir manualmente?", "CONFIRMATION", function confimed() {
                            //     ScreenService.closePopup();

                            //     this.newItem(widget.getParent());
                            // }.bind(this));
                            ScreenService.hideLoader();

                            MessageService.confirmMessageByCode("ERRO_IMPORT_AGRUPAMENTO", { "PARAMETRO": widget.currentRow.NMGRUPPROD }).then(function() {
                                ScreenService.closePopup();

                                this.newItem(widget.getParent());

                            }.bind(this));
                        }

                    }.bind(this), (error) => {
                        ScreenService.showMessage(error.error);
                        ScreenService.hideLoader();
                    });
                }
            }
            catch (error) {
                ScreenService.showMessage(error);
                ScreenService.hideLoader();
            }
        }
    };

    this.saveImport = (widget) => new Promise(resolve => {
        var widgetItem = widget.container.getWidget('Produtos');

        //SALVA A PRIMEIRA ROW + AJUSTE COM UM REQUEST
        //CONTROLE DE ALMOXARIFADO--colocar aqui
        var rowItem = {};
        Util.extend(rowItem, widgetItem.dataSource.data[0]);
        rowItem.CDFILIMOVI = widgetItem.getParent().currentRow.CDFILIMOVI;
        rowItem.IDAJUSTEIMPORT = 'S';
        var widgetLancamento = widgetItem.getParent();
        var rowAjuste = widgetLancamento.currentRow;
        delete rowItem.__rightActions;
        var closureObj = {};
        closureObj.rowItem = rowItem;
        closureObj.widgetItem = widgetItem;
        if (widget.currentRow.CDGRUPPROD) //codigo do grupo da importação
            rowAjuste.CDGRUPPRODLCEST = widget.currentRow.CDGRUPPROD;

        ScreenService.showLoader();

        Ajuste_Service.validaAjuste(rowItem, rowAjuste.DTLANCESTQ).then(function(success) {
            ValidacoesEstoque_Service.validaDataLancamento(rowAjuste.CDFILIMOVI, rowAjuste.DTLANCESTQ, 'AJUSTE_IMP', 'EST', null).then(function(success) {
                Ajuste_Service.save(rowItem, rowAjuste).then(function(lancamento) {
                    ScreenService.hideLoader();
                    widgetItem.getAction('import').isVisible = false;

                    if (lancamento.dataset['/lanctoestoq/save'] !== undefined) {
                        var rowLancamento = lancamento.dataset['/lanctoestoq/save'];
                        var itens = rowLancamento.ITENS;
                        delete rowLancamento.itens;
                        widgetItem.dataSource.data[0] = itens[0];
                        widgetLancamento.currentRow.NRLANCESTQ = rowLancamento.NRLANCESTQ;
                        widgetLancamento.currentRow.__is_new = false;
                    }
                    else {
                        widgetLancamento.currentRow.NRLANCESTQ = rowAjuste.NRLANCESTQ;
                    }

                    widgetLancamento.view();
                    widgetItem.dataSource.data[0].IDEVOLESTOQ = 'N';
                    widgetItem.dataSource.data[0].__is_new = false;
                    widgetItem.dataSource.data[0].changed = false;
                    widgetItem.dataSource.data[0].NRLANCESTQ = widgetLancamento.currentRow.NRLANCESTQ;
                    //widgetItem.updateGridActions();

                    //SALVA AS DEMAIS ROWS
                    var dataSemCusto = widgetItem.dataSource.data.filter(produto => {
                        if (produto.hasOwnProperty('CUSTO'))
                            return produto.CUSTO.valrMed == 0;
                        else
                            return false;
                    });
                    var dataComCusto = widgetItem.dataSource.data.filter(produto => {
                        if (produto.hasOwnProperty('CUSTO'))
                            return produto.CUSTO.valrMed != 0;
                        else
                            return true;
                    });

                    if (dataSemCusto.length > 0) {
                        ScreenService.confirmMessage("Existem produtos sem custo em estoque. Deseja gerar implantação para estes produtos?", "CONFIRMATION", () => {
                            dataSemCusto.forEach((row) => {
                                row.NRSEQUITEM = Ajuste_Service.geraNrsequitem(dataSemCusto);
                                row.__is_new = true;
                                row.changed = true;
                                row.IDTIPOMOVI = '0';
                            });

                            var ignoredKeys = ["__rightActions", "__createdLocal", "__index", "$error"];
                            var rowAjuste = Util.clone(widgetLancamento.currentRow, ignoredKeys.concat(["ITENS"]));
                            rowAjuste.__is_new = true;
                            rowAjuste.IDTPLANCTO = '0';
                            rowAjuste.NRLANCESTQ = "";
                            rowAjuste.DSLANCESTQ = "Lançamento realizado por importação de inventário";
                            rowAjuste.DTLANCESTQ = dataSemCusto[0].DTLANCESTQ;

                            Ajuste_Service.gerarImplantacao(rowAjuste, dataSemCusto).then(() => {
                                MessageService.successNotificationByCode('IMPLANTATION_SUCCESS');
                                ScreenService.hideLoader();
                                resolve();
                            }).catch(error => {
                                if (error.data) {
                                    ScreenService.showMessage(error.data.error);
                                    ScreenService.hideLoader();
                                }
                                else {
                                    ScreenService.showMessage(error);
                                    ScreenService.hideLoader();
                                }
                            });
                        });
                    }

                    widgetItem.dataSource.data = dataComCusto;
                    ScreenService.hideLoader(); // verificar se este hideloader pode realmente ficar aqui. Pois em caso de lentidao tem chance de perder os registros
                    Ajuste_Service.saveImportRows(dataComCusto, Util.clone(rowAjuste, ["__rightActions", "__leftActions"])).then(function(success) {
                        widgetItem.dataSource.data.forEach(function(row) {
                            row.IDEVOLESTOQ = 'N';
                            row.IDAJUSTEIMPORT = 'S';
                            row.__is_new = false;
                            row.changed = false;
                            row.NRLANCESTQ = widgetLancamento.currentRow.NRLANCESTQ;
                        });
                        //widgetItem.updateGridActions();
                        widgetItem.list();
                        //widgetItem.reload(); nao perder o calculo do vrunibrut
                        resolve();
                        ScreenService.hideLoader();
                    }, function(error) {
                        ScreenService.showMessage(error.data.error);
                        ScreenService.hideLoader();
                    });
                }, function(error) {
                    ScreenService.showMessage(error.data.error);
                    ScreenService.hideLoader();
                });
            }, function(error) {
                ScreenService.showMessage(error.data.error);
                ScreenService.hideLoader();
            });
        }, function(error) {
            var closure = this[1];
            closure.rowItem.SEMCUSTO = true;
            closure.widgetItem.dataSource.data[0].SEMCUSTO = true;
            ScreenService.showMessage(error.data.error);
            ScreenService.hideLoader();
        }.bind([this, closureObj]));
    });

    this.onSaveAjuste = function(widget) {
        var widgetItens = widget.widgets[0];
        var row = Util.clone(widget.currentRow, ['__rightActions', '__leftActions']);

        if (widget.isValid()) {
            if (row.__is_new) {
                ScreenService.showLoader();
                ValidacoesEstoque_Service.validaDataLancamento(row.CDFILIMOVI, row.DTLANCESTQ, 'AJUSTE_IMP', 'EST', null).then(function() {
                    if (widgetItens.getAction('import'))
                        widgetItens.getAction('import').isVisible = true;

                    widget.container.getWidget('Produtos').enabled = true;

                    widgetItens.activate();
                    ScreenService.hideLoader();
                }, function(error) {
                    ScreenService.showMessage(error.data.error);
                    ScreenService.hideLoader();
                });
            }
            else {
                Ajuste_Service.editAjuste(row).then(function() {
                    widget.view();
                    widget.container.getWidget('Produtos').enabled = true;
                });
            }
        }
    };

    this.openNewRowOnProdutoGrid = function(widget) {
        if (widget.getParent().currentRow.__is_new && widget.mode === 'list') {
            widget.dataSource.data = [];
            widget.newRow();
            ScreenService.showMessage('É necessário ao menos um produto para efetuar o lançamento!');
        }
    };

    var rowEditedInline = function(row) {
        let boolEdited = false;

        $.each(row, function(index, value) {
            if (row.hasOwnProperty('changed_' + index)) {
                boolEdited = true;
            }
        });

        return boolEdited;
    };

    this.newOnSaveItemBackend = function(widgetItem) {
        var self = this;
        if (widgetItem.isValid()) {
            var rows = widgetItem.dataSource.data;

            rows = rows.filter(function(row) {
                return row.changed === true && (!row.__is_new || rowEditedInline(row));
            });

            if (widgetItem.currentRow.__is_new) {
                if (Ajuste_Service.validaItem(widgetItem.currentRow, widgetItem.dataSource.data)) {
                    var boolDontPush = rows.some(row => widgetItem.currentRow === row);
                    if (!boolDontPush)
                        rows.push(widgetItem.currentRow);
                }
                else
                    ScreenService.showMessage('Produto já foi incluido neste lançamento!');
            }
            ScreenService.showLoader();
            Ajuste_Service.retornaParamUnidade(widgetItem.getParent().currentRow.CDFILIMOVI).then(paramfilial => {
                var closure = {};
                var param = paramfilial.dataset.paramfilial[0];
                var rowAjuste = widgetItem.getParent().currentRow;
                var mensagemCusto = param.IDVISPOSAJINVE == 'S' && ((rowAjuste.IDTIPOMOVI == '1' ? true : false) || (rowAjuste.IDTPLANCTO == '1' ? true : false));
                var dtLanc = rowAjuste.DTLANCESTQ;
                var popupCustoProduto = templateManager.container.getWidget('PopUpCustoProduto');
                var arrayCustos = [];
                var arrayItemsToSave = [];
                var validacoesProcessadas = 0;
                var rowsLength = rows.length;
                let qntValida = true;
                var rowGetMax = {
                    CDFILIAL: widgetItem.getParent().currentRow.CDFILIAL,
                    NRLANCESTQ: widgetItem.getParent().currentRow.NRLANCESTQ
                };

                closure.validacoesProcessadas = validacoesProcessadas;

                rows.forEach(row => {
                    row.CDFILIMOVI = row.CDFILIMOVI ? row.CDFILIMOVI : widgetItem.getParent().currentRow.CDFILIMOVI;
                    var rowItem = {};
                    var arraySemcusto = [];

                    Util.extend(rowItem, row);
                    delete rowItem.__rightActions;

                    rowItem.VRUNITBRUT = rowItem.VRUNITBRUT_inline || rowItem.VRUNITBRUT;
                    rowItem.VRUNILANCTO = rowItem.VRUNILANCTO_inline || rowItem.VRUNILANCTO;
                    rowItem.QTTOTLANCTO = rowItem.QTTOTLANCTO_inline || rowItem.QTTOTLANCTO;

                    arrayItemsToSave.push(rowItem);
                });

                General.callRestEngine("est_ajuste#/validaAjusteTotal", "DataSet", arrayItemsToSave).then((response) => {
                    var dataSetComCusto = response.dataset.custos;
                    var notEditedDataset = widgetItem.dataSource.data.filter(row => {
                        return !dataSetComCusto.some(custo => custo.NRSEQUITEM == row.NRSEQUITEM);
                    });

                    if (dataSetComCusto.some(produto => produto.SEMCUSTO != "false"))
                        ScreenService.showMessage('Produto com custo de estoque zerado.');

                    if (dataSetComCusto.some(produto => produto.IMPLANTADOMESMODIA != "false"))
                        ScreenService.showMessage('Produto com implantação no mesmo dia.');

                    let arr = dataSetComCusto;

                    for (let i = 0, len = arr.length; i < len; i++) {
                        let custo = {},
                            row = arr[i];
                        custo.valrAnte = row.VALRANTE;
                        custo.qtdeAnte = row.QTDEANTE;
                        custo.valrMedi = row.VALRMEDI;
                        custo.vlAnterior = row.VLANTERIOR;
                        custo.item = Util.clone(row);
                        Util.extend(row, custo);
                    }

                    // se mensagemCusto for false a validação não será feita no Ajuste_Service.openPopupCusto
                    if (!mensagemCusto && dataSetComCusto) {
                        dataSetComCusto.forEach(row => {
                            var boolValido = qntValida === false ? qntValida : this.validaCusto(row, row);
                            if (!boolValido) {
                                ScreenService.showMessage('Quantidade informada fora do Permitido');
                                qntValida = false;
                            }
                        });
                    }

                    if (mensagemCusto) {
                        ScreenService.hideLoader();
                        Ajuste_Service.openPopupCusto(dataSetComCusto, popupCustoProduto).then(() => {
                            ScreenService.showLoader();
                            let dataCustoToSave = dataSetComCusto.filter(row => {
                                return ((row.SEMCUSTO ? row.SEMCUSTO == "false" : true) &&
                                    (row.IRREGULAR ? row.IRREGULAR == "false" : true) &&
                                    (row.qntValida ? row.qntValida == true : true) &&
                                    (row.IMPLANTADOMESMODIA ? row.IMPLANTADOMESMODIA == "false" : true));
                            });

                            let dataNeedNrseq = dataCustoToSave.filter(row => {
                                return !row.NRSEQUITEM;
                            });

                            General.callRestEngine("est_ajuste#/getMaxNrSeqItemAjuste", "Row", rowGetMax).then((response) => {
                                let nextSeq = response.dataset.getMaxNrSeqItemAjuste[0];

                                let dataSetWithSeq = dataCustoToSave.filter(row => {
                                    return !dataNeedNrseq.some(data => data.NRSEQUITEM == row.NRSEQUITEM);
                                });
                                if (dataNeedNrseq.length > 0) {
                                    let arr = dataNeedNrseq;
                                    for (let i = 0, len = arr.length; i < len; i++) {
                                        let cRow = arr[i];
                                        cRow.NRSEQUITEM = nextSeq + i;
                                    }

                                    dataSetWithSeq = dataSetWithSeq.concat(dataNeedNrseq);
                                }

                                Ajuste_Service.saveImportRows(dataSetWithSeq, rowAjuste).then(function(success) {
                                        widgetItem.dataSource.data = dataSetComCusto.concat(notEditedDataset);
                                        widgetItem.dataSource.data.forEach(function(row) {
                                            row.IDEVOLESTOQ = 'N';
                                            row.__is_new = false;
                                            for (var e in row) {
                                                if (e.indexOf('changed') !== -1 || e.indexOf('_inline') !== -1)
                                                    delete row[e];
                                            }
                                            row.changed = false;
                                        });

                                        //widgetItem.updateGridActions();
                                        widgetItem.list();
                                        ScreenService.hideLoader();
                                    },
                                    function(error) {
                                        ScreenService.showMessage(error.data.error);
                                        ScreenService.hideLoader();
                                    });

                            });

                        });
                    }
                    else {
                        if (qntValida) {
                            let dataCustoToSave = dataSetComCusto.filter(row => {
                                return ((row.SEMCUSTO ? row.SEMCUSTO == "false" : true) &&
                                    (row.IRREGULAR ? row.IRREGULAR == "false" : true) &&
                                    (row.qntValida ? row.qntValida == true : true));
                            });

                            let dataNeedNrseq = dataCustoToSave.filter(row => {
                                return !row.NRSEQUITEM;
                            });

                            General.callRestEngine("est_ajuste#/getMaxNrSeqItemAjuste", "Row", rowGetMax).then((response) => {
                                let nextSeq = response.dataset.getMaxNrSeqItemAjuste[0];

                                let dataSetWithSeq = dataCustoToSave.filter(row => {
                                    return !dataNeedNrseq.some(data => data.NRSEQUITEM == row.NRSEQUITEM);
                                });
                                if (dataNeedNrseq.length > 0) {
                                    let arr = dataNeedNrseq;
                                    for (let i = 0, len = arr.length; i < len; i++) {
                                        let cRow = arr[i];
                                        cRow.NRSEQUITEM = nextSeq + i;
                                    }

                                    dataSetWithSeq = dataSetWithSeq.concat(dataNeedNrseq);
                                }

                                Ajuste_Service.saveImportRows(dataSetWithSeq, rowAjuste).then(function(success) {
                                        widgetItem.dataSource.data = dataSetComCusto.concat(notEditedDataset);
                                        widgetItem.dataSource.data.forEach(function(row) {
                                            row.IDEVOLESTOQ = 'N';
                                            row.__is_new = false;
                                            for (var e in row) {
                                                if (e.indexOf('changed') !== -1 || e.indexOf('_inline') !== -1)
                                                    delete row[e];
                                            }
                                            row.changed = false;
                                        });

                                        //widgetItem.updateGridActions();
                                        widgetItem.list();
                                        ScreenService.hideLoader();
                                    },
                                    function(error) {
                                        ScreenService.showMessage(error.data.error);
                                        ScreenService.hideLoader();
                                    });

                            });
                        }
                    }
                    var newDataset = notEditedDataset.concat(dataSetComCusto);
                    widgetItem.dataSource.data = newDataset;
                });
            });

        }
    };

    this.onSaveItem = function(widgetItem) {
        var self = this;
        if (widgetItem.isValid()) {
            ScreenService.showLoader();
            var rows = widgetItem.dataSource.data;

            rows = rows.filter(function(row) {
                return row.changed === true && (!row.__is_new || rowEditedInline(row));
            });

            if (widgetItem.currentRow.__is_new) {
                if (Ajuste_Service.validaItem(widgetItem.currentRow, widgetItem.dataSource.data)) {
                    var boolDontPush = rows.some(row => widgetItem.currentRow === row);
                    if (!boolDontPush)
                        rows.push(widgetItem.currentRow);
                }
                else {
                    ScreenService.showMessage('Produto já incluído neste lançamento!');
                    ScreenService.hideLoader();
                }
            }

            //NOVO DAQUI PARA BAIXO
            //VALIDACOES
            //Foreach do popupCusto
            Ajuste_Service.retornaParamUnidade(widgetItem.getParent().currentRow.CDFILIMOVI).then(paramfilial => {
                var closure = {};
                var param = paramfilial.dataset.paramfilial[0];
                var rowAjuste = widgetItem.getParent().currentRow;
                var mensagemCusto = param.IDVISPOSAJINVE == 'S' && ((rowAjuste.IDTIPOMOVI == '1' ? true : false) || (rowAjuste.IDTPLANCTO == '1' ? true : false));
                var dtLanc = rowAjuste.DTLANCESTQ;
                var popupCustoProduto = templateManager.container.getWidget('PopUpCustoProduto');
                var arrayCustos = [];
                var arrayItemsToSave = [];
                var validacoesProcessadas = 0;
                var rowsLength = rows.length;
                let qntValida = true;

                closure.validacoesProcessadas = validacoesProcessadas;

                //if(rows.length == 0)
                //  ScreenService.hideLoader();
                rows.forEach(row => {
                    row.CDFILIMOVI = row.CDFILIMOVI ? row.CDFILIMOVI : widgetItem.getParent().currentRow.CDFILIMOVI;
                    var rowItem = {};
                    var arraySemcusto = [];
                    var rowGetMax = {
                        CDFILIAL: widgetItem.getParent().currentRow.CDFILIAL,
                        NRLANCESTQ: widgetItem.getParent().currentRow.NRLANCESTQ
                    };

                    Util.extend(rowItem, row);
                    delete rowItem.__rightActions;

                    rowItem.VRUNITBRUT = rowItem.VRUNITBRUT_inline || rowItem.VRUNITBRUT;
                    rowItem.VRUNILANCTO = rowItem.VRUNILANCTO_inline || rowItem.VRUNILANCTO;
                    rowItem.QTTOTLANCTO = rowItem.QTTOTLANCTO_inline || rowItem.QTTOTLANCTO;
                    closure.rowItem = rowItem;
                    var closureObj = {};
                    closureObj.closure = closure;
                    closureObj.row = row;
                    closureObj.rowItem = rowItem;

                    Ajuste_Service.validaAjuste(rowItem, dtLanc).then(function(success) {
                        if (success) {
                            var custo = success.dataset.custo;
                            custo.CDARVPROD = rowItem.CDARVPROD;
                            custo.NMPRODUTO = rowItem.NMPRODUTO;
                            custo.item = rowItem;
                            arrayCustos.push(custo);

                            // se mensagemCusto for false a validação não será feita no Ajuste_Service.openPopupCusto
                            if (!mensagemCusto && success) {
                                qntValida = qntValida === false ? qntValida : this.validaCusto(custo, row);
                            }
                        }

                        ValidacoesEstoque_Service.validaDataLancamento(rowAjuste.CDFILIMOVI, rowAjuste.DTLANCESTQ, 'AJUSTE_IMP', 'EST', null).then(success => {
                                if (!row.NRSEQUITEM) {
                                    General.callRestEngine("est_ajuste#/getMaxNrSeqItemAjuste", "Row", rowGetMax).then((response) => {
                                        rowItem.NRSEQUITEM = response.dataset.getMaxNrSeqItemAjuste[0];
                                        arrayItemsToSave.push(rowItem);
                                        closure.validacoesProcessadas++;
                                        if (closure.validacoesProcessadas == rowsLength) {
                                            if (qntValida) {
                                                if (mensagemCusto) {
                                                    ScreenService.hideLoader();
                                                    Ajuste_Service.openPopupCusto(arrayCustos, popupCustoProduto).then(() => {
                                                        ScreenService.showLoader();
                                                        popupCustoProduto.dataSourceFilter = [];
                                                        arrayItemsToSave.forEach(item => {
                                                            Ajuste_Service.save(item, rowAjuste).then((lancamento) => {
                                                                if (widgetItem.getAction('import'))
                                                                    widgetItem.getAction('import').isVisible = false;

                                                                var widgetLancamento = widgetItem.getParent();
                                                                if (lancamento.dataset['/lanctoestoq/save'] !== undefined) {
                                                                    var rowLancamento = lancamento.dataset['/lanctoestoq/save'];
                                                                    var itens = rowLancamento.ITENS;
                                                                    delete rowLancamento.itens;

                                                                    Object.keys(itens[0]).forEach(function(key) {
                                                                        widgetItem.currentRow[key] = itens[0][key];
                                                                    });

                                                                    widgetLancamento.currentRow.NRLANCESTQ = rowLancamento.NRLANCESTQ;
                                                                    widgetLancamento.currentRow.__is_new = false;
                                                                }
                                                                else {
                                                                    widgetLancamento.currentRow.NRLANCESTQ = rowAjuste.NRLANCESTQ;
                                                                }

                                                                widgetLancamento.view();

                                                                if (rowItem.__is_new && !rowEditedInline(rowItem)) {
                                                                    ScreenService.closeSwipe().then(function() {
                                                                        widgetItem.list();
                                                                    });
                                                                }
                                                                else {
                                                                    widgetItem.commitInlineEdit();
                                                                    widgetItem.list();
                                                                }

                                                                rowItem.IRREGULAR = false;
                                                                row.IDEVOLESTOQ = 'N';

                                                                widgetItem.currentRow.__is_new = false;
                                                                widgetItem.currentRow.changed = false;
                                                                widgetItem.currentRow.NRLANCESTQ = widgetLancamento.currentRow.NRLANCESTQ;
                                                                //widgetItem.updateGridActions();
                                                                widgetItem.reload();

                                                                ScreenService.hideLoader();
                                                            }, function(error) {
                                                                ScreenService.showMessage(error.data.error);
                                                                ScreenService.hideLoader();
                                                            });
                                                        });
                                                    });
                                                }
                                                else {
                                                    arrayItemsToSave.forEach(item => {
                                                        Ajuste_Service.save(item, rowAjuste).then((lancamento) => {
                                                            if (widgetItem.getAction('import'))
                                                                widgetItem.getAction('import').isVisible = false;

                                                            var widgetLancamento = widgetItem.getParent();
                                                            if (lancamento.dataset['/lanctoestoq/save'] !== undefined) {
                                                                var rowLancamento = lancamento.dataset['/lanctoestoq/save'];
                                                                var itens = rowLancamento.ITENS;
                                                                delete rowLancamento.itens;

                                                                Object.keys(itens[0]).forEach(function(key) {
                                                                    widgetItem.currentRow[key] = itens[0][key];
                                                                });

                                                                widgetLancamento.currentRow.NRLANCESTQ = rowLancamento.NRLANCESTQ;
                                                                widgetLancamento.currentRow.__is_new = false;
                                                            }
                                                            else {
                                                                widgetLancamento.currentRow.NRLANCESTQ = rowAjuste.NRLANCESTQ;
                                                            }

                                                            widgetLancamento.view();

                                                            if (rowItem.__is_new && !rowEditedInline(rowItem)) {
                                                                ScreenService.closeSwipe().then(function() {
                                                                    widgetItem.list();
                                                                });
                                                            }
                                                            else {
                                                                widgetItem.commitInlineEdit();
                                                                widgetItem.list();
                                                            }

                                                            rowItem.IRREGULAR = false;
                                                            row.IDEVOLESTOQ = 'N';

                                                            widgetItem.currentRow.__is_new = false;
                                                            widgetItem.currentRow.changed = false;
                                                            widgetItem.currentRow.NRLANCESTQ = widgetLancamento.currentRow.NRLANCESTQ;
                                                            //widgetItem.updateGridActions();
                                                            widgetItem.reload();

                                                            ScreenService.hideLoader();
                                                        }, function(error) {
                                                            ScreenService.showMessage(error.data.error);
                                                            ScreenService.hideLoader();
                                                        });
                                                    });
                                                }
                                            }
                                            else {
                                                ScreenService.showMessage('Quantidade informada fora do permitido.');
                                                ScreenService.hideLoader();
                                            }
                                        }
                                    });
                                }
                                else {
                                    arrayItemsToSave.push(rowItem);
                                    closure.validacoesProcessadas++;
                                    if (closure.validacoesProcessadas == rowsLength) {
                                        if (qntValida) {
                                            if (mensagemCusto) {
                                                ScreenService.hideLoader();
                                                Ajuste_Service.openPopupCusto(arrayCustos, popupCustoProduto).then(() => {
                                                    ScreenService.showLoader();
                                                    popupCustoProduto.dataSourceFilter = [];
                                                    arrayItemsToSave.forEach(item => {
                                                        Ajuste_Service.save(item, rowAjuste).then((lancamento) => {
                                                            if (widgetItem.getAction('import'))
                                                                widgetItem.getAction('import').isVisible = false;

                                                            var widgetLancamento = widgetItem.getParent();
                                                            if (lancamento.dataset['/lanctoestoq/save'] !== undefined) {
                                                                var rowLancamento = lancamento.dataset['/lanctoestoq/save'];
                                                                var itens = rowLancamento.ITENS;
                                                                delete rowLancamento.itens;

                                                                Object.keys(itens[0]).forEach(function(key) {
                                                                    widgetItem.currentRow[key] = itens[0][key];
                                                                });

                                                                widgetLancamento.currentRow.NRLANCESTQ = rowLancamento.NRLANCESTQ;
                                                                widgetLancamento.currentRow.__is_new = false;
                                                            }
                                                            else {
                                                                widgetLancamento.currentRow.NRLANCESTQ = rowAjuste.NRLANCESTQ;
                                                            }

                                                            widgetLancamento.view();

                                                            if (rowItem.__is_new && !rowEditedInline(rowItem)) {
                                                                ScreenService.closeSwipe().then(function() {
                                                                    widgetItem.list();
                                                                });
                                                            }
                                                            else {
                                                                widgetItem.commitInlineEdit();
                                                                widgetItem.list();
                                                            }

                                                            rowItem.IRREGULAR = false;
                                                            row.IDEVOLESTOQ = 'N';

                                                            widgetItem.currentRow.__is_new = false;
                                                            widgetItem.currentRow.changed = false;
                                                            widgetItem.currentRow.NRLANCESTQ = widgetLancamento.currentRow.NRLANCESTQ;
                                                            //widgetItem.updateGridActions();
                                                            widgetItem.reload();
                                                            ScreenService.hideLoader();
                                                        }, function(error) {
                                                            ScreenService.showMessage(error.data.error);
                                                            ScreenService.hideLoader();
                                                        });
                                                    });
                                                });
                                            }
                                            else {
                                                arrayItemsToSave.forEach(item => {
                                                    Ajuste_Service.save(item, rowAjuste).then((lancamento) => {
                                                        if (widgetItem.getAction('import'))
                                                            widgetItem.getAction('import').isVisible = false;

                                                        var widgetLancamento = widgetItem.getParent();
                                                        if (lancamento.dataset['/lanctoestoq/save'] !== undefined) {
                                                            var rowLancamento = lancamento.dataset['/lanctoestoq/save'];
                                                            var itens = rowLancamento.ITENS;
                                                            delete rowLancamento.itens;

                                                            Object.keys(itens[0]).forEach(function(key) {
                                                                widgetItem.currentRow[key] = itens[0][key];
                                                            });

                                                            widgetLancamento.currentRow.NRLANCESTQ = rowLancamento.NRLANCESTQ;
                                                            widgetLancamento.currentRow.__is_new = false;
                                                        }
                                                        else {
                                                            widgetLancamento.currentRow.NRLANCESTQ = rowAjuste.NRLANCESTQ;
                                                        }

                                                        widgetLancamento.view();

                                                        if (rowItem.__is_new && !rowEditedInline(rowItem)) {
                                                            ScreenService.closeSwipe().then(function() {
                                                                widgetItem.list();
                                                            });
                                                        }
                                                        else {
                                                            widgetItem.commitInlineEdit();
                                                            widgetItem.list();
                                                        }

                                                        rowItem.IRREGULAR = false;
                                                        row.IDEVOLESTOQ = 'N';

                                                        widgetItem.currentRow.__is_new = false;
                                                        widgetItem.currentRow.changed = false;
                                                        widgetItem.currentRow.NRLANCESTQ = widgetLancamento.currentRow.NRLANCESTQ;
                                                        //widgetItem.updateGridActions();
                                                        widgetItem.reload();
                                                        ScreenService.hideLoader();
                                                    }, function(error) {
                                                        ScreenService.showMessage(error.data.error);
                                                        ScreenService.hideLoader();
                                                    });
                                                });
                                            }
                                        }
                                        else {
                                            ScreenService.showMessage('Quantidade informada fora do permitido.');
                                            ScreenService.hideLoader();
                                        }
                                    }
                                }
                            },
                            (error) => {
                                ScreenService.showMessage(error.data.error);
                                ScreenService.hideLoader();
                            });
                    }.bind(this)).catch(function(rejected) {
                        var closur = this[1];
                        closur.row.SEMCUSTO = true;
                        closur.closure.validacoesProcessadas++;
                        ScreenService.showMessage(rejected.data.error);
                        ScreenService.hideLoader();
                    }.bind([this, closureObj]));
                });
            });
        }
    };

    this.getDtprocessa = function(cdfilial) {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdfilial
            }
        ];
        return restEngine.post("est#/crud/paramfilial", {
            "requestType": "FilterData",
            "filter": filter
        }).then(function(response) {
            return response.dataset.paramfilial[0].DTPROCESSA;
        });
    };

    this.getFieldValueFilter = function(filedName, widgetName) {
        var container = templateManager.container;
        if (container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if (filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if (f) {
                    filterWidget = container.getWidget(widgetName).appliedFilter;
                }

            }

            if (filterWidget) {
                var valueField;
                if (filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[filedName];
                else
                    valueField = filterWidget.FILTER_ROW[filedName];

                return valueField || null;
            }
            else return null;
        }
        else return null;
    };

    this.newAjuste = function(widget) {
        if (EstUtil.onValidateFilterRequirement(widget)) return;

        let tabbedWidget = widget.container.getWidget('tabImportWidget');
        tabbedWidget.enable = false;

        ScreenService.showMessage('Atenção! O lançamento será considerado pelo sistema como o ' + Ajuste_Service.getOrdemAjuste() + '.');
        widget.newRow();

        var widgetFilter = widget.appliedFilter.FILTER_ROW;

        if (widgetFilter.CDALMOXARIFE) {
            widget.currentRow.CDALMOXARIFE = widgetFilter.CDALMOXARIFE;
        }

        widget.container.getWidget('Produtos').enabled = false;
        Ajuste_Service.getOperadorLogado().then(function(operador) {
            widget.currentRow.NMOPERADOR = operador[0].NMOPERADOR;
            widget.currentRow.CDOPERADOR = operador[0].CDOPERADOR;
            widget.currentRow.CDFILIAL = operador[0].CDFILIALSELECIONADA;
            widget.currentRow.CDFILIMOVI = widget.appliedFilter.FILTER_ROW.CDFILIAL;

            this.getDtprocessa(operador[0].CDFILIALSELECIONADA).then(function(DTPROCESSA) {
                widget.currentRow.DTLANCESTQ = (DTPROCESSA) ? DTPROCESSA : moment().format('DD/MM/YYYY');
            });
        }.bind(this));
    };

    this.newItem = function(widget) {
        var ordem = Ajuste_Service.idOrdAjuInv == 'P' ? 0 : 'U';
        var row = widget.dataSource.data[0];
        if (!row || !row.IDORDEMCLC || row.IDORDEMCLC == ordem) {
            widget.newRow();
            widget.currentRow.IDTIPOMOVI = widget.currentRow.IDTIPOMOVI == 0 ? '0' : '1';

            if (widget.currentRow.IDTIPOMOVI == null)
                widget.currentRow.IDTIPOMOVI = '1';

            if (General.getFieldValue('CDALMOXARIFE', 'est42000_implantacao_estoque'))
                widget.currentRow.CDALMOXARIFE = General.getFieldValue('CDALMOXARIFE', 'est42000_implantacao_estoque');

            if (General.getFieldValue('CDLOCALESTOQ', 'est42000_implantacao_estoque'))
                widget.currentRow.CDLOCALESTOQ = General.getFieldValue('CDLOCALESTOQ', 'est42000_implantacao_estoque');

        }
        else if (row.IDORDEMCLC != ordem) {
            MessageService.showMessageByCode("ORDEM_ATUALIZACAO");
        }
        else {
            MessageService.showMessageByCode("ERRO_NEW_ITEM");
        }
    };

    this.confirmDeleteItem = function(widget) {
        var deleteFunction = this.deleteItem.bind(this, widget);
        var row = widget.getParent().currentRow;
        let rowItem = widget.currentRow;
        if (rowItem.IDPERMEXCITINVE != 'N' || rowItem.IDAJUSTEIMPORT != 'S') {
            if (!widget.currentRow.__is_new) {
                ValidacoesEstoque_Service.validaDataLancamento(row.CDFILIMOVI, row.DTLANCESTQ, 'AJUSTE_IMP', 'EST', null).then(
                    function(success) {
                        ScreenService.confirmMessage('Excluir registro?', 'question', deleteFunction);
                    },
                    function(error) {
                        ScreenService.showMessage(error.data.error);
                    }
                );
            }
            else {
                widget.dataSource.data = widget.dataSource.data.filter(rows => {
                    if (rows != rowItem)
                        return true;
                    else
                        return false;
                });
                widget.currentRow = {};
                ScreenService.closeSwipe();
            }
        }
        else {

            ScreenService.showMessage('Não é permitido a exclusão de itens gerados por importação');
        }
    };

    this.getParamsImpEst = function(CDFILIMOVI) {
        return new Promise((resolve) => {
            General.callRestEngine('est_ajuste#/crud/paramimpest_params_codebar', 'FilterData', []).then(paramsData => {
                let hasDefaultParams = paramsData.dataset.paramimpest_params_codebar.filter(row => {
                    return row.IDFILIPADIMPEST == 'S';
                });

                if (hasDefaultParams.length == 0) {
                    return resolve(
                        paramsData.dataset.paramimpest_params_codebar.filter(row => {
                            return row.CDFILPARIMPEST === CDFILIMOVI;
                        })
                    );
                }
                else {
                    return resolve(hasDefaultParams);
                }
            });
        });

    };

    this.confirmDeleteLanc = function(widget) {
        var deleteFunction = this.deleteLancamento.bind(this, widget);
        var row = Util.clone(widget.currentRow, ['__rightActions', '__leftActions']);
        ScreenService.showLoader();
        ValidacoesEstoque_Service.validaDataLancamento(row.CDFILIMOVI, row.DTLANCESTQ, 'AJUSTE_IMP', 'EST', null).then(
            function(success) {
                ScreenService.hideLoader();
                ScreenService.confirmMessage('Excluir registro?', 'question', deleteFunction);
            },
            function(error) {
                ScreenService.showMessage(error.data.error);
                ScreenService.hideLoader();
            }
        );
    };

    this.deleteItem = function(widget) {
        if (widget.dataSource.data.length > 1) {
            var row = {};
            Util.extend(row, widget.currentRow);
            delete row.__rightActions;
            Ajuste_Service.deleteItem(row).then(function(success) {
                ScreenService.closeSwipe();
                widget.reload();
                widget.list();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
        else {
            ScreenService.showMessage('É necessário ao menos um produto no lançamento!');
        }

    };

    this.deleteLancamento = function(widget) {
        var lancamento = Util.clone(widget.currentRow, ['__rightActions', '__leftActions']);
        var promises = [];
        ScreenService.showLoader();
        Ajuste_Service.deleteLancamento(lancamento).then(function() {
            ScreenService.showMessage("Lançamento excluido com sucesso");
            ScreenService.closeSwipe();
            widget.list();
            ScreenService.hideLoader();
            widget.reload();
        }, function(error) {
            ScreenService.showMessage(error.data.error);
            ScreenService.hideLoader();
        });
    };

    this.atualizaEstoque = function(widgetLancamento) {
        var row = widgetLancamento.currentRow;
        var widgetItems = widgetLancamento.container.getWidget('Produtos');

        var notSaved = widgetItems.dataSource.data.some(row => {
            return row.__is_new == true;
        });
        if (notSaved) {
            this.saveImport(widgetItems);
        }
        else {
            ValidacoesEstoque_Service.validaDataLancamento(row.CDFILIMOVI, row.DTLANCESTQ, 'AJUSTE_IMP', 'EST', null).then(
                function(success) {
                    if (widgetLancamento.container.getWidget('Produtos').dataSource.data.length > 0) {
                        let popUpRetirada = widgetLancamento.container.getWidget('PopUpRetiradaAjuste');
                        var popUpRetiradaPromisse = ZHPromise.defer();
                        //if (widgetLancamento.container.name == 'est41000_ajuste_inventario' && widgetLancamento.appliedFilter.FILTER_ROW.IDGERARETAJU == 'S') { //aqui mudar para 'S'
                        if (widgetLancamento.container.name == 'est41000_ajuste_inventario' && widgetLancamento.appliedFilter.FILTER_ROW.IDGERARETAJU == 'Mudar') { //aqui mudar para 'S'
                            Ajuste_Service.alreadyHasRetirada(widgetLancamento.currentRow).then(dataSetRetirada => {
                                let boolRetirada = dataSetRetirada.dataset.confereRetiradaAjuste.length > 0;
                                if (!boolRetirada) {
                                    widgetLancamento.currentRow.RETIESTQ = 'S';
                                    popUpRetirada.promisse = popUpRetiradaPromisse;
                                    popUpRetirada.currentRow.TPRETIAJUSTE = 'E';
                                    ScreenService.openPopup(popUpRetirada);
                                }
                                else {
                                    widgetLancamento.currentRow.RETIESTQ = dataSetRetirada.dataset.confereRetiradaAjuste[0].RETIESTQ;
                                    popUpRetiradaPromisse.resolve();
                                }
                            });
                        }
                        else {
                            popUpRetiradaPromisse.resolve();
                        }
                        popUpRetiradaPromisse.then(resolved => {
                            Ajuste_Service.questionConsolidaEmProdEstoque(widgetItems.dataSource.data).then(boolConsolidateToStock => {
                                let isImport = widgetItems.dataSource.data.some(row => row.IDAJUSTEIMPORT != null);
                                if (isImport) {
                                    importEstoqQuestion(widgetLancamento, boolConsolidateToStock);
                                }
                                else {
                                    defaultEstoqQuestion(widgetLancamento, boolConsolidateToStock);
                                }
                            });
                        });
                    }
                    else {
                        ScreenService.showMessage('Lance um produto antes de atualizar');

                        this.openNewRowOnProdutoGrid(widgetLancamento.widgets[0]);
                    }
                }.bind(this),
                function(error) {
                    ScreenService.showMessage(error.data.error);
                    ScreenService.hideLoader();
                }
            );
        }
    };

    var importEstoqQuestion = function(widgetLancamento, boolConsolidateToStock) {
        let importGroupCode = widgetLancamento.currentRow.CDGRUPPRODLCEST;
        var filter = [
            {
                "name": "CDGRUPPROD",
                "operator": "=",
                "value": importGroupCode
            }
	    ];
        return General.callRestEngine('est#/crud/grupprod', 'FilterData', filter).then(messageType => {
            if (!messageType.dataset.grupprod.length)
                return defaultEstoqQuestion(widgetLancamento, boolConsolidateToStock);
            else
                messageType = messageType.dataset.grupprod[0].IDMSGATUEST;

            if (messageType == "0")
                return defaultEstoqQuestion(widgetLancamento, boolConsolidateToStock);

            if (messageType == "1") {
                return ScreenService.showCustomDialog('Deseja atualizar o estoque? [Atualizar] para atualizar o estoque. [Voltar] para voltar para o lançamento', 'question', 'question', [
                    {
                        label: ScreenService.i18n("Atualizar"),
                        "default": true,
                        code: () => gravarFunction(widgetLancamento, boolConsolidateToStock)
                    },
                    {
                        label: ScreenService.i18n("Voltar"),
                        code: () => {}
                    }
                ]);
            }
            if (messageType == "2") {
                return ScreenService.showCustomDialog('Deseja atualizar o estoque? [Atualizar] para atualizar o estoque. [Voltar] para voltar para o lançamento', 'question', 'question', [
                    {
                        label: ScreenService.i18n("Atualizar"),
                        "default": true,
                        code: () => zerarFunction(widgetLancamento, boolConsolidateToStock)
                    },
                    {
                        label: ScreenService.i18n("Voltar"),
                        code: () => {}
                    }
                ]);
            }

        });
    };

    var defaultEstoqQuestion = function(widgetLancamento, boolConsolidateToStock) {
        ScreenService.showCustomDialog('[Gravar] para atualizar o estoque.[Zerar] para atualizar o estoque e zerar os demais produtos.[Continuar] para continuar o cadastramento.', 'question', 'question', [
            {
                label: ScreenService.i18n("Gravar"),
                "default": true,
                code: () => gravarFunction(widgetLancamento, boolConsolidateToStock)
            },
            {
                label: ScreenService.i18n("Zerar"),
                code: () => zerarFunction(widgetLancamento, boolConsolidateToStock)
            },
            {
                label: ScreenService.i18n("Continuar"),
                code: () => {}
            }
        ]);
    };

    var gravarFunction = function(widgetLancamento, boolConsolidateToStock) {
        let row = widgetLancamento.currentRow;
        ScreenService.showLoader();

        let functioGravar = 'atualizaEstoque';
        let paramsFunction = [row.CDFILIAL, row.NRLANCESTQ, boolConsolidateToStock];

        // this if statement should be uncommented when the logic of the function atualizaEstoqueRetirada has done.
        /*if (widgetLancamento.appliedFilter.FILTER_ROW.IDGERARETAJU == 'S' && widgetLancamento.container.name == 'est41000_ajuste_inventario') {
             functioGravar = 'atualizaEstoqueRetirada';
             paramsFunction.push(Util.clone(row, ignoredCloneKeys));
        }*/

        ValidacoesEstoque_Service[functioGravar].apply(this, paramsFunction).then(function() {
            ScreenService.showMessage('Estoque atualizado com sucesso');
            ScreenService.closeSwipe(0);
            ScreenService.hideLoader();
            widgetLancamento.reload();
        }, function(error) {
            if (error.dataset) {
                if (error.dataset.irregularProducts) {
                    let widgetProdutos = widgetLancamento.container.getWidget('Produtos');
                    widgetProdutos.itemsPerPage = 999999;
                    widgetProdutos.hasPagination = false;
                    widgetProdutos.reload().then(reloaded => {
                        let irregularProducts = error.dataset.irregularProducts;
                        let dados = widgetProdutos.dataSource.data;
                        dados.forEach((d, index) => {
                            if (irregularProducts[d.CDPRODMOVI]) {
                                d.IRREGULAR = true;
                            }
                        });
                        widgetProdutos.redraw();
                    });
                }
            }
            ScreenService.showMessage(error.data.error);
            ScreenService.hideLoader();
        });
    };

    var zerarFunction = function(widgetLancamento, boolConsolidateToStock) {
        let row = widgetLancamento.currentRow;
        ScreenService.showLoader();

        let functioZerar = 'zeraInventario';
        let paramsFunction = [row.CDFILIAL, row.NRLANCESTQ, Ajuste_Service.getIdordemclc(), boolConsolidateToStock];

        // this if statement should be uncommented when the logic of the function atualizaEstoqueRetirada has done.
        /*if (widgetLancamento.appliedFilter.FILTER_ROW.IDGERARETAJU == 'S' && widgetLancamento.container.name == 'est41000_ajuste_inventario') {
            functioZerar = 'zeraInventarioRetirada';
            paramsFunction.push(Util.clone(row, ignoredCloneKeys));
        }*/

        ValidacoesEstoque_Service[functioZerar].apply(this, paramsFunction).then(function() {
            ScreenService.showMessage('Estoque Zerado com sucesso');
            widgetLancamento.widgets[0].reload();
            ScreenService.hideLoader();
        }, function(error) {
            if (error.dataset) {
                if (error.dataset.irregularProducts) {
                    let widgetProdutos = widgetLancamento.container.getWidget('Produtos');
                    widgetProdutos.itemsPerPage = 999999;
                    widgetProdutos.hasPagination = false;
                    widgetProdutos.reload().then(reloaded => {
                        let irregularProducts = error.dataset.irregularProducts;
                        let dados = widgetProdutos.dataSource.data;
                        dados.forEach((d, index) => {
                            if (irregularProducts[d.CDPRODMOVI]) {
                                d.IRREGULAR = true;
                            }
                        });
                        widgetProdutos.redraw();
                    });
                }
            }
            ScreenService.showMessage(error.data.error);
            ScreenService.hideLoader();
        });
    };

    this.zerarEstoque = function(widgetLancamento) {
        var row = widgetLancamento.currentRow;
        var widgetItems = widgetLancamento.container.getWidget('Produtos');

        var notSaved = widgetItems.dataSource.data.some(row => {
            return row.__is_new == true;
        });
        if (notSaved) {
            this.saveImport(widgetItems);
        }
        else {
            ValidacoesEstoque_Service.validaDataLancamento(row.CDFILIMOVI, row.DTLANCESTQ, 'AJUSTE_IMP', 'EST', null).then(
                function(success) {
                    if (widgetLancamento.container.getWidget('Produtos').dataSource.data.length > 0) {
                        ScreenService.showCustomDialog('O estoque será atualizado e os demais produtos serão zerados. Deseja realizar esta operação ?', 'question', 'question', [
                            {
                                label: ScreenService.i18n("Zerar"),
                                "default": true,
                                code: function() {
                                    ValidacoesEstoque_Service.zeraInventario(row.CDFILIAL, row.NRLANCESTQ, Ajuste_Service.getIdordemclc()).then(function() {
                                        ScreenService.showMessage('Estoque Zerado com sucesso');
                                        widgetLancamento.widgets[0].reload();
                                    }, function(error) {
                                        if (error.dataset) {
                                            if (error.dataset.irregularProducts) {
                                                let irregularProducts = error.dataset.irregularProducts;
                                                let dados = widgetLancamento.widgets[0].dataSource.data;
                                                dados.forEach((d, index) => {
                                                    if (irregularProducts[d.CDPRODMOVI]) {
                                                        d.IRREGULAR = true;
                                                    }
                                                });
                                                widgetLancamento.widgets[0].redraw();
                                            }
                                        }
                                        ScreenService.showMessage(error.data.error);
                                    });
                                }.bind(this)
                            },
                            {
                                label: ScreenService.i18n("Cancelar"),
                                code: function() {}
                            }
                        ]);
                        let quest = option == 'Zerar' ? 'O estoque será atualizado e os demais produtos serão zerados.' : 'O estoque será atualizado.';
                        ScreenService.showCustomDialog(quest + ' Deseja realizar esta operação ?', 'question', 'question', [
                            {
                                label: ScreenService.i18n("Sim"),
                                default: true,
                                code: () => {
                                    if (option == "sdb") {
                                        ValidacoesEstoque_Service.atualizaEstoque(row.CDFILIAL, row.NRLANCESTQ).then(function() {
                                            ScreenService.showMessage('Estoque atualizado com sucesso');
                                            ScreenService.closeSwipe(0);
                                            widgetLancamento.reload();
                                        }, function(error) {
                                            if (error.dataset) {
                                                if (error.dataset.irregularProducts) {
                                                    let irregularProducts = error.dataset.irregularProducts;
                                                    let dados = widgetLancamento.widgets[0].dataSource.data;
                                                    dados.forEach((d, index) => {
                                                        if (irregularProducts[d.CDPRODMOVI]) {
                                                            d.IRREGULAR = true;
                                                        }
                                                    });
                                                    widgetLancamento.widgets[0].redraw();
                                                }
                                            }
                                            ScreenService.showMessage(error.data.error);
                                        });

                                    }
                                    else {
                                        ValidacoesEstoque_Service.zeraInventario(row.CDFILIAL, row.NRLANCESTQ, Ajuste_Service.getIdordemclc()).then(function() {
                                            ScreenService.showMessage('Estoque Zerado com sucesso');
                                            widgetLancamento.widgets[0].reload();
                                        }, function(error) {
                                            if (error.dataset) {
                                                if (error.dataset.irregularProducts) {
                                                    let irregularProducts = error.dataset.irregularProducts;
                                                    let dados = widgetLancamento.widgets[0].dataSource.data;
                                                    dados.forEach((d, index) => {
                                                        if (irregularProducts[d.CDPRODMOVI]) {
                                                            d.IRREGULAR = true;
                                                        }
                                                    });
                                                    widgetLancamento.widgets[0].redraw();
                                                }
                                            }
                                            ScreenService.showMessage(error.data.error);
                                        });
                                    }
                                }
                            },
                            {
                                label: ScreenService.i18n("Não"),
                                code: () => {}
                            }
                        ]);



                    }
                    else {
                        ScreenService.showMessage('Lance um produto antes de atualizar');
                        ScreenService.closePopup(true);
                        this.openNewRowOnProdutoGrid(widgetLancamento.widgets[0]);
                    }
                }.bind(this),
                function(error) {
                    ScreenService.showMessage(error.data.error);

                }
            );
        }
        ScreenService.closePopup();
    };

    this.setVrBruto = function(row) {
        if (templateManager.container.getActiveWidget().mode === 'edit') {
            row.VRUNITBRUT = row.VRUNILANCTO;
            row.VRUNITBRUT_inline = row.VRUNILANCTO_inline;
        }
    };

    this.setVrBrutoChecking = function(row) {
        var widget = templateManager.container.getActiveWidget();
        if (row.__is_new && widget.mode === 'edit') {
            if (row.VRUNILANCTO_inline) {
                row.VRUNILANCTO = row.VRUNILANCTO_inline;
            }
            row.VRUNITBRUT = row.VRUNILANCTO;
            var valorPraUsar = row.VRUNILANCTO_inline || row.VRUNILANCTO;
            var fieldElement = widget.getField("VRUNITBRUT").element;
            fieldElement = $("#VRUNITBRUT");
            fieldElement.text(valorPraUsar);
            fieldElement.val(Util.applyMask(valorPraUsar, widget.getField("VRUNITBRUT").mask, maskEngine, ZhUtil));
        }

    };

    this.setCustoItem = function(widgetItem) {
        var rowAjuste = widgetItem.getParent().currentRow;
        var rowItem = widgetItem.currentRow;
        var fieldLiq = widgetItem.getField('VRUNILANCTO');
        var fieldBrut = widgetItem.getField('VRUNITBRUT');
        var almox = ' ';
        var local = ' ';
        var lote = ' ';
        var sublote = ' ';
        var tipoCustoLiq = Ajuste_Service.idTipoCustoLiq;
        var tipoCustoBrut = Ajuste_Service.idTipoCustoBrut;

        var fnLiq = ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(rowAjuste.CDFILIAL, rowItem.CDPRODUTO, rowAjuste.DTLANCESTQ, tipoCustoLiq, almox, local, lote, sublote);
        var fnBrut = ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(rowAjuste.CDFILIAL, rowItem.CDPRODUTO, rowAjuste.DTLANCESTQ, tipoCustoBrut, almox, local, lote, sublote);
        ZHPromise.all([fnLiq, fnBrut]).then(function(data) {
            var vrLiq = data[0].dataset.consultaCustoQuantidadeProduto.VRCUSTOPROD;
            var vrBrut = data[1].dataset.consultaCustoQuantidadeProduto.VRCUSTOPROD;
            if (vrLiq > 0 && vrBrut > 0) {
                var params = Util.extend(Util.clone(fieldBrut.mask.params), { showSymbol: true, symbol: "R$" }, true);
                var stringVrLiq = $.currency.string(vrLiq, params);
                var stringVrBrut = $.currency.string(vrBrut, params);
                var mensagens = {
                    "vrLiq": stringVrLiq,
                    "vrBrut": stringVrBrut
                };
                // ScreenService.confirmMessage('Último custo de movimentação estoque: Líquido - __vrLiq__ - Bruto - __vrBrut__. Deseja utilizá-lo?',undefined,function(){
                //     rowItem.VRUNITBRUT = vrLiq;
                //     rowItem.VRUNILANCTO = vrBrut;
                // }.bind(this),undefined,undefined,mensagens);

                var variaveis = {
                    vrLiq: stringVrLiq,
                    vrBrut: stringVrBrut
                };

                MessageService.confirmMessageByCode("ULTIMOCUSTOMOVEST", variaveis).then(function() {
                    rowItem.VRUNITBRUT = vrLiq;
                    rowItem.VRUNILANCTO = vrBrut;
                }.bind(this));
            }
        });
    };

    this.backAjusteLancto = function(args) {
        var widget = args.owner.widget.container.widgets[0];
        if (widget.currentRow.__is_new) {
            widget.reload();
        }
        General.backOrCancel(args);
    };

    this.backAjusteItem = function(args) {
        var widget = args.owner.widget;
        if (widget.currentRow.__is_new) {
            General.backOrCancel(args);
        }
        else {
            widget.list();
            widget.rollbackInlineEdit();
        }
    };

    this.validaFilialControlaEstoque = function(args) {
        if (this.validaFilial) {
            args.e.preventDefault();
            Ajuste_Service.filialControlEstq().then(
                function() {
                    this.validaFilial = false;
                    ScreenService.openWindow('est_ajuste#' + args.owner.name);
                }.bind(this),
                function(error) {
                    ScreenService.showMessage(error);
                }
            );
        }
        else {
            this.validaFilial = true;
        }
    };

    this.openPosicaoDeEstoque = function(widget) {
        ValidacoesEstoque_Service.openPosicaoDeEstoque(widget);
    };

    this.habilitaCamposImportacao = function(widget) {
        let origem = widget.getField("IDGRUPPROD");
        let GRUPPROD = widget.getField("NMGRUPPROD");
        if (widget.getField("AJUSTEPOR").value() === "P") {
            widget.label = "Importação de Produtos por Grupo";
            origem.isVisible = true;
            GRUPPROD.isVisible = true;
        }
        else {
            widget.label = "Ajuste Por Filial";
            origem.isVisible = false;
            GRUPPROD.isVisible = false;
        }
        widget.reload();
    };

    this.confirmarAjuste = function(widget) {
        if (widget.dataSource.data[0].hasOwnProperty('ajusteValido')) {
            if (widget.dataSource.data[0].ajusteValido === false) {
                ScreenService.closePopup();
                widget.promisse.reject();
                widget.dataSource.data = [];
                ScreenService.showMessage('Quantidade informada fora do Permitido');

            }
            else {
                ScreenService.closePopup();
                widget.promisse.resolve();
                widget.dataSource.data = [];
            }
        }
        else {
            ScreenService.closePopup();
            widget.promisse.resolve();
            widget.dataSource.data = [];
        }

    };

    this.validaCusto = function(rowCusto, row) {
        let rowItem = rowCusto.item;
        let valrAnte = rowCusto.valrAnte * rowItem.VRFATOCONV;
        let qtdeAnte = rowCusto.qtdeAnte;
        let valrMedi = rowCusto.valrMedi * rowItem.VRFATOCONV;
        let qtdeAtu = parseFloat(rowItem.QTTOTLANCTO.replace('.', ',').replace(',', '.'));
        let valrAtu = valrMedi * qtdeAtu;

        if (rowItem.VRPTOLAJUSINV > 0 && rowItem.VRPTOLAJUSINV != null) {
            if (valrAnte * rowItem.VRFATOCONV != 0) {
                if (Math.abs(((valrMedi * qtdeAtu * rowItem.VRFATOCONV * 100) / valrAnte * rowItem.VRFATOCONV) - 100) > rowItem.VRFATOCONV) {
                    row.qntValida = false;
                    return false;
                }
            }
        }
        row.qntValida = true;
        return true;



    };

    this.emissaoAjuste = function(widget) {
        let ajusteRow = widget.currentRow;
        ScreenService.showLoader();
        ScreenService.openWindow("est_relatorios#est45000_emissao_inventario").then(opened => {
            let timeOutTime = 10; //time in seconds
            let recallTimes = 0;
            var autoFilter = setInterval(filterTimeout => {
                let widgetFilter = templateManager.container.getActiveWidget();
                if (widgetFilter.name == 'zhFilterPreferencesSelectConditions') {
                    clearInterval(autoFilter);
                    widgetFilter.currentRow.CDFILIAL = ajusteRow.CDFILIAL;
                    widgetFilter.currentRow.NMFILIAL = ajusteRow.NMFILIAL;
                    widgetFilter.currentRow.DTLANCESTQ = ajusteRow.DTLANCESTQ;
                    widgetFilter.currentRow.NRLANCESTQ = ajusteRow.NRLANCESTQ;
                    widgetFilter.currentRow.CDFILIMOVI = ajusteRow.CDFILIMOVI;
                    widgetFilter.currentRow.CDFILLANC = ajusteRow.CDFILIAL;
                    widgetFilter.getField('DTLANCESTQ').element.click();
                    widgetFilter.getField('NRLANCESTQ').element.click();
                    var widgetRelatorio = templateManager.container.getWidget('EmissaoInventarioGrid');
                    if (widgetFilter.getField('DTLANCESTQ') != "") {
                        widgetRelatorio.dataSource.name = "est_relatorios#/crud/itlanctoest_produto";
                    }
                    else {
                        widgetRelatorio.dataSource.name = "est_relatorios#/produto_posestatu_paramfilial";
                    }
                    widgetRelatorio.getAction('Voltar').isVisible = true;
                    //setReturnAction(widgetRelatorio);
                    widgetFilter.fields.forEach(f => {
                        if (f.validations)
                            delete f.validations;
                    });
                    setTimeout(time => {
                        widgetFilter.getAction('never').element.click();
                        ScreenService.hideLoader();
                    }, 1000);
                }
                else {
                    recallTimes++;
                    if (recallTimes == timeOutTime) {
                        clearInterval(autoFilter);
                        ScreenService.goBack();
                        ScreenService.hideLoader();
                    }
                }

            }, 1000);
        });
    };

    var setReturnAction = function(widget) {
        var actionGoBack = {
            name: 'back',
            label: 'Voltar',
            showAsAction: 'back_view|back_list',
            isVisible: true,
            showOnList: true,
            events: [{
                name: 'ActionEvent',
                code: 'ScreenService.goBack();',
            }]
        };
        actionGoBack = metaDataFactory.actionFactory(actionGoBack);
        actionGoBack.widget = widget;
        widget.actions.push(actionGoBack);
    };

    this.validaCorAlternada = function(indicerow) {
        if (indicerow % 2 != 0) {
            return false;
        }
        else {
            return true;
        }
    };

    this.controlaIndiceRow = function(widget) {
        var data = widget.dataSource.data;
        data.forEach(function(item, indice) {
            data[indice].INDICEROW = indice;
        });
    };

    this.setStripeData = function(widget) {
        let widgetStripe = widget.container.getWidget('stripeFooter');
        widgetStripe.currentRow.CDALMOXARIFE = widget.currentRow.CDALMOXARIFE;
        widgetStripe.currentRow.CDLOCALESTOQ = widget.currentRow.CDLOCALESTOQ;
        widgetStripe.currentRow.NRLOTEESTQ = widget.currentRow.NRLOTEESTQ;
        widgetStripe.currentRow.NRSUBLOTE = widget.currentRow.NRSUBLOTE;
    };

    this.validaCtrlLoteDtvalidade = function(widget) {
        let hasDtlote = !!widget.currentRow.DTVALILOTE;
        if (filutlote == 'S' && !hasDtlote) {
            this.openLibPopup(widget);
        }
    };

    this.processBeforeReport = function(widget) {
        widget.dataSourceFilter.forEach(filter => bkpDatasourceFilter.push(Util.clone(filter)));
        widget.afterOpenreport = () => {
            widget.dataSourceFilter = bkpDatasourceFilter;
        };
    };

    this.sugestProductCosts = function(widget) {
        ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(
            widget.currentRow.CDFILIAL,
            widget.currentRow.CDPRODUTO,
            widget.getParent().currentRow.DTLANCESTQ, //somente os 3 primeiros parametros sao obrigatórios
            null, //tipocusto null para pegar do parametro de estoque
            widget.currentRow.CDALMOXARIFE,
            widget.currentRow.CDLOCALESTOQ,
            widget.currentRow.NRLOTEESTQ,
            widget.currentRow.NRSUBLOTE
        ).then((success) => {
            if (success.dataset) {
                let custos = success.dataset.consultaCustoQuantidadeProduto;
                ScreenService.confirmMessage('Último custo de movimentação estoque: Líquido R$' + custos.VRCUSTOPRODLIQ + ' - Bruto R$' + custos.VRCUSTOPRODBRUT + '. Deseja utilizá-lo??', 'question', function() {
                    widget.currentRow.VRUNILANCTO = custos.VRCUSTOPRODLIQ;
                    widget.currentRow.VRUNITBRUT = custos.VRCUSTOPRODBRUT;
                }.bind(this));
            }
        });

    };

    this.changeTipoRetirada = function(widget) {
        if (widget.currentRow.TPRETIAJUSTE == 'E') {
            widget.getField('NMSERVICO').validations = { "required": true };
            widget.getField('NMSERVICO').isVisible = true;
            widget.getField('NMSETOR').isVisible = false;
            delete widget.getField('NMSETOR').validations;
            widget.currentRow.CDSETOR = '';
            widget.currentRow.NMSETOR = '';
            templateManager.updateTemplate();
        }
        else {
            widget.getField('NMSETOR').validations = { "required": true };
            widget.getField('NMSETOR').isVisible = true;
            widget.getField('NMSERVICO').isVisible = false;
            delete widget.getField('NMSERVICO').validations;
            widget.currentRow.CDSERVFILI = '';
            widget.currentRow.NMSERVICO = '';
            templateManager.updateTemplate();
        }
    };

    this.outDataPopupRetirada = function(popUpRetirada) {
        let widgetLancamento = popUpRetirada.container.getWidget('est41000_ajusteInventario');
        Object.assign(widgetLancamento.currentRow, popUpRetirada.currentRow);
    };

    this.validaProdutoPRetirada = function(widget) {
        Ajuste_Service.validaProdutoPRetirada(widget.currentRow.CDTIPORETI).then(result => {
            if (result.dataset.prodreti.length == 0) {
                widget.currentRow.CDTIPORETI = '';
                widget.currentRow.NMTIPORETI = '';
                MessageService.showMessageByCode('DONT_EXISTS_PRODUCTS_RETIRADA');
            }
        });
    };

    this.validaAllProdutosPRetirada = function(popUpRetirada) {
        let widgetProdutos = popUpRetirada.container.getWidget('Produtos');
        if (!popUpRetirada.currentRow.CDCENTCUST) {
            MessageService.showMessageByCode('SETOR_WO_CENTCUST');
            return;
        }
        widgetProdutos.dataSource.load(widgetProdutos.getDataSourceFilter(), 'load_all').then(loaded => {
            let arrArvProd = [];
            loaded.dataset.itlanctoest_produto.forEach(row => {
                arrArvProd.push(row.CDARVPROD);
            });
            Ajuste_Service.validaAllProdutosPRetirada(arrArvProd, popUpRetirada.currentRow.CDTIPORETI).then(result => {
                let arrProdReti = [];
                result.dataset.produto_prodreti.forEach(row => {
                    arrProdReti.push(row.CDARVPROD);
                });
                let difference = arrArvProd.filter(prods => !arrProdReti.includes(prods));
                widgetProdutos.dataSource.data.forEach((row, index, array) => {
                    if (difference.includes(row.CDARVPROD))
                        array[index].IRREGULAR = true;
                });
                if (difference.length > 0) {
                    MessageService.showMessageByCode('DONT_EXISTS_PRODUCTS_RETIRADA');
                    popUpRetirada.promisse.reject();
                    ScreenService.closePopup();
                    widgetProdutos.redraw();
                    return;
                }

                this.outDataPopupRetirada(popUpRetirada);
                popUpRetirada.promisse.resolve();
                ScreenService.closePopup();
            });
        });
    };

    this.onManageXlsModelFields = function(isAjuste = true) {
        return new Promise((resolve, reject) => {

            const modelXlsWidget = templateManager.container.getWidget('modelWidget');
            const uploadWidget = templateManager.container.getWidget('wdgUpload');

            const almoxField = modelXlsWidget.getField('CDALMOXARIFE');
            const localField = modelXlsWidget.getField('CDLOCALESTOQ');
            const loteField = modelXlsWidget.getField('NRLOTEESTQ');
            const subLoteField = modelXlsWidget.getField('NRSUBLOTE');
            const priceField = modelXlsWidget.getField('VRUNILANCTO');
            const dtFabField = modelXlsWidget.getField('DTFABRPROD');
            const dtValiField = modelXlsWidget.getField('DTVALILOTE');
            const cdProdutoField = modelXlsWidget.getField('CDPRODUTO');
            const cdArvProdField = modelXlsWidget.getField('CDARVPROD');
            const cdBarProdField = modelXlsWidget.getField('CDBARPRODUTO');

            const codProdTypesVisibility = {
                'C': () => cdProdutoField.isVisible = true,
                'A': () => cdArvProdField.isVisible = true,
                'B': () => cdBarProdField.isVisible = true,
                notVisible: () => {
                    cdProdutoField.isVisible = false;
                    cdArvProdField.isVisible = false;
                    cdBarProdField.isVisible = false;
                }
            };
            const widgetName = isAjuste ? 'est41000_ajusteInventario' : 'est42000_implantacao_estoque';

            this.getParamsImpEst(EstUtil.getFieldValue('CDFILIAL', widgetName, true)).then(response => {
                if (response.length > 0) {
                    uploadWidget.currentRow.PARAMS = response[0];

                    codProdTypesVisibility.notVisible();
                    codProdTypesVisibility[response[0].IDTIPOPRODPAR]();

                    almoxField.isVisible = sessionStorage.getItem('IDUTILALMOX') == 'S' ? true : false;
                    localField.isVisible = sessionStorage.getItem('IDUTILLCESTQ') == 'S' ? true : false;
                    loteField.isVisible = sessionStorage.getItem('IDUTILLOTE') == 'S' ? true : false;
                    subLoteField.isVisible = sessionStorage.getItem('IDUTILLOTE') == 'S' ? true : false;

                    priceField.isVisible = isAjuste ? false : true;
                    dtFabField.isVisible = isAjuste ? false : true;
                    dtValiField.isVisible = isAjuste ? false : true;

                    templateManager.updateTemplate();
                    resolve(true);

                }
                else {
                    ScreenService.hideLoader();
                    ScreenService.showMessage("Parâmetros para importação de estoque não encontrados.");
                }
            }).catch(error => {
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage(error);
                ScreenService.hideLoader();
                reject(false);
            });
        });
    };

    // This generates glossary fields
    this.buildGlossary = function(widget) {
        const modelXlsWidget = templateManager.container.getWidget('modelWidget');

        modelXlsWidget.fields.forEach(field => {
            this.geraFields(field, widget).then(() => {
                this.setaFields(field, widget);
            });
        });
    };

    this.geraFields = function(field, widget) {
        var deferred = ZHPromise.defer();

        field = {
            "name": field.name,
            "label": field.name,
            "template": "field/text-edit.html",
            "isVisible": true,
            "showOnList": false,
            "showOnForm": true,
            "showOnView": false,
            "readOnly": true,
            "filterProperties": {
                "isVisible": false,
                "showOnForm": false
            }
        };

        metaDataFactory.fieldFactory(field, widget, widget);

        deferred.resolve();
        return deferred.promise;
    };

    this.setaFields = function(field, widget) {
        var deferred = ZHPromise.defer();

        if (glossary.hasOwnProperty(field.name))
            widget.getField(field.name).setValue(glossary[field.name].desc);

        deferred.resolve();
        return deferred.promise;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Ajuste_Controller', Ajuste_Controller);
});

//recommit


// FILE: ./assets/modules/ajuste/assets/js/controllers/Est43000_ManutencaoDoUltimoCusto_Controller.js
var Est43000_ManutencaoDoUltimoCusto_Controller = function(EstUtil, templateManager, MessageService, ScreenService, General, ZhFilterPreferences) {

    let IDUTILALMOX, IDUTILLCESTQ, IDUTILLOTE;

    this.setNameLayout = function(args) {
        var widget = args.owner;
        if (!widget.hasOwnProperty('labelPrincipal')) {
            widget.labelPrincipal = widget.label;
        }
        if (args.e.layout.ID == 'ORIGINAL') {
            widget.container.label = widget.labelPrincipal;
            widget.label = widget.labelPrincipal;
        }
        else {
            widget.container.label = args.e.layout.LABEL_TO_PREFERENCE;
            widget.label = args.e.layout.LABEL_TO_PREFERENCE;
        }
    };

    this.getFilialLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.preparaFiltroEFields = function(widget) {
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        this.getFilialLogada().then(function(success) {
            const { CDFILIAL, NMFILIAL, } = success.dataset.filial[0];

            rowsToSet.CDFILIAL = CDFILIAL;
            rowsToSet.NMFILIAL = NMFILIAL;
            rowsToSet.DTREFERENCIA = getLocalVar("DTPROCESSA");
            rowsToSet.DTPROCESSA = getLocalVar("DTPROCESSA");

            ZhFilterPreferences.setOriginalFilter(widget.id, rowsToSet);
            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(function() {
                let filtro = templateManager.container.getActiveWidget();
                this.validaFilial(filtro);
            }.bind(this));
        }.bind(this));
    };

    this.setFieldsGerais = function(widget) {
        templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
    };

    this.validaFilial = function(widget, fieldName) {
        const currentRow = widget.currentRow;
        const widgetProduto = templateManager.container.getWidget("widgetProduto");

        const almoxField = widget.getField('DSALMOXARIFE');
        const localField = widget.getField('DSLOCALESTOQ');
        const almoxFieldProduto = widgetProduto.getField('DSALMOXARIFE');
        const localFieldProduto = widgetProduto.getField('DSLOCALESTOQ');
        const almoxDestField = widget.getField('DSALMOXARIFEDEST');
        const localDestField = widget.getField('DSLOCALESTOQDEST');

        let cdfilial = currentRow.CDFILIAL;

        if (cdfilial) {
            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ];

            General.callRestEngine('est_ajuste#/crud/paramfilial', 'FilterData', filter).then(function(success) {
                const {
                    DTPROCESSA,
                    IDCTRLESTQ,
                    IDUTILALMOX: controlaAlmox,
                    IDUTILLCESTQ: controlaLocal,
                    IDUTILLOTE: controlaLote
                } = success.dataset.paramfilial[0];

                currentRow.DTPROCESSA = DTPROCESSA;
                IDUTILALMOX = controlaAlmox;
                IDUTILLCESTQ = controlaLocal;
                IDUTILLOTE = controlaLote;

                if (IDCTRLESTQ === 'S') {
                    if (IDUTILALMOX === 'S') {
                        if (fieldName === 'NMFILIALDEST') {
                            almoxDestField.readOnly = false;
                            currentRow.controlaLocalDest = IDUTILLCESTQ == 'S' ? true : false;
                        }
                        else {
                            almoxField.readOnly = false;
                            almoxField.validations = { "required": { "message": "Campo Obrigatório!" } };
                            almoxFieldProduto.filterProperties.readOnly = false;
                            almoxFieldProduto.filterProperties.validations = { "required": { "message": "Campo Obrigatório!" } };
                        }

                        almoxField.dataSourceFilter.forEach(function(fil) {
                            if (fil.name == "CDFILIAL")
                                fil.value = currentRow.CDFILIAL;
                        });
                    }
                    else {
                        MessageService.showMessageByCode('CONTROLA_ALMOX');
                        localField.readOnly = true;
                        almoxField.readOnly = true;
                        delete almoxField.validations;

                        localFieldProduto.filterProperties.readOnly = true;
                        almoxFieldProduto.filterProperties.readOnly = true;
                        delete almoxFieldProduto.filterProperties.validations;

                        if (almoxDestField) {
                            almoxDestField.readOnly = true;
                            localDestField.readOnly = true;
                            almoxDestField.reload();
                            localDestField.reload();
                        }
                    }
                    almoxField.reload();
                    localField.reload();
                }
                else {
                    if (fieldName === 'NMFILIALDEST') {
                        currentRow.CDFILIALDEST = '';
                        currentRow.NMFILIALDEST = '';
                        MessageService.showMessageByCode('CONTROLA_ESTOQUE');
                        currentRow.controlaLocalDest = false;
                    }
                    else {
                        currentRow.CDFILIAL = '';
                        currentRow.NMFILIAL = '';
                    }
                }
                localField.readOnly = true;
                delete localField.validations;

                localFieldProduto.filterProperties.readOnly = true;
                delete localFieldProduto.filterProperties.validations;
            });
        }
        else {
            almoxField.readOnly = true;
            localField.readOnly = true;

            almoxFieldProduto.filterProperties.readOnly = true;
            localFieldProduto.filterProperties.readOnly = true;
        }
        currentRow.CDALMOXARIFE = '';
        currentRow.DSALMOXARIFE = '';
        currentRow.CDLOCALESTOQ = '';
        currentRow.DSLOCALESTOQ = '';
    };

    this.setReadOnlyLocal = function(widget, fieldName) {
        const {
            DSALMOXARIFE,
            IDUTILLCESTQ
        } = widget.currentRow;

        const localField = widget.getField('DSLOCALESTOQ');
        const localDestField = widget.getField('DSLOCALESTOQDEST');

        if (DSALMOXARIFE) {
            if (fieldName === 'DSALMOXARIFEDEST') {
                if (IDUTILLCESTQ == 'S')
                    localDestField.readOnly = false;
            }
            else {
                if (IDUTILLCESTQ == 'S') {
                    localField.readOnly = false;
                    localField.validations = { "required": { "message": "Campo Obrigatório!" } };
                    localField.filterProperties.readOnly = false;
                    localField.filterProperties.validations = { "required": { "message": "Campo Obrigatório!" } };
                }
                else {
                    localField.readOnly = true;
                    delete localField.validations;
                    localField.filterProperties.readOnly = true;
                    delete localField.filterProperties.validations;
                }
            }
            localField.dataSourceFilter.forEach(function(fil) {
                switch (fil.name) {
                    case 'CDFILIAL':
                        fil.value = widget.currentRow.CDFILIAL;
                        break;

                    case 'CDALMOXARIFE':
                        fil.value = widget.currentRow.CDALMOXARIFE;
                        break;
                }
            });
        }
        else {
            if (fieldName === 'DSALMOXARIFEDEST') {
                if (IDUTILLCESTQ == 'N') {
                    localDestField.readOnly = true;
                }
                widget.currentRow.CDLOCALESTOQDEST = '';
                widget.currentRow.DSLOCALESTOQDEST = '';
            }
            else {
                if (IDUTILLCESTQ == 'S') {
                    localField.readOnly = false;
                    localField.validations = { "required": { "message": "Campo Obrigatório!" } };
                    localField.filterProperties.readOnly = false;
                    localField.filterProperties.validations = { "required": { "message": "Campo Obrigatório!" } };
                }
                else {
                    localField.readOnly = true;
                    delete localField.validations;
                    localField.filterProperties.readOnly = true;
                    delete localField.filterProperties.validations;
                }
                widget.currentRow.CDLOCALESTOQ = '';
                widget.currentRow.DSLOCALESTOQ = '';
            }
        }
    };

    this.FilterProdIni = function(args) {
        var widget = args.owner.field.widget;
        var row = widget.currentRow;
        var prodIniIsSet = this.fieldIsSet(row.CDPRODINI);
        if (!prodIniIsSet) {
            widget.currentRow.NMPRODFIN = '';

        }
        widget.getField('NMPRODFIN').readOnly = !prodIniIsSet;
        widget.getField('NMPRODFIN').readOnlyOnEdit = !prodIniIsSet;
    };

    this.FilterProdFin = function(args) {
        var widget = args.owner.field.widget;
        var row = widget.currentRow;
        var prodFinIsSet = this.fieldIsSet(row.CDPRODFIN);
        if (prodFinIsSet) {
            if (row.CDPRODINI > row.P_PRODFIN) {
                MessageService.showMessageByCode('PRODUTO_FINAL_INVALIDO');
                return 0;
            }
        }
        widget.reload();
    };

    this.fieldIsSet = function(fieldValue) {
        if (fieldValue) return true;
        else return false;
    };

    this.openPopup = function(widgetName) {
        var widgetPopup = templateManager.container.getWidget(widgetName);
        widgetPopup.isVisible = true;
        widgetPopup.activate();
        widgetPopup.edit();
        ScreenService.openPopup(widgetPopup);
    };

    this.saveNewManutencao = function(widget) {
        if (widget.isValid()) {
            if (!widget.currentRow.NRLOTEESTQ || widget.currentRow.NRLOTEESTQ === ' ') {
                widget.currentRow.NRLOTEESTQ = ' ';
                widget.currentRow.NRSUBLOTE = ' ';
            }

            General.callRestEngine('est_ajuste#/saveManutencao', 'Row', widget.currentRow).then(function(success) {
                    if (widget.currentRow.__is_new) {
                        ScreenService.goBack();
                        widget.reload();
                    }
                    else {
                        ScreenService.closePopup();
                        widget.view();
                        widget.reload();
                    }
                }.bind(this),
                function(callback) {
                    if (callback.data) {
                        ScreenService.showMessage(callback.data.error);
                    }
                }
            );
        }
    };

    this.validaData = function(widget) {
        if (widget.currentRow.DTPROCESSA) {
            if (widget.currentRow.DTPROCESSA < widget.currentRow.DTCUSTCMPPRO) {
                MessageService.showMessageByCode('DATA_ERROR');
            }
        }
    };

    this.adicionar = function(widget) {
        if (EstUtil.onValidateFilterRequirement(widget)) {
            this.validaFilial(EstUtil.getFilterWidget(widget));
            return;
        }
        var rowFilter = widget.appliedFilter.FILTER_ROW;
        if (rowFilter.CDFILIAL) {
            widget.newRow();
            var Data = new Date();
            var dataDia;
            var dataMonth = Data.getMonth() + 1;
            if (Data.getDate() < 10) dataDia = '0' + Data.getDate();
            else dataDia = Data.getDate();
            if (dataMonth < 10) dataMonth = '0' + (dataMonth);
            var dataAtual = dataDia + '/' + dataMonth + '/' + Data.getFullYear();
            widget.currentRow.DTCUSTCMPPRO = dataAtual;
            widget.currentRow.CDFILIAL = rowFilter.CDFILIAL;
            widget.currentRow.CDALMOXARIFE = rowFilter.CDALMOXARIFE || ' ';
            widget.currentRow.CDLOCALESTOQ = rowFilter.CDLOCALESTOQ || ' ';
            widget.currentRow.DTPROCESSA = rowFilter.DTPROCESSA;
        }
        else {
            MessageService.showMessageByCode('ADICIONAR_ERROR');
        }
    };

    this.controlaLoteProduto = function(widget) {
        if (widget.currentRow.NMPRODUTO) {
            if (IDUTILLOTE == 'S') {
                widget.currentRow.NRLOTEESTQ = ' ';
                widget.getField('NRLOTEESTQ').reload();
                widget.getField('NRLOTEESTQ').readOnly = false;
            }
            else {
                widget.currentRow.NRLOTEESTQ = ' ';
                widget.getField('NRLOTEESTQ').reload();
                widget.getField('NRLOTEESTQ').readOnly = true;
            }
        }
        else {
            widget.currentRow.NRLOTEESTQ = ' ';
            widget.getField('NRLOTEESTQ').reload();
            widget.currentRow.NRLOTEESTQ.readOnly = true;
        }
    };

    this.saveImportacao = function(widget) {
        if (widget.isValid()) {
            var row = widget.currentRow;
            row.CDNVPRODUTO = General.getLevelByMask('CDPICTPROD', 1).length;
            if (row.CDFILIALDEST == row.CDFILIAL && row.CDALMOXARIFE == row.CDALMOXARIFEDEST && row.CDLOCALESTOQDEST == row.CDLOCALESTOQDEST) {
                MessageService.showMessageByCode('ORI_DEST_IGUAL');
            }
            else {
                MessageService.confirmMessageByCode("CONFIRMA_IMPORTACAO").then(() => {
                    General.callRestEngine('est_ajuste#/importaUltimoCusto', 'Row', row).then(() => {
                            var widgetActivate = templateManager.container.getWidget('widgetProduto');
                            var widgetPopup = templateManager.container.getWidget('IMPORTACUSTO');
                            ScreenService.goBack();
                            widgetPopup.isVisible = false;
                            widgetPopup.currentRow = {};
                            ScreenService.closePopup(widgetPopup);
                            MessageService.showMessageByCode('CUSTO_IMPORTADO');
                            widgetActivate.activate();
                            widgetActivate.reload();
                            ScreenService.closePopup();
                        },
                        function(callback) {
                            if (callback.data) {
                                ScreenService.showMessage(callback.data.error);
                            }
                        }
                    );

                });
            }
        }
    };

    this.atualizaUltimoCusto = function(widget) {
        if (widget.isValid()) {
            MessageService.confirmMessageByCode('CONFIRMA_ATUALIZACAO').then(() => {
                var row = widget.currentRow;
                row.CDNVPRODUTO = General.getLevelByMask('CDPICTPROD', 1).length;
                General.callRestEngine('est_ajuste#/atualizaUltimoCusto', 'Row', row).then(() => {
                        var widgetActivate = templateManager.container.getWidget('widgetProduto');
                        var widgetPopup = templateManager.container.getWidget('ATUALIZACUSTO');
                        widgetPopup.isVisible = false;
                        widgetPopup.currentRow = {};
                        ScreenService.closePopup(widgetPopup);
                        MessageService.showMessageByCode('CUSTO_ATUALIZADO');
                        widgetActivate.activate();
                        widgetActivate.reload();
                        ScreenService.closePopup();

                    },
                    function(callback) {
                        if (callback.data) {
                            MessageService.showMessageByCode("FINANCEIRO_ERROR");
                        }
                    }
                );
            });
        }
    };

    this.closePopup = function(widgetName, popupName) {
        var widget = templateManager.container.getWidget(popupName);
        widget.currentRow = {};
        var widgetActivate = templateManager.container.getWidget(widgetName);
        widget.isVisible = false;
        ScreenService.closePopup(widget);
        widgetActivate.activate();
        widgetActivate.reload();
    };

    this.changeFilter = function(widget, preference) {
        let filterRow = preference.FILTER_ROW;
        filterRow.CDPRODFIN = filterRow.CDPRODFIN.padEnd(20, "z");

        let filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": filterRow.CDFILIAL ? filterRow.CDFILIAL : null
            },
            {
                "name": "DTREFERENCIA",
                "operator": "=",
                "value": filterRow.DTREFERENCIA ? filterRow.DTREFERENCIA : null
            },
            {
                "name": "CDALMOXARIFE",
                "operator": "=",
                "value": filterRow.CDALMOXARIFE ? filterRow.CDALMOXARIFE : null
            },
            {
                "name": "CDLOCALESTOQ",
                "operator": "=",
                "value": filterRow.CDLOCALESTOQ ? filterRow.CDLOCALESTOQ : null
            },
            {
                "name": "CDPRODINI",
                "operator": "=",
                "value": filterRow.CDPRODINI ? filterRow.CDPRODINI : null
            },
            {
                "name": "CDPRODFIN",
                "operator": "=",
                "value": filterRow.CDPRODFIN ? filterRow.CDPRODFIN : null
            }
        ];

        widget.dataSourceFilter = filtro;
        widget.reload();

    };

    this.actionsHandler = function(widget) {
        if (IDUTILLOTE == "S")
            General.setDisableAction("Novo", widget.name);
        else
            General.setEnableAction("Novo", widget.name);

        widget.reload();
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est43000_ManutencaoDoUltimoCusto_Controller', Est43000_ManutencaoDoUltimoCusto_Controller);
});


// FILE: ./assets/modules/ajuste/assets/js/controllers/Est44000RecalculoEstoque_Controller.js
var Est44000RecalculoEstoque_Controller = function(ScreenService, General, Est44000RecalculoEstoque_Service, templateManager, ZhFilterPreferences) {
	
	this.setFieldsOnEnter = function(widget){
	    Est44000RecalculoEstoque_Service.getParamFilial().then(function(paramfilial){
	    	var row = widget.currentRow;
	        paramfilial = paramfilial.dataset.paramfilial[0];
	        var fieldAlmox = widget.getField('DSALMOXARIFE');
	        var fieldLocal = widget.getField('DSLOCALESTOQ');
	        if(paramfilial.IDUTILALMOX == 'S'){
            	row.CDALMOXARIFE = null;
	            fieldAlmox.readOnly = false;
	            if(paramfilial.IDUTILLCESTQ == 'S'){
	            	row.CDLOCALESTOQ = null;
	                fieldLocal.readOnly = false;
	            }else{
	            	row.CDLOCALESTOQ = ' ';
	                fieldLocal.readOnly = true;
	            }
	        }else{
            	row.CDLOCALESTOQ = ' ';
            	row.CDALMOXARIFE = ' ';
	            fieldAlmox.readOnly = true;
	            fieldLocal.readOnly = true;
	        }
	        this.getFilialLogada().then(function(success){
	        	row.CDFILIAL = success.dataset.filial[0].CDFILIAL;
	        	row.NMFILIAL = success.dataset.filial[0].NMFILIAL;
				this.filialOnChange(widget);
	        }.bind(this));
		}.bind(this));
	};
	
	this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
	
	this.recalcularEstoque = function(widget){
		var row = widget.currentRow;
		row.CDPRODFIN = row.CDPRODFIN + "zzzzzzzzzzzzzzzzzzz";
        row.CDPRODFIN = row.CDPRODFIN.substring(0, 20);
		if(widget.isValid()){
			if(!General.comparaPratoProduto(row.CDPRODINI,row.CDPRODFIN,getLocalVar('GER_CDPICTPROD'))){
				ScreenService.showMessage('Produto inicial deve ser menor ou igual ao produto final.');
			}else{
				ScreenService.showLoader();
				Est44000RecalculoEstoque_Service.recalcularEstoque(row).then(function(success){
					ScreenService.showMessage('Recálculo de estoque foi realizado com sucesso.');
					ScreenService.hideLoader();
				},function(error){
					error = error.data.error;
					if(typeof error == 'object'){
						Est44000RecalculoEstoque_Service.saveProdutosIrregulares(error).then(function(saved){
							ScreenService.openWindow('est_ajuste#est44001_produtos_irregulares_recalculo');
							ScreenService.hideLoader();
						});
					}else{
						ScreenService.showMessage(error);
						ScreenService.hideLoader();
					}
				});
			}
		}
    };
    
    this.apagaValorAlmoxarifado = function(widget,codigo, nome){
        widget.currentRow[codigo] ='';
        widget.currentRow[nome] ='';
    };

    this.filialOnChange = widget => {
        widget.getField('DSALMOXARIFE').reload();
        General.callRestEngine('est#/crud/paramfilial', "FilterData", [{"name": "CDFILIAL", "operator": "=", "value": widget.currentRow.CDFILIAL}]).then((success) => {
        	if(success.dataset.paramfilial[0].IDUTILALMOX == 'S') {
        		widget.getField('DSALMOXARIFE').readOnly = false;
        		widget.currentRow.controlaLocal = success.dataset.paramfilial[0].IDUTILLCESTQ;
        	}
        	this.apagaValorAlmoxarifado(widget,'CDALMOXARIFE','DSALMOXARIFE');
	        widget.getField('NMPRODINI').dataSourceFilter.forEach(function(fil){
	        	if(fil.name == 'CDFILIAL'){
	        		fil.value = widget.currentRow.CDFILIAL;
	        	}
	        });
	        widget.getField('NMPRODFIN').dataSourceFilter.forEach(function(fil){
	        	if(fil.name == 'CDFILIAL'){
	        		fil.value = widget.currentRow.CDFILIAL;
	        	}
	        });
	        widget.getField('NMPRODINI').reload();
	        widget.getField('NMPRODFIN').reload();
        	
        });
       
    };
    
    this.checkProductRange = function (field) {
        var currentRow = field.widget.currentRow;

        if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = currentRow.CDPRODFIN ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if (!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;
                field.clearValue();
                field.reload();
                
				ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        }
    };
    
    this.almoxarifadoOnChange = (args) => {
    	let widget = args.owner.field.widget;
    	if(widget.currentRow.DSALMOXARIFE && widget.currentRow.controlaLocal == 'S') {
    		widget.getField('DSLOCALESTOQ').readOnly = false;
    	}	
    	else {
    		widget.getField('DSLOCALESTOQ').readOnly = true;
    	}
    	widget.getField('DSLOCALESTOQ').value('');
    	widget.getField('DSLOCALESTOQ').reload();
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est44000RecalculoEstoque_Controller', Est44000RecalculoEstoque_Controller);
});


// FILE: ./assets/modules/ajuste/assets/js/controllers/Est45500_ManutencaoDoLote_Controller.js
var Est45500_ManutencaoDoLote_Controller = function(templateManager, ScreenService, General, ZhFilterPreferences){

    this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
    
    this.setStripeData = function(widget) {
        var currentRow = widget.appliedFilter.FILTER_ROW;
        currentRow.DTENTRLOTE = (currentRow.DTENTRLOTE) ? currentRow.DTENTRLOTE : {'start': '', 'end': ''};
        var stripeWidget = templateManager.container.getWidget("stripe");

        stripeWidget.currentRow = {};
        stripeWidget.currentRow.NMFILIAL   = currentRow.CDFILIAL + ' - ' + currentRow.NMFILIAL;
        stripeWidget.currentRow.NMPRODUTO   = currentRow.CDARVPROD + ' - ' + currentRow.NMPRODUTO;
        stripeWidget.currentRow.DTENTRLOTE = 'De ' + currentRow.DTENTRLOTE.start + ' até ' + currentRow.DTENTRLOTE.end;
    };
    
    this.getFieldValueFilter = function(filedName, widgetName) {
        var container =  templateManager.container;
        if(container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
              filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if(f) {
                     filterWidget = container.getWidget(widgetName).appliedFilter;
                }
               
            }
            
            if(filterWidget) {
                var valueField;
                if(filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[filedName];
                else
                    valueField = filterWidget.FILTER_ROW[filedName];
                    
                return valueField || null;
            }
            else return null;
        }
        else return null;
    };
    
    this.openFilter = function(widget){
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        this.getFilialLogada().then(function(success) {
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            ZhFilterPreferences.setOriginalFilter(widget.id, rowsToSet);
            ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
        });
    };
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };
    
    this.reloadIfEmpty = function(field){
        if(field.dataSource.data.length == 0)
            field.reload();
    };
    
    this.saveData = function(widget){
        if(datasValidas(widget)){
            General.saveCrud(widget).then(sucess => {
                widget.list();
                widget.reload();
            });  
        }
    };
    
    var datasValidas = function(widget){
        let dataset = widget.dataSource.data;
        let valido = true;
        let arrDataFields = ['DTVALILOTE', 'DTENTRLOTE', 'DTFABRPROD'];
        dataset = dataset.filter(row => row.changed);
        dataset.forEach(row => {
            arrDataFields.forEach(param => {
                if(!row[param]){
                    ScreenService.showMessage('O campo '+widget.getField(param).label+' do lote '+row.NRLOTEESTQ+' deve ser informado');
                    valido = false;
                }
            });
            
            if(moment(row.DTENTRLOTE,'DD/MM/YYYY').isAfter(moment(row.DTVALILOTE,'DD/MM/YYYY'))){
                ScreenService.showMessage('Data de entrada do lote '+row.NRLOTEESTQ+' deve ser menor ou igual a data de validade.');
                valido = false;
            }
                
            if(moment(row.DTFABRPROD,'DD/MM/YYYY').isAfter(moment(row.DTVALILOTE,'DD/MM/YYYY'))){
                ScreenService.showMessage('Data de fabricação do lote '+row.NRLOTEESTQ+' deve ser menor ou igual a data de validade.');
                valido = false;
            }
            
        });
        return valido;
    };
    
    this.cancelInlineEdit = function(widget) {
        // TODO criar função para desmarcar linhas editadas
        // GerUtil.desmarcaLinhasEditadas(lancamentoEncargoWidget, CANCELAR);
        widget.dataSource.data.forEach(row => {
            Object.keys(row).forEach(function (key) {
                if(key.indexOf('changed_') > -1){
                    widget.currentRow[key.replace('changed_','')] = row[key];
                    delete widget.currentRow[key];
                }
            });
            delete widget.currentRow.changed;
        });
        widget.list();
    };
    
    this.validaFilial = function(widget, fieldName) {
        /*
        var cdfilial =  widget.currentRow.CDFILIAL;
        if(cdfilial) {
            var filter = [
                {
                    "name":"CDFILIAL",
                    "operator":"=",
                    "value": cdfilial
                }
            ];
            
            General.callRestEngine('est_ajuste#/crud/paramfilial', 'FilterData', filter).then(function(success){
                widget.currentRow.DTPROCESSA = success.dataset.paramfilial[0].DTPROCESSA;
                widget.currentRow.controlaLote = success.dataset.paramfilial[0].IDUTILLOTE;
                if(success.dataset.paramfilial[0].IDCTRLESTQ === 'S') {
                    if(success.dataset.paramfilial[0].IDUTILALMOX === 'S') {
                        if(fieldName === 'NMFILIALDEST') {
                            widget.getField('DSALMOXARIFEDEST').readOnly = false;
                            if(success.dataset.paramfilial[0].IDUTILLCESTQ == 'S') {
                                widget.currentRow.controlaLocalDest = true;
                            }
                            else {
                                widget.currentRow.controlaLocalDest = false;
                            }
                        }
                        else {
                            widget.getField('DSALMOXARIFE').readOnly = false;
                            if(success.dataset.paramfilial[0].IDUTILLCESTQ == 'S') {
                                widget.currentRow.controlaLocal = true;
                            }
                            else {
                                widget.currentRow.controlaLocal = false;
                            }
                        }
                        widget.getField('DSALMOXARIFE').dataSourceFilter.forEach(function(fil){
                            if(fil.name == "CDFILIAL"){
                                fil.value = widget.currentRow.CDFILIAL;
                            }
                        });
                        widget.getField('DSALMOXARIFE').reload();
                    }
                    else {
                        if(fieldName === 'NMFILIALDEST') {
                            widget.getField('DSALMOXARIFEDEST').readOnly = true;
                            widget.getField('DSLOCALESTOQDEST').readOnly = true;    
                        }
                        else {
                            ScreenService.showMessage(mensagens.controlaAlmoxarifado);
                            widget.getField('DSALMOXARIFE').readOnly = true;
                            widget.getField('DSLOCALESTOQ').readOnly = true;
                        }
                    }
                }
                else {
                    if(fieldName === 'NMFILIALDEST') {
                        widget.currentRow.CDFILIALDEST = '';
                        widget.currentRow.NMFILIALDEST = '';
                        ScreenService.showMessage(mensagens.controlaEstoque);
                        widget.currentRow.controlaLocalDest = false;
                    }
                    else {
                        widget.currentRow.CDFILIAL = '';
                        widget.currentRow.NMFILIAL = '';
                        widget.currentRow.controlaLocal = false;
                    }
                }
            });
        }
        else {
            widget.getField('DSALMOXARIFE').readOnly = true;
            widget.getField('DSLOCALESTOQ').readOnly = true;
        }
        widget.currentRow.CDALMOXARIFE = '';
        widget.currentRow.DSALMOXARIFE = '';
        widget.currentRow.CDLOCALESTOQ = '';
        widget.currentRow.DSLOCALESTOQ = '';
        */
	};


};
Configuration(function (ContextRegister){
    ContextRegister.register('Est45500_ManutencaoDoLote_Controller', Est45500_ManutencaoDoLote_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est90000_PerfilOperadoresInventarioCego_Controller.js
var Est90000_PerfilOperadoresInventarioCego_Controller = function(templateManager, ScreenService, MessageService, General, EstUtil, ZHPromise, ZhFilterPreferences, Est90000_PerfilOperadoresInventarioCego_Services) {

    //Operation Widget Functions
    this.onAddOperation = function(widget) {
        widget.newRow();
        General.setEnableField('NMOPERADOR', widget.name);
        widget.currentRow.IDGEST = "N";
        widget.currentRow.IDGGER = "N";
        widget.currentRow.IDGAUD = "N";
    };

    this.onSaveOperation = function(widget) {
        const currentRow = widget.currentRow;

        if (currentRow.IDGEST === 'N' && currentRow.IDGGER === 'N' && currentRow.IDGAUD === 'N') {
            return MessageService.showMessageByCode('BLOCK_NO_PROFILE_OPERATOR_SELECTED');
        }
        Est90000_PerfilOperadoresInventarioCego_Services.onSaveOperation(widget.currentRow).then((saved) => {
            widget.reload().then(() => {
                ScreenService.goBack();
                widget.list();
            });
        }).catch(
            error => {
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage(error);
            }
        );
    };

    this.onDeleteOperation = function(widget) {
        MessageService.confirmMessageByCode('lib#GEN_DELETE_REG').then((success) => {
            Est90000_PerfilOperadoresInventarioCego_Services.onDeleteOperation(widget.currentRow).then((saved) => {
                ScreenService.closeSwipe();
                widget.reload();
            }).catch(
                error => {
                    if (error.data)
                        ScreenService.showMessage(error.data.error);
                    else
                        ScreenService.showMessage(error);
                }
            );
        });
    };

    this.onChangeOperatorField = function(widget) {
        const data = widget.dataSource.data;
        const currentRow = widget.currentRow;

        let check = data.filter(oper => oper.CDOPERADOR === currentRow.CDOPERADOR && oper.__is_new !== true);
        
        return check.length !== 0 ? MessageService.showMessageByCode('BLOCK_OPERATOR_CODE_DUPLICATED') : "";

    };

    //Support Functions

    this.changeHideInconLabelVisible = function(data, label, hide, name, visibility) {
        data.forEach(element => {
            if (element.name === name) {
                element.label = label;
                element.hideIcon = hide;
                element.isVisible = visibility;
            }
        });
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est90000_PerfilOperadoresInventarioCego_Controller', Est90000_PerfilOperadoresInventarioCego_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est91000_ProgramacaoDoInventarioCego_Controller.js
var Est91000_ProgramacaoDoInventarioCego_Controller = function(templateManager, ScreenService, MessageService, General, EstUtil, ZHPromise, ZhFilterPreferences, Lib00206_InfComplProduto_Controller, Est91000_ProgramacaoDoInventarioCego_Services) {

    let IDUTILALMOX, IDUTILLCESTQ, IDUTILLOTE;

    //Inventory Widget Functions
    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est91000_ProgramacaoDoInventarioCego_Services.getFilialLogada().then(success => {
            const dtInicial = getLocalVar('DTPROCESSA');
            const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

            rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };
            rowsToSet.NMFILIAL = [success.dataset.filial[0].NMFILIAL];
            rowsToSet.CDFILIAL = [success.dataset.filial[0].CDFILIAL];
            rowsToSet.IDSITESTCEGO = "PR";

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(success => {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').fields.map(field => {
                    if (field.name === "NMFILIAL" || field.name === "DTCONTAGEM" || field.name === "IDSITESTCEGO") field.reload();
                });
            });
        });
    };

    this.setFilterParams = function(widget, preference = null) {
        let filtro = [];
        const filterRow = preference.FILTER_ROW;

        filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDFILIAL', 'inventoryGrid', true)
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.start : null
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.end : null
            },
            {
                "name": "IDSITESTCEGO",
                "operator": "=",
                "value": filterRow.IDSITESTCEGO ? filterRow.IDSITESTCEGO : null
            },
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": getLocalVar("CDOPERADOR")
            }
        ];

        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    this.onValidateCheckedRowsHandler = function(widget, mode) {
        const checkedRowsValidated = widget.getCheckedRows().filter(row => row.IDSITESTCEGO !== 'PR');

        if (checkedRowsValidated.length === 0) {
            this.changeHideInconLabelVisible(widget.actions, "Excluir", false, "deleteInventory", true);
        }
        else {
            if (mode === 'select' || mode === 'selectAll') {
                widget.unSelectAll(checkedRowsValidated);
                MessageService.showMessageByCode("BLOCK_CHECKED_ROWS");
            }
        }
    };

    this.onGenerateInventory = function(widget) {
        const inventoryGrid = templateManager.container.getWidget('inventoryGrid');
        const currentRow = widget.currentRow;

        if (!widget.isValid()) return;

        this.checkInventoryHandler(currentRow).then(checkedFiliais => {
            ScreenService.showLoader();
            currentRow.CDOPERADOR = getLocalVar("CDOPERADOR");
            Est91000_ProgramacaoDoInventarioCego_Services.onGenerateInventory(currentRow, checkedFiliais).then(success => {
                inventoryGrid.reload().then(() => {
                    ScreenService.closePopup();
                    ScreenService.hideLoader();
                    inventoryGrid.activate();
                });
            }).catch(error => {
                if (error.data)
                    ScreenService.showMessage(error.data.error);
                else
                    ScreenService.showMessage(error);
            });
        });
    };

    this.checkInventoryHandler = (currentRow) => new Promise((resolve, reject) => {
        let checkedFiliais = [],
            confirmationFiliais = [];
        let hasHolidays = false;

        const validatingFilials = () => new Promise((resolve, reject) => {
            let count = 1;
            currentRow.CDFILIAL.forEach(filial => {
                Est91000_ProgramacaoDoInventarioCego_Services.checkInventoryHandler(filial, currentRow.DTCONTAGEM).then((success) => {
                    switch (success.dataset.filialParams.programado) {
                        case "B":
                            hasHolidays = false;
                            confirmationFiliais = [];
                            checkedFiliais = [];
                            return reject('BLOCK_INVENT_EM_ANDAMENTO');
                        case "P":
                            confirmationFiliais.push(filial);
                            break;
                        case "N":
                            if (success.dataset.filialParams.feriado == "S") hasHolidays = true;
                            else checkedFiliais.push(filial);
                            break;
                        default:
                            break;
                    }
                    if (count === currentRow.CDFILIAL.length) return resolve(true);
                    else count++;
                });
            });
        });

        const promiseConfirmMessage = () => new Promise(resolve => {
            MessageService.confirmMessageByCode("UNIDADE_INVENT_PROGRAMADO_CONFIRMATION").then(() => {
                let count = 1;
                confirmationFiliais.forEach(filial => {
                    Est91000_ProgramacaoDoInventarioCego_Services.checkInventoryHandler(filial, currentRow.DTCONTAGEM, false).then((success) => {
                        if (success.dataset.filialParams.feriado == "S") hasHolidays = true;
                        else checkedFiliais.push(filial);
                        if (count === currentRow.CDFILIAL.length) return resolve(true);
                        else count++;
                    });
                });
            });
        });

        validatingFilials().then(() => {
            if (confirmationFiliais.length != 0) {
                promiseConfirmMessage().then(() => {
                    if (hasHolidays === true) MessageService.showMessageByCode('WARNING_GERA_INVENT_FERIADO');
                    return resolve(checkedFiliais);
                });
            }
            else {
                if (hasHolidays === true) MessageService.showMessageByCode('WARNING_GERA_INVENT_FERIADO');
                return resolve(checkedFiliais);
            }
        }).catch(error => {
            MessageService.showMessageByCode(error);
        });
    });

    this.onDeleteIventory = function(widget) {
        widget.dataSource.name = 'est_estoqueCego#/crud/inventestcego';

        if (widget.mode == 'view') {
            General.deleteCrud(widget).then(() => {
                widget.dataSource.name = 'est_estoqueCego#/inventarioCego';
                widget.reload();
            });
        }
        else {
            General.deleteCrudCheckedRows(widget).then(() => {
                widget.dataSource.name = 'est_estoqueCego#/inventarioCego';
                setTimeout(() => {
                    widget.reload();
                }, 200);
            });
        }
    };

    this.onOpenPopUpScheduleInventory = function() {
        Est91000_ProgramacaoDoInventarioCego_Services.getParamEstoqueData().then(success => {
            const { NRDIASMOVESTCEG } = success.dataset.paramestoque[0];
            const widget = templateManager.container.getWidget('scheduleInventoryPopUp');
            let dtInicial = moment(getLocalVar('DTPROCESSA'), 'DDMMYYYY').subtract(NRDIASMOVESTCEG, 'days').format('DD/MM/YYYY');
            let dtFinal = moment(getLocalVar('DTPROCESSA'), 'DDMMYYYY').format('DD/MM/YYYY');

            ScreenService.openPopup(widget).then(() => {
                const { INTERVALO } = widget.currentRow;
                INTERVALO.start = dtInicial;
                INTERVALO.formattedStart = dtInicial;
                INTERVALO.formattedEnd = dtFinal;
                INTERVALO.end = dtFinal;

                widget.getField('INTERVALO').reload();
            });
        });
    };

    this.onClosePopUpScheduleInventory = function() {
        const widgetInventory = templateManager.container.getWidget('inventoryGrid');
        ScreenService.closePopup();
        widgetInventory.activate();
        widgetInventory.reload();
    };

    this.inventoryActionsControl = function(widget, eventType) {
        switch (eventType) {
            case 'onActivate':
                if (widget.currentRow.IDSITESTCEGO !== "PR")
                    this.changeHideInconLabelVisible(widget.actions, "Excluir", true, "deleteInventory", false);
                else
                    this.changeHideInconLabelVisible(widget.actions, "Excluir", true, "deleteInventory", true);
                break;
            default:
                return;
        }
    };

    this.onChangeUnity = function(widget) {

        Est91000_ProgramacaoDoInventarioCego_Services.getParamEstoqueData().then(success => {
            const { NRDIASMOVESTCEG } = success.dataset.paramestoque[0];
            const row = widget.currentRow;
            const dtProcessa = getLocalVar('DTPROCESSA');
            let dtInicial, dtFinal;


            if (row.CDFILIAL.length > 1) {
                dtInicial = moment().subtract(NRDIASMOVESTCEG, 'days').format('DD/MM/YYYY');
                dtFinal = moment().format('DD/MM/YYYY');
            }
            else {
                dtInicial = moment(dtProcessa, 'DDMMYYYY').subtract(NRDIASMOVESTCEG, 'days').format('DD/MM/YYYY');
                dtFinal = moment(dtProcessa, 'DDMMYYYY').format('DD/MM/YYYY');
            }

            row.INTERVALO.start = dtInicial;
            row.INTERVALO.formattedStart = dtInicial;
            row.INTERVALO.formattedEnd = dtFinal;
            row.INTERVALO.end = dtFinal;

            widget.getField('INTERVALO').reload();
        });
    };

    //Product Widget Functions
    this.onAddProducts = function(widget) {
        const widgetInventory = widget.getParent();
        widget.newRow();
        widget.currentRow.CDFILIAL = widgetInventory.currentRow.CDFILIAL;
        widget.currentRow.CDFILIMOVI = widgetInventory.currentRow.CDFILIAL;
        widget.currentRow.CDALMOXARIFE = widgetInventory.currentRow.CDALMOXARIFE;
        widget.currentRow.CDLOCALESTOQ = widgetInventory.currentRow.CDLOCALESTOQ;
        widget.currentRow.DTCONTAGEM = widgetInventory.currentRow.DTCONTAGEM;
    };

    this.onSaveProducts = function(widget) {

        if (widget.currentRow.__is_new == true) {
            widget.currentRow.IDGERITINVCEGPR = "A";
            Est91000_ProgramacaoDoInventarioCego_Services.saveProducts(widget.currentRow).then((saved) => {
                widget.reload().then(() => {
                    ScreenService.goBack();
                    widget.list();
                });

            }).catch(
                error => {
                    if (error.data)
                        ScreenService.showMessage(error.data.error);
                    else
                        ScreenService.showMessage(error);
                }
            );
        }
    };

    this.onDeleteProducts = function(widget) {
        return widget.mode == 'view' ? General.deleteCrud(widget) : General.deleteCrudCheckedRows(widget);
    };

    this.productsActionsControl = function(widget, eventType) {
        switch (eventType) {
            case 'onEnter':
                if (widget.getParent().currentRow.IDSITESTCEGO !== "PR")
                    this.changeHideInconLabelVisible(widget.actions, "Adicionar", true, "Novo", false);
                else
                    this.changeHideInconLabelVisible(widget.actions, "Adicionar", true, "Novo", true);
                break;
            case 'onActivate':
                if (widget.getParent().currentRow.IDSITESTCEGO !== "PR")
                    this.changeHideInconLabelVisible(widget.actions, "Excluir", true, "deleteProducts", false);
                else
                    this.changeHideInconLabelVisible(widget.actions, "Excluir", true, "deleteProducts", true);

                break;
            default:
                return;
        }
    };

    //Support Functions

    //This opens swipe details without change widget mode to view
    this.onOpenDetail = function(widget) {

        Est91000_ProgramacaoDoInventarioCego_Services.getParamFilialData().then((filialParams) => {
            if (filialParams.dataset.paramfilial.length != 0) {

                // Set Global variables
                IDUTILALMOX = filialParams.dataset.paramfilial[0].IDUTILALMOX;
                IDUTILLCESTQ = filialParams.dataset.paramfilial[0].IDUTILLCESTQ;
                IDUTILLOTE = filialParams.dataset.paramfilial[0].IDUTILLOTE;

                if (IDUTILLOTE == 'S') {
                    General.setVisibleField('NRLOTEESTQ', widget.name);
                    General.setVisibleField('NRSUBLOTE', widget.name);
                }
                else {
                    General.setInvisibleField('NRLOTEESTQ', widget.name);
                    General.setInvisibleField('NRSUBLOTE', widget.name);
                }
                widget.redraw();
            }
        });
    };

    this.grayColorHandler = function(row) {
        if (typeof(row.fields) !== 'undefined') return;
        let color = false;
        if (row.IDSITESTCEGO === "PN" || row.IDSITESTCEGO === "DN" || row.IDSITESTCEGO === "EN" ||
            Est91000_ProgramacaoDoInventarioCego_Services.getDate(row.DTCONTAGEM) <
            Est91000_ProgramacaoDoInventarioCego_Services.getDate(row.DTPROCESSA)) color = true;
        return color;
    };

    this.changeHideInconLabelVisible = function(data, label, hide, name, visibility) {
        data.forEach(element => {
            if (element.name === name) {
                element.label = label;
                element.hideIcon = hide;
                element.isVisible = visibility;
            }
        });
    };

    this.popUpFieldsHandler = function(field) {
        let widget = field.widget;
        let fieldProdIni = widget.getField('NMPRODINI');
        let fieldProdFim = widget.getField('NMPRODFIN');
        let fieldGrupProd = widget.getField('NMGRUPPROD');

        switch (field.name) {
            case 'NMPRODFIN':
            case 'NMPRODINI':
                fieldProdIni.validations = { "required": true };
                fieldProdFim.validations = { "required": true };
                delete fieldGrupProd.validations;
                widget.currentRow.NMGRUPPROD = '';
                break;
            case 'NMGRUPPROD':
                widget.currentRow.NMPRODINI = '';
                widget.currentRow.NMPRODFIN = '';
                delete fieldProdIni.validations;
                delete fieldProdFim.validations;
                fieldGrupProd.validations = { "required": true };
                if (widget.currentRow.NMGRUPPROD === '') delete fieldGrupProd.validations;
                break;
            default:
                return;
        }
    };

    this.checkProductRange = function(field) {
        let currentRow = field.widget.currentRow;

        if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if (!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                MessageService.showMessageByCode("PRODUTO_INICIAL_MAIOR_FINAL");
            }
        }
    };

    this.checkDatePopUp = function(field) {
        const currentRow = field.widget.currentRow;
        const todayString = moment().format('DD/MM/YYYY');
        const todayDate = Est91000_ProgramacaoDoInventarioCego_Services.getDate(todayString);
        const dtContagemDate = Est91000_ProgramacaoDoInventarioCego_Services.getDate(currentRow.DTCONTAGEM);

        if (dtContagemDate < todayDate && dtContagemDate != '') currentRow.DTCONTAGEM = todayString;
    };

    this.fieldsFilterPopUpControl = function(field) {
        const popUpWidget = templateManager.container.getWidget("scheduleInventoryPopUp");
        const prodIniField = popUpWidget.getField('NMPRODINI');
        const prodFinField = popUpWidget.getField('NMPRODFIN');
        const groupField = popUpWidget.getField('NMGRUPPROD');

        switch (field.name) {
            case 'NMGRUPPROD':
            case 'NMPRODINI':
            case 'NMPRODFIN':
                if ((popUpWidget.currentRow.NMGRUPPROD == null || popUpWidget.currentRow.NMGRUPPROD == "") &&
                    (popUpWidget.currentRow.NMPRODINI == null || popUpWidget.currentRow.NMPRODINI == "") &&
                    (popUpWidget.currentRow.NMPRODFIN == null || popUpWidget.currentRow.NMPRODFIN == "")) {
                    groupField.validations.required = true;
                }
                else {
                    delete groupField.validations.required;
                }
                break;
            default:
                break;
        }
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est91000_ProgramacaoDoInventarioCego_Controller', Est91000_ProgramacaoDoInventarioCego_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est92000_ConsultaInventarioCego_Controller.js
var Est92000_ConsultaInventarioCego_Controller = function(templateManager, ScreenService, MessageService, General, Colors, EstUtil, ZHPromise, ZhFilterPreferences, localStorageEngine, Est92000_ConsultaInventarioCego_Services) {

    //Inventory Widget Functions

    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est92000_ConsultaInventarioCego_Services.getFilialLogada().then(success => {
            const dtInicial = getLocalVar('DTPROCESSA');
            const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

            rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };
            rowsToSet.NMFILIAL = [success.dataset.filial[0].NMFILIAL];
            rowsToSet.CDFILIAL = [success.dataset.filial[0].CDFILIAL];

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(success => {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').fields.map(field => {
                    if (field.name === "NMFILIAL" || field.name === "DTCONTAGEM") field.reload();
                });
            });
        });
    };

    this.setFilterParams = function(widget, preference = null) {
        let filtro = [];
        const filterRow = preference.FILTER_ROW;

        filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDFILIAL', 'inventoryGrid', true)
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.start : null
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.end : null
            },
            {
                "name": "IDSITESTCEGO",
                "operator": "=",
                "value": "T"
            },
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": getLocalVar("CDOPERADOR")
            }
        ];

        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    this.onOpenSwipe = function(widget) {
        const productsWidget = templateManager.container.getWidget('productsGrid');

        const fieldsVisibilitHandler = (field, WidgetName) => {
            return widget.currentRow.IDUTILLOTE == "S" ? General.setVisibleField(field, WidgetName) : General.setInvisibleField(field, WidgetName);
        };

        ["NRLOTEESTQ", "NRSUBLOTE"].map(field => fieldsVisibilitHandler(field, productsWidget.name));

        ScreenService.openSwipe(productsWidget).then(() => {
            productsWidget.reload();
        });
    };

    //Reports products Functions

    this.openPopupReport = function(args) {
        const widgetPopup = templateManager.container.getWidget('popupReport');
        ScreenService.openPopup(widgetPopup).then(()=>{
            widgetPopup.getField('REPORT_TYPE').value('CIC'); // "defaultValue" JSON wasn't working 
        });
        
    };

    this.openGridReport = function(widget) {
        const inventoryWidget = templateManager.container.getWidget("inventoryGrid");
        const checkedRows = inventoryWidget.getCheckedRows();

        if (checkedRows.length === 0) return MessageService.showMessageByCode('EMPTY_DATA');

        ScreenService.openWindow("est_estoqueCego#est92000_grid_relatorios").then(() => {
            const widgetGrid = templateManager.containers["est_estoqueCego#est92000_grid_relatorios"].getWidget('gridReport');
            const reportType = widget.currentRow.REPORT_TYPE;

            if (widgetGrid) {
                widgetGrid.dataSourceFilter = [
                    {
                        "name": "DATA",
                        "operator": "=",
                        "value": checkedRows
                    },
                    {
                        "name": "REPORT_TYPE",
                        "operator": "=",
                        "value": reportType
	                }
	        	];
                this.reportFormatLayout(reportType, widgetGrid);
            }
            else {
                MessageService.showMessageByCode('GENERATION_REPORT_ERROR');
            }
        });
    };

    //Support Functions
    this.grayColorHandler = function(row) {
        return Est92000_ConsultaInventarioCego_Services.getDate(row.DTCONTAGEM) < Est92000_ConsultaInventarioCego_Services.getDate ? true : false;
    };

    this.reportFormatLayout = function(reportType, widgetGrid) {
        if (reportType === 'LIC') {
            widgetGrid.label = 'Listagem Inventário Cego';

            widgetGrid.getField("QTINVENTESTCEGO").isVisible = false;
            widgetGrid.getField("QTINVENTESTCEGO_LIC").isVisible = true;

        }
        else {
            widgetGrid.label = 'Consulta do Inventário Cego';

            widgetGrid.getField("QTINVENTESTCEGO").isVisible = true;
            widgetGrid.getField("QTINVENTESTCEGO_LIC").isVisible = false;

        }
    };

};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est92000_ConsultaInventarioCego_Controller', Est92000_ConsultaInventarioCego_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est93000_DigitacaoInventarioCego_Controller.js
var Est93000_DigitacaoInventarioCego_Controller = function(templateManager, ScreenService, MessageService, General, Colors, EstUtil, ZHPromise, ZhFilterPreferences, localStorageEngine, Est93000_DigitacaoInventarioCego_Services, ValidacoesEstoque_Service) {

    //Inventory Widget Functions

    this.setFilterParams = function(widget, preference = null) {
        const dtInicial = getLocalVar('DTPROCESSA');
        let filtro = [];
        
        filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
                },
            {
                "name": "DTCONTAGEM",
                "operator": "BETWEEN",
                "value": [dtInicial, General.today()]
                }
            ];
        widget.dataSourceFilter = filtro;
        widget.reload();
    };

    this.onAdd = function(widget) {
        const widgetInventory = widget.getParent();
        widget.newRow();
        widget.currentRow.CDFILIAL = widgetInventory.currentRow.CDFILIAL;
        widget.currentRow.CDFILIMOVI = widgetInventory.currentRow.CDFILIAL;
        widget.currentRow.CDALMOXARIFE = widgetInventory.currentRow.CDALMOXARIFE;
        widget.currentRow.CDLOCALESTOQ = widgetInventory.currentRow.CDLOCALESTOQ;
        widget.currentRow.DTCONTAGEM = widgetInventory.currentRow.DTCONTAGEM;
    };

    this.onSaveHandler = function(widget) {

        const onSave = (widget, row, inlineEdit) => {
            Est93000_DigitacaoInventarioCego_Services.save(row).then((saved) => {
                widget.getParent().currentRow.IDSITESTCEGO = saved.dataset.inventory.IDSITESTCEGO;
                General.setEnableAction('concluir', widget.name);
                widget.reload();
                if (inlineEdit) widget.list();
                else ScreenService.goBack();
            }).catch(error => {
                if (error.data) ScreenService.showMessage(error.data.error);
                else ScreenService.showMessage(error);
            });
        };

        if (widget.mode === 'inline_edit') {

            let filteredRows = widget.dataSource.data.filter(function(row) {
                return row.changed === true && (!row.__is_new || this.rowEditedInline(row));
            });
            filteredRows.push({ 'inlineEdit': true });

            onSave(widget, filteredRows, true);
        }
        else {
            if (widget.isNewRow()) {
                let row = widget.currentRow;
                if (!widget.isValid()) return;
                ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(row.CDFILIAL, row.CDPRODUTO, row.DTCONTAGEM, '', row.CDALMOXARIFE, row.CDLOCALESTOQ, row.NRLOTEESTQ, row.NRSUBLOTE).then((success) => {
                    if (success.dataset.consultaCustoQuantidadeProduto.VRCUSTOPROD > 0) {
                        widget.currentRow.inlineEdit = false;
                        onSave(widget, widget.currentRow, false);
                    }
                    else {
                        return MessageService.showMessageByCode("BLOCK_SEM_CUSTO_MEDIO");
                    }
                });
            }
            else {
                widget.currentRow.inlineEdit = false;
                onSave(widget, widget.currentRow, false);
            }
        }
    };

    this.onDeleteHandler = function(widget) {
        return General.deleteCrud(widget);
    };

    this.onConcludeDigitation = function(widget) {
        const IDSITESTCEGO = widget.getParent().currentRow.IDSITESTCEGO;
        if (IDSITESTCEGO === 'ED') {
            MessageService.confirmMessageByCode("DIGITATION_CONFIRMATION").then(_ => {
                Est93000_DigitacaoInventarioCego_Services.onConcludeDigitation(widget.getParent().currentRow).then((saved) => {
                    ScreenService.goBack();
                    widget.getParent().reload();
                }).catch(error => {
                    if (error.data) ScreenService.showMessage(error.data.error);
                    else ScreenService.showMessage(error);
                });
            });
        }
    };

    //Support Functions

    this.rowEditedInline = function(row) {
        let boolEdited = false;

        $.each(row, function(index, value) {
            if (row.hasOwnProperty('changed_' + index)) {
                boolEdited = true;
            }
        });

        return boolEdited;
    };

    this.ActionsHandler = function(widget) {
        const IDGERITINVCEGPR = widget.currentRow.IDGERITINVCEGPR;
        const IDSITESTCEGO = widget.getParent().currentRow.IDSITESTCEGO;

        if (IDSITESTCEGO === 'DG') {
            General.setDisableAction('editar', widget.name);
            General.setDisableAction('delete', widget.name);
            General.setDisableAction('Novo', widget.name);

            General.setDisableAction('concluir', widget.name);

            widget.getField("QTINVENTESTCEGO").inlineEditable = false;
        }
        else {
            if (IDGERITINVCEGPR === 'E') {
                General.setEnableAction('delete', widget.name);
            }
            else {
                General.setDisableAction('delete', widget.name);
            }

            if (IDSITESTCEGO === 'ED') {
                General.setEnableAction('concluir', widget.name);
            }
            else {
                General.setDisableAction('concluir', widget.name);
            }

            General.setEnableAction('editar', widget.name);
            General.setEnableAction('Novo', widget.name);
            widget.getField("QTINVENTESTCEGO").inlineEditable = "number";
            widget.redraw();
        }
    };

    this.IconHandler = function(widget) {
        let data = widget.dataSource.data;
        return data.map(row => {
            row.icon = row.QTINVENTESTCEGO == 0 ? "⚠" : "";
        });
    };
};
Configuration(function(ContextRegister) {
    ContextRegister.register('Est93000_DigitacaoInventarioCego_Controller', Est93000_DigitacaoInventarioCego_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est94000_AprovacaoInventarioCego_Controller.js
var Est94000_AprovacaoInventarioCego_Controller = function(templateManager, ScreenService, MessageService, General, Colors, EstUtil, ZHPromise, ZhFilterPreferences, localStorageEngine, Est94000_AprovacaoInventarioCego_Services) {

    let IDUTILALMOX, IDUTILLCESTQ, IDUTILLOTE;

    // Open initial screen filter with default values
    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est94000_AprovacaoInventarioCego_Services.getFilialLogada().then(success => {
            const dtInicial = moment().startOf('month').format('DD/MM/YYYY');
            const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

            rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(success => {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').getField("NMFILIAL").reload();
                this.verAlmoxEstoque(templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions'));
            });
        });
    };

    // Creates parameter bag to send to backend
    this.setFilterParams = function(widget, preference = null) {
        const row = preference.FILTER_ROW;

        let startDate = row.DTCONTAGEM.start;
        let endDate = row.DTCONTAGEM.end;
        let CDFILIAL = row.CDFILIAL;
        let CDALMOXARIFE = row.CDALMOXARIFE ? row.CDALMOXARIFE : ' ';
        let CDLOCALESTOQ = row.CDLOCALESTOQ ? row.CDLOCALESTOQ : ' ';

        let params = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            },
            {
                "name": "IDSITESTCEGO",
                "operator": "=",
                "value": "DG"
            },
            {
                "name": "CDALMOXARIFE",
                "operator": "=",
                "value": CDALMOXARIFE
            },
            {
                "name": "CDLOCALESTOQ",
                "operator": "=",
                "value": CDLOCALESTOQ
            },
            {
                "name": "DTCONTAGEM",
                "operator": "BETWEEN",
                "value": [startDate, endDate]
            }
        ];

        if (CDALMOXARIFE == ' ')
            params = params.filter(field => {
                return field.name != 'CDALMOXARIFE' && field.name != 'CDLOCALESTOQ';
            });

        widget.dataSourceFilter = params;
        widget.reload();
    };

    // Almox and/or locale controller
    this.verAlmoxEstoque = function(args) {

        let widget = templateManager.container.getActiveWidget();

        if (widget.name != 'zhFilterPreferencesSelectConditions') widget = args;

        Est94000_AprovacaoInventarioCego_Services.getParamFilialData().then((filialParams) => {
            if (filialParams.dataset.paramfilial.length != 0) {

                // Set Global variables
                IDUTILALMOX = filialParams.dataset.paramfilial[0].IDUTILALMOX;
                IDUTILLCESTQ = filialParams.dataset.paramfilial[0].IDUTILLCESTQ;
                IDUTILLOTE = filialParams.dataset.paramfilial[0].IDUTILLOTE;

                let almoxField = widget.getField('DSALMOXARIFE');
                let LocalField = widget.getField('DSLOCALESTOQ');
                let cdAlmoxField = widget.getField('CDALMOXARIFE');
                let cdLocalEstField = widget.getField('CDLOCALESTOQ');

                widget.currentRow.CDALMOXARIFE = cdAlmoxField.value() ? cdAlmoxField.value() : "";

                //Verifies  warehouse control
                if (IDUTILALMOX === 'N') {
                    almoxField.readOnly = true;
                    almoxField.clearValue();
                    cdAlmoxField.clearValue("");
                    General.setInvisibleField('CDALMOXARIFE', 'inventoryGrid');

                }
                else {
                    General.setVisibleField('CDALMOXARIFE', 'inventoryGrid');
                    almoxField.readOnly = false;
                }

                //Verifies stock locale
                if (IDUTILLCESTQ === 'N') {
                    LocalField.readOnly = true;
                    LocalField.clearValue();
                    cdLocalEstField.clearValue("");
                    General.setInvisibleField('CDLOCALESTOQ', 'inventoryGrid');
                }
                else {
                    General.setVisibleField('CDLOCALESTOQ', 'inventoryGrid');
                    cdLocalEstField.clearValue();
                    LocalField.readOnly = false;
                }

                this.checkDataFilter(widget);
            }
        });
    };

    // Clear filter fields if unit field has change
    this.clearFieldsOnChangeFilial = function(widget) {
        let fields = [
            "DSALMOXARIFE",
            "DSLOCALESTOQ",
            "CDALMOXARIFE",
            "CDLOCALESTOQ"
            ];

        fields.forEach((field) => {
            let fieldToClear = widget.getField(field);
            fieldToClear.clearValue("");

            if (fieldToClear.name === "DSALMOXARIFE" || fieldToClear.name === "DSLOCALESTOQ") {
                fieldToClear.reload();
            }
        });
    };

    //This opens swipe details without change widget mode to view
    this.onOpenDetail = function(widget) {
        let ProductsGrid = templateManager.container.getWidget('productsGrid');

        ScreenService.openSwipe(ProductsGrid).then(() => {
            if (IDUTILLOTE == 'S') {
                General.setVisibleField('NRLOTEESTQ', ProductsGrid.name);
                General.setVisibleField('NRSUBLOTE', ProductsGrid.name);
            }
            else {
                General.setInvisibleField('NRLOTEESTQ', ProductsGrid.name);
                General.setInvisibleField('NRSUBLOTE', ProductsGrid.name);
            }
            ProductsGrid.reload();
        });
    };

    // This function saves approved inventories or send them back to retyping  situation 
    this.onChangeSituation = function(widget, situationAction) {

        const CheckedRows = widget.getCheckedRows();
        let message = situationAction == 'approved' ? "APPROVE_CONFIRMATION" : "RETYPE_CONFIRMATION";

        MessageService.confirmMessageByCode(message).then(_ => {
            Est94000_AprovacaoInventarioCego_Services.onChangeSituation(CheckedRows, situationAction).then((saved) => {
                widget.activate();
                widget.reload();
                if (situationAction == 'approved') MessageService.successNotificationByCode('APPROVE_SUCCESS');
            }).catch(error => {
                if (error.data) ScreenService.showMessage(error.data.error);
                else ScreenService.showMessage(error);
            });
        });
    };

    //It hides or show actions buttons to change the situation of inventories
    this.onValidateCheckedRowsHandler = function(widget) {
        const CheckedRows = widget.getCheckedRows();

        if (CheckedRows.length === 0) {
            General.setDisableAction('approved', widget.name);
            General.setDisableAction('retype', widget.name);
        }
        else {
            General.setEnableAction('approved', widget.name);
            General.setEnableAction('retype', widget.name);
        }
    };

    // It controls fields mode on filter
    this.checkDataFilter = function(args) {

        let widget = templateManager.container.getActiveWidget();

        if (widget.name != 'zhFilterPreferencesSelectConditions') {
            widget = args;
        }

        let LocalField = widget.getField('DSLOCALESTOQ');
        let cdLocalEstField = widget.getField('CDLOCALESTOQ');

        if (IDUTILALMOX === 'S' && IDUTILLCESTQ === 'S') {
            if (widget.currentRow.CDALMOXARIFE !== "") {
                LocalField.validations = { "required": { "message": "Campo Obrigatório" } };
                LocalField.readOnly = false;
            }
            else {
                LocalField.clearValue();
                cdLocalEstField.clearValue();
                delete LocalField.validations;
                LocalField.readOnly = true;
            }

            LocalField.reload();
        }

        if (IDUTILALMOX === 'S' && IDUTILLCESTQ === 'N') {
            if (widget.currentRow.CDALMOXARIFE !== "")
                cdLocalEstField.value(' ');
            else
                cdLocalEstField.clearValue("");
        }

    };

    // It creates icon in a row if quatity == zero
    this.IconHandler = function(widget) {
        let data = widget.dataSource.data;
        return data.map(row => {
            row.icon = row.QTINVENTESTCEGO == 0 ? "⚠" : '';
        });
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est94000_AprovacaoInventarioCego_Controller', Est94000_AprovacaoInventarioCego_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est95100_AnaliseInventarioCego_Controller.js
var Est95100_AnaliseInventarioCego_Controller = function(ZHPromise, metaDataFactory, templateManager, ScreenService, ZhLayoutPreferences, MessageService, General, EstUtil, ZhFilterPreferences, Est95100_AnaliseInventarioCego_Services) {

    //this function sets initial values for fields in te filter
    this.setInitialParams = (widget) => {

        let SetTime = setInterval((time) => {
            let levels = General.getLevelByMask("GER_CDPICTPROD", 1);
            let nivelField = widget.getField("NVTOT");
            let data = [];

            const currentRow = widget.currentRow;
            const dtInicial = getLocalVar('DTPROCESSA');
            const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

            levels.forEach((level) => {
                data.push({ NVTOT: level.nivel });
            });
            nivelField.dataSource.data = data;

            currentRow.NMRELATORIO = "EST951001";
            currentRow.ORDENACAO = "1";
            currentRow.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };

            Est95100_AnaliseInventarioCego_Services.getFilialLogada().then((success) => {

                const result = success.dataset.filial_paramfilial_tipocusto[0];
                currentRow.CDFILIAL = [];
                currentRow.CDFILIAL.push(result.CDFILIAL);
                currentRow.NMFILIAL = [];
                currentRow.NMFILIAL.push(result.NMFILIAL);
                currentRow.IDUTILALMOX = result.IDUTILALMOX;
                currentRow.IDUTILLCESTQ = result.IDUTILLCESTQ;
                this.onChangeReport(widget);
                clearInterval(SetTime);
            });
        }, 1000);
    };

    // It manages fields status on change report 
    this.onChangeReport = function(widget) {
        const currentRow = widget.currentRow;
        const grupoField = widget.getField('NMGRUPPROD');
        const totalizacaoField = widget.getField('NVTOT');
        const ordenacaoField = widget.getField('ORDENACAO');

        this.onChangeUnidade(widget);

        if (currentRow.NMRELATORIO == 'EST951002') {
            grupoField.readOnly = false;
            grupoField.validations = { "required": { "message": "Campo obrigatório" } };

            delete(totalizacaoField.validations);
            totalizacaoField.readOnly = true;
            totalizacaoField.clearValue();

            delete(ordenacaoField.validations);
            ordenacaoField.readOnly = true;
            ordenacaoField.clearValue();

        }
        else {
            totalizacaoField.readOnly = false;
            totalizacaoField.validations = { "required": { "message": "Campo obrigatório" } };

            ordenacaoField.readOnly = false;
            ordenacaoField.validations = { "required": { "message": "Campo obrigatório" } };

            delete(grupoField.validations);
            grupoField.readOnly = true;
            grupoField.clearValue();
        }

        grupoField.reload();
        ordenacaoField.reload();
        totalizacaoField.reload();
    };

    // It manages fields status on change unity 
    this.onChangeUnidade = function(widget) {
        const unidadeField = widget.getField('NMFILIAL');
        const almoxField = widget.getField('DSALMOXARIFE');
        const localField = widget.getField('DSLOCALESTOQ');
        const grupoField = widget.getField('NMGRUPPROD');
        const currentRow = widget.currentRow;

        if (currentRow.NMFILIAL.length == 1) {
            if (currentRow.IDUTILALMOX == 'S') {
                //almoxField.validations = { "required": { "message": "Campo obrigatório" } };
                almoxField.readOnly = false;
                if (currentRow.IDUTILLCESTQ == 'S') {
                    //localField.validations = { "required": { "message": "Campo obrigatório" } };
                    localField.readOnly = false;
                }
                else {
                    //delete(localField.validations);
                    localField.readOnly = true;
                }
            }
            else {
                delete(almoxField.validations);
                almoxField.readOnly = true;

                //delete(localField.validations);
                localField.readOnly = true;
            }
        }
        else {
            //delete(almoxField.validations);
            almoxField.readOnly = true;

            //delete(localField.validations);
            localField.readOnly = true;

            delete(grupoField.validations);
            grupoField.readOnly = true;

            if (currentRow.NMFILIAL.length == 0) {
                currentRow.CDALMOXARIFE = '';
                currentRow.DSALMOXARIFE = '';
                currentRow.CDLOCALESTOQ = '';
                currentRow.DSLOCALESTOQ = '';
                currentRow.CDGRUPPROD = '';
                currentRow.NMGRUPPROD = '';
            }
        }
        unidadeField.reload();
    };

    // It manages fields status on change almoxarife field 
    this.onChangeAlmox = function(widget) {
        const currentRow = widget.currentRow;
        const localField = widget.getField("DSLOCALESTOQ");
        currentRow.DSLOCALESTOQ = null;
        if (currentRow.IDUTILLCESTQ == 'S') {
            currentRow.CDLOCALESTOQ = null;
            if (currentRow.CDALMOXARIFE) {
                localField.readOnly = false;
                localField.reload();
            }
            else {
                localField.readOnly = true;
            }
        }
    };

    // It checks if the final product is smaller than initial product
    this.checkProductRange = function(field) {
        const currentRow = field.widget.currentRow;

        if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if (!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                MessageService.showMessageByCode("PRODUTO_INICIAL_MAIOR_FINAL");
            }
        }
    };

    // It manages report generation
    this.onGenerateReport = function(widget, format) {
        const { currentRow } = widget;
        currentRow.__FORMAT = format;

        if (widget.isValid()) {
            General.openReport("est_estoqueCego#/analiseInventario", currentRow.NMRELATORIO, currentRow);
        }
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est95100_AnaliseInventarioCego_Controller', Est95100_AnaliseInventarioCego_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est95200_AceiteRecusaDivergencias_Controller.js
var Est95200_AceiteRecusaDivergencias_Controller = function(templateManager, ZhLayoutPreferences, EstUtil, ZhFilterPreferences, Est95200_AceiteRecusaDivergencias_Services) {

    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est95200_AceiteRecusaDivergencias_Services.getFilialLogada().then(success => {
            const { NMFILIAL, CDFILIAL } = success.dataset.filial[0];
            const dtInicial = getLocalVar('DTPROCESSA');
            const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

            rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };
            rowsToSet.NMFILIAL = [NMFILIAL];
            rowsToSet.CDFILIAL = [CDFILIAL];
            rowsToSet.NMRELATORIO = 'EST95200';
            rowsToSet.IDSITUACEITEINV = 'A';
            rowsToSet.ORDENACAO = '1';

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(() => {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').fields.map(field => {
                    if (field.name === "NMFILIAL" || field.name === "DTCONTAGEM") {
                        field.reload();
                    }
                });
            });
        });
    };

    this.setFilterParams = function(widget, preference = null) {
        let filtro = [];
        const filterRow = preference.FILTER_ROW;

        filtro = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDFILIAL', 'MainWidget', true)
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.start : null
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.end : null
            },
            {
                "name": "ORD",
                "operator": "=",
                "value": EstUtil.getFieldValue('ORD', 'MainWidget', true)
            }
        ];

        let IDSITUACEITEINV = EstUtil.getFieldValue('IDSITUACEITEINV', 'MainWidget', true);
        let situationParam = [];
        switch (IDSITUACEITEINV) {
            case 'A':
                situationParam = [
                    {
                        "name": "SIT",
                        "operator": "=",
                        "value": "A"
                    },
                    {
                        "name": "SIT2",
                        "operator": "=",
                        "value": " "
                    },
                ];
                break;
            case 'R':
                situationParam = [
                    {
                        "name": "SIT",
                        "operator": "=",
                        "value": "R"
                    },
                    {
                        "name": "SIT2",
                        "operator": "=",
                        "value": " "
                    },
                ];
                break;
            case 'T':
                situationParam = [
                    {
                        "name": "SIT",
                        "operator": "=",
                        "value": "A"
                    },
                    {
                        "name": "SIT2",
                        "operator": "=",
                        "value": "R"
                    },
                ];
                break;
            default:
                break;
        }
        filtro.push(situationParam);

        widget.dataSourceFilter = filtro.flat();
        widget.reload();
    };

    this.reportProcess = function(args, afterOpenReport = false) {
        const { owner: widget } = args;

        if (afterOpenReport) {
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
        else {
            let metadata = args.e.metadata;
            let { start, end } = EstUtil.getFieldValue('DTCONTAGEM', widget.name, true);
            let statusAceite = this.formatReportHeaderData(widget, 'IDSITUACEITEINV');
            let ordenação = this.formatReportHeaderData(widget, 'ORD');

            let filter = [
                {
                    label: "Data de Contagem",
                    value: start + ' a ' + end
                },
                {
                    label: "Situação do Aceite",
                    value: statusAceite
                },
                {
                    label: "Ordenação",
                    value: ordenação
                },
            ];

            metadata.filter = filter;
            metadata.title = 'Aceite/Recusa de Divergências';
            metadata.orientation = 'LANDSCAPE';
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
    };

    this.formatReportHeaderData = (widget, fieldName) => {
        let value = EstUtil.getFieldValue(fieldName, widget.name, true);

        let newString;
        switch (value) {
            case 'A':
                newString = 'Aprovadas';
                break;
            case 'R':
                newString = 'Recusadas';
                break;
            case 'T':
                newString = 'Ambas';
                break;
            case '1':
                newString = 'Por Unidade - Código';
                break;
            case '2':
                newString = 'Por Unidade - Nome';
                break;
            case '3':
                newString = 'Por Situação do Aceite';
                break;
            case '4':
                newString = 'Por Data/hora do Aceite/Recusa';
                break;
            default:
                break;
        }
        return newString;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est95200_AceiteRecusaDivergencias_Controller', Est95200_AceiteRecusaDivergencias_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est95300_ControleFiliaisInventarioCego_Controller.js
var Est95300_ControleFiliaisInventarioCego_Controller = function(templateManager, ZhLayoutPreferences, EstUtil, ZhFilterPreferences, Est95300_ControleFiliaisInventarioCego_Services) {

    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est95300_ControleFiliaisInventarioCego_Services.getFilialLogada().then(success => {
            const { NMFILIAL, CDFILIAL } = success.dataset.filial[0];
            const dtInicial = getLocalVar('DTPROCESSA');
            const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

            rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };
            rowsToSet.NMFILIAL = [NMFILIAL];
            rowsToSet.CDFILIAL = [CDFILIAL];
            rowsToSet.NMRELATORIO = 'EST95300';
            rowsToSet.ORD = '1';

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(() => {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').fields.map(field => {
                    if (field.name === "NMFILIAL" || field.name === "DTCONTAGEM") {
                        field.reload();
                    }
                });
            });
        });
    };

    this.setFilterParams = function(widget, preference = null) {
        const { FILTER_ROW: filterRow } = preference;
        const UnitiesCode = EstUtil.getFieldValue('CDFILIAL', 'MainWidget', true);

        let filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "_" + UnitiesCode.join("_") + "_"
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.start : null
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": filterRow.DTCONTAGEM ? filterRow.DTCONTAGEM.end : null
            },
            {
                "name": "ORD",
                "operator": "=",
                "value": EstUtil.getFieldValue('ORD', 'MainWidget', true)
            }
        ];

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.reportProcess = function(args, afterOpenReport = false) {
        const { owner: widget } = args;

        if (afterOpenReport) {
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
        else {
            let metadata = args.e.metadata;
            let { start, end } = EstUtil.getFieldValue('DTCONTAGEM', widget.name, true);
            let ordenação = this.formatReportHeaderData(widget, 'ORD');

            let filter = [
                {
                    label: "Ordenação",
                    value: ordenação
                },
                {
                    label: "Data de Contagem",
                    value: start + ' a ' + end
                }
            ];

            metadata.filter = filter;
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
    };

    this.formatReportHeaderData = (widget, fieldName) => {
        let value = EstUtil.getFieldValue(fieldName, widget.name, true);

        let newString;
        switch (value) {
            case '1':
                newString = 'Por Unidade - Código';
                break;
            case '2':
                newString = 'Por Unidade - Nome';
                break;
            case '3':
                newString = 'Por Qtde Inventários – Crescente';
                break;
            case '4':
                newString = 'Por Qtde Inventários – Decrescente';
                break;
            default:
                break;
        }
        return newString;
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est95300_ControleFiliaisInventarioCego_Controller', Est95300_ControleFiliaisInventarioCego_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est96000_DivergenciasInventarioCego_Controller.js
var Est96000_DivergenciasInventarioCego_Controller = function(ZHPromise, metaDataFactory, templateManager, ScreenService, ZhLayoutPreferences, MessageService, General, EstUtil, ZhFilterPreferences, Est96000_DivergenciasInventarioCego_Services) {

    let IDUTILALMOX, IDUTILLCESTQ, IDUTILLOTE, DTPROCESSA, IDINVESTCEGO, IDOBREALACEDIV;

    // Open initial screen filter with default values
    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est96000_DivergenciasInventarioCego_Services.getFilialLogada().then(success => {
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;

            Est96000_DivergenciasInventarioCego_Services.getParamFilialData().then(success => {
                const paramFilialData = success.dataset.paramfilial[0];

                DTPROCESSA = getLocalVar("DTPROCESSA");
                IDINVESTCEGO = paramFilialData.IDINVESTCEGO;
                IDOBREALACEDIV = paramFilialData.IDOBREALACEDIV;

                const dtInicial = DTPROCESSA;
                const dtFinal = moment().endOf('month').format('DD/MM/YYYY');

                rowsToSet.DTCONTAGEM = { "start": dtInicial, "end": dtFinal, "formattedStart": dtInicial, "formattedEnd": dtFinal };

                ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(success => {
                    templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').getField("NMFILIAL").reload();
                });

            });
        });
    };

    // Creates parameter bag to send to backend
    this.setFilterParams = function(widget, preference = null) {
        const row = preference.FILTER_ROW;

        let startDate = row.DTCONTAGEM.start;
        let endDate = row.DTCONTAGEM.end;
        let CDFILIAL = row.CDFILIAL;

        let params = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": CDFILIAL
            },
            {
                "name": "IDSITESTCEGO",
                "operator": "=",
                "value": "AP"
            },
            {
                "name": "DTCONTAGEM",
                "operator": "BETWEEN",
                "value": [startDate, endDate]
            }
        ];

        widget.dataSourceFilter = params;
        widget.reload();
    };

    // This opens swipe details without change widget mode to view
    this.onOpenDetail = function(widget) {
        const InventoryTabbed = templateManager.container.getWidget('InventoryTabbed');
        const writeOffTabbed = templateManager.container.getWidget('writeOffItemGrid');
        const controlTabbed = templateManager.container.getWidget('controlItemGrid');

        ScreenService.openSwipe(InventoryTabbed).then(_ => {
            let promises = [];
            promises.push(writeOffTabbed.reload());
            promises.push(controlTabbed.reload());

            return ZHPromise.all(promises).then(_ => {
                this.actionsVisibility(writeOffTabbed);
            });
        });
    };

    // This function saves approved inventories or send them back to retyping situation 
    this.reportProcess = function(args, afterOpenReport = false) {
        const widget = args.owner;
        const descriptionField = widget.getField('DESCRIPTIONDATES');
        const almoxField = widget.getField('CDALMOXARIFE');
        const localField = widget.getField('CDLOCALESTOQ');

        if (afterOpenReport) {
            descriptionField.grouped = false;
            almoxField.grouped = false;
            localField.grouped = false;
            ZhLayoutPreferences.applyOriginalLayout(widget);

        }
        else {
            let writeOffLabel = "Relatório de Divergências do Estoque Cego - Itens a Baixar ";
            let controlLabel = "Relatório de Divergências do Estoque Cego - Itens a Controlar";

            args.e.metadata.title = widget.name === 'writeOffItemGrid' ? writeOffLabel : controlLabel;

            descriptionField.grouped = true;
            almoxField.grouped = true;
            localField.grouped = true;
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
    };

    // Manages actions functionalities according to buttons
    this.actionsManagementHandler = function(widget, action, firstTime = true) {

        widget = widget.name == 'writeOffItemGrid' ? widget : templateManager.container.getWidget('writeOffItemGrid');
        const rows = widget.dataSource.data;
        const ignoredKeys = ["__rightActions", "__leftActions", "__createdLocal", "__index", "$error"];
        const InventoryRow = widget.getParent().getParent().currentRow;

        switch (action) {
            case 'accept':

                if (rows.length > 0) {
                    MessageService.confirmMessageByCode('ACCEPT_CONFIRMATION').then(_ => {
                        Est96000_DivergenciasInventarioCego_Services.actionsManagementHandler(InventoryRow, rows, true, action).then(_ => {

                        }).catch(
                            error => {
                                if (error.data) ScreenService.showMessage(error.data.error);
                                else ScreenService.showMessage(error);
                            }
                        );
                    });
                }
                else {
                    MessageService.confirmMessageByCode('OPERATION_CONFIRMATION').then(_ => {
                        Est96000_DivergenciasInventarioCego_Services.actionsManagementHandler(InventoryRow, rows, false, action);

                    }).catch(
                        error => {
                            if (error.data) ScreenService.showMessage(error.data.error);
                            else ScreenService.showMessage(error);
                        }
                    );
                }

                break;
            case 'refuse':
                if (firstTime) {
                    MessageService.confirmMessageByCode('REFUSE_CONFIRMATION').then(_ => {
                        const refusePopUp = templateManager.container.getWidget('refuseInventoryPopUp');

                        ScreenService.openPopup(refusePopUp).then(_ => {
                            refusePopUp.currentRow = Util.clone(InventoryRow, ignoredKeys);
                            refusePopUp.currentRow.NMFILIAL = EstUtil.getFieldValue('NMFILIAL', 'inventoryGrid', true);
                            refusePopUp.currentRow.DSOBSRECAJUST = '';
                            refusePopUp.getField('NMFILIAL').reload();
                        });

                    });
                }
                else {
                    const refuseWidget = templateManager.container.getWidget('refuseInventoryPopUp');

                    Est96000_DivergenciasInventarioCego_Services.actionsManagementHandler(refuseWidget.currentRow, [], '', action).then(success => {
                        MessageService.successNotificationByCode('REFUSE_SUCCESS');
                    }).catch(
                        error => {
                            if (error.data) ScreenService.showMessage(error.data.error);
                            else ScreenService.showMessage(error);
                        }
                    );
                }
                break;

            case 'adjust':
                MessageService.confirmMessageByCode('ADJUST_CONFIRMATION').then(_ => {
                    Est96000_DivergenciasInventarioCego_Services.actionsManagementHandler(InventoryRow, rows, false, action).then(success => {
                        let IrregularItems = success.dataset.ResultData.ItemsAjusted;
                        let ItemsWithoutCost = success.dataset.ResultData.ItemsWithoutCost;
                        let Nrlancestq = success.dataset.ResultData.nrlancestq;

                        if (IrregularItems.length == 0 && ItemsWithoutCost.length == 0)
                            MessageService.successNotificationByCode('GERA_AJUST_SUCCESS', 3000, { "NRLANCEST": Nrlancestq });

                        else {
                            if (IrregularItems.length != 0)
                                MessageService.showMessageByCode('WARNING_GERA_AJUST_IRREGULAR', { "NRLANCEST": Nrlancestq });

                            if (ItemsWithoutCost.length != 0) {
                                let messageItems = '<br>';
                                ItemsWithoutCost.forEach(item => {
                                    messageItems += '<br>' + item.CDPRODUTO + ' - ' + item.NMPRODUTO + '<br>';
                                });
                                MessageService.showMessageByCode('WARNING_GERA_AJUST_NO_COST_STOCK', { "ITEMS": messageItems });
                            }
                        }
                    }).catch(
                        error => {
                            if (error.data) ScreenService.showMessage(error.data.error);
                            else ScreenService.showMessage(error);
                        }
                    );
                });
                break;
            default:
                break;

        }
    };

    // Controls actions visibility on enter widget
    this.actionsVisibility = function(widget) {
        const InventoryRow = widget.getParent().getParent().currentRow;
        const controlTabbed = templateManager.container.getWidget('controlItemGrid');

        Est96000_DivergenciasInventarioCego_Services.getGroupOperatorInfo().then((success) => {
            
            const dataReturn = success.dataset.autgrupop_grupoper_opergrupop;
            const ValidateGroupOperator = (OperatorGroupInfo, action) => {
                if (action == 'adjust') {
                    return OperatorGroupInfo.filter((item) => {
                        return item.IDAUTGRUPOP === 'GAUD';
                    });
                }
                else {
                    return OperatorGroupInfo.filter((item) => {
                        return item.IDAUTGRUPOP === 'GGER' || item.IDAUTGRUPOP === 'GAUD';
                    });
                }
            };
            const DtprocessaDate = Est96000_DivergenciasInventarioCego_Services.getDate(DTPROCESSA);
            const DtContagemDate = Est96000_DivergenciasInventarioCego_Services.getDate(InventoryRow.DTCONTAGEM);
            const DtutilDate = Est96000_DivergenciasInventarioCego_Services.getDate(InventoryRow.DTDIAUTIL);
            let hasItems = (widget.dataSource.data.length != 0 || controlTabbed.dataSource.data.length != 0) ? true : false;

            if (
                ValidateGroupOperator(dataReturn, 'accept').length != 0 &&
                InventoryRow.IDSITUACEITEINV == "P" && hasItems &&
                (DtprocessaDate > DtContagemDate &&
                    DtprocessaDate <= DtutilDate)
            ) {
                General.setEnableAction('accept', widget.name);
                General.setEnableAction('refuse', widget.name);
                if (IDOBREALACEDIV == 'S')
                    General.setDisableAction('refuse', widget.name);
            }
            else {
                General.setDisableAction('accept', widget.name);
                General.setDisableAction('refuse', widget.name);
            }
            if (
                ValidateGroupOperator(dataReturn, 'adjust').length != 0 &&
                    hasItems && (IDINVESTCEGO == "S" &&
                    InventoryRow.IDINVGERAAJUSTE != "S" &&
                    InventoryRow.IDSITUACEITEINV != "R") &&
                    (DtprocessaDate > DtContagemDate &&
                    DtprocessaDate <= DtutilDate)
            ) {
                General.setEnableAction('adjust', widget.name);
            }
            else {
                General.setDisableAction('adjust', widget.name);
            }
        });
    };

    //this function closes the refuse inventory pop up
    this.onClosePopUpRefuseInventory = function(widget) {
        const writeOffGrid = templateManager.container.getWidget('writeOffItemGrid');
        ScreenService.closePopup();
        writeOffGrid.activate();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est96000_DivergenciasInventarioCego_Controller', Est96000_DivergenciasInventarioCego_Controller);
});


// FILE: ./assets/modules/estoqueCego/assets/js/controllers/Est97000_PercentualDivergenciasInventarioCego_Controller.js
var Est97000_PercentualDivergenciasInventarioCego_Controller = function(templateManager, ScreenService, MessageService, General, EstUtil, ZhFilterPreferences, Est97000_PercentualDivergenciasInventarioCego_Services) {

    // Open initial screen filter with default values
    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        Est97000_PercentualDivergenciasInventarioCego_Services.getFilialLogada().then(success => {
            const { NMFILIAL, CDFILIAL } = success.dataset.filial[0];
            rowsToSet.NMFILIAL = NMFILIAL;
            rowsToSet.CDFILIAL = CDFILIAL;

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(() => {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').getField("NMFILIAL").reload();
            });
        });
    };

    // This function creates a new currentRow with default values
    this.onAddProducts = function(widget) {
        if (EstUtil.onValidateFilterRequirement(widget,true)) return;
        widget.currentRow.CDFILIAL = EstUtil.getFieldValue('CDFILIAL', 'MainWidget', true);
    };

    // It saves products
    this.onSaveProducts = function(widget) {
        if (!widget.isValid()) return;

        if (widget.name == 'MainWidget') {
            EstUtil.saveCrud(widget).then(() => {
                ScreenService.closeSwipe();
                widget.list();
                widget.reload();
                MessageService.successNotificationByCode('REGISTER_SUCCESS');
            });
        }
        else {
            Est97000_PercentualDivergenciasInventarioCego_Services.onCheckDataset(widget.currentRow).then((dataSet) => {
                EstUtil.saveDataSetCrud(dataSet, 'parperdivestceg').then(() => {
                    widget.currentRow = [];
                    this.onClosePopUp(widget);
                    MessageService.successNotificationByCode('AUTO_REGISTER_SUCCESS');
                });
            });
        }
    };

    // It opens auto register pop up
    this.onOpenPopUp = function() {
        const PopUpAutoRegister = templateManager.container.getWidget('AutoRegisterPopUp');
        ScreenService.openPopup(PopUpAutoRegister);
    };

    // It closes auto register pop up and reactivate main widget
    this.onClosePopUp = function() {
        const MainWidget = templateManager.container.getWidget('MainWidget');
        ScreenService.closePopup();
        MainWidget.activate();
        MainWidget.reload();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Est97000_PercentualDivergenciasInventarioCego_Controller', Est97000_PercentualDivergenciasInventarioCego_Controller);
});


// FILE: ./assets/modules/lancamentos/assets/js/controllers/Est11000_EntradaEstoque_Controller.js
function Est11000_EntradaEstoque_Controller($rootScope, ScreenService, General, templateManager, Est11000_EntradaEstoque_Service, ValidacoesEstoque_Service, Lib00206_InfComplProduto_Controller, ZHPromise, metaDataFactory, ZhFilterPreferences, MessageService, EstUtil) {

	this.cdTipCustEst = 3;
	this.filialctrlestoq = 'S';

	/**
	 * Essa função tem como objetivo executar a função do botão de voltar da tela. Quando em modo de
	 * inserir retirada ele roda o data da widget para achar qual posição estava sendo editada/inserida.
	 * Quando encontrado, seta a currentRow e a posição no data com o backup feito na função do modo
	 * de edição, troca a editabilidade de alguns campos e passa a widget para o modo view.
	 * Quando o registro esta sendo inserido, ainda no modo inserir retirada, a posição do data é
	 * retirada.
	 * Quando não está no modo de inserir uma retirada, chama função do General que cancela a edição
	 * ou inserção.
	 *
	 * @param args | evento da action
	 */
	this.backEditItens = function(args) {
		var widget = args.owner.widget;
		if (widget.call === 'insertRetirada' && !widget.currentRow.novo) {
			widget.dataSource.data.forEach(function(row, rowIndex, rows) {
				if (widget.currentRow === row) {
					rows[rowIndex] = widget.backupRow;
					widget.currentRow = widget.backupRow;

					//this.changeReadOnlyByName(widget, 'QTTOTLANCTO', true);
					//this.changeReadOnlyByName(widget, 'QTDEVOLESTOQ', true);

					widget.view();
				}
			}.bind(this));
		}
		
		else {
			General.backOrCancel(args);
		}
	};

	/**
	 * Essa função tem como finalidade verificar o valor do campo de produto na inserção
	 * de um item na retirada. Quando tem valor chama a Lib que abre o popup de Informações
	 * Complementares de Produto e na sequência permite que outros campos sejam editavéis.
	 * Caso o campo não tenha valor, os campos, antes habilitados, são desabilitados para
	 * edição. Depois os valores dos campos que aparecem na tela são zerados.
	 *
	 * @param widget | widget de itens da retirada
	 */
	this.callInformacoesComplmentares = function(widget) {
		var promise = ZHPromise.defer();
		if (widget.currentRow.CDARVPROD) {
			widget.promise = promise;
			let widgetEntrada = widget.container.getWidget('entrada');
			let filterRow = isset(() => widgetEntrada.appliedFilter.FILTER_ROW) ? widgetEntrada.appliedFilter.FILTER_ROW : false;
			if (isset(() => filterRow.CDALMOXARIFE)) {
				if (filterRow.CDALMOXARIFE != ' ')
					widget.currentRow.CDALMOXARIFE = filterRow.CDALMOXARIFE;
			}
			if (isset(() => filterRow.CDLOCALESTOQ)) {
				if (filterRow.CDLOCALESTOQ != ' ')
					widget.currentRow.CDLOCALESTOQ = filterRow.CDLOCALESTOQ;
			}
			Lib00206_InfComplProduto_Controller.infComplementares(widget, widget.currentRow);
		}
		else {
			promise.reject();
		}
		return promise;
	};

	/**
	 * Essa função tem como finalidade carregar a widget de itens da retirada, e setar na
	 * widget de retirada o campo de Tipo de Retirada, colocando na currentRow no código
	 * e nome do tipo de retirada.
	 *
	 * @param widget | widget de retirada
	 */
	this.carregaItems = function(widget) {
		var widgetItens = widget.widgets[0];
		if (!widget.currentRow.__is_new) {
			widgetItens.reload().then(function() {
				widget.currentRow.CDTIPORETI = widgetItens.dataSource.data[0].CDTIPORETI;
				widget.currentRow.NMTIPORETI = widgetItens.dataSource.data[0].NMTIPORETI;
			});
		}
		else {
			widgetItens.dataSource.data = [];
		}
	};

	/**
	 * Essa função tem como finalidade percorrer o filtro montado pelo ZhFilter. É verificado
	 * se o usuario selecinou almoxarifado e/ou localização, se não tiver selecionado é
	 * inserido no filtro almoxarifado e/ou localizacao com o valor de T para aquele que não
	 * foi selecionado no filtro. É verificado também, se o filtro recebeu data, se não tiver
	 * recebido é passado no filtro DATA o valor T, se tiver recebido é passado NULL.
	 *
	 * @param widget | widget com name de "retirada"
	 */
	this.changeFilter = function(widget, preference) {
		/*	var filterRow = preference.FILTER_ROW;
		var filterData;
		var dtinicial, dtfinal, data, cdalmoxar, cdlocal;
		let actAdicionar = widget.getAction('adicionar');
		actAdicionar.isVisible = this.filialctrlestoq == 'S'? true: false;
		
		if(filterRow.DTINICIAL && filterRow.DTFINAL){
			dtinicial = filterRow.DTINICIAL;
			dtfinal = filterRow.DTFINAL;
			data = null;
		} else {
			dtinicial = null;
			dtfinal = null;
			data = 'T';
		}
		if(filterRow.ALMOXARIFE && (filterRow.ALMOXARIFE != "")){
			cdalmoxar = filterRow.ALMOXARIFE;
		} else {
			cdalmoxar = 'T';
		}
		if(filterRow.CDLOCALESTOQ && (filterRow.CDLOCALESTOQ != "")){
			cdlocal = filterRow.CDLOCALESTOQ;
		} else {
			cdlocal = 'T';
		}
        var filter = [
        	{
                "name":"CDFILIAL",
                "operator":"=",
                "value": filterRow.CDFILIAL
            },
            {
                "name":"CDALMOXARIFE",
                "operator":"=",
                "value": cdalmoxar
            },
            {
                "name":"CDLOCALESTOQ",
                "operator":"=",
                "value": cdlocal
            },
            {
                "name":"DTINICIAL",
                "operator":"=",
                "value": dtinicial
            },
            {
                "name":"DTFINAL",
                "operator":"=",
                "value": dtfinal
            },
            {
                "name":"DATA",
                "operator":"=",
                "value": data
            }
        ];
		//filter.push(filterData[0]);
        widget.dataSourceFilter = filter;
		widget.reload();*/
	};

	/**
	 * A função tem a finalidade de trocar as propriedades label, hideicon e isvisible
	 * de uma action que será selecionada pelo seu nome.
	 *
	 * @param data | array de objetos (as actions)
	 * @param label | string com o novo valor da propriedade label
	 * @param hide | boolean com o valor para setar a propridade hideIcon
	 * @param name | string com o nome da action que vai ser alterada
	 * @param visibility | boolean para setar a visibilidade de action
	 */
	this.changeHideInconLabelVisible = function(data, label, hide, name, visibility) {
		data.forEach(function(element) {
			if (element.name === name) {
				element.label = label;
				element.hideIcon = hide;
				element.isVisible = visibility;
			}
		});
	};

	/**
	 * Essa função tem como finalidade trocar o valor da propriedade readOnly
	 * de um field de uma widget.
	 *
	 * @param widget | widget onde sera pego o campo.
	 * @param name | nome do field que vai ser pegado da widget e alterado a propriedade
	 * @param read | true ou false para habilitar ou não a propriedade
	 */
	this.changeReadOnlyByName = function(widget, name, read) {
		widget.getField(name).readOnly = read;
	};

	/**
	 * Essa função tem como finalidade controlar as validações do campo de campo de
	 * localização chegando se a unidade do filtro controla localização de almoxarifado
	 * e se o campo de almoxarifado está preenchido. Se ambos forem verdadeiros, habilita
	 * o campo, se não desabilita o campo e limpa a currentRow dos dados de localização
	 *
	 * @param widget | widget do filtro
	 */
	this.changeReadOnlyLocal = function(widget) {
		var almox = widget.currentRow.CDALMOXARIFE;
		var flocal = widget.getField('DSLOCALESTOQ');
		if (almox && Est11000_EntradaEstoque_Service.utilizaLocalizacao()) {
			flocal.readOnly = false;
			flocal.reload();
		}
		else {
			widget.currentRow.CDLOCALESTOQ = null;
			widget.currentRow.DSLOCALESTOQ = null;
			flocal.readOnly = true;
		}
	};

	/**
	 * Essa função tem a finalidade de voltar as actions para o estado normal depois
	 * de cancelar a inserção de uma retirada sem requisição.
	 *
	 * @param args | evento da action
	 */
	this.checkBackAction = function(args) {
		var widget = args.owner.widget;
		let tabbedWidget = templateManager.container.getWidget('entradaTabbed');
		let widgetPrincipal = templateManager.container.getWidget('entrada');
		let widgetGridItens = templateManager.container.getWidget('itens');

		if (tabbedWidget.currentRow.__is_new) {
			widgetPrincipal.dataSource.data.pop();
			tabbedWidget.currentRow = {};
			widgetPrincipal.currentRow = {};
			ScreenService.goBack();
		}
		else {
			if (widgetPrincipal.dataSource.data[widgetPrincipal.dataSource.data.length - 1].__is_new)
				widgetPrincipal.dataSource.data.pop();

			ScreenService.goBack();
		}

		if (tabbedWidget.currentRow.changed) {
			tabbedWidget.currentRow.changed = false;
		}


	};

	/**
	 * Essa função tem a finalidade de verificar se existe algum produto que ainda não teve
	 * suas informações complementares colocadas e está sendo salvo. Se isso acontecer, dá uma
	 * mensagem ao usuário e retorna false para a função que lhe chamou.
	 *
	 *	@param data | data da widget de itens
	 */
	this.checkProdutosCompletos = function(data) {
		var validação = true;
		data.forEach(function(produto) {
			if (produto.COMPLETO !== 'S') {
				validação = false;

				// ScreenService.showMessage('Não é possível salvar pois faltam informações para o produto ' + produto.NMPRODUTO + '.');

				/* MSG NOVA */
				var variaveis = {
					NMPRODUTO: produto.NMPRODUTO
				};

				MessageService.showMessageByCode("PRODUTOIRREGULAR", variaveis);
				/* FIM MSG NOVA */

			}
		});
		return validação;
	};

	this.checkRequisicaoComplementarCreation = function(widget) {
		var promise = ZHPromise.defer();
		var item = widget.currentRow;

		if (this.checkShouldCreateRequisicaoComplementar(item)) {
			var msg = "Quantidade requisitada não disponível em estoque para o produto " + item.NMPRODUTO + " : " + item.QTESTOQDIA + " " + item.SGUNIDADE + ". Deseja gerar uma requisição complementar?";

			ScreenService.confirmMessage(msg,
				'question',
				function() { //sim
					item.createReqComp = true;
					item.QTLANCREQCOMP = item.QTREQUESTO - item.QTESTOQDIA;

					promise.resolve();
				}.bind(this),
				function() { //não
					item.createReqComp = false;

					promise.resolve();
				}.bind(this));
		}
		else promise.resolve();

		return promise;
	};

	this.checkShouldCreateRequisicaoComplementar = function(item) {
		return item.NRREQUESTO && item.IDREQCOMPEST == 'S' &&
			item.QTESTOQDIA == item.QTLANCTOEST &&
			item.QTESTOQDIA < item.QTREQUESTO;
	};

	/**
	 * Essa função tem como finalidade controlar em qual estado o usuário esta usando a tela
	 * e chamar a função correspondente ao estado que ela está.
	 *
	 * @param widget | widget de retirada
	 */
	this.checkSaveMod = function(widget) {
		if (widget.isValid()) {
			if (widget.currentRow.__is_new && widget.currentRow.NRREQUESTO) {
				widget.enabled = false;
				this.getItensOfRequisicao(widget.widgets[0], widget.currentRow.CDFILIAL, widget.currentRow.NRREQUESTO);
				this.focusAddProduto(widget.widgets[0]);
			}
			else if (widget.currentRow.__is_new) {
				widget.enabled = false;
				this.focusAddProduto(widget.widgets[0]);
			}
			else if (!widget.currentRow.__is_new) {
				this.saveEditRetirada(widget);
			}
		}
	};

	/**
	 * Essa função tem como finalidade limpar alguns dados da currentRow quando o produto mudar
	 * (inserindo um produto em uma retirada que está sendo inserida). Qunado troca-se o produto,
	 * só os capos de valor são alterados, quando o valor do campo é null, é passado true para
	 * clearAll e valores relacionados ao produto são limpados também.
	 *
	 * @param widget | widget de itens
	 * @param clearAll | parametro para lipar ou não todos os capos da currentRow
	 */
	this.clearRowItens = function(widget, clearAll) {
		if (widget.currentRow.COMPLETO === 'S') {
			widget.currentRow.QTREQUESTO = "";
			widget.currentRow.QTTOTLANCTO = "";
			widget.currentRow.QTDEVOLESTOQ = "";
			widget.currentRow.QTLANCTOEST = "";
			widget.currentRow.VRTOTLANCTO = "";
			if (clearAll) {
				widget.currentRow.NMPRODUTO = "";
				widget.currentRow.CDARVPROD = "";
				widget.currentRow.SGUNIDADE = "";
				widget.currentRow.VRUNILANCTO = "";
			}
		}
	};

	/**
	 * Essa função tem como finalidede clonar todos os dados do dataSource da widget, de
	 * itens principalmente. Não clonando escopos das actions que podem estar na row da
	 * widget.
	 *
	 * @param data | array de objetos; dataSource.data da widget
	 * @return arrray | array de objetos clonados do data da widget
	 */
	this.copyDataWidgetItens = function(data) {
		var copy = [];
		data.forEach(function(item) {
			copy.push(Util.clone(item, ['__rightActions', '__leftActions']));
		});
		return copy;
	};

	/**
	 * Essa função tem a finalidade verificar em qual modo está sendo chamado a função de excluir um
	 * item da retirada. Conforme o modo, chama a função respectiva.
	 *
	 * @param widget | widget de itens
	 */
	this.deleteItem = function(args) {
		let widget = args.owner.widget;
		let widgetTabbed = templateManager.container.getWidget('entradaTabbed');
		let bloqEdit = validaLançamentoDeIntegracaoes(widgetTabbed.currentRow, args);
		if (!bloqEdit) {
			this.removeItemEntrada(args);
		}
	};

	/**
	 * Inicialmente a função verifica se aquela retirada pode ser editada (inserir/excliuir/editar um
	 * item é editar uma retirada), se não puder é dada uma mesagem para o usuario. Se puder ser editado
	 * a currentRow é clonada (não é clonada __rightActions e __leftActions pois são escopos), a widget é
	 * colocado em modo de edição e a editabilidade de alguns campos é alterada
	 *
	 * @param widget | widget de itens
	 */
	this.editItens = function(args) {
		let widget = args.owner.widget;
		let widgetTabbed = templateManager.container.getWidget('entradaTabbed');
		let bloqEdit = validaLançamentoDeIntegracaoes(widgetTabbed.currentRow, args);

		if (!bloqEdit) {
			if (!widget.currentRow.NRORDPROD) {
				widget.backupRow = Util.clone(widget.currentRow, ['__rightActions', '__leftActions']);
				widget.edit();
			}
			//this.changeReadOnlyByName(widget, 'QTTOTLANCTO', false);
			//this.changeReadOnlyByName(widget, 'QTDEVOLESTOQ', false);
		}

	};

	/**
	 * Essa função tem como finalidade trocar o label do botão de salvar (para garantir que ficará
	 * com a label certa), colocar a widget em modo de edição e desabilitar a widget de produtos.
	 * Isso acontece ao acionar o botão de editar a retirada.
	 * Antes disso é verificado se a retirada pode ser editada, verificando os capos NRORDPROD e
	 * NRLANCESTRET, onde se um deles estiver preencido, é dado sua respctiva mensagem, impedindo
	 * o usuário de editar a retirada.
	 *
	 * @param widget | widget de retirada
	 */
	this.editRetirada = function(widget) {

		if (widget.currentRow.EXISTECONSUMO == "N") {
			if (!widget.currentRow.NRORDPROD) {
				if (!widget.currentRow.NRLANCESTRET) {
					General.edit(widget);
					this.enabledWidget(true);
				}
				else {
					ScreenService.showMessage("Operação bloqueada! Retirada oriunda de um ajuste de inventário.");
				}
			}
			else {
				ScreenService.showMessage("Operação bloqueada! Retirada gerada pela Ordem de Produção.");
			}
		}
		else {
			ScreenService.showMessage("Operação bloqueada! Não é possível editar pois existe um consumo.");
		}
	};

	/**
	 * Essa função tem como finalidade habilitar e desabilitar a widget "itens", onde
	 * são exibidos os produtos da retirada. Ela é a widget filha da widget "retirada".
	 *
	 * @param enabled | booleano para habilitar ou desabilitar a widget de itens
	 */
	this.enabledWidget = function(enabled) {
		var widget = templateManager.container.getWidget('itens');
		widget.enabled = enabled;
	};

	/**
	 * Essa função tem a finalidade de ativar a widget de produto, deixá-la ativa,
	 * setar uma propriedade de controle e mudar a configuração das actions para
	 * funcionarem no modo de inserir retirada sem requisição.
	 *
	 * @param widget | widget de itens
	 */
	this.focusAddProduto = function(widget) {
		this.enabledWidget(true);
		widget.call = 'insertRetirada';
		widget.activate();
		widget.floatingControl.refreshAction = false;
		this.changeHideInconLabelVisible(widget.actions, 'Cancelar', true, 'VoltarList', true);
		this.changeHideInconLabelVisible(widget.actions, 'Adicionar', true, 'salvar', true);
		this.changeHideInconLabelVisible(widget.actions, 'Salvar', true, 'salvarNaRetirada', true);
	};


	/**
	 * Essa função tem como finalidade consultar as parametrizações da filial selecionada
	 * no filtro, caso ainda não tenha sido selecionada nenhuma filial no filtro, é usado
	 * o valor FILIAL_LOGADA, que pegara o código da filial logada no backend. Se não
	 * retornar nenhum erro, chama uma função que vai setar valores default para outros
	 * campos do filtro.
	 *
	 * @param widget | widget
	 */
	this.getParamFilialOnce = function(widget) {
		this.setFieldsGerais(widget);
		var filterRow = {};
		this.getFilialLogada().then(function(success) {
			let returnedDataset = success.dataset.paramfilial_paramvalidacoes[0];
			filterRow.CDFILIMOVI = returnedDataset.CDFILIAL;
			filterRow.NMFILIAL = returnedDataset.NMFILIAL;
			this.filialctrlestoq = returnedDataset.IDCTRLESTQ;

			if (returnedDataset.IDCTRLESTQ == 'N')
				ScreenService.showMessage('Filial logada não controla estoque.');

			ZhFilterPreferences.openFilterWidget(widget, filterRow).then(function() {
				var filial = filterRow.CDFILIMOVI || widget.currentRow.CDFILIMOVI || 'FILIAL_LOGADA';
				Est11000_EntradaEstoque_Service.getParamFilial(filial, true).then(function(succes) {
					let widgetFiltro = templateManager.container.getActiveWidget();
					if (succes) {
						this.setCampos(widgetFiltro, succes, filial);
					}
					else {
						ScreenService.showMessage(succes);
					}
				}.bind(this), function(error) {
					ScreenService.showMessage(error.data.error);
				});
			}.bind(this));
		}.bind(this));
	};

	this.getFilialLogada = function(filial = 'FILIAL_LOGADA') {
		var filter = [
			{
				"name": "CDFILIAL",
				"operator": "=",
				"value": filial
                }
		    ];
		return General.callRestEngine('est#/crud/paramfilial_paramvalidacoes', 'FilterData', filter);
	};

	this.setFieldsGerais = function(widget) {
		templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
	};

	this.getParamFilial = function(widget) {
		let widgetGridProdutos = widget.container.getWidget('itens');
		var filial = widget.currentRow.CDFILIMOVI;
		if (filial) {
			Est11000_EntradaEstoque_Service.getParamFilial(filial, true).then(function(succes) {
				if (succes) {
					let filialctrlestoq = succes.IDCTRLESTQ;
					if (filialctrlestoq == 'S' ? false : true)
						ScreenService.showMessage('Filial não Controla Estoque');
					if (succes.IDUTILLOTE == 'S')
						widgetGridProdutos.paramInfoCompl.ShowLote = true;
					else
						widgetGridProdutos.paramInfoCompl.ShowLote = false;

					this.setCampos(widget, succes, filial);
				}
				else {
					ScreenService.showMessage(succes);
				}
			}.bind(this), function(error) {
				ScreenService.showMessage(error.data.error);
			});
		}
	};

	/**
	 * Essa função tem como finalidade dar um newRow na widget de itens e com base nos
	 * dados que a mesma já tem, calcular qual será o próximo sequencial do item para
	 * aquela retirada de estoque.
	 * Antes disso é verificado se a retirada pode ser editada, verificando os capos NRORDPROD e
	 * NRLANCESTRET, onde se um deles estiver preencido, é dado sua respctiva mensagem, impedindo
	 * o usuário de editar a retirada.
	 *
	 * @param widget | widget de itens da retirada
	 */
	this.newItem = function(args) {
		let widget = args.owner.widget;
		let widgetTabbed = widget.container.getWidget('entradaTabbed');
		let bloqEdit = validaLançamentoDeIntegracaoes(widgetTabbed.currentRow, args);
		if (!bloqEdit) {
			widget.newRow();
			widget.currentRow.novo = true;
			widget.currentRow.CDFILIMOVI = widgetTabbed.currentRow.CDFILIMOVI;
			widget.currentRow.NRLANCESTQ = widgetTabbed.currentRow.NRLANCESTQ;
			widget.currentRow.NRSEQUITEM = Est11000_EntradaEstoque_Service.geraNrsequitem(widget.dataSource.data);
			widget.currentRow.DTLANCMOVI = widget.getParent().currentRow.DTLANCESTQ.substr(0, 10);
		}
	};

	/**
	 * Essa função tem a finalidade de validar a relação entre os valores dos campos de valor
	 * na widget de itens.
	 *
	 * @param widget | widget de itens
	 */
	this.onChangeQt = function(widget) {
		var row = widget.currentRow;
		var calcTotValues = true,
			isValid = false;
		var permitEstoqNeg = widget.getParent().currentRow.idpermestneg;
		if (parseFloat(row.QTDEVOLESTOQ) < 0) {
			row.QTDEVOLESTOQ = null;
			ScreenService.showMessage('A quantidade devolvida não pode ser menor que 0.');
		}
		else if (parseFloat(row.QTTOTLANCTO) <= 0) {
			row.QTTOTLANCTO = null;
			ScreenService.showMessage('A quantidade retirada não pode ser menor ou igual a 0.');
		}
		else if (parseFloat(row.QTTOTLANCTO) < parseFloat(row.QTDEVOLESTOQ)) {
			row.QTDEVOLESTOQ = null;
			ScreenService.showMessage('A quantidade retirada não pode ser menor que a quantidade devolvida.');
		}
		else if ((parseFloat(row.QTTOTLANCTO) > parseFloat(row.QTESTOQDIA)) && permitEstoqNeg == 'N') {
			ScreenService.showMessage('Quantidade informada não disponível no estoque.');
			row.QTTOTLANCTO = null;
		}
		else if (parseFloat(row.QTTOTLANCTO) === parseFloat(row.QTDEVOLESTOQ)) {
			calcTotValues = false;
			ScreenService.confirmMessage('Quantidade movimentada igual a 0, portanto o item não realizará atualização de estoque. Confirmar operação?',
				'question',
				function() { //sim
					Est11000_EntradaEstoque_Service.setQtLanctoEst(row);
					Est11000_EntradaEstoque_Service.setVrTotLancto(row);
					Est11000_EntradaEstoque_Service.setVrlanctoest(row);

					this.checkSaveProduto(widget, true);
				}.bind(this),
				function() { //não
					row.QTDEVOLESTOQ = null;
					Est11000_EntradaEstoque_Service.setQtLanctoEst(row);
					Est11000_EntradaEstoque_Service.setVrTotLancto(row);
					Est11000_EntradaEstoque_Service.setVrlanctoest(row);
				});
		}
		else {
			isValid = true;
		}

		if (calcTotValues) {
			Est11000_EntradaEstoque_Service.setQtLanctoEst(row);
			Est11000_EntradaEstoque_Service.setVrTotLancto(row);
			Est11000_EntradaEstoque_Service.setVrlanctoest(row);
		}

		return isValid;
	};

	this.openPosicaoDeEstoque = function(widget) {
		var popup = this.getPosicaoDeEstoquePopup(widget);

		ScreenService.openPopup(popup);

		popup.currentRow = widget.currentRow;
		popup.parentWidget = widget;

		this.getDataForPosicaoDeEstoquePopup(popup);
	};

	this.getDataForPosicaoDeEstoquePopup = function(popup) {
		Est11000_EntradaEstoque_Service.getPosicaoDeEstoquePorProduto(popup.currentRow).then(function(posestdia) {
			if (posestdia) {
				popup.currentRow = posestdia;
				let qtToSet = posestdia.QTESTOQDIA;
				if (qtToSet == 'undefined' || qtToSet == null)
					qtToSet = 0;
				popup.parentWidget.currentRow.QTESTOQDIA = qtToSet;
			}
			else {
				popup.currentRow.QTESTOQDIA = 0;
			}
		}).catch(function(error) {
			ScreenService.showMessage(error.data.error);
		});
	};

	this.getPosicaoDeEstoquePopup = function(parentWidget) {
		var widget = parentWidget.posestPopup;

		if (!widget) {
			widget = this.getPosicaoDeEstoquePopUpObject();
			widget.container = parentWidget.container;
			widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

			widget = metaDataFactory.widgetFactory(widget, parentWidget);

			parentWidget.posestPopup = widget;
			widget.parent = parentWidget;
		}
		else
			widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

		return widget;
	};

	this.getPosicaoDeEstoquePopUpObject = function() {
		return {
			"name": "posestPopup",
			"defaultLabel": "Posição de Estoque do Produto ",
			"template": "widget/form.html",
			"isVisible": true,
			"showAsTab": false,
			"actions": [
				{
					"name": "back",
					"label": "Voltar",
					"showAsAction": "back",
					"isVisible": true,
					"events": [
						{
							"name": "ActionEvent",
							"code": "ScreenService.closePopup()"
                        }
                    ]
                },
				{
					"name": "refresh",
					"label": "Atualizar",
					"showAsAction": "never",
					"isVisible": true,
					"events": [
						{
							"name": "ActionEvent",
							"code": "Est11000_EntradaEstoque_Controller.getDataForPosicaoDeEstoquePopup(args.owner.widget)"
                        }
                    ]
                }
            ],
			"fields": [
				{
					"name": "CDALMOXARIFE",
					"label": "Almoxarifado",
					"isVisible": true,
					"readOnly": true,
					"template": "field/text-edit.html",
					"class": 6,
					"showOnForm": true,
					"events": []
                },
				{
					"name": "CDLOCALESTOQ",
					"label": "Localização",
					"isVisible": true,
					"readOnly": true,
					"template": "field/text-edit.html",
					"class": 6,
					"showOnForm": true,
					"events": []
                },
				{
					"name": "NRLOTEESTQ",
					"label": "Lote",
					"isVisible": true,
					"readOnly": true,
					"template": "field/text-edit.html",
					"class": 6,
					"showOnForm": true,
					"events": []
                },
				{
					"name": "NRSUBLOTE",
					"label": "Sublote",
					"isVisible": true,
					"readOnly": true,
					"template": "field/text-edit.html",
					"class": 6,
					"showOnForm": true,
					"events": []
                },
				{
					"name": "QTESTOQDIA",
					"label": "Quantidade",
					"isVisible": true,
					"readOnly": true,
					"template": "field/text-edit.html",
					"class": 6,
					"showOnForm": true,
					"events": []
                },
				{
					"name": "DTVALILOTE",
					"label": "Data de Validade",
					"isVisible": true,
					"sequenceInForm": 80,
					"readOnly": true,
					"defaultValue": null,
					"template": "field/calendar.html",
					"dateFormat": "DD/MM/YYYY",
					"class": 6,
					"showOnForm": true,
					"events": []
                }
            ]
		};
	};

	/**
	 * Essa função tem a finalidade de ser executada quando o usuário quer remover um item de uma
	 * retirada que está sendo inserido, dessa forma é procurado em qual posição do data da widget
	 * está o item que o usuário quer retirar, e então é tirado do data da widget.
	 *
	 *	@param widget | widget de itens
	 */
	this.removeItemDataSource = function(widget) {
		var itens = widget.dataSource.data;
		for (var i = 0; i < itens.length; i++) {
			if (itens[i].CDARVPROD === widget.currentRow.CDARVPROD &&
				itens[i].CDALMOXARIFE === widget.currentRow.CDALMOXARIFE &&
				itens[i].CDLOCALESTOQ === widget.currentRow.CDLOCALESTOQ &&
				itens[i].NRLOTEESTQ === widget.currentRow.NRLOTEESTQ &&
				itens[i].NRSUBLOTE === widget.currentRow.NRSUBLOTE
			) {
				itens.splice(i, 1);
				i = itens.length;
				ScreenService.closeSwipe();
			}
		}
	};

	/**
	 * Inicialmente a função verifica se a retirada pode ser editada (inserir/excluir/editar um item
	 * é editar uma retirada), se tiver problema é dado uma mensagem na tela. Se tudo estiver certo,
	 * a função de excluir do General é chamada para excluir o item. Essa função é chamada quando
	 * se está excluindo um item de uma retirada que já existe.
	 *
	 * @param widget | widget de itens
	 */
	this.removeItemEntrada = function(args) {
		let widget = args.owner.widget;
		let widgetTabbed = templateManager.container.getWidget('entradaTabbed');
		let bloqEdit = validaLançamentoDeIntegracaoes(widgetTabbed.currentRow, args);
		if (!bloqEdit) {
			if (widget.dataSource.data.length > 1) {
				General.deleteCrud(widget);
			}
			else {
				ScreenService.showMessage("Operação bloqueada! Não é possível existir uma retirada sem item.");
			}
		}
	};

	/**
	 *
	 *
	 * @param widget | widget de retirada
	 */
	this.removeEntrada = function(args) {
		let widget = args.owner.widget;
		let widgetTabbed = templateManager.container.getWidget('entradaTabbed');
		let widgetPrincipal = templateManager.container.getWidget('entrada');
		let bloqEdit = validaLançamentoDeIntegracaoes(widgetTabbed.currentRow, args);

		if (!bloqEdit) {
			if (widget.currentRow.__is_new) {
				widget.dataSource.data = widget.dataSource.data.filter(row => {
					return row != widget.currentRow;
				});
				ScreenService.closeSwipe();
			}
			else {
				General.deleteCrud(widgetPrincipal);
			}
		}
	};

	/**
	 * Essa função tem como finalidade retornar todas as propriedades e labels de todas as widgets,
	 * para o estado normal da tela. Ela é chamado quando se termina ou cancela o processo de inserção
	 * de uma retirada.
	 *
	 * @param widget | widget de retirada
	 */
	this.returnStatus = function(widget) {
		//Mudanças na widget de Retirada para voltar ao estado de visualização
		widget.enabled = true;
		this.changeHideInconLabelVisible(widget.actions, 'Salvar', true, 'salvar', true);
		//Mudanças na widget de Itens para voltar ao estado de visualização
		var widgetItens = widget.widgets[0];
		widgetItens.enabled = true;
		this.changeHideInconLabelVisible(widgetItens.actions, 'Voltar', false, 'VoltarList', true);
		this.changeHideInconLabelVisible(widgetItens.actions, 'Salvar', true, 'salvarNaRetirada', false);
		widgetItens.floatingControl.refreshAction = true;
		widgetItens.call = 'padrão';
	};

	/**
	 * Essa função tem como finalidade salvar um produto novo em uma retirada já existente
	 * ou salvar a edição de um produto existente em uma retirada já existente. A função seta
	 * alguns campos da currentRow de itens com dados de referência da currentRow de retirada
	 * quando estiver inserindo um novo produto. Na sequência valida se essa operação não vai
	 * gerar estoque negativo e por fim chama a função de salvar do General.
	 *
	 */
	this.save = function(widgetRetirada) {
		var widgetItens = widgetRetirada.container.getWidget('itens');
		var itens = widgetItens.currentRow;
		var lancamento = widgetRetirada.currentRow;
		if (widgetItens.currentRow.__is_new) {
			itens.NRREQUESTO = lancamento.NRREQUESTO;
			itens.NRLANCESTQ = lancamento.NRLANCESTQ;
			itens.CDTIPORETI = lancamento.CDTIPORETI;
			itens.CDCENTCUST = lancamento.CDCENTCUST;
			itens.DTLANCMOVI = lancamento.DTLANCESTQ;
			itens.CDPRODMOVI = itens.CDPRODUTO;
		}
		General.saveCrud(widgetItens).then(() => {
			widgetRetirada.reload();
			this.carregaItems(widgetRetirada);
		});
	};

	/**
	 * Essa função tem a finalidade de executar os processos da edição de uma retirada. Chamando
	 * a função de salvar da widget, e no sucesso deixando a widget no modo view e dando reload
	 * na widget filha.
	 *
	 * @param widget | widget de retirada
	 */
	this.saveEditRetirada = function(widget) {
		//if(widget.isValid()) {
		widget.saveRow().then(function() {
			widget.view();
			widget.widgets[0].enabled = true;
			widget.widgets[0].reload();
		}, function(error) {
			ScreenService.showMessage(error.data.error);
		});
		//}
	};

	/**
	 * Essa função tem a finalidade de salvar a retirada e executar as mudanças na widget pós
	 * salvamento. Para salvar é necessário que existam itens sendo inseridos junto à retirada
	 * e que os itens que estão sendo inseridos estejam com as informações completas (passado
	 * pela tela Informações Complementares de Produto).
	 *
	 * @param widget | widget de itens
	 */
	this.saveNewEntrada = function(widget) {
		var widgetRetirada = widget.getParent();
		var itens = widget.dataSource.data;
		if (itens.length <= 0) {
			ScreenService.showMessage('É preciso adicionar itens à retirada para salva-la.');
		}
		else if (this.checkProdutosCompletos(itens)) {
			widgetRetirada.currentRow.ITENS = itens;
			ScreenService.showLoader();
			Est11000_EntradaEstoque_Service.saveRetirada(widgetRetirada.currentRow).then(function(sucess) {
				var lancamento = sucess.dataset.lancamento[0];
				var parentCurrentRowIndex = widget.getParent().dataSource.data.indexOf(widget.getParent().currentRow);

				this.returnStatus(widget.getParent());

				lancamento.NMFILIAL = widget.getParent().appliedFilter.FILTER_ROW.NMFILIAL;

				widget.getParent().currentRow = lancamento;
				widget.getParent().currentRow.__is_new = false;
				widget.getParent().dataSource.data[parentCurrentRowIndex] = widget.getParent().currentRow;
				widget.getParent().view();

				widget.reload();
				widget.activate();
				ScreenService.hideLoader();
			}.bind(this), function(error) {
				ScreenService.showMessage(error.data.error);
				ScreenService.hideLoader();
			});
		}
	};

	/**
	 * Essa função tem como finalidade setar datas defaut para o filtro da tela conforme a
	 * data de processamento da filial selecionada, trocar a editabilidade do campo de
	 * almoxarifado conforme a parametrização da filial selecionada, e setar o nome e
	 * código da filial logada caso o evento estiver sendo chamado no início da tela.
	 * Depois chama a função para validar a regra dos campos de data.
	 *
	 * @param widget | widget do filtro
	 * @param array | recebe um array com código e nome da filial selecionada no filtro
	 * @param filial | codigo da filial para saber se seta ou não o campo de filial no filtro
	 */
	this.setCampos = function(widget, array, filial) {
		//Setando datas do campo Data
		var dtprocessa = Est11000_EntradaEstoque_Service.getDtprocessa();
		var dtinicial = moment(dtprocessa, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY');
		widget.currentRow.DTINICIAL = dtinicial;
		widget.currentRow.DTFINAL = dtprocessa;

		if (!widget.currentRow.INTERVALO) {
			widget.currentRow.INTERVALO = {};
		}
		widget.currentRow.INTERVALO.start = dtinicial;
		widget.currentRow.INTERVALO.end = dtprocessa;

		//Habilitando Almoxarifado, reseta a currentRow e desabilita Localização
		if (Est11000_EntradaEstoque_Service.utilizaAlmoraifado()) {
			widget.getField('DSLOCALESTOQ').readOnly = false;
			widget.currentRow.CDLOCALESTOQ = null;
			widget.currentRow.DSLOCALESTOQ = null;
			widget.getField('DSALMOXARIFE').readOnly = false;
			widget.currentRow.CDALMOXARIFE = null;
			widget.currentRow.DSALMOXARIFE = null;
			widget.getField('DSALMOXARIFE').reload();
		}
		else {
			widget.getField('DSLOCALESTOQ').readOnly = true;
			widget.currentRow.CDLOCALESTOQ = null;
			widget.currentRow.DSLOCALESTOQ = null;
			widget.getField('DSALMOXARIFE').readOnly = true;
			widget.currentRow.CDALMOXARIFE = null;
			widget.currentRow.DSALMOXARIFE = null;
		}
		//Setando campo de filial se for no início da tela
		if (filial === 'FILIAL_LOGADA') {
			widget.currentRow.CDFILIAL = array.CDFILIAL;
			widget.currentRow.NMFILIAL = array.NMFILIAL;
		}
		this.setMinMaxAndRequired(widget);
	};

	this.intervaloChange = function(field) {
		let widget = field.widget;
		widget.currentRow.DTINICIAL = widget.currentRow.INTERVALO.start;
		widget.currentRow.DTFINAL = widget.currentRow.INTERVALO.end;
	};

	/**
	 * Essa função tem como finalidade setar o código da filial que será inserida na retirada
	 * na row do novo item a ser inserido naquela retirada.
	 *
	 * @param widget | widget itens da retirada
	 */
	this.setCdfilimovi = function(widget) {
		let widgetTabbed = widget.container.getWidget('entradaTabbed');
		var newRow = widget.currentRow;
		newRow.CDFILIMOVI = widgetTabbed.currentRow.CDFILIMOVI;
		newRow.DTLANCMOVI = widgetTabbed.currentRow.DTLANCESTQ;
	};

	/**
	 * Essa função tem como finalidade setar a filial e a data de lançamento quando é
	 * acionado o botão de adicionar uma retirada. É sugerida a filial que foi passada
	 * no filtro, e a data de lançamento é a data de processamento da filial. Depois
	 * chama uma função para desabilitar a aba de produtos, e outra para trocar a label
	 * do botão inferior direito da tela.
	 *
	 * @param widget | widget de retirada
	 */
	this.setCurrentRowNew = function(widget) {
		widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
		widget.currentRow.CDFILILANC = widget.appliedFilter.FILTER_ROW.CDFILIAL;
		widget.currentRow.CDFILIMOVI = widget.appliedFilter.FILTER_ROW.CDFILIAL;
		widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
		widget.currentRow.DTLANCESTQ = Est11000_EntradaEstoque_Service.getDtprocessa();

		//this.enabledWidget(false);
		//this.changeHideInconLabelVisible(widget.actions, "Adicionar Produtos", true, "salvar", true);
	};

	/**
	 * Essa função tem a finalidade de pegar os dois campos de data do filtro, e caso o
	 * valor deles seja null, em ambos, tira sua obrigatoriedade. Caso um deles tenha
	 * valor, é setado o max e min date de cada um com o valor do outro campo e ambos
	 * são colocados como obrigatórios. Dessa forma se um deles estiver com valor, ambos
	 * serão obrigatórios, mas se o usuário não quiser filtrar por data, e deixar ambos
	 * vazios, eles deixam de ser obrigatórios.
	 *
	 * @param widget | widget do filtro
	 */
	this.setMinMaxAndRequired = function(widget) {
		var fdateIni = widget.getField('DTINICIAL');
		var fdateFin = widget.getField('DTFINAL');

		if (!fdateIni.value() && !fdateFin.value()) {
			delete fdateIni.filterProperties.validations.required;
			delete fdateFin.filterProperties.validations.required;
		}
		else {
			fdateIni.filterProperties = { "validations": {} };
			fdateIni.filterProperties.maxDate = fdateFin.value();
			fdateIni.filterProperties.validations.required = true;

			fdateFin.filterProperties = { "validations": {} };
			fdateFin.filterProperties.minDate = fdateIni.value();
			fdateFin.filterProperties.validations.required = true;
		}
	};

	/**
	 * Essa função tem a finalidade de setar o campo de Valor Unitário do produto que foi selecionado
	 * na widget de itens. Para pegar esse valor é necessario exeutar uma função do banco de dados.
	 * O resultado da função é o valor unitário do produto e sua quantidade em estoque. Ambos os valores
	 * são colocados na currentRow da widget.
	 *
	 * @param widget | widget de itens
	 */
	this.setValorUnitario = function(widget) {
		var deffer = ZHPromise.defer();
		Est11000_EntradaEstoque_Service.getCdtipocusto(widget.currentRow.CDFILIAL).then(function(cdtipocusto) {
			widget.currentRow.DTLANCMOVI = widget.getParent().currentRow.DTLANCESTQ;
			Est11000_EntradaEstoque_Service.setBlankLocalEstFields(widget);
			ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(
				widget.currentRow.CDFILIAL,
				widget.currentRow.CDPRODUTO,
				widget.currentRow.DTLANCMOVI.substr(0, 10),
				cdtipocusto,
				widget.currentRow.CDALMOXARIFE,
				widget.currentRow.CDLOCALESTOQ,
				widget.currentRow.NRLOTEESTQ,
				widget.currentRow.NRSUBLOTE
			).then(function(success) {
				var consulta = success.dataset.consultaCustoQuantidadeProduto;
				widget.currentRow.QTESTOQDIA = consulta.QTESTOQDIA;
				widget.currentRow.VRUNILANCTO = consulta.VRCUSTOPROD;
				Est11000_EntradaEstoque_Service.setVrTotLancto(widget.currentRow);
				Est11000_EntradaEstoque_Service.setVrlanctoest(widget.currentRow);
				widget.currentRow.COMPLETO = 'S';
				deffer.resolve();
			}, function(error) {
				ScreenService.showMessage(error.data.error);
				deffer.reject();
			});
		}, function(error) {
			ScreenService.showMessage(error.data.error);
			deffer.reject();
		});
		return deffer;
	};

	/**
	 * Essa função tem a finalidade de mudar a data de lançamento durante a inserção
	 * de uma nova retirada. É verificada a data de processamento da filial que o
	 * usuário selecionou.
	 *
	 * @param widget | widget de retirada, no modo de inserção (swipe)
	 */
	this.suggestDtprocessa = function(widget) {
		var cdfilial = widget.currentRow.CDFILIAL;
		Est11000_EntradaEstoque_Service.getParamFilial(cdfilial, false).then(function(succes) {
			if (typeof succes === 'string') {
				ScreenService.showMessage(succes);
			}
			else {
				widget.currentRow.DTLANCESTQ = succes.DTPROCESSA;
			}
		}, function(error) {
			ScreenService.showMessage(error.data.error);
		});
	};

	this.loadDataSource = function(widget) {
		widget.getField('DSALMOXARIFE').dataSourceFilter.forEach(function(fil) {
			if (fil.name === 'CDFILIAL') {
				fil.value = widget.currentRow.CDFILIMOVI;
			}
		});

		widget.getField('DSLOCALESTOQ').dataSourceFilter.forEach(function(fil) {
			if (fil.name === 'CDFILIAL') {
				fil.value = widget.currentRow.CDFILIMOVI;
			}
		});

		widget.getField('DSALMOXARIFE').reload();
		widget.getField('DSLOCALESTOQ').reload();
	};

	this.actViewStockVisibility = function(args) {
		let widget = {};
		if (args.owner.container)
			widget = args.owner.container.getWidget('itens');
		else
			widget = args.owner.field.widget;
		if (widget.currentRow.CDPRODUTO)
			widget.getAction('viewstock').isVisible = true;
		else
			widget.getAction('viewstock').isVisible = false;
	};

	var isset = (accessor) => {
		try {
			// Note we're seeing if the returned value of our function is not
			// undefined
			return typeof accessor() !== 'undefined' && accessor() !== null;
		}
		catch (e) {
			// And we're able to catch the Error it would normally throw for
			// referencing a property of undefined
			return false;
		}
	};

	this.detailSwitchTemplate = function(widget) {
		if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html") && (widget.template != "widget/master_detail/grid.html")) return false;

		if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
		if (!widget.templateToView) widget.templateToView = "widget/form-view.html";

		if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
		else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

		templateManager.updateTemplate();

		return true;
	};

	this.manipulaScope = function(args) {
		console.log('break aqui');
	};

	this.activateWidgetTabbed = function(args) {
		let widgetPrincipal = args.owner;
		let widgetTabbed = widgetPrincipal.container.getWidget('entradaTabbed');
		let widgetGridProdutos = widgetPrincipal.container.getWidget('itens');
		let widgetToEdit = widgetPrincipal.container.getWidget('wid_toEdit');

		let row = widgetPrincipal.currentRow;
		let fieldDescricao = widgetTabbed.getField('DSLANCESTQ');


		if (row.NRNOTAFISC || row.NRLANESTENTR)
			fieldDescricao.readOnly = true;
		else
			fieldDescricao.readOnly = false;

		widgetTabbed.redraw();
		templateManager.updateTemplate();

		widgetTabbed.currentRow = widgetPrincipal.currentRow;
		widgetPrincipal.container.getWidget('entrada').activate().then(() => {
			widgetPrincipal.container.getWidget('entradaTabbed').activate();
			widgetGridProdutos.reload();
		});
	};

	this.saveGridProduto = function(args) {
		var promFirstSave = ZHPromise.defer();
		let widgetGridPrincipal = templateManager.container.getWidget('entrada');
		let widgetItens = templateManager.container.getWidget('itens');
		let widgetTabbed = templateManager.container.getWidget('entradaTabbed');
		let widgetFakeToEdit = templateManager.container.getWidget('wid_toEdit');

		if (widgetTabbed.currentRow.changed || widgetTabbed.currentRow.__is_new || widgetItens.currentRow.__is_new || widgetItens.currentRow.changed) {
			widgetItens.currentRow.CDFILIAL = widgetTabbed.currentRow.CDFILIAL;
			widgetItens.currentRow.LANCAMENTO = widgetTabbed.currentRow;
			if (widgetTabbed.currentRow.CDFILIMOVI) {
				EstUtil.saveCrud(widgetItens, false, 'list').then(saved => {
					let nrlancto = saved.dataset.columnsToReturn[0].NRLANCESTQ;
					widgetTabbed.currentRow.__is_new = false;
					widgetTabbed.currentRow.changed = false;
					widgetItens.currentRow.__is_new = false;
					widgetItens.currentRow.changed = false;

					widgetTabbed.currentRow.NRLANCESTQ = nrlancto;
					widgetTabbed.currentRow.NRORG = getLocalVar("NRORG");

					widgetItens.dataSource.data.forEach(row => {
						row.__is_new = false;
						row.changed = false;
						row.NRLANCESTQ = nrlancto;
						row.NRORG = getLocalVar("NRORG");
					});
					widgetItens.reload();
					this.saveActionVisibility(null);
				});
			}
			else {
				ScreenService.showMessage('Filial de movimentação não informada, favor selecionar novamente no filtro.');
			}
		}
	};

	this.fakeEditMode = function(args) {
		let fakeWidget = templateManager.container.getWidget('wid_toEdit');
		if (fakeWidget.mode != 'edit')
			fakeWidget.edit();
	};

	this.saveActionVisibility = function(args) {
		let widgetGrid = templateManager.container.getWidget('itens');
		let widgetTabbed = templateManager.container.getWidget('entradaTabbed');
		let newProduct = widgetGrid.dataSource.data.some(row => row.__is_new);

		if ((widgetGrid.dataSource.data.some(row => row.__is_new || row.changed) || widgetTabbed.currentRow.changed) && widgetGrid.dataSource.data.length > 0)
			widgetGrid.getAction('salvarLanc').isVisible = true;
		else
			widgetGrid.getAction('salvarLanc').isVisible = false;
	};

	var validaLançamentoDeIntegracaoes = function(row, args) {
		let boolBloquear = false;
		if (row.NRNOTAFISC) {
			ScreenService.showMessage('Operação Bloqueada. Lançamento Proveniente de Nota Fiscal!');
			boolBloquear = true;
		}
		if (row.NRLANESTENTR) {
			ScreenService.showMessage('Operação Bloqueada. Lançamento Proveniente de Integração!');
			boolBloquear = true;
		}
		if (row.NRORDPROD && args.owner.name != 'adicionar') {
			ScreenService.showMessage('Operação Bloqueada. Lançamento Proveniente de Ordem de Produção (OP)!');
			boolBloquear = true;
		}
		return boolBloquear;
	};

	this.saveItemValidation = function(args) {
		let widgetGrid = templateManager.container.getWidget('itens');
		let widgetTabbed = templateManager.container.getWidget('entradaTabbed');

		widgetTabbed.currentRow.DTLANCESTQ = widgetTabbed.currentRow.DTLANCESTQ.substr(0, 10);
		if (widgetTabbed.currentRow.DTEMISSAO)
			widgetTabbed.currentRow.DTEMISSAO = widgetTabbed.currentRow.DTEMISSAO.substr(0, 10);

		if (widgetGrid.currentRow.QTTOTLANCTO == 0)
			ScreenService.showMessage('Quantidade deve ser informada.');
		else if (widgetGrid.currentRow.VRUNILANCTO == 0)
			ScreenService.showMessage('Valor unitário deve ser maior que zero.');
		else if (!widgetGrid.currentRow.NMPRODUTO)
			ScreenService.showMessage('Produto deve ser informado.');
		else {
			ScreenService.goBack();
			this.saveActionVisibility(args);
		}
	};

	this.newLanctoEntrada = function(widgetEntrada) {

		if (EstUtil.onValidateFilterRequirement(widgetEntrada, true)) return;

		let widgetTabbed = templateManager.container.getWidget('entradaTabbed');
		widgetEntrada.currentRow.CDFILIMOVI = widgetEntrada.appliedFilter.FILTER_ROW.CDFILIMOVI;
		widgetTabbed.currentRow.CDFILIMOVI = widgetEntrada.appliedFilter.FILTER_ROW.CDFILIMOVI;
	};

	this.clearProductsOldData = function(args) {
		let widgetProdutos = templateManager.container.getWidget('itens');
		widgetProdutos.dataSource.data = [];
	};

}
Configuration(function(ContextRegister) {
	ContextRegister.register('Est11000_EntradaEstoque_Controller', Est11000_EntradaEstoque_Controller);
});


// FILE: ./assets/modules/lancamentos/assets/js/controllers/Est58000_RetiradaEstoque_Controller.js
function Est58000_RetiradaEstoque_Controller(ScreenService, General, templateManager, Est58000_RetiradaEstoque_Service, ValidacoesEstoque_Service, Lib00206_InfComplProduto_Controller, ZHPromise, restEngine, metaDataFactory, ZhFilterPreferences, MessageService, EstUtil) {

    this.cdTipCustEst = 3;
    this.filialctrlestoq = 'S';

	/**
	 * Essa função tem como objetivo executar a função do botão de voltar da tela. Quando em modo de
	 * inserir retirada ele roda o data da widget para achar qual posição estava sendo editada/inserida.
	 * Quando encontrado, seta a currentRow e a posição no data com o backup feito na função do modo
	 * de edição, troca a editabilidade de alguns campos e passa a widget para o modo view.
	 * Quando o registro esta sendo inserido, ainda no modo inserir retirada, a posição do data é
	 * retirada.
	 * Quando não está no modo de inserir uma retirada, chama função do General que cancela a edição
	 * ou inserção.
	 *
	 * @param args | evento da action
	 */
	this.backEditItens = function(args) {
		var widget = args.owner.widget;
		if(widget.call === 'insertRetirada' && !widget.currentRow.novo) {
			widget.dataSource.data.forEach(function(row, rowIndex, rows) {
				if(widget.currentRow === row) {
					rows[rowIndex] = widget.backupRow;
					widget.currentRow = widget.backupRow;

					this.changeReadOnlyByName(widget, 'QTTOTLANCTO', true);
					this.changeReadOnlyByName(widget, 'QTDEVOLESTOQ', true);

					widget.view();
				}
			}.bind(this));
		} else {
			General.backOrCancel(args);
		}
	};

	/**
	 * Essa função tem como finalidade verificar o valor do campo de produto na inserção
	 * de um item na retirada. Quando tem valor chama a Lib que abre o popup de Informações
	 * Complementares de Produto e na sequência permite que outros campos sejam editavéis.
	 * Caso o campo não tenha valor, os campos, antes habilitados, são desabilitados para
	 * edição. Depois os valores dos campos que aparecem na tela são zerados.
	 *
	 * @param widget | widget de itens da retirada
	 */
	this.callInformacoesComplmentares = function(widget) {
		var promise = ZHPromise.defer();
		if(widget.currentRow.CDARVPROD) {
			widget.promise = promise;
			let widgetReti = widget.container.getWidget('retirada');
			let filterRow = isset(() => widgetReti.appliedFilter.FILTER_ROW)? widgetReti.appliedFilter.FILTER_ROW : false;
			if(isset(() => filterRow.CDALMOXARIFE)){
				if(filterRow.CDALMOXARIFE != ' ')
					widget.currentRow.CDALMOXARIFE = filterRow.CDALMOXARIFE;
			}
			if(isset(() => filterRow.CDLOCALESTOQ)){
				if(filterRow.CDLOCALESTOQ != ' ')
					widget.currentRow.CDLOCALESTOQ = filterRow.CDLOCALESTOQ;
			}
			Lib00206_InfComplProduto_Controller.infComplementares(widget, widget.currentRow);
			this.changeReadOnlyByName(widget, "QTTOTLANCTO", false);
			this.changeReadOnlyByName(widget, "QTDEVOLESTOQ", false);
			this.clearRowItens(widget, false);
		} else {
			this.changeReadOnlyByName(widget, "QTTOTLANCTO", true);
			this.changeReadOnlyByName(widget, "QTDEVOLESTOQ", true);
			this.clearRowItens(widget, true);
			promise.reject();
		}
		return promise;
	};

	/**
	 * Essa função tem como finalidade carregar a widget de itens da retirada, e setar na
	 * widget de retirada o campo de Tipo de Retirada, colocando na currentRow no código
	 * e nome do tipo de retirada.
	 *
	 * @param widget | widget de retirada
	 */
	this.carregaItems = function(widget) {
        var widgetItens = widget.widgets[0];
        if(!widget.currentRow.__is_new) {
        	widgetItens.reload().then(function(){
        	    widget.currentRow.CDTIPORETI = widgetItens.dataSource.data[0].CDTIPORETI;
        	    widget.currentRow.NMTIPORETI = widgetItens.dataSource.data[0].NMTIPORETI;
        	});
        } else {
        	widgetItens.dataSource.data = [];
        }
    };

	/**
	 * Essa função tem como finalidade percorrer o filtro montado pelo ZhFilter. É verificado
	 * se o usuario selecinou almoxarifado e/ou localização, se não tiver selecionado é
	 * inserido no filtro almoxarifado e/ou localizacao com o valor de T para aquele que não
	 * foi selecionado no filtro. É verificado também, se o filtro recebeu data, se não tiver
	 * recebido é passado no filtro DATA o valor T, se tiver recebido é passado NULL.
	 *
	 * @param widget | widget com name de "retirada"
	 */
	this.changeFilter = function(widget, preference) {
		var filterRow = preference.FILTER_ROW;
		var filterData;
		var dtinicial, dtfinal, data, cdalmoxar, cdlocal;
		let actAdicionar = widget.getAction('adicionar');
		actAdicionar.isVisible = this.filialctrlestoq == 'S'? true: false;
		
		if(filterRow.DTINICIAL && filterRow.DTFINAL){
			dtinicial = filterRow.DTINICIAL;
			dtfinal = filterRow.DTFINAL;
			data = null;
		} else {
			dtinicial = null;
			dtfinal = null;
			data = 'T';
		}
		if(filterRow.ALMOXARIFE && (filterRow.ALMOXARIFE != "")){
			cdalmoxar = filterRow.ALMOXARIFE;
		} else {
			cdalmoxar = 'T';
		}
		if(filterRow.CDLOCALESTOQ && (filterRow.CDLOCALESTOQ != "")){
			cdlocal = filterRow.CDLOCALESTOQ;
		} else {
			cdlocal = 'T';
		}
        var filter = [
        	{
                "name":"CDFILIAL",
                "operator":"=",
                "value": filterRow.CDFILIAL
            },
            {
                "name":"CDALMOXARIFE",
                "operator":"=",
                "value": cdalmoxar
            },
            {
                "name":"CDLOCALESTOQ",
                "operator":"=",
                "value": cdlocal
            },
            {
                "name":"DTINICIAL",
                "operator":"=",
                "value": dtinicial
            },
            {
                "name":"DTFINAL",
                "operator":"=",
                "value": dtfinal
            },
            {
                "name":"DATA",
                "operator":"=",
                "value": data
            }
        ];
		//filter.push(filterData[0]);
        widget.dataSourceFilter = filter;
		widget.reload();
	};
	
	this.intervaloChange = function(field){
		let widget = field.widget;
		widget.currentRow.DTINICIAL = widget.currentRow.INTERVALO.start;
		widget.currentRow.DTFINAL = widget.currentRow.INTERVALO.end;
	};

	/**
	 * A função tem a finalidade de trocar as propriedades label, hideicon e isvisible
	 * de uma action que será selecionada pelo seu nome.
	 *
	 * @param data | array de objetos (as actions)
	 * @param label | string com o novo valor da propriedade label
	 * @param hide | boolean com o valor para setar a propridade hideIcon
	 * @param name | string com o nome da action que vai ser alterada
	 * @param visibility | boolean para setar a visibilidade de action
	 */
	this.changeHideInconLabelVisible = function(data, label, hide, name, visibility) {
		data.forEach(function(element){
			if(element.name === name) {
				element.label = label;
				element.hideIcon = hide;
				element.isVisible = visibility;
			}
		});
	};

	/**
	 * Essa função tem como finalidade trocar o valor da propriedade readOnly
	 * de um field de uma widget.
	 *
	 * @param widget | widget onde sera pego o campo.
	 * @param name | nome do field que vai ser pegado da widget e alterado a propriedade
	 * @param read | true ou false para habilitar ou não a propriedade
	 */
	this.changeReadOnlyByName = function(widget, name, read) {
		widget.getField(name).readOnly = read;
	};

	/**
	 * Essa função tem como finalidade controlar as validações do campo de campo de
	 * localização chegando se a unidade do filtro controla localização de almoxarifado
	 * e se o campo de almoxarifado está preenchido. Se ambos forem verdadeiros, habilita
	 * o campo, se não desabilita o campo e limpa a currentRow dos dados de localização
	 *
	 * @param widget | widget do filtro
	 */
	this.changeReadOnlyLocal = function(widget) {
		var almox = widget.currentRow.CDALMOXARIFE;
		var flocal = widget.getField('DSLOCALESTOQ');
		if(almox && Est58000_RetiradaEstoque_Service.utilizaLocalizacao()) {
			flocal.readOnly = false;
			flocal.reload();
		} else {
			widget.currentRow.CDLOCALESTOQ  = null;
			widget.currentRow.DSLOCALESTOQ  = null;
			flocal.readOnly = true;
		}
	};

	/**
	 * Essa função é chamada quando é aberta alguma row na widget de itens de retirada. Ela
	 * tem a função de verificar se o registro selecionado está com as informações completas.
	 * Se não estiver abre a tela de Informações Complementares de Produto, se estiver completo,
	 * não faz nada.
	 * A tela de Informações Complementares de Produto sempre será chamada quando for inserida
	 * uma retirada com requisição, mesmo que os produtos da requisição venham com todas as
	 * informações complementares, pois é necessária a confirmação das mesmas por parte do usuário.
	 *
	 * @param args | evento da widget
	 */
	this.checkAlmoxStatus = function(args) {
		var widget = args.owner;
		if(widget.currentRow.COMPLETO !== 'S'){
			widget.currentRow.CDFILIMOVI = EstUtil.getFieldValue('CDFILIAL', 'retirada', true);
			this.editItens(widget);

			this.callInformacoesComplmentares(widget);
		}
	};

	/**
	 * Essa função tem a finalidade de voltar as actions para o estado normal depois
	 * de cancelar a inserção de uma retirada sem requisição.
	 *
	 * @param args | evento da action
	 */
	this.checkBackAction = function(args) {
		var widget = args.owner.widget;
		if(widget.call === 'insertRetirada') {
			General.backOrCancel(args);
			this.returnStatus(widget.getParent());
			ScreenService.closeSwipe(0);
			widget.getParent().list(true);
			widget.getParent().dataSource.data.pop();
			widget.getParent().currentRow = {};
			widget.getParent().enabled = true;
			widget.getParent().activate();
		} else if (widget.call === 'padrão') {
			ScreenService.goBack();
		}
	};

	/**
	 * Essa função tem a finalidade de verificar se existe algum produto que ainda não teve
	 * suas informações complementares colocadas e está sendo salvo. Se isso acontecer, dá uma
	 * mensagem ao usuário e retorna false para a função que lhe chamou.
	 *
	 *	@param data | data da widget de itens
	 */
	this.checkProdutosCompletos = function(data) {
		var validação = true;
		data.forEach(function(produto){
			if(produto.COMPLETO !== 'S') {
				validação = false;
				
				// ScreenService.showMessage('Não é possível salvar pois faltam informações para o produto ' + produto.NMPRODUTO + '.');
			
				/* MSG NOVA */
				var variaveis = {
                    NMPRODUTO: produto.NMPRODUTO
                };
				
				MessageService.showMessageByCode("PRODUTOIRREGULAR", variaveis);
				/* FIM MSG NOVA */
				
			}
		});
		return validação;
	};

	this.checkRequisicaoComplementarCreation = function(widget) {
		var promise = ZHPromise.defer();
		var item = widget.currentRow;

		if(this.checkShouldCreateRequisicaoComplementar(item)) {
			var msg = "Quantidade requisitada não disponível em estoque para o produto " + item.NMPRODUTO + " : " + item.QTESTOQDIA + " " + item.SGUNIDADE + ". Deseja gerar uma requisição complementar?";

			ScreenService.confirmMessage(msg,
			'question', function(){ //sim
				item.createReqComp = true;
				item.QTLANCREQCOMP = item.QTREQUESTO - item.QTESTOQDIA;

				promise.resolve();
			}.bind(this), function(){ //não
				item.createReqComp = false;

				promise.resolve();
			}.bind(this));
		} else promise.resolve();

		return promise;
	};

	this.checkShouldCreateRequisicaoComplementar = function(item) {
		return item.NRREQUESTO && item.IDREQCOMPEST == 'S' &&
		       item.QTESTOQDIA == item.QTLANCTOEST &&
		       item.QTESTOQDIA  < item.QTREQUESTO;
	};

	/**
	 * Essa função tem como finalidade controlar em qual estado o usuário esta usando a tela
	 * e chamar a função correspondente ao estado que ela está.
	 *
	 * @param widget | widget de retirada
	 */
	this.checkSaveMod = function(widget) {
		if(widget.isValid()) {
			if(widget.currentRow.__is_new && widget.currentRow.NRREQUESTO) {
				widget.enabled = false;
				this.getItensOfRequisicao(widget.widgets[0], widget.currentRow.CDFILIAL, widget.currentRow.NRREQUESTO);
				this.focusAddProduto(widget.widgets[0]);
			} else if(widget.currentRow.__is_new) {
				widget.enabled = false;
				this.focusAddProduto(widget.widgets[0]);
			} else if(!widget.currentRow.__is_new) {
				this.saveEditRetirada(widget);
			}
		}
	};

	/**
	 * Essa função tem como finalidade, primeiramente, verificar se o produto que está sendo
	 * inserido ja foi inserido anteriormente na retirada. Depois verifica o estado da widget para
	 * executar a função certa para o estado correto da widget. Quando essa função é chamada
	 * no momento de adicionar um item em uma retirada, que está sendo incluida, executa as
	 * funções próprias desse estado. Se estiver no estado de adicionar um item a uma
	 * retirada que já existe, executa as funções deste modo da widget.
	 *
	 * @param widget | widget de itens
	 */
	this.checkSaveProduto = function(widget, forceSave) {
		if(!forceSave) forceSave = false;
		var row = widget.currentRow;

		if( widget.mode === 'edit' ||
			!Est58000_RetiradaEstoque_Service.checaItensDaRetirada(this.copyDataWidgetItens(widget.dataSource.data), row.CDARVPROD,row.CDALMOXARIFE,row.CDLOCALESTOQ,row.NRLOTEESTQ,row.NRSUBLOTE)) {
			if(widget.call === 'insertRetirada') {
				if(widget.isValid() && (forceSave || this.onChangeQt(widget))) {
					ScreenService.showLoader();
					this.checkRequisicaoComplementarCreation(widget).then(function() {
						this.changeReadOnlyByName(widget, 'QTTOTLANCTO', true);
						this.changeReadOnlyByName(widget, 'QTDEVOLESTOQ', true);

						widget.currentRow.novo = false;
						
						ScreenService.hideLoader();
						ScreenService.closeSwipe();
					}.bind(this));
				}
			} else if(widget.call === 'padrão'){
				widget.currentRow.novo = false;
				this.save(widget.getParent());
			}
			
			let widgetItens = templateManager.container.getWidget('itens');
			let impAction = widgetItens.getAction('importTxt');
        	impAction.isVisible = widgetItens.dataSource.data.length == 0;
		} else {
			ScreenService.showMessage('Produto informado já está sendo incluido para essa retirada.');
		}
	};

	/**
	 * Essa função tem como finalidade limpar alguns dados da currentRow quando o produto mudar
	 * (inserindo um produto em uma retirada que está sendo inserida). Qunado troca-se o produto,
	 * só os capos de valor são alterados, quando o valor do campo é null, é passado true para
	 * clearAll e valores relacionados ao produto são limpados também.
	 *
	 * @param widget | widget de itens
	 * @param clearAll | parametro para lipar ou não todos os capos da currentRow
	 */
	this.clearRowItens = function(widget, clearAll) {
		if (widget.currentRow.COMPLETO === 'S') {
			widget.currentRow.QTREQUESTO = "";
			widget.currentRow.QTTOTLANCTO = "";
			widget.currentRow.QTDEVOLESTOQ = "";
			widget.currentRow.QTLANCTOEST = "";
			widget.currentRow.VRTOTLANCTO = "";
			if(clearAll) {
				widget.currentRow.NMPRODUTO = "";
				widget.currentRow.CDARVPROD = "";
				widget.currentRow.SGUNIDADE = "";
				widget.currentRow.VRUNILANCTO = "";
			}
		}
	};

	/**
	 * Essa função tem como finalidede clonar todos os dados do dataSource da widget, de
	 * itens principalmente. Não clonando escopos das actions que podem estar na row da
	 * widget.
	 *
	 * @param data | array de objetos; dataSource.data da widget
	 * @return arrray | array de objetos clonados do data da widget
	 */
	this.copyDataWidgetItens = function(data) {
		var copy = [];
		data.forEach(function(item){
			copy.push(Util.clone(item, ['__rightActions', '__leftActions']));
		});
		return copy;
	};

	/**
	 * Essa função tem a finalidade verificar em qual modo está sendo chamado a função de excluir um
	 * item da retirada. Conforme o modo, chama a função respectiva.
	 *
	 * @param widget | widget de itens
	 */
	this.deleteItem = function(widget) {
		if(widget.getParent().currentRow.EXISTECONSUMO != 'S'){
			if(widget.call === 'insertRetirada') {
				//Exclui um item do grupod de itensque sera incluso junto com uma nova retirada
				this.removeItemDataSource(widget);
			} else if (widget.call === 'padrão') {
				this.removeItemRetirada(widget);
			}
		} else {
			ScreenService.showMessage("Operação bloqueada! Não é possível excluir pois existe um consumo.");
		}
		
	};

	/**
	 * Inicialmente a função verifica se aquela retirada pode ser editada (inserir/excliuir/editar um
	 * item é editar uma retirada), se não puder é dada uma mesagem para o usuario. Se puder ser editado
	 * a currentRow é clonada (não é clonada __rightActions e __leftActions pois são escopos), a widget é
	 * colocado em modo de edição e a editabilidade de alguns campos é alterada
	 *
	 * @param widget | widget de itens
	 */
    this.editItens = function(widget) {
    	
    	if(widget.getParent().currentRow.EXISTECONSUMO != 'S'){
	    	if(!widget.getParent().currentRow.NRORDPROD) {
				if(!widget.getParent().currentRow.NRLANCESTRET) {
					widget.backupRow = Util.clone(widget.currentRow, ['__rightActions', '__leftActions']);
					widget.edit();
					this.changeReadOnlyByName(widget, 'QTTOTLANCTO', false);
					this.changeReadOnlyByName(widget, 'QTDEVOLESTOQ', false);
				} else {
					ScreenService.showMessage("Operação bloqueada! Retirada oriunda de um ajuste de inventário.");
				}
			} else {
				ScreenService.showMessage("Operação bloqueada! Retirada gerada pela Ordem de Produção.");
			}
    	} else {
			ScreenService.showMessage("Operação bloqueada! Não é possível editar pois existe um consumo.");
		}
    };

	/**
	 * Essa função tem como finalidade trocar o label do botão de salvar (para garantir que ficará
	 * com a label certa), colocar a widget em modo de edição e desabilitar a widget de produtos.
	 * Isso acontece ao acionar o botão de editar a retirada.
	 * Antes disso é verificado se a retirada pode ser editada, verificando os capos NRORDPROD e
	 * NRLANCESTRET, onde se um deles estiver preencido, é dado sua respctiva mensagem, impedindo
	 * o usuário de editar a retirada.
	 *
	 * @param widget | widget de retirada
	 */
	this.editRetirada = function(widget) {
		
		if(widget.currentRow.EXISTECONSUMO == "N"){
			if(!widget.currentRow.NRORDPROD) {
				if(!widget.currentRow.NRLANCESTRET) {
					General.edit(widget);
					this.enabledWidget(false);
				} else{
					ScreenService.showMessage("Operação bloqueada! Retirada oriunda de um ajuste de inventário.");
				}
			} else{
				ScreenService.showMessage("Operação bloqueada! Retirada gerada pela Ordem de Produção.");
			}
		} else{
			ScreenService.showMessage("Operação bloqueada! Não é possível editar pois existe um consumo.");
		}
	};

	/**
	 * Essa função tem como finalidade habilitar e desabilitar a widget "itens", onde
	 * são exibidos os produtos da retirada. Ela é a widget filha da widget "retirada".
	 *
	 * @param enabled | booleano para habilitar ou desabilitar a widget de itens
	 */
	this.enabledWidget = function(enabled) {
		var widget = templateManager.container.getWidget('itens');
		widget.enabled = enabled;
	};

	/**
	 * Essa função tem a finalidade de ativar a widget de produto, deixá-la ativa,
	 * setar uma propriedade de controle e mudar a configuração das actions para
	 * funcionarem no modo de inserir retirada sem requisição.
	 *
	 * @param widget | widget de itens
	 */
  	this.focusAddProduto = function(widget) {
		this.enabledWidget(true);
		widget.call = 'insertRetirada';
		widget.activate();
		widget.floatingControl.refreshAction = false;
		this.changeHideInconLabelVisible(widget.actions, 'Cancelar', true, 'VoltarList', true);
		this.changeHideInconLabelVisible(widget.actions, 'Adicionar', true, 'salvar', true);
		this.changeHideInconLabelVisible(widget.actions, 'Salvar', true, 'salvarNaRetirada', true);
	};
	
	/*	this.forEachValorUnitario = async function(dataSource, widgetItens) {
			    for (var item of dataSource) {
			        // wait for this to resolve and after that move to next object 
			        widgetItens.currentRow = item;
					widgetItens.currentRow.QTTOTLANCTO = item.QTREQUESTO;
					this.setCdfilimovi(widgetItens);
			        await this.setValorUnitario(widgetItens);
			    }
	};*/
	
	/**
	 * Essa função tem a finalidade chamar uma função que retornara os itens da requisição
	 * que foi passada na inserção da retirada. Quando retornada o resultado da consulta,
	 * o dataSet é colocado no data da widget de itens.
	 *
	 * @param widgetsItens | widget de itens
	 * @param cdfilial | código da filial de lançamento
	 * @param nrrequesto | código da requisição
	 */
	this.getItensOfRequisicao = function(widgetItens, cdfilial, nrrequesto, itemData, firstTime = true) {
		if(!firstTime){
			var item = itemData.shift();
			if(item !== undefined){
				widgetItens.dataSource.data.shift();
				widgetItens.currentRow = item;
				widgetItens.currentRow.QTTOTLANCTO = item.QTREQUESTO;
				widgetItens.currentRow.IDCNTRESTOQ = 'S';
				widgetItens.currentRow.NRSEQUITEM = Est58000_RetiradaEstoque_Service.geraNrsequitem(widgetItens.dataSource.data);
				this.setCdfilimovi(widgetItens);
				this.callInformacoesComplmentares(widgetItens).then(() =>{
					this.setValorUnitario(widgetItens).then(() => {
						widgetItens.dataSource.data.push(widgetItens.currentRow);
						this.getItensOfRequisicao(widgetItens, cdfilial, nrrequesto, itemData, false);
					});
				});
			}else{
				
			}
		}else{
			Est58000_RetiradaEstoque_Service.getItensOfRequisicao(cdfilial, nrrequesto).then((itens) => {
				widgetItens.dataSource.data = itens;
				var data = Util.clone(widgetItens.dataSource.data);
				this.getItensOfRequisicao(widgetItens, cdfilial, nrrequesto, data, false);
			});
		}
		
	};

	/**
	 * Essa função tem como finalidade consultar as parametrizações da filial selecionada
	 * no filtro, caso ainda não tenha sido selecionada nenhuma filial no filtro, é usado
	 * o valor FILIAL_LOGADA, que pegara o código da filial logada no backend. Se não
	 * retornar nenhum erro, chama uma função que vai setar valores default para outros
	 * campos do filtro.
	 *
	 * @param widget | widget
	 */
	this.getParamFilialOnce = function(widget) {
		this.setFieldsGerais(widget);
        var filterRow = {};
        this.getFilialLogada().then(function(success) {
        	let returnedDataset = success.dataset.paramfilial_paramvalidacoes[0];
            filterRow.CDFILIAL = returnedDataset.CDFILIAL;
            filterRow.NMFILIAL = returnedDataset.NMFILIAL;
            this.filialctrlestoq = returnedDataset.IDCTRLESTQ;
            
            if(returnedDataset.IDCTRLESTQ == 'N')
            	ScreenService.showMessage('Filial logada não controla estoque.');
            
	        ZhFilterPreferences.openFilterWidget(widget, filterRow).then(function () {
	        	var filial = widget.currentRow.CDFILIAL || 'FILIAL_LOGADA';
				Est58000_RetiradaEstoque_Service.getParamFilial(filial, true).then(function(succes){
					let widgetFiltro = templateManager.container.getActiveWidget();
					if(Array.isArray(succes)) {
						this.setCampos(widgetFiltro, succes, filial);
					} else {
						ScreenService.showMessage(succes);
					}
				}.bind(this), function(error){
					ScreenService.showMessage(error.data.error);
				});
	        }.bind(this));
        }.bind(this));
	};
	
	this.getFilialLogada = function(filial = 'FILIAL_LOGADA') {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": filial
                }
		    ];
        return General.callRestEngine('est#/crud/paramfilial_paramvalidacoes', 'FilterData', filter);
    };
	
	this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };

	this.getParamFilial = function(widget) {
        var filial = widget.currentRow.CDFILIAL;
        if(filial){
			Est58000_RetiradaEstoque_Service.getParamFilial(filial, true).then(function(succes){
				if(Array.isArray(succes)) {
						this.setCampos(widget, succes, filial);
					} else {
						ScreenService.showMessage(succes);
					}
			}.bind(this), function(error){
				ScreenService.showMessage(error.data.error);
			});
        }
	  };

	/**
     * Essa função tem como finalidade dar um newRow na widget de itens e com base nos
     * dados que a mesma já tem, calcular qual será o próximo sequencial do item para
     * aquela retirada de estoque.
     * Antes disso é verificado se a retirada pode ser editada, verificando os capos NRORDPROD e
	 * NRLANCESTRET, onde se um deles estiver preencido, é dado sua respctiva mensagem, impedindo
	 * o usuário de editar a retirada.
     *
     * @param widget | widget de itens da retirada
     */
    this.newItem = function(widget) {
    	
    	if(widget.getParent().currentRow.EXISTECONSUMO != 'S'){
    		if(!widget.getParent().currentRow.NRORDPROD) {
				if(!widget.getParent().currentRow.NRLANCESTRET) {
					widget.newRow();
					widget.currentRow.novo = true;
	        		widget.currentRow.NRSEQUITEM = Est58000_RetiradaEstoque_Service.geraNrsequitem(widget.dataSource.data);
	        		widget.currentRow.DTLANCMOVI = widget.getParent().currentRow.DTLANCESTQ;
				} else {
					ScreenService.showMessage("Operação bloqueada! Retirada oriunda de um ajuste de inventário.");
				}
			} else {
				ScreenService.showMessage("Operação bloqueada! Retirada gerada pela Ordem de Produção.");
			}
    	} else {
    		ScreenService.showMessage("Operação bloqueada! Não é possivel adicionar item pois existe consumo.");
    	}
	    	
    };

	/**
     * Essa função tem a finalidade de validar a relação entre os valores dos campos de valor
     * na widget de itens.
     *
     * @param widget | widget de itens
     */
    this.onChangeQt = function(widget) {
    	var row = widget.currentRow;
    	var calcTotValues = true, isValid = false;
    	var permitEstoqNeg = widget.getParent().currentRow.idpermestneg;
		if(EstUtil.parseStrToFloat(row.QTDEVOLESTOQ) < 0) {
			row.QTDEVOLESTOQ = null;
			ScreenService.showMessage('A quantidade devolvida não pode ser menor que 0.');
		} else if(EstUtil.parseStrToFloat(row.QTTOTLANCTO) <= 0) {
			row.QTTOTLANCTO = null;
			ScreenService.showMessage('A quantidade retirada não pode ser menor ou igual a 0.');
		} else if(EstUtil.parseStrToFloat(row.QTTOTLANCTO) < EstUtil.parseStrToFloat(row.QTDEVOLESTOQ)) {
			row.QTDEVOLESTOQ = null;
			ScreenService.showMessage('A quantidade retirada não pode ser menor que a quantidade devolvida.');
		} else if((EstUtil.parseStrToFloat(row.QTTOTLANCTO) > EstUtil.parseStrToFloat(row.QTESTOQDIA)) && permitEstoqNeg == 'N'){
			ScreenService.showMessage('Quantidade informada não disponível no estoque.');
			row.QTTOTLANCTO = null;
		} else if (EstUtil.parseStrToFloat(row.QTTOTLANCTO) === EstUtil.parseStrToFloat(row.QTDEVOLESTOQ)) {
			calcTotValues = false;
			ScreenService.confirmMessage('Quantidade movimentada igual a 0, portanto o item não realizará atualização de estoque. Confirmar operação?',
			'question', function(){ //sim
				Est58000_RetiradaEstoque_Service.setQtLanctoEst(row);
    			Est58000_RetiradaEstoque_Service.setVrTotLancto(row);

				this.checkSaveProduto(widget, true);
			}.bind(this), function(){ //não
				row.QTDEVOLESTOQ = null;
				Est58000_RetiradaEstoque_Service.setQtLanctoEst(row);
    			Est58000_RetiradaEstoque_Service.setVrTotLancto(row);
			});
		} else {
			isValid = true;
		}

		if(calcTotValues) {
			Est58000_RetiradaEstoque_Service.setQtLanctoEst(row);
    		Est58000_RetiradaEstoque_Service.setVrTotLancto(row);
		}

		return isValid;
    };

    this.openPosicaoDeEstoque = function(widget) {
    	var popup = this.getPosicaoDeEstoquePopup(widget);

		ScreenService.openPopup(popup);

		popup.currentRow = widget.currentRow;
		popup.parentWidget = widget;

		this.getDataForPosicaoDeEstoquePopup(popup);
    };

    this.getDataForPosicaoDeEstoquePopup = function(popup) {
    	Est58000_RetiradaEstoque_Service.getPosicaoDeEstoquePorProduto(popup.currentRow).then(function(posestdia) {
    		if(posestdia) {
	    		popup.currentRow = posestdia;
	    		let qtToSet = posestdia.QTESTOQDIA;
	    		if(qtToSet == 'undefined' || qtToSet == null)
	    			qtToSet = 0;
	    		popup.parentWidget.currentRow.QTESTOQDIA = qtToSet;
    		}
    		else{
    			popup.currentRow.QTESTOQDIA = 0;
    		}
    	}).catch(function(error) {
    		ScreenService.showMessage(error.data.error);
    	});
    };

    this.getPosicaoDeEstoquePopup = function(parentWidget) {
        var widget = parentWidget.posestPopup;

        if(!widget) {
            widget = this.getPosicaoDeEstoquePopUpObject();
            widget.container = parentWidget.container;
            widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

            widget = metaDataFactory.widgetFactory(widget, parentWidget);

            parentWidget.posestPopup = widget;
            widget.parent = parentWidget;
        } else
            widget.label = widget.defaultLabel + parentWidget.currentRow.NMPRODUTO;

        return widget;
    };

    this.getPosicaoDeEstoquePopUpObject = function() {
    	return {
            "name": "posestPopup",
            "defaultLabel": "Posição de Estoque do Produto ",
            "template": "widget/form.html",
            "isVisible": true,
            "showAsTab": false,
            "actions": [
                {
                    "name": "back",
                    "label": "Voltar",
                    "showAsAction": "back",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ScreenService.closePopup()"
                        }
                    ]
                },
                {
                    "name": "refresh",
                    "label": "Atualizar",
                    "showAsAction": "never",
                    "isVisible": true,
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "Est58000_RetiradaEstoque_Controller.getDataForPosicaoDeEstoquePopup(args.owner.widget)"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "CDALMOXARIFE",
                    "label": "Almoxarifado",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "CDLOCALESTOQ",
                    "label": "Localização",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRLOTEESTQ",
                    "label": "Lote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "NRSUBLOTE",
                    "label": "Sublote",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "QTESTOQDIA",
                    "label": "Quantidade",
                    "isVisible": true,
                    "readOnly": true,
                    "template": "field/text-edit.html",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                },
                {
                    "name": "DTVALILOTE",
                    "label": "Data de Validade",
                    "isVisible": true,
                    "sequenceInForm": 80,
                    "readOnly": true,
                    "defaultValue": null,
                    "template": "field/calendar.html",
                    "dateFormat": "DD/MM/YYYY",
                    "class": 6,
                    "showOnForm": true,
                    "events": []
                }
            ]
        };
    };

	/**
	 * Essa função tem a finalidade de ser executada quando o usuário quer remover um item de uma
	 * retirada que está sendo inserido, dessa forma é procurado em qual posição do data da widget
	 * está o item que o usuário quer retirar, e então é tirado do data da widget.
	 *
	 *	@param widget | widget de itens
	 */
	this.removeItemDataSource = function(widget){
		var itens = widget.dataSource.data;
		for (var i = 0; i < itens.length; i++) {
			if(itens[i].CDARVPROD === widget.currentRow.CDARVPROD &&
			 itens[i].CDALMOXARIFE === widget.currentRow.CDALMOXARIFE &&
			 itens[i].CDLOCALESTOQ === widget.currentRow.CDLOCALESTOQ &&
			 itens[i].NRLOTEESTQ === widget.currentRow.NRLOTEESTQ &&
			 itens[i].NRSUBLOTE === widget.currentRow.NRSUBLOTE
			) {
				itens.splice(i, 1);
				i = itens.length;
				ScreenService.closeSwipe();
			}
		}
	};

	/**
	 * Inicialmente a função verifica se a retirada pode ser editada (inserir/excluir/editar um item
	 * é editar uma retirada), se tiver problema é dado uma mensagem na tela. Se tudo estiver certo,
	 * a função de excluir do General é chamada para excluir o item. Essa função é chamada quando
	 * se está excluindo um item de uma retirada que já existe.
	 *
	 * @param widget | widget de itens
	 */
	this.removeItemRetirada = function(widget) {
		if(!widget.getParent().currentRow.NRORDPROD) {
			if(!widget.getParent().currentRow.NRLANCESTRET) {
				if (widget.dataSource.data.length > 1) {
					General.deleteCrud(widget);
				} else {
					ScreenService.showMessage("Operação bloqueada! Não é possível existir uma retirada sem item.");
				}
			} else {
				ScreenService.showMessage("Operação bloqueada! Retirada oriunda de um ajuste de inventário.");
			}
		} else {
			ScreenService.showMessage("Operação bloqueada! Retirada gerada pela Ordem de Produção.");
		}
	};

	/**
	 *
	 *
	 * @param widget | widget de retirada
	 */
	this.removeRetirada = function(widget) {
		if(!widget.currentRow.NRORDPROD) {
			if(widget.currentRow.__is_new){
				let filteredDataset = widget.dataSource.data.filter(row => row.__is_new == false);
				widget.dataSource.data = filteredDataset;
				ScreenService.closeSwipe();
			}else{
				if(!widget.currentRow.NRLANCESTRET) {
					General.deleteCrud(widget);
				} else {
					ScreenService.showMessage("Operação bloqueada! Retirada oriunda de um ajuste de inventário.");
				}
			}
		} else {
			ScreenService.showMessage("Operação bloqueada! Retirada gerada pela Ordem de Produção.");
		}
	};

	/**
	 * Essa função tem como finalidade retornar todas as propriedades e labels de todas as widgets,
	 * para o estado normal da tela. Ela é chamado quando se termina ou cancela o processo de inserção
	 * de uma retirada.
	 *
	 * @param widget | widget de retirada
	 */
	this.returnStatus = function(widget) {
		//Mudanças na widget de Retirada para voltar ao estado de visualização
		widget.enabled = true;
		this.changeHideInconLabelVisible(widget.actions, 'Salvar', true, 'salvar', true);
		//Mudanças na widget de Itens para voltar ao estado de visualização
		var widgetItens = widget.widgets[0];
		widgetItens.enabled = true;
		this.changeHideInconLabelVisible(widgetItens.actions, 'Voltar', false, 'VoltarList', true);
		this.changeHideInconLabelVisible(widgetItens.actions, 'Salvar', true, 'salvarNaRetirada', false);
		widgetItens.floatingControl.refreshAction = true;
		widgetItens.call = 'padrão';
	};

	/**
	 * Essa função tem como finalidade salvar um produto novo em uma retirada já existente
	 * ou salvar a edição de um produto existente em uma retirada já existente. A função seta
	 * alguns campos da currentRow de itens com dados de referência da currentRow de retirada
	 * quando estiver inserindo um novo produto. Na sequência valida se essa operação não vai
	 * gerar estoque negativo e por fim chama a função de salvar do General.
	 *
	 */
	this.save = function(widgetRetirada) {
        var widgetItens = widgetRetirada.container.getWidget('itens');
        var itens = widgetItens.currentRow;
        var lancamento = widgetRetirada.currentRow;
        if(widgetItens.currentRow.__is_new) {
        	itens.NRREQUESTO = lancamento.NRREQUESTO;
			itens.NRLANCESTQ = lancamento.NRLANCESTQ;
        	itens.CDTIPORETI = lancamento.CDTIPORETI;
			itens.CDCENTCUST = lancamento.CDCENTCUST;
			itens.DTLANCMOVI = lancamento.DTLANCESTQ;
        	itens.CDPRODMOVI = itens.CDPRODUTO;
        }
		General.saveCrud(widgetItens).then(() => {
			widgetRetirada.reload();
			this.carregaItems(widgetRetirada);
		});
	};

	/**
	 * Essa função tem a finalidade de executar os processos da edição de uma retirada. Chamando
	 * a função de salvar da widget, e no sucesso deixando a widget no modo view e dando reload
	 * na widget filha.
	 *
	 * @param widget | widget de retirada
	 */
	this.saveEditRetirada = function(widget) {
		//if(widget.isValid()) {
			widget.saveRow().then(function(){
				widget.view();
				widget.widgets[0].enabled = true;
				widget.widgets[0].reload();
			}, function(error){
				ScreenService.showMessage(error.data.error);
			});
		//}
	};

	/**
	 * Essa função tem a finalidade de salvar a retirada e executar as mudanças na widget pós
	 * salvamento. Para salvar é necessário que existam itens sendo inseridos junto à retirada
	 * e que os itens que estão sendo inseridos estejam com as informações completas (passado
	 * pela tela Informações Complementares de Produto).
	 *
	 * @param widget | widget de itens
	 */
	this.saveNewRetirada = function(widget) {
		var widgetRetirada = widget.getParent();
		var itens = widget.dataSource.data;
		if(itens.length <= 0) {
			ScreenService.showMessage('É preciso adicionar itens à retirada para salva-la.');
		} else if (this.checkProdutosCompletos(itens)) {
			widgetRetirada.currentRow.ITENS = itens;
			ScreenService.showLoader();
			Est58000_RetiradaEstoque_Service.saveRetirada(widgetRetirada.currentRow).then(function(sucess){
				var lancamento = sucess.dataset.lancamento[0];
				var parentCurrentRowIndex = widget.getParent().dataSource.data.indexOf(widget.getParent().currentRow);

				this.returnStatus(widget.getParent());
				
				lancamento.NMFILIAL = widget.getParent().appliedFilter.FILTER_ROW.NMFILIAL;
				
				widget.getParent().currentRow = lancamento;
				widget.getParent().currentRow.__is_new = false;
				widget.getParent().dataSource.data[parentCurrentRowIndex] = widget.getParent().currentRow;
				widget.getParent().view();

				widget.reload();
				widget.activate();
				ScreenService.hideLoader();
			}.bind(this), function(error){
				if (error.dataset.irregularProducts) {
                    let irregularProducts = error.dataset.irregularProducts;
                    let dados = widget.dataSource.data;
                    dados.forEach((d, index) => {
                        if (irregularProducts[d.CDPRODESTO]) {
                            d.IRREGULAR = true;
                        }
                    });
                    widget.redraw();
                }
				ScreenService.showMessage(error.data.error);
				ScreenService.hideLoader();
			});
		}
	};

	/**
	 * Essa função tem como finalidade setar datas defaut para o filtro da tela conforme a
	 * data de processamento da filial selecionada, trocar a editabilidade do campo de
	 * almoxarifado conforme a parametrização da filial selecionada, e setar o nome e
	 * código da filial logada caso o evento estiver sendo chamado no início da tela.
	 * Depois chama a função para validar a regra dos campos de data.
	 *
	 * @param widget | widget do filtro
	 * @param array | recebe um array com código e nome da filial selecionada no filtro
	 * @param filial | codigo da filial para saber se seta ou não o campo de filial no filtro
	 */
	this.setCampos = function(widget, array, filial) {
		//Setando datas do campo Data
		var dtprocessa = Est58000_RetiradaEstoque_Service.getDtprocessa();
		var dtinicial = moment(dtprocessa, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY');
		widget.currentRow.DTINICIAL = dtinicial;
		widget.currentRow.DTFINAL = dtprocessa;
		if (!widget.currentRow.INTERVALO) {
            widget.currentRow.INTERVALO = {};
        }
        widget.currentRow.INTERVALO.start = dtinicial;
        widget.currentRow.INTERVALO.end = dtprocessa;
		//Habilitando Almoxarifado, reseta a currentRow e desabilita Localização
		if(Est58000_RetiradaEstoque_Service.utilizaAlmoraifado()) {
			widget.getField('DSLOCALESTOQ').readOnly = false;
			widget.currentRow.CDLOCALESTOQ = null;
			widget.currentRow.DSLOCALESTOQ = null;
			widget.getField('DSALMOXARIFE').readOnly = false;
			widget.currentRow.CDALMOXARIFE = null;
			widget.currentRow.DSALMOXARIFE = null;
			widget.getField('DSALMOXARIFE').reload();
		} else {
			widget.getField('DSLOCALESTOQ').readOnly = true;
			widget.currentRow.CDLOCALESTOQ = null;
			widget.currentRow.DSLOCALESTOQ = null;
			widget.getField('DSALMOXARIFE').readOnly = true;
			widget.currentRow.CDALMOXARIFE = null;
			widget.currentRow.DSALMOXARIFE = null;
		}
		//Setando campo de filial se for no início da tela
		if(filial === 'FILIAL_LOGADA') {
			widget.currentRow.CDFILIAL = array[0];
			widget.currentRow.NMFILIAL = array[1];
		}
		this.setMinMaxAndRequired(widget);
	};

	/**
	 * Essa função tem como finalidade setar o código da filial que será inserida na retirada
	 * na row do novo item a ser inserido naquela retirada.
	 *
	 * @param widget | widget itens da retirada
	 */
	this.setCdfilimovi = function(widget) {
		var newRow  = widget.currentRow;
		//newRow.CDFILIMOVI = widget.getParent().currentRow.CDFILIAL;
		newRow.CDFILIMOVI = EstUtil.getFieldValue('CDFILIAL', 'retirada', true);
	};

	/**
     * Essa função tem como finalidade setar a filial e a data de lançamento quando é
     * acionado o botão de adicionar uma retirada. É sugerida a filial que foi passada
     * no filtro, e a data de lançamento é a data de processamento da filial. Depois
     * chama uma função para desabilitar a aba de produtos, e outra para trocar a label
     * do botão inferior direito da tela.
     *
     * @param widget | widget de retirada
     */
    this.setCurrentRowNew = function(widget) {
    	widget.currentRow.CDFILIAL = widget.appliedFilter.FILTER_ROW.CDFILIAL;
    	widget.currentRow.CDFILILANC = widget.appliedFilter.FILTER_ROW.CDFILIAL;
    	widget.currentRow.NMFILIAL = widget.appliedFilter.FILTER_ROW.NMFILIAL;
    	widget.currentRow.DTLANCESTQ = Est58000_RetiradaEstoque_Service.getDtprocessa();
    	this.enabledWidget(false);
    	this.changeHideInconLabelVisible(widget.actions, "Adicionar Produtos", true, "salvar", true);
    	
    	var widgetItens = templateManager.container.getWidget('itens');
    	let impAction = widgetItens.getAction('importTxt');
    	impAction.isVisible = widgetItens.dataSource.data.length == 0;
    };

	/**
	 * Essa função tem a finalidade de pegar os dois campos de data do filtro, e caso o
	 * valor deles seja null, em ambos, tira sua obrigatoriedade. Caso um deles tenha
	 * valor, é setado o max e min date de cada um com o valor do outro campo e ambos
	 * são colocados como obrigatórios. Dessa forma se um deles estiver com valor, ambos
	 * serão obrigatórios, mas se o usuário não quiser filtrar por data, e deixar ambos
	 * vazios, eles deixam de ser obrigatórios.
	 *
	 * @param widget | widget do filtro
	 */
	this.setMinMaxAndRequired = function(widget) {
		var fdateIni = widget.getField('DTINICIAL');
		var fdateFin = widget.getField('DTFINAL');

		if(!fdateIni.value() && !fdateFin.value()) {
			delete fdateIni.filterProperties.validations.required;
			delete fdateFin.filterProperties.validations.required;
		} else {
			fdateIni.filterProperties = {"validations": {}};
			fdateIni.filterProperties.maxDate = fdateFin.value();
			fdateIni.filterProperties.validations.required = true;

			fdateFin.filterProperties = {"validations": {}};
			fdateFin.filterProperties.minDate = fdateIni.value();
			fdateFin.filterProperties.validations.required = true;
		}
	};

	/**
	 * Essa função tem a finalidade de setar o campo de Valor Unitário do produto que foi selecionado
	 * na widget de itens. Para pegar esse valor é necessario exeutar uma função do banco de dados.
	 * O resultado da função é o valor unitário do produto e sua quantidade em estoque. Ambos os valores
	 * são colocados na currentRow da widget.
	 *
	 * @param widget | widget de itens
	 */
	this.setValorUnitario = function(widget) {
		var	deffer = ZHPromise.defer();
		Est58000_RetiradaEstoque_Service.getCdtipocusto(widget.currentRow.CDFILIAL).then(function(cdtipocusto){
			widget.currentRow.DTLANCMOVI = widget.getParent().currentRow.DTLANCESTQ; 
			Est58000_RetiradaEstoque_Service.setBlankLocalEstFields(widget);
			ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(
				widget.currentRow.CDFILIAL,
				widget.currentRow.CDPRODUTO,
				widget.currentRow.DTLANCMOVI.substr(0, 10),
				cdtipocusto,
				widget.currentRow.CDALMOXARIFE,
				widget.currentRow.CDLOCALESTOQ,
				widget.currentRow.NRLOTEESTQ,
				widget.currentRow.NRSUBLOTE
			).then(function(success){
				var consulta = success.dataset.consultaCustoQuantidadeProduto;
				widget.currentRow.QTESTOQDIA = consulta.QTESTOQDIA;
				widget.currentRow.VRUNILANCTO = consulta.VRCUSTOPROD;
				Est58000_RetiradaEstoque_Service.setVrTotLancto(widget.currentRow);
				widget.currentRow.COMPLETO = 'S';
				deffer.resolve();
			}, function(error){
				ScreenService.showMessage(error.data.error);
				deffer.reject();
			});
		}, function(error){
			ScreenService.showMessage(error.data.error);
			deffer.reject();
		});
		return deffer;
	};

	/**
	 * Essa função tem a finalidade de mudar a data de lançamento durante a inserção
	 * de uma nova retirada. É verificada a data de processamento da filial que o
	 * usuário selecionou.
	 *
	 * @param widget | widget de retirada, no modo de inserção (swipe)
	 */
	this.suggestDtprocessa = function(widget) {
		var cdfilial = widget.currentRow.CDFILIAL;
		Est58000_RetiradaEstoque_Service.getParamFilial(cdfilial, false).then(function(succes){
			if(typeof succes === 'string') {
				ScreenService.showMessage(succes);
			} else {
				widget.currentRow.DTLANCESTQ = succes.DTPROCESSA;
			}
		}, function(error){
			ScreenService.showMessage(error.data.error);
		});
	};
	
	this.loadDataSource = function(widget){
		widget.getField('DSALMOXARIFE').dataSourceFilter.forEach(function(fil) {
    		if(fil.name === 'CDFILIAL'){
    			fil.value = widget.currentRow.CDFILIAL;
    		}
    	});
    	
    	widget.getField('DSLOCALESTOQ').dataSourceFilter.forEach(function(fil) {
    		if(fil.name === 'CDFILIAL'){
    			fil.value = widget.currentRow.CDFILIAL;
    		}
    	});
    	
    	widget.getField('DSREQUESTO').dataSourceFilter.forEach(function(fil) {
    		if(fil.name === 'CDFILIAL'){
    			fil.value = widget.currentRow.CDFILIAL;
    		}
    	});
    	
    	widget.getField('DSALMOXARIFE').reload();
    	widget.getField('DSLOCALESTOQ').reload();
    	widget.getField('DSREQUESTO').reload();
    	
    	this.getFilialLogada(widget.currentRow.CDFILIAL).then((success) => {
    		if(success.dataset.paramfilial_paramvalidacoes.length > 0){
	    		let returnedDataset = success.dataset.paramfilial_paramvalidacoes[0];
	            this.filialctrlestoq = returnedDataset.IDCTRLESTQ;
	            if(this.filialctrlestoq == 'S'? false : true)
	            	ScreenService.showMessage('Filial não Controla Estoque');
    		}
    	});
    	
	};
	
	this.actViewStockVisibility = function(args){
		let widget = {};
		if(args.owner.container)
			widget = args.owner.container.getWidget('itens');
		else
			widget = args.owner.field.widget;
		if(widget.currentRow.CDPRODUTO)
			widget.getAction('viewstock').isVisible = true;
		else
			widget.getAction('viewstock').isVisible = false;
	};
	
	var isset = (accessor) => {
	  try {
	    // Note we're seeing if the returned value of our function is not
	    // undefined
	    return typeof accessor() !== 'undefined' && accessor() !== null;
	  } catch (e) {
	    // And we're able to catch the Error it would normally throw for
	    // referencing a property of undefined
	    return false;
	  }
	};

    this.openAnexoSwipe = function() {
        let widgetImportTxt = templateManager.container.getWidget('wdgUploadTxt');
        widgetImportTxt.isVisible = true;
        ScreenService.openSwipe(widgetImportTxt).then(function(){
            widgetImportTxt.newRow();
        });
    };
    
    this.closeAnexoSwipe = function(widget) {
    	let widgetItens = templateManager.container.getWidget('itens');
    	widgetItens.activate();
        widget.dataSource.data.pop();
        ScreenService.closeSwipe();
        widget.isVisible = false;
    };
    
    this.importarProdutosTxt = function(widget){
    	
    	if(!widget.isValid()){
            return;
        }
        
        var widgetItens = templateManager.container.getWidget('itens');
        var widgetRetirada = templateManager.container.getWidget('retirada');
        var ignoredKeys = ["__rightActions", "__createdLocal", "__index", "$error"];
        
        widget.isVisible = false;
    	ScreenService.closeSwipe().then( () => {
            widget.currentRow.CDFILIAL = widgetRetirada.currentRow.CDFILIAL;
            widget.currentRow.DTLANCESTQ = widgetRetirada.currentRow.DTLANCESTQ;
            widget.currentRow.CONTAINER = widget.container.name;
            widget.currentRow.CDFILIMOVI = widgetRetirada.appliedFilter.FILTER_ROW.CDFILIAL;
            this.getParamsImpEst(widget.currentRow.CDFILIMOVI).then(response => {
                if(response.dataset.paramimpest_params_codebar.length > 0){
                    widget.currentRow.PARAMS = response.dataset.paramimpest_params_codebar[0];
                    Est58000_RetiradaEstoque_Service.importarProdutosTxt(Util.clone(widget.currentRow, ignoredKeys)).then(success => {
                        var itens = success.dataset.itensRetirada;
                        var itensDataSource = widgetItens.dataSource;
        
                        if(itens.length > 0) {
                            widgetRetirada.currentRow.DTLANCESTQ = itens[0].DTLANCESTQ;
                            itensDataSource.data = itens;
                            
                            itensDataSource.data.forEach( (row) => {
                                row.NRSEQUITEM = Est58000_RetiradaEstoque_Service.geraNrsequitem(itensDataSource.data);
                                row.__is_new = true;
                                row.changed  = true;
                                
                                Est58000_RetiradaEstoque_Service.getCdtipocusto(row.CDFILIAL).then(function(cdtipocusto){
									row.DTLANCMOVI = widgetRetirada.currentRow.DTLANCESTQ;
									
									if(!row.CDALMOXARIFE) row.CDALMOXARIFE = ' ';
									if(!row.CDLOCALESTOQ) row.CDLOCALESTOQ = ' ';
									if(!row.NRLOTEESTQ) row.NRLOTEESTQ = ' ';
									if(!row.NRSUBLOTE) row.NRSUBLOTE = ' ';
									if(!row.QTDEVOLESTOQ) row.QTDEVOLESTOQ = 0;
									
									ValidacoesEstoque_Service.consultaCustoQuantidadeProduto(
										row.CDFILIAL,
										row.CDPRODUTO,
										row.DTLANCMOVI.substr(0, 10),
										cdtipocusto,
										row.CDALMOXARIFE,
										row.CDLOCALESTOQ,
										row.NRLOTEESTQ,
										row.NRSUBLOTE
									).then(function(success){
										var consulta = success.dataset.consultaCustoQuantidadeProduto;
										row.QTESTOQDIA = consulta.QTESTOQDIA;
										row.VRUNILANCTO = consulta.VRCUSTOPROD;
										Est58000_RetiradaEstoque_Service.setVrTotLancto(row);
										row.COMPLETO = 'S';
									});
								});
                            });
							
                        	let impAction = widgetItens.getAction('importTxt');
                        	impAction.isVisible = widgetItens.dataSource.data.length == 0;
                        	
                        }else{
                            ScreenService.showMessage("Não foram encontrados nenhum produto a partir do arquivo");
                        }
                    }).catch(error => {
                        if(error.data){
                            if (error.data.errorCode == 400) {
                            	ScreenService.closeSwipe().then(()=>{
                                	let impAction = widget.getParent().getAction("importTxt");
                                    this.openAnexoSwipe(impAction);
                            		ScreenService.showMessage(error.data.error);
                            	});
                            }else{
                            	ScreenService.showMessage(error.data.error);
                            }
                        }else{
                            ScreenService.showMessage(error);
                        }
                        
                    });
                } else{
                    ScreenService.showMessage("Parâmetros para importação de estoque não encontrados.");
                }
            }).catch(error => {
                if(error.data){
                    ScreenService.showMessage(error.data.error);
                }else{
                    ScreenService.showMessage(error);
                }
        	});
        });
    };
    
    this.getParamsImpEst = function(cdfilial) {
        var filter = [
                {
                    "name": "CDFILPARIMPEST",
                    "operator": "=",
                    "value": cdfilial
                }
		    ];
        return General.callRestEngine('est_ajuste#/crud/paramimpest_params_codebar', 'FilterData', filter);
    };

}
Configuration(function(ContextRegister) {
	ContextRegister.register('Est58000_RetiradaEstoque_Controller', Est58000_RetiradaEstoque_Controller);
});

// FILE: ./assets/modules/relatorios/assets/js/controllers/Est12000_EmissaoDosLancamentosDeEntrada_Controller.js
function Est12000_EmissaoDosLancamentosDeEntrada_Controller(EstUtil, ScreenService, General, templateManager,ZhFilterPreferences, Est12000_EmissaoDosLancamentosDeEntrada_Service) {

    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        let rowsToSet  = {};
        this.getFilialLogada().then(function(success) {
            rowsToSet.CDFILIAL = [success.dataset.filial[0].CDFILIAL];
            rowsToSet.NMFILIAL = [success.dataset.filial[0].NMFILIAL];
            ZhFilterPreferences.openFilterWidget(widget,rowsToSet).then(function(success) {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').getField("NMFILIAL").reload();
                this.verAlmoxEstoque(templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions'));
            }.bind(this));
        }.bind(this));
        
    };
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };
    
    this.getFilialLogada = function() {
    var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
    return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
    
    // Habilita/Desabilita Almoxarifado e Localização
    this.verAlmoxEstoque = function(args){
        
        
        let widget = templateManager.container.getActiveWidget();
        
        if(widget.name != 'zhFilterPreferencesSelectConditions'){
            widget = args;
        }
        
        Est12000_EmissaoDosLancamentosDeEntrada_Service.getParamFilialData(widget).then(() =>{
            
            let almoxField = widget.getField('DSALMOXARIFE');
            let LocalField = widget.getField('DSLOCALESTOQ');
            let cdAlmoxField = widget.getField('CDALMOXARIFE');
            let cdLocalEstField = widget.getField('CDLOCALESTOQ');
            
            widget.currentRow.CDALMOXARIFE = cdAlmoxField.value() ? cdAlmoxField.value() : "";
            
            //verifica almoxarifado
            if(widget.currentRow.IDUTILALMOX === 'N'){
                almoxField.readOnly = true;
                almoxField.clearValue();
                cdAlmoxField.clearValue("");
            }else{
                almoxField.readOnly = false;
            }
            
            //verifica local do estoque
            if(widget.currentRow.IDUTILLCESTQ === 'N'){
                LocalField.readOnly = true;
                LocalField.clearValue();
                cdLocalEstField.clearValue("");
            }else{
                cdLocalEstField.clearValue();
                LocalField.readOnly = false;
            }
            
            this.checkDataFilter(widget);
            
        });
    };
    
	// Realiza a limpeza de valores do filtro caso a filial seja mudada
    this.clearFieldsOnChangeFilial = function(widget){
        let fields = [
            "DSALMOXARIFE",
            "DSLOCALESTOQ",
            "CDALMOXARIFE",
            "CDLOCALESTOQ",
            "INTERVALO",
            "CDPRODINI",
            "CDPRODFIN",
            "NMPRODINI",
            "NMPRODFIN"
            ];
        
        fields.forEach((field)=>{
            let fieldToClear = widget.getField(field);
            fieldToClear.clearValue("");
            
            if(fieldToClear.name === "DSALMOXARIFE" || fieldToClear.name === "DSLOCALESTOQ" ||
            fieldToClear.name === "NMPRODINI" || fieldToClear.name === "NMPRODFIN"){
                fieldToClear.reload();
            }
        });
    };
    
    // redefine filtro com parametros corretos a serem enviados para o backend
    this.changeFilter = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        
        var tamanhoMask = getLocalVar('CDPICTPROD').replace(/\./g,'').length;
        filterRow.CDPRODFIN = filterRow.CDPRODFIN? filterRow.CDPRODFIN : "";
        var tamanho = tamanhoMask - filterRow.CDPRODFIN.length;
        
        for(var i = 0; i < tamanho; i++){
        	filterRow.CDPRODFIN = filterRow.CDPRODFIN+"Z";
        }

        var filtro= [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value": filterRow.CDFILIAL
            },
            {
                "name":"CDALMOXARIFE",
                "operator":"=",
                "value": filterRow.CDALMOXARIFE ? filterRow.CDALMOXARIFE : null
            },
            {
                "name":"CDLOCALESTOQ",
                "operator":"=",
                "value": filterRow.CDLOCALESTOQ ? filterRow.CDLOCALESTOQ : null
            },
            {
                "name":"DTINICIAL",
                "operator":"=",
                "value": filterRow.INTERVALO ? filterRow.INTERVALO.start : null
            },
            {
                "name":"DTFINAL",
                "operator":"=",
                "value": filterRow.INTERVALO ? filterRow.INTERVALO.end : null
            },
            {
                "name":"CDPRODINI",
                "operator":"=",
                "value": filterRow.CDPRODINI ? filterRow.CDPRODINI : null
            },
            {
                "name":"CDPRODFIN",
                "operator":"=",
                "value": filterRow.CDPRODFIN ? filterRow.CDPRODFIN : null
            },
            {
                "name":"INTERVALO",
                "operator":"=",
                "value": null
            }
        ];
        
        
        widget.dataSourceFilter = filtro;
        widget.reload();
        
    };
    
	// Realiza a limpeza de valores do filtro caso os campos de Produtos seja modificado
	this.clearFieldsOnChangeProduto = function(field){
        let widget = field.widget;
        let fieldProdIni = widget.getField('NMPRODINI');
        let fieldProdFim = widget.getField('NMPRODFIN');
        let fieldGrupProd = widget.getField('NMGRUPPROD');
        
        switch(field.name){
            case 'NMPRODFIN':
            case 'NMPRODINI':
                fieldProdIni.validations = {"required": true};
                fieldProdFim.validations = {"required": true};
                delete fieldGrupProd.validations;
                widget.currentRow.NMGRUPPROD = '';
                break;
            case 'NMGRUPPROD':
                widget.currentRow.NMPRODINI = '';
                widget.currentRow.NMPRODFIN = '';
                delete fieldProdIni.validations;
                delete fieldProdFim.validations;
                fieldGrupProd.validations = {"required": true};
        }
        
    };
    
    this.checkDataFilter = function(args){
        
        let widget = templateManager.container.getActiveWidget();
        
        if(widget.name != 'zhFilterPreferencesSelectConditions'){
            widget = args;
        }
        
        let LocalField = widget.getField('DSLOCALESTOQ');
        let cdLocalEstField = widget.getField('CDLOCALESTOQ');
        
        if(widget.currentRow.IDUTILALMOX === 'S' && widget.currentRow.IDUTILLCESTQ === 'S'){
            if(widget.currentRow.CDALMOXARIFE !== ""){
                LocalField.validations = {"required": {"message": "Campo Obrigatório"}};
                LocalField.readOnly = false;
            }else{
                LocalField.clearValue();
                cdLocalEstField.clearValue();
                delete LocalField.validations;
                LocalField.readOnly = true;
            }
            
            LocalField.reload();
        }
        
        if(widget.currentRow.IDUTILALMOX === 'S' && widget.currentRow.IDUTILLCESTQ === 'N'){
            if(widget.currentRow.CDALMOXARIFE !== ""){
                cdLocalEstField.value(' ');
            }else{
                cdLocalEstField.clearValue("");
            }
        }
        
    };
    
    this.checkProductRange = function(field) {
        var currentRow = field.widget.currentRow;
      
        if(currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = (currentRow.CDPRODFIN)? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if(!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        } 
    };
    
    
}


Configuration(function(ContextRegister){
    ContextRegister.register('Est12000_EmissaoDosLancamentosDeEntrada_Controller', Est12000_EmissaoDosLancamentosDeEntrada_Controller);
});

// FILE: ./assets/modules/relatorios/assets/js/controllers/Est31100_PosicaoDeEstoque_Controller.js
function Est31100_PosicaoDeEstoque_Controller(ScreenService, General, templateManager, ProductField, ProdTreeGroup, localStorageEngine) {

	this.almoxarifadoOnChange = function(widget) {
		var currentRow = widget.currentRow;
		var localField = widget.getField("DSLOCALESTOQ");
		currentRow.DSLOCALESTOQ = null;
		if (currentRow.IDUTILLCESTQ == 'S') {
			currentRow.CDLOCALESTOQ = null;
			if (currentRow.CDALMOXARIFE) {
				localField.readOnly = false;
				localField.reload();
			}
			else {
				localField.readOnly = true;
			}
		}
	};

	this.changeFilter = function(widget) {
		var filterPopup = widget.appliedFilter.FILTER_ROW;
		var filtersArray = widget.dataSourceFilter.map(function(filter) {
			filter.operator = '=';
			filter.value = filterPopup[filter.name];
			return filter;
		});
		filtersArray.push({
			name: "DTPOSICAO",
			operator: "=",
			value: filterPopup.DTPOSICAO
		}, {
			name: "NMTIPOCUSTO",
			operator: "=",
			value: filterPopup.NMTIPOCUSTO
		});
		widget.dataSourceFilter = filtersArray;
		widget.getField("NMPRODUTO").grouped = filterPopup.IDUTILLOTE === 'S';
	};

	this.checkDataSourceData = function(widget) {
		var data = widget.dataSource.data;
		if (Array.isArray(data) && data.length === 0) {
			ScreenService.showMessage(ScreenService.i18n("Não há dados a serem listados."));
		}
	};

	this.produtoFinalIsValid = function(widget) {
		if (widget.currentRow.CDARVPRODFIN < widget.currentRow.CDARVPRODINI) {
			ScreenService.showMessage(ScreenService.i18n("Produto inicial deve ser menor ou igual ao produto final."));
			widget.currentRow.CDARVPRODFIN = null;
			widget.currentRow.NMPRODUTOFIN = null;
		}
	};

	this.produtoInicialIsValid = function(widget) {
		if (widget.currentRow.CDARVPRODFIN < widget.currentRow.CDARVPRODINI) {
			ScreenService.showMessage(ScreenService.i18n("Produto inicial deve ser menor ou igual ao produto final."));
			widget.currentRow.CDARVPRODINI = null;
			widget.currentRow.NMPRODUTOINI = null;
		}
	};

	this.setNivelData = (widget) => {

		let SetTime = setInterval((time) => {
			var levels = General.getLevelByMask("GER_CDPICTPROD", 1);
			var nivelField = widget.getField("NVTOT");
			var data = [];
			levels.forEach((level) => {
				data.push({ NVTOT: level.nivel });
			});
			nivelField.dataSource.data = data;

			widget.currentRow.NMRELATORIO = "EST31100";
			widget.currentRow.PAGE_BREAK = "S";
			widget.currentRow.ORDENANOME = "S";
			widget.currentRow.DATAPOS = getLocalVar("DTPROCESSA");

			this.getFilialLogada().then((success) => {
				widget.currentRow.CDFILIAL = success.dataset.filial_paramfilial_tipocusto[0].CDFILIAL;
				widget.currentRow.NMFILIAL = success.dataset.filial_paramfilial_tipocusto[0].NMFILIAL;
				widget.currentRow.CDTIPOCUSTO = success.dataset.filial_paramfilial_tipocusto[0].CDTIPOCUSTO;
				// widget.currentRow.NMTIPOCUSTO = success.dataset.filial_paramfilial_tipocusto[0].NMTIPOCUSTO;
				widget.currentRow.IDUTILALMOX = success.dataset.filial_paramfilial_tipocusto[0].IDUTILALMOX;
				widget.currentRow.IDUTILLCESTQ = success.dataset.filial_paramfilial_tipocusto[0].IDUTILLCESTQ;
				widget.currentRow.IDUTILLOTE = success.dataset.filial_paramfilial_tipocusto[0].IDUTILLOTE;
				this.relatorionOrUnidadeOnChange(widget);
				clearInterval(SetTime);
			});
		}, 2500);
	};

	this.getFilialLogada = function() {
		var filter = [
			{
				"name": "CDOPERADOR",
				"operator": "=",
				"value": "OPERADOR_LOGADO"
                },
			{
				"name": "CDFILIAL",
				"operator": "=",
				"value": "FILIAL_LOGADA"
				}

		    ];
		return General.callRestEngine('est_relatorios#/crud/filial_paramfilial_tipocusto', 'FilterData', filter);
	};

	this.toggleProdutoFinalReadonly = function(widget) {
		var produtoFinalField = widget.getField("NMPRODUTOFIN");
		produtoFinalField.readOnly = (widget.currentRow.CDARVPRODINI) ? false : true;
		widget.currentRow.CDARVPRODFIN = null;
		widget.currentRow.NMPRODUTOFIN = null;
	};

	this.relatorionOrUnidadeOnChange = function(widget) {
		var almoxField = widget.getField("DSALMOXARIFE");
		var localField = widget.getField("DSLOCALESTOQ");
		var nomeRelatorio = widget.currentRow.NMRELATORIO;
		var currentRow = widget.currentRow;

		almoxField.setValue(null);
		localField.setValue(null);
		widget.currentRow.CDALMOXARIFE = null;
		widget.currentRow.CDLOCALESTOQ = null;

		widget.getField("PAGE_BREAK").readOnly = nomeRelatorio != "EST31100";

		if (nomeRelatorio == 'EST31100') {
			widget.getAction('GRID').isVisible = true;
		}
		else {
			widget.getAction('GRID').isVisible = false;
		}


		// Array of Consolidated reports to manage filter fields
		let consolidatedReports = ['EST32800', 'EST32804', 'EST32808', 'EST32802'];

		if (consolidatedReports.includes(nomeRelatorio)) {
			delete(almoxField.validations);
			almoxField.readOnly = true;
			delete(localField.validations);
			localField.readOnly = true;
		}
		else {
			if (currentRow.CDFILIAL) {
				if (currentRow.IDUTILALMOX == 'S') {
					almoxField.validations = { "required": { "message": "Campo obrigatório" } };
					almoxField.readOnly = false;
					if (currentRow.IDUTILLCESTQ == 'S') {
						localField.validations = { "required": { "message": "Campo obrigatório" } };
						localField.readOnly = false;
					}
				}
				else {
					delete(almoxField.validations);
					almoxField.readOnly = true;
					delete(localField.validations);
					localField.readOnly = true;
				}
			}
			else {
				delete(almoxField.validations);
				almoxField.readOnly = true;
				delete(localField.validations);
				localField.readOnly = true;
			}
		}

		if (nomeRelatorio === 'EST32804' || nomeRelatorio === 'EST32808') {
			widget.getField('NMFILIAL').isVisible = false;
			widget.getField('NMGRUPFILI').isVisible = true;
			widget.currentRow.NMFILIAL = null;
			widget.currentRow.CDFILIAL = null;
		}
		else {
			widget.getField('NMFILIAL').isVisible = true;
			widget.getField('NMGRUPFILI').isVisible = false;
			widget.currentRow.NMGRUPFILI = null;
			widget.currentRow.CDGRUPFILI = null;
		}
		widget.redraw();

		ProductField.updateProductFilters(widget);

	};

	this.validateFilterFields = function(currentRow) {
		if (currentRow.CDARVPRODFIN) {
			var cdarvprodfin = currentRow.CDARVPRODFIN;//.replace(/[^0-9]/g, '');
			var mascara = 20 - cdarvprodfin.length;
			currentRow.CDARVPRODFIN = currentRow.CDARVPRODFIN + 'z'.repeat(mascara).substring(0, mascara);
		}
		currentRow.DTPOSICAO = moment(currentRow.DATAPOS, 'DD/MM/YYYY').add(1, 'days').format('DD/MM/YYYY');
	};

	this.generateReport = function(widget, format) {
		widget.currentRow.__FORMAT = format;
		if (widget.isValid()) {
			const { currentRow } = widget;

			this.validateFilterFields(currentRow);

			if (currentRow.PAGE_BREAK == 'N' && currentRow.NMRELATORIO == "EST31100")
				currentRow.NMRELATORIO = "EST31100_noHeader";
			currentRow.NRTPARVPROD = (currentRow.hasOwnProperty('NRTPARVPROD')) ? currentRow.NRTPARVPROD : null;
			currentRow.NRARVOREPROD = (currentRow.hasOwnProperty('NRARVOREPROD')) ? currentRow.NRARVOREPROD : null;

			General.openReport("est_relatorios#/posicao_de_estoque", currentRow.NMRELATORIO, currentRow);
		}
	};

	this.validaEstadoRangeProduto = function(widget) {
		var produtoIni = widget.getField('NMPRODUTOINI');
		var produtoFin = widget.getField('NMPRODUTOFIN');
		var tipoProduto = widget.getField('DSTPARVPROD');
		if (tipoProduto.value()) {
			produtoIni.readOnly = true;
			widget.currentRow.NMPRODUTOINI = null;
			widget.currentRow.CDARVPRODINI = null;
			widget.currentRow.CDPRODUTOINI = null;
			widget.currentRow.CDNVPRODUTOINI = null;
			produtoIni.validations = {};
			produtoFin.readOnly = true;
			produtoFin.validations = {};
			widget.currentRow.NMPRODUTOFIN = null;
			widget.currentRow.CDARVPRODFIN = null;
			widget.currentRow.CDPRODUTOFIN = null;
			widget.currentRow.CDNVPRODUTOFIN = null;
		}
		else {
			produtoIni.readOnly = false;
			produtoIni.validations = { "required": true };
			produtoFin.readOnly = false;
			produtoFin.validations = { "required": true };
		}
	};

	this.setGrupoArvore = function(widget) {
		var nmarvoreprod = widget.getField('NMARVOREPROD');
		if (widget.getField('DSTPARVPROD').value()) {
			nmarvoreprod.dataSourceFilter[0].value = widget.currentRow.NRTPARVPROD;
			nmarvoreprod.readOnly = false;
			nmarvoreprod.reload();
		}
		else {
			nmarvoreprod.dataSourceFilter[0].value = null;
			nmarvoreprod.readOnly = true;
		}
		widget.currentRow.NMTPARVORE = '';
		widget.currentRow.NMARVOREPROD = '';
		widget.currentRow.NRARVOREPROD = '';
		this.validaEstadoRangeProduto(widget);
	};

	this.validaEstadoTipoArvore = function(widget) {
		var produtoIni = widget.getField('NMPRODUTOINI');
		var produtoFin = widget.getField('NMPRODUTOFIN');
		var tipoProduto = widget.getField('DSTPARVPROD');
		var grupo = widget.getField('NMARVOREPROD');
		if (produtoIni.value() || produtoFin.value()) {
			tipoProduto.readOnly = true;
			grupo.readOnly = true;
			widget.currentRow.NRTPARVPROD = null;
			widget.currentRow.DSTPARVPROD = null;
			widget.currentRow.NMTPARVORE = null;
			widget.currentRow.NMARVOREPROD = null;
			widget.currentRow.NRARVOREPROD = null;
		}
		else {
			tipoProduto.readOnly = false;
			grupo.readOnly = false;
		}
	};

	this.requestGrupoTipoArvore = function(widget, fieldTreeProd) {
		ProdTreeGroup.getAllGroups(fieldTreeProd)
			.then(function(allGroups) {
				widget.currentRow.NRARVOREPROD = allGroups.join('_');
			})
			.catch(function(erro) {
				ScreenService.showMessage(erro.data.error);
			});
	};

	this.openGridReport = function(widget) {
		if (widget.isValid()) {
			var currentRow = widget.currentRow;
			var rowStripe = {};
			rowStripe.DSFILIAL = currentRow.CDFILIAL + ' - ' + currentRow.NMFILIAL;
			rowStripe.DSPRODUTO = currentRow.CDARVPRODINI + ' à ' + currentRow.CDARVPRODFIN.replaceAll('z', '');
			rowStripe.DATAPOS = currentRow.DATAPOS;
			rowStripe.CUSTO = currentRow.NMTIPOCUSTO;
			rowStripe.CDALMOXARIFE = currentRow.CDALMOXARIFE || ' ';
			rowStripe.CDLOCALESTOQ = currentRow.CDLOCALESTOQ || ' ';

			localStorageEngine.setDataSource('/StripePosicao', rowStripe).then(() => {
				this.validateFilterFields(currentRow);
				ScreenService.openWindow("est_relatorios#est31100_grid_relatorio_posicao").then(() => {
					setTimeout(function() { //(Verificar a necessidade deste timeout quando convertar a tela para o filtro 3.
						let widgetGrid = templateManager.containers["est_relatorios#est31100_grid_relatorio_posicao"].getWidget('gridWidget');
						if (widgetGrid) {
							widgetGrid.dataSourceFilter = [
								{
									"name": "CDARVPRODFIN",
									"operator": "=",
									"value": currentRow.CDARVPRODFIN
								},
								{
									"name": "CDALMOXARIFE",
									"operator": "=",
									"value": currentRow.CDALMOXARIFE || ' '
								},
								{
									"name": "ORDENANOME",
									"operator": "=",
									"value": currentRow.ORDENANOME
								},
								{
									"name": "CDTIPOCUSTO",
									"operator": "=",
									"value": currentRow.CDTIPOCUSTO
								},
								{
									"name": "P_NVTOT",
									"operator": "=",
									"value": currentRow.NVTOT
								},
								{
									"name": "DTPOSICAO",
									"operator": "=",
									"value": currentRow.DTPOSICAO
								},
								{
									"name": "CDARVPRODINI",
									"operator": "=",
									"value": currentRow.CDARVPRODINI
								},
								{
									"name": "CDLOCALESTOQ",
									"operator": "=",
									"value": currentRow.CDLOCALESTOQ || ' '
								},
								{
									"name": "CDFILIAL",
									"operator": "=",
									"value": currentRow.CDFILIAL
								},
								{
									"name": "NRARVOREPROD",
									"operator": "=",
									"value": currentRow.NRARVOREPROD
								},
								{
									"name": "NRTPARVPROD",
									"operator": "=",
									"value": currentRow.NRTPARVPROD

								}
			        	];

							widgetGrid.reload();
						}
						else {
							ScreenService.showMessage('Erro ao Gerar Relatório com os Dados Informados.');
						}
					}, 1000);
				});
			});
		}
	};

	this.onEnterRelatorio = function() {
		var container = templateManager.container;
		var stripe = container.getWidget('stripe');
		localStorageEngine.getDataSource('/StripePosicao').then(function(row) {
			stripe.currentRow.DSFILIAL = row[0].DSFILIAL;
			stripe.currentRow.DATAPOS = row[0].DATAPOS;
			stripe.currentRow.DSPRODUTO = row[0].DSPRODUTO;
			stripe.currentRow.CUSTO = row[0].CUSTO;
			stripe.currentRow.CDALMOXARIFE = row[0].CDALMOXARIFE;
			stripe.currentRow.CDLOCALESTOQ = row[0].CDLOCALESTOQ;
		});
	};

	this.defineIdiomaTipoCusto = function(widget) {
		// setLocalVar('language', 'en_us');
		var idioma = getLocalVar('language');
		var nomeField = null;

		if (idioma == 'pt_br')
			nomeField = 'NMTIPOCUSTO';
		else if (idioma == 'en_us')
			nomeField = 'NMTIPOCUSING';
		else if (idioma == 'es_es' || idioma == 'es_cl')
			nomeField = 'NMTIPOCUSESP';

		var fields = widget.getField('NMTIPOCUSTO').fields;

		fields.forEach(function(field) {
			if (field.name == nomeField || field.name == 'CDTIPOCUSTO')
				field.isVisible = true;
			else
				field.isVisible = false;
		});

		widget.getField('NMTIPOCUSTO').displayedFields[1] = nomeField;
		widget.getField('NMTIPOCUSTO').searchableFields[1] = nomeField;
		delete widget.getField('NMTIPOCUSTO').outData.NMTIPOCUSTO;
		widget.getField('NMTIPOCUSTO').outData[nomeField] = "NMTIPOCUSTO";

		widget.getField('NMTIPOCUSTO').reload();

		this.getFilialLogada().then(function(success) {
			widget.currentRow.CDTIPOCUSTO = success.dataset.filial_paramfilial_tipocusto[0].CDTIPOCUSTO;
			widget.currentRow.NMTIPOCUSTO = success.dataset.filial_paramfilial_tipocusto[0][nomeField];
		});
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est31100_PosicaoDeEstoque_Controller', Est31100_PosicaoDeEstoque_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est31200_IrregularidadesdeEstoque_Controller.js
function Est31200_IrregularidadesdeEstoque_Controller(General, restEngine, ScreenService, ProdTreeGroup, ZhFilterPreferences, templateManager) {
    
    this.checkUnidadeField = function(widget) {
        var currentRow = widget.currentRow;

        if(currentRow.REPORT_NAME === 'Est31200IrrEstoque') {
            var almoxarifadoField = widget.getField('DSALMOXARIFE');
            var localestoqueField = widget.getField('DSLOCALESTOQ');

            if(currentRow.CDFILIAL) {
                    var filter = [
                        {
                            "name": "CDFILIAL",
                            "operator": "=",
                            "value": currentRow.CDFILIAL
                        }
        		    ];
                    General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter).then(validation => {
                    currentRow.ALMOXARIFADO = validation.dataset.paramfilial[0].IDUTILALMOX == 'S' ? validation.dataset.paramfilial[0].IDUTILALMOX : null;
                    currentRow.LOCALESTOQUE = validation.dataset.paramfilial[0].IDUTILLCESTQ == 'S' ? validation.dataset.paramfilial[0].IDUTILLCESTQ : null;
                    if(currentRow.ALMOXARIFADO) {
                        almoxarifadoField.dataSourceFilter[1] = 
                        {
                            "name": "CDFILIAL", 
                            "operator": "=", 
                            "value": currentRow.CDFILIAL
                        };
                        almoxarifadoField.readOnly            = false;
                        almoxarifadoField.validations         = {"required": true};

                        if(currentRow.LOCALESTOQUE) {
                            localestoqueField.dataSourceFilter[1] = 
                            {
                                "name": "CDFILIAL",
                                "operator": "=", 
                                "value": currentRow.CDFILIAL
                            };
                            localestoqueField.readOnly = false;
                            localestoqueField.validations = {"required": true};

                            this.checkAlmoxariFields(widget, true);
                        } else {
                            localestoqueField.readOnly    = true;
                            localestoqueField.validations = null;
                        }
                    } else {
                        almoxarifadoField.readOnly    = true;
                        almoxarifadoField.validations = null;

                        localestoqueField.readOnly    = true;
                        localestoqueField.validations = null;
                    }
                });
            } else {
                currentRow.ALMOXARIFADO = false;
                currentRow.LOCALESTOQUE = false;

                almoxarifadoField.readOnly = true;
                localestoqueField.readOnly = true;

                almoxarifadoField.validations = null;
                localestoqueField.validations = null;

                reloadFields(widget, true, true);
            }
        }
    };

    this.checkAlmoxariFields = function(widget, flag) {
        var currentRow = widget.currentRow;

        if(currentRow.ALMOXARIFADO == 'S' && currentRow.LOCALESTOQUE) {
            widget.getField('DSLOCALESTOQ').readOnly = false;
            widget.getField('DSLOCALESTOQ').dataSourceFilter[2] = 
                {
                    "name": "CDALMOXARIFE", 
                    "operator": "=", 
                    "value": currentRow.CDALMOXARIFE
                };
        } else widget.getField('DSLOCALESTOQ').readOnly = true;

        if(flag) reloadFields(widget, true, true);
        else reloadFields(widget, false, true);
    };

    function reloadFields(widget, almoxarifado, localestoque) {
        var currentRow = widget.currentRow;

        if(almoxarifado) {
            currentRow.CDALMOXARIFE = (currentRow.ALMOXARIFADO)? null : ' ';
            currentRow.DSALMOXARIFE = (currentRow.ALMOXARIFADO)? null : ' ';

            widget.getField('DSALMOXARIFE').reload();
        }

        if(localestoque) {
            currentRow.CDLOCALESTOQ = (currentRow.LOCALESTOQUE)? null : ' ';
            currentRow.DSLOCALESTOQ = (currentRow.LOCALESTOQUE)? null : ' ';

            widget.getField('DSLOCALESTOQ').reload();
        }
    }

    this.initialize = function(widget) {
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        this.getFilialLogada().then(success => {
            let dataAtual = moment().format('DD/MM/YYYY');
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            rowsToSet.REPORT_NAME = 'Est31200IrrEstoque';
            rowsToSet.INTERVALO = {
                                    "start": dataAtual,
                                    "end": dataAtual
                                  };
            rowsToSet.DTINICIAL = dataAtual;
            rowsToSet.DTFINAL = dataAtual;
            
            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                },
                {
                    "name": "IDUTILALMOX",
                    "operator": "=",
                    "value": "S"
                }
		    ];
            
            General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter).then( response => {
                if(response.dataset.paramfilial[0]) {
                    widget.getField("DSALMOXARIFE").filterProperties.readOnly = false;
                    widget.getField("DSALMOXARIFE").validations = {"required": true};
                    if(response.dataset.paramfilial[0].IDUTILLCESTQ == 'S'){
                        widget.getField("DSLOCALESTOQ").filterProperties.readOnly = false;
                        widget.getField("DSLOCALESTOQ").validations = {"required": true};
                    } else{
                        widget.getField("DSLOCALESTOQ").filterProperties.readOnly = true;
                        widget.getField("DSLOCALESTOQ").validations = null;
                    }
                }else {
                    widget.getField("DSALMOXARIFE").filterProperties.readOnly = true;
                    widget.getField("DSALMOXARIFE").validations = null;
                    widget.getField("DSLOCALESTOQ").filterProperties.readOnly = true;
                    widget.getField("DSLOCALESTOQ").validations = null;
                }
                ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
            });
            
            
        });        
    };
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };
    
    function setDefaultValues(widget) {
        var data = moment().format('DD/MM/YYYY');

        widget.currentRow.INTERVALO =   {
                                            "start": data,
                                            "end": data
                                        };
        widget.currentRow.DTINICIAL = data;
        widget.currentRow.DTFINAL = data;
        widget.currentRow.REPORT_NAME = 'Est31200IrrEstoque';
        widget.currentRow.NRTPARVPROD = null;
        changeReport(widget);
    }

    function changeReport(widget) {
        switch(widget.currentRow.REPORT_NAME) {
            case 'Est31200IrrEstoque':
                setEST31200DefaultValues(widget.currentRow); break;
        }
    }

    this.changeReport = changeReport;

    function setEST31200DefaultValues(currentRow) {
        currentRow.CDALMOXARIFE = ' ';
        currentRow.CDLOCALESTOQ = ' ';
    }

    this.setMinMaxDate = function(widget){
        var interval = widget.getField("INTERVALO").value();
        if(interval){
            widget.getField('DTINICIAL').setValue(interval.start);
            widget.getField('DTFINAL').setValue(interval.end);
        }
    };

    this.checkProductRange = function(field) {
        var currentRow = field.widget.currentRow;

        if(currentRow.CDPRODUTOINI && currentRow.CDPRODUTOFIN) {
            var compproduto = (currentRow.CDPRODUTOFIN)? currentRow.CDPRODUTOFIN.replace(/[\.\-]/g, '') : null;

            if(!General.comparaPratoProduto(currentRow.CDPRODUTOINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        }
    };

    this.changeFilter = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;

        var filtro= [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value": filterRow.CDFILIAL
            },
            {
                "name":"CDALMOXARIFE",
                "operator":"=",
                "value": filterRow.CDALMOXARIFE ? filterRow.CDALMOXARIFE : null
            },
            {
                "name":"CDLOCALESTOQ",
                "operator":"=",
                "value": filterRow.CDLOCALESTOQ ? filterRow.CDLOCALESTOQ : null
            },
            {
                "name":"DTINICIAL",
                "operator":"=",
                "value": filterRow.DTINICIAL ? filterRow.DTINICIAL : null
            },
            {
                "name":"DTFINAL",
                "operator":"=",
                "value": filterRow.DTFINAL ? filterRow.DTFINAL : null
            },
            {
                "name":"NRTPARVPROD",
                "operator":"=",
                "value": filterRow.NRTPARVPROD ? filterRow.NRTPARVPROD : null
            },
            {
                "name":"CDARVPRODINI",
                "operator":"=",
                "value": filterRow.CDARVPRODINI ? filterRow.CDARVPRODINI : null
            },
            {
                "name":"CDARVPRODFIN",
                "operator":"=",
                "value": filterRow.CDARVPRODFIN ? filterRow.CDARVPRODFIN : null
            },
            {
                "name":"NRARVOREPROD",
                "operator":"=",
                "value": filterRow.NRARVOREPROD ? filterRow.NRARVOREPROD : 'T'
            }
        ];
        
        widget.dataSourceFilter = filtro;
        widget.reload();
        
    };
    
    function getFilter(widget, fields, prodFin) {
        let row = widget.appliedFilter.FILTER_ROW;
        let filter = [];
        fields.forEach((f) => {
            filter.push(
            {
                "name": f,
                "operator": "=",
                "value": (f !== 'CDPRODUTOFIN') ? row[f] : getProdFinFormatado(row.CDPRODUTOFIN)
            });
        });
        return filter;
    }
    
    function getProdFinFormatado(prodfin) {
        var masksize = getLocalVar('CDPICTPROD').replace(/[\.\-]/g, '').length - prodfin.length;
        return General.fill(prodfin, masksize, true, 'z');
    }
    
    this.setGrupoArvore = function(widget){
		var nmarvoreprod = widget.getField('NMARVOREPROD');
		if (widget.getField('DSTPARVPROD').value()){
			nmarvoreprod.dataSourceFilter[0].value = widget.currentRow.NRTPARVPROD;
			nmarvoreprod.readOnly = false;
			nmarvoreprod.reload();
		}else{
			nmarvoreprod.dataSourceFilter[0].value = null;
			nmarvoreprod.readOnly = true;
		}
		widget.currentRow.NMTPARVORE = '';
		widget.currentRow.NMARVOREPROD = '';
		widget.currentRow.NRARVOREPROD = '';
	    this.validaEstadoRangeProduto(widget);
	};
    
    this.validaEstadoRangeProduto = function(widget){
		var produtoIni = widget.getField('NMPRODUTOINI');
		var produtoFin = widget.getField('NMPRODUTOFIN');
		var tipoProduto = widget.getField('DSTPARVPROD');
		if (tipoProduto.value()){
			produtoIni.readOnly = true;
			widget.currentRow.NMPRODUTOINI = null;
			widget.currentRow.CDARVPRODINI = null;
			widget.currentRow.CDPRODUTOINI = null;
			widget.currentRow.CDNVPRODUTOINI = null;
			produtoIni.validations = {};
			produtoFin.readOnly = true;
			produtoFin.validations = {};
			widget.currentRow.NMPRODUTOFIN = null;
			widget.currentRow.CDARVPRODFIN = null;
			widget.currentRow.CDPRODUTOFIN = null;
			widget.currentRow.CDNVPRODUTOFIN = null;
		}else{
			produtoIni.readOnly = false;
			produtoIni.validations = {"required":true};
			produtoFin.readOnly = false;
			produtoFin.validations = {"required":true};
		}
	};
	
	this.validaEstadoTipoArvore = function(widget){
		var produtoIni = widget.getField('NMPRODUTOINI'); 
		var produtoFin = widget.getField('NMPRODUTOFIN');
		var tipoProduto = widget.getField('DSTPARVPROD');
		var grupo = widget.getField('NMARVOREPROD');
		if (produtoIni.value() || produtoFin.value()){
			tipoProduto.readOnly = true;
			grupo.readOnly = true;
			widget.currentRow.NRTPARVPROD = null;
			widget.currentRow.DSTPARVPROD = null;
			widget.currentRow.NMTPARVORE = null;
			widget.currentRow.NMARVOREPROD = null;
			widget.currentRow.NRARVOREPROD = null;
		}else{
			tipoProduto.readOnly = false;
			grupo.readOnly = false;
		}
	};
	
	this.requestGrupoTipoArvore = function(widget, fieldTreeProd){
		ProdTreeGroup.getAllGroups(fieldTreeProd)
			.then(function(allGroups){
				widget.currentRow.NRARVOREPROD = allGroups.join('_');
			})
			.catch(function(erro){
				ScreenService.showMessage(erro.data.error);
			});
	};
	
	this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
	
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est31200_IrregularidadesdeEstoque_Controller', Est31200_IrregularidadesdeEstoque_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est31300_MovimentacaodeEstoque_Controller.js
function Est31300_MovimentacaodeEstoque_Controller(restEngine, General, ScreenService, Est31300_MovimentacaodeEstoque_Service, ProductField, templateManager, metaDataFactory, eventEngine) {

    this.checkUnidadeFields = function(widget) {
        var currentRow = widget.currentRow;

        ProductField.updateProductFilters(widget);
        if (currentRow.REPORT_NAME === 'Est31300MovEstoque' ||
            currentRow.REPORT_NAME === 'Est31300MovEstoqueSintetico' ||
            currentRow.REPORT_NAME === 'Est31300MovEstoqueSinteticoQuantidade') {
            if (currentRow.CDFILIAL) {
                Est31300_MovimentacaodeEstoque_Service.requestValidations(currentRow.CDFILIAL).then(function(checks) {
                    currentRow.ALMOXARIFADO = checks.IDUTILALMOX;
                    currentRow.LOCALESTOQUE = checks.IDUTILLCESTQ;
                    currentRow.LOTE = checks.IDUTILLOTE;

                    var almoxarifado = widget.getField('DSALMOXARIFE');
                    var localestoque = widget.getField('DSLOCALESTOQ');
                    var lote = widget.getField('NRLOTEESTQ');

                    if (currentRow.LOTE) {
                        lote.dataSourceFilter[1] = { "name": "CDFILIAL", "operator": "=", "value": currentRow.CDFILIAL };
                        lote.readOnly = false;
                        currentRow.NRLOTEESTOQ = null;

                        this.reloadFields(widget, false, false, true);
                    }
                    else {
                        lote.readOnly = true;
                        lote.setValue('');
                        currentRow.NRLOTEESTOQ = " ";

                    }
                    if (currentRow.ALMOXARIFADO) {
                        almoxarifado.dataSourceFilter[0] = { "name": "CDFILIAL", "operator": "=", "value": currentRow.CDFILIAL };
                        almoxarifado.readOnly = false;
                        almoxarifado.validations = { "required": true };
                        this.reloadFields(widget, true, false, false);

                        if (currentRow.LOCALESTOQUE) {
                            localestoque.dataSourceFilter[0] = { "name": "CDFILIAL", "operator": "=", "value": currentRow.CDFILIAL };
                            localestoque.validations = { "required": true };

                            this.checkAlmoxariFields(widget, true);
                        }
                        else {
                            localestoque.readOnly = true;
                            localestoque.setValue(' ');
                            localestoque.validations = null;
                        }
                    }
                    else {
                        almoxarifado.readOnly = true;
                        almoxarifado.validations = null;
                        almoxarifado.setValue(' ');
                        currentRow.CDALMOXARIFE = ' ';

                        localestoque.readOnly = true;
                        localestoque.validations = null;
                        localestoque.setValue(' ');
                        currentRow.CDLOCALESTOQ = ' ';

                    }

                }.bind(this));
            }
            else {
                currentRow.ALMOXARIFADO = false;
                currentRow.LOCALESTOQUE = false;
                currentRow.NRLOTEESTOQ = false;

                widget.getField('DSALMOXARIFE').readOnly = true;
                widget.getField('DSLOCALESTOQ').readOnly = true;
                widget.getField('NRLOTEESTQ').readOnly = true;

                widget.getField('DSALMOXARIFE').validations = null;
                widget.getField('DSLOCALESTOQ').validations = null;
                widget.getField('NRLOTEESTQ').validations = null;

                this.reloadFields(widget, true, true, true);
            }
        }
    };

    this.checkAlmoxariFields = function(widget) {
        var currentRow = widget.currentRow;

        if (currentRow.CDALMOXARIFE && currentRow.LOCALESTOQUE) {
            widget.getField('DSLOCALESTOQ').readOnly = false;
            widget.getField('DSLOCALESTOQ').dataSourceFilter[2] = { "name": "CDALMOXARIFE", "operator": "=", "value": currentRow.CDALMOXARIFE };
        }
        else widget.getField('DSLOCALESTOQ').readOnly = true;

        if (arguments[1]) this.reloadFields(widget, true, true, true);
        else this.reloadFields(widget, false, true, false);
    };

    this.reloadFields = function(widget, almoxarifado, localestoque, lote) {
        var currentRow = widget.currentRow;

        if (almoxarifado) {
            currentRow.CDALMOXARIFE = (currentRow.ALMOXARIFADO) ? null : ' ';
            currentRow.DSALMOXARIFE = (currentRow.ALMOXARIFADO) ? null : ' ';

            widget.getField('DSALMOXARIFE').reload();
        }

        if (localestoque) {
            currentRow.CDLOCALESTOQ = (currentRow.LOCALESTOQUE) ? null : ' ';
            currentRow.DSLOCALESTOQ = (currentRow.LOCALESTOQUE) ? null : ' ';

            widget.getField('DSLOCALESTOQ').reload();
        }

        if (lote) {
            currentRow.NRLOTEESTQ = null;
            currentRow.NRSUBLOTE = null;

            widget.getField('NRLOTEESTQ').reload();
        }
    };

    this.setDefaultValues = function(currentRow, formWidget, first) {
        var data = moment().format('DD/MM/YYYY');
        // var primeiroDia = moment(data, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY');
        // var ultimoDia   = moment(data, 'DD/MM/YYYY').endOf('month').format('DD/MM/YYYY');

        if (!currentRow.DTPERINI || !currentRow.DTPERFIN) {
            currentRow.DTPERINI = data;
            currentRow.DTPERFIN = data;
        }
        if (!currentRow.INTERVALO) {
            currentRow.INTERVALO = {};
        }
        currentRow.INTERVALO.start = data;
        currentRow.INTERVALO.end = data;

        currentRow.HEADER_DESCR = 'S';
        if (currentRow.REPORT_NAME === 'Est31300MovEstoque' ||
            currentRow.REPORT_NAME === 'Est31300MovEstoqueSintetico' ||
            currentRow.REPORT_NAME === 'Est31400MovEstoqueConsolidadoAnali' ||
            currentRow.REPORT_NAME === 'Est31400MovEstoqueConsolidadoSint' ||
            currentRow.REPORT_NAME === 'Est31300MovEstoqueSinteticoQuantidade' ||
            first) {

            currentRow.IDLANCTO = 3;
            currentRow.CUSTOS = 1;
            currentRow.EXIBE = 'E';
            
            var campoNivel = formWidget.getField('NPTOTALIZA');

            let SetTime = setInterval((time) => {
                var levels = General.getLevelByMask('GER_CDPICTPROD', 1);
                campoNivel.dataSource.data = [];
                for (var index in levels) {
                    if (index < levels.length)
                        campoNivel.dataSource.data.push({
                            "name": levels[index].nivel,
                            "value": levels[index].caracteres
                        });
                }

                campoNivel.readOnly = false;
                //campoNivel.setValue(campoNivel.dataSource.data[0].value);

                currentRow.CDALMOXARIFE = ' ';
                currentRow.CDLOCALESTOQ = ' ';
                currentRow.NRLOTEESTOQ = ' ';

                currentRow.ALMOXARIFADO = false;
                currentRow.LOCALESTOQUE = false;
                currentRow.LOTE = false;
                clearInterval(SetTime);
            }, 2500);
        }
        if (first) {
            this.getFilialLogada().then(function(success) {
                currentRow.CDFILIAL = success.dataset.filial[0].CDFILIAL;
                currentRow.NMFILIAL = success.dataset.filial[0].NMFILIAL;
                currentRow.DTPERINI = moment().format('DD/MM/YYYY');
                currentRow.DTPERFIN = moment().format('DD/MM/YYYY');
                var widget = templateManager.container.getActiveWidget();
                widget.getField('DSALMOXARIFE').dataSourceFilter.forEach(function(fil) {
                    if (fil.name == "CDFILIAL") {
                        fil.value = currentRow.CDFILIAL;
                    }
                });
                widget.getField('DSLOCALESTOQ').dataSourceFilter.forEach(function(fil) {
                    if (fil.name == "CDFILIAL") {
                        fil.value = currentRow.CDFILIAL;
                    }
                });
                widget.getField('DSALMOXARIFE').reload();
                widget.getField('DSLOCALESTOQ').reload();
                ProductField.updateProductFilters(widget);
            }.bind(this));
        }

    };

    this.intervaloChange = function(field) {
        let widget = field.widget;
        widget.currentRow.DTPERINI = widget.currentRow.INTERVALO.start;
        widget.currentRow.DTPERFIN = widget.currentRow.INTERVALO.end;
    };

    this.getFilialLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.setMinMaxDate = function(widget, MaxOrMin) {
        if (MaxOrMin === 'MAX') {
            var maxDate = widget.getField('DTPERFIN').value();
            var fieldInicial = widget.getField('DTPERINI');
            fieldInicial.maxDate = maxDate;
        }
        else {
            var minDate = widget.getField('DTPERINI').value();
            var fieldFinal = widget.getField('DTPERFIN');
            fieldFinal.minDate = minDate;
        }
    };

    this.validateReportPDF = function(widget) {
        widget.currentRow.__FORMAT = "pdf";
        this.validateReport(widget);
    };

    this.validateReportXLSX = function(widget) {
        widget.currentRow.__FORMAT = "xlsx";
        this.validateReport(widget);
    };

    this.validateReport = function(widget) {
        var currentRow = widget.currentRow;
        var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

        if (widget.isValid()) {
            if (General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('GER_CDPICTPROD'))) {
                Est31300_MovimentacaodeEstoque_Service.requestReport(currentRow).then(function(birtURL) {
                    window.open(birtURL.dataset.URL[0]);
                });
            }
            else {
                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        }
    };

    this.fieldOnChangeRelatorio = function(widget) {

        //datasource removido do Json, relatorio desativado pois movimentacao customizavel o substituiu. Caso volte usar este relatorio deve ser feita manutenção nas tabelas de totalizacao.
        /*
        {
            "name": "Movimentação de Estoque Consolidado - Sintético (por Filial)",
            "value": "EST31300SINTCONS"
        }
        */

        //default fields bfore change
        widget.getField('NMGRUPPROD').isVisible = false;
        widget.getField('DSALMOXARIFE').setValue(null);
        widget.getField('DSLOCALESTOQ').setValue(null);

        if (widget.currentRow.REPORT_NAME === 'Est31300MovEstoqueApInventario' ||
            widget.currentRow.REPORT_NAME === 'EST31308Movdiaria') {
            widget.getField('DSALMOXARIFE').readOnly = true;
            widget.getField('DSALMOXARIFE').validations = {};
            widget.currentRow.DSALMOXARIFE = '';

            widget.getField('DSLOCALESTOQ').readOnly = true;
            widget.getField('DSLOCALESTOQ').validations = {};
            widget.currentRow.DSLOCALESTOQ = '';

            widget.getField('NRLOTEESTQ').readOnly = true;
            widget.getField('NRLOTEESTQ').validations = {};
            widget.currentRow.NRLOTEESTQ = '';

            widget.getField('IDLANCTO').readOnly = true;
            widget.currentRow.IDLANCTO = ' ';
            //widget.getField('IDLANCTO').dataSource.data.push({"name": " ", "value": " "});
            widget.getField('CUSTOS').readOnly = true;
            //widget.getField('CUSTOS').dataSource.data.push({"name": " ", "value": " "});
            widget.currentRow.CUSTOS = ' ';
            widget.getField('NPTOTALIZA').readOnly = true;
            //widget.getField('NPTOTALIZA').dataSource.data.push({"name": " ", "value": " "});
            widget.currentRow.NPTOTALIZA = ' ';
            widget.getField('NPTOTALIZA').validations = { "required": false };

            widget.getField('NMGRUPPROD').isVisible = true;

            changeSelectFieldBehavior('autocomplete', widget);
            changeDateVisibility('dia', widget);
        }

        else if (widget.currentRow.REPORT_NAME === 'Est31300MovEstoque' ||
            widget.currentRow.REPORT_NAME === 'Est31300MovEstoqueSintetico') {
            if (widget.currentRow.CDFILIAL) {
                this.checkUnidadeFields(widget);
            }
            widget.getField('IDLANCTO').readOnly = false;
            //widget.getField('IDLANCTO').dataSource.data.pop();
            widget.getField('CUSTOS').readOnly = false;
            //widget.getField('CUSTOS').dataSource.data.pop();
            widget.getField('NPTOTALIZA').readOnly = false;
            widget.getField('NPTOTALIZA').validations = { "required": true };
            widget.getField('NPTOTALIZA').dataSource = { "data": [{ "name": "", "value": "" }] };

            changeSelectFieldBehavior('autocomplete', widget);
            changeDateVisibility('dia', widget);
        }

        else if (widget.currentRow.REPORT_NAME === 'Est31400MovEstoqueConsolidadoAnali' ||
            widget.currentRow.REPORT_NAME === 'Est31400MovEstoqueConsolidadoSint') {
            if (widget.currentRow.CDFILIAL) {
                this.checkUnidadeFields(widget);
            }
            widget.getField('DSALMOXARIFE').readOnly = true;
            widget.getField('DSALMOXARIFE').validations = {};
            widget.currentRow.DSALMOXARIFE = ' ';

            widget.getField('DSLOCALESTOQ').readOnly = true;
            widget.getField('DSLOCALESTOQ').validations = {};
            widget.currentRow.DSLOCALESTOQ = ' ';

            widget.getField('NRLOTEESTQ').readOnly = true;
            widget.getField('NRLOTEESTQ').validations = {};
            widget.currentRow.NRLOTEESTQ = ' ';

            widget.getField('NPTOTALIZA').readOnly = false;
            widget.getField('NPTOTALIZA').validations = { "required": true };
            widget.getField('NPTOTALIZA').dataSource = { "data": [{ "name": "", "value": "" }] };

            changeSelectFieldBehavior('autocomplete', widget);
            changeDateVisibility('dia', widget);
        }

        else if (widget.currentRow.REPORT_NAME === 'Est31300MovEstoqueSinteticoQuantidade') {
            if (widget.currentRow.CDFILIAL) {
                this.checkUnidadeFields(widget);
            }
            widget.getField('IDLANCTO').readOnly = false;
            widget.getField('CUSTOS').readOnly = false;

            widget.getField('DSALMOXARIFE').readOnly = true;
            widget.getField('DSALMOXARIFE').validations = {};
            widget.currentRow.DSALMOXARIFE = ' ';

            widget.getField('DSLOCALESTOQ').readOnly = true;
            widget.getField('DSLOCALESTOQ').validations = {};
            widget.currentRow.DSLOCALESTOQ = ' ';

            widget.getField('NRLOTEESTQ').readOnly = true;
            widget.getField('NRLOTEESTQ').validations = {};
            widget.currentRow.NRLOTEESTQ = ' ';

            widget.getField('NPTOTALIZA').readOnly = false;
            widget.getField('NPTOTALIZA').validations = { "required": true };
            widget.getField('NPTOTALIZA').dataSource = { "data": [{ "name": "", "value": "" }] };

            changeSelectFieldBehavior('autocomplete', widget);
            changeDateVisibility('dia', widget);
        }

        // else if (widget.currentRow.REPORT_NAME === 'EST31300SINTCONS') {
        //     if (widget.currentRow.CDFILIAL) {
        //         this.checkUnidadeFields(widget);
        //     }
        //     widget.getField('DSALMOXARIFE').readOnly = true;
        //     widget.getField('DSALMOXARIFE').validations = {};
        //     widget.currentRow.DSALMOXARIFE = ' ';

        //     widget.getField('DSLOCALESTOQ').readOnly = true;
        //     widget.getField('DSLOCALESTOQ').validations = {};
        //     widget.currentRow.DSLOCALESTOQ = ' ';

        //     widget.getField('NRLOTEESTQ').readOnly = true;
        //     widget.getField('NRLOTEESTQ').validations = {};
        //     widget.currentRow.NRLOTEESTQ = ' ';

        //     widget.getField('NPTOTALIZA').readOnly = true;
        //     widget.getField('NPTOTALIZA').validations = { "required": false };
        //     widget.currentRow.NPTOTALIZA = ' ';

        //     widget.getField('DTPERINI').readOnly = true;
        //     widget.getField('DTPERFIN').readOnly = true;

        //     widget.getField('IDLANCTO').readOnly = true;
        //     widget.currentRow.IDLANCTO = ' ';
        //     widget.getField('CUSTOS').readOnly = true;

        //     changeSelectFieldBehavior('multiple', widget);
        //     changeDateVisibility('mes', widget);

        //     //Criar função para buscar parametrização e preencher as variaveis readonly; Ou nao
        // }

        //manipulação do field header
        if (widget.currentRow.REPORT_NAME != 'Est31300MovEstoqueApInventario') {
            widget.getField('HEADER_DESCR').readOnly = true;
            widget.getField('HEADER_DESCR').setValue('S');
            widget.currentRow.HEADER_DESCR = 'S';
        }
        else {
            widget.getField('HEADER_DESCR').readOnly = false;
        }
        this.setDefaultValues(widget.currentRow, widget, false);
        templateManager.updateTemplate();
        widget.reload();
    };

    var changeSelectFieldBehavior = function(type, widget) {
        if (type == 'autocomplete') {
            if (widget.getField('NMFILIAL').isVisible == false) {
                widget.getField('NMFILIAL').isVisible = true;
                widget.getField('NMFILIALMULTIPLE').isVisible = false;
                widget.getField('NMFILIALMULTIPLE').setValue(null);
                widget.getField('NMFILIAL').setValue(null);
                widget.currentRow.CDFILIAL = null;
            }
        }
        else {
            if (widget.getField('NMFILIALMULTIPLE').isVisible == false) {
                widget.getField('NMFILIALMULTIPLE').isVisible = true;
                widget.getField('NMFILIAL').isVisible = false;
                widget.getField('NMFILIALMULTIPLE').setValue(null);
                widget.getField('NMFILIAL').setValue(null);
                widget.currentRow.CDFILIAL = null;
            }
        }
    };

    var changeDateVisibility = function(type, widget) {
        if (type != 'mes') {
            if (widget.getField('DTPERINI').isVisible == false) {
                widget.getField('DTPERINI').isVisible = true;
                widget.getField('DTPERFIN').isVisible = true;
                widget.getField('DTPESQUISA').isVisible = false;
                widget.getField('DTPESQUISA').setValue(null);
                widget.getField('DTPERINI').setValue(null);
                widget.getField('DTPERFIN').isVisible = true;
            }
        }
        else {
            if (widget.getField('DTPESQUISA').isVisible == false) {
                widget.getField('DTPERINI').isVisible = false;
                widget.getField('DTPERFIN').isVisible = false;
                widget.getField('DTPESQUISA').isVisible = true;
                widget.getField('DTPESQUISA').setValue(null);
                widget.getField('DTPERINI').setValue(null);
                widget.getField('DTPERFIN').setValue(null);
            }
        }
    };

    this.changeToSelectMultiple = function(fieldSelectArgs, selected = false) {
        var selectField = fieldSelectArgs.owner.name == 'confirmarSelect' ? fieldSelectArgs.owner.widget.selectField : fieldSelectArgs.owner.field;
        var formWidget = selectField.getParent();
        var selectFieldWidget = selectField.selectWidget;

        if (fieldSelectArgs.owner.name != 'confirmarSelect')
            selectFieldWidget.unSelectAll();

        if (!selected) {
            if (formWidget.currentRow.REPORT_NAME != 'EST31300SINTCONS') {
                selectFieldWidget.showCheckbox = false;
                if (!!selectFieldWidget.getAction('confirmarSelect'))
                    selectFieldWidget.getAction('confirmarSelect').isVisible = false;

                eventEngine.unbindEvent(selectFieldWidget, 'WidgetAfterMoveRow', 0);
                eventEngine.bindEvent(selectFieldWidget, 'WidgetAfterMoveRow', 'args.owner.setSelected(args);');
            }
            else {
                if (!selectFieldWidget.getAction('confirmarSelect')) {
                    selectFieldWidget.showCheckbox = true;

                    var actionConfirmarSelectMultiple = {
                        name: 'confirmarSelect',
                        label: 'Confirmar',
                        showAsAction: 'never_list',
                        isVisible: true,
                        showOnList: true,
                        events: [{
                            name: 'ActionEvent',
                            code: 'args.owner.widget.setSelected(args.owner.widget.dataSource.getCheckedRows(), args.owner.widget.selectField); args.owner.widget.afterClosePopup(args.owner.widget.selectField);',
                        }]
                    };
                    actionConfirmarSelectMultiple = metaDataFactory.actionFactory(actionConfirmarSelectMultiple);
                    actionConfirmarSelectMultiple.widget = selectFieldWidget;
                    selectFieldWidget.actions.push(actionConfirmarSelectMultiple);

                    eventEngine.unbindEvent(selectFieldWidget, 'WidgetAfterMoveRow', 0);
                    eventEngine.bindEvent(selectFieldWidget, 'WidgetAfterMoveRow', 'args.owner.toggle(args.owner.currentRow);');

                    selectFieldWidget.getAction('confirmarSelect').isVisible = true;
                }
                else {
                    eventEngine.unbindEvent(selectFieldWidget, 'WidgetAfterMoveRow', 0);
                    eventEngine.bindEvent(selectFieldWidget, 'WidgetAfterMoveRow', 'args.owner.toggle(args.owner.currentRow);');
                }
            }
        }
        else {
            var selectedRows = selectFieldWidget.getCheckedRows();
            var multiItens = selectedRows.length !== 1;

            if (!multiItens) {
                args.owner = selectFieldWidget;
                selectFieldWidget.setSelected(args);
                return args;
            }

            ScreenService.closePopup();
        }
        templateManager.updateTemplate();
    };

    this.onChangeData = function(args) {
        if (args.owner.field.value()) {
            var widget = args.owner.field.widget;
            var mes = args.owner.field.value().split("/")[0];
            var ano = args.owner.field.value().split("/")[1];
            var firstDay = new Date(ano, mes - 1, 1).getDateBr();
            var lastDay = new Date(ano, mes, 0).getDateBr();
            var fieldDTINI = widget.getField('DTPERINI');
            var fieldDTFIN = widget.getField('DTPERFIN');
            fieldDTINI.setValue(firstDay);
            fieldDTFIN.setValue(lastDay);
        }
    };

    this.cleanFieldsOnChange = function(field) {
        let widget = field.widget;
        let fieldProdIni = widget.getField('NMPRODINI');
        let fieldProdFim = widget.getField('NMPRODFIN');
        let fieldGrupProd = widget.getField('NMGRUPPROD');

        switch (field.name) {
            case 'NMPRODFIN':
            case 'NMPRODINI':
                fieldProdIni.validations = { "required": true };
                fieldProdFim.validations = { "required": true };
                delete fieldGrupProd.validations;
                widget.currentRow.NMGRUPPROD = '';
                widget.currentRow.CDGRUPPROD = '';
                break;
            case 'NMGRUPPROD':
                widget.currentRow.NMPRODINI = '';
                widget.currentRow.NMPRODFIN = '';
                widget.currentRow.CDPRODINI = '';
                widget.currentRow.CDPRODFIN = '';
                delete fieldProdIni.validations;
                delete fieldProdFim.validations;
                fieldGrupProd.validations = { "required": true };
        }

    };

    this.saveParamsRelat = function(args) {
        let widget = args.owner.widget;
        let row = widget.currentRow;
        General.callRestEngine("est_relatorios#/saveParamsMoviest", "Row", row).then((result) => {

            ScreenService.closePopup();
            widget.container.getWidget('est31300').edit();
            widget.container.getWidget('est31300').activate();
        });
    };

    this.openPopupParams = function(args) {
        let widgetPopup = templateManager.container.getWidget('popupParametrizacao');
        widgetPopup.reload().then(loaded => {
            if (loaded) {
                widgetPopup.currentRow = loaded.dataset.getParamsMoviest[0];
                widgetPopup.fields.forEach(field => field.reload());
            }

            ScreenService.openPopup(widgetPopup).then(done => {
                widgetPopup.edit();
                templateManager.updateTemplate();
            });
        });

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est31300_MovimentacaodeEstoque_Controller', Est31300_MovimentacaodeEstoque_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est31300_gridMovimentacaoEstoque_Controller.js
function Est31300_gridMovimentacaoEstoque_Controller(General, restEngine, ScreenService, ProdTreeGroup, ZhFilterPreferences, templateManager, EstUtil, MessageService) {
    const orderby = "CDFILIAL, CDPRODMOVIEST, DTLANCMOVIEST, IDTIPOMOVIEST, NRLANCMOVIEST, NRSEQUMOVIEST";
    
    this.checkUnidadeField = function(widget) {
        var currentRow = widget.currentRow;

        if(currentRow.REPORT_NAME === 'Est31200IrrEstoque') {
            var almoxarifadoField = widget.getField('DSALMOXARIFE');
            var localestoqueField = widget.getField('DSLOCALESTOQ');

            if(currentRow.CDFILIAL) {
                    var filter = [
                        {
                            "name": "CDFILIAL",
                            "operator": "=",
                            "value": currentRow.CDFILIAL
                        }
        		    ];
                    General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter).then(validation => {
                    currentRow.ALMOXARIFADO = validation.dataset.paramfilial[0].IDUTILALMOX == 'S' ? validation.dataset.paramfilial[0].IDUTILALMOX : null;
                    currentRow.LOCALESTOQUE = validation.dataset.paramfilial[0].IDUTILLCESTQ == 'S' ? validation.dataset.paramfilial[0].IDUTILLCESTQ : null;
                    if(currentRow.ALMOXARIFADO) {
                        almoxarifadoField.dataSourceFilter[1] = 
                        {
                            "name": "CDFILIAL", 
                            "operator": "=", 
                            "value": currentRow.CDFILIAL
                        };
                        almoxarifadoField.readOnly            = false;
                        almoxarifadoField.validations         = {"required": true};

                        if(currentRow.LOCALESTOQUE) {
                            localestoqueField.dataSourceFilter[1] = 
                            {
                                "name": "CDFILIAL",
                                "operator": "=", 
                                "value": currentRow.CDFILIAL
                            };
                            localestoqueField.readOnly = false;
                            localestoqueField.validations = {"required": true};

                            this.checkAlmoxariFields(widget, true);
                        } else {
                            localestoqueField.readOnly    = true;
                            localestoqueField.validations = null;
                        }
                    } else {
                        almoxarifadoField.readOnly    = true;
                        almoxarifadoField.validations = null;

                        localestoqueField.readOnly    = true;
                        localestoqueField.validations = null;
                    }
                });
            } else {
                currentRow.ALMOXARIFADO = false;
                currentRow.LOCALESTOQUE = false;

                almoxarifadoField.readOnly = true;
                localestoqueField.readOnly = true;

                almoxarifadoField.validations = null;
                localestoqueField.validations = null;

                reloadFields(widget, true, true);
            }
        }
    };

    this.checkAlmoxariFields = function(widget, flag) {
        var currentRow = widget.currentRow;

        if(currentRow.ALMOXARIFADO == 'S' && currentRow.LOCALESTOQUE) {
            widget.getField('DSLOCALESTOQ').readOnly = false;
            widget.getField('DSLOCALESTOQ').dataSourceFilter[2] = 
                {
                    "name": "CDALMOXARIFE", 
                    "operator": "=", 
                    "value": currentRow.CDALMOXARIFE
                };
        } else widget.getField('DSLOCALESTOQ').readOnly = true;

        if(flag) reloadFields(widget, true, true);
        else reloadFields(widget, false, true);
    };

    function reloadFields(widget, almoxarifado, localestoque) {
        var currentRow = widget.currentRow;

        if(almoxarifado) {
            currentRow.CDALMOXARIFE = (currentRow.ALMOXARIFADO)? null : ' ';
            currentRow.DSALMOXARIFE = (currentRow.ALMOXARIFADO)? null : ' ';

            widget.getField('DSALMOXARIFE').reload();
        }

        if(localestoque) {
            currentRow.CDLOCALESTOQ = (currentRow.LOCALESTOQUE)? null : ' ';
            currentRow.DSLOCALESTOQ = (currentRow.LOCALESTOQUE)? null : ' ';

            widget.getField('DSLOCALESTOQ').reload();
        }
    }

    this.initialize = function(widget) {
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        this.getFilialLogada().then(success => {
            let dataAtual = moment().format('DD/MM/YYYY');
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            /*rowsToSet.INTERVALO = {
                                    "start": dataAtual,
                                    "end": dataAtual
                                  };
            rowsToSet.DTINICIAL = dataAtual;
            rowsToSet.DTFINAL = dataAtual; */
            
            var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                },
                {
                    "name": "IDUTILALMOX",
                    "operator": "=",
                    "value": "S"
                }
		    ];
            
            General.callRestEngine('est#/crud/paramfilial', 'FilterData', filter).then( response => {
                if(response.dataset.paramfilial[0]) {
                    widget.getField("DSALMOXARIFE").filterProperties.readOnly = false;
                    widget.getField("DSALMOXARIFE").validations = {"required": true};
                    if(response.dataset.paramfilial[0].IDUTILLCESTQ == 'S'){
                        widget.getField("DSLOCALESTOQ").filterProperties.readOnly = false;
                        widget.getField("DSLOCALESTOQ").validations = {"required": true};
                    } else{
                        widget.getField("DSLOCALESTOQ").filterProperties.readOnly = true;
                        widget.getField("DSLOCALESTOQ").validations = null;
                    }
                }else {
                    widget.getField("DSALMOXARIFE").filterProperties.readOnly = true;
                    widget.getField("DSALMOXARIFE").validations = null;
                    widget.getField("DSLOCALESTOQ").filterProperties.readOnly = true;
                    widget.getField("DSLOCALESTOQ").validations = null;
                }
                //rowsToSet
                ZhFilterPreferences.openFilterWidget(widget, []);
            });
            
            
        });       
        
        //funcao desnecessária basta colocar propiedade 'translate': true na coluna desejada.
       /* let fieldTipos = widget.getField('IDTIPOMOVIEST');
        let arrToTranslate = fieldTipos.dataSource.data;
        arrToTranslate.forEach(row =>{
            row.DSTIPOMOVIEST = ScreenService.i18n(row.DSTIPOMOVIEST);
        });*/
    };
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };
    
    function setDefaultValues(widget) {
        var data = moment().format('DD/MM/YYYY');

        widget.currentRow.INTERVALO =   {
                                            "start": data,
                                            "end": data
                                        };
        widget.currentRow.DTINICIAL = data;
        widget.currentRow.DTFINAL = data;
        widget.currentRow.REPORT_NAME = 'Est31200IrrEstoque';
        widget.currentRow.NRTPARVPROD = null;
        changeReport(widget);
    }
    
    this.setProductRangeOnFilter = function(args){
        var filter = args.e.preference.FILTER_ROW;
        var widget = args.owner;
        
        var prodfin = filter.CDPRODFIN;
        prodfin = prodfin + "zzzzzzzzzzzzzzzzzzzz";
        prodfin = prodfin.substring(0, 20);
        var prodIni = {name: "CDPRODINI", operator: "=", value: filter.CDPRODINI};
        var prodFin = {name: "CDPRODFIN", operator: "=", value: prodfin};
        widget.dataSourceFilter.push(prodIni);
        widget.dataSourceFilter.push(prodFin);
    };

    function changeReport(widget) {
        switch(widget.currentRow.REPORT_NAME) {
            case 'Est31200IrrEstoque':
                setEST31200DefaultValues(widget.currentRow); break;
        }
    }

    this.changeReport = changeReport;

    function setEST31200DefaultValues(currentRow) {
        currentRow.CDALMOXARIFE = ' ';
        currentRow.CDLOCALESTOQ = ' ';
    }

    this.setMinMaxDate = function(widget){
        var interval = widget.getField("INTERVALO").value();
        if(interval){
            widget.getField('DTINICIAL').setValue(interval.start);
            widget.getField('DTFINAL').setValue(interval.end);
        }
    };

    this.checkProductRange = function(field) {
        var currentRow = field.widget.currentRow;

        if(currentRow.CDPRODUTOINI && currentRow.CDPRODUTOFIN) {
            var produtoIni = (currentRow.CDPRODUTOFIN)? currentRow.CDPRODINI.replace(/[\.\-]/g, '') : null;
            var produtoFin = (currentRow.CDPRODUTOFIN)? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if(!General.comparaPratoProduto(produtoIni, produtoFin, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        }
    };

    this.changeFilter = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;

        var filtro= [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value": filterRow.CDFILIAL
            },
            {
                "name":"CDALMOXARIFE",
                "operator":"=",
                "value": filterRow.CDALMOXARIFE ? filterRow.CDALMOXARIFE : null
            },
            {
                "name":"CDLOCALESTOQ",
                "operator":"=",
                "value": filterRow.CDLOCALESTOQ ? filterRow.CDLOCALESTOQ : null
            },
            {
                "name":"DTINICIAL",
                "operator":"=",
                "value": filterRow.DTINICIAL ? filterRow.DTINICIAL : null
            },
            {
                "name":"DTFINAL",
                "operator":"=",
                "value": filterRow.DTFINAL ? filterRow.DTFINAL : null
            },
            {
                "name":"NRTPARVPROD",
                "operator":"=",
                "value": filterRow.NRTPARVPROD ? filterRow.NRTPARVPROD : null
            },
            {
                "name":"CDARVPRODINI",
                "operator":"=",
                "value": filterRow.CDARVPRODINI ? filterRow.CDARVPRODINI : null
            },
            {
                "name":"CDARVPRODFIN",
                "operator":"=",
                "value": filterRow.CDARVPRODFIN ? filterRow.CDARVPRODFIN : null
            },
            {
                "name":"NRARVOREPROD",
                "operator":"=",
                "value": filterRow.NRARVOREPROD ? filterRow.NRARVOREPROD : 'T'
            }
        ];
        
        widget.dataSourceFilter = filtro;
        widget.reload();
        
    };
    
    function getFilter(widget, fields, prodFin) {
        let row = widget.appliedFilter.FILTER_ROW;
        let filter = [];
        fields.forEach((f) => {
            filter.push(
            {
                "name": f,
                "operator": "=",
                "value": (f !== 'CDPRODUTOFIN') ? row[f] : getProdFinFormatado(row.CDPRODUTOFIN)
            });
        });
        return filter;
    }
    
    function getProdFinFormatado(prodfin) {
        var masksize = getLocalVar('CDPICTPROD').replace(/[\.\-]/g, '').length - prodfin.length;
        return General.fill(prodfin, masksize, true, 'z');
    }
    
    this.setGrupoArvore = function(widget){
		var nmarvoreprod = widget.getField('NMARVOREPROD');
		if (widget.getField('DSTPARVPROD').value()){
			nmarvoreprod.dataSourceFilter[0].value = widget.currentRow.NRTPARVPROD;
			nmarvoreprod.readOnly = false;
			nmarvoreprod.reload();
		}else{
			nmarvoreprod.dataSourceFilter[0].value = null;
			nmarvoreprod.readOnly = true;
		}
		widget.currentRow.NMTPARVORE = '';
		widget.currentRow.NMARVOREPROD = '';
		widget.currentRow.NRARVOREPROD = '';
	    this.validaEstadoRangeProduto(widget);
	};
    
    this.validaEstadoRangeProduto = function(widget){
		var produtoIni = widget.getField('NMPRODUTOINI');
		var produtoFin = widget.getField('NMPRODUTOFIN');
		var tipoProduto = widget.getField('DSTPARVPROD');
		if (tipoProduto.value()){
			produtoIni.readOnly = true;
			widget.currentRow.NMPRODUTOINI = null;
			widget.currentRow.CDARVPRODINI = null;
			widget.currentRow.CDPRODUTOINI = null;
			widget.currentRow.CDNVPRODUTOINI = null;
			produtoIni.validations = {};
			produtoFin.readOnly = true;
			produtoFin.validations = {};
			widget.currentRow.NMPRODUTOFIN = null;
			widget.currentRow.CDARVPRODFIN = null;
			widget.currentRow.CDPRODUTOFIN = null;
			widget.currentRow.CDNVPRODUTOFIN = null;
		}else{
			produtoIni.readOnly = false;
			produtoIni.validations = {"required":true};
			produtoFin.readOnly = false;
			produtoFin.validations = {"required":true};
		}
	};
	
	this.validaEstadoTipoArvore = function(widget){
		var produtoIni = widget.getField('NMPRODUTOINI'); 
		var produtoFin = widget.getField('NMPRODUTOFIN');
		var tipoProduto = widget.getField('DSTPARVPROD');
		var grupo = widget.getField('NMARVOREPROD');
		if (produtoIni.value() || produtoFin.value()){
			tipoProduto.readOnly = true;
			grupo.readOnly = true;
			widget.currentRow.NRTPARVPROD = null;
			widget.currentRow.DSTPARVPROD = null;
			widget.currentRow.NMTPARVORE = null;
			widget.currentRow.NMARVOREPROD = null;
			widget.currentRow.NRARVOREPROD = null;
		}else{
			tipoProduto.readOnly = false;
			grupo.readOnly = false;
		}
	};
	
	this.requestGrupoTipoArvore = function(widget, fieldTreeProd){
		ProdTreeGroup.getAllGroups(fieldTreeProd)
			.then(function(allGroups){
				widget.currentRow.NRARVOREPROD = allGroups.join('_');
			})
			.catch(function(erro){
				ScreenService.showMessage(erro.data.error);
			});
	};
	
	this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
    
    this.setNameLayout = function(args){
        var widget = args.owner;
        if (!widget.hasOwnProperty('labelPrincipal')){
            widget.labelPrincipal = widget.label;
        }
        if (args.e.layout.ID == 'ORIGINAL'){
            widget.container.label = widget.labelPrincipal;
            widget.label = widget.labelPrincipal;
        }else{
            widget.container.label = args.e.layout.LABEL_TO_PREFERENCE;
            widget.label = args.e.layout.LABEL_TO_PREFERENCE;
            ScreenService.showLoader();
            setTimeout((ok)=>{changeLayoutOrderby(args);}, 2000);
        }
    };
    
    var changeLayoutOrderby = function (args){
        
        var widget = args.owner;
        /*var orderByFilter = widget.dataSourceFilter.filter(filter => filter.operator == "ORDER_BY");
        if(orderByFilter)
            orderByFilter[0].name[0].name = orderby;
        */
        
        var orderByFilter = {name: [{name: orderby, direction: "ASC"}], value: null, operator: "ORDER_BY"};
        var filterToSet = widget.dataSourceFilter.filter(filter => filter.operator != "ORDER_BY");
        filterToSet.push(orderByFilter);
        widget.dataSourceFilter = filterToSet;
        widget.reload();
        ScreenService.hideLoader();
    };
    
    this.getDateFilter = function (interval){
        let value = EstUtil.getFieldValue('DTLANCMOVIEST','est31300_grid_movimentacao_de_estoque_grid', true);
        if(value){
            return value[interval];
        }
        return null;
    };
	
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est31300_gridMovimentacaoEstoque_Controller', Est31300_gridMovimentacaoEstoque_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est31700_Etiqueta_de_Produto_Controller.js
function Est31700_Etiqueta_de_Produto_Controller(ScreenService, restEngine, General, templateManager, ZhFilterPreferences, EstUtil) {


    this.validacoesWidgetOnEnterOnce = function(filtroArgs) {
        var cabecalhoWidget = filtroArgs.owner;
        const defaultFilter = {
            "DTENTRSAID": "01/01/2018",
            "IDENTRSAIDOP": "E",
            "FILTRADOPOR": "",
            "CDFILIAL": "",
            "NRORG": ""
        };
        General.callRestEngine("est#/crud/filial", "FilterData", [{ name: "CDFILIAL", operator: "=", value: "FILIAL_LOGADA" }]).then((response) => {
                defaultFilter.CDFILIAL = response.dataset.filial[0].CDFILIAL;
                defaultFilter.NMFILIAL = response.dataset.filial[0].NMFILIAL;
                ZhFilterPreferences.setOriginalFilter(cabecalhoWidget.id, defaultFilter);
                ZhFilterPreferences.openFilterWidget(cabecalhoWidget, defaultFilter);
            },
            (error) => {
                ZhFilterPreferences.openFilterWidget(cabecalhoWidget);
            });
    };

    this.removeCaposFiltrer = function(widget, preference) {

        let filterRow = preference.FILTER_ROW;
        let filter = widget.dataSourceFilter;
        let newDataFilter = [];

        filter = [
            {
                name: "CDFILIAL",
                operator: "=",
                value: filterRow.CDFILIAL
            }
        ];

        switch (filterRow.FILTRADOPOR) {
            case "NF":
            case "LNF":
                newDataFilter = [
                    {
                        name: "NRLANCTONF",
                        operator: "=",
                        value: filterRow.NUMERO
                        },
                    {
                        name: "IDENTRSAIDOP",
                        operator: "=",
                        value: filterRow.IDENTRSAIDOP
                        },
                    {
                        name: "IDORIGDESTNF",
                        operator: "=",
                        value: filterRow.IDORIGDESTNF
                        }
                    ];

                filter = filter.concat(newDataFilter);

                break;
            case "LEst":
                newDataFilter = {
                    name: "NRLANCESTQ",
                    operator: "=",
                    value: filterRow.NUMERO
                };

                filter.push(newDataFilter);

                break;
            case "OP":
                newDataFilter = [
                    {
                        name: "NRORDPROD",
                        operator: "=",
                        value: filterRow.NUMERO
                    },
                    {
                        name: "NRSUBORDPROD",
                        operator: "=",
                        value: filterRow.NRSUBORDPROD
                    }
                ];

                filter = filter.concat(newDataFilter);

                break;
        }

        widget.dataSourceFilter = filter;
        widget.reload();
    };

    this.getFieldValueFilter = function(filedName, widgetName) {
        var container = templateManager.container;
        if (container) {
            var filterWidget;
            var filterBforeApply = templateManager.container.getActiveWidget();
            if (filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                filterWidget = filterBforeApply;
            else {
                var f = container.getWidget(widgetName);
                if (f) {
                    filterWidget = container.getWidget(widgetName).appliedFilter;
                }

            }

            if (filterWidget) {
                var valueField;
                if (filterBforeApply.name == 'zhFilterPreferencesSelectConditions')
                    valueField = filterWidget.currentRow[filedName];
                else
                    valueField = filterWidget.FILTER_ROW[filedName];

                return valueField || null;
            }
            else return null;
        }
        else return null;
    };

    /****************************************************************
     * Ao escolher as opções “Nº Nota Fiscal” ou “Nº Lançamento de NF” deverá ser 
     * apresentado na pesquisa de número os seguintes campos:
     * Número NF, Série, Data Entrada, Razão Social
     * Quando a pesquisa for por “Nº Lançamento de Estoque” deverá ser exibibido os campos:
     * Número, Data Lançamento
     * ***************************************************************/
    this.HabilitaCamposNumero = function(widget) {
        let fieldsBool = [];
        let fieldNumero = widget.getField("NUMERO");

        switch (widget.currentRow.FILTRADOPOR) {

            case "NF":

                fieldNumero.dataSource.name = "est_relatorios#/crud/numero_nota_fiscal";
                fieldNumero.searchableFields = ["NRNOTAFISC", "SERIE", "NMRAZSOCFORN"];
                fieldNumero.displayedFields = ["NRNOTAFISC", "SERIE", "NMRAZSOCFORN"];
                fieldNumero.dataSourceFilter[2].name = "DTENTRSAID";

                fieldsBool = [
                    { "name": "NRNOTAFISC", "value": false },
                    { "name": "NRLANCTONF", "value": true },
                    { "name": "NRLANCESTQ", "value": false },
                    { "name": "NRORDPROD", "value": false },
                    { "name": "NRSUBORDPROD", "value": false },
                    { "name": "SERIE", "value": true },
                    { "name": "DATA", "value": true },
                    { "name": "NMRAZSOCFORN", "value": true },
                    { "name": "IDENTRSAIDOP", "value": false }
                ];

                this.checkFieldsVisibility(fieldsBool, fieldNumero);
                break;
            case "LNF":

                fieldNumero.dataSource.name = "est_relatorios#/crud/numero_lancamento_nf";
                fieldNumero.searchableFields = ["NRLANCTONF", "SERIE", "DATA"];
                fieldNumero.displayedFields = ["NRLANCTONF", "SERIE", "DATA"];
                fieldNumero.dataSourceFilter[2].name = "DTENTRSAID";

                fieldsBool = [
                    { "name": "NRNOTAFISC", "value": false },
                    { "name": "NRLANCTONF", "value": true },
                    { "name": "NRLANCESTQ", "value": false },
                    { "name": "NRORDPROD", "value": false },
                    { "name": "NRSUBORDPROD", "value": false },
                    { "name": "SERIE", "value": true },
                    { "name": "DATA", "value": true },
                    { "name": "NMRAZSOCFORN", "value": false },
                    { "name": "IDENTRSAIDOP", "value": false }
                ];

                this.checkFieldsVisibility(fieldsBool, fieldNumero);
                break;
            case "LEst":

                fieldNumero.dataSource.name = "est_relatorios#/crud/numero_lancamento_estoque";
                fieldNumero.searchableFields = ["NRLANCESTQ", "DATA"];
                fieldNumero.displayedFields = ["NRLANCESTQ", "DATA"];
                fieldNumero.dataSourceFilter[2].name = "DTLANCESTQ";

                fieldsBool = [
                    { "name": "NRNOTAFISC", "value": false },
                    { "name": "NRLANCTONF", "value": false },
                    { "name": "NRLANCESTQ", "value": true },
                    { "name": "NRORDPROD", "value": false },
                    { "name": "NRSUBORDPROD", "value": false },
                    { "name": "SERIE", "value": false },
                    { "name": "DATA", "value": true },
                    { "name": "NMRAZSOCFORN", "value": false },
                    { "name": "IDENTRSAIDOP", "value": false }
                ];

                this.checkFieldsVisibility(fieldsBool, fieldNumero);
                break;
            case "OP":

                fieldNumero.dataSource.name = "est_relatorios#/crud/numero_ordem_producao";
                fieldNumero.searchableFields = ["NRORDPROD", "DATA"];
                fieldNumero.displayedFields = ["NRORDPROD", "DATA"];
                fieldNumero.dataSourceFilter[2].name = "DTHRINICIAOP";

                fieldsBool = [
                    { "name": "NRNOTAFISC", "value": false },
                    { "name": "NRLANCTONF", "value": false },
                    { "name": "NRLANCESTQ", "value": false },
                    { "name": "NRORDPROD", "value": true },
                    { "name": "NRSUBORDPROD", "value": false },
                    { "name": "SERIE", "value": false },
                    { "name": "DATA", "value": true },
                    { "name": "NMRAZSOCFORN", "value": false },
                    { "name": "IDENTRSAIDOP", "value": false }
                ];

                this.checkFieldsVisibility(fieldsBool, fieldNumero);

                break;
            default:
                fieldNumero.readOnly = true;
        }
    };

    this.checkFieldsVisibility = function(fieldsObj, numero) {

        if (numero.selectWidget) {
            fieldsObj.forEach((field) => {
                numero.selectWidget.getField(field.name).showOnList = field.value;
            });
        }
        else {
            fieldsObj.forEach((field) => {
                numero.getField(field.name).showOnList = field.value;
            });
        }
    };

    this.HabilitaNumero = function(widget) {
        if ((widget.currentRow.FILTRADOPOR == null || widget.currentRow.FILTRADOPOR == undefined || widget.currentRow.FILTRADOPOR == "") || (widget.currentRow.IDENTRSAIDOP == null || widget.currentRow.IDENTRSAIDOP == undefined || widget.currentRow.IDENTRSAIDOP == "") || (widget.currentRow.DTENTRSAID == null || widget.currentRow.DTENTRSAID == undefined || widget.currentRow.DTENTRSAID == "") || (widget.currentRow.NMFILIAL == null || widget.currentRow.NMFILIAL == undefined || widget.currentRow.NMFILIAL == "")) {
            widget.getField("NUMERO").readOnly = true;
        }
        else {
            widget.getField("NUMERO").readOnly = false;
        }
        widget.currentRow.NUMERO = '';
        widget.getField("NUMERO").reload();
    };
    
    this.gerarEtiqueta = function (widget) {
       let rows = widget.getCheckedRows();
       let ignoresProperties = ["__is_new" , "__isSelected"];
       let produtos = [];
       
       rows.forEach( function(row){
           produtos.push(Util.clone(row, ignoresProperties));
       } );
    
       if(widget.hasCheckedRows()){
           this.requestReport(produtos).then(function(birtURL){
               window.open(birtURL.dataset.URL[0]);
           });
       }
    };

    this.requestReport = function(rows) {
        return restEngine.post('est_relatorios#/etiqueta_De_Produto', {
            "requestType": "DataSet",
            "dataset": rows
        });
    };

    this.onChangeFilial = function(widget) {
        var numero = widget.getField("NUMERO");

        numero.dataSourceFilter[1] = {
            "name": "CDFILIAL",
            "operator": "=",
            "value": widget.currentRow.CDFILIAL
        };
    };

    this.onChangeData = function(widget) {
        var numero = widget.getField("NUMERO");
        var data = widget.getField("DTENTRSAID");

        numero.dataSourceFilter[2] = {
            "name": "DTENTRSAID",
            "operator": "=",
            "value": data.value()
        };
    };

    this.onChangeId = function(widget) {
        var numero = widget.getField("NUMERO");
        var id = widget.getField("IDENTRSAIDOP");

        numero.dataSourceFilter[0] = {
            "name": "IDENTRSAIDOP",
            "operator": "=",
            "value": id.value()
        };
    };

    this.initializeNumero = function(args) {
        let widget = args.owner.field.widget;
        let numero = widget.getField("NUMERO");
        let complementaryData = [];

        numero.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": widget.currentRow.CDFILIAL
            },
            {
                "name": "IDENTRSAIDOP",
                "operator": "=",
                "value": widget.getField("IDENTRSAIDOP").value()
            },
            {
                "name": "DTENTRSAID",
                "operator": "=",
                "value": widget.getField("DTENTRSAID").value()
            }

        ];

        switch (widget.currentRow.FILTRADOPOR) {
            case "NF":

                complementaryData = {
                    "name": "NRNOTAFISC",
                    "operator": "=",
                    "value": ""
                };

                numero.dataSourceFilter.push(complementaryData);
                this.checkFieldOutData(numero, "NRLANCTONF");

                break;
            case "LNF":
                complementaryData = {
                    "name": "NRLANCTONF",
                    "operator": "=",
                    "value": ""
                };

                numero.dataSourceFilter.push(complementaryData);
                this.checkFieldOutData(numero, "NRLANCTONF");

                break;
            case "LEst":
                numero.dataSourceFilter.pop();
                complementaryData = [
                    {
                        "name": "NRLANCESTQ",
                        "operator": "=",
                        "value": ""
                    },
                    {
                        "name": "DTLANCESTQ",
                        "operator": "=",
                        "value": widget.getField("DTENTRSAID").value()
                    }
                ];
                numero.dataSourceFilter = numero.dataSourceFilter.concat(complementaryData);
                this.checkFieldOutData(numero, "NRLANCESTQ");

                break;
            case "OP":
                numero.dataSourceFilter = numero.dataSourceFilter.filter((data) => {
                    return data.name == 'CDFILIAL';
                });

                complementaryData = [
                    {
                        "name": "NRORDPROD",
                        "operator": "=",
                        "value": ""
                    },
                    {
                        "name": "NRSUBORDPROD",
                        "operator": "=",
                        "value": ""
                    },
                    {
                        "name": "DTHRINICIAOP",
                        "operator": "=",
                        "value": widget.getField("DTENTRSAID").value()
                    }
                ];

                numero.dataSourceFilter = numero.dataSourceFilter.concat(complementaryData);
                this.checkFieldOutData(numero, "NRORDPROD");

                break;
        }
        numero.reload();
    };

    this.checkFieldOutData = function(numero, fieldName) {
        delete numero.outData.NRNOTAFISC;
        delete numero.outData.NRLANCTONF;
        delete numero.outData.NRLANCESTQ;
        delete numero.outData.NRORDPROD;
        delete numero.outData.NRSUBORDPROD;
        delete numero.outData.IDORIGDESTNF;

        numero.outData[fieldName] = "NUMERO";

        if (fieldName == 'NRORDPROD') {
            fieldName = "NRSUBORDPROD";
            numero.outData[fieldName] = "NRSUBORDPROD";
        }
        if (fieldName == 'NRNOTAFISC' || fieldName == 'NRLANCTONF') {
            fieldName = "IDORIGDESTNF";
            numero.outData[fieldName] = "IDORIGDESTNF";
        }
    };
}
Configuration(function(ContextRegister) {
    ContextRegister.register('Est31700_Etiqueta_de_Produto_Controller', Est31700_Etiqueta_de_Produto_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est32000_GiroDeEstoque_Controller.js
function Est32000_GiroDeEstoque_Controller(ZhFilterPreferences, ScreenService, General, templateManager, ProductField, ProdTreeGroup, localStorageEngine, Est32000_GiroDeEstoque_Service) {

	this.enableDisableData = function(widget) {
		var PARAM = widget.currentRow.PARAM;

		if(PARAM == 'D'){
			widget.getField('NRDIAS').readOnly    = false;
			widget.getField('DATAINI').readOnly   = true;
			widget.getField('DATAFIN').readOnly   = true;
			widget.getField('INTERVALO').readOnly = true;
			
			widget.getField('NRDIAS').validations = {"min": {"limit": 1}, "max": {"value": 999}};
			widget.getField('DATAINI').validations = {};
			widget.getField('DATAFIN').validations = {};
			widget.getField('INTERVALO').validations = {};
			
		}else if(PARAM == 'P'){
			widget.getField('NRDIAS').readOnly = true;
			widget.getField('DATAINI').readOnly = false;
			widget.getField('DATAFIN').readOnly = false;
			widget.getField('INTERVALO').readOnly = false;
			
			widget.getField('NRDIAS').validations = {};
			widget.getField('DATAINI').validations = {"required": true};
			widget.getField('DATAFIN').validations = {"required": true};
		}
		
		widget.getField('NRDIAS').reload();
		widget.getField('DATAINI').reload();
		widget.getField('DATAFIN').reload();
		widget.getField('INTERVALO').reload();
		widget.redraw();
	
	};

	this.setNameLayout = function(args){
        var widget = args.owner;
        if (!widget.hasOwnProperty('labelPrincipal')){
            widget.labelPrincipal = widget.label;
        }
        if (args.e.layout.ID == 'ORIGINAL'){
            widget.container.label = widget.labelPrincipal;
            widget.label = widget.labelPrincipal;
        }else{
            widget.container.label = args.e.layout.LABEL_TO_PREFERENCE;
            widget.label = args.e.layout.LABEL_TO_PREFERENCE;
        }
    };
    
    this.openFilter = function(widget){
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        this.getFilialLogada().then(function(success) {
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            rowsToSet.PARAM = 'D';
            rowsToSet.DADOSPOR = '1';
            rowsToSet.ORDENA = '0';
            
            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(function(){
                templateManager.container.getActiveWidget().getAction('close').isVisible = false;
                this.changeFilial(widget);
            }.bind(this));
        }.bind(this));
    };
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    }; 
	
	this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
	
	//Parâmetros Filtro
    this.changeFilter = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        var filtro = [];
        
        var mascara = 20 - filterRow.CDARVPRODFIN.length;
		var produtoFinal = filterRow.CDARVPRODFIN + 'z'.repeat(mascara).substring(0, mascara);
		
		if(filterRow.DATAINI && filterRow.DATAFIN && !filterRow.NRDIAS){
		    
		    var dtini = filterRow.DATAINI.split('/').reverse().join('/');
		    var dtfin = filterRow.DATAFIN.split('/').reverse().join('/');
		    
		    var ini = new Date(dtini); // Data ini
            var fin = new Date(dtfin); // Data fin
            var diff = Math.abs(fin.getTime() - ini.getTime()); // Subtrai uma data pela outra
            var days = Math.ceil(diff / (1000 * 60 * 60 * 24)); // Divide o total pelo total de milisegundos correspondentes a 1 dia. (1000 milisegundos = 1 segundo).
            
            filterRow.NRDIAS = days;
		}
        
        filtro= [
            {
                "name": "CDFILIAL",
                "operator":"=",
                "value": filterRow.CDFILIAL
            },
            {
                "name":"CDALMOXARIFE",
                "operator":"=",
                "value": filterRow.CDALMOXARIFE ? filterRow.CDALMOXARIFE : null
            },
            {
                "name":"CDLOCALESTOQ",
                "operator":"=",
                "value": filterRow.CDLOCALESTOQ ? filterRow.CDLOCALESTOQ : null
            },
            {
                "name":"CDPRODINI",
                "operator":"=",
                "value": filterRow.CDARVPRODINI ? filterRow.CDARVPRODINI : null
            },
            {
                "name":"CDPRODFIN",
                "operator":"=",
                "value": produtoFinal
            },
            {
                "name":"NRDIAS",
                "operator":"=",
                "value": filterRow.NRDIAS ? filterRow.NRDIAS : null
            },
            {
                "name": "ORDENA",
                "operator": "=",
                "value": filterRow.ORDENA
            },
            {
                "name": "PARAM",
                "operator": "=",
                "value": filterRow.PARAM
            },
            {
                "name": "DATAINI",
                "operator": "=",
                "value": filterRow.DATAINI ? filterRow.DATAINI : null
            },
            {
                "name": "DATAFIN",
                "operator": "=",
                "value": filterRow.DATAFIN ? filterRow.DATAFIN : null
            },
            {
                "name": "DADOSPOR",
                "operator": "=",
                "value": filterRow.DADOSPOR
            }
        ];
        
        widget.dataSourceFilter = filtro;
        widget.reload();
        
    };
    
    // Habilita/Desabilita Almoxarifado
    this.changeFilial = function (widget) {
        var cdfilial = widget.currentRow.CDFILIAL;

        if(cdfilial) {
            Est32000_GiroDeEstoque_Service.getParamFilial(cdfilial).then(function (paramfilial) {
                widget.getField("DSALMOXARIFE").readOnly = !((paramfilial.dataset.paramfilial[0].IDCTRLESTQ == 'S') && (paramfilial.dataset.paramfilial[0].IDUTILALMOX  == 'S'));
                
                widget.getField("DSALMOXARIFE").dataSourceFilter[0].value = cdfilial;
                widget.getField("DSALMOXARIFE").reload();
                widget.getField("CDALMOXARIFE").reload();
                
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }else{
            widget.getField("DSALMOXARIFE").readOnly = true;
            widget.getField("DSLOCALESTOQ").readOnly = true;
        }
        widget.currentRow.DSALMOXARIFE = '';
        widget.currentRow.DSLOCALESTOQ = '';
        widget.currentRow.CDALMOXARIFE = '';
        widget.currentRow.CDLOCALESTOQ = '';
    };

    // Habilita/Desabilita Localização
    this.changeAlmoxarifado = function (args) {
        var cdfilial     = args.row.CDFILIAL;
        var cdalmoxarife = args.row.CDALMOXARIFE;
        if(cdfilial && cdalmoxarife) {
            Est32000_GiroDeEstoque_Service.getParamFilial(cdfilial).then(function (paramfilial) {
                args.owner.field.widget.getField("DSLOCALESTOQ").readOnly = !((paramfilial.dataset.paramfilial[0].IDCTRLESTQ == 'S') && (paramfilial.dataset.paramfilial[0].IDUTILLCESTQ == 'S'));
                
                args.owner.field.widget.getField("DSLOCALESTOQ").dataSourceFilter[0].value = cdfilial;
                args.owner.field.widget.getField("DSLOCALESTOQ").dataSourceFilter[1].value = cdalmoxarife;
                
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }else{
            args.owner.field.widget.getField("DSLOCALESTOQ").readOnly = true;
        }
        args.row.DSLOCALESTOQ = '';
        args.row.CDLOCALESTOQ = '';
    };
    
    this.changeFilterLancEstoq = function(args){
        var cdfilial = args.owner.field.widget.currentRow.CDFILIAL;   
        var dtlancestq = args.owner.field.widget.currentRow.DTLANCESTQ;   
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[0].value = cdfilial;
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[1].value = dtlancestq;
        if(dtlancestq != ""){
            args.owner.field.widget.getField('NRLANCESTQ').reload();
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/crud/itlanctoest_produto";
        }else{
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/produto_posestatu_paramfilial";
        }
    };
    
    this.changeFilterLancEstoqByFilial = function(args){
        var cdfilial = args.owner.field.widget.currentRow.CDFILIAL;   
        var dtlancestq = args.owner.field.widget.currentRow.DTLANCESTQ;   
        
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[0].value = cdfilial;
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[1].value = dtlancestq;
        args.owner.field.widget.currentRow.NRLANCESTQ = "";
        if(dtlancestq != ""){
            args.owner.field.widget.getField('NRLANCESTQ').reload();
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/crud/itlanctoest_produto";
        }else{
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/produto_posestatu_paramfilial";
        }
    };
    
    this.validaCamposVisiveisGrid = function(widget, preference){
        var filterPopup = preference.FILTER_ROW;
        
        if(filterPopup.DADOSPOR == '1'){ // QTDE
            widget.getField('QTESTOQDIA').isVisible = true;
            widget.getField('QTLANCTOEST').isVisible = true;
            widget.getField('VRESTOQDIA').isVisible = false;
            widget.getField('VRTOTLANCTO').isVisible = false;
        }else{ // VALOR
            widget.getField('QTESTOQDIA').isVisible = false;
            widget.getField('QTLANCTOEST').isVisible = false;
            widget.getField('VRESTOQDIA').isVisible = true;
            widget.getField('VRTOTLANCTO').isVisible = true;
        }
    };
    
    this.setTotalizadores = function(widget){
        /*var qtdeProdutos = widget.dataSource.data.length;
        
        var filterPopup = widget.appliedFilter.FILTER_ROW;
        var totQTESTOQDIA = parseFloat(0);
        var totQTLANCTOEST = parseFloat(0);
        var totVRESTOQDIA = parseFloat(0);
        var totVRTOTLANCTO = parseFloat(0);
        var totDIASRESTANTES = parseFloat(0);
        
        widget.dataSource.data.forEach(row => {
            totQTESTOQDIA += parseFloat(row.QTESTOQDIA);
            totQTLANCTOEST += parseFloat(row.QTLANCTOEST);
            totVRESTOQDIA += parseFloat(row.VRESTOQDIA);
            totVRTOTLANCTO += parseFloat(row.VRTOTLANCTO);
            totDIASRESTANTES += parseFloat(row.DIASRESTANTES);
        });
        var mediaDias = totDIASRESTANTES / qtdeProdutos;
            
        if(filterPopup.DADOSPOR == '1'){
            widget.dataSource.data.push({"QTESTOQDIA": totQTESTOQDIA, "QTLANCTOEST": totQTLANCTOEST, "DIASRESTANTES": NaN, "CDLOCALESTOQ": "Totais"});
        } else if(filterPopup.DADOSPOR == '2'){
            widget.dataSource.data.push({"VRESTOQDIA": totVRESTOQDIA, "VRTOTLANCTO": totVRTOTLANCTO, "DIASRESTANTES": NaN, "CDLOCALESTOQ": "Totais"});
        }
            
        widget.dataSource.data.push({"QTESTOQDIA": NaN, "QTLANCTOEST": NaN, 
                                    "VRESTOQDIA": NaN, "VRTOTLANCTO": NaN, 
                                    "DIASRESTANTES": mediaDias, "CDLOCALESTOQ": "Nº médio de dias"});
        
        
        if(filterPopup.DADOSPOR == '1'){
            var qtdeMediaDia = (totQTESTOQDIA/totQTLANCTOEST)*filterPopup.NRDIAS;
            widget.dataSource.data.push({"QTESTOQDIA": NaN, "QTLANCTOEST": NaN, 
                                    "VRESTOQDIA": NaN, "VRTOTLANCTO": NaN, 
                                    "DIASRESTANTES": qtdeMediaDia, "CDLOCALESTOQ": "Qtde. média dia"});
        } else if(filterPopup.DADOSPOR == '2'){
            var vrMediaDia = (totVRESTOQDIA/totVRTOTLANCTO)*filterPopup.NRDIAS;
            widget.dataSource.data.push({"QTESTOQDIA": NaN, "QTLANCTOEST": NaN, 
                                    "VRESTOQDIA": NaN, "VRTOTLANCTO": NaN, 
                                    "DIASRESTANTES": vrMediaDia, "CDLOCALESTOQ": "Valor médio dia"});
        }
        */
    };
    
    this.intervaloChange = function(field){
		let widget = field.widget;
		widget.currentRow.DATAINI = widget.currentRow.INTERVALO.start;
		widget.currentRow.DATAFIN = widget.currentRow.INTERVALO.end;
	};
    
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est32000_GiroDeEstoque_Controller', Est32000_GiroDeEstoque_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est32100_EvolucaoDeCusto_Controller.js
function Est32100_EvolucaoDeCusto_Controller(restEngine, General, ScreenService, Est32100_EvolucaoDeCusto_Service, ProductField, templateManager) {
   
    this.checkUnidadeFields = function(widget) {
        var currentRow = widget.currentRow;

        ProductField.updateProductFilters(widget);
    };

    this.setDefaultValues = function(currentRow, formWidget, first) {
        var data        = moment().format('DD/MM/YYYY');
        // var primeiroDia = moment(data, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY');
        // var ultimoDia   = moment(data, 'DD/MM/YYYY').endOf('month').format('DD/MM/YYYY');

        currentRow.DTPERINI = data;
        currentRow.DTPERFIN = data;
        
        if (!currentRow.INTERVALO) {
            currentRow.INTERVALO = {};
        }
        currentRow.INTERVALO.start = data;
        currentRow.INTERVALO.end = data;
        
        if(first) {
            this.getFilialLogada().then(function(success) {
                currentRow.CDFILIAL = success.dataset.filial[0].CDFILIAL;
                currentRow.NMFILIAL = success.dataset.filial[0].NMFILIAL;
                currentRow.DTPERINI = moment().format('DD/MM/YYYY');
                currentRow.DTPERFIN = moment().format('DD/MM/YYYY');
                var widget = templateManager.container.getActiveWidget();
                ProductField.updateProductFilters(widget);
            }.bind(this));
        }
        
        
    };
    
     /* **************************************************
     * Procedimento que checa a validade de algum valor
     * percentual. Possui mensagens padrão para campos
     * específicos.
     *
     * @param Field field
     */
    this.checkAdmittedPercentage = function(field) {
        var value = field.widget.currentRow[field.name];

        //parametrized error messages
        var msgs = {
            VRPERCINVROT: "Percentual deve estar entre 0 e 100%.",
            VRPETOLEENTR: "Percentual deve estar entre 0 e 100%.",
            VRPETOLEINVROT: "Percentual deve estar entre 0 e 100%.",
            DEFAULT: "Valor percentual deve estar entre 0 e 100%."
        };

        //in case of invalid percentage
        if(value !== null && (0 > value || value > 100)) {
            //cleans invalid data
            field.widget.currentRow[field.name] = null;

            //picks error message
            var msg = (msgs[field.name])? msgs[field.name] : msgs.DEFAULT;

            ScreenService.showMessage(msg);
        }
    };

    
	this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
	
    this.setMinMaxDate = function(widget, MaxOrMin){
        if(MaxOrMin === 'MAX') {
            var maxDate = widget.getField('DTPERFIN').value();
            var fieldInicial = widget.getField('DTPERINI');
            fieldInicial.maxDate = maxDate;
        } else {
            var minDate = widget.getField('DTPERINI').value();
            var fieldFinal = widget.getField('DTPERFIN');
            fieldFinal.minDate = minDate;
        }
    };

    this.validateReport = function(widget, format) {
        var currentRow  = widget.currentRow;
        currentRow.__format = format;
        var compproduto = (currentRow.CDPRODFIN)? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

        if (widget.isValid()) {
            if(General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('GER_CDPICTPROD'))) {
                Est32100_EvolucaoDeCusto_Service.requestReport(currentRow).then(function(birtURL){
                    window.open(birtURL.dataset.URL[0]);
                });
            } else {
                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        }
    };

    this.fieldOnChangeRelatorio = function(widget) {
        if(widget.currentRow.REPORT_NAME === 'Est32100EvolCust') {
            widget.getField('PERMINIMO').readOnly           = true;
            widget.getField('PERMINIMO').validations        = {};
            widget.currentRow.PERMINIMO                     = '';
        }
        else if(widget.currentRow.REPORT_NAME === 'Est32101PercentMinimo') {
            widget.getField('PERMINIMO').readOnly          = false;
            widget.getField('PERMINIMO').validations       = {"required": true};

        }
        this.setDefaultValues(widget.currentRow, widget, false);
        widget.reload();
    };
    
    this.intervaloChange = function(field){
		let widget = field.widget;
		widget.currentRow.DTPERINI = widget.currentRow.INTERVALO.start;
		widget.currentRow.DTPERFIN = widget.currentRow.INTERVALO.end;
	};
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est32100_EvolucaoDeCusto_Controller', Est32100_EvolucaoDeCusto_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est32200_ProdutosSemMovimentacao_Controller.js
function Est32200_ProdutosSemMovimentacao_Controller(ProdTreeGroup, templateManager, General, ScreenService, Est32200_ProdutosSemMovimentacao_Service, ProductField, ZhFilterPreferences, EstUtil) {

    // Realiza a limpeza de valores do filtro caso a filial seja mudada
    this.clearFieldsOnChangeFilial = function(widget) {
        const fields = [
                "DSALMOXARIFE",
                "DSLOCALESTOQ",
                "NRDIAS",
                "NMPRODINI",
                "NMPRODFIN",
                "DSTPARVPROD",
                "NRARVOREPROD"
            ];

        fields.forEach(field => {
            let fieldToClear = widget.getField(field);
            fieldToClear.clearValue("");

            if (fieldToClear.name === "DSALMOXARIFE" || fieldToClear.name === "DSLOCALESTOQ" ||
                fieldToClear.name === "NMPRODINI" || fieldToClear.name === "NMPRODFIN") {
                fieldToClear.reload();
            }
        });
    };

    this.verAlmoxLocal = function(widget) {
        var currentRow = widget.currentRow;

        ProductField.updateProductFilters(widget);
        if (currentRow.CDFILIAL) {
            Est32200_ProdutosSemMovimentacao_Service.setParamFilial(currentRow.CDFILIAL).then(function(success) {
                this.setAlmoxField(widget);
                this.setLocalField(widget);
            }.bind(this), function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
        else {
            currentRow.ALMOXARIFADO = false;
            currentRow.LOCALESTOQUE = false;

            widget.getField('DSALMOXARIFE').readOnly = true;
            widget.getField('DSLOCALESTOQ').readOnly = true;

            widget.getField('DSALMOXARIFE').validations.required = null;
            widget.getField('DSLOCALESTOQ').validations.required = null;

        }
    };

    this.setAlmoxField = function(widget) {
        const utilizaAlmox = Est32200_ProdutosSemMovimentacao_Service.utilizaAlmox();
        const fieldAmox = widget.getField('DSALMOXARIFE');
        const fieldLocal = widget.getField('DSLOCALESTOQ');

        if (utilizaAlmox) {
            fieldAmox.dataSourceFilter[0] = { "name": "CDFILIAL", "operator": "=", "value": widget.currentRow.CDFILIAL };
            fieldAmox.readOnly = false;
            fieldAmox.validations.required = true;
        }
        else {
            fieldAmox.readOnly = true;
            fieldAmox.validations.required = null;
            fieldAmox.setValue(' ');
            widget.currentRow.CDALMOXARIFE = ' ';

            fieldLocal.readOnly = true;
            fieldLocal.validations.required = null;
            fieldLocal.setValue(' ');
            widget.currentRow.CDLOCALESTOQ = ' ';
        }
    };

    this.setLocalField = function(widget) {
        var utilizaLocal = Est32200_ProdutosSemMovimentacao_Service.utilizaLocal();
        var fieldLocal = widget.getField('DSLOCALESTOQ');

        if (utilizaLocal) {
            const filter = [
                { "name": "CDFILIAL", "operator": "=", "value": widget.currentRow.CDFILIAL },
                { "name": "CDALMOXARIFE", "operator": "=", "value": widget.currentRow.CDALMOXARIFE },
                { "name": "CDLOCALESTOQ", "operator": "<>", "value": " " }];

            fieldLocal.dataSourceFilter = filter;
            fieldLocal.validations.required = true;
            fieldLocal.readOnly = false;
        }
        else {
            fieldLocal.validations.required = null;
            fieldLocal.readOnly = true;
            widget.currentRow.CDLOCALESTOQ = " ";
        }
    };

    //filter est32200_produtos_sem_movimentacao
    this.setDefault = function(widget) {
        this.setFieldsGerais(widget);
        this.getFilialLogada().then(success => {
            widget.currentRow.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            widget.currentRow.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            widget.currentRow.ORDENACAO = '0';
            widget.currentRow.IDTIPOMOVI = 'T';

            widget.currentRow.CDFILIAL = success.dataset.filial[0].CDFILIAL;

            this.verAlmoxLocal(widget);

        });

    };

    this.getFilialLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.setFieldsGerais = function(widget) {
        templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
    };

    this.clonaFiltro = function(widget) {
        if (widget.appliedFilter) {
            widget.filterReport = Util.clone(widget.appliedFilter.FILTER_ROW);
            widget.filterReport.NRDIAS = widget.appliedFilter.FILTER_ROW.DTLIMITE;
        }
        else {
            var filtro = widget.container.getActiveWidget();
            widget.filterReport = Util.clone(filtro.currentRow);
            widget.filterReport.NRDIAS = filtro.currentRow.DTLIMITE;
        }
    };

    this.setDtlimite = function(widget) {
        var dtlimite = Est32200_ProdutosSemMovimentacao_Service.getDtlimite(widget.appliedFilter.FILTER_ROW.DTLIMITE);

        for (var i = 0; i < widget.dataSourceFilter.length; i++) {
            if (widget.dataSourceFilter[i].name === 'DTLIMITE') {
                widget.filterReport.NRDIAS = widget.dataSourceFilter[i].value;
                widget.filterReport.DTLIMITE = dtlimite;
                widget.dataSourceFilter.splice(i, 1);
                this.setDtlimite(widget);
            }
            else if (widget.dataSourceFilter[i].name === 'CDPRODFIN') {
                var mascara = getLocalVar('CDPICTPROD');
                mascara = mascara.split('.');
                var mascaraSemPonto = "";
                for (var index in mascara) {
                    mascaraSemPonto = mascaraSemPonto + mascara[index];
                }
                mascara = mascaraSemPonto.length;
                var tamanhoValue = widget.dataSourceFilter[i].value.length;
                mascara = mascara - tamanhoValue;
                var maximo = 'z';
                widget.dataSourceFilter[i].value = widget.dataSourceFilter[i].value + maximo.repeat(mascara);
                widget.filterReport.CDPRODFIN = widget.dataSourceFilter[i].value;
            }
            else if (widget.dataSourceFilter[i].name === 'IDTIPOMOVI') {
                widget.dataSourceFilter.splice(i, 1);
                this.setDtlimite(widget);
            }
        }
    };

    this.validateFilterFields = function(currentRow) {
        if (currentRow.CDPRODFIN) {
            var cdprodfin = currentRow.CDPRODFIN.replace(/[^0-9]/g, '');
            var mascara = 20 - cdprodfin.length;
            currentRow.CDPRODFIN = (currentRow.CDPRODFIN + 'z'.repeat(mascara)).substring(0, mascara);
        }
    };

    this.gerarRelatorio = function(format) {
        var widget = templateManager.container.getWidget('est32200_produtos_sem_movimentacao_grid');
        var currentRow = widget.currentRow;

        //TODO MUDAR ESTA FUNCÇÂO
        var birthFilter = Est32200_ProdutosSemMovimentacao_Service.gerarRelatorio(currentRow);

        if (!birthFilter.NRARVOREPROD) {
            birthFilter.NRARVOREPROD = '';
        }
        if (!birthFilter.NRTPARVPROD) {
            birthFilter.NRTPARVPROD = '';
        }

        birthFilter.DTLIMITE = Est32200_ProdutosSemMovimentacao_Service.getDtlimite(currentRow.NRDIAS);
        birthFilter.__format = format;
        birthFilter.CDPRODFIN += 'zzzzzzzzzzzzzzzzzzzz';
        birthFilter.CDPRODFIN = birthFilter.CDPRODFIN.substring(0, 10);

        General.openReport("est_relatorios#/gerarRelatorioEst32200", 'EST32200', birthFilter);

    };

    this.checkProductRange = function(field) {
        var currentRow = field.widget.currentRow;
        var treeField = field.widget.getField('DSTPARVPROD');
        var groupField = field.widget.getField('NRARVOREPROD');

        if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if (!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        }
        else if (!currentRow.CDPRODINI && !currentRow.CDPRODFIN) {
            treeField.readOnly = false;
        }
        else if (currentRow.CDPRODINI || currentRow.CDPRODFIN) {
            treeField.readOnly = true;
            groupField.readOnly = true;
        }
    };

    this.validaEstadoRangeProduto = function(widget) {
        var produtoIni = widget.getField('NMPRODINI');
        var produtoFin = widget.getField('NMPRODFIN');
        var tipoProduto = widget.getField('DSTPARVPROD');
        var groupProduto = widget.getField('NRARVOREPROD');
        if (tipoProduto.value()) {
            //Controle de campos de produto
            produtoIni.readOnly = true;
            produtoIni.setValue('');
            produtoIni.validations = {};
            widget.currentRow.CDPRODINI = '';
            produtoFin.readOnly = true;
            produtoFin.setValue('');
            produtoFin.validations = {};
            widget.currentRow.CDPRODFIN = '';
            //Controle campo de grupo
            groupProduto.readOnly = false;
            groupProduto.dataSourceFilter[0].value = widget.currentRow.NRTPARVPROD;
            groupProduto.reload();
        }
        else {
            //Controle de campos de produto
            produtoIni.readOnly = false;
            produtoIni.validations = { "required": true };
            produtoFin.readOnly = false;
            produtoFin.validations = { "required": true };
            //Controle campo de grupo
            groupProduto.readOnly = true;
            groupProduto.value(null);
        }
    };

    this.getGroups = function(widget) {
        var field = widget.getField('NRARVOREPROD');
        if (field.value()) {
            ProdTreeGroup.getAllGroups(field).then(function(result) {
                widget.currentRow.NRARVOREPRODFILTER = result.join('_');
            });
        }
        else {
            widget.currentRow.NRARVOREPRODFILTER = 'T';
        }
    };

}
Configuration(function(ContextRegister) {
    ContextRegister.register('Est32200_ProdutosSemMovimentacao_Controller', Est32200_ProdutosSemMovimentacao_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est32500_ReSuprimentoDeEstoque_Controller.js
function Est32500_ReSuprimentoDeEstoque_Controller(templateManager, ZhLayoutPreferences, ZhFilterPreferences, ScreenService, MessageService, maskEngine, ZhUtil, General, EstUtil) {

	this.onOpenFilter = function(widget) {
		const rowsToSet = {
			defaultFilter: widget.dataSourceFilter,
			REPORT_NAME: "EST325000",
			CDFILIAL: getLocalVar('CDFILIAL'),
			NMFILIAL: getLocalVar('NMFILIAL'),
			DTPOSICAO: getLocalVar('DTPROCESSA'),
			DTSEGU: 0
		};

		ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
	};

	this.setFilterParams = function(widget, preference = null) {
		const filterRow = preference.FILTER_ROW;

		let filter = [
			{
				"name": "CDFILIAL",
				"operator": "=",
				"value": filterRow.CDFILIAL
            },
			{
				"name": "DTINICIAL",
				"operator": "=",
				"value": filterRow.DTPOSICAO
            },
			{
				"name": "DTFINAL",
				"operator": "=",
				"value": moment(filterRow.DTPOSICAO, "DD/MM/YYYY").add(Number(filterRow.DTSEGU), 'days').format("DD/MM/YYYY")
            },
			{
				"name": "CDPRODINI",
				"operator": "=",
				"value": filterRow.CDPRODINI
            },
			{
				"name": "CDPRODFIN",
				"operator": "=",
				"value": filterRow.CDPRODFIN.padEnd(20, 'Z')
            },
			{
				"name": "DTPOSICAO",
				"operator": "=",
				"value": filterRow.DTPOSICAO
            }
        ];

		widget.dataSourceFilter = filter;
		widget.reload();
	};

	this.checkProductRange = function(field) {
		var currentRow = field.widget.currentRow;

		if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
			var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

			if (!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {

				MessageService.showMessageByCode("PRODUTO_INICIAL_MAIOR_FINAL").then(success => {
					currentRow[field.outData[field.descriptionField]] = null;
					currentRow[field.outData[field.valueField]] = null;
					currentRow.CDPRODFIN = null;
					field.widget.getField('NMPRODFIN').clearValue();
					field.widget.getField('NMPRODFIN').reload();
				}).catch(error => {
					console.log(error);
				});
			}
		}
	};

	this.reportProcess = function(args, afterOpenReport = false) {
		const { owner: widget } = args;

		if (!afterOpenReport) {
			const metadata = args.e.metadata;
			const CDFILIAL = EstUtil.getFieldValue('CDFILIAL', widget.name, true);
			const NMFILIAL = EstUtil.getFieldValue('NMFILIAL', widget.name, true);
			const CDPRODINI = EstUtil.getFieldValue('CDPRODINI', widget.name, true);
			const CDPRODFIN = EstUtil.getFieldValue('CDPRODFIN', widget.name, true);
			const mask = {
				type: "fix",
				params: {
					mask: getLocalVar('CDPICTPROD')
				}
			};

			metadata.filter = [
				{
					label: "Unidade",
					value: CDFILIAL + ' - ' + NMFILIAL
				},
				{
					label: "Data Inicial",
					value: EstUtil.getFieldValue('DTPOSICAO', widget.name, true)
				},
				{
					label: "Produto Inicial",
					value: Util.applyMask(CDPRODINI, mask, maskEngine, ZhUtil).split('.').toString().replaceAll(',', '.')
				},
				{
					label: "Produto Final",
					value: Util.applyMask(CDPRODFIN, mask, maskEngine, ZhUtil).split('.').toString().replaceAll(',', '.')
				}
			];
			ZhLayoutPreferences.applyOriginalLayout(widget);
		}
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est32500_ReSuprimentoDeEstoque_Controller', Est32500_ReSuprimentoDeEstoque_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est32700_ValidadeDeProdutosController.js
function Est32700_ValidadeDeProdutosController(EstUtil, ScreenService, General, templateManager,ZhFilterPreferences) {

}


Configuration(function(ContextRegister){
    ContextRegister.register('Est32700_ValidadeDeProdutosController', Est32700_ValidadeDeProdutosController);
});

// FILE: ./assets/modules/relatorios/assets/js/controllers/Est32700_ValidadeDeProdutos_Controller.js
function Est32700_ValidadeDeProdutos_Controller(EstUtil, ScreenService, General, templateManager,ZhFilterPreferences, Est32700_ValidadeDeProdutos_Service) {

    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        let rowsToSet  = {};
        this.getFilialLogada().then(function(success) {
            rowsToSet.CDFILIAL = [success.dataset.filial[0].CDFILIAL];
            rowsToSet.NMFILIAL = [success.dataset.filial[0].NMFILIAL];
            ZhFilterPreferences.openFilterWidget(widget,rowsToSet).then(function(success) {
                templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions').getField("NMFILIAL").reload();
                this.verAlmoxEstoque(templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions'));
            }.bind(this));
        }.bind(this));
        
    };
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    };
    
    this.getFilialLogada = function() {
    var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
	    ];
    return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
    
    this.verAlmoxEstoque = function(args){
        
        
        let widget = templateManager.container.getActiveWidget();
        
        if(widget.name != 'zhFilterPreferencesSelectConditions'){
            widget = args;
        }
        
        Est32700_ValidadeDeProdutos_Service.getParamFilialData(widget).then(() =>{
            
            let almoxField = widget.getField('DSALMOXARIFE');
            let LocalField = widget.getField('DSLOCALESTOQ');
            let cdAlmoxField = widget.getField('CDALMOXARIFE');
            let cdLocalEstField = widget.getField('CDLOCALESTOQ');
            
            widget.currentRow.CDALMOXARIFE = cdAlmoxField.value() ? cdAlmoxField.value() : "";
            
            //verifica almoxarifado
            if(widget.currentRow.IDUTILALMOX === 'N'){
                almoxField.readOnly = true;
                almoxField.clearValue();
                cdAlmoxField.clearValue(" ");
                delete almoxField.validations;
                
            }else{
                almoxField.readOnly = false;
                almoxField.validations = {"required": {"message": "Campo Obrigatório"}};
            }
            
            //verifica local do estoque
            if(widget.currentRow.IDUTILLCESTQ === 'N'){
                LocalField.readOnly = true;
                LocalField.clearValue();
                cdLocalEstField.clearValue(" ");
            }else{
                cdLocalEstField.clearValue();
                LocalField.readOnly = false;
            }
            
            this.checkDataForm(widget);
            
        });
    };
    
	// Realiza a limpeza de valores do filtro caso a filial seja mudada
    this.clearFieldsOnChangeFilial = function(widget){
        let fields = [
            "CDALMOXARIFE",
            "DSALMOXARIFE",
            "CDLOCALESTOQ",
            "DSLOCALESTOQ",
            "VALIDADE",
            "ENTREGA",
            "CDPRODINI",
            "NMPRODINI",
            "CDPRODFIN",
            "NMPRODFIN"
            ];
        
        fields.forEach((field)=>{
            let fieldToClear = widget.getField(field);
            fieldToClear.clearValue("");
            
            if(fieldToClear.name === "DSALMOXARIFE" || fieldToClear.name === "DSLOCALESTOQ" ||
            fieldToClear.name === "NMPRODINI" || fieldToClear.name === "NMPRODFIN"){
                fieldToClear.reload();
            }
        });
    };
    
    this.checkDataForm = function(args){
        
        let widget = templateManager.container.getActiveWidget();
        
        if(widget.name != 'zhFilterPreferencesSelectConditions'){
            widget = args;
        }
        
        let LocalField = widget.getField('DSLOCALESTOQ');
        let cdLocalEstField = widget.getField('CDLOCALESTOQ');
        
        if(widget.currentRow.IDUTILALMOX === 'S' && widget.currentRow.IDUTILLCESTQ === 'S'){
            if(widget.currentRow.CDALMOXARIFE !== ""){
                LocalField.validations = {"required": {"message": "Campo Obrigatório"}};
                LocalField.readOnly = false;
            }else{
                LocalField.clearValue();
                cdLocalEstField.clearValue();
                delete LocalField.validations;
                LocalField.readOnly = true;
            }
            
            LocalField.reload();
        }
        
        if(widget.currentRow.IDUTILALMOX === 'S' && widget.currentRow.IDUTILLCESTQ === 'N'){
            if(widget.currentRow.CDALMOXARIFE !== ""){
                cdLocalEstField.value(' ');
            }else{
                cdLocalEstField.clearValue("");
            }
        }
        
    };
    
    // redefine filtro com parametros corretos a serem enviados para o backend
    this.changeFilter = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        
        var tamanhoMask = getLocalVar('CDPICTPROD').replace(/\./g,'').length;
        filterRow.CDPRODFIN = filterRow.CDPRODFIN ? filterRow.CDPRODFIN : "";
        var tamanho = tamanhoMask - filterRow.CDPRODFIN.length;
        
        for(var i = 0; i < tamanho; i++){
        	filterRow.CDPRODFIN = filterRow.CDPRODFIN+"Z";
        }

        var filtro= [
            {
                "name":"CDFILIAL",
                "operator":"=",
                "value": filterRow.CDFILIAL
            },
            {
                "name":"CDALMOXARIFE",
                "operator":"=",
                "value": filterRow.CDALMOXARIFE ? filterRow.CDALMOXARIFE : " "
            },
            {
                "name":"CDLOCALESTOQ",
                "operator":"=",
                "value": filterRow.CDLOCALESTOQ ? filterRow.CDLOCALESTOQ : " "
            },
            {
                "name":"DTVALIINI",
                "operator":"=",
                "value": filterRow.VALIDADE ? filterRow.VALIDADE.start : null
            },
            {
                "name":"DTVALIFIN",
                "operator":"=",
                "value": filterRow.VALIDADE ? filterRow.VALIDADE.end : null
            },
            {
                "name":"DTEMIINI",
                "operator":"=",
                "value": filterRow.ENTREGA ? filterRow.ENTREGA.start : null
            },
            {
                "name":"DTEMIFIN",
                "operator":"=",
                "value": filterRow.ENTREGA ? filterRow.ENTREGA.end : null
            },
            {
                "name":"CDPRODINI",
                "operator":"=",
                "value": filterRow.CDPRODINI ? filterRow.CDPRODINI : null
            },
            {
                "name":"CDPRODFIN",
                "operator":"=",
                "value": filterRow.CDPRODFIN ? filterRow.CDPRODFIN : null
            },
            {
                "name":"NMPRODINI",
                "operator":"=",
                "value": filterRow.NMPRODINI ? filterRow.NMPRODINI : null
            },
            {
                "name":"NMPRODFIN",
                "operator":"=",
                "value": filterRow.NMPRODFIN ? filterRow.NMPRODFIN : null
            }
        ];
        
        widget.dataSourceFilter = filtro;
        widget.reload();
        
    };
    
    this.checkProductRange = function(field) {
        var currentRow = field.widget.currentRow;
      
        if(currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = (currentRow.CDPRODFIN)? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if(!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        } 
    };
}


Configuration(function(ContextRegister){
    ContextRegister.register('Est32700_ValidadeDeProdutos_Controller', Est32700_ValidadeDeProdutos_Controller);
});

// FILE: ./assets/modules/relatorios/assets/js/controllers/Est40000_MapasPoliciaCivil_Controller.js
function Est40000_MapasPoliciaCivil_Controller(ScreenService, General, templateManager, localStorageEngine) {

	//This function sets default values for especifcs fields
	this.onEnterScreen = function(widget) {
		const { currentRow } = widget;
		currentRow.NMRELATORIO = "EST40000";
		currentRow.CDTRIMESTRE = "1";
		widget.reload();
	};

	// It calls back to manage and generate BIRT report.
	this.generateReport = function(widget, format) {
		if (widget.isValid()) {
			const { currentRow } = widget;
			const { startDate, endDate } = this.getTrimesterDates(currentRow);
			currentRow.DTINICIAL = startDate;
			currentRow.DTFINAL = endDate;
			currentRow.__FORMAT = format;


			General.openReport("est_relatorios#/mapas_policia_civil", currentRow.NMRELATORIO, currentRow);
		}
	};

	//It gets first and last trimester date
	this.getTrimesterDates = function(row) {
		let { CDTRIMESTRE, CDANO } = row;
		let startDate, endDate = "";

		switch (CDTRIMESTRE) {
			case '1':
				startDate = new Date('01/01/' + CDANO);
				endDate = new Date('01/01/' + CDANO);
				endDate.setMonth(2);
				break;
			case '2':
				startDate = new Date('04/01/' + CDANO);
				endDate = new Date('04/01/' + CDANO);
				endDate.setMonth(5);
				break;
			case '3':
				startDate = new Date('07/01/' + CDANO);
				endDate = new Date('07/01/' + CDANO);
				endDate.setMonth(8);
				break;
			case '4':
				startDate = new Date('10/01/' + CDANO);
				endDate = new Date('10/01/' + CDANO);
				endDate.setMonth(11);
				break;
			default:
				break;
		}
		return {
			"startDate": moment(startDate).format('DD/MM/YYYY'),
			"endDate": moment(endDate).endOf('month').format('DD/MM/YYYY')
		};
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est40000_MapasPoliciaCivil_Controller', Est40000_MapasPoliciaCivil_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est45000_EmissaoInventario_Controller.js
function est45000_EmissaoInventario_Controller(ScreenService, General, templateManager, Est45000_EmissaoInventario_Service, ProductField, ZhFilterPreferences) {
 
    this.setNameLayout = function(args){
        var widget = args.owner;
        if (!widget.hasOwnProperty('labelPrincipal')){
            widget.labelPrincipal = widget.label;
        }
        if (args.e.layout.ID == 'ORIGINAL'){
            widget.container.label = widget.labelPrincipal;
            widget.label = widget.labelPrincipal;
        }else{
            widget.container.label = args.e.layout.LABEL_TO_PREFERENCE;
            widget.label = args.e.layout.LABEL_TO_PREFERENCE;
        }
    };
    
    this.openFilter = function(widget){
        this.setFieldsGerais(widget);
        var rowsToSet = {};
        this.getFilialLogada().then(function(success) {
            rowsToSet.CDFILIAL = success.dataset.filial[0].CDFILIAL;
            rowsToSet.NMFILIAL = success.dataset.filial[0].NMFILIAL;
            rowsToSet.PRODUTOATIVO = 'T';
            
            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(function(){
                this.changeFilial(widget);
            }.bind(this));
        }.bind(this));        
    };
    
    this.setFieldsGerais = function(widget){
        templateManager.container.fieldsGerais = widget.fields.map(function(field){return field.name;});
    }; 
	
	this.getFilialLogada = function() {
        var filter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };
	
	//Parâmetros Filtro
    this.changeFilter = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        var filtro = [];
        
        if(widget.dataSource.name == 'est_relatorios#/crud/itlanctoest_produto') {
            widget.floatingControl.xlsAction.dataSourceName = "itlanctoest_produto";
            widget.floatingControl.pdfAction.dataSourceName = "itlanctoest_produto";
            
            filtro = [
                {
                    "name": "CDFILIAL",
                    "operator":"=",
                    "value": filterRow.CDFILLANC
                },
                {
                    "name":"NRLANCESTQ",
                    "operator":"=",
                    "value": filterRow.NRLANCESTQ ? filterRow.NRLANCESTQ : null
                }
            ];
        }
        else {
            widget.floatingControl.xlsAction.dataSourceName = "produto_posestatu_paramfilial";
            widget.floatingControl.pdfAction.dataSourceName = "produto_posestatu_paramfilial";
            filtro= [
                {
                    "name": filterRow.CDFILIMOVI? "CDFILIMOVI" : "CDFILIAL",
                    "operator":"=",
                    "value": filterRow.CDFILIMOVI? filterRow.CDFILIMOVI : filterRow.CDFILIAL
                },
                {
                    "name":"CDALMOXARIFE",
                    "operator":"=",
                    "value": filterRow.CDALMOXARIFE ? filterRow.CDALMOXARIFE : null
                },
                {
                    "name":"CDLOCALESTOQ",
                    "operator":"=",
                    "value": filterRow.CDLOCALESTOQ ? filterRow.CDLOCALESTOQ : null
                },
                {
                    "name":"PRODINI",
                    "operator":"=",
                    "value": filterRow.PRODINI ? filterRow.PRODINI : null
                },
                {
                    "name":"PRODFIM",
                    "operator":"=",
                    "value": filterRow.PRODFIM ? filterRow.PRODFIM : null
                },
                {
                    "name":"PRODEST",
                    "operator":"=",
                    "value": filterRow.PRODEST ? filterRow.PRODEST : 'S'
                },
                {
                    "name": "CDGRUPPROD",
                    "operator": "=",
                    "value": filterRow.CDGRUPPROD ? filterRow.CDGRUPPROD : null
                },
                {
                    "name": "NMGRUPPROD",
                    "operator": "=",
                    "value": filterRow.NMGRUPPROD ? filterRow.NMGRUPPROD : null
                },
                {
                    "name": "PRODUTOATIVO",
                    "operator": "=",
                    "value": filterRow.PRODUTOATIVO != 'T' ? filterRow.PRODUTOATIVO : null
                }
            ];
            
            filtro.map(filter=>{
                if(filter.name == "PRODUTOATIVO" && filter.value == null)
                    filtro.pop(filter);
            });
        }
        widget.dataSourceFilter = filtro;
        widget.reload();
        
    };
    
    // Habilita/Desabilita Almoxarifado
    this.changeFilial = function (widget) {
        var cdfilial = widget.currentRow.CDFILIAL;

        if(cdfilial) {
            Est45000_EmissaoInventario_Service.getParamFilial(cdfilial).then(function (paramfilial) {
                widget.getField("DSALMOXARIFE").readOnly = !((paramfilial.dataset.paramfilial[0].IDCTRLESTQ == 'S') && (paramfilial.dataset.paramfilial[0].IDUTILALMOX  == 'S'));
                
                widget.getField("DSALMOXARIFE").dataSourceFilter[0].value = cdfilial;
                widget.getField("DSALMOXARIFE").reload();
                
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }else{
            widget.getField("DSALMOXARIFE").readOnly = true;
            widget.getField("DSLOCALESTOQ").readOnly = true;
        }
        widget.currentRow.DSALMOXARIFE = '';
        widget.currentRow.DSLOCALESTOQ = '';
    };

    // Habilita/Desabilita Localização
    this.changeAlmoxarifado = function (args) {
        var cdfilial     = args.row.CDFILIAL;
        var cdalmoxarife = args.row.CDALMOXARIFE;
        if(cdfilial && cdalmoxarife) {
            Est45000_EmissaoInventario_Service.getParamFilial(cdfilial).then(function (paramfilial) {
                args.owner.field.widget.getField("DSLOCALESTOQ").readOnly = !((paramfilial.dataset.paramfilial[0].IDCTRLESTQ == 'S') && (paramfilial.dataset.paramfilial[0].IDUTILLCESTQ == 'S'));
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }else{
            args.owner.field.widget.getField("DSLOCALESTOQ").readOnly = true;
        }
        args.row.DSLOCALESTOQ = '';
    };
    
    this.habilitaNumLancamento = function(args){
        // var widget = templateManager.container.getWidget('EmissaoInventarioGrid');
        if(args.row.DTLANCESTQ != ""){
            args.owner.field.widget.getField('NRLANCESTQ').readOnly = false;
            args.owner.field.widget.getField('NMPRODINI').readOnly = true;
            args.owner.field.widget.getField('NMPRODINI').setValue('');
            args.owner.field.widget.getField('NMPRODFIM').readOnly = true;
            args.owner.field.widget.getField('NMPRODFIM').setValue('');
            args.owner.field.widget.getField('PRODEST').readOnly = true;
            
            args.owner.field.widget.getField('NRLANCESTQ').validations = {required: true};
            args.owner.field.widget.getField('NMPRODINI').validations = {};
            args.owner.field.widget.getField('NMPRODFIM').validations = {};
            args.owner.field.widget.getField('PRODEST').validations = {};
            this.changeFilterLancEstoq(args);
        }else{
            args.owner.field.widget.getField('NRLANCESTQ').readOnly = true;
            args.owner.field.widget.getField('NMPRODINI').readOnly = false;
            args.owner.field.widget.getField('NMPRODFIM').readOnly = false;
            args.owner.field.widget.getField('PRODEST').readOnly = false;
            
            args.owner.field.widget.getField('NRLANCESTQ').validations = {};
            args.owner.field.widget.getField('NMPRODINI').validations = {required: true};
            args.owner.field.widget.getField('NMPRODFIM').validations = {required: true};
            args.owner.field.widget.getField('PRODEST').validations = {required: true};
            args.owner.field.widget.getField('NRLANCESTQ').setValue('');
            
            this.changeFilterLancEstoq(args);
            args.owner.field.widget.getField('NRLANCESTQ').reload();
        }
    };
    
    this.changeFilterLancEstoq = function(args){
        var cdfilial = args.owner.field.widget.currentRow.CDFILIAL;   
        var dtlancestq = args.owner.field.widget.currentRow.DTLANCESTQ;   
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[0].value = cdfilial;
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[1].value = dtlancestq;
        if(dtlancestq != "" && dtlancestq != undefined){
            args.owner.field.widget.getField('NRLANCESTQ').reload();
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/crud/itlanctoest_produto";
        }else{
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/produto_posestatu_paramfilial";
            delete args.owner.field.widget.currentRow.CDFILIMOVI;
        }
    };
    
    this.changeFilterLancEstoqByFilial = function(args){
        var cdfilial = args.owner.field.widget.currentRow.CDFILIAL;   
        var dtlancestq = args.owner.field.widget.currentRow.DTLANCESTQ;   
        
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[0].value = cdfilial;
        args.owner.field.widget.getField('NRLANCESTQ').dataSourceFilter[1].value = dtlancestq;
        args.owner.field.widget.currentRow.NRLANCESTQ = "";
        if(dtlancestq != ""  && dtlancestq != undefined){
            args.owner.field.widget.getField('NRLANCESTQ').reload();
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/crud/itlanctoest_produto";
        }else{
            templateManager.container.getWidget('EmissaoInventarioGrid').dataSource.name = "est_relatorios#/produto_posestatu_paramfilial";
            delete args.owner.field.widget.currentRow.CDFILIMOVI;
        }
    };

}

Configuration(function(ContextRegister) {
	ContextRegister.register('Est45000_EmissaoInventario_Controller', est45000_EmissaoInventario_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est46000_EmissaoAjusteInventario_Controller.js
function est46000_EmissaoAjusteInventario_Controller(General, MessageService, ScreenService, restEngine, maskEngine, ZhUtil, templateManager, localStorageEngine, ZhReports, ProductField, Est46000_EmissaoAjusteInventario_Service, Est31300_MovimentacaodeEstoque_Service, ZhLayoutPreferences) {

    this.defaultValueEmissaoAjuste = function(args) {
        var formWidget = args.owner;
        args.row.REPORT_NAME = 'EST46000';
        args.row.DTPERINI = moment().startOf('month').format('DD/MM/YYYY');
        args.row.DTPERFIN = moment().endOf('month').format('DD/MM/YYYY');
        args.row.INTERVALO = {
            "start": args.row.DTPERINI,
            "end": args.row.DTPERFIN,
            "formattedStart": args.row.DTPERINI,
            "formattedEnd": args.row.DTPERFIN
        };
        args.row.CUSTOS = 1;
        args.row.NRORG = getLocalVar('ERP_NRORGTRAB') || getLocalVar('NRORGTRAB');
        args.row.ORDENA = 1;

        var campoNivel = formWidget.getField('NPTOTALIZA');

        let SetTime = setInterval((time) => {
            var levels = General.getLevelByMask('GER_CDPICTPROD', 1);
            campoNivel.dataSource.data = [];
            for (var index in levels) {
                if (index < levels.length)
                    campoNivel.dataSource.data.push({
                        "name": levels[index].nivel,
                        "value": levels[index].caracteres
                    });
            }

            // campoNivel.setValue(campoNivel.dataSource.data[0].value);

            this.getFilialLogada().then(success => {
                args.row.CDFILIAL = success.dataset.filial[0].CDFILIAL;
                args.row.NMFILIAL = success.dataset.filial[0].NMFILIAL;
                var widget = templateManager.container.getActiveWidget();
                let currentRow = widget.currentRow;

                widget.getField('DSALMOXARIFE').dataSourceFilter.forEach(function(fil) {
                    if (fil.name == "CDFILIAL") {
                        fil.value = currentRow.CDFILIAL;
                    }
                });
                widget.getField('DSLOCALESTOQ').dataSourceFilter.forEach(function(fil) {
                    if (fil.name == "CDFILIAL") {
                        fil.value = currentRow.CDFILIAL;
                    }
                });
                widget.getField('DSALMOXARIFE').reload();
                widget.getField('DSLOCALESTOQ').reload();
                this.checkUnidadeFields(widget);
            });
            clearInterval(SetTime);
        }, 2500);
    };

    this.getFilialLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.geraRelatorio = function(widget, type) { //receber em type xls ou pdf
        var REPORT_JSON = "./json/reports/est46001_relatorio.json";

        ScreenService.showLoader();
        restEngine.requestMetaData(REPORT_JSON).then(function(reportMetaData) {
            ScreenService.hideLoader();
            ZhReports.openReport(reportMetaData.data, widget, null, null, type);
        });
    };

    this.validaData = function(currentRow) {

        var dataInicial = currentRow.DTPERINI ? moment(currentRow.DTPERINI, 'DD/MM/YYYY')._d : null;
        var dataFinal = currentRow.DTPERFIN ? moment(currentRow.DTPERFIN, 'DD/MM/YYYY')._d : null;

        if (dataInicial && dataFinal && dataFinal < dataInicial) {
            MessageService.showMessageByCode('DATA_RANGE');
            return false;
        }
        return true;
    };

    this.comparaPratoProduto = function(pinicial, pfinal, mascara) {
        var mask = {
            type: "fix",
            params: {
                mask: mascara
            }
        };
        pinicial = Util.applyMask(pinicial, mask, maskEngine, ZhUtil).split('.');
        pfinal = Util.applyMask(pfinal, mask, maskEngine, ZhUtil).split('.');
        var nvinicial = pinicial.length;
        var nvfinal = pfinal.length;
        var resultado = true;
        if (nvinicial != nvfinal) {
            if (nvinicial > nvfinal) {
                resultado = false;
            }
        }
        else {
            var i = 0;
            do {
                if (pinicial[i] > pfinal[i]) {
                    resultado = false;
                }
                i++;
            }
            while (i < pinicial.length && resultado);
        }
        return resultado;
    };

    this.validaNivel = function(currentRow) {
        var retorno = '';

        if (currentRow.CDARVPRODINI && currentRow.CDARVPRODFIN) {
            retorno = this.comparaPratoProduto(currentRow.CDARVPRODINI, currentRow.CDARVPRODFIN, getLocalVar('CDPICTPROD'));
            if (!retorno) {
                MessageService.showMessageByCode('PRODUTO_INICIAL_MAIOR_FINAL');
                return false;
            }
            return true;
        }
        return false;
    };

    this.validaRelatorioGrid = function(currentRow, widget) {
        if (widget.isValid() && this.validaData(currentRow) && this.validaNivel(currentRow)) {
            var rowStripe = {};
            rowStripe.DSFILIAL = currentRow.CDFILIAL + ' - ' + currentRow.NMFILIAL;
            rowStripe.DSPRODUTO = currentRow.CDARVPRODINI + ' à ' + currentRow.CDARVPRODFIN;
            rowStripe.DSPERIODO = currentRow.DTPERINI + ' à ' + currentRow.DTPERFIN;
            rowStripe.CUSTOS = currentRow.CUSTOS;
            localStorageEngine.setDataSource('/StripeEmissao', rowStripe);
            currentRow.CDPRODFIN = currentRow.CDARVPRODFIN + "zzzzzzzzzzzzzzzzzzzz";
            currentRow.CDPRODFIN = currentRow.CDPRODFIN.substring(0, 20);
            return restEngine.post('est_relatorios#/emissao_ajuste', {
                "requestType": "Row",
                "row": currentRow
            }).then(function(callback) {
                var datasetEmissao = callback.dataset.EmissaoAjuste;
                if (datasetEmissao.Details.length) {
                    localStorageEngine.setDataSource('est_relatorios#/emissao_ajuste_details', datasetEmissao.Details).then(() => {
                        localStorageEngine.setDataSource('est_relatorios#/emissao_ajuste_totalizator', datasetEmissao.Totalizator).then(() => {
                            ScreenService.openWindow('est_relatorios#est46001_grid_relatorio_ajuste');
                        });
                    });
                }
                else {
                    MessageService.showMessageByCode('EMPTY_DATA');
                }

            });
        }
    };

    this.validateBirthReport = function(widget, format) {
        var currentRow = widget.currentRow;
        currentRow.__format = format;
        if (!currentRow.CDPRODFIN)
            currentRow.CDPRODFIN = currentRow.CDARVPRODFIN;

        var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDARVPRODFIN.replace(/[\.\-]/g, '') : null;

        if (widget.isValid()) {
            if (General.comparaPratoProduto(currentRow.CDARVPRODINI, compproduto, getLocalVar('GER_CDPICTPROD'))) {
                if (currentRow.REPORT_NAME == 'GRID') {
                    this.validaRelatorioGrid(currentRow, widget);
                }
                else {
                    Est46000_EmissaoAjusteInventario_Service.requestReport(currentRow).then(function(birtURL) {
                        window.open(birtURL.dataset.URL[0]);
                    });
                }
            }
            else {
                MessageService.showMessageByCode('PRODUTO_INICIAL_MAIOR_FINAL');
            }
        }
    };

    this.onEnterRelatorio = function() {
        var container = templateManager.container;
        var stripe = container.getWidget('stripe');
        localStorageEngine.getDataSource('/StripeEmissao').then(function(row) {
            stripe.currentRow.DSFILIAL = row[0].DSFILIAL;
            stripe.currentRow.DSPERIODO = row[0].DSPERIODO;
            stripe.currentRow.DSPRODUTO = row[0].DSPRODUTO;
            if (row[0].CUSTOS == '1') {
                stripe.currentRow.CUSTOS = 'Líquido';
            }
            else {
                stripe.currentRow.CUSTOS = 'Bruto';
            }
        });
    };

    var applyMask = function(fieldValue, maxlength) {
        var mask = {
            "type": "zerofill",
            "params": {
                "right": true,
                "maxlength": maxlength
            }
        };
        return Util.applyMask(fieldValue, mask, maskEngine, ZhUtil);
    };

    this.changeHideInconLabelVisible = function(data, label, hide, name, visibility) {
        data.forEach(element => {
            if (element.name === name) {
                element.label = label;
                element.hideIcon = hide;
                element.isVisible = visibility;
            }
        });
    };

    this.fieldOnChangeRelatorio = function(widget) {

        const ReportName = widget.currentRow.REPORT_NAME;
        const buttonGeneratePdf = widget.getAction('TK_VIEW_BIRTH');

        switch (ReportName) {
            case 'GRID':
                buttonGeneratePdf.label = 'Gerar GRID';
                buttonGeneratePdf.icon = 'file-search';
                break;
            case 'EST46000_retirada':
            case 'EST46000':
                buttonGeneratePdf.label = 'Gerar Pdf';
                buttonGeneratePdf.icon = 'pdf';
                break;
        }
        templateManager.updateTemplate();
    };
    // this.fieldOnChangeRelatorio = function(widget) {

    //     const ReportName = widget.currentRow.REPORT_NAME;
    //     const TotalizaField = widget.getField('NPTOTALIZA');

    //     switch (ReportName) {
    //         case 'GRID':
    //             TotalizaField.readOnly = true;
    //             widget.currentRow.NPTOTALIZA = 1;
    //             TotalizaField.validations = { "required": false };
    //             break;
    //         case 'EST46000_retirada':
    //         case 'EST46000':
    //             TotalizaField.readOnly = false;
    //             TotalizaField.validations = { "required": true };
    //             break;
    //     }
    //     TotalizaField.reload();
    // };

    this.checkUnidadeFields = function(widget) {
        var currentRow = widget.currentRow;

        ProductField.updateProductFilters(widget);
        if (currentRow.REPORT_NAME === 'EST46000' ||
            currentRow.REPORT_NAME === 'GRID' ||
            currentRow.REPORT_NAME === 'EST46000_retirada') {
            if (currentRow.CDFILIAL) {
                Est31300_MovimentacaodeEstoque_Service.requestValidations(currentRow.CDFILIAL).then(function(checks) {
                    currentRow.ALMOXARIFADO = checks.IDUTILALMOX;
                    currentRow.LOCALESTOQUE = checks.IDUTILLCESTQ;
                    currentRow.LOTE = checks.IDUTILLOTE;

                    var almoxarifado = widget.getField('DSALMOXARIFE');
                    var localestoque = widget.getField('DSLOCALESTOQ');
                    var lote = widget.getField('NRLOTEESTQ');

                    if (currentRow.LOTE) {
                        lote.dataSourceFilter[1] = { "name": "CDFILIAL", "operator": "=", "value": currentRow.CDFILIAL };
                        lote.readOnly = false;
                        currentRow.NRLOTEESTOQ = null;
                        this.reloadFields(widget, false, false, true);
                    }
                    else {
                        lote.readOnly = true;
                        lote.setValue(' ');
                        currentRow.NRLOTEESTOQ = " ";
                    }
                    if (currentRow.ALMOXARIFADO) {
                        almoxarifado.dataSourceFilter[0] = { "name": "CDFILIAL", "operator": "=", "value": currentRow.CDFILIAL };
                        almoxarifado.readOnly = false;
                        almoxarifado.validations = { "required": true };
                        this.reloadFields(widget, true, false, false);

                        if (currentRow.LOCALESTOQUE) {
                            localestoque.dataSourceFilter[0] = { "name": "CDFILIAL", "operator": "=", "value": currentRow.CDFILIAL };
                            localestoque.validations = { "required": true };

                            this.checkAlmoxariFields(widget, true);
                        }
                        else {
                            localestoque.readOnly = true;
                            localestoque.validations = null;
                            localestoque.setValue(' ');
                            currentRow.CDLOCALESTOQ = ' ';
                        }
                    }
                    else {
                        almoxarifado.readOnly = true;
                        almoxarifado.validations = null;
                        almoxarifado.setValue(' ');
                        currentRow.CDALMOXARIFE = ' ';
                        localestoque.readOnly = true;
                        localestoque.validations = null;
                        localestoque.setValue(' ');
                        currentRow.CDLOCALESTOQ = ' ';
                    }

                }.bind(this));
            }
            else {
                currentRow.ALMOXARIFADO = false;
                currentRow.LOCALESTOQUE = false;
                currentRow.NRLOTEESTOQ = false;

                widget.getField('DSALMOXARIFE').readOnly = true;
                widget.getField('DSLOCALESTOQ').readOnly = true;
                widget.getField('NRLOTEESTQ').readOnly = true;

                widget.getField('DSALMOXARIFE').validations = null;
                widget.getField('DSLOCALESTOQ').validations = null;
                widget.getField('NRLOTEESTQ').validations = null;

                this.reloadFields(widget, true, true, true);
            }
        }
    };

    this.reloadFields = function(widget, almoxarifado, localestoque, lote) {
        var currentRow = widget.currentRow;

        if (almoxarifado) {
            currentRow.CDALMOXARIFE = (currentRow.ALMOXARIFADO) ? null : ' ';
            currentRow.DSALMOXARIFE = (currentRow.ALMOXARIFADO) ? null : ' ';

            widget.getField('DSALMOXARIFE').reload();
        }

        if (localestoque) {
            currentRow.CDLOCALESTOQ = (currentRow.LOCALESTOQUE) ? null : ' ';
            currentRow.DSLOCALESTOQ = (currentRow.LOCALESTOQUE) ? null : ' ';

            widget.getField('DSLOCALESTOQ').reload();
        }

        if (lote) {
            currentRow.NRLOTEESTQ = null;
            currentRow.NRSUBLOTE = null;

            widget.getField('NRLOTEESTQ').reload();
        }
    };

    this.checkAlmoxariFields = function(widget) {
        var currentRow = widget.currentRow;

        if (currentRow.CDALMOXARIFE && currentRow.LOCALESTOQUE) {
            widget.getField('DSLOCALESTOQ').readOnly = false;
            widget.getField('DSLOCALESTOQ').dataSourceFilter[2] = { "name": "CDALMOXARIFE", "operator": "=", "value": currentRow.CDALMOXARIFE };
        }
        else widget.getField('DSLOCALESTOQ').readOnly = true;

        if (arguments[1]) this.reloadFields(widget, true, true, true);
        else this.reloadFields(widget, false, true, false);
    };

    this.intervaloChange = function(field) {
        let widget = field.widget;
        widget.currentRow.DTPERINI = widget.currentRow.INTERVALO.start;
        widget.currentRow.DTPERFIN = widget.currentRow.INTERVALO.end;
    };

    /*  This function will change reports titles according to each widget name  */
    this.onFormatReport = function(args, afterOpenReport = false) {
        const { owner: widget } = args;
        const metadata = args.e.metadata;
        const detailTitle = "Ajuste de Inventário - Analítico";
        const totalizatorTitle = "Ajuste de Inventário - Analítico (Totalizador)";

        if (afterOpenReport)
            ZhLayoutPreferences.applyOriginalLayout(widget);
        else {
            metadata.title = widget.name == 'detailsWidget' ? detailTitle : totalizatorTitle;
            ZhLayoutPreferences.applyOriginalLayout(widget);
        }
    };

    /*  This function will hide or show the totalization div according to widegt name*/
    this.onFormatGrid = function(widget) {
        const interval = setInterval(() => {
            if (widget.name == 'totalizatorWidget') {
                $("div.tr.cell1.group-footer").css("display", "none");
            }
            else {
                $("div.tr.cell1.group-footer").css("display", "");
            }
            clearInterval(interval);
        }, 1300);
    };

    /*  This function will set the previous window as default always that the 
     *  grid window gets opened */
    this.windowChangeHandler = function() {
        templateManager.historyItems = [
            "est_relatorios#est46000_emissao_ajuste_inventario",
            "est_relatorios#est46001_grid_relatorio_ajuste"
        ];
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est46000_EmissaoAjusteInventario_Controller', est46000_EmissaoAjusteInventario_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est58100_RelatorioRetiradaEstoque_Controller.js
function Est58100_RelatorioRetiradaEstoque_Controller(ScreenService, General, templateManager, ProductField, ZhFilterPreferences) {
    let arrCopy = [];

    this.setNameLayout = function(args) {
        var widget = args.owner;
        if (!widget.hasOwnProperty('labelPrincipal')) {
            widget.labelPrincipal = widget.label;
        }
        if (args.e.layout.ID == 'ORIGINAL') {
            widget.container.label = widget.labelPrincipal;
            widget.label = widget.labelPrincipal;
        }
        else {
            widget.container.label = args.e.layout.LABEL_TO_PREFERENCE;
            widget.label = args.e.layout.LABEL_TO_PREFERENCE;
        }
    };

    this.openFilter = function(widget) {
        this.setFieldsGerais(widget);
        let rowsToSet = {};
        this.getFilialLogada().then(success => {
            rowsToSet.CDFILIAL = [success.dataset.filial[0].CDFILIAL];
            rowsToSet.NMFILIAL = [success.dataset.filial[0].NMFILIAL];

            ZhFilterPreferences.setOriginalFilter(widget.id, rowsToSet);
            ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(() => {
                const filterWidget = templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions');
                const filialField = filterWidget.getField("NMFILIAL");

                filterWidget.currentRow.CDFILIAL = rowsToSet.CDFILIAL;
                filialField.reload().then(() => {
                    filialField.widget.currentRow.CDFILIAL = rowsToSet.CDFILIAL;
                    this.verAlmoxarifado(filterWidget);
                });
            });
        });
    };

    this.getFilialLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
                }
		    ];
        return General.callRestEngine('est#/crud/filial', 'FilterData', filter);
    };

    this.disablePagination = function(args) {
        args.owner.hasPagination = false;
    };

    this.reloadDataSoure = function(args) {
        //(filter, page, itemsPerPage, callback, errorCallback, retryCount, timeout, retryCallback, owner)
        if(args.owner.groups && args.owner.groups.length > 0){
            args.owner.useLocalDataOnReport = true;
            args.owner.dataSource.load(args.owner.getDataSourceFilter(), -1, -1, undefined, undefined, undefined, undefined, undefined, args.owner).then((response) => {
                arrCopy = Util.clone(response.dataset.relatorio_retirada_estoque);
            });
        }else{
            args.owner.useLocalDataOnReport = false;
        }
        args.owner.hasPagination = args.owner.appliedLayout.ID != 'ORIGINAL' ? false : true;
        args.owner.reload();
    };

    this.setFieldsGerais = function(widget) {
        templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
    };

    this.verFilial = function(widget) {
        var NMFILIAL = widget.getField('CDFILIAL').value();

    };

    this.verLocalEstVazio = function(widget) {
        if (widget.getField('DSLOCALESTOQ').value() == "") {
            widget.getField('CDLOCALESTOQ').clearValue();
        }
    };

    // Habilita/Desabilita Almoxarifado
    this.verAlmoxarifado = function(widget) {
        var NMFILIAL = widget.getField('NMFILIAL').value();
        if (NMFILIAL.length > 0) {
            widget.getField('DSALMOXARIFE').readOnly = false;
            widget.getField('DSALMOXARIFE').dataSourceFilter.forEach(function(fil) {
                if (fil.name == 'CDFILIAL') {
                    fil.value = "";
                    if (Array.isArray(widget.getField('CDFILIAL').value())) {
                        fil.value = widget.getField('CDFILIAL').value().join("_");
                        fil.value = "_" + fil.value + "_";
                    }
                }
            });
        }
        else {
            widget.getField('DSALMOXARIFE').readOnly = true;
            widget.getField('DSALMOXARIFE').clearValue();
            widget.getField('CDALMOXARIFE').clearValue();
            widget.getField('DSLOCALESTOQ').readOnly = true;
            widget.getField('DSLOCALESTOQ').clearValue();
            widget.getField('CDLOCALESTOQ').clearValue();
        }
        if (NMFILIAL.length == 0) {
            widget.getField('CDFILIAL').clearValue();
        }
    };

    this.verLocalEst = function(widget) {
        var NMFILIAL = widget.getField('CDFILIAL').value();
        var DSALMOXARIFE = widget.getField('DSALMOXARIFE').value();
        if ((NMFILIAL.length) > 0 && (DSALMOXARIFE)) {
            widget.getField('DSLOCALESTOQ').readOnly = false;
            widget.getField('DSLOCALESTOQ').dataSourceFilter.forEach(function(fil) {
                if (fil.name == 'CDFILIAL') {
                    fil.value = "";
                    if (Array.isArray(widget.getField('CDFILIAL').value())) {
                        fil.value = widget.getField('CDFILIAL').value().join("_");
                        fil.value = "_" + fil.value + "_";
                    }
                }
                else if (fil.name == 'CDALMOXARIFE') {
                    fil.value = widget.getField('CDALMOXARIFE').value();
                }
            });
        }
        else if (DSALMOXARIFE == "") {
            widget.getField('DSLOCALESTOQ').readOnly = true;
            widget.getField('DSLOCALESTOQ').clearValue();
            widget.getField('CDLOCALESTOQ').clearValue();
            widget.getField('DSALMOXARIFE').clearValue();
            widget.getField('CDALMOXARIFE').clearValue();
        }
        else if (NMFILIAL.length == 0) {
            widget.getField('DSLOCALESTOQ').readOnly = true;
            widget.getField('DSLOCALESTOQ').clearValue();
            widget.getField('CDLOCALESTOQ').clearValue();
            widget.getField('DSALMOXARIFE').readOnly = true;
            widget.getField('DSALMOXARIFE').clearValue();
            widget.getField('CDALMOXARIFE').clearValue();
            widget.getField('NMFILIAL').clearValue();
        }
        if (widget.getField('DSALMOXARIFE').value() == "") {
            widget.getField('CDALMOXARIFE').clearValue();
        }
    };

    // Parâmetros Filtro
    this.preparaParametosQueryRequisicaoEstoque = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;

        var tamanhoMask = getLocalVar('CDPICTPROD').replace(/\./g, '').length;
        var tamanho = tamanhoMask - filterRow.CDPRODFIN.length;

        for (var i = 0; i < tamanho; i++) {
            filterRow.CDPRODFIN = filterRow.CDPRODFIN + "Z";
        }

        var filtroFilial = [];
        if (Array.isArray(filterRow.CDFILIAL) && filterRow.CDFILIAL.length > 0) {
            filtroFilial = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": "_" + filterRow.CDFILIAL.join("_") + "_"
                }
            ];
        }
        /*else{                 Removido devido a controlle de acesso giraffas.
            filtroFilial= [
                {
                    "name":"CDFILIAL",
                    "operator":"=",
                    "value": "T"
                }
            ];
        }*/

        var filtroTipoReti = [];
        if (Array.isArray(filterRow.CDTIPORETI) && filterRow.CDTIPORETI.length > 0) {
            filtroTipoReti = [
                {
                    "name": "CDTIPORETI",
                    "operator": "=",
                    "value": "_" + filterRow.CDTIPORETI.join("_") + "_"
                }
            ];
        }
        else {
            filtroTipoReti = [
                {
                    "name": "CDTIPORETI",
                    "operator": "=",
                    "value": (filterRow.CDTIPORETI && filterRow.CDTIPORETI != 'T') ? "_" + filterRow.CDTIPORETI + "_" : 'T'
                }
            ];
        }

        var filtroCentiCust = [];
        if (Array.isArray(filterRow.CDCENTCUST) && filterRow.CDCENTCUST.length > 0) {
            filtroCentiCust = [
                {
                    "name": "CDCENTCUST",
                    "operator": "=",
                    "value": "_" + filterRow.CDCENTCUST.join("_") + "_"
                }
            ];
        }
        else {
            filtroCentiCust = [
                {
                    "name": "CDCENTCUST",
                    "operator": "=",
                    "value": (filterRow.CDCENTCUST && filterRow.CDCENTCUST != 'T') ? "_" + filterRow.CDCENTCUST + "_" : 'T'
                }
            ];
        }

        var filtro = [
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": filterRow.DTRETIRADA ? filterRow.DTRETIRADA.start : null
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": filterRow.DTRETIRADA ? filterRow.DTRETIRADA.end : null
            },
            {
                "name": "CDPRODINI",
                "operator": "=",
                "value": filterRow.CDPRODINI ? filterRow.CDPRODINI : null
            },
            {
                "name": "CDPRODFIN",
                "operator": "=",
                "value": filterRow.CDPRODFIN ? filterRow.CDPRODFIN : null
            },
            {
                "name": "CDALMOXARIFE",
                "operator": "=",
                "value": filterRow.CDALMOXARIFE ? filterRow.CDALMOXARIFE : null
            },
            {
                "name": "CDLOCALESTOQ",
                "operator": "=",
                "value": filterRow.CDLOCALESTOQ ? filterRow.CDLOCALESTOQ : null
            }
        ];

        filtro.push(filtroFilial[0]);
        filtro.push(filtroTipoReti[0]);
        filtro.push(filtroCentiCust[0]);
        widget.dataSourceFilter = filtro;
        if(widget.groups && widget.groups.length > 0){
            widget.useLocalDataOnReport = true;
            widget.dataSource.load(widget.getDataSourceFilter(), -1, -1, undefined, undefined, undefined, undefined, undefined, widget).then((response) => {
                arrCopy = Util.clone(response.dataset.relatorio_retirada_estoque);
            });
        }else{
            widget.useLocalDataOnReport = false;
        }
        setTimeout(() => {
            widget.reload();
        }, 500);
    };

    this.prepareForDataExporter = function(widget) {
        /*
        var fieldAlmoxarife = widget.getField('DSALMOXARIFE');
        var fieldLocalEstoq = widget.getField('DSLOCALESTOQ');
        var fieldUnidade = widget.getField('NMFILIAL');
        var fieldCentCust = widget.getField('NMCENTCUST');
        if(fieldAlmoxarife.value() == "" || fieldAlmoxarife.value() == undefined)
            fieldAlmoxarife.dataSource.name = 'est_relatorios#/crud/almoxarifado';
        if(fieldLocalEstoq.value() == "" || fieldLocalEstoq.value() == undefined)
            fieldLocalEstoq.dataSource.name = 'est_relatorios#/crud/localestoque';
        if(fieldAlmoxarife.dataSource.data.length == 0){
            fieldAlmoxarife.dataSourceFilter = [];
            fieldAlmoxarife.reload();
            //fieldAlmoxarife.dataSource.load();
        }
        if(fieldLocalEstoq.dataSource.data.length == 0){
            fieldLocalEstoq.dataSourceFilter = [];
            fieldLocalEstoq.reload();
            //fieldLocalEstoq.dataSource.load();
        }
        if(fieldUnidade.dataSource.data.length == 0){
            fieldUnidade.dataSource.name = 'est#/crud/filial';
            fieldUnidade.dataSourceFilter = [];
            fieldUnidade.reload();
            //fieldUnidade.dataSource.load();
        }
        fieldCentCust.reload().then(()=> {
            fieldCentCust.dataSource.data.push({'NMCENTCUST': ' ', 'CDCENTCUST' : null}); 
        });
        */
        /* widget.fields.forEach(f => {
            if(f.dataSource)
                f.dataSource.load(); nao pode ser com foreach....
         });*/
    };

    this.changeHeaderValues = function(args) {
        if(args.owner.groups && args.owner.groups.length > 0){
            args.owner.useLocalDataOnReport = true;
            args.owner.dataSource.data = arrCopy;
        }else{
            args.owner.useLocalDataOnReport = false;
        }

        this.prepareForDataExporter(args.owner);
        /*var codigo; 
        var nmempresa = widget.owner.dataSource.data[0].NMEMPRESA;
        widget.owner.dataSourceFilter.forEach(function(empresa){
           if(empresa.name == 'CDEMPRESA'){
               codigo = empresa.value;
           } 
        });
        
        var cdempresa = {
            'label' : "Empresa",
            'value' : "is equal to".concat(codigo).concat(' - ').concat(nmempresa)
        };
        
        var filter = widget.e.metadata.filter;
        
        newFilter.push(cdempresa);
        filter.forEach(function(x){
            if(x.value != "is equal to."){
                newFilter.push(x);
            }
        });
        */
        /*
        var newFilter = [];
        var correctFilter = [];
        
        var value;
        newFilter.forEach(function(aux){
            if(aux.value.match(/is equal to/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }else if(aux.value.match(/is between/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }
        });
        
        widget.e.metadata.filter = correctFilter;    
    */
    };

    this.checkProductRange = function(field) {
        var currentRow = field.widget.currentRow;

        if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
            var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

            if (!General.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {
                currentRow[field.outData[field.descriptionField]] = null;
                currentRow[field.outData[field.valueField]] = null;

                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
            }
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est58100_RelatorioRetiradaEstoque_Controller', Est58100_RelatorioRetiradaEstoque_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Est65000_ControleContagemEstoque_Controller.js
function Est65000_ControleContagemEstoque_Controller(General, restEngine, ScreenService, ZhFilterPreferences, templateManager) {

    this.initialize = function(widget) {
        var rowsToSet = {};
        var dataAtual = moment();
        var mes = dataAtual.format("M");
        var ano = dataAtual.format("YYYY");
        
        widget.dataSourceFilter[0].value = new Date(ano, mes, 0).getDateBr();
        widget.dataSourceFilter[1].value = new Date(ano, mes-1, 1).getDateBr();
        
        widget.getField("NMFILIAL").filterProperties.readOnly = true;
        widget.getField("NMFILIAL").filterProperties.validations = {};
        rowsToSet.DTPESQUISA = dataAtual.format('MM/YYYY');
        rowsToSet.TCDFILIAL = "T";
        rowsToSet.IMPORTACAO = "A";
        rowsToSet.CONTAGEM = "A";
        rowsToSet.ENCERRADO = "A";
        rowsToSet.NOOUDIFERENTE = "A";
        
        ZhFilterPreferences.setOriginalFilter(widget.id, rowsToSet);
        ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
        
    };
    
    this.onChangeData = function(args){
        if(args.owner.field.value()){
            var mes = args.owner.field.value().split("/")[0];
            var ano = args.owner.field.value().split("/")[1];
            var widget = templateManager.container.getWidget("tabela");
            widget.dataSourceFilter[0].value = new Date(ano, mes, 0).getDateBr();
            widget.dataSourceFilter[1].value = new Date(ano, mes-1, 1).getDateBr();
        }
         
    };
    
    this.onChangeDia = function(args){
        let dia = args.owner.field.value();
        if(dia){
            let data = dia + args.owner.field.widget.getField("DTPESQUISA").value();
            if(moment(data, "DDMMYYYY")._d == "Invalid Date"){
                args.owner.field.clearValue();
            }
        }
    };
    
    /*  Função para bloquear os fields filhos ligados a um field pai.
        Caso desbloqueiaComValor = true o field filho será habilitado 
        se o field pai possuir valor. No caso de valueDesbloqueia != null
        o valor para desbloquear o filho deve ser igual ao valor de valueDesbloqueia.
        
        @param field field 
    */
    
    this.alteraReadOnlyFilhos = function(field) {
        let widget = field.widget;
        field.filhos.forEach((filho) => {
            var desbloqueiaComValor = (filho.desbloqueiaComValor) ? filho.desbloqueiaComValor : true;
            var valueDesbloqueia    = (filho.valueDesbloqueia)    ? filho.valueDesbloqueia    : null;
            var isRequired          = (filho.isRequired)          ? filho.isRequired          : false;
            
            let filhoField = widget.getField(filho.name);
            
            
            if(valueDesbloqueia == null) {
                filhoField.readOnly =  ((field.value() == "") == desbloqueiaComValor);
                if(((field.value() == "") == desbloqueiaComValor)) widget.currentRow[filho.name] = "";
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
            else {
                var habilita = (field.value() == valueDesbloqueia);
                if(habilita) {
                    if(filhoField.readOnly) {
                        if(filho.clearFields) {
                            filho.clearFields.forEach((clearField) => {
                                widget.currentRow[clearField] = "";
                            });
                        }
                        widget.currentRow[filho.name] = "";
                    }
                    
                    filhoField.readOnly = false;
                    
                    if(isRequired) {
                        if(filhoField.validations) {
                            filhoField.validations.required = true;
                        }
                        else {
                            filhoField.validations = {required : true};
                        }
                    }
                }
                else {
                    var filhoOutData = filhoField.outData;
                    for(var fieldOut in filhoOutData) {
                        widget.currentRow[fieldOut] = "";
                    } 
                    widget.currentRow[filho.name] = "";
                    filhoField.readOnly = true;
                    if(isRequired) {
                        if(filhoField.validations) {
                            if(filhoField.validations.required) delete filhoField.validations.required;
                        }
                    }
                }
                if(filhoField.filhos) this.alteraReadOnlyFilhos(filhoField, true);
                widget.getField(filho.name).reload();
            }
        });
    };

    /*
        Percorre uma widget procurando por fields com a propiedade "filhos".
        Para os fields com a propiedade fields a função adiciona o evento 
        de alteraReadOnlyFilhos.
        
        @param widget widget
        @param boolean filter
    */
    
    this.blockFieldsFilhosAndSetEvents = function(widget, filter = true) {
        let fields = widget.fields;
        fields.forEach((field) => {
            if(field.filhos) {
                field.events.push(
                    {
                        "name": "FieldOnChange",
                        "code": "Est65000_ControleContagemEstoque_Controller.alteraReadOnlyFilhos(args.owner.field)"
                    }
                );
                
                field.filhos.forEach((filho) => {
                    if(filter) {
                        if(!widget.getField(filho.name).filterProperties) 
                            widget.getField(filho.name).filterProperties = {};
                        
                        widget.getField(filho.name).filterProperties.readOnly = false; 
                    }
                    else  widget.getField(filho.name).readOnly = false; 
                });
            }
        });
        
        this.initialize(widget);
        
    };
    
    this.filterAfterApply = function(args) {
        args.e.preventDefault();
        var widget = args.owner;
        let dia, nooudif;
        
        widget.dataSourceFilter.forEach( filter => {
            
            if(filter.name == "NODIA" ||  filter.name == "EXCETODIA"){
                filter.value = null;
            } else if(filter.name == "DIA"){
                dia = filter;
            } else if(filter.name == "NOOUDIFERENTE"){
                nooudif = filter;
            }
        });
        
        if(dia && dia.value && nooudif.value != 'A'){
            widget.dataSourceFilter.push(
                {
                    name: nooudif.value == "N" ? "NODIA" : "EXCETODIA",
                    operator: nooudif.value == "N" ? "=" : "<>",
                    value: dia.value
                }
            );
        }
        
        widget.dataSourceFilter = widget.dataSourceFilter.filter((f) => {
            if( (f.name == "IMPORTACAO" || f.name == "CONTAGEM" || f.name == "ENCERRADO") && (f.value == "A" || !f.value)){
                return false;
            } else if(f.name == "TCDFILIAL" || f.name == "NOOUDIFERENTE" || f.name == "DIA"){
                return false;
            } else if( (f.name == "NODIA" || f.name == "EXCETODIA") && !f.value ){
                return false;
            } else if(f.name == "DTPESQUISA"){
                return false;
            } else{
                return true;
            }
        });
        widget.reload();
    };
    
    this.openSwipeAjustes = function(widget){
        let widgetAjustes = widget.container.getWidget('gridAjustes');
        
        let dtInicial = widget.dataSourceFilter.filter(row => row.name == 'P_DTINICIAL')[0].value;
        let dtFinal   = widget.dataSourceFilter.filter(row => row.name == 'P_DTFINAL')[0].value;
        widgetAjustes.dataSourceFilter = [
                {
                    "name": "IDTPLANCTO",
                    "operator": "=",
                    "value": "1"
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": widget.currentRow.CDFILIAL
                },
                {
                    "name": "DTLANCESTQ",
                    "operator": "BETWEEN",
                    "value": [dtInicial, dtFinal]
                }
            ];
            
		 ScreenService.openSwipe(widgetAjustes).then(()=>{
		     widgetAjustes.reload();
		 });

    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est65000_ControleContagemEstoque_Controller', Est65000_ControleContagemEstoque_Controller);
});

// FILE: ./assets/modules/relatorios/assets/js/controllers/Est70000_ResumoDeOperacoes_Controller.js
function Est70000_ResumoDeOperacoes_Controller(General, ZhLayoutPreferences, localStorageEngine, metaDataFactory, ScreenService, EstUtil, ZhFilterPreferences, templateManager, Est70000_ResumoDeOperacoes_Service) {

    let IDUTILALMOX, IDUTILLCESTQ;

    this.onOpenFilter = function(widget) {
        let rowsToSet = {};
        const actualDate = getLocalVar('DTPROCESSA');

        rowsToSet.INTERVALO = { "start": actualDate, "end": actualDate, "formattedStart": actualDate, "formattedEnd": actualDate };
        rowsToSet.NMFILIAL = getLocalVar('NMFILIAL');
        rowsToSet.CDFILIAL = getLocalVar('CDFILIAL');

        ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(success => {
            const filterWidget = templateManager.container.getActiveWidget('zhFilterPreferencesSelectConditions');

            filterWidget.getField('NMFILIAL').reload().then(_ => {
                this.onChangeUnity(filterWidget);
            });
        });
    };

    this.onManageFilterParams = function(widget) {
        const filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDFILIAL', 'mainWidget', true)
            },
            {
                "name": "CDALMOXARIFE",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDALMOXARIFE', 'mainWidget', true) || ' '
            },
            {
                "name": "CDLOCALESTOQ",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDLOCALESTOQ', 'mainWidget', true) || ' '
            },
            {
                "name": "CDGRUPPROD",
                "operator": "=",
                "value": EstUtil.getFieldValue('CDGRUPPROD', 'mainWidget', true)
            },
            {
                "name": "DTINICIAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('INTERVALO', 'mainWidget', true).start
            },
            {
                "name": "DTFINAL",
                "operator": "=",
                "value": EstUtil.getFieldValue('INTERVALO', 'mainWidget', true).end
            }
        ];

        General.callRestEngine('est_relatorios#/get_resumo_operacoes', 'FilterData', filter).then(result => {
            localStorageEngine.setDataSource('/get_resumo_operacoes', result.dataset.get_resumo_operacoes).then(_ => {
                this.clearResults(widget);
                localStorageEngine.getDataSource('/get_resumo_operacoes').then(result => {
                    widget.dataSource.data = result;
                    this.onShowFields(widget);
                });
            });
        });
    };

    this.onChangeUnity = function(widget) {
        Est70000_ResumoDeOperacoes_Service.getUnityInfo(widget.currentRow.CDFILIAL).then(data => {
            const {
                IDUTILALMOX: CTRLALMOX,
                IDUTILLCESTQ: CTRLLOCAL
            } = data.dataset.paramfilial[0];
            const almoxField = widget.getField('DSALMOXARIFE');
            const LocalField = widget.getField('DSLOCALESTOQ');

            IDUTILALMOX = CTRLALMOX == 'S' ? true : false;
            IDUTILLCESTQ = CTRLLOCAL == 'S' ? true : false;

            almoxField.clearValue();

            if (IDUTILALMOX) {
                almoxField.readOnly = false;
                almoxField.validations = { "required": { "message": "Campo Obrigatório" } };
                if (IDUTILLCESTQ) {
                    LocalField.readOnly = false;
                    LocalField.validations = { "required": { "message": "Campo Obrigatório" } };
                }
            }
            else {
                delete almoxField.validations;
                almoxField.readOnly = true;
                almoxField.clearValue();
            }
            almoxField.reload();
            this.onChangeAlmox(widget);
        });
    };

    this.onChangeAlmox = function(widget) {
        const LocalField = widget.getField('DSLOCALESTOQ');

        if (IDUTILLCESTQ && widget.currentRow.CDALMOXARIFE) {
            LocalField.validations = { "required": { "message": "Campo Obrigatório" } };
            LocalField.readOnly = false;
            LocalField.clearValue();
        }
        else {
            LocalField.setValue(' ');
            widget.currentRow.CDLOCALESTOQ = ' ';
            delete LocalField.validations;
            LocalField.readOnly = true;
        }
        LocalField.reload();
        templateManager.updateTemplate();
    };

    this.onShowFields = function(widget) {
        const results = widget.dataSource.data;

        ZhLayoutPreferences.applyOriginalLayout(widget);
        if (results instanceof Array) {
            const cdgrupprod = EstUtil.getFieldValue('CDGRUPPROD', 'mainWidget', true);

            Est70000_ResumoDeOperacoes_Service.getfieldsLabels(cdgrupprod).then(success => {
                const originalLabels = ["TITLE", "TOTAL"];
                const exceptionsFields = ["__isSelected", "__is_new"];

                for (var object in results[0]) {
                    this.createField(widget, object, success.dataset.subgrupprod, originalLabels, exceptionsFields);
                    if (object == "TOTAL")
                        widget.redraw();
                }
            });
        }
    };

    this.createField = function(widget, name, groups, originalLabels, exceptionsFields) {
        if (exceptionsFields.includes(name)) return;

        let field;
        let label = originalLabels.includes(name) ? name : groups.filter(gp => gp.CDSUBGRPROD == name)[0].NMSUBGRPROD;

        if (name === "TITLE") {
            field = {
                "name": name,
                "label": " ",
                "showOnForm": true,
                "showOnView": true,
                "showOnList": true,
                "readOnly": true,
                "template": "field/text-edit.html"
            };
        }
        else {
            field = {
                "name": name,
                "label": ScreenService.i18n(label),
                "showOnForm": true,
                "showOnView": true,
                "showOnList": true,
                "readOnly": true,
                "template": "field/text-edit.html",
                "align": "right",
                "filterProperties": {
                    "isVisible": false,
                    "showOnForm": false
                },
                "mask": {
                    "type": "float",
                    "params": {
                        "showSymbol": false,
                        "precision": 2,
                        "thousands": ".",
                        "decimal": ","
                    }
                }
            };
        }

        metaDataFactory.factoryField(field, widget, true);
    };

    this.clearResults = function(widget) {

        const originalFields = [
            "CDFILIAL",
            "NMFILIAL",
            "CDALMOXARIFE",
            "DSALMOXARIFE",
            "CDLOCALESTOQ",
            "DSLOCALESTOQ",
            "CDGRUPPROD",
            "NMGRUPPROD",
            "INTERVALO"
        ];

        widget.fields = widget.fields.filter(field => {
            return originalFields.includes(field.name);
        });
        widget.dataSource.data = [];
    };

    this.changeRowStyles = function(row) {
        const totalizationTitles = [
            "Ticket's Caixa",
            "Quantidade Ticket's Caixa",
            "Quantidade Itens",
            "Ticket's Médio",
            "Itens P/T.C"
        ];

        return totalizationTitles.includes(row.TITLE) ? true : false;
    };

    this.resetCustomFilter = function(args) {
        args.e.preference.CUSTOM_FILTER_ROW = {};
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Est70000_ResumoDeOperacoes_Controller', Est70000_ResumoDeOperacoes_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/Ger09700_ProjecaoDeEstoque_Controller.js
function Ger09700_ProjecaoDeEstoque_Controller(templateManager, ZhLayoutPreferences, ZhFilterPreferences, ScreenService, MessageService, ZhUtil, maskEngine, EstUtil) {

	this.openFilter = function(widget) {
		ScreenService.showLoader();
		this.setFieldsGerais(widget);
		const rowsToSet = {};
		const tipoCustoField = widget.getField('NMTIPOCUSTO');
		rowsToSet.defaultFilter = widget.dataSourceFilter;
		rowsToSet.REPORT_NAME = "GER09700";
		rowsToSet.DTLEV = {
			formattedStart: moment().format('L'),
			formattedEnd: moment().format('L'),
			start: moment().format('L'),
			end: moment().format('L'),
		};
		rowsToSet.TFILESTOQ = 'T';
		rowsToSet.TFILIROM = 'T';
		rowsToSet.TSERIROM = 'T';
		rowsToSet.CDOPERADOR = getLocalVar('CDOPERADOR');

		tipoCustoField.reload().then(() => {
			rowsToSet.CDTIPOCUSTO = tipoCustoField.dataSource.data[0].CDTIPOCUSTO;
			rowsToSet.NMTIPOCUSTO = tipoCustoField.dataSource.data[0].NMTIPOCUSTO;

			ZhFilterPreferences.openFilterWidget(widget, rowsToSet).then(opened => {
				var filtro = widget.container.getActiveWidget();
				this.filiProd(filtro);
				this.onChangeReport(filtro);
				filtro.reload();
				ScreenService.hideLoader();
			}).catch(error => {
				console.log(error);
				ScreenService.hideLoader();
			});
		});
	};

	this.filterValidations = function(args) {
		let filterRow = args.e.preference.FILTER_ROW;

		if (filterRow.REPORT_NAME == 'GER09700Dis' && filterRow.IDFUNCAOFILI != 'O') {
			args.e.preventDefault();
			return MessageService.showMessageByCode('BLOCK_UNITY_SHOULD_BE_LOGISTC_OPERATOR');
		}
	};

	this.onChangeReport = function(widget) {
		const reportName = widget.currentRow.REPORT_NAME;
		let fieldsToHideOrClear, fieldsToShow;

		switch (reportName) {
			case 'GER09700':
				{
					fieldsToHideOrClear = [
						'NMFILESTOQ',
						'NMFILIROM',
						'DSSERIE',
						'DTENTRPEN',
						'DTSAIPEN',
						'TFILESTOQ',
						'TFILIROM',
						'TSERIROM'
					];
					this.hideFieldsManager(widget, fieldsToHideOrClear);

					fieldsToShow = [
						'CONSSOL',
						'CONSAF',
						'CONSEST',
						'CONSROM'
					];
					this.showFieldsManager(widget, fieldsToShow);
					break;
				}
			case 'GER09700Dis':
				{
					fieldsToHideOrClear = [
							'NMFILESTOQ',
							'NMFILIROM',
							'DSSERIE',
							'TFILESTOQ',
							'TFILIROM',
							'CONSSOL',
							'CONSAF',
							'CONSEST',
							'CONSROM',
							'TSERIROM'
						];
					this.hideFieldsManager(widget, fieldsToHideOrClear);

					fieldsToShow = [
						'DTENTRPEN',
						'DTSAIPEN'
					];
					this.showFieldsManager(widget, fieldsToShow);
					break;
				}
			default:
				// code
		}
		// widget.reload();
	};

	this.hideFieldsManager = function(widget, fieldsToHideOrClear) {

		const hideFieldsManager = (field) => {
			const exceptionHideFields = [
				'NMPRODINI'
			];

			if (field.template == 'field/checkbox.html') {
				field.setValue('N');
				field.checked = false;
			}
			else
				field.clearValue();

			if (exceptionHideFields.includes(field.name))
				field.readOnly = false;
			else
				field.readOnly = true;

			delete field.validations.required;
		};

		const fieldsHidePack = widget.fields.filter(field => {
			return fieldsToHideOrClear.includes(field.name);
		});

		fieldsHidePack.forEach(field => {
			hideFieldsManager(field);
			field.reload();
		});
	};

	this.showFieldsManager = function(widget, fieldsToShow) {
		const showFieldsManager = (field) => {
			const requiredFieldsBag = [
				"NMPRODINI",
				"NMTIPOCUSTO",
				"NMFILIAL",
				"REPORT_NAME",
				"DTLEV"
			];

			if (requiredFieldsBag.includes(field.name))
				field.validations.required = true;
			else
				delete field.validations.required;

			field.readOnly = false;
		};

		const fieldShowBag = widget.fields.filter(field => {
			return fieldsToShow.includes(field.name);
		});

		fieldShowBag.forEach(field => {
			showFieldsManager(field);
			field.reload();
		});
	};

	this.filiProd = function(widget) {
		var row = widget.currentRow;
		var filialIsSet = this.fieldIsSet(row.CDFILIAL);
		if (!filialIsSet) {
			widget.currentRow.NMPRODINI = '';
			widget.currentRow.NMPRODFIN = '';
			widget.currentRow.P_CDPRODINI = '';
			widget.currentRow.P_CDPRODFIN = '';
			delete widget.getField('NMPRODFIN').validations.required;
			widget.getField('NMPRODFIN').readOnly = true;
			widget.getField('NMPRODFIN').readOnlyOnEdit = true;
		}
		if (row.NMFILIAL && (row.NMFILIAL != "")) {
			widget.getField('NMPRODINI').dataSourceFilter.forEach(function(fil) {
				if (fil.name === 'CDFILIAL') {
					fil.value = widget.currentRow.CDFILIAL;
				}
			});
			widget.getField('NMPRODFIN').dataSourceFilter.forEach(function(fil) {
				if (fil.name === 'CDFILIAL') {
					fil.value = widget.currentRow.CDFILIAL;
				}
			});
		}
		widget.getField('NMPRODINI').reload();
		widget.getField('NMPRODFIN').reload();
		widget.getField('NMPRODINI').readOnly = false;
		widget.getField('NMPRODINI').readOnlyOnEdit = false;
		widget.getField('NMPRODINI').reload();
		widget.getField('NMPRODFIN').reload();
	};

	this.setFieldsGerais = function(widget) {
		templateManager.container.fieldsGerais = widget.fields.map(function(field) { return field.name; });
	};

	this.changeFieldUnidadeEstoque = function(args) {
		var widget = args.owner.field.widget;
		ScreenService.showLoader();
		if (args.row.TFILESTOQ == 'T') {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("NMFILESTOQ").readOnly = true;
					widget.getField("NMFILESTOQ").readOnlyOnEdit = true;
					delete widget.getField("NMFILESTOQ").validations.required;
					widget.currentRow.NMFILESTOQ = "";
					widget.currentRow.CDFILESTOQ = "";
					widget.getField("NMFILESTOQ").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
		else if (args.row.TFILESTOQ == 'L') {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("NMFILESTOQ").readOnly = false;
					widget.getField("NMFILESTOQ").readOnlyOnEdit = false;
					widget.getField("NMFILESTOQ").validations.required = true;
					widget.getField("NMFILESTOQ").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
	};

	this.changeFieldUnidadeRomaneio = function(args) {
		var widget = args.owner.field.widget;
		ScreenService.showLoader();
		if (args.row.TFILIROM == 'T') {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("NMFILIROM").readOnly = true;
					widget.getField("NMFILIROM").readOnlyOnEdit = true;
					delete widget.getField("NMFILIROM").validations.required;
					widget.currentRow.NMFILIROM = "";
					widget.currentRow.CDFILIROM = "";
					widget.getField("NMFILIROM").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
		else if (args.row.TFILIROM == 'L') {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("NMFILIROM").readOnly = false;
					widget.getField("NMFILIROM").readOnlyOnEdit = false;
					widget.getField("NMFILIROM").validations.required = true;
					widget.getField("NMFILIROM").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
	};

	this.changeFieldSerieNF = function(args) {
		var widget = args.owner.field.widget;
		ScreenService.showLoader();
		if (args.row.TSERIROM == 'T') {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("DSSERIE").readOnly = true;
					widget.getField("DSSERIE").readOnlyOnEdit = true;
					delete widget.getField("DSSERIE").validations.required;
					widget.currentRow.DSSERIE = "";
					widget.currentRow.CDSERIE = "";
					widget.getField("DSSERIE").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
		else if (args.row.TSERIROM == 'L') {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("DSSERIE").readOnly = false;
					widget.getField("DSSERIE").readOnlyOnEdit = false;
					widget.getField("DSSERIE").validations.required = true;
					widget.getField("DSSERIE").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
	};

	this.fieldIsSet = function(fieldValue) {
		if (fieldValue) return true;
		else return false;
	};

	this.filterProdIni = function(args) {
		var widget = args.owner.field.widget;
		var row = widget.currentRow;
		var prodIniIsSet = this.fieldIsSet(row.CDPRODINI);
		if (!prodIniIsSet) {
			widget.currentRow.NMPRODFIN = '';
			widget.currentRow.CDPRODFIN = '';
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				widget.getField('NMPRODFIN').readOnly = true;
				widget.getField('NMPRODFIN').readOnlyOnEdit = true;
				widget.getField('NMPRODFIN').clearValue();
			}
		}
		else if (widget.name === "zhFilterPreferencesSelectConditions") {
			widget.getField('NMPRODFIN').readOnly = false;
			widget.getField('NMPRODFIN').readOnlyOnEdit = false;
		}
		this.checkProductRange(args.owner.field);
	};

	this.filterProdFin = function(args) {
		var widget = args.owner.field.widget;
		var row = widget.currentRow;
		var prodFinIsSet = this.fieldIsSet(row.CDPRODFIN);
		var prodIniIsSet = this.fieldIsSet(row.CDPRODINI);

		if (!prodIniIsSet) {
			widget.getField('NMPRODFIN').readOnly = true;
			widget.getField('NMPRODFIN').readOnlyOnEdit = true;
			widget.getField('NMPRODFIN').clearValue();
			widget.getField('NMPRODFIN').reload();
		}

		this.checkProductRange(args.owner.field);
	};

	this.checkProductRange = function(field) {
		var currentRow = field.widget.currentRow;

		if (currentRow.CDPRODINI && currentRow.CDPRODFIN) {
			var compproduto = (currentRow.CDPRODFIN) ? currentRow.CDPRODFIN.replace(/[\.\-]/g, '') : null;

			if (!this.comparaPratoProduto(currentRow.CDPRODINI, compproduto, getLocalVar('CDPICTPROD'))) {

				MessageService.showMessageByCode("PRODUTO_INICIAL_MAIOR_FINAL").then(success => {
					currentRow[field.outData[field.descriptionField]] = null;
					currentRow[field.outData[field.valueField]] = null;
					currentRow.CDPRODFIN = null;
					field.widget.getField('NMPRODFIN').clearValue();
					field.widget.getField('NMPRODFIN').reload();
				}).catch(error => {
					console.log(error);
				});
			}
		}
	};

	this.comparaPratoProduto = function(pinicial, pfinal, mascara) {
		var mask = {
			type: "fix",
			params: {
				mask: mascara
			}
		};
		pinicial = Util.applyMask(pinicial, mask, maskEngine, ZhUtil).split('.');
		pfinal = Util.applyMask(pfinal, mask, maskEngine, ZhUtil).split('.');
		var nvinicial = pinicial.length;
		var nvfinal = pfinal.length;
		var resultado = true;
		if (nvinicial != nvfinal) {
			if (nvinicial > nvfinal) {
				resultado = false;
			}
		}
		else {
			var i = 0;
			do {
				if (pinicial[i] > pfinal[i]) {
					resultado = false;
				}
				i++;
			}
			while (i < pinicial.length && resultado);
		}
		return resultado;
	};

	this.changeBlockEstoque = function(args) {
		var widget = args.owner.field.widget;
		ScreenService.showLoader();
		if (args.row.CONSEST == 'N') {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("TFILESTOQ").readOnly = true;
					widget.getField("TFILESTOQ").readOnlyOnEdit = true;
					if (widget.getField("TFILESTOQ").value() == 'L') {
						widget.currentRow.TFILESTOQ = 'T';
						this.changeFieldUnidadeEstoque(args);
					}
					widget.getField("TFILESTOQ").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
		else {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("TFILESTOQ").readOnly = false;
					widget.getField("TFILESTOQ").readOnlyOnEdit = false;
					widget.getField("TFILESTOQ").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
	};

	this.changeBlockRomaneio = function(args) {
		var widget = args.owner.field.widget;
		ScreenService.showLoader();
		if (args.row.CONSROM == 'N') {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("TFILIROM").readOnly = true;
					widget.getField("TFILIROM").readOnlyOnEdit = true;
					if (widget.getField("TFILIROM").value() == 'L') {
						widget.currentRow.TFILIROM = 'T';
						this.changeFieldUnidadeRomaneio(args);
					}
					widget.getField("TFILIROM").reload();
					widget.getField("TSERIROM").readOnly = true;
					widget.getField("TSERIROM").readOnlyOnEdit = true;
					if (widget.getField("TSERIROM").value() == 'L') {
						widget.currentRow.TSERIROM = 'T';
						this.changeFieldSerieNF(args);
					}
					widget.getField("TSERIROM").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
		else {
			if (widget.name === "zhFilterPreferencesSelectConditions") {
				setTimeout(() => {
					widget.getField("TFILIROM").readOnly = false;
					widget.getField("TFILIROM").readOnlyOnEdit = false;
					widget.getField("TFILIROM").reload();
					widget.getField("TSERIROM").readOnly = false;
					widget.getField("TSERIROM").readOnlyOnEdit = false;
					widget.getField("TSERIROM").reload();
					ScreenService.hideLoader();
				}, 200);
			}
		}
	};

	this.defineIdiomaTipoCusto = function(widget) {
		const custoField = widget.getField('NMTIPOCUSTO');
		// setLocalVar('language', 'es_cl');
		const idioma = getLocalVar('language');

		const nomeField = {
			pt_br: 'NMTIPOCUSTO',
			en_us: 'NMTIPOCUSING',
			es_es: 'NMTIPOCUSESP',
			es_cl: 'NMTIPOCUSESP',
			es_co: 'NMTIPOCUSESP',
			es_mx: 'NMTIPOCUSESP',
		};

		let fields = custoField.fields;

		fields.forEach(function(field) {
			if (field.name == nomeField[idioma] || field.name == 'CDTIPOCUSTO')
				field.isVisible = true;
			else
				field.isVisible = false;
		});

		custoField.displayedFields[1] = nomeField[idioma];
		custoField.searchableFields[1] = nomeField[idioma];
		delete custoField.outData.NMTIPOCUSTO;
		custoField.outData[nomeField[idioma]] = "NMTIPOCUSTO";

		custoField.reload();
	};

	this.GridFieldsVisibilityManager = function() {
		const widget = templateManager.container.getWidget('widgetProjecao');
		const widgetFields = widget.fields.map(field => field.name);
		const changeFieldsVisibility = (VisibleFields, widgetFields) => {
			widgetFields.forEach(field => {
				if (VisibleFields.includes(field)) {
					widget.getField(field).isVisible = true;
					widget.getField(field).showOnList = true;
				}
				else {
					widget.getField(field).isVisible = false;
					widget.getField(field).showOnList = false;
				}
			});
		};
		const fieldUnityGroud = widget.getField('UNIDADEAGRUP');
		const reportName = EstUtil.getFieldValue('REPORT_NAME', widget.name, true);
		let VisibleFields = [];

		switch (reportName) {
			case 'GER09700':
				VisibleFields = [
					'CDPROESTPROJ',
					'NMPRODUTO',
					'SGUNIDADE',
					'QTPRODSOLI',
					'QTPRODROMA',
					'QTPRODNFPREV',
					'QTPRODLANCAF',
					'QTPRODESTOQ',
					'QTPROJESTPRO',
					'DTPRIFALTEST'
				];
				changeFieldsVisibility(VisibleFields, widgetFields);
				delete widget.getField('VRTOTAL').expression;
				fieldUnityGroud.grouped = false;
				fieldUnityGroud.isVisible = false;
				break;
			case 'GER09700Dis':
				VisibleFields = [
					'CDPROESTPROJ',
					'NMPRODUTO',
					'SGUNIDADE',
					'POSICAO',
					'VRCUSTOPROD',
					'VRTOTAL',
					'QTDE',
					'ENTRADAS',
					'SAIDAS',
					'SALDO'

				];
				changeFieldsVisibility(VisibleFields, widgetFields);
				widget.getField('VRTOTAL').expression = 'SUM';
				fieldUnityGroud.grouped = true;
				fieldUnityGroud.isVisible = true;
				break;
		}
		widget.redraw();
		// templateManager.updateTemplate();

	};

	this.reportProcess = function(args, afterOpenReport = false) {
		const { owner: widget } = args;

		if (afterOpenReport) {
			ZhLayoutPreferences.applyOriginalLayout(widget);
		}
		else {
			const metadata = args.e.metadata;
			const reportName = EstUtil.getFieldValue('REPORT_NAME', widget.name, true);
			const UnityName = EstUtil.getFieldValue('NMFILIAL', widget.name, true);
			const UnityCode = EstUtil.getFieldValue('CDFILIAL', widget.name, true);
			let { start } = EstUtil.getFieldValue('DTLEV', widget.name, true);

			const logisticOperator = metadata.filter.filter(filter => filter.label == 'Operador Logístico');
			logisticOperator[0].value = UnityCode + ' - ' + UnityName;
				
			metadata.filter.push({
				label: "Dt. Posição",
				value: start
			});
			
			if(reportName == 'GER09700Dis'){
				metadata.title = 'Projeção de Estoque - Distribuição';
				metadata.orientation = 'LANDSCAPE';
				
			}else{
				metadata.title = 'Projeção de Estoque';
				metadata.orientation = 'PORTRAIT';
			}
			ZhLayoutPreferences.applyOriginalLayout(widget);
		}
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Ger09700_ProjecaoDeEstoque_Controller', Ger09700_ProjecaoDeEstoque_Controller);
});


// FILE: ./assets/modules/relatorios/assets/js/controllers/relatorio_customizavel_integracoes_Controller.js
function relatorio_customizavel_integracoes_Controller(General, restEngine, ScreenService, ProdTreeGroup, ZhFilterPreferences, templateManager, EstUtil, MessageService) {
  
    this.returnFilterArray = function() {
        let initValue = EstUtil.getFieldValue('SEQLOGINI', 'est31300_grid_movimentacao_de_estoque_grid', true) || 1;
        let endValue = EstUtil.getFieldValue('SEQLOGFIM', 'est31300_grid_movimentacao_de_estoque_grid', true) || Number.MAX_SAFE_INTEGER;
        return [initValue, endValue];
    };
	
}

Configuration(function(ContextRegister) {
    ContextRegister.register('relatorio_customizavel_integracoes_Controller', relatorio_customizavel_integracoes_Controller);
});

